/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Component, Host, h, State, Prop, Event, Watch, Listen, Method } from '@stencil/core';
import { createDate, translate } from '../../../utils/utils';
import { formattingServiceFactory } from '../service';
import { dateServiceFactory } from '../service/date-service';
import { format, getMonth, getYear, isBefore, parse } from 'date-fns';
import { utcToZonedTime } from 'date-fns-tz';
export class DateTimeDialog {
  constructor() {
    this.showTemplates = false;
    /**
     * Case null: Year/Month Selection closed.
     * Case 'year': Year Selection active.
     * Case 'month': Month Selection active.
     */
    this.yearMonthSelectionStep = null;
    /** Tracks how often the user switched pages in the year selection */
    this.yearSelectionPageOffset = 0;
    /** Object to keep track of the currently selected date range */
    this.datesObj = {
      startDate: null,
      endDate: null,
    };
    /**
     * Optional - inject the FormattingService instance
     */
    this.formatService = formattingServiceFactory();
    /**
     * Optional - inject the DateService instance
     */
    this.dateService = dateServiceFactory({ formatService: this.formatService });
  }
  emitSelectionChanged(event) {
    this.selectionChanged.emit(event);
  }
  onTemplateUpdate({ detail: { firstEmit, timeStamp, id } }) {
    const event = {
      reason: 'frame',
      timestampStart: timeStamp.timestampStart,
      timestampEnd: timeStamp.timestampEnd,
      firstEmit,
      id,
    };
    this.selectionChanged.emit(event);
  }
  timeSelectedHandler({ detail: data }) {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    let { timeStart, timeEnd } = this.datePickerInputObj;
    if (data.timepickerId === 'timeStart_' + this.dateTimePickerId) {
      timeStart = data.updatedTime;
    }
    if (data.timepickerId === 'timeEnd_' + this.dateTimePickerId) {
      timeEnd = data.updatedTime;
    }
    const event = {
      reason: 'time',
      timeStart,
      timeEnd,
    };
    this.selectionChanged.emit(event);
  }
  showTimeRange() {
    this.showTemplates = false;
  }
  /**
   * Clears internal timer and resets selected template
   */
  async close() {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
  }
  /**
   * Closes the Year/Month Selection
   */
  async closeYearMonthSelection() {
    this.yearMonthSelectionStep = null;
  }
  showTemplate() {
    this.showTemplates = true;
  }
  toggleShowYearMonthSelectionHandler(event) {
    // null means the yearMonthSelection is closed
    this.yearMonthSelectionStep = event.detail.newState ? 'year' : null;
    this.yearSelectionPageOffset = 0;
  }
  arrowClickedHandler(event) {
    if (!!this.yearMonthSelectionStep && this.yearMonthSelectionStep === 'year') {
      switch (event.detail.direction) {
        case 'next':
          this.yearSelectionPageOffset++;
          break;
        case 'prev':
          this.yearSelectionPageOffset--;
          break;
        default:
          break;
      }
      return;
    }
    this.handleMonthChangeOnArrowClick(event.detail);
  }
  handleMonthChangeOnArrowClick(detail) {
    let newYear = this.dateObj.calendarDateYear;
    let newMonth;
    switch (detail.direction) {
      case 'next':
        newMonth = (this.dateObj.calendarDateMonth + 1) % 12;
        if (newMonth === 0) {
          newYear = this.dateObj.calendarDateYear + 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
      case 'prev':
        newMonth = this.dateObj.calendarDateMonth - 1;
        if (newMonth === -1) {
          newMonth = 11;
          newYear = this.dateObj.calendarDateYear - 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
      default:
        break;
    }
  }
  yearMonthSelectedHandler(event) {
    if (!this.yearMonthSelectionStep)
      return;
    if (this.yearMonthSelectionStep === 'year') {
      const newYear = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: newYear });
      this.yearMonthSelectionStep = 'month';
    }
    else if (this.yearMonthSelectionStep === 'month') {
      const newMonth = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth });
      this.yearMonthSelectionStep = null;
    }
  }
  singleDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDate: format(newDate, this.datePickerConfig.formatDate), selectedDate: format(newDate, this.datePickerConfig.formatDate) });
    const dateSelectedEvent = {
      reason: 'date',
      startDate: this.dateObj.selectedDate,
      endDate: null,
      rangeCompleted: false,
    };
    this.emitSelectionChanged(dateSelectedEvent);
  }
  rangeDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    let rangeComplete = false;
    // reset on subsequent clicks after first and second dates have been set
    if (this.datesObj.startDate && this.datesObj.endDate && this.datesObj.startDate !== this.datesObj.endDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: format(newDate, this.datePickerConfig.formatDate), endDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (!this.datesObj.startDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (isBefore(newDate, parse(this.datesObj.startDate, this.datePickerConfig.formatDate, new Date()))) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: this.datesObj.startDate, startDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    else {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    const dateSelectedEvent = Object.assign(Object.assign({}, this.datesObj), { reason: 'date', rangeCompleted: rangeComplete });
    this.emitSelectionChanged(dateSelectedEvent);
  }
  userInputDate(newValue) {
    if (!newValue)
      return;
    if (newValue.dateStart) {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: getMonth(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())), calendarDateYear: getYear(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())) });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: newValue.dateStart, endDate: newValue.dateEnd });
    }
    else {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), selectedDate: null });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: null, endDate: null });
    }
  }
  async componentWillLoad() {
    this.dateObj = {
      calendarDate: format(utcToZonedTime(new Date(), this.datePickerConfig.timeZone), this.datePickerConfig.formatDate),
      calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      selectedDate: null,
    };
  }
  applyTemplate(option, firstEmit) {
    this.onTemplateUpdate({ detail: { timeStamp: option.calculateTemplate(), firstEmit, id: option.id } });
  }
  render() {
    var _a, _b, _c, _d;
    return (h(Host, { id: `overlay_${this.dateTimePickerId}` },
      h("div", { class: "bci-core-date-time-picker-container", tabindex: "-1" },
        !this.showTemplates && this.hide !== 'calendar' && (h("div", null,
          h("bci-month-header", { "data-test": "webcore.webcomponents.bci-datepicker.year-month", year: this.dateObj.calendarDateYear, month: this.dateObj.calendarDateMonth, yearMonthSelectionStep: this.yearMonthSelectionStep }),
          h("div", { class: `bci-core-date-time-picker-sub-container year-month ${!this.yearMonthSelectionStep ? 'hidden' : ''} ${this.hide !== 'time' ? 'has-time' : ''}` },
            h("bci-year-month", { selectionStep: this.yearMonthSelectionStep, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, dateObj: this.dateObj, yearSelectionPageOffset: this.yearSelectionPageOffset })))),
        !this.showTemplates && (h("div", { class: `bci-core-date-time-picker-sub-container ${!!this.yearMonthSelectionStep ? 'hidden' : ''} ` },
          this.hide !== 'calendar' && (h("bci-datepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.dateTimePickerInput", bciDatepickerId: this.dateTimePickerId, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, rangeDate: this.datePickerConfig.rangeDateTime, sundayFirst: this.datePickerConfig.sundayFirst, formatDate: this.datePickerConfig.formatDate, datePickerInputObj: this.datePickerInputObj, timeZone: this.datePickerConfig.timeZone, selectedYear: this.dateObj.calendarDateYear, selectedMonth: this.dateObj.calendarDateMonth, selectedDate: this.dateObj.selectedDate, today: this.formatService.formatDate(this.dateService.getTodayValue(this.datePickerConfig.timeZone)), datesObj: this.datesObj })),
          this.hide !== 'time' && (h("div", { class: { 'bci-core-time-picker-container': true, 'no-border': this.hide === 'calendar' } },
            h("bci-timepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.starttime", class: { 'full-height': !this.timePickerConfig.rangeDateTime }, timeInput: (_a = this.datePickerInputObj) === null || _a === void 0 ? void 0 : _a.timeStart, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeStart_' + this.dateTimePickerId, rangeTimePicker: false, timeTitle: this.timePickerConfig.rangeDateTime ? translate('startTime') : translate('time') }),
            h("bci-timepicker", { "data-test": "datetime-picker.core-datetime-picker.endtime", class: { hidden: !this.timePickerConfig.rangeDateTime }, timeInput: (_b = this.datePickerInputObj) === null || _b === void 0 ? void 0 : _b.timeEnd, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeEnd_' + this.dateTimePickerId, rangeTimePicker: true, timeTitle: translate('endTime') }))))),
        !this.showTemplates && (h("div", { class: "bci-core-datepicker-selection" },
          ((_c = this.templatesList) === null || _c === void 0 ? void 0 : _c.length) > 1 && this.hide !== 'calendar' && (h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.showTemplate() }, translate('templates'))),
          ((_d = this.templatesList) === null || _d === void 0 ? void 0 : _d.length) === 1 && this.hide !== 'calendar' && (h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.applyTemplate(this.templatesList[0], true) }, this.templatesList[0].label)),
          h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.close", class: "button-primary no-arrow close", onClick: () => this.toggleDialogHandler() }, translate('close')))),
        h("bci-template", { templatesList: this.templatesList, visible: this.showTemplates && !!this.templatesList, ref: (el) => (this.templatesElement = el) }))));
  }
  static get is() { return "bci-datetime-dialog"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dialog.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["dialog.css"]
  }; }
  static get properties() { return {
    "dateTimePickerId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "This id reused within the children ids"
      },
      "attribute": "date-time-picker-id",
      "reflect": false
    },
    "datePickerInputObj": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "DateTime",
        "resolved": "DateTime",
        "references": {
          "DateTime": {
            "location": "import",
            "path": "../datetime-picker.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Pass through of the current time state"
      }
    },
    "templatesList": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Array<Template | TemplateGroup>",
        "resolved": "(Template | TemplateGroup)[]",
        "references": {
          "Array": {
            "location": "global"
          },
          "Template": {
            "location": "import",
            "path": "../datetime-picker.interface"
          },
          "TemplateGroup": {
            "location": "import",
            "path": "../datetime-picker.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Which templates should be displayed"
      }
    },
    "templatesDisabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether the \"Templates\"-Button should be disabled"
      },
      "attribute": "templates-disabled",
      "reflect": false
    },
    "hide": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'calendar' | 'time'",
        "resolved": "\"calendar\" | \"time\"",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Which parts of the datepicker should be hidden"
      },
      "attribute": "hide",
      "reflect": false
    },
    "datePickerConfig": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "DatePickerConfig",
        "resolved": "DatePickerConfig",
        "references": {
          "DatePickerConfig": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Contains the settings for the date picker"
      }
    },
    "timePickerConfig": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "TimePickerConfig",
        "resolved": "TimePickerConfig",
        "references": {
          "TimePickerConfig": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Contains the settings for the time picker"
      }
    },
    "formatService": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FormattingService",
        "resolved": "FormattingService",
        "references": {
          "FormattingService": {
            "location": "import",
            "path": "../service"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optional - inject the FormattingService instance"
      },
      "defaultValue": "formattingServiceFactory()"
    },
    "dateService": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "DateService",
        "resolved": "DateService",
        "references": {
          "DateService": {
            "location": "import",
            "path": "../service/date-service"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optional - inject the DateService instance"
      },
      "defaultValue": "dateServiceFactory({ formatService: this.formatService })"
    },
    "toggleDialogHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "() => void",
        "resolved": "() => void",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Handler for toggling the dialog"
      }
    }
  }; }
  static get states() { return {
    "showTemplates": {},
    "yearMonthSelectionStep": {},
    "yearSelectionPageOffset": {},
    "dateObj": {},
    "datesObj": {}
  }; }
  static get events() { return [{
      "method": "selectionChanged",
      "name": "selectionChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits if any child value changes"
      },
      "complexType": {
        "original": "SelectionChanged",
        "resolved": "DateChanged | FrameChanged | TimeChanged",
        "references": {
          "SelectionChanged": {
            "location": "import",
            "path": "../model"
          }
        }
      }
    }]; }
  static get methods() { return {
    "close": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clears internal timer and resets selected template",
        "tags": []
      }
    },
    "closeYearMonthSelection": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the Year/Month Selection",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "datePickerInputObj",
      "methodName": "userInputDate"
    }]; }
  static get listeners() { return [{
      "name": "templateUpdate",
      "method": "onTemplateUpdate",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "updatedTime",
      "method": "timeSelectedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "showTimeRanges",
      "method": "showTimeRange",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "toggleShowYearMonthSelection",
      "method": "toggleShowYearMonthSelectionHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "arrowClicked",
      "method": "arrowClickedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "yearMonthSelected",
      "method": "yearMonthSelectedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "singleDateSelected",
      "method": "singleDateSelectedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "rangeDateSelected",
      "method": "rangeDateSelectedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}

