/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Element, Event, h, Host, Prop, State, Watch } from '@stencil/core';
import { format, parse } from 'date-fns';
import { isHideSeconds, timeFormatHasMeridiem } from '../../../utils/utils';
var TimeDirection;
(function (TimeDirection) {
  TimeDirection[TimeDirection["UP"] = 0] = "UP";
  TimeDirection[TimeDirection["DOWN"] = 1] = "DOWN";
})(TimeDirection || (TimeDirection = {}));
var MeridiemValue;
(function (MeridiemValue) {
  MeridiemValue["AM"] = "AM";
  MeridiemValue["PM"] = "PM";
})(MeridiemValue || (MeridiemValue = {}));
export class Timepicker {
  constructor() {
    this.showMeridiem = true;
    this.meridiem = MeridiemValue.AM;
  }
  handleTimeInput(newValue) {
    this.hour = !!newValue
      ? format(parse(newValue, this.timeFormat, new Date()), timeFormatHasMeridiem(this.timeFormat) ? 'hh' : 'HH')
      : timeFormatHasMeridiem(this.timeFormat)
        ? null
        : null;
    this.hour = this.formatTime(this.hour);
    this.minutes = !!newValue ? format(parse(newValue, this.timeFormat, new Date()), 'mm') : null;
    this.minutes = this.formatTime(this.minutes);
    this.seconds = !!newValue ? format(parse(newValue, this.timeFormat, new Date()), 'ss') : null;
    this.seconds = this.formatTime(this.seconds);
    this.showMeridiem = !timeFormatHasMeridiem(this.timeFormat);
    this.hideSeconds = !isHideSeconds(this.timeFormat);
    this.meridiem = (newValue === null || newValue === void 0 ? void 0 : newValue.includes(MeridiemValue.PM)) ? MeridiemValue.PM : MeridiemValue.AM;
  }
  componentDidLoad() {
    if (isNaN(Date.parse(this.timeInput))) {
      this.handleTimeInput(this.timeInput);
    }
  }
  getUpdatedTime() {
    if (this.hour || this.minutes || this.seconds) {
      const newTime = `${!!this.hour ? this.hour : timeFormatHasMeridiem(this.timeFormat) ? '01' : '00'}:${this.minutes || '00'}:${this.seconds || '00'}${timeFormatHasMeridiem(this.timeFormat) ? ' ' + this.meridiem : ''}`;
      return format(parse(newTime, timeFormatHasMeridiem(this.timeFormat) ? 'hh:mm:ss a' : 'HH:mm:ss', new Date()), this.timeFormat);
    }
    else {
      return null;
    }
  }
  emitUpdatedTime(isDelete) {
    this.updatedTime.emit({
      updatedTime: isDelete ? null : this.getUpdatedTime(),
      timepickerId: this.bciTimepickerId,
    });
  }
  updateTimeOnType(ev, hmsmr, maxValue, minValue = 0) {
    let newTime;
    newTime = ev.target.value;
    if (!newTime || isNaN(newTime) || newTime === '') {
      newTime = null;
    }
    else {
      newTime = parseInt(newTime, 10);
      newTime = Math.min(newTime, maxValue);
      newTime = Math.max(newTime, minValue);
      newTime = this.formatTime(newTime);
    }
    switch (hmsmr) {
      case 'h':
        this.hour = !!newTime ? String(newTime) : newTime;
        this.hourInput.value = `${newTime}`;
        break;
      case 'm':
        this.minutes = !!newTime ? String(newTime) : newTime;
        this.minutesInput.value = `${newTime}`;
        break;
      case 's':
        this.seconds = !!newTime ? String(newTime) : newTime;
        this.secondsInput.value = `${newTime}`;
        break;
    }
    if (!newTime && typeof newTime !== 'number') {
      this.emitUpdatedTime(true);
    }
    else {
      this.emitUpdatedTime();
    }
  }
  handleInputType(ev, hmsmr, maxValue, minValue = 0) {
    let newTime = String(ev.target.value);
    /** Wheter the event is triggered by pressing number-keys on the keyboard.
     * Otherwise this handler is triggered by pressing ArrowUp/ArrowDown keys. */
    const isInputEvent = ev instanceof InputEvent;
    // automatically move to next input if input is valid
    const newTimeAsInt = parseInt(newTime, 10);
    if (isInputEvent) {
      if (newTime.length === 2 && newTimeAsInt >= minValue && newTimeAsInt <= maxValue) {
        this.selectNextInput(hmsmr);
      }
    }
    else {
      ev.target.select();
    }
    // prevent negative values
    newTime = newTime.replace('-', '');
    // prevent more than 2 digits
    if (newTime.length > 2) {
      newTime = newTime.substring(0, 2);
    }
    switch (hmsmr) {
      case 'h':
        this.hourInput.value = `${newTime}`;
        break;
      case 'm':
        this.minutesInput.value = `${newTime}`;
        break;
      case 's':
        this.secondsInput.value = `${newTime}`;
        break;
    }
  }
  handleKeyPressed(ev, hmsmr, maxValue, minValue = 0) {
    if (ev.key === 'Enter') {
      this.updateTimeOnType(ev, hmsmr, maxValue, minValue);
      this.selectNextInput(hmsmr);
    }
  }
  selectNextInput(hmsmr) {
    switch (hmsmr) {
      case 'h':
        this.minutesInput.select();
        break;
      case 'm':
        this.secondsInput.select();
        break;
      case 's':
        if (timeFormatHasMeridiem(this.timeFormat))
          this.meridiemInput.focus();
        break;
    }
  }
  formatTime(value) {
    if (!value && typeof value !== 'number')
      return null;
    return String(value).padStart(2, '0');
  }
  updateTimeOnClick(direction, value, maxValue, minValue = 0) {
    if (!value) {
      value = direction === TimeDirection.UP ? minValue : maxValue;
      return this.formatTime(value);
    }
    value = parseInt(value, 10);
    if (direction === TimeDirection.UP) {
      value = value === maxValue ? minValue : value + 1;
    }
    if (direction === TimeDirection.DOWN) {
      value = value === minValue ? maxValue : value - 1;
    }
    return this.formatTime(value);
  }
  updateMeridiemOnClick() {
    this.meridiem = this.meridiem === MeridiemValue.AM ? MeridiemValue.PM : MeridiemValue.AM;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "bci-core-timepicker-title" }, this.timeTitle),
      h("div", { class: "bci-core-timepicker-container" },
        h("div", { class: {
            'bci-core-input-container': true,
            rangeTimePicker: this.rangeTimePicker,
            'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
            'no-secondsandmerdiam': this.hideSeconds && this.showMeridiem,
          } },
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
                this.hour = this.updateTimeOnClick(TimeDirection.UP, this.hour, timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, timeFormatHasMeridiem(this.timeFormat) ? 1 : 0);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.hour-up.span" })),
          h("input", { "data-test": "webcore.webcomponents.timepicker.update-time-on-type", type: "number", placeholder: "HH", step: 1, min: timeFormatHasMeridiem(this.timeFormat) ? 1 : 0, max: timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, pattern: "[012][0-9]*", "data-hours": "true", onFocus: () => this.hourInput.select(), onInput: (ev) => this.handleInputType(ev, 'h', timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), onKeyPress: (ev) => this.handleKeyPressed(ev, 'h', timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), onBlur: (ev) => this.updateTimeOnType(ev, 'h', timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), value: this.hour, ref: (el) => (this.hourInput = el) }),
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
                this.hour = this.updateTimeOnClick(TimeDirection.DOWN, this.hour, timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, timeFormatHasMeridiem(this.timeFormat) ? 1 : 0);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.hour-down.span" }))),
        h("div", { class: "bci-core-input-container divider" },
          h("span", null, ":")),
        h("div", { class: {
            'bci-core-input-container': true,
            rangeTimePicker: this.rangeTimePicker,
            'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
            'no-secondsandmerdiam': this.hideSeconds && this.showMeridiem,
          } },
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
                this.minutes = this.updateTimeOnClick(TimeDirection.UP, this.minutes, 59);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.minute-up.span" })),
          h("input", { "data-test": "webcore.webcomponents.timepicker.update-time-on-type-minute", min: 0, max: 59, placeholder: "MM", type: "number", step: 1, pattern: "[0-5][0-9]*", "data-minutes": "true", onFocus: () => this.minutesInput.select(), onInput: (ev) => this.handleInputType(ev, 'm', 59), onKeyPress: (ev) => this.handleKeyPressed(ev, 'm', 59), onBlur: (ev) => this.updateTimeOnType(ev, 'm', 59), value: this.minutes, ref: (el) => (this.minutesInput = el) }),
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
                this.minutes = this.updateTimeOnClick(TimeDirection.DOWN, this.minutes, 59);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.minute-down.span" }))),
        h("div", { class: {
            'bci-core-input-container': true,
            divider: true,
            hidden: this.hideSeconds,
          } },
          h("span", null, ":")),
        h("div", { class: {
            'bci-core-input-container': true,
            hidden: this.hideSeconds,
            rangeTimePicker: this.rangeTimePicker,
            'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
          } },
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
                this.seconds = this.updateTimeOnClick(TimeDirection.UP, this.seconds, 59);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.second-up.span" })),
          h("input", { "data-test": "webcore.webcomponents.timepicker.bci-core-timepicker-seconds-input", type: "number", placeholder: "SS", step: 1, min: 0, max: 59, pattern: "[0-5][0-9]*", "data-seconds": "true", onFocus: () => this.secondsInput.select(), onInput: (ev) => this.handleInputType(ev, 's', 59), onKeyPress: (ev) => this.handleKeyPressed(ev, 's', 59), onBlur: (ev) => this.updateTimeOnType(ev, 's', 59), value: this.seconds, ref: (el) => (this.secondsInput = el) }),
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
                this.seconds = this.updateTimeOnClick(TimeDirection.DOWN, this.seconds, 59);
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.second-down.span" }))),
        h("div", { class: {
            'bci-core-input-container': true,
            divider: true,
            hidden: this.showMeridiem,
          } }),
        h("div", { class: {
            'bci-core-input-container': true,
            'no-seconds': this.hideSeconds && !this.showMeridiem,
            hidden: this.showMeridiem,
            rangeTimePicker: this.rangeTimePicker,
          } },
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
                this.updateMeridiemOnClick();
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.meridiem-up.span" })),
          h("input", { "data-test": "webcore.webcomponents.timepicker.bci-core-timepicker-meridiem-input", type: "text", readonly: "readonly", "data-seconds": "true", onKeyDown: (ev) => {
              if (['ArrowDown', 'ArrowUp'].includes(ev.key)) {
                this.updateMeridiemOnClick();
                this.emitUpdatedTime();
              }
            }, value: this.meridiem, ref: (el) => (this.meridiemInput = el) }),
          h("span", { class: "arrow-button" },
            h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
                this.updateMeridiemOnClick();
                this.emitUpdatedTime();
              }, "data-test": "webcore.webcomponents.timepicker.update-time.meridiem-down.span" }))))));
  }
  static get is() { return "bci-timepicker"; }
  static get originalStyleUrls() { return {
    "$": ["timepicker.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["timepicker.css"]
  }; }
  static get properties() { return {
    "timeInput": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "time-input",
      "reflect": false
    },
    "timeFormat": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "TimePickerTimeFormat",
        "resolved": "\"H:mm\" | \"HH:mm:ss\" | \"hh:mm a\" | \"hh:mm:ss a\"",
        "references": {
          "TimePickerTimeFormat": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "time-format",
      "reflect": false
    },
    "bciTimepickerId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "bci-timepicker-id",
      "reflect": false
    },
    "rangeTimePicker": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "range-time-picker",
      "reflect": false
    },
    "timeTitle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "time-title",
      "reflect": false
    }
  }; }
  static get states() { return {
    "hideSeconds": {},
    "showMeridiem": {},
    "hour": {},
    "minutes": {},
    "seconds": {},
    "meridiem": {}
  }; }
  static get events() { return [{
      "method": "updatedTime",
      "name": "updatedTime",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "TimeSelectedObject",
        "resolved": "TimeSelectedObject",
        "references": {
          "TimeSelectedObject": {
            "location": "import",
            "path": "./timepicker.model"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "timeInput",
      "methodName": "handleTimeInput"
    }]; }
}

