/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Component, Event, Host, Prop, State, Watch, h } from '@stencil/core';
import { isSameMonth, isBefore, isAfter, getDaysInMonth, isSameYear, format, setMonth } from 'date-fns';
import { createDate } from '../../../utils/utils';
import state from '../../../utils/locale-store';
import { DEFAULT_LOCALE, LOCALE_MAP } from '../../../utils/localization';
/** number of years displayed per page */
const YEAR_PAGE_SIZE = 20;
/** Where the currently selected year is displayed on the yearSelection Page. 0-based. */
const SELECTED_YEAR_SPOT = 9;
export class YearMonthSelection {
  handleSelectionStepChange(newValue, oldValue) {
    // Reopen of YearMonthSelection
    if (!oldValue && !!newValue) {
      this.displayedYears = this.getDisplayedYearsFromYear(this.dateObj.calendarDateYear);
    }
  }
  handleSelectionPageChange(newValue) {
    this.displayedYears = this.getDisplayedYearsFromYear(this.dateObj.calendarDateYear + newValue * YEAR_PAGE_SIZE);
  }
  getDisplayedYearsFromYear(year) {
    const yearsBefore = [];
    for (let i = year - SELECTED_YEAR_SPOT; i < year; i++) {
      yearsBefore.push(i);
    }
    const yearsAfter = [];
    for (let i = year + 1; i < year + (YEAR_PAGE_SIZE - SELECTED_YEAR_SPOT); i++) {
      yearsAfter.push(i);
    }
    const years = [...yearsBefore, year, ...yearsAfter];
    return this.createGrid(years, YEAR_PAGE_SIZE, 4);
  }
  get months() {
    const locale = { locale: LOCALE_MAP[state.language || DEFAULT_LOCALE.language] };
    const months = [];
    for (let i = 0; i < 12; i++) {
      months.push(format(setMonth(new Date(), i), 'LLL', locale));
    }
    return this.createGrid(months, 12, 3);
  }
  // use generics here because years are numbers and months are strings
  createGrid(array, gridItems, rowSize) {
    const grid = [];
    for (let i = 0; i < gridItems; i += rowSize) {
      const row = array.slice(i, i + rowSize);
      grid.push(row);
    }
    return grid;
  }
  handleMonthClick(monthIndex) {
    this.yearMonthSelected.emit({ selectedYearMonth: monthIndex });
  }
  handleYearClick(year) {
    this.yearMonthSelected.emit({ selectedYearMonth: year });
  }
  render() {
    const selectedYear = this.dateObj.calendarDateYear;
    const selectedMonth = this.dateObj.calendarDateMonth;
    const selectionAsDate = createDate(new Date(), selectedYear, selectedMonth);
    return (h(Host, null, this.selectionStep === 'year' ? (h("div", { class: "year-month-grid" }, this.displayedYears.map((yearRow) => (h("div", { class: "row" }, yearRow.map((year) => {
      const selected = isSameYear(createDate(new Date(), year, 0), selectionAsDate);
      const isTodaysDate = isSameYear(createDate(new Date(), year, 0), new Date());
      const notInRange = isBefore(createDate(new Date(), year, 0, 1), this.minDate) ||
        isAfter(createDate(new Date(), year, 11, getDaysInMonth(new Date(year, 11))), this.maxDate);
      const buttonClasses = {
        'selection-chip': true,
        selected,
        today: isTodaysDate,
      };
      return (h("button", { class: buttonClasses, "aria-label": `Select Year ${year}`, key: year, disabled: notInRange, onClick: () => this.handleYearClick(year) }, year));
    })))))) : this.selectionStep === 'month' ? (h("div", { class: "year-month-grid" }, this.months.map((monthRow, rowI) => (h("div", { class: "row" }, monthRow.map((month, monthI) => {
      const monthIndex = rowI * (this.months.length - 1) + monthI;
      const selected = isSameMonth(createDate(new Date(), selectedYear, monthIndex), selectionAsDate);
      const isTodaysDate = isSameMonth(createDate(new Date(), selectedYear, monthIndex), new Date());
      const notInRange = isBefore(createDate(new Date(), selectedYear, monthIndex, 1), this.minDate) ||
        isAfter(createDate(new Date(), selectedYear, monthIndex, getDaysInMonth(new Date(selectedYear, monthIndex))), this.maxDate);
      const buttonClasses = {
        'selection-chip': true,
        selected,
        today: isTodaysDate,
      };
      return (h("button", { class: buttonClasses, "aria-label": `Select Month ${month}`, disabled: notInRange, onClick: () => this.handleMonthClick(monthIndex), key: month }, month));
    })))))) : undefined));
  }
  static get is() { return "bci-year-month"; }
  static get originalStyleUrls() { return {
    "$": ["year-month.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["year-month.css"]
  }; }
  static get properties() { return {
    "selectionStep": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'year' | 'month'",
        "resolved": "\"month\" | \"year\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "selection-step",
      "reflect": false
    },
    "yearSelectionPageOffset": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "year-selection-page-offset",
      "reflect": false
    },
    "dateObj": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "date-obj",
      "reflect": false
    },
    "minDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "min-date",
      "reflect": false
    },
    "maxDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "max-date",
      "reflect": false
    }
  }; }
  static get states() { return {
    "displayedYears": {}
  }; }
  static get events() { return [{
      "method": "yearMonthSelected",
      "name": "yearMonthSelected",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "emits",
            "text": "new year or month"
          }],
        "text": ""
      },
      "complexType": {
        "original": "YearMonthSelectedObject",
        "resolved": "YearMonthSelectedObject",
        "references": {
          "YearMonthSelectedObject": {
            "location": "import",
            "path": "../datepicker/datepicker.models"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "selectionStep",
      "methodName": "handleSelectionStepChange"
    }, {
      "propName": "yearSelectionPageOffset",
      "methodName": "handleSelectionPageChange"
    }]; }
}

