/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, State, Element, Listen, Event, Prop } from '@stencil/core';
import { MDCDialog } from '@material/dialog';
import { formatDate, translate } from '../../utils/utils';
import { MessagesController } from '../../utils/messages-controller';
import { CollectionType } from '../checkbox-collection/checkbox-collection.interface';
import { MESSAGE_TYPE_FILTERS, MESSAGE_SORT_FILTER, SortOrder } from './messages.interface';
import { setLocale, listenToLanguageChange } from '../../utils/locale-store';
export class MessagesComponent {
  constructor() {
    this.controller = new MessagesController();
    this.searchString = '';
    this.filterArray = [];
    this.sortOrder = '';
    this.showMessgeFilterOptions = false;
  }
  async componentWillLoad() {
    await setLocale();
    this.loadMessages();
    listenToLanguageChange(() => this.updateFilterDisplayString());
  }
  componentDidRender() {
    this.dialog = new MDCDialog(this.el.shadowRoot.querySelector('.bci-portal-message-history-dialog'));
    this.dialog.listen('MDCDialog:opened', () => {
      const filters = this.el.shadowRoot.querySelectorAll('bci-checkbox-collection');
      filters.forEach((item) => item.setDefaultSelection());
    });
    this.dialog.listen('MDCDialog:closed', () => {
      window.document.querySelector('body').classList.remove('mdc-dialog-blur');
      this.resetFilters();
    });
  }
  resetFilters() {
    this.searchString = '';
    this.showMessgeFilterOptions = false;
    const filters = this.el.shadowRoot.querySelectorAll('bci-checkbox-collection');
    filters.forEach((item) => item.reset());
    this.filterArray = [];
    this.sortOrder = '';
    this.updateFilterDisplayString();
  }
  onMessageCreated() {
    this.loadMessages();
  }
  loadMessages() {
    this.messages = this.controller.getMessages(this.localStorageKey);
  }
  multiFilterSelected(event) {
    this.filterArray = [];
    this.filterArray = event.detail;
    this.updateFilterDisplayString();
  }
  singleFilterSelected(event) {
    this.sortOrder = event.detail;
    this.updateFilterDisplayString();
  }
  // Method to form the selected filter display string
  updateFilterDisplayString() {
    let str = '';
    if (this.filterArray.length > 0) {
      if (this.filterArray.length === MESSAGE_TYPE_FILTERS.length) {
        str = translate('allTypes');
      }
      else {
        str = this.filterArray.map((filterString) => translate(filterString)).join(', ');
      }
    }
    if (this.sortOrder !== '') {
      str += ', ' + translate(this.sortOrder);
    }
    // Removing unnecessary commas
    this.filterSelectList.value = str.replace(/^,|,$/g, '');
  }
  dateToDayString(msg) {
    return formatDate(msg.timeStamp, this.dateFormat, this.timeFormat);
  }
  openLink(link) {
    this.headerNavItemClicked.emit(link);
    this.closeDialog();
  }
  setSearchString(string) {
    this.searchString = string;
  }
  getMessages() {
    return [...this.messages]
      .filter((msg) => !this.searchString ||
      (msg.title ? msg.title.toLowerCase() : '').includes(this.searchString.toLowerCase()) ||
      (msg.description ? msg.description.toLowerCase() : '').includes(this.searchString.toLowerCase()))
      .filter((msg) => (this.filterArray.length > 0 ? this.filterArray.includes(msg.class) : false))
      .sort((a, b) => this.sortMessagesBasedOnTime(a, b))
      .map((msg) => {
      var _a;
      msg.isExpanded = !!msg.isExpanded;
      msg.description = (_a = msg.description) !== null && _a !== void 0 ? _a : '-';
      return msg;
    });
  }
  sortMessagesBasedOnTime(a, b) {
    if (this.sortOrder === '') {
      return 0;
    }
    else if (this.sortOrder === SortOrder.Ascending.valueOf()) {
      return a.timeStamp - b.timeStamp;
    }
    else if (this.sortOrder === SortOrder.Descending.valueOf()) {
      return b.timeStamp - a.timeStamp;
    }
  }
  updateMessage(message, updates) {
    const toBeUpdated = this.messages.find((msg) => JSON.stringify(msg) === JSON.stringify(message));
    if (toBeUpdated) {
      Object.assign(toBeUpdated, updates);
    }
    this.messages = [...this.messages];
  }
  deleteMessages() {
    this.controller.deleteMessages(this.localStorageKey);
    this.messages = [];
  }
  closeDialog() {
    this.messages.forEach((msg) => (msg.isRead = true));
    this.messages = [...this.messages];
    this.controller.updateMessages(this.messages, this.localStorageKey);
    this.dialog.open();
    this.dialog.close('close');
  }
  render() {
    return (h("div", { class: "mdc-dialog bci-portal-message-history-dialog" },
      h("div", { class: "mdc-dialog__container" },
        h("div", { class: "mdc-dialog__surface" },
          h("div", { class: "bci-portal-dialog-header" },
            h("span", { "data-test": "webcore.webcomponents.messages.close.span", class: "bosch-ic bosch-ic-clickable bosch-ic-close bci-portal-x-button", onClick: () => this.closeDialog() }),
            h("div", { class: "bci-portal-dialog-header-title" }, translate('messages'))),
          h("div", { class: "bci-portal-dialog-filter" },
            h("div", { class: "form-group search-input" },
              h("input", { "data-test": "webcore.webcomponents.messages.search.input", value: this.searchString, onInput: (event) => this.setSearchString(event.target.value) }),
              h("label", { class: "label-top" }, translate('search'))),
            h("div", { class: "bci-portal-message-filter-container" },
              h("div", { class: "form-group", "data-test": "webcore.webcomponents.messages.show.div", onClick: () => (this.showMessgeFilterOptions = !this.showMessgeFilterOptions) },
                h("input", { "data-test": "webcore.webcomponents.messages.portal-message-filter-input", type: "text", readOnly: true, class: "bci-portal-message-filter-input", ref: (el) => (this.filterSelectList = el) }),
                h("div", { class: "bci-portal-message-filter-toggle" },
                  h("span", { "data-test": "webcore.webcomponents.messages.message-filter-toggle.bosch-ic-up", class: 'bosch-ic bosch-ic-up bosch-ic-clickable ' + (this.showMessgeFilterOptions ? 'show' : 'hide') }),
                  h("span", { "data-test": "webcore.webcomponents.messages.message-filter-toggle.bosch-ic-down", class: 'bosch-ic  bosch-ic-down bosch-ic-clickable ' + (this.showMessgeFilterOptions ? 'hide' : 'show') })),
                h("label", { class: "label-top" }, translate('messageFilters'))),
              h("div", { class: 'bci-portal-message-filters-options ' + (this.showMessgeFilterOptions ? 'show' : 'hide') },
                h("span", { class: "bci-portal-message-group" }, translate('messageTypes')),
                h("bci-checkbox-collection", { "data-test": "webcore.webcomponents.messages.multi-select", type: CollectionType.Multi, list: MESSAGE_TYPE_FILTERS, onMultiItemSelected: (ev) => this.multiFilterSelected(ev) }),
                h("span", { class: "bci-portal-message-group" }, translate('dateTime')),
                h("bci-checkbox-collection", { "data-test": "webcore.webcomponents.messages.single-select", type: CollectionType.Single, list: MESSAGE_SORT_FILTER, onSingleItemSelected: (ev) => this.singleFilterSelected(ev) })))),
          h("div", { class: `bci-portal-dialog-content ${this.getMessages().length === 0 ? 'empty' : ''}` },
            h("div", { class: "bci-portal-dialog-content-empty-hint" }, translate('emptyMessageSearchResult')),
            this.getMessages().map((message) => (h("div", { "data-test": "webcore.webcomponents.messages.portal-message-history-entry.expand", class: `bci-portal-message-history-entry ${message.isExpanded ? 'expanded' : ''} ${this.controller.determineClassColor(message)}`, onClick: () => {
                this.updateMessage(message, { isExpanded: !message.isExpanded });
              } },
              h("div", { class: "bci-portal-message-history-entry-icon" },
                h("span", { class: this.controller.determineClassIcon(message) })),
              h("div", { class: "bci-portal-message-history-entry-flex-container" },
                h("div", { class: "datetime" },
                  h("span", { class: "label" }, translate('dateTime')),
                  this.dateToDayString(message)),
                h("div", { class: "title" },
                  h("span", { class: "label" }, translate('title')),
                  h("span", { class: "txt" }, message.title)),
                h("div", { class: "description" },
                  h("span", { class: "label" }, translate('description')),
                  h("span", { class: "txt" }, message.description),
                  message.link && message.isExpanded ? (h("button", { "data-test": "webcore.webcomponents.messages.portal-message.show-more-details-button", type: "button", class: "show-more button-integrated", onClick: () => this.openLink(message.link) }, translate('showMoreDetails'))) : (''))),
              h("div", { class: "expand-icon" },
                h("span", { class: {
                    'bosch-ic': true,
                    'bosch-ic-medium': true,
                    'bosch-ic-clickable': true,
                    'bosch-ic-up': message.isExpanded,
                    'bosch-ic-down': !message.isExpanded,
                  } })))))),
          h("footer", { class: "mdc-dialog__actions" },
            h("button", { type: "button", class: "button-secondary", "data-test": "webcore.webcomponents.messages.delete.button", onClick: () => this.deleteMessages() },
              h("span", { class: "mdc-button__label" }, translate('discardAll'))),
            h("button", { type: "button", class: "button-primary", "data-test": "webcore.webcomponents.messages.close.button", onClick: () => this.closeDialog() },
              h("span", { class: "mdc-button__label" }, translate('close')))))),
      h("div", { class: "mdc-dialog__scrim", "data-test": "webcore.webcomponents.messages.close.div", onClick: () => this.closeDialog() })));
  }
  static get is() { return "bci-portal-messages"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["messages.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["messages.component.css"]
  }; }
  static get properties() { return {
    "dateFormat": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "date-format",
      "reflect": false
    },
    "timeFormat": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "time-format",
      "reflect": false
    },
    "localStorageKey": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "local-storage-key",
      "reflect": false
    }
  }; }
  static get states() { return {
    "messages": {},
    "searchString": {},
    "filterArray": {},
    "sortOrder": {},
    "showMessgeFilterOptions": {}
  }; }
  static get events() { return [{
      "method": "headerNavItemClicked",
      "name": "bci.portal.header.navItemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "bci.portal.message.created",
      "method": "onMessageCreated",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.message.updated",
      "method": "loadMessages",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}

