/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Element, Event, h, Host, Listen, Prop, Watch } from '@stencil/core';
import { translate } from '../../../utils/utils';
import { WidthObserver } from '../service/width-observer-service';
/**
 *  Only the MultiSelector-Overlay is providing the information of its children for the parent components
 *  Therefore No Event of the children should cross this layer
 */
export class MultiSelectorOverlay {
  constructor() {
    /** title of the overlay header*/
    this.headerLabel = '';
    /** text that needs to be highlighted in the result list*/
    this.filterText = '';
    /** rendered/shown levels on tree */
    this.activeLevels = [];
    /** enable or disable resetSelectionButton */
    this.resetSelectionButtonEnabled = true;
  }
  onEnforceViewportChange(enforceViewport) {
    return this.applyViewport(enforceViewport);
  }
  /** Intercept the nodeSelectionChanged and transform it to a SelectionChangedEvent */
  handleNodeSelectionChange(event) {
    event.stopPropagation();
    this.nodeSelectionChange.emit(event.detail);
  }
  handleNodeActiveChange(event) {
    event.stopPropagation();
    this.nodeActiveChange.emit(event.detail);
  }
  componentDidLoad() {
    this.widthObserver = new WidthObserver(this.container, {
      onVp2: () => {
        if (!this.enforceViewport) {
          this.showVP2();
        }
      },
      onVp3: () => {
        if (!this.enforceViewport) {
          this.showVP3();
        }
      },
    });
    this.applyViewport(this.enforceViewport);
    this.widthObserver.connect();
  }
  applyViewport(enforceViewport) {
    const previousViewportWithoutVP1 = this.widthObserver.currentViewport === 'vp1' ? 'vp2' : this.widthObserver.currentViewport;
    switch (enforceViewport || previousViewportWithoutVP1) {
      case 'vp1':
        return this.showVP1();
      case 'vp2':
        return this.showVP2();
      case 'vp3':
        return this.showVP3();
    }
  }
  showVP1() {
    this.manageClasses('vp1', ['vp2', 'vp3']);
  }
  showVP2() {
    this.manageClasses('vp2', ['vp1', 'vp3']);
  }
  showVP3() {
    this.manageClasses('vp3', ['vp1', 'vp2']);
  }
  manageClasses(setClass, removeClasses) {
    this.container.classList.add(setClass);
    removeClasses.forEach((removeClass) => this.container.classList.remove(removeClass));
  }
  connectedCallback() {
    // e.g. called when element is moved in DOM
    if (!!this.container) {
      this.widthObserver.connect();
    }
  }
  disconnectedCallback() {
    this.widthObserver.disconnect();
  }
  render() {
    return (h(Host, null,
      h("div", { class: "overlay-wrapper", ref: (el) => (this.container = el) },
        h("header", { class: "not-vp1" },
          h("a", { class: "close-button", onClick: () => this.finishSelection(), "data-test": "webcore.webcomponents.multi-selector.vp1-close" },
            h("bci-icon", { name: "close" })),
          h("h3", { class: "small" },
            translate('selectSomething').replace('{{search-type-label}}', this.headerLabel),
            " ")),
        h("div", { class: "not-vp1" },
          h("slot", { name: "input-and-label" })),
        h("div", { class: "selector-wrapper" },
          h("main", null,
            h("bci-multi-selector-tree", { filterText: this.filterText, activeLevels: this.activeLevels, onNavigateBack: () => this.handleNavigateBack(), isLoading: this.isLoading })),
          h("footer", null,
            h("button", { "data-test": "webcore.webcomponents.multi-selector.reset-selection", class: "button-sm no-arrow reset", disabled: !this.resetSelectionButtonEnabled, onClick: () => this.selectionReset.emit() },
              h("bci-icon", { name: "reset", class: "reset-icon" }),
              translate('reset-selection')),
            this.showSave ? (h("button", { "data-test": "webcore.webcomponents.multi-selector.save", class: "button-link button-sm no-arrow save", onClick: () => this.finishSelection() }, translate('save'))) : (''))),
        h("button", { "data-test": "webcore.webcomponents.multi-selector.close", class: "not-vp1 close-button-bottom button-primary", onClick: () => this.finishSelection() }, translate('close')))));
  }
  finishSelection() {
    this.selectionFinished.emit();
  }
  handleNavigateBack() {
    this.navigateBack.emit();
  }
  static get is() { return "bci-multi-selector-overlay"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["multi-selector-overlay.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["multi-selector-overlay.css"]
  }; }
  static get properties() { return {
    "headerLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "title of the overlay header"
      },
      "attribute": "header-label",
      "reflect": false,
      "defaultValue": "''"
    },
    "filterText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "text that needs to be highlighted in the result list"
      },
      "attribute": "filter-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "showSave": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show save button"
      },
      "attribute": "show-save",
      "reflect": false
    },
    "enforceViewport": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ViewportClasses",
        "resolved": "\"vp1\" | \"vp2\" | \"vp3\"",
        "references": {
          "ViewportClasses": {
            "location": "import",
            "path": "../service/width-observer-service"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The viewport logic is ignored if this property is set"
      },
      "attribute": "enforce-viewport",
      "reflect": false
    },
    "activeLevels": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Level[]",
        "resolved": "Level[]",
        "references": {
          "Level": {
            "location": "import",
            "path": "./multi-selector-level/model/multi-selector-level"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "rendered/shown levels on tree"
      },
      "defaultValue": "[]"
    },
    "resetSelectionButtonEnabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "enable or disable resetSelectionButton"
      },
      "attribute": "reset-selection-button-enabled",
      "reflect": false,
      "defaultValue": "true"
    },
    "isLoading": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "show a loading spinner"
      },
      "attribute": "is-loading",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "selectionFinished",
      "name": "selectionFinished",
      "bubbles": true,
      "cancelable": true,
      "composed": false,
      "docs": {
        "tags": [],
        "text": "Provides an event on saving the selection"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "nodeSelectionChange",
      "name": "nodeSelectionChange",
      "bubbles": true,
      "cancelable": true,
      "composed": false,
      "docs": {
        "tags": [],
        "text": "Event includes all selected elements and emits when any element selection was added/removed/changed"
      },
      "complexType": {
        "original": "NodeSelectionChangedEvent",
        "resolved": "NodeSelectionChangedEvent",
        "references": {
          "NodeSelectionChangedEvent": {
            "location": "import",
            "path": "../model"
          }
        }
      }
    }, {
      "method": "clearSearch",
      "name": "clearSearch",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Clear the search filed"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "navigateBack",
      "name": "navigateBack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "`triggered on click on back button"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "selectionReset",
      "name": "selectionReset",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "reset of the selection"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "nodeActiveChange",
      "name": "nodeActiveChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "triggered when active state of a node has changed"
      },
      "complexType": {
        "original": "NodeActiveChangedEvent",
        "resolved": "NodeActiveChangedEvent",
        "references": {
          "NodeActiveChangedEvent": {
            "location": "import",
            "path": "../model"
          }
        }
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "enforceViewport",
      "methodName": "onEnforceViewportChange"
    }]; }
  static get listeners() { return [{
      "name": "nodeSelectionChanged",
      "method": "handleNodeSelectionChange",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "nodeActiveChanged",
      "method": "handleNodeActiveChange",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}

