/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { groupNodesByKey } from '../../../../service/data-source-service';
/** Stateless Service to handle the Multi-Selector Tree */
/**
 * returns an array of the nodes along the given nodePath and given node link ids as a flat array
 *
 * @param tree
 * @param nodePath
 * @returns
 * @example
 * Data set (tree structure):
 * BaP
 * - A100
 * -- L1c
 * -- L1b
 * - A200
 * -- L2a
 * -- L2b
 * AnP
 * - A300
 * -- L3a
 * -- L3c
 * - A400
 * -- L4b
 * -- L4c
 *
 * Path: BaP, A200, L2a
 *
 * Result (array): [
 * {
 * BaP
 * - A100
 * -- L1c
 * -- L1b
 * - A200
 * -- L2a
 * -- L2b},
 * {
 * - A200
 * -- L2a
 * -- L2},
 * {
 * -- L2}
 * ]
 *
 * The nodes are returned in an array form, but they retain there references to there children
 */
export function getNodesFromTreeByNodePath(tree, nodePath) {
  let partialTree = tree;
  let nodes = [];
  for (const nodeLink of nodePath) {
    const nextNode = partialTree.find((node) => node.id === nodeLink.id);
    if (!nextNode) {
      break;
    }
    partialTree = nextNode.children;
    nodes.push(nextNode);
  }
  return nodes;
}
/**
 * sets all nodes and subNodes inactive in the given tree (deep)
 * @param tree
 * @returns
 */
export function setAllNodesInTreeToInactive(tree) {
  if (!tree || tree.length === 0) {
    return [];
  }
  tree.map((node) => {
    const dirtyNode = node.active;
    node.active = false;
    node.children = setAllNodesInTreeToInactive(node.children);
    if (dirtyNode) {
      return cloneNode(node);
    }
    return node;
  });
  return tree;
}
/**
 * clones the given node and sets also the proper connection to the parent Node (if exists)
 * !!No deep cloning!!
 * @param node
 * @returns
 */
export function cloneNode(node) {
  const clone = Object.assign({}, node);
  if (clone.parent) {
    const parent = clone.parent;
    for (let childIndex = 0; childIndex < parent.children.length; childIndex++) {
      const child = parent.children[childIndex];
      if (child.id === clone.id) {
        parent.children[childIndex] = clone;
      }
    }
  }
  return clone;
}
/**
 * returns all active levels of the tree (deep)
 * @param tree
 * @returns
 */
export function getActiveLevels(tree, previousLevelActiveNode = null) {
  if (tree.length === 0) {
    return [];
  }
  const firstLevel = createLevelByNodes(tree, previousLevelActiveNode);
  let activeLevels = [firstLevel];
  const activeNode = tree.find((node) => node.active);
  if (activeNode && activeNode.children) {
    activeLevels = activeLevels.concat(getActiveLevels(activeNode.children, activeNode));
  }
  return activeLevels;
}
export function getNodeFromTreeByNodePath(tree, nodePath) {
  if (!nodePath) {
    return null;
  }
  const nodes = getNodesFromTreeByNodePath(tree, nodePath);
  if (nodes.length !== nodePath.length) {
    return undefined;
  }
  return nodes[nodes.length - 1];
}
/**
 * create a level with sections
 * @param nodes
 * @returns
 */
export function createLevelByNodes(nodes, previousLevelActiveNode = null) {
  const level = {
    parentNode: previousLevelActiveNode,
    sections: [],
  };
  level.sections = createLevelSectionsByNodes(nodes);
  return level;
}
/**
 * create a level with sections
 * @param nodes
 * @returns
 */
export function createLevelSectionsByNodes(nodes) {
  const levelObj = groupNodesByKey(nodes, 'typeLabel');
  return Object.entries(levelObj)
    .map(([key, items]) => {
    return {
      label: key === 'undefined' ? '' : key,
      items: sortByLabelAsc(items),
    };
  })
    .sort((sectionA, sectionB) => sectionA.label.localeCompare(sectionB.label));
}
export function getNodesByLevel(level) {
  return level.sections.reduce((acc, levelSection) => acc.concat(levelSection.items), []);
}
export function sortByLabelAsc(items) {
  return items.sort((sectionA, sectionB) => sectionA.label.localeCompare(sectionB.label));
}

