/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Event, h, Host, Prop, State, Watch } from '@stencil/core';
import * as menuSurfaceConstants from '@material/menu-surface/constants';
import { KEY, normalizeKey } from '@material/dom/keyboard';
import { setLocale } from '../../../utils/locale-store';
import { MDCMenu } from '@material/menu/component';
import { translate } from '../../../utils/utils';
import { Corner } from '@material/menu';
export class MultiSelectorSearchFilter {
  constructor() {
    this.open = false;
    this.closing = false;
    this.handleMenuOpened = () => {
      this.open = true;
    };
    this.handleMenuClosed = () => {
      this.open = false;
      this.closing = false;
    };
    this.handleMenuClosing = () => {
      this.closing = true;
    };
  }
  async componentWillLoad() {
    await setLocale();
    this.handleNewSearchTypes(this.searchTypes);
  }
  componentDidLoad() {
    if (this.menuElement !== undefined) {
      this.mdcMenu = new MDCMenu(this.menuElement);
      this.mdcMenu.listen(menuSurfaceConstants.strings.OPENED_EVENT, this.handleMenuOpened);
      this.mdcMenu.listen(menuSurfaceConstants.strings.CLOSED_EVENT, this.handleMenuClosed);
      this.mdcMenu.listen(menuSurfaceConstants.strings.CLOSING_EVENT, this.handleMenuClosing);
    }
  }
  disconnectedCallback() {
    if (this.mdcMenu !== undefined) {
      this.mdcMenu.unlisten(menuSurfaceConstants.strings.OPENED_EVENT, this.handleMenuOpened);
      this.mdcMenu.unlisten(menuSurfaceConstants.strings.CLOSED_EVENT, this.handleMenuClosed);
      this.mdcMenu.unlisten(menuSurfaceConstants.strings.CLOSING_EVENT, this.handleMenuClosing);
      this.mdcMenu.destroy();
    }
  }
  handleNewSearchTypes(newValue) {
    this.selectedSearchType = !!this.defaultSearchType || !newValue || newValue.length < 1 ? this.defaultSearchType : newValue[0];
    if (!!this.mdcMenu && this.mdcMenu.open && (!newValue || newValue.length < 2)) {
      this.mdcMenu.open = false;
    }
  }
  handleNewDefaultSearchType(newValue) {
    this.selectedSearchType = newValue;
  }
  render() {
    return h(Host, null, this.renderDropdownMenu());
  }
  toggleMenu() {
    if (this.mdcMenu) {
      this.mdcMenu.setAnchorCorner(Corner.BOTTOM_LEFT);
      // click on toggle = click outside menu -> only toggle menu itself if status matches
      if (!this.closing) {
        this.mdcMenu.open = !this.mdcMenu.open;
      }
    }
  }
  handleToggleKeydown(event) {
    const isEnter = normalizeKey(event) === KEY.ENTER;
    if (isEnter) {
      this.toggleMenu();
      event.preventDefault();
    }
  }
  handleSelectSearchType(newSearchType) {
    this.selectedSearchType = newSearchType;
    this.selectSearchType.emit(newSearchType);
    this.toggleMenu();
  }
  renderDropdownMenu() {
    return (h("div", { class: "mdc-menu-surface--anchor", onClick: () => {
        (!this.searchTypes || this.searchTypes.length < 2) && this.clickDisabledSearchType.emit();
      } },
      this.renderToggle(),
      h("div", { class: "mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth", ref: (el) => (this.menuElement = el) }, this.renderSearchTypeOptions())));
  }
  renderToggle() {
    return !this.searchTypes || this.searchTypes.length < 2 ? (h("div", { class: "multi-selector-search-filter-placeholder", "data-test": "webcore.webcomponents.multi-selector.search-filter.menu.toggle" },
      h("bci-icon", { name: "search" }),
      h("span", { class: "dropdown-arrow disabled " }))) : (h("div", { class: `multi-selector-search-filter-button bosch-ic-clickable ${this.open || this.closing ? 'mdc-menu--activated' : ''}`, tabindex: "0", "data-test": "webcore.webcomponents.multi-selector.search-filter.menu.toggle", onClick: () => this.toggleMenu(), onKeyDown: (event) => this.handleToggleKeydown(event) },
      h("bci-icon", { name: "search", clickable: true }),
      h("span", { class: "dropdown-arrow" })));
  }
  renderSearchTypeOptions() {
    if (!!this.searchTypes && this.searchTypes.length > 0) {
      return (h("ul", { class: "mdc-list", role: "menu", "aria-hidden": "true", "aria-orientation": "vertical", tabindex: "-1" },
        h("li", null,
          h("span", { class: "mdc-menu__selection-group-header" }, translate('searchType')),
          h("ul", { class: "mdc-menu__selection-group" }, this.searchTypes.map((option, index) => this.renderSearchTypeOption(option, index))))));
    }
  }
  renderSearchTypeOption(option, index) {
    let isSelected = !!this.selectedSearchType && this.selectedSearchType.id === option.id;
    return (h("li", { "data-test": "webcore.webcomponents.multi-selector.search-type.option", "data-value": option, key: index, role: "menuitem", "aria-selected": `${String(isSelected)}`, tabindex: "0", onClick: () => this.handleSelectSearchType(option), class: `mdc-list-item ${isSelected ? 'mdc-list-item--selected mdc-menu-item--selected' : ''}` },
      h("span", { class: "mdc-list-item__graphic mdc-menu__selection-group-icon", "aria-hidden": "true" },
        h("bci-icon", { name: "checkmark" })),
      h("span", { class: "mdc-list-item__text" }, option.label)));
  }
  static get is() { return "bci-multi-selector-search-filter"; }
  static get originalStyleUrls() { return {
    "$": ["multi-selector-search-filter.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["multi-selector-search-filter.css"]
  }; }
  static get properties() { return {
    "searchTypes": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "RootNode[]",
        "resolved": "RootNode[]",
        "references": {
          "RootNode": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "the list of searchTypes to choose out of within the dropdown"
      }
    },
    "defaultSearchType": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "RootNode",
        "resolved": "RootNode",
        "references": {
          "RootNode": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "the default searchType"
      }
    }
  }; }
  static get states() { return {
    "selectedSearchType": {},
    "open": {},
    "closing": {}
  }; }
  static get events() { return [{
      "method": "selectSearchType",
      "name": "selectSearchType",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "emits on search type selection a new rootNode the multi-selector displays the tree for"
      },
      "complexType": {
        "original": "RootNode",
        "resolved": "RootNode",
        "references": {
          "RootNode": {
            "location": "import",
            "path": "../model"
          }
        }
      }
    }, {
      "method": "clickDisabledSearchType",
      "name": "clickDisabledSearchType",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "emits on search type click if it is disabled"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "searchTypes",
      "methodName": "handleNewSearchTypes"
    }, {
      "propName": "defaultSearchType",
      "methodName": "handleNewDefaultSearchType"
    }]; }
}

