/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, Prop, Event, Listen, Watch, State, Host } from '@stencil/core';
import { ExpanderIcon } from '../expander-icon';
import { NavigationController } from '../../../utils/navigation-controller';
export class NavigationItemComponent {
  constructor() {
    /**
     * controls whether window path has hash navigations
     */
    this.hashNavigation = true;
    this.navigationClosed = false;
    this.expanded = false;
  }
  handlePopStateEvent() {
    // activate related to the browser location the related main and child item
    if (this.pathMatch(this.item.path)) {
      this.setActive(true);
      return;
    }
    if ((this.item.children || []).some((childItem) => this.pathMatch(childItem.path) || (childItem.children || []).some((grandChildItem) => this.pathMatch(grandChildItem.path)))) {
      this.expanded = true;
      return;
    }
    this.setActive(false);
    this.expanded = false;
  }
  watchNavigationClosed(newValue) {
    this.navigationClosed = newValue;
  }
  watchItem(newValue) {
    this.item = newValue;
    this.expanded = !!this.item.expanded && this.item.children && this.item.children.length > 0;
  }
  /**
   * Popstate is sometimes not triggered on page initialization. With this methiod
   * it is ensured that active and expanded state of navigation item are set.
   */
  componentWillLoad() {
    this.expanded = this.item && !!this.item.expanded;
    this.handlePopStateEvent();
  }
  pathMatch(path) {
    if (!path) {
      return false;
    }
    const itemPath = NavigationController.preprocessPath(path);
    const windowPath = NavigationController.preprocessPath(`${window.location.pathname}${this.hashNavigation ? window.location.hash || '' : ''}`);
    return !!path && windowPath === itemPath;
  }
  /**
   * Navigation item is clicked. Expanded state is changed and a event for clicked or expanded is emitted.
   * The expanded state has to be changed because the watcher does not get this information.
   */
  selectItem(event) {
    event.preventDefault();
    if (this.item.icon && this.item.children) {
      if (!this.navigationClosed || !this.expanded) {
        this.expanded = !this.expanded;
      }
      this.navigationItemExpanded.emit(this.item);
    }
    else if (this.item.jsFiles || this.item.htmlFile) {
      this.navigationItemClicked.emit(this.item);
    }
  }
  /**
   * Change active state of navigation item and rerender.
   *
   * @param active new active state
   */
  setActive(active) {
    this.item.active = active;
    if (!active && this.item.children) {
      this.item.expanded = false;
      this.item.children.forEach((item) => {
        item.active = false;
      });
    }
    this.item = Object.assign({}, this.item);
  }
  /**
   * Add class bci-portal-navigation-item-expandable to host element to enable expand icon rotation if expanded.
   *
   * @returns class list
   */
  navigationItemExpandedClass() {
    return this.expanded && !this.navigationClosed && this.expanderIcon !== ExpanderIcon.Right
      ? 'bci-portal-navigation-item-expandable'
      : '';
  }
  /**
   * Navigation item class list. Contains
   * - bci-portal-navigation-item-active to highlight if the item (or any child item if collapsed) is active
   * - bci-portal-navigation-item-has-childs to make the background of active child elements darker
   *
   * @returns class list
   */
  navigationItemClass() {
    let className = 'bci-portal-navigation-item';
    const hasActiveChildren = (acc, cur) => {
      if (cur.active || (cur.children && cur.children.reduce(hasActiveChildren, false))) {
        return true;
      }
      return acc;
    };
    const hasActiveChildItem = this.item.children && this.item.children.reduce(hasActiveChildren, false);
    const collapsedActiveItem = this.item.active && !this.expanded;
    const minifiedActiveItem = this.navigationClosed && hasActiveChildItem;
    const collapsedMainMenuItem = !this.expanded && this.expanderIcon === ExpanderIcon.Down && hasActiveChildItem;
    const activeSubMenuItem = hasActiveChildItem && !this.item.icon;
    if (collapsedActiveItem || minifiedActiveItem || collapsedMainMenuItem || activeSubMenuItem) {
      className += ' bci-portal-navigation-item-active';
    }
    if (hasActiveChildItem) {
      className += ' bci-portal-navigation-item-has-childs';
    }
    return className;
  }
  /**
   * Class list for the div which contains the text paragraph.
   *
   * @returns class list
   */
  navigationItemTextClass() {
    let className = 'bci-portal-navigation-item-text';
    if ((!this.item.children && !this.item.icon) || !this.item.icon) {
      className += ' bci-portal-navigation-item-sub-text';
    }
    return className;
  }
  navigationPath() {
    if (this.item.navigationPath) {
      return '/' + this.item.navigationPath.join('/');
    }
  }
  render() {
    return (h(Host, { class: this.navigationItemExpandedClass(), "data-test-nav": this.item.title, "data-test-path": this.navigationPath() },
      h("div", { "data-test": "webcore.webcomponents.navigation-item.select.div", onClick: (event) => this.selectItem(event), class: this.navigationItemClass() },
        h("a", { title: this.item.title, class: "bci-portal-navigation-item-content", href: !!this.item.path ? this.item.path : undefined },
          this.item.icon && h("span", { class: `bci-portal-navigation-item-icon-category bosch-ic ${this.item.icon}` }),
          h("div", { class: this.navigationItemTextClass() },
            h("p", null, this.item.title)),
          this.item.children && h("span", { class: `bci-portal-navigation-item-icon-expand bosch-ic ${this.expanderIcon}` }))),
      this.item.children && (h("div", { class: "bci-portal-navigation-items" },
        h("slot", null)))));
  }
  static get is() { return "bci-portal-navigation-item"; }
  static get originalStyleUrls() { return {
    "$": ["navigation-item.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["navigation-item.component.css"]
  }; }
  static get properties() { return {
    "hashNavigation": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "controls whether window path has hash navigations"
      },
      "attribute": "hash-navigation",
      "reflect": false,
      "defaultValue": "true"
    },
    "item": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "SidebarNavItem",
        "resolved": "SidebarNavItem",
        "references": {
          "SidebarNavItem": {
            "location": "import",
            "path": "../../../interfaces/navigation.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "expanderIcon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ExpanderIcon",
        "resolved": "ExpanderIcon.Down | ExpanderIcon.Right",
        "references": {
          "ExpanderIcon": {
            "location": "import",
            "path": "../expander-icon"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "expander-icon",
      "reflect": false
    },
    "navigationClosed": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "navigation-closed",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "expanded": {}
  }; }
  static get events() { return [{
      "method": "navigationItemExpanded",
      "name": "bci.portal.navigation.itemExpanded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "SidebarNavItem",
        "resolved": "SidebarNavItem",
        "references": {
          "SidebarNavItem": {
            "location": "import",
            "path": "../../../interfaces/navigation.interface"
          }
        }
      }
    }, {
      "method": "navigationItemClicked",
      "name": "bci.portal.navigation.itemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "SidebarNavItem",
        "resolved": "SidebarNavItem",
        "references": {
          "SidebarNavItem": {
            "location": "import",
            "path": "../../../interfaces/navigation.interface"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "navigationClosed",
      "methodName": "watchNavigationClosed"
    }, {
      "propName": "item",
      "methodName": "watchItem"
    }]; }
  static get listeners() { return [{
      "name": "hashchange",
      "method": "handlePopStateEvent",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "popstate",
      "method": "handlePopStateEvent",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.changedUrl",
      "method": "handlePopStateEvent",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}

