/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Event, h, Host, Listen, Prop, State, Watch } from '@stencil/core';
import { NavigationController } from '../../utils/navigation-controller';
import { ExpanderIcon } from './expander-icon';
const SCREEN_SIZE_TABLET = 768;
export class NavigationComponent {
  constructor() {
    /**
     * controls whether window path has hash navigations
     */
    this.hashNavigation = true;
    this.navigationItems = [];
    this.threeLevels = false;
    this.useLogo = false;
    /**
     * controls whether the bci.portal.navigation.itemClicked event is triggered upon startup or not
     */
    this.emitItemClickOnStartup = true;
    this.toggleNavigationAnimationActive = false;
  }
  handleClickNavItemEvent(item) {
    var _a;
    // updates the selected item. This is most likely not directly required and depends how quickly the
    // page reloads. History events are processed on the items itself e.g. updating states.
    this.selectedNavItem = (_a = new NavigationController().setActiveNavigationByPath(item.detail.path, this.navigationItems)) !== null && _a !== void 0 ? _a : undefined;
    if (this.navigationItems.filter((navItem) => this.sidebarNavItemsEqual(navItem, item.detail)).length > 0) {
      this.expandedNavItem = undefined;
    }
    const browserWidth = window.innerWidth || document.body.offsetWidth;
    if (browserWidth <= SCREEN_SIZE_TABLET) {
      this.toggleNavigation(true);
    }
  }
  handleExpandedNavItemEvent(event) {
    const activeItem = this.navigationItems.find((navItem) => this.sidebarNavItemsEqual(navItem, event.detail));
    if (!activeItem) {
      return;
    }
    if (this.closed && this.expandedNavItem && this.sidebarNavItemsEqual(this.expandedNavItem, activeItem)) {
      return this.toggleNavigation(false);
    }
    this.toggleNavigation(false);
    const expandNavItem = !this.expandedNavItem || (this.expandedNavItem && !this.sidebarNavItemsEqual(activeItem, this.expandedNavItem));
    this.expandedNavItem = expandNavItem ? activeItem : undefined;
  }
  handleToggleNavigationEvent(event) {
    this.closed = event.detail && typeof event.detail.closed !== 'undefined' ? event.detail.closed : !this.closed;
    this.toggleNavigationAnimationActive = true;
    // send resize event after animation time is over
    setTimeout(() => {
      window.dispatchEvent(new CustomEvent('resize'));
    }, 200);
  }
  loadNavItem() {
    var _a;
    // Sets the selected item mainly required after reloading the page.
    if (Array.isArray(this.navigationItems) && this.navigationItems.length > 0) {
      this.selectedNavItem =
        (_a = new NavigationController().setActiveNavigationByPath(`${window.location.pathname}${window.location.hash || ''}`, this.navigationItems)) !== null && _a !== void 0 ? _a : undefined;
      this.detectExpandedNavItem();
      if (this.selectedNavItem && this.emitItemClickOnStartup) {
        this.navigationItemClicked.emit(this.selectedNavItem);
      }
    }
  }
  componentWillLoad() {
    this.loadNavItem();
  }
  /**
   * Check if SidebarNavItems are equal. Dynamic attributes are ignored.
   */
  sidebarNavItemsEqual(item1, item2) {
    return item1.title === item2.title && item1.path === item2.path;
  }
  /**
   * Detect which navigation item is expanded and set the expandedNavItem property if the expanded navigation
   * item has children. Otherwise a null will be set.
   */
  detectExpandedNavItem() {
    const expandedItem = this.navigationItems.find((item) => item.expanded);
    this.expandedNavItem = expandedItem && expandedItem.children && expandedItem.children.length > 0 ? expandedItem : undefined;
  }
  /**
   * Check if a child or any sub child navigation item is active.
   *
   * @param item SidebarNavItem
   */
  hasActiveChildItem(item) {
    if (!item.active || !item.children) {
      return false;
    }
    return item.children.some((child) => this.hasActiveChildItem(child));
  }
  renderFlatItem(item, showChildItems = !this.threeLevels) {
    const className = !showChildItems && this.hasActiveChildItem(item) ? 'bci-portal-navigation-item-active' : '';
    const expanderIcon = !showChildItems ? ExpanderIcon.Right : ExpanderIcon.Down;
    return (h("bci-portal-navigation-item", { hashNavigation: this.hashNavigation, "data-test": "webcore.webcomponents.navigation.component.bci-portal-navigation-item.navigation-close", class: className, item: item, expanderIcon: expanderIcon, navigationClosed: this.closed }, showChildItems &&
      item.children &&
      item.children.map((childItem) => (h("bci-portal-navigation-item", { hashNavigation: this.hashNavigation, item: childItem })))));
  }
  renderThreeFoldMenu() {
    var _a, _b, _c;
    return (h("bci-portal-navigation-submenu", { expandedNavItem: this.expandedNavItem },
      h("div", null, (_c = (_b = (_a = this.expandedNavItem) === null || _a === void 0 ? void 0 : _a.children) === null || _b === void 0 ? void 0 : _b.map((item) => this.renderFlatItem(item, true))) !== null && _c !== void 0 ? _c : '')));
  }
  render() {
    return (h(Host, { class: { 'bci-portal-sidebar-navigation-host-closed': this.closed } },
      h("div", { class: 'bci-portal-sidebar-navigation' +
          (this.closed ? ' bci-portal-sidebar-navigation-closed' : ' bci-portal-sidebar-navigation-open') +
          (this.toggleNavigationAnimationActive ? ' bci-portal-sidebar-navigation-transition' : '') },
        h("section", { class: "bci-portal-sidebar-navigation-container" },
          h("header", { class: "bci-portal-sidebar-header" },
            h("h1", { class: "bci-portal-sidebar-header-title" }, this.useLogo ? h("img", { class: "bci-portal-logo" }) : this.mainTitle),
            h("div", { class: "bci-portal-burger burger", "data-test": "webcore.webcomponents.navigation.toggle.div", onClick: () => this.toggleNavigation() },
              h("div", { class: "bci-portal-burger-line bci-portal-burger-top" }),
              h("div", { class: "bci-portal-burger-line bci-portal-burger-middle" }),
              h("div", { class: "bci-portal-burger-line bci-portal-burger-bottom" }))),
          h("section", { class: 'bci-portal-sidebar-navigation-container' +
              (this.threeLevels ? ' bci-portal-sidebar-navigation-container-three-level-mode' : '') },
            h("div", { "data-test": "webcore.webcomponents.navigation.component.bci-portal-navigation-list.expanded-nav-item", class: 'bci-portal-navigation-list' + (this.threeLevels && this.expandedNavItem ? ' bci-portal-navigation-list-closed' : '') },
              [0, 1, 2, 3, 4, 5].map(() => (h("div", { class: 'bci-portal-navigation-item-placeholder' +
                  (!Array.isArray(this.navigationItems) || this.navigationItems.length === 0 ? '' : ' hidden') },
                h("div", { class: "icon-placeholder" }),
                h("div", { class: "text-placeholder" })))),
              Array.isArray(this.navigationItems) &&
                this.navigationItems.filter((item) => !item.hidden).map((item) => this.renderFlatItem(item))),
            this.threeLevels && this.expandedNavItem && this.expandedNavItem.children && this.renderThreeFoldMenu()),
          h("section", { class: 'bci-portal-sidebar-navigation-footer ' + (this.closed ? 'hide' : '') },
            h("div", null, "Powered by Nexeed from"),
            h("div", { class: "bci-portal-bosch-logo" }))))));
  }
  toggleNavigation(closed = !this.closed) {
    window.dispatchEvent(new CustomEvent('bci.portal.navigation.toggle', { detail: { closed: closed } }));
  }
  static get is() { return "bci-portal-navigation"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["navigation.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["navigation.component.css"]
  }; }
  static get properties() { return {
    "hashNavigation": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "controls whether window path has hash navigations"
      },
      "attribute": "hash-navigation",
      "reflect": false,
      "defaultValue": "true"
    },
    "closed": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "closed",
      "reflect": false
    },
    "mainTitle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "main-title",
      "reflect": false
    },
    "navigationItems": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "SidebarNavItem[]",
        "resolved": "SidebarNavItem[]",
        "references": {
          "SidebarNavItem": {
            "location": "import",
            "path": "../../interfaces/navigation.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "defaultValue": "[]"
    },
    "threeLevels": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "three-levels",
      "reflect": false,
      "defaultValue": "false"
    },
    "useLogo": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "use-logo",
      "reflect": false,
      "defaultValue": "false"
    },
    "emitItemClickOnStartup": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "controls whether the bci.portal.navigation.itemClicked event is triggered upon startup or not"
      },
      "attribute": "emit-item-click-on-startup",
      "reflect": false,
      "defaultValue": "true"
    }
  }; }
  static get states() { return {
    "selectedNavItem": {},
    "expandedNavItem": {},
    "toggleNavigationAnimationActive": {}
  }; }
  static get events() { return [{
      "method": "navigationItemClicked",
      "name": "bci.portal.navigation.itemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "SidebarNavItem",
        "resolved": "SidebarNavItem",
        "references": {
          "SidebarNavItem": {
            "location": "import",
            "path": "../../interfaces/navigation.interface"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "navigationItems",
      "methodName": "loadNavItem"
    }]; }
  static get listeners() { return [{
      "name": "bci.portal.navigation.itemClicked",
      "method": "handleClickNavItemEvent",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.navigation.itemExpanded",
      "method": "handleExpandedNavItemEvent",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.navigation.toggle",
      "method": "handleToggleNavigationEvent",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}

