/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { MDCMenu } from '@material/menu/component';
import { MDCSelect } from '@material/select';
import { Component, Event, h, Host, Listen, Prop, State, Watch } from '@stencil/core';
import { translate } from '../../utils/utils';
import { setLocale } from '../../utils/locale-store';
export class Paginator {
  constructor() {
    /** Number of items to display on a page. By default set to 10. */
    this.pageSize = 10;
    /** The set of provided page size options to display to the user. */
    this.pageSizeOptions = '[]';
    /** The length of the total number of items that are being paginated. Defaulted to 0. */
    this.length = 0;
    /** The zero-based page index of the displayed list of items. Defaulted to 0. */
    this.pageIndex = 0;
    /** If page size option is not needed */
    this.showPageSizeSelector = false;
    /** If page index calculation is not needed then set to true */
    this.pageIndexCalculationOff = false;
    /** Set to true when used inside a dialog container to avoid overflow */
    this.insideDialogContainer = false;
    this.paginatorInput = {
      length: 0,
      pageIndex: 0,
      pageSize: 10,
      pageSizeOptions: [],
      showPageSizeSelector: false,
    };
    this.pages = [];
    this.pagesNumber = 0;
    this.previousPageIndex = 0;
    this.previousPageSize = 0;
    this.isFirstLoad = true;
    this.isPageSizeUpdate = false;
  }
  watchLengthHandler(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { length: newValue });
    this.recalculatePages();
  }
  watchPageSizeOptionsHandler(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSizeOptions: typeof newValue === 'string' ? JSON.parse(newValue) : newValue });
    this.recalculatePages();
  }
  watchPageIndex(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageIndex: newValue });
  }
  watchShowPageSizeSelector(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { showPageSizeSelector: newValue });
  }
  watchPageSize(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSize: newValue });
    this.recalculatePages();
  }
  clickHandler(ev) {
    const currentTarget = ev.composedPath()[0];
    if (!currentTarget || currentTarget.offsetParent === this.menuElement) {
      return;
    }
    if (this.selectElement !== undefined && this.selectElement.className.includes('mdc-select--activated') && this.mdcMenu !== undefined) {
      this.mdcMenu.open = false;
    }
  }
  async componentWillLoad() {
    await setLocale();
    this.paginatorInput = {
      length: this.length,
      pageIndex: this.pageIndex,
      pageSize: this.pageSize,
      pageSizeOptions: typeof this.pageSizeOptions === 'string' ? JSON.parse(this.pageSizeOptions) : this.pageSizeOptions,
      showPageSizeSelector: this.showPageSizeSelector,
    };
    this.previousPageIndex = this.paginatorInput.pageIndex;
    this.previousPageSize = this.paginatorInput.pageSize;
    this.recalculatePages();
    this.isFirstLoad = false;
  }
  componentDidLoad() {
    if (this.selectElement !== undefined && this.menuElement !== undefined) {
      this.mdcSelect = new MDCSelect(this.selectElement);
      this.mdcMenu = new MDCMenu(this.menuElement);
    }
  }
  disconnectedCallback() {
    if (this.mdcSelect === undefined) {
      return;
    }
    this.mdcSelect.destroy();
    if (this.mdcMenu === undefined) {
      return;
    }
    this.mdcMenu.destroy();
  }
  render() {
    const hidePageSizeSelector = this.shouldHidePageSizeSelector();
    const hidePageIndexSelector = this.shouldHidePageIndexSelector();
    return (h(Host, { class: { 'hidden-page-size-selector': hidePageSizeSelector, hidden: hidePageSizeSelector && hidePageIndexSelector } },
      this.renderPageSizeSelector(hidePageSizeSelector),
      this.renderPageIndexSelector(hidePageIndexSelector)));
  }
  shouldHidePageSizeSelector() {
    return (!this.paginatorInput.showPageSizeSelector ||
      this.paginatorInput.pageSizeOptions.length === 0 ||
      this.paginatorInput.length <= Math.min(...this.paginatorInput.pageSizeOptions));
  }
  shouldHidePageIndexSelector() {
    return this.paginatorInput.length <= this.paginatorInput.pageSize;
  }
  renderPageIndexSelector(hidden) {
    return (h("div", { class: { 'bci-core-paginator-pages-container': true, hidden: hidden } },
      h("span", { onClick: () => this.previousPage(), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-prev", class: `bci-core-paginator-page-prev ${this.isAtFirstPage() ? 'disabled' : ''}` }),
      this.positionBasedRender(),
      this.renderMobilePages(),
      h("span", { onClick: () => this.nextPage(), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-next", class: `bci-core-paginator-page-next ${this.isAtLastPage() ? 'disabled' : ''}` })));
  }
  updatePageSize(newVal) {
    this.previousPageSize = this.paginatorInput.pageSize;
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSize: parseFloat(newVal) });
    this.isPageSizeUpdate = true;
    this.recalculatePages();
  }
  updatePageIndex(newVal) {
    if (newVal < 0 || newVal > this.pagesNumber - 1) {
      return;
    }
    this.previousPageIndex = this.paginatorInput.pageIndex;
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageIndex: newVal });
    this.positionBasedRender();
    this.emitPageEvent();
  }
  previousPage() {
    this.updatePageIndex(this.paginatorInput.pageIndex - 1);
  }
  nextPage() {
    this.updatePageIndex(this.paginatorInput.pageIndex + 1);
  }
  recalculatePages() {
    let pageIndex;
    this.pagesNumber = Math.ceil(this.paginatorInput.length / this.paginatorInput.pageSize);
    if (!this.pageIndexCalculationOff) {
      const indexOfFirstItemOnCurrentPage = this.paginatorInput.pageIndex * this.previousPageSize;
      const newPageIndex = Math.floor(indexOfFirstItemOnCurrentPage / this.paginatorInput.pageSize);
      pageIndex = newPageIndex < this.pagesNumber ? newPageIndex : this.pagesNumber - 1;
    }
    else {
      pageIndex = this.isPageSizeUpdate ? 0 : this.pageIndex;
    }
    this.updatePageIndex(pageIndex);
    this.pages = [...Array(this.pagesNumber)].map((_, i) => i + 1);
    this.isPageSizeUpdate = false;
    this.previousPageSize = this.paginatorInput.pageSize;
  }
  isAtFirstPage() {
    return this.paginatorInput.pageIndex === 0;
  }
  isAtLastPage() {
    return this.paginatorInput.pageIndex === this.pagesNumber - 1;
  }
  emitPageEvent() {
    if (!this.isFirstLoad) {
      this.page.emit({
        length: this.paginatorInput.length,
        pageIndex: this.paginatorInput.pageIndex,
        pageSize: this.paginatorInput.pageSize,
        previousPageIndex: this.previousPageIndex,
      });
    }
  }
  getSelectedMenuItemClass(option) {
    return this.paginatorInput.pageSize === option ? ' mdc-list-item--selected' : '';
  }
  getPageButtonClass(index, page) {
    let buttonClass = 'bci-core-paginator-page-button ';
    buttonClass += this.paginatorInput.pageIndex === index ? ' selected' : '';
    buttonClass += page.toString().length > 2 ? ' wide' : '';
    return buttonClass;
  }
  getEllipsesElement() {
    return h("span", { class: "ellipsis" }, "...");
  }
  getPageButtonElement(page, index) {
    return (h("span", { key: index, onClick: () => this.updatePageIndex(index), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-button", class: `${this.getPageButtonClass(index, page)}` }, page));
  }
  getPredicateBasedRender(acc, page, index, pagePredicate, ellipsisPredicate) {
    if (pagePredicate) {
      return acc.concat(this.getPageButtonElement(page, index));
    }
    else if (ellipsisPredicate) {
      return acc.concat(this.getEllipsesElement());
    }
    else {
      return acc;
    }
  }
  positionBasedRender() {
    if (this.paginatorInput.pageIndex < 4) {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index < 5 || index === this.pages.length - 1, index === 5), []);
    }
    else if (this.paginatorInput.pageIndex > this.pagesNumber - 5) {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index === 0 || index > this.pages.length - 6, index === 1), []);
    }
    else {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index === 0 ||
        index === this.pages.length - 1 ||
        (index >= this.paginatorInput.pageIndex - 1 && index <= this.paginatorInput.pageIndex + 1), index === 1 || index === this.pages.length - 2), []);
    }
  }
  renderMobilePages() {
    return (h("span", { class: "mobile" },
      this.paginatorInput.pageIndex + 1,
      "/",
      this.pagesNumber));
  }
  renderPageSizeSelector(hidden) {
    return (h("div", { class: { 'mdc-select': true, 'mdc-select--filled': true, hidden: hidden }, ref: (el) => (this.selectElement = el) },
      h("div", { class: "mdc-select__anchor bci-core-paginator-dropdown" },
        h("span", { class: "mdc-select__ripple" }),
        h("span", { class: "mdc-floating-label mdc-floating-label--float-above" }, translate('listItemsPerPage')),
        h("span", { class: "mdc-select__selected-text" }),
        h("span", { class: "mdc-select__dropdown-icon bosch-ic bosch-ic-down" })),
      h("div", { class: {
          'mdc-select__menu': true,
          'mdc-menu': true,
          'mdc-menu-surface': true,
          'mdc-menu-surface--fixed': this.insideDialogContainer,
          'mdc-menu-surface--fullwidth': !this.insideDialogContainer,
          'bci-core-paginator-dropdown-select': true,
        }, ref: (el) => (this.menuElement = el) },
        h("ul", { class: "mdc-list" }, this.paginatorInput.pageSizeOptions.map((option, index) => (h("li", { "data-test": "webcore.webcomponents.paginator.component.bci-paginator.pagesize-option", "data-value": option, key: index, "aria-selected": "false", role: "option", onClick: () => this.updatePageSize(option), class: `mdc-list-item ${this.getSelectedMenuItemClass(option)}` },
          h("span", { class: "mdc-list-item__text" }, option))))))));
  }
  static get is() { return "bci-paginator"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["paginator.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["paginator.component.css"]
  }; }
  static get properties() { return {
    "pageSize": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Number of items to display on a page. By default set to 10."
      },
      "attribute": "page-size",
      "reflect": false,
      "defaultValue": "10"
    },
    "pageSizeOptions": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The set of provided page size options to display to the user."
      },
      "attribute": "page-size-options",
      "reflect": false,
      "defaultValue": "'[]'"
    },
    "length": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The length of the total number of items that are being paginated. Defaulted to 0."
      },
      "attribute": "length",
      "reflect": false,
      "defaultValue": "0"
    },
    "pageIndex": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The zero-based page index of the displayed list of items. Defaulted to 0."
      },
      "attribute": "page-index",
      "reflect": false,
      "defaultValue": "0"
    },
    "showPageSizeSelector": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If page size option is not needed"
      },
      "attribute": "show-page-size-selector",
      "reflect": false,
      "defaultValue": "false"
    },
    "pageIndexCalculationOff": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If page index calculation is not needed then set to true"
      },
      "attribute": "page-index-calculation-off",
      "reflect": false,
      "defaultValue": "false"
    },
    "insideDialogContainer": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true when used inside a dialog container to avoid overflow"
      },
      "attribute": "inside-dialog-container",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "paginatorInput": {},
    "pages": {}
  }; }
  static get events() { return [{
      "method": "page",
      "name": "page",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Event emitted when the paginator changes the page size or page index."
      },
      "complexType": {
        "original": "PageEvent",
        "resolved": "PageEvent",
        "references": {
          "PageEvent": {
            "location": "import",
            "path": "./paginator.interface"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "length",
      "methodName": "watchLengthHandler"
    }, {
      "propName": "pageSizeOptions",
      "methodName": "watchPageSizeOptionsHandler"
    }, {
      "propName": "pageIndex",
      "methodName": "watchPageIndex"
    }, {
      "propName": "showPageSizeSelector",
      "methodName": "watchShowPageSizeSelector"
    }, {
      "propName": "pageSize",
      "methodName": "watchPageSize"
    }]; }
  static get listeners() { return [{
      "name": "mousedown",
      "method": "clickHandler",
      "target": "body",
      "capture": false,
      "passive": true
    }]; }
}

