/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/**
 * Copyright (C) 2021. Robert Bosch GmbH Copyright (C) 2021. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { flattenArray } from '../utils';
import { FacilitySelectionState } from './facility-selection-state';
import { FacilitySelectionContoller } from './facility-selection.controller';
export class FacilitySelectorController {
  constructor(getRoot, getChildren, getParent, getSearchResults) {
    this.getRoot = getRoot;
    this.getChildren = getChildren;
    this.getParent = getParent;
    this.getSearchResults = getSearchResults;
    this._isStateSet = false;
    this._isMultiSelectEnabled = false;
    this._update = () => { };
    this.setSelectionStateInstance();
    this.setStateInstance(new FacilitySelectionState(this));
  }
  get selectionState() {
    return this._selectionController;
  }
  get isStateSet() {
    return this._isStateSet;
  }
  onChange(callback) {
    if (callback && typeof callback === 'function') {
      this._update = callback;
    }
  }
  setStateInstance(state) {
    this._state = state;
  }
  setSelectionStateInstance(selectionState) {
    this._selectionController = selectionState
      ? new FacilitySelectionContoller(selectionState.selectedItemPerLevel, selectionState.checkedItemPerLevel, selectionState.selectedItems, selectionState.checkedItems, selectionState.indeterminateItems, selectionState.isMultiSelectEnabled)
      : new FacilitySelectionContoller([], [], new Set(), new Set(), new Map(), this._isMultiSelectEnabled);
  }
  initialize(checkedItems) {
    this.getRoot().then((res) => {
      this.resetNodeList();
      this.setColumns([res]);
      this.setDefaultState(checkedItems);
      this._update();
    });
  }
  enableMultiSelection(isMultiSelectEnabled) {
    this.selectionState.isMultiSelectEnabled = isMultiSelectEnabled;
    this._isMultiSelectEnabled = isMultiSelectEnabled;
  }
  hasParent(node) {
    return !!node.parentLocationId;
  }
  hasColumns() {
    return !!this._columns;
  }
  hasColumnAt(level) {
    return !!this.getColumns()[level];
  }
  getColumns() {
    return this._columns || [];
  }
  setColumns(columns) {
    this._columns = [...columns];
  }
  getColumnAtLevel(level) {
    return this.hasColumnAt(level) ? this.getColumns()[level] : [];
  }
  removeColumnsFromIndex(index) {
    if (!this.hasColumns) {
      return;
    }
    this._columns = [...this._columns.slice(0, index + 1)];
  }
  appendColumn(column) {
    this._columns = [...(this._columns || []), column];
  }
  async getChildColumn(node) {
    return this.getChildren(node).then((res) => {
      this.appendColumn(res);
      this._update();
      return Promise.resolve(true);
    });
  }
  setDefaultState(checkedItems) {
    if (checkedItems) {
      this._state.setDefaultState(checkedItems);
    }
    this._isStateSet = true;
  }
  resetToDefaultState() {
    this.setColumns(this._state.columnCache);
    this.setSelectionStateInstance(this._state.selectionStateCache);
    this._update();
  }
  updateDefaultState() {
    this._state.updateDefaultState();
    this._update();
  }
  async selectNode(node, level) {
    if (level <= this.getColumns().length - 1) {
      this.removeColumnsFromIndex(level);
    }
    if (!this.selectionState.isSelected(node)) {
      this.selectionState.selectItem(node, level);
    }
    if (node.hasChildren) {
      return this.getChildColumn(node);
    }
    this._update();
    return Promise.resolve(true);
  }
  deSelectNode(node, level) {
    this.selectionState.deselectItem(node);
    this.removeColumnsFromIndex(level);
    this._update();
  }
  checkNode(node, level) {
    this.selectionState.checkItem(node, level);
    this.markAllParentsAsIndeterminate(node);
    this._update();
  }
  uncheckNode(node, level) {
    this.selectionState.uncheckItem(node, level);
    this.unmarkAllParentsAsIndeterminate(node);
    this._update();
  }
  markNodeAsIndeterminate(node) {
    this.selectionState.markAsIndeterminate(node);
    this._update();
  }
  unMarkNodeAsIndeterminate(node) {
    this.selectionState.unmarkAsIndeterminate(node);
    this._update();
  }
  checkAll(checkedItems) {
    checkedItems.forEach((item) => this.checkNodeWithoutLevelInformation(true, item));
  }
  async search(searchQuery) {
    return this.getSearchResults(searchQuery);
  }
  async selectSearchResult(selectedSearchResult, searchRoot) {
    return this.getSelectionPathFrom(selectedSearchResult, searchRoot).then((selectionPath) => {
      return Promise.all(selectionPath.map((item) => this.selectNode(item)));
    });
  }
  checkSearchResult(checked, searchResult) {
    this.checkNodeWithoutLevelInformation(checked, searchResult);
  }
  selectRecentSeachItem(recentSearchItem) {
    const checkedItems = (flattenArray(recentSearchItem.checkedItemsByLevel) || []).filter((item) => !!item);
    this.checkAll(checkedItems);
    this.loadColumnsBasedOn(checkedItems);
  }
  async loadColumnsBasedOn(checkedItems) {
    return this.getCommonSelectionPath(checkedItems).then((selectionPath) => {
      if (!selectionPath || !Array.isArray(selectionPath) || selectionPath.length === 0) {
        return Promise.resolve(false);
      }
      // Update selection
      selectionPath.forEach((item, index) => {
        this.selectionState.selectItem(item, index);
      });
      // Fetch columns
      return Promise.all(selectionPath.map((item) => this.getChildren(item))).then((columns) => {
        if (!columns || !Array.isArray(columns) || columns.length === 0) {
          return Promise.resolve(false);
        }
        columns.forEach((column) => this.appendColumn(column));
        this._update();
        return Promise.resolve(true);
      });
    });
  }
  resetNodeList() {
    this._columns = [];
  }
  reset() {
    this.setSelectionStateInstance();
    this.removeColumnsFromIndex(0); // Keep only root level elements
    this._update();
  }
  async markAllParentsAsIndeterminate(node, depth = 0) {
    if (!this.hasParent(node)) {
      return Promise.resolve(depth);
    }
    return this.getParent(node).then((res) => {
      if (!res) {
        return;
      }
      depth++;
      this.markNodeAsIndeterminate(res);
      return this.markAllParentsAsIndeterminate(res, depth);
    });
  }
  async unmarkAllParentsAsIndeterminate(node, depth = 0) {
    if (!this.hasParent(node)) {
      return Promise.resolve(depth);
    }
    return this.getParent(node).then((res) => {
      if (!res) {
        return;
      }
      depth++;
      this.unMarkNodeAsIndeterminate(res);
      return this.unmarkAllParentsAsIndeterminate(res);
    });
  }
  checkNodeWithoutLevelInformation(checked, node) {
    this.getSelectionPathFrom(node, null).then((selectionPath) => {
      if (!selectionPath) {
        return;
      }
      for (let i = selectionPath.length - 1; i >= 0; i--) {
        if (checked) {
          if (selectionPath[i].id === node.id) {
            this.selectionState.checkItem(selectionPath[i], i);
          }
          else {
            this.selectionState.markAsIndeterminate(selectionPath[i]);
          }
        }
        else {
          if (selectionPath[i].id === node.id) {
            this.selectionState.uncheckItem(selectionPath[i], i);
          }
          else {
            this.selectionState.unmarkAsIndeterminate(selectionPath[i]);
          }
        }
      }
      this._update();
    });
  }
  async getCommonSelectionPath(checkedItems) {
    return Promise.all(checkedItems.map((checkedItem) => this.getSelectionPathFrom(checkedItem))).then((selectionPaths) => {
      return this.getDeepestCommonSelectionPath(selectionPaths);
    });
  }
  async getSelectionPathFrom(endNode, startNode = null, selectionPath = []) {
    if (!startNode && !this.hasParent(endNode)) {
      selectionPath = [endNode, ...selectionPath]; // adding the root node
      return Promise.resolve(selectionPath);
    }
    else if (startNode && endNode.id === startNode.id) {
      return Promise.resolve(selectionPath);
    }
    else {
      return this.getParent(endNode).then((parent) => {
        selectionPath = [endNode, ...selectionPath];
        return this.getSelectionPathFrom(parent, startNode, selectionPath);
      });
    }
  }
  getDeepestCommonSelectionPath(selectionPaths) {
    const selectionPath = []; // holds all selections for a level
    const deepestPath = [];
    let i = 0;
    selectionPaths.forEach((path) => {
      path.forEach((item, index) => {
        if (!selectionPath[index]) {
          selectionPath[index] = [];
        }
        if (selectionPath[index].filter((s) => s.id === item.id).length > 0) {
          return;
        }
        selectionPath[index].push(item);
      });
    });
    if (selectionPath.length > 0) {
      while (selectionPath[i].length === 1 && i < selectionPath.length - 1) {
        deepestPath.push(...selectionPath[i]);
        i++;
      }
    }
    return [...deepestPath];
  }
}

