/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { differenceInDays, format, isToday, isYesterday, setDate, setHours, setMinutes, setMonth, setSeconds, setYear } from 'date-fns';
import state from './locale-store';
import { DEFAULT_LOCALE, LOCALE_MAP, localization } from './localization';
/**
 * Translate the given key to the desired language based on the localization dictionary provided
 * under BCIPortal global object.
 *
 * @param key language key of the related text
 * @param language language code. Default is the user language.
 */
export function translate(key) {
  try {
    if (localization[key]) {
      return localization[key][state.language] || localization[key]['en'] || key;
    }
    return key;
  }
  catch (_a) {
    return key;
  }
}
/**
 * Format a timestamp according to the user's configured date format
 *
 * @param timestamp
 */
export function formatDate(timestamp, dateFormat, timeFormat) {
  try {
    return format(new Date(timestamp), (dateFormat || state.dateFormat) + ' ' + (timeFormat || state.timeFormat));
  }
  catch (_a) {
    return format(new Date(timestamp), DEFAULT_LOCALE.dateFormat + ' ' + DEFAULT_LOCALE.timeFormat);
  }
}
/**
 * TODO
 *
 * @param timestamp
 */
export function formatDateAsDayString(timestamp, dateFormat = DEFAULT_LOCALE.dateFormat) {
  if (!timestamp) {
    return;
  }
  try {
    const date = new Date(timestamp);
    if (isToday(date)) {
      return translate('today');
    }
    else if (isYesterday(date)) {
      return translate('yesterday');
    }
    else if (differenceInDays(new Date(), date) < 7) {
      return (translate('last') + ' ' + format(new Date(timestamp), 'EEEE', { locale: LOCALE_MAP[state.language || DEFAULT_LOCALE.language] }));
    }
    return format(new Date(timestamp), dateFormat || state.dateFormat);
  }
  catch (_a) {
    return format(new Date(timestamp), DEFAULT_LOCALE.dateFormat);
  }
}
/**
 * Gets a new logger instance to log to the browser console.
 */
export function getLogger() {
  const defaultPrefix = '[bci-portal]';
  return {
    info: (msg, object) => {
      object ? console.log(`${defaultPrefix} ${msg}`, object) : console.log(`${defaultPrefix} ${msg}`);
    },
    warn: (msg, object) => {
      object ? console.warn(`${defaultPrefix} ${msg}`, object) : console.warn(`${defaultPrefix} ${msg}`);
    },
    error: (msg, object) => {
      object ? console.error(`${defaultPrefix} ${msg}`, object) : console.error(`${defaultPrefix} ${msg}`);
    },
  };
}
/**
 * truncate a string in the middle, e.g. MyVeryLongString > MyV...ing
 * @param str
 * @param firstCharCount
 * @param endCharCount
 * @param dotCount
 */
export function truncateStringPortion(str, maxLength = 40, dotCount = 3) {
  if (str.length <= maxLength) {
    return str;
  }
  const halfTheLength = Math.floor((maxLength - dotCount) / 2);
  let convertedStr = '';
  convertedStr += str.substring(0, halfTheLength);
  convertedStr += '.'.repeat(dotCount);
  convertedStr += str.substring(str.length - halfTheLength, str.length);
  return convertedStr;
}
/**
 * creates a date instance with specified parameters
 * @param dateInstance to use as basis
 * @param year year to set
 * @param month month so set
 * @param day day in month to set
 */
export function createDate(dateInstance, year, month, day = null) {
  dateInstance = setYear(dateInstance, year);
  dateInstance = setMonth(dateInstance, month);
  if (day !== null) {
    dateInstance = setDate(dateInstance, day);
  }
  return dateInstance;
}
/**
 * creates a date instance with specified parameters
 * @param dateInstance to use as basis
 * @param hours hours to set
 * @param minutes minutes so set
 * @param seconds seconds to set
 */
export function createTime(dateInstance, hours, minutes, seconds = 0) {
  dateInstance = setHours(dateInstance, hours);
  dateInstance = setMinutes(dateInstance, minutes);
  dateInstance = setSeconds(dateInstance, seconds);
  return dateInstance;
}
/**
 * flattens an array
 * @param array
 */
export function flattenArray(array) {
  return [].concat.apply([], array);
}
/**
 * Returns whether app is in responsive mode
 */
export function isResponsiveMode() {
  const currentWidth = window.innerWidth;
  if (currentWidth < 768) {
    return true;
  }
  return false;
}
//this method checks if timeformat is with Meridiem(AM/PM) value
export function timeFormatHasMeridiem(timeformat) {
  // space is required as timeformats with AM/PM always have a space between the time and the meridiem
  return timeformat.includes(' a');
}
//this method checks if timeformat has seconds
export function isHideSeconds(timeformat) {
  return timeformat.includes(':ss');
}
export function tap(callback) {
  return function (value) {
    callback(value);
    return value;
  };
}

