/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { p as proxyCustomElement, H, c as createEvent, h } from './p-6fc5b736.js';
import { c as createDate, i as isSameDay } from './p-9dead869.js';
import { p as parse } from './p-596db46e.js';
import { i as isAfter, a as isBefore } from './p-15f94564.js';
import { r as requiredArgs, t as toDate } from './p-d4c86b19.js';

/**
 * @name getDate
 * @category Day Helpers
 * @summary Get the day of the month of the given date.
 *
 * @description
 * Get the day of the month of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the day of month
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which day of the month is 29 February 2012?
 * const result = getDate(new Date(2012, 1, 29))
 * //=> 29
 */

function getDate(dirtyDate) {
  requiredArgs(1, arguments);
  var date = toDate(dirtyDate);
  var dayOfMonth = date.getDate();
  return dayOfMonth;
}

/**
 * @name getMonth
 * @category Month Helpers
 * @summary Get the month of the given date.
 *
 * @description
 * Get the month of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the month
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which month is 29 February 2012?
 * const result = getMonth(new Date(2012, 1, 29))
 * //=> 1
 */

function getMonth(dirtyDate) {
  requiredArgs(1, arguments);
  var date = toDate(dirtyDate);
  var month = date.getMonth();
  return month;
}

/**
 * @name getYear
 * @category Year Helpers
 * @summary Get the year of the given date.
 *
 * @description
 * Get the year of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the year
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which year is 2 July 2014?
 * const result = getYear(new Date(2014, 6, 2))
 * //=> 2014
 */

function getYear(dirtyDate) {
  requiredArgs(1, arguments);
  return toDate(dirtyDate).getFullYear();
}

/**
 * @name isWithinInterval
 * @category Interval Helpers
 * @summary Is the given date within the interval?
 *
 * @description
 * Is the given date within the interval? (Including start and end.)
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - The function was renamed from `isWithinRange` to `isWithinInterval`.
 *   This change was made to mirror the use of the word "interval" in standard ISO 8601:2004 terminology:
 *
 *   ```
 *   2.1.3
 *   time interval
 *   part of the time axis limited by two instants
 *   ```
 *
 *   Also, this function now accepts an object with `start` and `end` properties
 *   instead of two arguments as an interval.
 *   This function now throws `RangeError` if the start of the interval is after its end
 *   or if any date in the interval is `Invalid Date`.
 *
 *   ```javascript
 *   // Before v2.0.0
 *
 *   isWithinRange(
 *     new Date(2014, 0, 3),
 *     new Date(2014, 0, 1), new Date(2014, 0, 7)
 *   )
 *
 *   // v2.0.0 onward
 *
 *   isWithinInterval(
 *     new Date(2014, 0, 3),
 *     { start: new Date(2014, 0, 1), end: new Date(2014, 0, 7) }
 *   )
 *   ```
 *
 * @param {Date|Number} date - the date to check
 * @param {Interval} interval - the interval to check
 * @returns {Boolean} the date is within the interval
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} The start of an interval cannot be after its end
 * @throws {RangeError} Date in interval cannot be `Invalid Date`
 *
 * @example
 * // For the date within the interval:
 * isWithinInterval(new Date(2014, 0, 3), {
 *   start: new Date(2014, 0, 1),
 *   end: new Date(2014, 0, 7)
 * })
 * //=> true
 *
 * @example
 * // For the date outside of the interval:
 * isWithinInterval(new Date(2014, 0, 10), {
 *   start: new Date(2014, 0, 1),
 *   end: new Date(2014, 0, 7)
 * })
 * //=> false
 *
 * @example
 * // For date equal to interval start:
 * isWithinInterval(date, { start, end: date }) // => true
 *
 * @example
 * // For date equal to interval end:
 * isWithinInterval(date, { start: date, end }) // => true
 */
function isWithinInterval(dirtyDate, interval) {
  requiredArgs(2, arguments);
  var time = toDate(dirtyDate).getTime();
  var startTime = toDate(interval.start).getTime();
  var endTime = toDate(interval.end).getTime(); // Throw an exception if start date is after end date or if any date is `Invalid Date`

  if (!(startTime <= endTime)) {
    throw new RangeError('Invalid interval');
  }

  return time >= startTime && time <= endTime;
}

const weekdaysCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bci-core-weekdays-table{width:calc(100% - 10px);border-spacing:0;border-collapse:collapse;margin:0 5px 5px 5px}.bci-core-weekdays-table td{position:relative;height:0;line-height:0;text-align:center;outline:0;cursor:pointer;padding-top:8px;padding-bottom:32px}.bci-core-weekdays-table td .bci-core-date{color:#000000;font-size:16px;font-weight:400;text-align:center;display:flex;line-height:1;border-width:1px;border-style:solid;border-color:transparent;border-radius:50%;width:32px;height:32px;position:absolute;left:4px;top:4px;align-items:center;justify-content:center;box-sizing:border-box}.bci-core-weekdays-table td.bci-core-disabled{cursor:default}.bci-core-weekdays-table td.bci-core-disabled .bci-core-date{background:transparent;color:#c1c7cc}.bci-core-weekdays-table td.bci-core-disabled.bci-core-today .bci-core-date{border:1px solid #c1c7cc}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected .bci-core-date{background-color:#c1c7cc;color:#8a9097}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range{border-radius:0px}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range .bci-core-date{background-color:transparent}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range.bci-core-first-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;border-radius:16px 0 0 16px}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range.bci-core-last-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;width:32px;border-radius:0 16px 16px 0}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range)::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;border-radius:0}.bci-core-weekdays-table td.bci-core-week-days{cursor:pointer}.bci-core-weekdays-table td.bci-core-week-days .bci-core-date:hover{background-color:#e0e2e5}.bci-core-weekdays-table td.bci-core-week-days .bci-core-date:active{background-color:#c1c7cc}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date{background:transparent;border:1px solid #000000;color:#000000}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date:hover{background-color:#e0e2e5}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date:active{background-color:#c1c7cc}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date{background-color:#007bc0;color:#ffffff}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date:hover{background-color:#00629a}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date:active{background-color:#004975}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range{border-radius:0px}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date{background-color:transparent}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date:hover{background-color:#00629a}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date:active{background-color:#004975}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range.bci-core-first-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;border-radius:16px 0 0 16px}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range.bci-core-last-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;width:32px;border-radius:0 16px 16px 0}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range)::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;border-radius:0}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range).bci-core-last-week-day::before{width:32px}.bci-core-weekdays-table .bci-core-empty{visibility:hidden}.bci-core-weekdays-table .bci-core-empty:hover{cursor:not-allowed}";

const Weekdays = /*@__PURE__*/ proxyCustomElement(class extends H {
  constructor() {
    super();
    this.__registerHost();
    this.singleDateSelected = createEvent(this, "singleDateSelected", 7);
    this.rangeDateSelected = createEvent(this, "rangeDateSelected", 7);
  }
  singleDateSelectedHandler(input) {
    this.singleDateSelected.emit(input);
  }
  rangeDateSelectedHandler(input) {
    this.rangeDateSelected.emit(input);
  }
  render() {
    const cells = [];
    const offset = this.offset % 7;
    for (let blank = 0; blank < offset; blank++) {
      cells.push(this.emptyDateCell());
    }
    for (let day = 1; day <= this.daysInMonth; day++) {
      /** Whether the day is selected  */
      let selected;
      /** Whether the day is within the selected range */
      let selectedRange;
      /** Whether the day is the first day of the selected range */
      let selectedFirstDateRange;
      /** Whether the day is the last day of the selected range */
      let selectedLastDateRange;
      const maxDate = this.dateRestrictionObj.maximumDate;
      const minDate = this.dateRestrictionObj.minimumDate;
      const currentDate = this.timeZone === undefined
        ? createDate(new Date(), this.year, this.month, day)
        : createDate(new Date(new Date().toLocaleString('en-US', { timeZone: this.timeZone })), this.year, this.month, day);
      const todaysDate = isSameDay(currentDate, parse(this.todaysDate, this.formatDate, new Date()));
      // disabled cells
      if ((!isSameDay(minDate, currentDate) && isAfter(minDate, currentDate)) ||
        (!isSameDay(maxDate, currentDate) && isBefore(maxDate, currentDate))) {
        cells.push(h("td", { class: {
            'bci-core-disabled': true,
            'bci-core-today': todaysDate,
          } }, h("div", { class: "bci-core-date" }, day)));
        continue;
      }
      // clickable cells
      if (this.datesObj.startDate && this.datesObj.endDate) {
        // if startDate and endDate are selected
        const startDate = parse(this.datesObj.startDate, this.formatDate, new Date());
        const endDate = parse(this.datesObj.endDate, this.formatDate, new Date());
        if (isAfter(endDate, startDate) && isWithinInterval(currentDate, { start: startDate, end: endDate })) {
          selected = true;
          selectedRange = true;
        }
        else {
          selected = false;
        }
        if (isSameDay(currentDate, startDate)) {
          selectedFirstDateRange = true;
          selected = true;
        }
        if (isSameDay(currentDate, endDate)) {
          selectedLastDateRange = true;
          selected = true;
          selectedRange = true;
        }
        if (isSameDay(currentDate, startDate) && isSameDay(startDate, endDate)) {
          selected = true;
          selectedFirstDateRange = false;
          selectedLastDateRange = false;
          selectedRange = false;
        }
      }
      else {
        // single date picker
        if (this.datesObj.startDate || this.selectedDate) {
          const selectedDate = parse(this.datesObj.startDate || this.selectedDate, this.formatDate, new Date());
          selected = getDate(selectedDate) === day && getMonth(selectedDate) === this.month && getYear(selectedDate) === this.year;
        }
      }
      const classes = {
        'bci-core-week-days': true,
        'bci-core-selected': selected,
        'bci-core-range': selectedRange,
        'bci-core-first-date-range': selectedFirstDateRange,
        'bci-core-last-date-range': selectedLastDateRange,
        'bci-core-today': selected ? false : todaysDate,
        'bci-core-last-week-day': (cells.length + 1) % 7 === 0,
      };
      cells.push(h("td", { "data-test": "webcore.webcomponents.weekdays.bci-weekdays.selected-day", class: classes, onClick: () => this.rangeDate ? this.rangeDateSelectedHandler({ selectedDay: day }) : this.singleDateSelectedHandler({ selectedDay: day }) }, h("div", { class: "bci-core-date" }, day)));
    }
    while (cells.length % 7 !== 0) {
      cells.push(this.emptyDateCell());
    }
    const table = cells.reduce((acc, curr, index) => {
      const weekNumber = Math.floor(index / 7);
      acc[weekNumber].push(curr);
      return acc;
    }, [[], [], [], [], [], []]);
    return (h("table", { class: "bci-core-weekdays-table" }, h("tbody", { class: "bci-core-week-days-container" }, table.sort().map((row, index) => (h("tr", { key: index }, row.map((cell) => cell)))))));
  }
  emptyDateCell() {
    return (h("td", { class: "bci-core-empty" }, h("div", { class: "bci-core-date" })));
  }
  static get style() { return weekdaysCss; }
}, [0, "bci-weekdays", {
    "datesObj": [16],
    "daysInMonth": [2, "days-in-month"],
    "lastDayOfMonth": [16],
    "month": [2],
    "rangeDate": [4, "range-date"],
    "dateRestrictionObj": [8, "date-restriction-obj"],
    "offset": [2],
    "selectedDate": [1, "selected-date"],
    "year": [2],
    "todaysDate": [1, "todays-date"],
    "formatDate": [1, "format-date"],
    "timeZone": [1, "time-zone"]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bci-weekdays"];
  components.forEach(tagName => { switch (tagName) {
    case "bci-weekdays":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, Weekdays);
      }
      break;
  } });
}

export { Weekdays as W, getYear as a, defineCustomElement as d, getMonth as g };

