/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { p as proxyCustomElement, H, c as createEvent, h, b as Host } from './p-6fc5b736.js';
import { s as startOfDay, c as createDate, t as translate } from './p-9dead869.js';
import { r as requiredArgs, t as toDate$1, a as toInteger$1, D as DEFAULT_LOCALE } from './p-d4c86b19.js';
import { f as format } from './p-98fe95a7.js';
import { p as parse } from './p-596db46e.js';
import { g as getDefaultExportFromCjs, c as createCommonjsModule } from './p-3e8ff66b.js';
import { i as isAfter, a as isBefore } from './p-15f94564.js';
import { d as defineCustomElement$7 } from './p-2224aabe.js';
import { d as defineCustomElement$6 } from './p-85e694ad.js';
import { d as defineCustomElement$5 } from './p-9a60a159.js';
import { d as defineCustomElement$4 } from './p-9055836b.js';
import { d as defineCustomElement$3 } from './p-73431532.js';
import { d as defineCustomElement$2, g as getMonth, a as getYear } from './p-c9d9902e.js';
import { d as defineCustomElement$1 } from './p-9639d457.js';

/**
 * @name max
 * @category Common Helpers
 * @summary Return the latest of the given dates.
 *
 * @description
 * Return the latest of the given dates.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - `max` function now accepts an array of dates rather than spread arguments.
 *
 *   ```javascript
 *   // Before v2.0.0
 *   var date1 = new Date(1989, 6, 10)
 *   var date2 = new Date(1987, 1, 11)
 *   var maxDate = max(date1, date2)
 *
 *   // v2.0.0 onward:
 *   var dates = [new Date(1989, 6, 10), new Date(1987, 1, 11)]
 *   var maxDate = max(dates)
 *   ```
 *
 * @param {Date[]|Number[]} datesArray - the dates to compare
 * @returns {Date} the latest of the dates
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which of these dates is the latest?
 * var result = max([
 *   new Date(1989, 6, 10),
 *   new Date(1987, 1, 11),
 *   new Date(1995, 6, 2),
 *   new Date(1990, 0, 1)
 * ])
 * //=> Sun Jul 02 1995 00:00:00
 */

function max(dirtyDatesArray) {
  requiredArgs(1, arguments);
  var datesArray; // `dirtyDatesArray` is Array, Set or Map, or object with custom `forEach` method

  if (dirtyDatesArray && typeof dirtyDatesArray.forEach === 'function') {
    datesArray = dirtyDatesArray; // If `dirtyDatesArray` is Array-like Object, convert to Array.
  } else if (typeof dirtyDatesArray === 'object' && dirtyDatesArray !== null) {
    datesArray = Array.prototype.slice.call(dirtyDatesArray);
  } else {
    // `dirtyDatesArray` is non-iterable, return Invalid Date
    return new Date(NaN);
  }

  var result;
  datesArray.forEach(function (dirtyDate) {
    var currentDate = toDate$1(dirtyDate);

    if (result === undefined || result < currentDate || isNaN(Number(currentDate))) {
      result = currentDate;
    }
  });
  return result || new Date(NaN);
}

/**
 * @name min
 * @category Common Helpers
 * @summary Returns the earliest of the given dates.
 *
 * @description
 * Returns the earliest of the given dates.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - `min` function now accepts an array of dates rather than spread arguments.
 *
 *   ```javascript
 *   // Before v2.0.0
 *   const date1 = new Date(1989, 6, 10)
 *   const date2 = new Date(1987, 1, 11)
 *   const minDate = min(date1, date2)
 *
 *   // v2.0.0 onward:
 *   const dates = [new Date(1989, 6, 10), new Date(1987, 1, 11)]
 *   const minDate = min(dates)
 *   ```
 *
 * @param {Date[]|Number[]} datesArray - the dates to compare
 * @returns {Date} - the earliest of the dates
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which of these dates is the earliest?
 * const result = min([
 *   new Date(1989, 6, 10),
 *   new Date(1987, 1, 11),
 *   new Date(1995, 6, 2),
 *   new Date(1990, 0, 1)
 * ])
 * //=> Wed Feb 11 1987 00:00:00
 */

function min(dirtyDatesArray) {
  requiredArgs(1, arguments);
  var datesArray; // `dirtyDatesArray` is Array, Set or Map, or object with custom `forEach` method

  if (dirtyDatesArray && typeof dirtyDatesArray.forEach === 'function') {
    datesArray = dirtyDatesArray; // If `dirtyDatesArray` is Array-like Object, convert to Array.
  } else if (typeof dirtyDatesArray === 'object' && dirtyDatesArray !== null) {
    datesArray = Array.prototype.slice.call(dirtyDatesArray);
  } else {
    // `dirtyDatesArray` is non-iterable, return Invalid Date
    return new Date(NaN);
  }

  var result;
  datesArray.forEach(function (dirtyDate) {
    var currentDate = toDate$1(dirtyDate);

    if (result === undefined || result > currentDate || isNaN(currentDate.getDate())) {
      result = currentDate;
    }
  });
  return result || new Date(NaN);
}

/**
 * @name clamp
 * @category Interval Helpers
 * @summary Return a date bounded by the start and the end of the given interval
 *
 * @description
 * Clamps a date to the lower bound with the start of the interval and the upper
 * bound with the end of the interval.
 *
 * - When the date is less than the start of the interval, the start is returned.
 * - When the date is greater than the end of the interval, the end is returned.
 * - Otherwise the date is returned.
 *
 * @example
 * // What is Mar, 21, 2021 bounded to an interval starting at Mar, 22, 2021 and ending at Apr, 01, 2021
 * const result = clamp(new Date(2021, 2, 21), {
 *   start: new Date(2021, 2, 22),
 *   end: new Date(2021, 3, 1),
 * })
 * //=> Mon Mar 22 2021 00:00:00
 *
 * @param {Date | Number} date - the date to be bounded
 * @param {Interval} interval - the interval to bound to
 * @returns {Date} the date bounded by the start and the end of the interval
 * @throws {TypeError} 2 arguments required
 */

function clamp(date, _ref) {
  var start = _ref.start,
      end = _ref.end;
  requiredArgs(2, arguments);
  return min([max([date, start]), end]);
}

/**
 * @name endOfDay
 * @category Day Helpers
 * @summary Return the end of a day for the given date.
 *
 * @description
 * Return the end of a day for the given date.
 * The result will be in the local timezone.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the original date
 * @returns {Date} the end of a day
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // The end of a day for 2 September 2014 11:55:00:
 * const result = endOfDay(new Date(2014, 8, 2, 11, 55, 0))
 * //=> Tue Sep 02 2014 23:59:59.999
 */

function endOfDay(dirtyDate) {
  requiredArgs(1, arguments);
  var date = toDate$1(dirtyDate);
  date.setHours(23, 59, 59, 999);
  return date;
}

/**
 * @name fromUnixTime
 * @category Timestamp Helpers
 * @summary Create a date from a Unix timestamp.
 *
 * @description
 * Create a date from a Unix timestamp (in seconds). Decimal values will be discarded.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Number} unixTime - the given Unix timestamp (in seconds)
 * @returns {Date} the date
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Create the date 29 February 2012 11:45:05:
 * const result = fromUnixTime(1330515905)
 * //=> Wed Feb 29 2012 11:45:05
 */

function fromUnixTime(dirtyUnixTime) {
  requiredArgs(1, arguments);
  var unixTime = toInteger$1(dirtyUnixTime);
  return toDate$1(unixTime * 1000);
}

/**
 * @name getTime
 * @category Timestamp Helpers
 * @summary Get the milliseconds timestamp of the given date.
 *
 * @description
 * Get the milliseconds timestamp of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the timestamp
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Get the timestamp of 29 February 2012 11:45:05.123:
 * const result = getTime(new Date(2012, 1, 29, 11, 45, 5, 123))
 * //=> 1330515905123
 */

function getTime(dirtyDate) {
  requiredArgs(1, arguments);
  var date = toDate$1(dirtyDate);
  var timestamp = date.getTime();
  return timestamp;
}

/**
 * @name getUnixTime
 * @category Timestamp Helpers
 * @summary Get the seconds timestamp of the given date.
 *
 * @description
 * Get the seconds timestamp of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the timestamp
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Get the timestamp of 29 February 2012 11:45:05 CET:
 * const result = getUnixTime(new Date(2012, 1, 29, 11, 45, 5))
 * //=> 1330512305
 */

function getUnixTime(dirtyDate) {
  requiredArgs(1, arguments);
  return Math.floor(getTime(dirtyDate) / 1000);
}

var toInteger_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = toInteger;

function toInteger(dirtyNumber) {
  if (dirtyNumber === null || dirtyNumber === true || dirtyNumber === false) {
    return NaN;
  }

  var number = Number(dirtyNumber);

  if (isNaN(number)) {
    return number;
  }

  return number < 0 ? Math.ceil(number) : Math.floor(number);
}

module.exports = exports.default;
});

const toInteger = /*@__PURE__*/getDefaultExportFromCjs(toInteger_1);

var getTimezoneOffsetInMilliseconds_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getTimezoneOffsetInMilliseconds;

/**
 * Google Chrome as of 67.0.3396.87 introduced timezones with offset that includes seconds.
 * They usually appear for dates that denote time before the timezones were introduced
 * (e.g. for 'Europe/Prague' timezone the offset is GMT+00:57:44 before 1 October 1891
 * and GMT+01:00:00 after that date)
 *
 * Date#getTimezoneOffset returns the offset in minutes and would return 57 for the example above,
 * which would lead to incorrect calculations.
 *
 * This function returns the timezone offset in milliseconds that takes seconds in account.
 */
function getTimezoneOffsetInMilliseconds(date) {
  var utcDate = new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()));
  utcDate.setUTCFullYear(date.getFullYear());
  return date.getTime() - utcDate.getTime();
}

module.exports = exports.default;
});

const getTimezoneOffsetInMilliseconds = /*@__PURE__*/getDefaultExportFromCjs(getTimezoneOffsetInMilliseconds_1);

/**
 * Returns the [year, month, day, hour, minute, seconds] tokens of the provided
 * `date` as it will be rendered in the `timeZone`.
 */
function tzTokenizeDate(date, timeZone) {
  var dtf = getDateTimeFormat(timeZone);
  return dtf.formatToParts ? partsOffset(dtf, date) : hackyOffset(dtf, date)
}

var typeToPos = {
  year: 0,
  month: 1,
  day: 2,
  hour: 3,
  minute: 4,
  second: 5,
};

function partsOffset(dtf, date) {
  try {
    var formatted = dtf.formatToParts(date);
    var filled = [];
    for (var i = 0; i < formatted.length; i++) {
      var pos = typeToPos[formatted[i].type];

      if (pos >= 0) {
        filled[pos] = parseInt(formatted[i].value, 10);
      }
    }
    return filled
  } catch (error) {
    if (error instanceof RangeError) {
      return [NaN]
    }
    throw error
  }
}

function hackyOffset(dtf, date) {
  var formatted = dtf.format(date).replace(/\u200E/g, '');
  var parsed = /(\d+)\/(\d+)\/(\d+),? (\d+):(\d+):(\d+)/.exec(formatted);
  // var [, fMonth, fDay, fYear, fHour, fMinute, fSecond] = parsed
  // return [fYear, fMonth, fDay, fHour, fMinute, fSecond]
  return [parsed[3], parsed[1], parsed[2], parsed[4], parsed[5], parsed[6]]
}

// Get a cached Intl.DateTimeFormat instance for the IANA `timeZone`. This can be used
// to get deterministic local date/time output according to the `en-US` locale which
// can be used to extract local time parts as necessary.
var dtfCache = {};
function getDateTimeFormat(timeZone) {
  if (!dtfCache[timeZone]) {
    // New browsers use `hourCycle`, IE and Chrome <73 does not support it and uses `hour12`
    var testDateFormatted = new Intl.DateTimeFormat('en-US', {
      hour12: false,
      timeZone: 'America/New_York',
      year: 'numeric',
      month: '2-digit',
      day: '2-digit',
      hour: '2-digit',
      minute: '2-digit',
      second: '2-digit',
    }).format(new Date('2014-06-25T04:00:00.123Z'));
    var hourCycleSupported =
      testDateFormatted === '06/25/2014, 00:00:00' ||
      testDateFormatted === '‎06‎/‎25‎/‎2014‎ ‎00‎:‎00‎:‎00';

    dtfCache[timeZone] = hourCycleSupported
      ? new Intl.DateTimeFormat('en-US', {
          hour12: false,
          timeZone: timeZone,
          year: 'numeric',
          month: '2-digit',
          day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
        })
      : new Intl.DateTimeFormat('en-US', {
          hourCycle: 'h23',
          timeZone: timeZone,
          year: 'numeric',
          month: '2-digit',
          day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
        });
  }
  return dtfCache[timeZone]
}

/**
 * Use instead of `new Date(Date.UTC(...))` to support years below 100 which doesn't work
 * otherwise due to the nature of the
 * [`Date` constructor](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date#interpretation_of_two-digit_years.
 *
 * For `Date.UTC(...)`, use `newDateUTC(...).getTime()`.
 */
function newDateUTC(fullYear, month, day, hour, minute, second, millisecond) {
  var utcDate = new Date(0);
  utcDate.setUTCFullYear(fullYear, month, day);
  utcDate.setUTCHours(hour, minute, second, millisecond);
  return utcDate
}

var MILLISECONDS_IN_HOUR$1 = 3600000;
var MILLISECONDS_IN_MINUTE$1 = 60000;

var patterns$1 = {
  timezone: /([Z+-].*)$/,
  timezoneZ: /^(Z)$/,
  timezoneHH: /^([+-]\d{2})$/,
  timezoneHHMM: /^([+-]\d{2}):?(\d{2})$/,
};

// Parse various time zone offset formats to an offset in milliseconds
function tzParseTimezone(timezoneString, date, isUtcDate) {
  var token;
  var absoluteOffset;

  // Empty string
  if (timezoneString === '') {
    return 0
  }

  // Z
  token = patterns$1.timezoneZ.exec(timezoneString);
  if (token) {
    return 0
  }

  var hours;

  // ±hh
  token = patterns$1.timezoneHH.exec(timezoneString);
  if (token) {
    hours = parseInt(token[1], 10);

    if (!validateTimezone(hours)) {
      return NaN
    }

    return -(hours * MILLISECONDS_IN_HOUR$1)
  }

  // ±hh:mm or ±hhmm
  token = patterns$1.timezoneHHMM.exec(timezoneString);
  if (token) {
    hours = parseInt(token[1], 10);
    var minutes = parseInt(token[2], 10);

    if (!validateTimezone(hours, minutes)) {
      return NaN
    }

    absoluteOffset = Math.abs(hours) * MILLISECONDS_IN_HOUR$1 + minutes * MILLISECONDS_IN_MINUTE$1;
    return hours > 0 ? -absoluteOffset : absoluteOffset
  }

  // IANA time zone
  if (isValidTimezoneIANAString(timezoneString)) {
    date = new Date(date || Date.now());
    var utcDate = isUtcDate ? date : toUtcDate(date);

    var offset = calcOffset(utcDate, timezoneString);

    var fixedOffset = isUtcDate ? offset : fixOffset(date, offset, timezoneString);

    return -fixedOffset
  }

  return NaN
}

function toUtcDate(date) {
  return newDateUTC(
    date.getFullYear(),
    date.getMonth(),
    date.getDate(),
    date.getHours(),
    date.getMinutes(),
    date.getSeconds(),
    date.getMilliseconds()
  )
}

function calcOffset(date, timezoneString) {
  var tokens = tzTokenizeDate(date, timezoneString);

  // ms dropped because it's not provided by tzTokenizeDate
  var asUTC = newDateUTC(
    tokens[0],
    tokens[1] - 1,
    tokens[2],
    tokens[3] % 24,
    tokens[4],
    tokens[5],
    0
  ).getTime();

  var asTS = date.getTime();
  var over = asTS % 1000;
  asTS -= over >= 0 ? over : 1000 + over;
  return asUTC - asTS
}

function fixOffset(date, offset, timezoneString) {
  var localTS = date.getTime();

  // Our UTC time is just a guess because our offset is just a guess
  var utcGuess = localTS - offset;

  // Test whether the zone matches the offset for this ts
  var o2 = calcOffset(new Date(utcGuess), timezoneString);

  // If so, offset didn't change, and we're done
  if (offset === o2) {
    return offset
  }

  // If not, change the ts by the difference in the offset
  utcGuess -= o2 - offset;

  // If that gives us the local time we want, we're done
  var o3 = calcOffset(new Date(utcGuess), timezoneString);
  if (o2 === o3) {
    return o2
  }

  // If it's different, we're in a hole time. The offset has changed, but we don't adjust the time
  return Math.max(o2, o3)
}

function validateTimezone(hours, minutes) {
  return -23 <= hours && hours <= 23 && (minutes == null || (0 <= minutes && minutes <= 59))
}

var validIANATimezoneCache = {};
function isValidTimezoneIANAString(timeZoneString) {
  if (validIANATimezoneCache[timeZoneString]) return true
  try {
    validIANATimezoneCache[timeZoneString] = true;
    return true
  } catch (error) {
    return false
  }
}

/** Regex to identify the presence of a time zone specifier in a date string */
var tzPattern = /(Z|[+-]\d{2}(?::?\d{2})?| UTC| [a-zA-Z]+\/[a-zA-Z_]+(?:\/[a-zA-Z_]+)?)$/;

var MILLISECONDS_IN_HOUR = 3600000;
var MILLISECONDS_IN_MINUTE = 60000;
var DEFAULT_ADDITIONAL_DIGITS = 2;

var patterns = {
  dateTimePattern: /^([0-9W+-]+)(T| )(.*)/,
  datePattern: /^([0-9W+-]+)(.*)/,
  plainTime: /:/,

  // year tokens
  YY: /^(\d{2})$/,
  YYY: [
    /^([+-]\d{2})$/, // 0 additional digits
    /^([+-]\d{3})$/, // 1 additional digit
    /^([+-]\d{4})$/, // 2 additional digits
  ],
  YYYY: /^(\d{4})/,
  YYYYY: [
    /^([+-]\d{4})/, // 0 additional digits
    /^([+-]\d{5})/, // 1 additional digit
    /^([+-]\d{6})/, // 2 additional digits
  ],

  // date tokens
  MM: /^-(\d{2})$/,
  DDD: /^-?(\d{3})$/,
  MMDD: /^-?(\d{2})-?(\d{2})$/,
  Www: /^-?W(\d{2})$/,
  WwwD: /^-?W(\d{2})-?(\d{1})$/,

  HH: /^(\d{2}([.,]\d*)?)$/,
  HHMM: /^(\d{2}):?(\d{2}([.,]\d*)?)$/,
  HHMMSS: /^(\d{2}):?(\d{2}):?(\d{2}([.,]\d*)?)$/,

  // time zone tokens (to identify the presence of a tz)
  timeZone: tzPattern,
};

/**
 * @name toDate
 * @category Common Helpers
 * @summary Convert the given argument to an instance of Date.
 *
 * @description
 * Convert the given argument to an instance of Date.
 *
 * If the argument is an instance of Date, the function returns its clone.
 *
 * If the argument is a number, it is treated as a timestamp.
 *
 * If an argument is a string, the function tries to parse it.
 * Function accepts complete ISO 8601 formats as well as partial implementations.
 * ISO 8601: http://en.wikipedia.org/wiki/ISO_8601
 * If the function cannot parse the string or the values are invalid, it returns Invalid Date.
 *
 * If the argument is none of the above, the function returns Invalid Date.
 *
 * **Note**: *all* Date arguments passed to any *date-fns* function is processed by `toDate`.
 * All *date-fns* functions will throw `RangeError` if `options.additionalDigits` is not 0, 1, 2 or undefined.
 *
 * @param {Date|String|Number} argument - the value to convert
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - the additional number of digits in the extended year format
 * @param {String} [options.timeZone=''] - used to specify the IANA time zone offset of a date String.
 * @returns {Date} the parsed date in the local time zone
 * @throws {TypeError} 1 argument required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // Convert string '2014-02-11T11:30:30' to date:
 * var result = toDate('2014-02-11T11:30:30')
 * //=> Tue Feb 11 2014 11:30:30
 *
 * @example
 * // Convert string '+02014101' to date,
 * // if the additional number of digits in the extended year format is 1:
 * var result = toDate('+02014101', {additionalDigits: 1})
 * //=> Fri Apr 11 2014 00:00:00
 */
function toDate(argument, dirtyOptions) {
  if (arguments.length < 1) {
    throw new TypeError('1 argument required, but only ' + arguments.length + ' present')
  }

  if (argument === null) {
    return new Date(NaN)
  }

  var options = dirtyOptions || {};

  var additionalDigits =
    options.additionalDigits == null
      ? DEFAULT_ADDITIONAL_DIGITS
      : toInteger(options.additionalDigits);
  if (additionalDigits !== 2 && additionalDigits !== 1 && additionalDigits !== 0) {
    throw new RangeError('additionalDigits must be 0, 1 or 2')
  }

  // Clone the date
  if (
    argument instanceof Date ||
    (typeof argument === 'object' && Object.prototype.toString.call(argument) === '[object Date]')
  ) {
    // Prevent the date to lose the milliseconds when passed to new Date() in IE10
    return new Date(argument.getTime())
  } else if (
    typeof argument === 'number' ||
    Object.prototype.toString.call(argument) === '[object Number]'
  ) {
    return new Date(argument)
  } else if (
    !(
      typeof argument === 'string' || Object.prototype.toString.call(argument) === '[object String]'
    )
  ) {
    return new Date(NaN)
  }

  var dateStrings = splitDateString(argument);

  var parseYearResult = parseYear(dateStrings.date, additionalDigits);
  var year = parseYearResult.year;
  var restDateString = parseYearResult.restDateString;

  var date = parseDate(restDateString, year);

  if (isNaN(date)) {
    return new Date(NaN)
  }

  if (date) {
    var timestamp = date.getTime();
    var time = 0;
    var offset;

    if (dateStrings.time) {
      time = parseTime(dateStrings.time);

      if (isNaN(time)) {
        return new Date(NaN)
      }
    }

    if (dateStrings.timeZone || options.timeZone) {
      offset = tzParseTimezone(dateStrings.timeZone || options.timeZone, new Date(timestamp + time));
      if (isNaN(offset)) {
        return new Date(NaN)
      }
    } else {
      // get offset accurate to hour in time zones that change offset
      offset = getTimezoneOffsetInMilliseconds(new Date(timestamp + time));
      offset = getTimezoneOffsetInMilliseconds(new Date(timestamp + time + offset));
    }

    return new Date(timestamp + time + offset)
  } else {
    return new Date(NaN)
  }
}

function splitDateString(dateString) {
  var dateStrings = {};
  var parts = patterns.dateTimePattern.exec(dateString);
  var timeString;

  if (!parts) {
    parts = patterns.datePattern.exec(dateString);
    if (parts) {
      dateStrings.date = parts[1];
      timeString = parts[2];
    } else {
      dateStrings.date = null;
      timeString = dateString;
    }
  } else {
    dateStrings.date = parts[1];
    timeString = parts[3];
  }

  if (timeString) {
    var token = patterns.timeZone.exec(timeString);
    if (token) {
      dateStrings.time = timeString.replace(token[1], '');
      dateStrings.timeZone = token[1].trim();
    } else {
      dateStrings.time = timeString;
    }
  }

  return dateStrings
}

function parseYear(dateString, additionalDigits) {
  var patternYYY = patterns.YYY[additionalDigits];
  var patternYYYYY = patterns.YYYYY[additionalDigits];

  var token;

  // YYYY or ±YYYYY
  token = patterns.YYYY.exec(dateString) || patternYYYYY.exec(dateString);
  if (token) {
    var yearString = token[1];
    return {
      year: parseInt(yearString, 10),
      restDateString: dateString.slice(yearString.length),
    }
  }

  // YY or ±YYY
  token = patterns.YY.exec(dateString) || patternYYY.exec(dateString);
  if (token) {
    var centuryString = token[1];
    return {
      year: parseInt(centuryString, 10) * 100,
      restDateString: dateString.slice(centuryString.length),
    }
  }

  // Invalid ISO-formatted year
  return {
    year: null,
  }
}

function parseDate(dateString, year) {
  // Invalid ISO-formatted year
  if (year === null) {
    return null
  }

  var token;
  var date;
  var month;
  var week;

  // YYYY
  if (dateString.length === 0) {
    date = new Date(0);
    date.setUTCFullYear(year);
    return date
  }

  // YYYY-MM
  token = patterns.MM.exec(dateString);
  if (token) {
    date = new Date(0);
    month = parseInt(token[1], 10) - 1;

    if (!validateDate(year, month)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, month);
    return date
  }

  // YYYY-DDD or YYYYDDD
  token = patterns.DDD.exec(dateString);
  if (token) {
    date = new Date(0);
    var dayOfYear = parseInt(token[1], 10);

    if (!validateDayOfYearDate(year, dayOfYear)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, 0, dayOfYear);
    return date
  }

  // yyyy-MM-dd or YYYYMMDD
  token = patterns.MMDD.exec(dateString);
  if (token) {
    date = new Date(0);
    month = parseInt(token[1], 10) - 1;
    var day = parseInt(token[2], 10);

    if (!validateDate(year, month, day)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, month, day);
    return date
  }

  // YYYY-Www or YYYYWww
  token = patterns.Www.exec(dateString);
  if (token) {
    week = parseInt(token[1], 10) - 1;

    if (!validateWeekDate(year, week)) {
      return new Date(NaN)
    }

    return dayOfISOWeekYear(year, week)
  }

  // YYYY-Www-D or YYYYWwwD
  token = patterns.WwwD.exec(dateString);
  if (token) {
    week = parseInt(token[1], 10) - 1;
    var dayOfWeek = parseInt(token[2], 10) - 1;

    if (!validateWeekDate(year, week, dayOfWeek)) {
      return new Date(NaN)
    }

    return dayOfISOWeekYear(year, week, dayOfWeek)
  }

  // Invalid ISO-formatted date
  return null
}

function parseTime(timeString) {
  var token;
  var hours;
  var minutes;

  // hh
  token = patterns.HH.exec(timeString);
  if (token) {
    hours = parseFloat(token[1].replace(',', '.'));

    if (!validateTime(hours)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR
  }

  // hh:mm or hhmm
  token = patterns.HHMM.exec(timeString);
  if (token) {
    hours = parseInt(token[1], 10);
    minutes = parseFloat(token[2].replace(',', '.'));

    if (!validateTime(hours, minutes)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR + minutes * MILLISECONDS_IN_MINUTE
  }

  // hh:mm:ss or hhmmss
  token = patterns.HHMMSS.exec(timeString);
  if (token) {
    hours = parseInt(token[1], 10);
    minutes = parseInt(token[2], 10);
    var seconds = parseFloat(token[3].replace(',', '.'));

    if (!validateTime(hours, minutes, seconds)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR + minutes * MILLISECONDS_IN_MINUTE + seconds * 1000
  }

  // Invalid ISO-formatted time
  return null
}

function dayOfISOWeekYear(isoWeekYear, week, day) {
  week = week || 0;
  day = day || 0;
  var date = new Date(0);
  date.setUTCFullYear(isoWeekYear, 0, 4);
  var fourthOfJanuaryDay = date.getUTCDay() || 7;
  var diff = week * 7 + day + 1 - fourthOfJanuaryDay;
  date.setUTCDate(date.getUTCDate() + diff);
  return date
}

// Validation functions

var DAYS_IN_MONTH = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
var DAYS_IN_MONTH_LEAP_YEAR = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

function isLeapYearIndex(year) {
  return year % 400 === 0 || (year % 4 === 0 && year % 100 !== 0)
}

function validateDate(year, month, date) {
  if (month < 0 || month > 11) {
    return false
  }

  if (date != null) {
    if (date < 1) {
      return false
    }

    var isLeapYear = isLeapYearIndex(year);
    if (isLeapYear && date > DAYS_IN_MONTH_LEAP_YEAR[month]) {
      return false
    }
    if (!isLeapYear && date > DAYS_IN_MONTH[month]) {
      return false
    }
  }

  return true
}

function validateDayOfYearDate(year, dayOfYear) {
  if (dayOfYear < 1) {
    return false
  }

  var isLeapYear = isLeapYearIndex(year);
  if (isLeapYear && dayOfYear > 366) {
    return false
  }
  if (!isLeapYear && dayOfYear > 365) {
    return false
  }

  return true
}

function validateWeekDate(year, week, day) {
  if (week < 0 || week > 52) {
    return false
  }

  if (day != null && (day < 0 || day > 6)) {
    return false
  }

  return true
}

function validateTime(hours, minutes, seconds) {
  if (hours != null && (hours < 0 || hours >= 25)) {
    return false
  }

  if (minutes != null && (minutes < 0 || minutes >= 60)) {
    return false
  }

  if (seconds != null && (seconds < 0 || seconds >= 60)) {
    return false
  }

  return true
}

var assign_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = assign;

function assign(target, dirtyObject) {
  if (target == null) {
    throw new TypeError('assign requires that input parameter not be null or undefined');
  }

  dirtyObject = dirtyObject || {};

  for (var property in dirtyObject) {
    if (Object.prototype.hasOwnProperty.call(dirtyObject, property)) {
      target[property] = dirtyObject[property];
    }
  }

  return target;
}

module.exports = exports.default;
});

var cloneObject_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cloneObject;

var _index = _interopRequireDefault(assign_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function cloneObject(dirtyObject) {
  return (0, _index.default)({}, dirtyObject);
}

module.exports = exports.default;
});

const cloneObject = /*@__PURE__*/getDefaultExportFromCjs(cloneObject_1);

/**
 * @name utcToZonedTime
 * @category Time Zone Helpers
 * @summary Get a date/time representing local time in a given time zone from the UTC date
 *
 * @description
 * Returns a date instance with values representing the local time in the time zone
 * specified of the UTC time from the date provided. In other words, when the new date
 * is formatted it will show the equivalent hours in the target time zone regardless
 * of the current system time zone.
 *
 * @param {Date|String|Number} date - the date with the relevant UTC time
 * @param {String} timeZone - the time zone to get local time for, can be an offset or IANA time zone
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - passed to `toDate`. See [toDate]{@link https://date-fns.org/docs/toDate}
 * @returns {Date} the new date with the equivalent time in the time zone
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // In June 10am UTC is 6am in New York (-04:00)
 * const result = utcToZonedTime('2014-06-25T10:00:00.000Z', 'America/New_York')
 * //=> Jun 25 2014 06:00:00
 */
function utcToZonedTime(dirtyDate, timeZone, options) {
  var date = toDate(dirtyDate, options);

  var offsetMilliseconds = tzParseTimezone(timeZone, date, true);

  var d = new Date(date.getTime() - offsetMilliseconds);

  return new Date(
    d.getUTCFullYear(),
    d.getUTCMonth(),
    d.getUTCDate(),
    d.getUTCHours(),
    d.getUTCMinutes(),
    d.getUTCSeconds(),
    d.getUTCMilliseconds()
  )
}

/**
 * @name zonedTimeToUtc
 * @category Time Zone Helpers
 * @summary Get the UTC date/time from a date representing local time in a given time zone
 *
 * @description
 * Returns a date instance with the UTC time of the provided date of which the values
 * represented the local time in the time zone specified. In other words, if the input
 * date represented local time in time time zone, the timestamp of the output date will
 * give the equivalent UTC of that local time regardless of the current system time zone.
 *
 * @param {Date|String|Number} date - the date with values representing the local time
 * @param {String} timeZone - the time zone of this local time, can be an offset or IANA time zone
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - passed to `toDate`. See [toDate]{@link https://date-fns.org/docs/toDate}
 * @returns {Date} the new date with the equivalent time in the time zone
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // In June 10am in Los Angeles is 5pm UTC
 * const result = zonedTimeToUtc(new Date(2014, 5, 25, 10, 0, 0), 'America/Los_Angeles')
 * //=> 2014-06-25T17:00:00.000Z
 */
function zonedTimeToUtc(date, timeZone, options) {
  if (typeof date === 'string' && !date.match(tzPattern)) {
    var extendedOptions = cloneObject(options);
    extendedOptions.timeZone = timeZone;
    return toDate(date, extendedOptions)
  }

  var d = toDate(date, options);

  var utc = newDateUTC(
    d.getFullYear(),
    d.getMonth(),
    d.getDate(),
    d.getHours(),
    d.getMinutes(),
    d.getSeconds(),
    d.getMilliseconds()
  ).getTime();

  var offsetMilliseconds = tzParseTimezone(timeZone, new Date(utc));

  return new Date(utc + offsetMilliseconds)
}

/**
 * is used in the formattingServiceFactory if no valid is provided
 */
const defaultFactoryConfig = {
  dateFormat: DEFAULT_LOCALE.dateFormat,
  timeFormat: DEFAULT_LOCALE.timeFormat,
};
/**
 * Uses the defaultFactoryConfig if nothing was provided
 * @param config
 * @returns
 */
function formattingServiceFactory(config = {}) {
  const { dateFormat, timeFormat, timeZone } = Object.assign(Object.assign({}, defaultFactoryConfig), config);
  return new FormattingService(dateFormat, timeFormat, timeZone);
}
/**
 * Used to format date values
 */
class FormattingService {
  constructor(dateFormat, timeFormat, timeZone) {
    this.dateFormat = dateFormat;
    this.timeFormat = timeFormat;
    this.timeZone = timeZone;
  }
  /**
   * Returns date in any format with utc timezone set by property time-zone or with local browser timezone
   * @param date
   * @returns
   */
  formatDate(date) {
    if (!date) {
      return '';
    }
    if (typeof date === 'number') {
      return this.formatNumber(date, this.dateFormat);
    }
    if (typeof date === 'string') {
      return format(this.formatDateToDate(date), this.dateFormat);
    }
    return format(date, this.dateFormat);
  }
  formatDateToDate(date) {
    return parse(date, this.dateFormat, new Date());
  }
  /**
   * Returns time in any format with utc timezone set by property time-zone or with local browser timezone
   * @param time
   * @returns
   */
  formatTime(time) {
    if (!time) {
      return '';
    }
    if (typeof time === 'number') {
      return this.formatNumber(time, this.timeFormat);
    }
    return format(time, this.timeFormat);
  }
  /**
   * Formats a date/time range for display
   * @param {string} [dateStart] - The start date
   * @param {string} [dateEnd] - The end date
   * @param {string} [timeStart] - The start time
   * @param {string} [timeEnd] - The end time
   * @returns {string} The formatted date/time range string
   */
  formatRange(dateStart, dateEnd, timeStart, timeEnd) {
    let value = '';
    if (dateStart) {
      value += dateStart;
    }
    if (dateStart && timeStart) {
      value += ' ';
    }
    if (timeStart) {
      value += timeStart;
    }
    if ((dateStart && dateEnd && dateEnd !== dateStart) || (timeStart && timeEnd && timeEnd !== timeStart)) {
      value += ' – ';
    }
    if (dateEnd && dateEnd !== dateStart) {
      value += dateEnd;
    }
    if (dateEnd && timeEnd && dateEnd !== dateStart) {
      value += ' ';
    }
    if (timeEnd && (timeStart !== timeEnd || dateStart !== dateEnd)) {
      value += timeEnd;
    }
    return value;
  }
  formatDateWithOptionalTime(date, time) {
    const dateString = time ? date + ' ' + time : date;
    const format = time ? this.dateFormat + ' ' + this.timeFormat : this.dateFormat;
    return parse(dateString, format, new Date());
  }
  timestampToZonedTime(timestamp) {
    if (this.timeZone) {
      return utcToZonedTime(fromUnixTime(timestamp), this.timeZone);
    }
    return fromUnixTime(timestamp);
  }
  formatNumber(timestamp, formatting) {
    return format(this.timestampToZonedTime(timestamp / 1000), formatting);
  }
}

/**
 * This config is used when no config was provided to the dateServiceFactory function
 */
const defaultDateConfig = {
  formatService: formattingServiceFactory(),
  rangeDateTime: false,
  hideSeconds: false,
  minDate: 0,
  maxDate: 0,
};
const FORMAT_BLOCKS = {
  dd: /(0[1-9]|[12][0-9]|3[01])/,
  MM: /(0[1-9]|1[0-2])/,
  yy: /([0-9]{2})/,
  d: /([1-9]|[12][0-9]|3[01])/,
  M: /([1-9]|1[0-2])/,
  y: /([12][0-9]{3})/,
  h: /(0?[0-9]|1[0-2])/,
  H: /(0?[0-9]|[1][0-9]|2[0-3])/,
  mm: /([0-5][0-9])/,
  ss: /([0-5][0-9])/,
  ' a': /( [AaPp][Mm])/,
};
/** All the date formats that can be parsed from a string */
const SUPPORTED_DATE_FORMATS_DEFAULT = [
  'dd.MM.y',
  'MM/dd/y',
  'dd/MM/y',
  'y-MM-dd',
  'd.M.y',
  'M/d/y',
  'd/M/y',
  'y-M-d',
  'dd.MM.yy',
  'MM/dd/yy',
  'dd/MM/yy',
  'yy-MM-dd',
  'd.M.yy',
  'M/d/yy',
  'd/M/yy',
  'yy-M-d',
];
/** All the date formats that can be parsed from a string. Prioritising "/"-Formats, where Days are before Months */
const SUPPORTED_DATE_FORMATS_START_WITH_DAY = [
  'dd.MM.y',
  'dd/MM/y',
  'dd.MM.yy',
  'dd/MM/yy',
  'MM/dd/y',
  'MM/dd/yy',
  'y-MM-dd',
  'd.M.y',
  'd/M/y',
  'M/d/y',
  'y-M-d',
  'yy-MM-dd',
  'd.M.yy',
  'd/M/yy',
  'M/d/yy',
  'yy-M-d',
];
/** All the time formats that can be parsed from a string */
const SUPPORTED_TIME_FORMATS = ['h:mm:ss a', 'H:mm:ss', 'h:mm a', 'H:mm', 'h a'];
/**
 * Uses the defaultDateConfig if no value is provided
 * @param config
 * @returns
 */
function dateServiceFactory(config = {}) {
  const { hideSeconds, rangeDateTime, minDate, maxDate, formatService } = Object.assign(Object.assign({}, defaultDateConfig), config);
  return new DateService(hideSeconds, rangeDateTime, minDate, maxDate, formatService);
}
/**
 * Used to manipulate date values
 */
class DateService {
  constructor(hideSeconds, rangeDateTime, minDate, maxDate, formatService) {
    this.hideSeconds = hideSeconds;
    this.rangeDateTime = rangeDateTime;
    this.minDate = minDate;
    this.maxDate = maxDate;
    this.formatService = formatService;
  }
  getTodayValue(timeZone = undefined) {
    return timeZone === undefined ? new Date() : new Date(new Date().toLocaleString('en-US', { timeZone: timeZone }));
  }
  getDataFromInput(input) {
    const dates = this.getDateFromDateInput(input);
    const times = this.getTimeFromDateInput(input);
    const correctedData = this.correctInputData(Object.assign(Object.assign({}, dates), times));
    return Object.assign(Object.assign({}, correctedData), { valid: true });
  }
  /**
   * check whether the input contains valid date and time values and that they are also in the correct time order.
   * @param input
   * @returns
   */
  validateInput(input) {
    const dateStart = startOfDay(this.formatService.formatDateToDate(input.dateStart || ''));
    const dateEnd = startOfDay(this.formatService.formatDateToDate(input.dateEnd || ''));
    if (isAfter(dateStart, endOfDay(this.maxDate)) ||
      isBefore(dateStart, startOfDay(this.minDate)) ||
      isAfter(dateEnd, endOfDay(this.maxDate)) ||
      isBefore(dateEnd, startOfDay(this.minDate)) ||
      isAfter(dateStart, dateEnd)) {
      return { valid: false, timeEnd: null, dateEnd: null };
    }
    else {
      return { valid: true };
    }
  }
  /**
   * Corrects date values, puts them within allowed ranges and swaps values if necessary.
   * @param dateObj dateStart, dateEnd, timeStart, timeEnd
   * @returns corrected dateStart, dateEnd, timeStart, timeEnd
   */
  correctInputData(dateObj) {
    const { dateStart, dateEnd, timeStart, timeEnd } = dateObj;
    let parsedDateStart, parsedDateEnd;
    // Case with only one date and two times
    if (!!dateStart && !!timeStart && !!timeEnd && !dateEnd) {
      parsedDateStart = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeStart));
      parsedDateEnd = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeEnd));
      if (isBefore(parsedDateEnd, parsedDateStart)) {
        return Object.assign({}, this.createInputData(parsedDateEnd, parsedDateStart));
      }
    }
    // Case when nothing valid was entered into the input
    else if (!dateStart || !dateEnd) {
      return dateObj;
    }
    // Base case
    else {
      parsedDateStart = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeStart));
      parsedDateEnd = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateEnd, timeEnd));
      if (isBefore(parsedDateEnd, parsedDateStart)) {
        return Object.assign({}, this.createInputData(parsedDateEnd, parsedDateStart));
      }
    }
    return Object.assign({}, this.createInputData(parsedDateStart, parsedDateEnd));
  }
  dateWithinAllowedRange(date) {
    return clamp(date, {
      start: this.formatService.timestampToZonedTime(this.minDate / 1000),
      end: this.maxDate === 0 ? Infinity : this.formatService.timestampToZonedTime(this.maxDate / 1000),
    });
  }
  /**
   * Creates valid InputData object from two dates.
   * @param dateStart Date to infer dateStart and timeStart from
   * @param dateEnd Date to infer dateEnd and timeEnd from
   */
  createInputData(dateStart, dateEnd) {
    return {
      dateStart: this.formatService.formatDate(dateStart),
      dateEnd: this.formatService.formatDate(dateEnd) || null,
      timeStart: this.formatService.formatTime(dateStart),
      timeEnd: this.formatService.formatTime(dateEnd) || null,
    };
  }
  /**
   * Returns utc timestamp created by timezone from property time-zone or with local browser timezone
   * @param date string parsed with formatDate
   * @param time string in format 00:00
   */
  toSelectionTimestamp(date, time) {
    if (!date || !this.formatService.timeZone) {
      return undefined;
    }
    const dateTime = this.formatService.formatDateWithOptionalTime(date, time);
    return getUnixTime(zonedTimeToUtc(dateTime, this.formatService.timeZone)) * 1000;
  }
  getDateFromDateInput(input) {
    const matches = this.getPartFromDateInput(input, 'dateStart', 'dateEnd', this.DATE_FORMATS);
    const result = { dateStart: null, dateEnd: null };
    result.dateStart = matches.dateStart ? this.formatService.formatDate(matches.dateStart) : null;
    result.dateEnd = matches.dateEnd ? this.formatService.formatDate(matches.dateEnd) : null;
    return result;
  }
  getTimeFromDateInput(input) {
    const matches = this.getPartFromDateInput(input, 'timeStart', 'timeEnd', this.TIME_FORMATS);
    const result = { timeStart: null, timeEnd: null };
    result.timeStart = matches.timeStart ? this.formatService.formatTime(matches.timeStart) : null;
    result.timeEnd = matches.timeEnd ? this.formatService.formatTime(matches.timeEnd) : null;
    return result;
  }
  getPartFromDateInput(input, startKey, endKey, tests) {
    var _a, _b;
    let match;
    let countMatches = 0;
    const dateObj = {
      [startKey]: null,
      [endKey]: null,
    };
    const result = {
      [startKey]: null,
      [endKey]: null,
    };
    Object.keys(tests).forEach((key) => {
      var _a;
      const pattern = tests[key];
      // Match time input
      while ((match = pattern.exec(input)) !== null) {
        // This is necessary to avoid infinite loops with zero-width matches
        if (match.index === pattern.lastIndex) {
          pattern.lastIndex++;
        }
        if (match[0] !== null && match.index !== ((_a = dateObj[startKey]) === null || _a === void 0 ? void 0 : _a.atIndex)) {
          countMatches++;
          if (countMatches >= 3)
            break;
          if (this.rangeDateTime && countMatches === 2) {
            dateObj[endKey] = { match: match[0], fromFormat: key, atIndex: match.index };
          }
          else {
            // only match first date
            if (countMatches >= 2)
              break;
            dateObj[startKey] = { match: match[0], fromFormat: key, atIndex: match.index };
          }
        }
      }
    });
    // parse to a normal date as the matches can be from different formats
    if (!!dateObj[endKey]) {
      const { match, fromFormat } = dateObj[endKey];
      const normalizedDate = parse(match, fromFormat, new Date());
      result[endKey] = normalizedDate;
    }
    if (!!dateObj[startKey]) {
      const { match, fromFormat } = dateObj[startKey];
      const normalizedDate = parse(match, fromFormat, new Date());
      result[startKey] = normalizedDate;
    }
    // Swap dates if match found later is before other match in the input string.
    // This happens if match before is from a format that gets searched later.
    if (((_a = dateObj[startKey]) === null || _a === void 0 ? void 0 : _a.atIndex) > ((_b = dateObj[endKey]) === null || _b === void 0 ? void 0 : _b.atIndex)) {
      const temp = result[startKey];
      result[startKey] = result[endKey];
      result[endKey] = temp;
    }
    return result;
  }
  get DATE_FORMATS() {
    var _a, _b;
    const formats = {};
    const DATE_FORMATS = ((_a = this.formatService.dateFormat) === null || _a === void 0 ? void 0 : _a.toLowerCase().startsWith('dd/')) || ((_b = this.formatService.dateFormat) === null || _b === void 0 ? void 0 : _b.toLowerCase().startsWith('d/'))
      ? SUPPORTED_DATE_FORMATS_START_WITH_DAY
      : SUPPORTED_DATE_FORMATS_DEFAULT;
    DATE_FORMATS.forEach((format) => {
      let regex = format;
      ['.', '/', '-'].forEach((symbol) => {
        regex = regex.replace(new RegExp(`[${symbol}]`, 'g'), `[${symbol}]`);
      });
      Object.keys(FORMAT_BLOCKS).forEach((block) => {
        regex = regex.replace(new RegExp(block, 'g'), FORMAT_BLOCKS[block].source);
      });
      formats[format] = new RegExp(regex, 'g');
    });
    return formats;
  }
  get TIME_FORMATS() {
    const formats = {};
    SUPPORTED_TIME_FORMATS.map((format) => {
      let regex = format;
      [':'].forEach((symbol) => {
        regex = regex.replace(new RegExp(`[${symbol}]`, 'g'), `[${symbol}]`);
      });
      Object.keys(FORMAT_BLOCKS).forEach((block) => {
        regex = regex.replace(new RegExp(block, 'g'), FORMAT_BLOCKS[block].source);
      });
      formats[format] = new RegExp(regex, 'g');
    });
    return formats;
  }
}

const dialogCss = "/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:\"Bosch-Sans\";font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.clickable-element{transition:0.4s;cursor:pointer}.clickable-element:active,.clickable-element.active{color:#007bc0}.clickable-element:active:hover,.clickable-element:hover .clickable-element:focus{color:#9dc9ff}.clickable-element:disabled,.clickable-element.disabled{color:#a4abb3}.clickable-element:disabled:active,.clickable-element:disabled.active,.clickable-element:disabled:active:hover,.clickable-element:disabled:hover,.clickable-element.disabled:active,.clickable-element.disabled.active,.clickable-element.disabled:active:hover,.clickable-element.disabled:hover{color:#a4abb3}button.button-primary,button.button-default,input.button-primary,input.button-default{color:#ffffff;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;background-color:#007bc0;background-size:100% 200%;border-width:0;border-color:transparent;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-primary:hover,button.button-primary:focus-visible,button.button-default:hover,button.button-default:focus-visible,input.button-primary:hover,input.button-primary:focus-visible,input.button-default:hover,input.button-default:focus-visible{background-color:#00629a;color:#ffffff;outline:0}button.button-primary:active,button.button-default:active,input.button-primary:active,input.button-default:active{background-color:#004975;color:#ffffff;outline:0}button.button-primary:disabled,button.button-primary.disabled,button.button-default:disabled,button.button-default.disabled,input.button-primary:disabled,input.button-primary.disabled,input.button-default:disabled,input.button-default.disabled{background-color:#c1c7cc;color:#656a6f;background-image:none;cursor:default}button.button-primary.button-lg,button.button-default.button-lg,input.button-primary.button-lg,input.button-default.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-sm,button.button-default.button-sm,input.button-primary.button-sm,input.button-default.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-xs,button.button-default.button-xs,input.button-primary.button-xs,input.button-default.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary:not(.mat-icon-button):has(mat-icon),button.button-default:not(.mat-icon-button):has(mat-icon),input.button-primary:not(.mat-icon-button):has(mat-icon),input.button-default:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-primary.mat-icon-button,button.button-default.mat-icon-button,input.button-primary.mat-icon-button,input.button-default.mat-icon-button{padding:12px}button.button-secondary,input.button-secondary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 15px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-width:1px;border-color:#007bc0;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:hover,button.button-secondary:focus-visible,input.button-secondary:hover,input.button-secondary:focus-visible{border-color:#004975;outline:0;color:#004975;background-color:#d1e4ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:active,input.button-secondary:active{border-color:#004975;color:#004975;outline:0;background-color:#9dc9ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:disabled,button.button-secondary.disabled,input.button-secondary:disabled,input.button-secondary.disabled{color:#979ea4;border-color:#979ea4;background-color:transparent;cursor:default}button.button-secondary.button-lg,input.button-secondary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-sm,input.button-secondary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-xs,input.button-secondary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary:not(.mat-icon-button):has(mat-icon),input.button-secondary:not(.mat-icon-button):has(mat-icon){padding:0 15px 0 13px}button.button-secondary.mat-icon-button,input.button-secondary.mat-icon-button{padding:11px}button.button-tertiary,input.button-tertiary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-tertiary:hover,button.button-tertiary:focus-visible,input.button-tertiary:hover,input.button-tertiary:focus-visible{outline:0;color:#00629a;background-color:#d1e4ff}button.button-tertiary:active,input.button-tertiary:active{color:#004975;background-color:#9dc9ff;outline:0}button.button-tertiary:disabled,button.button-tertiary.disabled,input.button-tertiary:disabled,input.button-tertiary.disabled{color:#979ea4;background-color:transparent;cursor:default}button.button-tertiary.button-lg,input.button-tertiary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-sm,input.button-tertiary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-xs,input.button-tertiary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary:not(.mat-icon-button):has(mat-icon),input.button-tertiary:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-tertiary.mat-icon-button,input.button-tertiary.mat-icon-button{padding:12px}button.button-integrated,input.button-integrated{color:#000000;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-integrated:hover,button.button-integrated:focus-visible,input.button-integrated:hover,input.button-integrated:focus-visible{color:#007bc0;outline:0}button.button-integrated:active,input.button-integrated:active{color:#00629a;outline:0}button.button-integrated:disabled,button.button-integrated.disabled,input.button-integrated:disabled,input.button-integrated.disabled{color:#979ea4 !important;background-color:transparent;cursor:default}button.button-integrated:not(.mat-icon-button):has(mat-icon),input.button-integrated:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-integrated.mat-icon-button,input.button-integrated.mat-icon-button{padding:12px}button.button-link,input.button-link{color:#007bc0;background-color:transparent;border:none;-webkit-box-shadow:none;box-shadow:none;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0 16px 0 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link::after,input.button-link::after{position:absolute;font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:18px;content:\"\\e181\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}button.button-link:hover,button.button-link:focus-visible,input.button-link:hover,input.button-link:focus-visible{color:#00629a;text-decoration:underline;outline:0}button.button-link:active,input.button-link:active{color:#004975;text-decoration:underline}button.button-link.disabled,button.button-link[disabled],button.button-link fieldset[disabled],input.button-link.disabled,input.button-link[disabled],input.button-link fieldset[disabled]{color:#c1c7cc;text-decoration:none;-webkit-transition:none;-o-transition:none;transition:none}button.button-link.disabled::after,button.button-link[disabled]::after,button.button-link fieldset[disabled]::after,input.button-link.disabled::after,input.button-link[disabled]::after,input.button-link fieldset[disabled]::after{-webkit-transform:translate(0, 0);-ms-transform:translate(0, 0);-o-transform:translate(0, 0);transform:translate(0, 0)}button.button-link.button-lg,input.button-link.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-lg::after,input.button-link.button-lg::after{font-size:22px}button.button-link.button-sm,input.button-link.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:14px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-sm::after,input.button-link.button-sm::after{font-size:14px}button.button-link.no-arrow::after,input.button-link.no-arrow::after{content:\"\"}button.button-link:not(.mat-icon-button):has(mat-icon),input.button-link:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-link.mat-icon-button,input.button-link.mat-icon-button{padding:12px}button.button-block,input[type=submit].button-block,input[type=reset].button-block,input[type=button].button-block{display:block;width:100%}/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bci-core-date-time-picker-container{background:#ffffff;display:flex;flex-flow:column nowrap;outline:none;box-shadow:0px 0px 8px rgba(0, 0, 0, 0.25)}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container{display:flex}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.hidden{display:none}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.year-month{height:292px;min-width:292px}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.year-month.has-time{width:584px}.bci-core-date-time-picker-container .bci-core-save{color:#005587;text-align:end;border-top:1px solid #8a9097;font-size:16px;padding:5px 10px 5px 0px;height:29px}.bci-core-date-time-picker-container .bci-core-save .bosch-ic-clickable{position:relative;top:4px}.bci-core-date-time-picker-container .bci-core-datepicker-selection{text-align:right;border-top:1px solid #8a9097;display:flex;justify-content:space-between;align-items:center;padding:0 10px;height:68px}.bci-core-date-time-picker-container .bci-core-datepicker-selection .close{margin-left:auto}@media (max-width: 767px){.bci-core-date-time-picker-sub-container{flex-flow:column nowrap;width:292px !important}}@media (min-width: 768px){.bci-core-date-time-picker-sub-container{flex-flow:row nowrap}.bci-core-date-time-picker-sub-container>bci-datepicker{order:1;flex:1}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container{order:2;flex:1}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container:not(.no-border){border-left:1px solid #8a9097}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container bci-timepicker .bci-core-timepicker-container{padding-right:8px}.bci-core-date-time-picker-sub-container>.bci-core-save{order:3;flex:2}}";

const DateTimeDialog = /*@__PURE__*/ proxyCustomElement(class extends H {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.selectionChanged = createEvent(this, "selectionChanged", 7);
    this.showTemplates = false;
    /**
     * Case null: Year/Month Selection closed.
     * Case 'year': Year Selection active.
     * Case 'month': Month Selection active.
     */
    this.yearMonthSelectionStep = null;
    /** Tracks how often the user switched pages in the year selection */
    this.yearSelectionPageOffset = 0;
    /** Object to keep track of the currently selected date range */
    this.datesObj = {
      startDate: null,
      endDate: null,
    };
    /**
     * Optional - inject the FormattingService instance
     */
    this.formatService = formattingServiceFactory();
    /**
     * Optional - inject the DateService instance
     */
    this.dateService = dateServiceFactory({ formatService: this.formatService });
  }
  emitSelectionChanged(event) {
    this.selectionChanged.emit(event);
  }
  onTemplateUpdate({ detail: { firstEmit, timeStamp, id } }) {
    const event = {
      reason: 'frame',
      timestampStart: timeStamp.timestampStart,
      timestampEnd: timeStamp.timestampEnd,
      firstEmit,
      id,
    };
    this.selectionChanged.emit(event);
  }
  timeSelectedHandler({ detail: data }) {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    let { timeStart, timeEnd } = this.datePickerInputObj;
    if (data.timepickerId === 'timeStart_' + this.dateTimePickerId) {
      timeStart = data.updatedTime;
    }
    if (data.timepickerId === 'timeEnd_' + this.dateTimePickerId) {
      timeEnd = data.updatedTime;
    }
    const event = {
      reason: 'time',
      timeStart,
      timeEnd,
    };
    this.selectionChanged.emit(event);
  }
  showTimeRange() {
    this.showTemplates = false;
  }
  /**
   * Clears internal timer and resets selected template
   */
  async close() {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
  }
  /**
   * Closes the Year/Month Selection
   */
  async closeYearMonthSelection() {
    this.yearMonthSelectionStep = null;
  }
  showTemplate() {
    this.showTemplates = true;
  }
  toggleShowYearMonthSelectionHandler(event) {
    // null means the yearMonthSelection is closed
    this.yearMonthSelectionStep = event.detail.newState ? 'year' : null;
    this.yearSelectionPageOffset = 0;
  }
  arrowClickedHandler(event) {
    if (!!this.yearMonthSelectionStep && this.yearMonthSelectionStep === 'year') {
      switch (event.detail.direction) {
        case 'next':
          this.yearSelectionPageOffset++;
          break;
        case 'prev':
          this.yearSelectionPageOffset--;
          break;
      }
      return;
    }
    this.handleMonthChangeOnArrowClick(event.detail);
  }
  handleMonthChangeOnArrowClick(detail) {
    let newYear = this.dateObj.calendarDateYear;
    let newMonth;
    switch (detail.direction) {
      case 'next':
        newMonth = (this.dateObj.calendarDateMonth + 1) % 12;
        if (newMonth === 0) {
          newYear = this.dateObj.calendarDateYear + 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
      case 'prev':
        newMonth = this.dateObj.calendarDateMonth - 1;
        if (newMonth === -1) {
          newMonth = 11;
          newYear = this.dateObj.calendarDateYear - 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
    }
  }
  yearMonthSelectedHandler(event) {
    if (!this.yearMonthSelectionStep)
      return;
    if (this.yearMonthSelectionStep === 'year') {
      const newYear = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: newYear });
      this.yearMonthSelectionStep = 'month';
    }
    else if (this.yearMonthSelectionStep === 'month') {
      const newMonth = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth });
      this.yearMonthSelectionStep = null;
    }
  }
  singleDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDate: format(newDate, this.datePickerConfig.formatDate), selectedDate: format(newDate, this.datePickerConfig.formatDate) });
    const dateSelectedEvent = {
      reason: 'date',
      startDate: this.dateObj.selectedDate,
      endDate: null,
      rangeCompleted: false,
    };
    this.emitSelectionChanged(dateSelectedEvent);
  }
  rangeDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    let rangeComplete = false;
    // reset on subsequent clicks after first and second dates have been set
    if (this.datesObj.startDate && this.datesObj.endDate && this.datesObj.startDate !== this.datesObj.endDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: format(newDate, this.datePickerConfig.formatDate), endDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (!this.datesObj.startDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (isBefore(newDate, parse(this.datesObj.startDate, this.datePickerConfig.formatDate, new Date()))) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: this.datesObj.startDate, startDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    else {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    const dateSelectedEvent = Object.assign(Object.assign({}, this.datesObj), { reason: 'date', rangeCompleted: rangeComplete });
    this.emitSelectionChanged(dateSelectedEvent);
  }
  userInputDate(newValue) {
    if (!newValue)
      return;
    if (newValue.dateStart) {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: getMonth(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())), calendarDateYear: getYear(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())) });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: newValue.dateStart, endDate: newValue.dateEnd });
    }
    else {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), selectedDate: null });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: null, endDate: null });
    }
  }
  async componentWillLoad() {
    this.dateObj = {
      calendarDate: format(utcToZonedTime(new Date(), this.datePickerConfig.timeZone), this.datePickerConfig.formatDate),
      calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      selectedDate: null,
    };
  }
  applyTemplate(option, firstEmit) {
    this.onTemplateUpdate({ detail: { timeStamp: option.calculateTemplate(), firstEmit, id: option.id } });
  }
  render() {
    var _a, _b, _c, _d;
    return (h(Host, { id: `overlay_${this.dateTimePickerId}` }, h("div", { class: "bci-core-date-time-picker-container", tabindex: "-1" }, !this.showTemplates && this.hide !== 'calendar' && (h("div", null, h("bci-month-header", { "data-test": "webcore.webcomponents.bci-datepicker.year-month", year: this.dateObj.calendarDateYear, month: this.dateObj.calendarDateMonth, yearMonthSelectionStep: this.yearMonthSelectionStep }), h("div", { class: `bci-core-date-time-picker-sub-container year-month ${!this.yearMonthSelectionStep ? 'hidden' : ''} ${this.hide !== 'time' ? 'has-time' : ''}` }, h("bci-year-month", { selectionStep: this.yearMonthSelectionStep, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, dateObj: this.dateObj, yearSelectionPageOffset: this.yearSelectionPageOffset })))), !this.showTemplates && (h("div", { class: `bci-core-date-time-picker-sub-container ${!!this.yearMonthSelectionStep ? 'hidden' : ''} ` }, this.hide !== 'calendar' && (h("bci-datepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.dateTimePickerInput", bciDatepickerId: this.dateTimePickerId, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, rangeDate: this.datePickerConfig.rangeDateTime, sundayFirst: this.datePickerConfig.sundayFirst, formatDate: this.datePickerConfig.formatDate, datePickerInputObj: this.datePickerInputObj, timeZone: this.datePickerConfig.timeZone, selectedYear: this.dateObj.calendarDateYear, selectedMonth: this.dateObj.calendarDateMonth, selectedDate: this.dateObj.selectedDate, today: this.formatService.formatDate(this.dateService.getTodayValue(this.datePickerConfig.timeZone)), datesObj: this.datesObj })), this.hide !== 'time' && (h("div", { class: { 'bci-core-time-picker-container': true, 'no-border': this.hide === 'calendar' } }, h("bci-timepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.starttime", class: { 'full-height': !this.timePickerConfig.rangeDateTime }, timeInput: (_a = this.datePickerInputObj) === null || _a === void 0 ? void 0 : _a.timeStart, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeStart_' + this.dateTimePickerId, rangeTimePicker: false, timeTitle: this.timePickerConfig.rangeDateTime ? translate('startTime') : translate('time') }), h("bci-timepicker", { "data-test": "datetime-picker.core-datetime-picker.endtime", class: { hidden: !this.timePickerConfig.rangeDateTime }, timeInput: (_b = this.datePickerInputObj) === null || _b === void 0 ? void 0 : _b.timeEnd, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeEnd_' + this.dateTimePickerId, rangeTimePicker: true, timeTitle: translate('endTime') }))))), !this.showTemplates && (h("div", { class: "bci-core-datepicker-selection" }, ((_c = this.templatesList) === null || _c === void 0 ? void 0 : _c.length) > 1 && this.hide !== 'calendar' && (h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.showTemplate() }, translate('templates'))), ((_d = this.templatesList) === null || _d === void 0 ? void 0 : _d.length) === 1 && this.hide !== 'calendar' && (h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.applyTemplate(this.templatesList[0], true) }, this.templatesList[0].label)), h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.close", class: "button-primary no-arrow close", onClick: () => this.toggleDialogHandler() }, translate('close')))), h("bci-template", { templatesList: this.templatesList, visible: this.showTemplates && !!this.templatesList, ref: (el) => (this.templatesElement = el) }))));
  }
  static get watchers() { return {
    "datePickerInputObj": ["userInputDate"]
  }; }
  static get style() { return dialogCss; }
}, [1, "bci-datetime-dialog", {
    "dateTimePickerId": [1, "date-time-picker-id"],
    "datePickerInputObj": [16],
    "templatesList": [16],
    "templatesDisabled": [4, "templates-disabled"],
    "hide": [1],
    "datePickerConfig": [16],
    "timePickerConfig": [16],
    "formatService": [16],
    "dateService": [16],
    "toggleDialogHandler": [16],
    "showTemplates": [32],
    "yearMonthSelectionStep": [32],
    "yearSelectionPageOffset": [32],
    "dateObj": [32],
    "datesObj": [32],
    "close": [64],
    "closeYearMonthSelection": [64]
  }, [[0, "templateUpdate", "onTemplateUpdate"], [0, "updatedTime", "timeSelectedHandler"], [0, "showTimeRanges", "showTimeRange"], [0, "toggleShowYearMonthSelection", "toggleShowYearMonthSelectionHandler"], [0, "arrowClicked", "arrowClickedHandler"], [0, "yearMonthSelected", "yearMonthSelectedHandler"], [0, "singleDateSelected", "singleDateSelectedHandler"], [0, "rangeDateSelected", "rangeDateSelectedHandler"]]]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bci-datetime-dialog", "bci-datepicker", "bci-month-header", "bci-template", "bci-timepicker", "bci-week-header", "bci-weekdays", "bci-year-month"];
  components.forEach(tagName => { switch (tagName) {
    case "bci-datetime-dialog":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, DateTimeDialog);
      }
      break;
    case "bci-datepicker":
      if (!customElements.get(tagName)) {
        defineCustomElement$7();
      }
      break;
    case "bci-month-header":
      if (!customElements.get(tagName)) {
        defineCustomElement$6();
      }
      break;
    case "bci-template":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bci-timepicker":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bci-week-header":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bci-weekdays":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
    case "bci-year-month":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { DateTimeDialog as D, dateServiceFactory as a, defineCustomElement as d, formattingServiceFactory as f };

