/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c16876fc.js');
const utils = require('./utils-eb97218e.js');
const _commonjsHelpers = require('./_commonjsHelpers-bcc1208a.js');
const overlayHandler = require('./overlay-handler-7c1726c0.js');

/**
 * @name max
 * @category Common Helpers
 * @summary Return the latest of the given dates.
 *
 * @description
 * Return the latest of the given dates.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - `max` function now accepts an array of dates rather than spread arguments.
 *
 *   ```javascript
 *   // Before v2.0.0
 *   var date1 = new Date(1989, 6, 10)
 *   var date2 = new Date(1987, 1, 11)
 *   var maxDate = max(date1, date2)
 *
 *   // v2.0.0 onward:
 *   var dates = [new Date(1989, 6, 10), new Date(1987, 1, 11)]
 *   var maxDate = max(dates)
 *   ```
 *
 * @param {Date[]|Number[]} datesArray - the dates to compare
 * @returns {Date} the latest of the dates
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which of these dates is the latest?
 * var result = max([
 *   new Date(1989, 6, 10),
 *   new Date(1987, 1, 11),
 *   new Date(1995, 6, 2),
 *   new Date(1990, 0, 1)
 * ])
 * //=> Sun Jul 02 1995 00:00:00
 */

function max(dirtyDatesArray) {
  utils.requiredArgs(1, arguments);
  var datesArray; // `dirtyDatesArray` is Array, Set or Map, or object with custom `forEach` method

  if (dirtyDatesArray && typeof dirtyDatesArray.forEach === 'function') {
    datesArray = dirtyDatesArray; // If `dirtyDatesArray` is Array-like Object, convert to Array.
  } else if (typeof dirtyDatesArray === 'object' && dirtyDatesArray !== null) {
    datesArray = Array.prototype.slice.call(dirtyDatesArray);
  } else {
    // `dirtyDatesArray` is non-iterable, return Invalid Date
    return new Date(NaN);
  }

  var result;
  datesArray.forEach(function (dirtyDate) {
    var currentDate = utils.toDate(dirtyDate);

    if (result === undefined || result < currentDate || isNaN(Number(currentDate))) {
      result = currentDate;
    }
  });
  return result || new Date(NaN);
}

/**
 * @name min
 * @category Common Helpers
 * @summary Returns the earliest of the given dates.
 *
 * @description
 * Returns the earliest of the given dates.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - `min` function now accepts an array of dates rather than spread arguments.
 *
 *   ```javascript
 *   // Before v2.0.0
 *   const date1 = new Date(1989, 6, 10)
 *   const date2 = new Date(1987, 1, 11)
 *   const minDate = min(date1, date2)
 *
 *   // v2.0.0 onward:
 *   const dates = [new Date(1989, 6, 10), new Date(1987, 1, 11)]
 *   const minDate = min(dates)
 *   ```
 *
 * @param {Date[]|Number[]} datesArray - the dates to compare
 * @returns {Date} - the earliest of the dates
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which of these dates is the earliest?
 * const result = min([
 *   new Date(1989, 6, 10),
 *   new Date(1987, 1, 11),
 *   new Date(1995, 6, 2),
 *   new Date(1990, 0, 1)
 * ])
 * //=> Wed Feb 11 1987 00:00:00
 */

function min(dirtyDatesArray) {
  utils.requiredArgs(1, arguments);
  var datesArray; // `dirtyDatesArray` is Array, Set or Map, or object with custom `forEach` method

  if (dirtyDatesArray && typeof dirtyDatesArray.forEach === 'function') {
    datesArray = dirtyDatesArray; // If `dirtyDatesArray` is Array-like Object, convert to Array.
  } else if (typeof dirtyDatesArray === 'object' && dirtyDatesArray !== null) {
    datesArray = Array.prototype.slice.call(dirtyDatesArray);
  } else {
    // `dirtyDatesArray` is non-iterable, return Invalid Date
    return new Date(NaN);
  }

  var result;
  datesArray.forEach(function (dirtyDate) {
    var currentDate = utils.toDate(dirtyDate);

    if (result === undefined || result > currentDate || isNaN(currentDate.getDate())) {
      result = currentDate;
    }
  });
  return result || new Date(NaN);
}

/**
 * @name clamp
 * @category Interval Helpers
 * @summary Return a date bounded by the start and the end of the given interval
 *
 * @description
 * Clamps a date to the lower bound with the start of the interval and the upper
 * bound with the end of the interval.
 *
 * - When the date is less than the start of the interval, the start is returned.
 * - When the date is greater than the end of the interval, the end is returned.
 * - Otherwise the date is returned.
 *
 * @example
 * // What is Mar, 21, 2021 bounded to an interval starting at Mar, 22, 2021 and ending at Apr, 01, 2021
 * const result = clamp(new Date(2021, 2, 21), {
 *   start: new Date(2021, 2, 22),
 *   end: new Date(2021, 3, 1),
 * })
 * //=> Mon Mar 22 2021 00:00:00
 *
 * @param {Date | Number} date - the date to be bounded
 * @param {Interval} interval - the interval to bound to
 * @returns {Date} the date bounded by the start and the end of the interval
 * @throws {TypeError} 2 arguments required
 */

function clamp(date, _ref) {
  var start = _ref.start,
      end = _ref.end;
  utils.requiredArgs(2, arguments);
  return min([max([date, start]), end]);
}

/**
 * @name endOfDay
 * @category Day Helpers
 * @summary Return the end of a day for the given date.
 *
 * @description
 * Return the end of a day for the given date.
 * The result will be in the local timezone.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the original date
 * @returns {Date} the end of a day
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // The end of a day for 2 September 2014 11:55:00:
 * const result = endOfDay(new Date(2014, 8, 2, 11, 55, 0))
 * //=> Tue Sep 02 2014 23:59:59.999
 */

function endOfDay(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  date.setHours(23, 59, 59, 999);
  return date;
}

/**
 * @name endOfMonth
 * @category Month Helpers
 * @summary Return the end of a month for the given date.
 *
 * @description
 * Return the end of a month for the given date.
 * The result will be in the local timezone.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the original date
 * @returns {Date} the end of a month
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // The end of a month for 2 September 2014 11:55:00:
 * const result = endOfMonth(new Date(2014, 8, 2, 11, 55, 0))
 * //=> Tue Sep 30 2014 23:59:59.999
 */

function endOfMonth(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  var month = date.getMonth();
  date.setFullYear(date.getFullYear(), month + 1, 0);
  date.setHours(23, 59, 59, 999);
  return date;
}

function assign(target, dirtyObject) {
  if (target == null) {
    throw new TypeError('assign requires that input parameter not be null or undefined');
  }

  dirtyObject = dirtyObject || {};

  for (var property in dirtyObject) {
    if (Object.prototype.hasOwnProperty.call(dirtyObject, property)) {
      target[property] = dirtyObject[property];
    }
  }

  return target;
}

/**
 * @name fromUnixTime
 * @category Timestamp Helpers
 * @summary Create a date from a Unix timestamp.
 *
 * @description
 * Create a date from a Unix timestamp (in seconds). Decimal values will be discarded.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Number} unixTime - the given Unix timestamp (in seconds)
 * @returns {Date} the date
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Create the date 29 February 2012 11:45:05:
 * const result = fromUnixTime(1330515905)
 * //=> Wed Feb 29 2012 11:45:05
 */

function fromUnixTime(dirtyUnixTime) {
  utils.requiredArgs(1, arguments);
  var unixTime = utils.toInteger(dirtyUnixTime);
  return utils.toDate(unixTime * 1000);
}

/**
 * @name getDate
 * @category Day Helpers
 * @summary Get the day of the month of the given date.
 *
 * @description
 * Get the day of the month of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the day of month
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which day of the month is 29 February 2012?
 * const result = getDate(new Date(2012, 1, 29))
 * //=> 29
 */

function getDate(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  var dayOfMonth = date.getDate();
  return dayOfMonth;
}

/**
 * @name getISODay
 * @category Weekday Helpers
 * @summary Get the day of the ISO week of the given date.
 *
 * @description
 * Get the day of the ISO week of the given date,
 * which is 7 for Sunday, 1 for Monday etc.
 *
 * ISO week-numbering year: http://en.wikipedia.org/wiki/ISO_week_date
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the day of ISO week
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which day of the ISO week is 26 February 2012?
 * const result = getISODay(new Date(2012, 1, 26))
 * //=> 7
 */

function getISODay(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  var day = date.getDay();

  if (day === 0) {
    day = 7;
  }

  return day;
}

/**
 * @name getMonth
 * @category Month Helpers
 * @summary Get the month of the given date.
 *
 * @description
 * Get the month of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the month
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which month is 29 February 2012?
 * const result = getMonth(new Date(2012, 1, 29))
 * //=> 1
 */

function getMonth(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  var month = date.getMonth();
  return month;
}

/**
 * @name getTime
 * @category Timestamp Helpers
 * @summary Get the milliseconds timestamp of the given date.
 *
 * @description
 * Get the milliseconds timestamp of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the timestamp
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Get the timestamp of 29 February 2012 11:45:05.123:
 * const result = getTime(new Date(2012, 1, 29, 11, 45, 5, 123))
 * //=> 1330515905123
 */

function getTime(dirtyDate) {
  utils.requiredArgs(1, arguments);
  var date = utils.toDate(dirtyDate);
  var timestamp = date.getTime();
  return timestamp;
}

/**
 * @name getUnixTime
 * @category Timestamp Helpers
 * @summary Get the seconds timestamp of the given date.
 *
 * @description
 * Get the seconds timestamp of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the timestamp
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Get the timestamp of 29 February 2012 11:45:05 CET:
 * const result = getUnixTime(new Date(2012, 1, 29, 11, 45, 5))
 * //=> 1330512305
 */

function getUnixTime(dirtyDate) {
  utils.requiredArgs(1, arguments);
  return Math.floor(getTime(dirtyDate) / 1000);
}

/**
 * @name getYear
 * @category Year Helpers
 * @summary Get the year of the given date.
 *
 * @description
 * Get the year of the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the given date
 * @returns {Number} the year
 * @throws {TypeError} 1 argument required
 *
 * @example
 * // Which year is 2 July 2014?
 * const result = getYear(new Date(2014, 6, 2))
 * //=> 2014
 */

function getYear(dirtyDate) {
  utils.requiredArgs(1, arguments);
  return utils.toDate(dirtyDate).getFullYear();
}

/**
 * @name isAfter
 * @category Common Helpers
 * @summary Is the first date after the second one?
 *
 * @description
 * Is the first date after the second one?
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the date that should be after the other one to return true
 * @param {Date|Number} dateToCompare - the date to compare with
 * @returns {Boolean} the first date is after the second date
 * @throws {TypeError} 2 arguments required
 *
 * @example
 * // Is 10 July 1989 after 11 February 1987?
 * var result = isAfter(new Date(1989, 6, 10), new Date(1987, 1, 11))
 * //=> true
 */

function isAfter(dirtyDate, dirtyDateToCompare) {
  utils.requiredArgs(2, arguments);
  var date = utils.toDate(dirtyDate);
  var dateToCompare = utils.toDate(dirtyDateToCompare);
  return date.getTime() > dateToCompare.getTime();
}

/**
 * @name isBefore
 * @category Common Helpers
 * @summary Is the first date before the second one?
 *
 * @description
 * Is the first date before the second one?
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the date that should be before the other one to return true
 * @param {Date|Number} dateToCompare - the date to compare with
 * @returns {Boolean} the first date is before the second date
 * @throws {TypeError} 2 arguments required
 *
 * @example
 * // Is 10 July 1989 before 11 February 1987?
 * var result = isBefore(new Date(1989, 6, 10), new Date(1987, 1, 11))
 * //=> false
 */

function isBefore(dirtyDate, dirtyDateToCompare) {
  utils.requiredArgs(2, arguments);
  var date = utils.toDate(dirtyDate);
  var dateToCompare = utils.toDate(dirtyDateToCompare);
  return date.getTime() < dateToCompare.getTime();
}

// See issue: https://github.com/date-fns/date-fns/issues/376

function setUTCDay(dirtyDate, dirtyDay, dirtyOptions) {
  utils.requiredArgs(2, arguments);
  var options = dirtyOptions || {};
  var locale = options.locale;
  var localeWeekStartsOn = locale && locale.options && locale.options.weekStartsOn;
  var defaultWeekStartsOn = localeWeekStartsOn == null ? 0 : utils.toInteger(localeWeekStartsOn);
  var weekStartsOn = options.weekStartsOn == null ? defaultWeekStartsOn : utils.toInteger(options.weekStartsOn); // Test if weekStartsOn is between 0 and 6 _and_ is not NaN

  if (!(weekStartsOn >= 0 && weekStartsOn <= 6)) {
    throw new RangeError('weekStartsOn must be between 0 and 6 inclusively');
  }

  var date = utils.toDate(dirtyDate);
  var day = utils.toInteger(dirtyDay);
  var currentDay = date.getUTCDay();
  var remainder = day % 7;
  var dayIndex = (remainder + 7) % 7;
  var diff = (dayIndex < weekStartsOn ? 7 : 0) + day - currentDay;
  date.setUTCDate(date.getUTCDate() + diff);
  return date;
}

// See issue: https://github.com/date-fns/date-fns/issues/376

function setUTCISODay(dirtyDate, dirtyDay) {
  utils.requiredArgs(2, arguments);
  var day = utils.toInteger(dirtyDay);

  if (day % 7 === 0) {
    day = day - 7;
  }

  var weekStartsOn = 1;
  var date = utils.toDate(dirtyDate);
  var currentDay = date.getUTCDay();
  var remainder = day % 7;
  var dayIndex = (remainder + 7) % 7;
  var diff = (dayIndex < weekStartsOn ? 7 : 0) + day - currentDay;
  date.setUTCDate(date.getUTCDate() + diff);
  return date;
}

// See issue: https://github.com/date-fns/date-fns/issues/376

function setUTCISOWeek(dirtyDate, dirtyISOWeek) {
  utils.requiredArgs(2, arguments);
  var date = utils.toDate(dirtyDate);
  var isoWeek = utils.toInteger(dirtyISOWeek);
  var diff = utils.getUTCISOWeek(date) - isoWeek;
  date.setUTCDate(date.getUTCDate() - diff * 7);
  return date;
}

// See issue: https://github.com/date-fns/date-fns/issues/376

function setUTCWeek(dirtyDate, dirtyWeek, options) {
  utils.requiredArgs(2, arguments);
  var date = utils.toDate(dirtyDate);
  var week = utils.toInteger(dirtyWeek);
  var diff = utils.getUTCWeek(date, options) - week;
  date.setUTCDate(date.getUTCDate() - diff * 7);
  return date;
}

var MILLISECONDS_IN_HOUR$2 = 3600000;
var MILLISECONDS_IN_MINUTE$2 = 60000;
var MILLISECONDS_IN_SECOND = 1000;
var numericPatterns = {
  month: /^(1[0-2]|0?\d)/,
  // 0 to 12
  date: /^(3[0-1]|[0-2]?\d)/,
  // 0 to 31
  dayOfYear: /^(36[0-6]|3[0-5]\d|[0-2]?\d?\d)/,
  // 0 to 366
  week: /^(5[0-3]|[0-4]?\d)/,
  // 0 to 53
  hour23h: /^(2[0-3]|[0-1]?\d)/,
  // 0 to 23
  hour24h: /^(2[0-4]|[0-1]?\d)/,
  // 0 to 24
  hour11h: /^(1[0-1]|0?\d)/,
  // 0 to 11
  hour12h: /^(1[0-2]|0?\d)/,
  // 0 to 12
  minute: /^[0-5]?\d/,
  // 0 to 59
  second: /^[0-5]?\d/,
  // 0 to 59
  singleDigit: /^\d/,
  // 0 to 9
  twoDigits: /^\d{1,2}/,
  // 0 to 99
  threeDigits: /^\d{1,3}/,
  // 0 to 999
  fourDigits: /^\d{1,4}/,
  // 0 to 9999
  anyDigitsSigned: /^-?\d+/,
  singleDigitSigned: /^-?\d/,
  // 0 to 9, -0 to -9
  twoDigitsSigned: /^-?\d{1,2}/,
  // 0 to 99, -0 to -99
  threeDigitsSigned: /^-?\d{1,3}/,
  // 0 to 999, -0 to -999
  fourDigitsSigned: /^-?\d{1,4}/ // 0 to 9999, -0 to -9999

};
var timezonePatterns = {
  basicOptionalMinutes: /^([+-])(\d{2})(\d{2})?|Z/,
  basic: /^([+-])(\d{2})(\d{2})|Z/,
  basicOptionalSeconds: /^([+-])(\d{2})(\d{2})((\d{2}))?|Z/,
  extended: /^([+-])(\d{2}):(\d{2})|Z/,
  extendedOptionalSeconds: /^([+-])(\d{2}):(\d{2})(:(\d{2}))?|Z/
};

function parseNumericPattern(pattern, string, valueCallback) {
  var matchResult = string.match(pattern);

  if (!matchResult) {
    return null;
  }

  var value = parseInt(matchResult[0], 10);
  return {
    value: valueCallback ? valueCallback(value) : value,
    rest: string.slice(matchResult[0].length)
  };
}

function parseTimezonePattern(pattern, string) {
  var matchResult = string.match(pattern);

  if (!matchResult) {
    return null;
  } // Input is 'Z'


  if (matchResult[0] === 'Z') {
    return {
      value: 0,
      rest: string.slice(1)
    };
  }

  var sign = matchResult[1] === '+' ? 1 : -1;
  var hours = matchResult[2] ? parseInt(matchResult[2], 10) : 0;
  var minutes = matchResult[3] ? parseInt(matchResult[3], 10) : 0;
  var seconds = matchResult[5] ? parseInt(matchResult[5], 10) : 0;
  return {
    value: sign * (hours * MILLISECONDS_IN_HOUR$2 + minutes * MILLISECONDS_IN_MINUTE$2 + seconds * MILLISECONDS_IN_SECOND),
    rest: string.slice(matchResult[0].length)
  };
}

function parseAnyDigitsSigned(string, valueCallback) {
  return parseNumericPattern(numericPatterns.anyDigitsSigned, string, valueCallback);
}

function parseNDigits(n, string, valueCallback) {
  switch (n) {
    case 1:
      return parseNumericPattern(numericPatterns.singleDigit, string, valueCallback);

    case 2:
      return parseNumericPattern(numericPatterns.twoDigits, string, valueCallback);

    case 3:
      return parseNumericPattern(numericPatterns.threeDigits, string, valueCallback);

    case 4:
      return parseNumericPattern(numericPatterns.fourDigits, string, valueCallback);

    default:
      return parseNumericPattern(new RegExp('^\\d{1,' + n + '}'), string, valueCallback);
  }
}

function parseNDigitsSigned(n, string, valueCallback) {
  switch (n) {
    case 1:
      return parseNumericPattern(numericPatterns.singleDigitSigned, string, valueCallback);

    case 2:
      return parseNumericPattern(numericPatterns.twoDigitsSigned, string, valueCallback);

    case 3:
      return parseNumericPattern(numericPatterns.threeDigitsSigned, string, valueCallback);

    case 4:
      return parseNumericPattern(numericPatterns.fourDigitsSigned, string, valueCallback);

    default:
      return parseNumericPattern(new RegExp('^-?\\d{1,' + n + '}'), string, valueCallback);
  }
}

function dayPeriodEnumToHours(enumValue) {
  switch (enumValue) {
    case 'morning':
      return 4;

    case 'evening':
      return 17;

    case 'pm':
    case 'noon':
    case 'afternoon':
      return 12;

    case 'am':
    case 'midnight':
    case 'night':
    default:
      return 0;
  }
}

function normalizeTwoDigitYear(twoDigitYear, currentYear) {
  var isCommonEra = currentYear > 0; // Absolute number of the current year:
  // 1 -> 1 AC
  // 0 -> 1 BC
  // -1 -> 2 BC

  var absCurrentYear = isCommonEra ? currentYear : 1 - currentYear;
  var result;

  if (absCurrentYear <= 50) {
    result = twoDigitYear || 100;
  } else {
    var rangeEnd = absCurrentYear + 50;
    var rangeEndCentury = Math.floor(rangeEnd / 100) * 100;
    var isPreviousCentury = twoDigitYear >= rangeEnd % 100;
    result = twoDigitYear + rangeEndCentury - (isPreviousCentury ? 100 : 0);
  }

  return isCommonEra ? result : 1 - result;
}

var DAYS_IN_MONTH$1 = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
var DAYS_IN_MONTH_LEAP_YEAR$1 = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]; // User for validation

function isLeapYearIndex$1(year) {
  return year % 400 === 0 || year % 4 === 0 && year % 100 !== 0;
}
/*
 * |     | Unit                           |     | Unit                           |
 * |-----|--------------------------------|-----|--------------------------------|
 * |  a  | AM, PM                         |  A* | Milliseconds in day            |
 * |  b  | AM, PM, noon, midnight         |  B  | Flexible day period            |
 * |  c  | Stand-alone local day of week  |  C* | Localized hour w/ day period   |
 * |  d  | Day of month                   |  D  | Day of year                    |
 * |  e  | Local day of week              |  E  | Day of week                    |
 * |  f  |                                |  F* | Day of week in month           |
 * |  g* | Modified Julian day            |  G  | Era                            |
 * |  h  | Hour [1-12]                    |  H  | Hour [0-23]                    |
 * |  i! | ISO day of week                |  I! | ISO week of year               |
 * |  j* | Localized hour w/ day period   |  J* | Localized hour w/o day period  |
 * |  k  | Hour [1-24]                    |  K  | Hour [0-11]                    |
 * |  l* | (deprecated)                   |  L  | Stand-alone month              |
 * |  m  | Minute                         |  M  | Month                          |
 * |  n  |                                |  N  |                                |
 * |  o! | Ordinal number modifier        |  O* | Timezone (GMT)                 |
 * |  p  |                                |  P  |                                |
 * |  q  | Stand-alone quarter            |  Q  | Quarter                        |
 * |  r* | Related Gregorian year         |  R! | ISO week-numbering year        |
 * |  s  | Second                         |  S  | Fraction of second             |
 * |  t! | Seconds timestamp              |  T! | Milliseconds timestamp         |
 * |  u  | Extended year                  |  U* | Cyclic year                    |
 * |  v* | Timezone (generic non-locat.)  |  V* | Timezone (location)            |
 * |  w  | Local week of year             |  W* | Week of month                  |
 * |  x  | Timezone (ISO-8601 w/o Z)      |  X  | Timezone (ISO-8601)            |
 * |  y  | Year (abs)                     |  Y  | Local week-numbering year      |
 * |  z* | Timezone (specific non-locat.) |  Z* | Timezone (aliases)             |
 *
 * Letters marked by * are not implemented but reserved by Unicode standard.
 *
 * Letters marked by ! are non-standard, but implemented by date-fns:
 * - `o` modifies the previous token to turn it into an ordinal (see `parse` docs)
 * - `i` is ISO day of week. For `i` and `ii` is returns numeric ISO week days,
 *   i.e. 7 for Sunday, 1 for Monday, etc.
 * - `I` is ISO week of year, as opposed to `w` which is local week of year.
 * - `R` is ISO week-numbering year, as opposed to `Y` which is local week-numbering year.
 *   `R` is supposed to be used in conjunction with `I` and `i`
 *   for universal ISO week-numbering date, whereas
 *   `Y` is supposed to be used in conjunction with `w` and `e`
 *   for week-numbering date specific to the locale.
 */


var parsers = {
  // Era
  G: {
    priority: 140,
    parse: function (string, token, match, _options) {
      switch (token) {
        // AD, BC
        case 'G':
        case 'GG':
        case 'GGG':
          return match.era(string, {
            width: 'abbreviated'
          }) || match.era(string, {
            width: 'narrow'
          });
        // A, B

        case 'GGGGG':
          return match.era(string, {
            width: 'narrow'
          });
        // Anno Domini, Before Christ

        case 'GGGG':
        default:
          return match.era(string, {
            width: 'wide'
          }) || match.era(string, {
            width: 'abbreviated'
          }) || match.era(string, {
            width: 'narrow'
          });
      }
    },
    set: function (date, flags, value, _options) {
      flags.era = value;
      date.setUTCFullYear(value, 0, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['R', 'u', 't', 'T']
  },
  // Year
  y: {
    // From http://www.unicode.org/reports/tr35/tr35-31/tr35-dates.html#Date_Format_Patterns
    // | Year     |     y | yy |   yyy |  yyyy | yyyyy |
    // |----------|-------|----|-------|-------|-------|
    // | AD 1     |     1 | 01 |   001 |  0001 | 00001 |
    // | AD 12    |    12 | 12 |   012 |  0012 | 00012 |
    // | AD 123   |   123 | 23 |   123 |  0123 | 00123 |
    // | AD 1234  |  1234 | 34 |  1234 |  1234 | 01234 |
    // | AD 12345 | 12345 | 45 | 12345 | 12345 | 12345 |
    priority: 130,
    parse: function (string, token, match, _options) {
      var valueCallback = function (year) {
        return {
          year: year,
          isTwoDigitYear: token === 'yy'
        };
      };

      switch (token) {
        case 'y':
          return parseNDigits(4, string, valueCallback);

        case 'yo':
          return match.ordinalNumber(string, {
            unit: 'year',
            valueCallback: valueCallback
          });

        default:
          return parseNDigits(token.length, string, valueCallback);
      }
    },
    validate: function (_date, value, _options) {
      return value.isTwoDigitYear || value.year > 0;
    },
    set: function (date, flags, value, _options) {
      var currentYear = date.getUTCFullYear();

      if (value.isTwoDigitYear) {
        var normalizedTwoDigitYear = normalizeTwoDigitYear(value.year, currentYear);
        date.setUTCFullYear(normalizedTwoDigitYear, 0, 1);
        date.setUTCHours(0, 0, 0, 0);
        return date;
      }

      var year = !('era' in flags) || flags.era === 1 ? value.year : 1 - value.year;
      date.setUTCFullYear(year, 0, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'u', 'w', 'I', 'i', 'e', 'c', 't', 'T']
  },
  // Local week-numbering year
  Y: {
    priority: 130,
    parse: function (string, token, match, _options) {
      var valueCallback = function (year) {
        return {
          year: year,
          isTwoDigitYear: token === 'YY'
        };
      };

      switch (token) {
        case 'Y':
          return parseNDigits(4, string, valueCallback);

        case 'Yo':
          return match.ordinalNumber(string, {
            unit: 'year',
            valueCallback: valueCallback
          });

        default:
          return parseNDigits(token.length, string, valueCallback);
      }
    },
    validate: function (_date, value, _options) {
      return value.isTwoDigitYear || value.year > 0;
    },
    set: function (date, flags, value, options) {
      var currentYear = utils.getUTCWeekYear(date, options);

      if (value.isTwoDigitYear) {
        var normalizedTwoDigitYear = normalizeTwoDigitYear(value.year, currentYear);
        date.setUTCFullYear(normalizedTwoDigitYear, 0, options.firstWeekContainsDate);
        date.setUTCHours(0, 0, 0, 0);
        return utils.startOfUTCWeek(date, options);
      }

      var year = !('era' in flags) || flags.era === 1 ? value.year : 1 - value.year;
      date.setUTCFullYear(year, 0, options.firstWeekContainsDate);
      date.setUTCHours(0, 0, 0, 0);
      return utils.startOfUTCWeek(date, options);
    },
    incompatibleTokens: ['y', 'R', 'u', 'Q', 'q', 'M', 'L', 'I', 'd', 'D', 'i', 't', 'T']
  },
  // ISO week-numbering year
  R: {
    priority: 130,
    parse: function (string, token, _match, _options) {
      if (token === 'R') {
        return parseNDigitsSigned(4, string);
      }

      return parseNDigitsSigned(token.length, string);
    },
    set: function (_date, _flags, value, _options) {
      var firstWeekOfYear = new Date(0);
      firstWeekOfYear.setUTCFullYear(value, 0, 4);
      firstWeekOfYear.setUTCHours(0, 0, 0, 0);
      return utils.startOfUTCISOWeek(firstWeekOfYear);
    },
    incompatibleTokens: ['G', 'y', 'Y', 'u', 'Q', 'q', 'M', 'L', 'w', 'd', 'D', 'e', 'c', 't', 'T']
  },
  // Extended year
  u: {
    priority: 130,
    parse: function (string, token, _match, _options) {
      if (token === 'u') {
        return parseNDigitsSigned(4, string);
      }

      return parseNDigitsSigned(token.length, string);
    },
    set: function (date, _flags, value, _options) {
      date.setUTCFullYear(value, 0, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['G', 'y', 'Y', 'R', 'w', 'I', 'i', 'e', 'c', 't', 'T']
  },
  // Quarter
  Q: {
    priority: 120,
    parse: function (string, token, match, _options) {
      switch (token) {
        // 1, 2, 3, 4
        case 'Q':
        case 'QQ':
          // 01, 02, 03, 04
          return parseNDigits(token.length, string);
        // 1st, 2nd, 3rd, 4th

        case 'Qo':
          return match.ordinalNumber(string, {
            unit: 'quarter'
          });
        // Q1, Q2, Q3, Q4

        case 'QQQ':
          return match.quarter(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.quarter(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // 1, 2, 3, 4 (narrow quarter; could be not numerical)

        case 'QQQQQ':
          return match.quarter(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // 1st quarter, 2nd quarter, ...

        case 'QQQQ':
        default:
          return match.quarter(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.quarter(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.quarter(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 4;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMonth((value - 1) * 3, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'q', 'M', 'L', 'w', 'I', 'd', 'D', 'i', 'e', 'c', 't', 'T']
  },
  // Stand-alone quarter
  q: {
    priority: 120,
    parse: function (string, token, match, _options) {
      switch (token) {
        // 1, 2, 3, 4
        case 'q':
        case 'qq':
          // 01, 02, 03, 04
          return parseNDigits(token.length, string);
        // 1st, 2nd, 3rd, 4th

        case 'qo':
          return match.ordinalNumber(string, {
            unit: 'quarter'
          });
        // Q1, Q2, Q3, Q4

        case 'qqq':
          return match.quarter(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.quarter(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // 1, 2, 3, 4 (narrow quarter; could be not numerical)

        case 'qqqqq':
          return match.quarter(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // 1st quarter, 2nd quarter, ...

        case 'qqqq':
        default:
          return match.quarter(string, {
            width: 'wide',
            context: 'standalone'
          }) || match.quarter(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.quarter(string, {
            width: 'narrow',
            context: 'standalone'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 4;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMonth((value - 1) * 3, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'Q', 'M', 'L', 'w', 'I', 'd', 'D', 'i', 'e', 'c', 't', 'T']
  },
  // Month
  M: {
    priority: 110,
    parse: function (string, token, match, _options) {
      var valueCallback = function (value) {
        return value - 1;
      };

      switch (token) {
        // 1, 2, ..., 12
        case 'M':
          return parseNumericPattern(numericPatterns.month, string, valueCallback);
        // 01, 02, ..., 12

        case 'MM':
          return parseNDigits(2, string, valueCallback);
        // 1st, 2nd, ..., 12th

        case 'Mo':
          return match.ordinalNumber(string, {
            unit: 'month',
            valueCallback: valueCallback
          });
        // Jan, Feb, ..., Dec

        case 'MMM':
          return match.month(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.month(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // J, F, ..., D

        case 'MMMMM':
          return match.month(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // January, February, ..., December

        case 'MMMM':
        default:
          return match.month(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.month(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.month(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 11;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMonth(value, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'q', 'Q', 'L', 'w', 'I', 'D', 'i', 'e', 'c', 't', 'T']
  },
  // Stand-alone month
  L: {
    priority: 110,
    parse: function (string, token, match, _options) {
      var valueCallback = function (value) {
        return value - 1;
      };

      switch (token) {
        // 1, 2, ..., 12
        case 'L':
          return parseNumericPattern(numericPatterns.month, string, valueCallback);
        // 01, 02, ..., 12

        case 'LL':
          return parseNDigits(2, string, valueCallback);
        // 1st, 2nd, ..., 12th

        case 'Lo':
          return match.ordinalNumber(string, {
            unit: 'month',
            valueCallback: valueCallback
          });
        // Jan, Feb, ..., Dec

        case 'LLL':
          return match.month(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.month(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // J, F, ..., D

        case 'LLLLL':
          return match.month(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // January, February, ..., December

        case 'LLLL':
        default:
          return match.month(string, {
            width: 'wide',
            context: 'standalone'
          }) || match.month(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.month(string, {
            width: 'narrow',
            context: 'standalone'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 11;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMonth(value, 1);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'q', 'Q', 'M', 'w', 'I', 'D', 'i', 'e', 'c', 't', 'T']
  },
  // Local week of year
  w: {
    priority: 100,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'w':
          return parseNumericPattern(numericPatterns.week, string);

        case 'wo':
          return match.ordinalNumber(string, {
            unit: 'week'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 53;
    },
    set: function (date, _flags, value, options) {
      return utils.startOfUTCWeek(setUTCWeek(date, value, options), options);
    },
    incompatibleTokens: ['y', 'R', 'u', 'q', 'Q', 'M', 'L', 'I', 'd', 'D', 'i', 't', 'T']
  },
  // ISO week of year
  I: {
    priority: 100,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'I':
          return parseNumericPattern(numericPatterns.week, string);

        case 'Io':
          return match.ordinalNumber(string, {
            unit: 'week'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 53;
    },
    set: function (date, _flags, value, options) {
      return utils.startOfUTCISOWeek(setUTCISOWeek(date, value, options), options);
    },
    incompatibleTokens: ['y', 'Y', 'u', 'q', 'Q', 'M', 'L', 'w', 'd', 'D', 'e', 'c', 't', 'T']
  },
  // Day of the month
  d: {
    priority: 90,
    subPriority: 1,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'd':
          return parseNumericPattern(numericPatterns.date, string);

        case 'do':
          return match.ordinalNumber(string, {
            unit: 'date'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (date, value, _options) {
      var year = date.getUTCFullYear();
      var isLeapYear = isLeapYearIndex$1(year);
      var month = date.getUTCMonth();

      if (isLeapYear) {
        return value >= 1 && value <= DAYS_IN_MONTH_LEAP_YEAR$1[month];
      } else {
        return value >= 1 && value <= DAYS_IN_MONTH$1[month];
      }
    },
    set: function (date, _flags, value, _options) {
      date.setUTCDate(value);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'q', 'Q', 'w', 'I', 'D', 'i', 'e', 'c', 't', 'T']
  },
  // Day of year
  D: {
    priority: 90,
    subPriority: 1,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'D':
        case 'DD':
          return parseNumericPattern(numericPatterns.dayOfYear, string);

        case 'Do':
          return match.ordinalNumber(string, {
            unit: 'date'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (date, value, _options) {
      var year = date.getUTCFullYear();
      var isLeapYear = isLeapYearIndex$1(year);

      if (isLeapYear) {
        return value >= 1 && value <= 366;
      } else {
        return value >= 1 && value <= 365;
      }
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMonth(0, value);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['Y', 'R', 'q', 'Q', 'M', 'L', 'w', 'I', 'd', 'E', 'i', 'e', 'c', 't', 'T']
  },
  // Day of week
  E: {
    priority: 90,
    parse: function (string, token, match, _options) {
      switch (token) {
        // Tue
        case 'E':
        case 'EE':
        case 'EEE':
          return match.day(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // T

        case 'EEEEE':
          return match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // Tu

        case 'EEEEEE':
          return match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // Tuesday

        case 'EEEE':
        default:
          return match.day(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.day(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 6;
    },
    set: function (date, _flags, value, options) {
      date = setUTCDay(date, value, options);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['D', 'i', 'e', 'c', 't', 'T']
  },
  // Local day of week
  e: {
    priority: 90,
    parse: function (string, token, match, options) {
      var valueCallback = function (value) {
        var wholeWeekDays = Math.floor((value - 1) / 7) * 7;
        return (value + options.weekStartsOn + 6) % 7 + wholeWeekDays;
      };

      switch (token) {
        // 3
        case 'e':
        case 'ee':
          // 03
          return parseNDigits(token.length, string, valueCallback);
        // 3rd

        case 'eo':
          return match.ordinalNumber(string, {
            unit: 'day',
            valueCallback: valueCallback
          });
        // Tue

        case 'eee':
          return match.day(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // T

        case 'eeeee':
          return match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // Tu

        case 'eeeeee':
          return match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
        // Tuesday

        case 'eeee':
        default:
          return match.day(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.day(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.day(string, {
            width: 'short',
            context: 'formatting'
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 6;
    },
    set: function (date, _flags, value, options) {
      date = setUTCDay(date, value, options);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['y', 'R', 'u', 'q', 'Q', 'M', 'L', 'I', 'd', 'D', 'E', 'i', 'c', 't', 'T']
  },
  // Stand-alone local day of week
  c: {
    priority: 90,
    parse: function (string, token, match, options) {
      var valueCallback = function (value) {
        var wholeWeekDays = Math.floor((value - 1) / 7) * 7;
        return (value + options.weekStartsOn + 6) % 7 + wholeWeekDays;
      };

      switch (token) {
        // 3
        case 'c':
        case 'cc':
          // 03
          return parseNDigits(token.length, string, valueCallback);
        // 3rd

        case 'co':
          return match.ordinalNumber(string, {
            unit: 'day',
            valueCallback: valueCallback
          });
        // Tue

        case 'ccc':
          return match.day(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.day(string, {
            width: 'short',
            context: 'standalone'
          }) || match.day(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // T

        case 'ccccc':
          return match.day(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // Tu

        case 'cccccc':
          return match.day(string, {
            width: 'short',
            context: 'standalone'
          }) || match.day(string, {
            width: 'narrow',
            context: 'standalone'
          });
        // Tuesday

        case 'cccc':
        default:
          return match.day(string, {
            width: 'wide',
            context: 'standalone'
          }) || match.day(string, {
            width: 'abbreviated',
            context: 'standalone'
          }) || match.day(string, {
            width: 'short',
            context: 'standalone'
          }) || match.day(string, {
            width: 'narrow',
            context: 'standalone'
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 6;
    },
    set: function (date, _flags, value, options) {
      date = setUTCDay(date, value, options);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['y', 'R', 'u', 'q', 'Q', 'M', 'L', 'I', 'd', 'D', 'E', 'i', 'e', 't', 'T']
  },
  // ISO day of week
  i: {
    priority: 90,
    parse: function (string, token, match, _options) {
      var valueCallback = function (value) {
        if (value === 0) {
          return 7;
        }

        return value;
      };

      switch (token) {
        // 2
        case 'i':
        case 'ii':
          // 02
          return parseNDigits(token.length, string);
        // 2nd

        case 'io':
          return match.ordinalNumber(string, {
            unit: 'day'
          });
        // Tue

        case 'iii':
          return match.day(string, {
            width: 'abbreviated',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'short',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting',
            valueCallback: valueCallback
          });
        // T

        case 'iiiii':
          return match.day(string, {
            width: 'narrow',
            context: 'formatting',
            valueCallback: valueCallback
          });
        // Tu

        case 'iiiiii':
          return match.day(string, {
            width: 'short',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting',
            valueCallback: valueCallback
          });
        // Tuesday

        case 'iiii':
        default:
          return match.day(string, {
            width: 'wide',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'abbreviated',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'short',
            context: 'formatting',
            valueCallback: valueCallback
          }) || match.day(string, {
            width: 'narrow',
            context: 'formatting',
            valueCallback: valueCallback
          });
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 7;
    },
    set: function (date, _flags, value, options) {
      date = setUTCISODay(date, value, options);
      date.setUTCHours(0, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['y', 'Y', 'u', 'q', 'Q', 'M', 'L', 'w', 'd', 'D', 'E', 'e', 'c', 't', 'T']
  },
  // AM or PM
  a: {
    priority: 80,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'a':
        case 'aa':
        case 'aaa':
          return match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'aaaaa':
          return match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'aaaa':
        default:
          return match.dayPeriod(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    set: function (date, _flags, value, _options) {
      date.setUTCHours(dayPeriodEnumToHours(value), 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['b', 'B', 'H', 'k', 't', 'T']
  },
  // AM, PM, midnight
  b: {
    priority: 80,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'b':
        case 'bb':
        case 'bbb':
          return match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'bbbbb':
          return match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'bbbb':
        default:
          return match.dayPeriod(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    set: function (date, _flags, value, _options) {
      date.setUTCHours(dayPeriodEnumToHours(value), 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['a', 'B', 'H', 'k', 't', 'T']
  },
  // in the morning, in the afternoon, in the evening, at night
  B: {
    priority: 80,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'B':
        case 'BB':
        case 'BBB':
          return match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'BBBBB':
          return match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });

        case 'BBBB':
        default:
          return match.dayPeriod(string, {
            width: 'wide',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'abbreviated',
            context: 'formatting'
          }) || match.dayPeriod(string, {
            width: 'narrow',
            context: 'formatting'
          });
      }
    },
    set: function (date, _flags, value, _options) {
      date.setUTCHours(dayPeriodEnumToHours(value), 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['a', 'b', 't', 'T']
  },
  // Hour [1-12]
  h: {
    priority: 70,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'h':
          return parseNumericPattern(numericPatterns.hour12h, string);

        case 'ho':
          return match.ordinalNumber(string, {
            unit: 'hour'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 12;
    },
    set: function (date, _flags, value, _options) {
      var isPM = date.getUTCHours() >= 12;

      if (isPM && value < 12) {
        date.setUTCHours(value + 12, 0, 0, 0);
      } else if (!isPM && value === 12) {
        date.setUTCHours(0, 0, 0, 0);
      } else {
        date.setUTCHours(value, 0, 0, 0);
      }

      return date;
    },
    incompatibleTokens: ['H', 'K', 'k', 't', 'T']
  },
  // Hour [0-23]
  H: {
    priority: 70,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'H':
          return parseNumericPattern(numericPatterns.hour23h, string);

        case 'Ho':
          return match.ordinalNumber(string, {
            unit: 'hour'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 23;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCHours(value, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['a', 'b', 'h', 'K', 'k', 't', 'T']
  },
  // Hour [0-11]
  K: {
    priority: 70,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'K':
          return parseNumericPattern(numericPatterns.hour11h, string);

        case 'Ko':
          return match.ordinalNumber(string, {
            unit: 'hour'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 11;
    },
    set: function (date, _flags, value, _options) {
      var isPM = date.getUTCHours() >= 12;

      if (isPM && value < 12) {
        date.setUTCHours(value + 12, 0, 0, 0);
      } else {
        date.setUTCHours(value, 0, 0, 0);
      }

      return date;
    },
    incompatibleTokens: ['h', 'H', 'k', 't', 'T']
  },
  // Hour [1-24]
  k: {
    priority: 70,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'k':
          return parseNumericPattern(numericPatterns.hour24h, string);

        case 'ko':
          return match.ordinalNumber(string, {
            unit: 'hour'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 1 && value <= 24;
    },
    set: function (date, _flags, value, _options) {
      var hours = value <= 24 ? value % 24 : value;
      date.setUTCHours(hours, 0, 0, 0);
      return date;
    },
    incompatibleTokens: ['a', 'b', 'h', 'H', 'K', 't', 'T']
  },
  // Minute
  m: {
    priority: 60,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 'm':
          return parseNumericPattern(numericPatterns.minute, string);

        case 'mo':
          return match.ordinalNumber(string, {
            unit: 'minute'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 59;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMinutes(value, 0, 0);
      return date;
    },
    incompatibleTokens: ['t', 'T']
  },
  // Second
  s: {
    priority: 50,
    parse: function (string, token, match, _options) {
      switch (token) {
        case 's':
          return parseNumericPattern(numericPatterns.second, string);

        case 'so':
          return match.ordinalNumber(string, {
            unit: 'second'
          });

        default:
          return parseNDigits(token.length, string);
      }
    },
    validate: function (_date, value, _options) {
      return value >= 0 && value <= 59;
    },
    set: function (date, _flags, value, _options) {
      date.setUTCSeconds(value, 0);
      return date;
    },
    incompatibleTokens: ['t', 'T']
  },
  // Fraction of second
  S: {
    priority: 30,
    parse: function (string, token, _match, _options) {
      var valueCallback = function (value) {
        return Math.floor(value * Math.pow(10, -token.length + 3));
      };

      return parseNDigits(token.length, string, valueCallback);
    },
    set: function (date, _flags, value, _options) {
      date.setUTCMilliseconds(value);
      return date;
    },
    incompatibleTokens: ['t', 'T']
  },
  // Timezone (ISO-8601. +00:00 is `'Z'`)
  X: {
    priority: 10,
    parse: function (string, token, _match, _options) {
      switch (token) {
        case 'X':
          return parseTimezonePattern(timezonePatterns.basicOptionalMinutes, string);

        case 'XX':
          return parseTimezonePattern(timezonePatterns.basic, string);

        case 'XXXX':
          return parseTimezonePattern(timezonePatterns.basicOptionalSeconds, string);

        case 'XXXXX':
          return parseTimezonePattern(timezonePatterns.extendedOptionalSeconds, string);

        case 'XXX':
        default:
          return parseTimezonePattern(timezonePatterns.extended, string);
      }
    },
    set: function (date, flags, value, _options) {
      if (flags.timestampIsSet) {
        return date;
      }

      return new Date(date.getTime() - value);
    },
    incompatibleTokens: ['t', 'T', 'x']
  },
  // Timezone (ISO-8601)
  x: {
    priority: 10,
    parse: function (string, token, _match, _options) {
      switch (token) {
        case 'x':
          return parseTimezonePattern(timezonePatterns.basicOptionalMinutes, string);

        case 'xx':
          return parseTimezonePattern(timezonePatterns.basic, string);

        case 'xxxx':
          return parseTimezonePattern(timezonePatterns.basicOptionalSeconds, string);

        case 'xxxxx':
          return parseTimezonePattern(timezonePatterns.extendedOptionalSeconds, string);

        case 'xxx':
        default:
          return parseTimezonePattern(timezonePatterns.extended, string);
      }
    },
    set: function (date, flags, value, _options) {
      if (flags.timestampIsSet) {
        return date;
      }

      return new Date(date.getTime() - value);
    },
    incompatibleTokens: ['t', 'T', 'X']
  },
  // Seconds timestamp
  t: {
    priority: 40,
    parse: function (string, _token, _match, _options) {
      return parseAnyDigitsSigned(string);
    },
    set: function (_date, _flags, value, _options) {
      return [new Date(value * 1000), {
        timestampIsSet: true
      }];
    },
    incompatibleTokens: '*'
  },
  // Milliseconds timestamp
  T: {
    priority: 20,
    parse: function (string, _token, _match, _options) {
      return parseAnyDigitsSigned(string);
    },
    set: function (_date, _flags, value, _options) {
      return [new Date(value), {
        timestampIsSet: true
      }];
    },
    incompatibleTokens: '*'
  }
};

var TIMEZONE_UNIT_PRIORITY = 10; // This RegExp consists of three parts separated by `|`:
// - [yYQqMLwIdDecihHKkms]o matches any available ordinal number token
//   (one of the certain letters followed by `o`)
// - (\w)\1* matches any sequences of the same letter
// - '' matches two quote characters in a row
// - '(''|[^'])+('|$) matches anything surrounded by two quote characters ('),
//   except a single quote symbol, which ends the sequence.
//   Two quote characters do not end the sequence.
//   If there is no matching single quote
//   then the sequence will continue until the end of the string.
// - . matches any single character unmatched by previous parts of the RegExps

var formattingTokensRegExp = /[yYQqMLwIdDecihHKkms]o|(\w)\1*|''|'(''|[^'])+('|$)|./g; // This RegExp catches symbols escaped by quotes, and also
// sequences of symbols P, p, and the combinations like `PPPPPPPppppp`

var longFormattingTokensRegExp = /P+p+|P+|p+|''|'(''|[^'])+('|$)|./g;
var escapedStringRegExp = /^'([^]*?)'?$/;
var doubleQuoteRegExp = /''/g;
var notWhitespaceRegExp = /\S/;
var unescapedLatinCharacterRegExp = /[a-zA-Z]/;
/**
 * @name parse
 * @category Common Helpers
 * @summary Parse the date.
 *
 * @description
 * Return the date parsed from string using the given format string.
 *
 * > ⚠️ Please note that the `format` tokens differ from Moment.js and other libraries.
 * > See: https://git.io/fxCyr
 *
 * The characters in the format string wrapped between two single quotes characters (') are escaped.
 * Two single quotes in a row, whether inside or outside a quoted sequence, represent a 'real' single quote.
 *
 * Format of the format string is based on Unicode Technical Standard #35:
 * https://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table
 * with a few additions (see note 5 below the table).
 *
 * Not all tokens are compatible. Combinations that don't make sense or could lead to bugs are prohibited
 * and will throw `RangeError`. For example usage of 24-hour format token with AM/PM token will throw an exception:
 *
 * ```javascript
 * parse('23 AM', 'HH a', new Date())
 * //=> RangeError: The format string mustn't contain `HH` and `a` at the same time
 * ```
 *
 * See the compatibility table: https://docs.google.com/spreadsheets/d/e/2PACX-1vQOPU3xUhplll6dyoMmVUXHKl_8CRDs6_ueLmex3SoqwhuolkuN3O05l4rqx5h1dKX8eb46Ul-CCSrq/pubhtml?gid=0&single=true
 *
 * Accepted format string patterns:
 * | Unit                            |Prior| Pattern | Result examples                   | Notes |
 * |---------------------------------|-----|---------|-----------------------------------|-------|
 * | Era                             | 140 | G..GGG  | AD, BC                            |       |
 * |                                 |     | GGGG    | Anno Domini, Before Christ        | 2     |
 * |                                 |     | GGGGG   | A, B                              |       |
 * | Calendar year                   | 130 | y       | 44, 1, 1900, 2017, 9999           | 4     |
 * |                                 |     | yo      | 44th, 1st, 1900th, 9999999th      | 4,5   |
 * |                                 |     | yy      | 44, 01, 00, 17                    | 4     |
 * |                                 |     | yyy     | 044, 001, 123, 999                | 4     |
 * |                                 |     | yyyy    | 0044, 0001, 1900, 2017            | 4     |
 * |                                 |     | yyyyy   | ...                               | 2,4   |
 * | Local week-numbering year       | 130 | Y       | 44, 1, 1900, 2017, 9000           | 4     |
 * |                                 |     | Yo      | 44th, 1st, 1900th, 9999999th      | 4,5   |
 * |                                 |     | YY      | 44, 01, 00, 17                    | 4,6   |
 * |                                 |     | YYY     | 044, 001, 123, 999                | 4     |
 * |                                 |     | YYYY    | 0044, 0001, 1900, 2017            | 4,6   |
 * |                                 |     | YYYYY   | ...                               | 2,4   |
 * | ISO week-numbering year         | 130 | R       | -43, 1, 1900, 2017, 9999, -9999   | 4,5   |
 * |                                 |     | RR      | -43, 01, 00, 17                   | 4,5   |
 * |                                 |     | RRR     | -043, 001, 123, 999, -999         | 4,5   |
 * |                                 |     | RRRR    | -0043, 0001, 2017, 9999, -9999    | 4,5   |
 * |                                 |     | RRRRR   | ...                               | 2,4,5 |
 * | Extended year                   | 130 | u       | -43, 1, 1900, 2017, 9999, -999    | 4     |
 * |                                 |     | uu      | -43, 01, 99, -99                  | 4     |
 * |                                 |     | uuu     | -043, 001, 123, 999, -999         | 4     |
 * |                                 |     | uuuu    | -0043, 0001, 2017, 9999, -9999    | 4     |
 * |                                 |     | uuuuu   | ...                               | 2,4   |
 * | Quarter (formatting)            | 120 | Q       | 1, 2, 3, 4                        |       |
 * |                                 |     | Qo      | 1st, 2nd, 3rd, 4th                | 5     |
 * |                                 |     | QQ      | 01, 02, 03, 04                    |       |
 * |                                 |     | QQQ     | Q1, Q2, Q3, Q4                    |       |
 * |                                 |     | QQQQ    | 1st quarter, 2nd quarter, ...     | 2     |
 * |                                 |     | QQQQQ   | 1, 2, 3, 4                        | 4     |
 * | Quarter (stand-alone)           | 120 | q       | 1, 2, 3, 4                        |       |
 * |                                 |     | qo      | 1st, 2nd, 3rd, 4th                | 5     |
 * |                                 |     | qq      | 01, 02, 03, 04                    |       |
 * |                                 |     | qqq     | Q1, Q2, Q3, Q4                    |       |
 * |                                 |     | qqqq    | 1st quarter, 2nd quarter, ...     | 2     |
 * |                                 |     | qqqqq   | 1, 2, 3, 4                        | 3     |
 * | Month (formatting)              | 110 | M       | 1, 2, ..., 12                     |       |
 * |                                 |     | Mo      | 1st, 2nd, ..., 12th               | 5     |
 * |                                 |     | MM      | 01, 02, ..., 12                   |       |
 * |                                 |     | MMM     | Jan, Feb, ..., Dec                |       |
 * |                                 |     | MMMM    | January, February, ..., December  | 2     |
 * |                                 |     | MMMMM   | J, F, ..., D                      |       |
 * | Month (stand-alone)             | 110 | L       | 1, 2, ..., 12                     |       |
 * |                                 |     | Lo      | 1st, 2nd, ..., 12th               | 5     |
 * |                                 |     | LL      | 01, 02, ..., 12                   |       |
 * |                                 |     | LLL     | Jan, Feb, ..., Dec                |       |
 * |                                 |     | LLLL    | January, February, ..., December  | 2     |
 * |                                 |     | LLLLL   | J, F, ..., D                      |       |
 * | Local week of year              | 100 | w       | 1, 2, ..., 53                     |       |
 * |                                 |     | wo      | 1st, 2nd, ..., 53th               | 5     |
 * |                                 |     | ww      | 01, 02, ..., 53                   |       |
 * | ISO week of year                | 100 | I       | 1, 2, ..., 53                     | 5     |
 * |                                 |     | Io      | 1st, 2nd, ..., 53th               | 5     |
 * |                                 |     | II      | 01, 02, ..., 53                   | 5     |
 * | Day of month                    |  90 | d       | 1, 2, ..., 31                     |       |
 * |                                 |     | do      | 1st, 2nd, ..., 31st               | 5     |
 * |                                 |     | dd      | 01, 02, ..., 31                   |       |
 * | Day of year                     |  90 | D       | 1, 2, ..., 365, 366               | 7     |
 * |                                 |     | Do      | 1st, 2nd, ..., 365th, 366th       | 5     |
 * |                                 |     | DD      | 01, 02, ..., 365, 366             | 7     |
 * |                                 |     | DDD     | 001, 002, ..., 365, 366           |       |
 * |                                 |     | DDDD    | ...                               | 2     |
 * | Day of week (formatting)        |  90 | E..EEE  | Mon, Tue, Wed, ..., Sun           |       |
 * |                                 |     | EEEE    | Monday, Tuesday, ..., Sunday      | 2     |
 * |                                 |     | EEEEE   | M, T, W, T, F, S, S               |       |
 * |                                 |     | EEEEEE  | Mo, Tu, We, Th, Fr, Sa, Su        |       |
 * | ISO day of week (formatting)    |  90 | i       | 1, 2, 3, ..., 7                   | 5     |
 * |                                 |     | io      | 1st, 2nd, ..., 7th                | 5     |
 * |                                 |     | ii      | 01, 02, ..., 07                   | 5     |
 * |                                 |     | iii     | Mon, Tue, Wed, ..., Sun           | 5     |
 * |                                 |     | iiii    | Monday, Tuesday, ..., Sunday      | 2,5   |
 * |                                 |     | iiiii   | M, T, W, T, F, S, S               | 5     |
 * |                                 |     | iiiiii  | Mo, Tu, We, Th, Fr, Sa, Su        | 5     |
 * | Local day of week (formatting)  |  90 | e       | 2, 3, 4, ..., 1                   |       |
 * |                                 |     | eo      | 2nd, 3rd, ..., 1st                | 5     |
 * |                                 |     | ee      | 02, 03, ..., 01                   |       |
 * |                                 |     | eee     | Mon, Tue, Wed, ..., Sun           |       |
 * |                                 |     | eeee    | Monday, Tuesday, ..., Sunday      | 2     |
 * |                                 |     | eeeee   | M, T, W, T, F, S, S               |       |
 * |                                 |     | eeeeee  | Mo, Tu, We, Th, Fr, Sa, Su        |       |
 * | Local day of week (stand-alone) |  90 | c       | 2, 3, 4, ..., 1                   |       |
 * |                                 |     | co      | 2nd, 3rd, ..., 1st                | 5     |
 * |                                 |     | cc      | 02, 03, ..., 01                   |       |
 * |                                 |     | ccc     | Mon, Tue, Wed, ..., Sun           |       |
 * |                                 |     | cccc    | Monday, Tuesday, ..., Sunday      | 2     |
 * |                                 |     | ccccc   | M, T, W, T, F, S, S               |       |
 * |                                 |     | cccccc  | Mo, Tu, We, Th, Fr, Sa, Su        |       |
 * | AM, PM                          |  80 | a..aaa  | AM, PM                            |       |
 * |                                 |     | aaaa    | a.m., p.m.                        | 2     |
 * |                                 |     | aaaaa   | a, p                              |       |
 * | AM, PM, noon, midnight          |  80 | b..bbb  | AM, PM, noon, midnight            |       |
 * |                                 |     | bbbb    | a.m., p.m., noon, midnight        | 2     |
 * |                                 |     | bbbbb   | a, p, n, mi                       |       |
 * | Flexible day period             |  80 | B..BBB  | at night, in the morning, ...     |       |
 * |                                 |     | BBBB    | at night, in the morning, ...     | 2     |
 * |                                 |     | BBBBB   | at night, in the morning, ...     |       |
 * | Hour [1-12]                     |  70 | h       | 1, 2, ..., 11, 12                 |       |
 * |                                 |     | ho      | 1st, 2nd, ..., 11th, 12th         | 5     |
 * |                                 |     | hh      | 01, 02, ..., 11, 12               |       |
 * | Hour [0-23]                     |  70 | H       | 0, 1, 2, ..., 23                  |       |
 * |                                 |     | Ho      | 0th, 1st, 2nd, ..., 23rd          | 5     |
 * |                                 |     | HH      | 00, 01, 02, ..., 23               |       |
 * | Hour [0-11]                     |  70 | K       | 1, 2, ..., 11, 0                  |       |
 * |                                 |     | Ko      | 1st, 2nd, ..., 11th, 0th          | 5     |
 * |                                 |     | KK      | 01, 02, ..., 11, 00               |       |
 * | Hour [1-24]                     |  70 | k       | 24, 1, 2, ..., 23                 |       |
 * |                                 |     | ko      | 24th, 1st, 2nd, ..., 23rd         | 5     |
 * |                                 |     | kk      | 24, 01, 02, ..., 23               |       |
 * | Minute                          |  60 | m       | 0, 1, ..., 59                     |       |
 * |                                 |     | mo      | 0th, 1st, ..., 59th               | 5     |
 * |                                 |     | mm      | 00, 01, ..., 59                   |       |
 * | Second                          |  50 | s       | 0, 1, ..., 59                     |       |
 * |                                 |     | so      | 0th, 1st, ..., 59th               | 5     |
 * |                                 |     | ss      | 00, 01, ..., 59                   |       |
 * | Seconds timestamp               |  40 | t       | 512969520                         |       |
 * |                                 |     | tt      | ...                               | 2     |
 * | Fraction of second              |  30 | S       | 0, 1, ..., 9                      |       |
 * |                                 |     | SS      | 00, 01, ..., 99                   |       |
 * |                                 |     | SSS     | 000, 001, ..., 999                |       |
 * |                                 |     | SSSS    | ...                               | 2     |
 * | Milliseconds timestamp          |  20 | T       | 512969520900                      |       |
 * |                                 |     | TT      | ...                               | 2     |
 * | Timezone (ISO-8601 w/ Z)        |  10 | X       | -08, +0530, Z                     |       |
 * |                                 |     | XX      | -0800, +0530, Z                   |       |
 * |                                 |     | XXX     | -08:00, +05:30, Z                 |       |
 * |                                 |     | XXXX    | -0800, +0530, Z, +123456          | 2     |
 * |                                 |     | XXXXX   | -08:00, +05:30, Z, +12:34:56      |       |
 * | Timezone (ISO-8601 w/o Z)       |  10 | x       | -08, +0530, +00                   |       |
 * |                                 |     | xx      | -0800, +0530, +0000               |       |
 * |                                 |     | xxx     | -08:00, +05:30, +00:00            | 2     |
 * |                                 |     | xxxx    | -0800, +0530, +0000, +123456      |       |
 * |                                 |     | xxxxx   | -08:00, +05:30, +00:00, +12:34:56 |       |
 * | Long localized date             |  NA | P       | 05/29/1453                        | 5,8   |
 * |                                 |     | PP      | May 29, 1453                      |       |
 * |                                 |     | PPP     | May 29th, 1453                    |       |
 * |                                 |     | PPPP    | Sunday, May 29th, 1453            | 2,5,8 |
 * | Long localized time             |  NA | p       | 12:00 AM                          | 5,8   |
 * |                                 |     | pp      | 12:00:00 AM                       |       |
 * | Combination of date and time    |  NA | Pp      | 05/29/1453, 12:00 AM              |       |
 * |                                 |     | PPpp    | May 29, 1453, 12:00:00 AM         |       |
 * |                                 |     | PPPpp   | May 29th, 1453 at ...             |       |
 * |                                 |     | PPPPpp  | Sunday, May 29th, 1453 at ...     | 2,5,8 |
 * Notes:
 * 1. "Formatting" units (e.g. formatting quarter) in the default en-US locale
 *    are the same as "stand-alone" units, but are different in some languages.
 *    "Formatting" units are declined according to the rules of the language
 *    in the context of a date. "Stand-alone" units are always nominative singular.
 *    In `format` function, they will produce different result:
 *
 *    `format(new Date(2017, 10, 6), 'do LLLL', {locale: cs}) //=> '6. listopad'`
 *
 *    `format(new Date(2017, 10, 6), 'do MMMM', {locale: cs}) //=> '6. listopadu'`
 *
 *    `parse` will try to match both formatting and stand-alone units interchangably.
 *
 * 2. Any sequence of the identical letters is a pattern, unless it is escaped by
 *    the single quote characters (see below).
 *    If the sequence is longer than listed in table:
 *    - for numerical units (`yyyyyyyy`) `parse` will try to match a number
 *      as wide as the sequence
 *    - for text units (`MMMMMMMM`) `parse` will try to match the widest variation of the unit.
 *      These variations are marked with "2" in the last column of the table.
 *
 * 3. `QQQQQ` and `qqqqq` could be not strictly numerical in some locales.
 *    These tokens represent the shortest form of the quarter.
 *
 * 4. The main difference between `y` and `u` patterns are B.C. years:
 *
 *    | Year | `y` | `u` |
 *    |------|-----|-----|
 *    | AC 1 |   1 |   1 |
 *    | BC 1 |   1 |   0 |
 *    | BC 2 |   2 |  -1 |
 *
 *    Also `yy` will try to guess the century of two digit year by proximity with `referenceDate`:
 *
 *    `parse('50', 'yy', new Date(2018, 0, 1)) //=> Sat Jan 01 2050 00:00:00`
 *
 *    `parse('75', 'yy', new Date(2018, 0, 1)) //=> Wed Jan 01 1975 00:00:00`
 *
 *    while `uu` will just assign the year as is:
 *
 *    `parse('50', 'uu', new Date(2018, 0, 1)) //=> Sat Jan 01 0050 00:00:00`
 *
 *    `parse('75', 'uu', new Date(2018, 0, 1)) //=> Tue Jan 01 0075 00:00:00`
 *
 *    The same difference is true for local and ISO week-numbering years (`Y` and `R`),
 *    except local week-numbering years are dependent on `options.weekStartsOn`
 *    and `options.firstWeekContainsDate` (compare [setISOWeekYear]{@link https://date-fns.org/docs/setISOWeekYear}
 *    and [setWeekYear]{@link https://date-fns.org/docs/setWeekYear}).
 *
 * 5. These patterns are not in the Unicode Technical Standard #35:
 *    - `i`: ISO day of week
 *    - `I`: ISO week of year
 *    - `R`: ISO week-numbering year
 *    - `o`: ordinal number modifier
 *    - `P`: long localized date
 *    - `p`: long localized time
 *
 * 6. `YY` and `YYYY` tokens represent week-numbering years but they are often confused with years.
 *    You should enable `options.useAdditionalWeekYearTokens` to use them. See: https://git.io/fxCyr
 *
 * 7. `D` and `DD` tokens represent days of the year but they are ofthen confused with days of the month.
 *    You should enable `options.useAdditionalDayOfYearTokens` to use them. See: https://git.io/fxCyr
 *
 * 8. `P+` tokens do not have a defined priority since they are merely aliases to other tokens based
 *    on the given locale.
 *
 *    using `en-US` locale: `P` => `MM/dd/yyyy`
 *    using `en-US` locale: `p` => `hh:mm a`
 *    using `pt-BR` locale: `P` => `dd/MM/yyyy`
 *    using `pt-BR` locale: `p` => `HH:mm`
 *
 * Values will be assigned to the date in the descending order of its unit's priority.
 * Units of an equal priority overwrite each other in the order of appearance.
 *
 * If no values of higher priority are parsed (e.g. when parsing string 'January 1st' without a year),
 * the values will be taken from 3rd argument `referenceDate` which works as a context of parsing.
 *
 * `referenceDate` must be passed for correct work of the function.
 * If you're not sure which `referenceDate` to supply, create a new instance of Date:
 * `parse('02/11/2014', 'MM/dd/yyyy', new Date())`
 * In this case parsing will be done in the context of the current date.
 * If `referenceDate` is `Invalid Date` or a value not convertible to valid `Date`,
 * then `Invalid Date` will be returned.
 *
 * The result may vary by locale.
 *
 * If `formatString` matches with `dateString` but does not provides tokens, `referenceDate` will be returned.
 *
 * If parsing failed, `Invalid Date` will be returned.
 * Invalid Date is a Date, whose time value is NaN.
 * Time value of Date: http://es5.github.io/#x15.9.1.1
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - Old `parse` was renamed to `toDate`.
 *   Now `parse` is a new function which parses a string using a provided format.
 *
 *   ```javascript
 *   // Before v2.0.0
 *   parse('2016-01-01')
 *
 *   // v2.0.0 onward (toDate no longer accepts a string)
 *   toDate(1392098430000) // Unix to timestamp
 *   toDate(new Date(2014, 1, 11, 11, 30, 30)) // Cloning the date
 *   parse('2016-01-01', 'yyyy-MM-dd', new Date())
 *   ```
 *
 * @param {String} dateString - the string to parse
 * @param {String} formatString - the string of tokens
 * @param {Date|Number} referenceDate - defines values missing from the parsed dateString
 * @param {Object} [options] - an object with options.
 * @param {Locale} [options.locale=defaultLocale] - the locale object. See [Locale]{@link https://date-fns.org/docs/Locale}
 * @param {0|1|2|3|4|5|6} [options.weekStartsOn=0] - the index of the first day of the week (0 - Sunday)
 * @param {1|2|3|4|5|6|7} [options.firstWeekContainsDate=1] - the day of January, which is always in the first week of the year
 * @param {Boolean} [options.useAdditionalWeekYearTokens=false] - if true, allows usage of the week-numbering year tokens `YY` and `YYYY`;
 *   see: https://git.io/fxCyr
 * @param {Boolean} [options.useAdditionalDayOfYearTokens=false] - if true, allows usage of the day of year tokens `D` and `DD`;
 *   see: https://git.io/fxCyr
 * @returns {Date} the parsed date
 * @throws {TypeError} 3 arguments required
 * @throws {RangeError} `options.weekStartsOn` must be between 0 and 6
 * @throws {RangeError} `options.firstWeekContainsDate` must be between 1 and 7
 * @throws {RangeError} `options.locale` must contain `match` property
 * @throws {RangeError} use `yyyy` instead of `YYYY` for formatting years using [format provided] to the input [input provided]; see: https://git.io/fxCyr
 * @throws {RangeError} use `yy` instead of `YY` for formatting years using [format provided] to the input [input provided]; see: https://git.io/fxCyr
 * @throws {RangeError} use `d` instead of `D` for formatting days of the month using [format provided] to the input [input provided]; see: https://git.io/fxCyr
 * @throws {RangeError} use `dd` instead of `DD` for formatting days of the month using [format provided] to the input [input provided]; see: https://git.io/fxCyr
 * @throws {RangeError} format string contains an unescaped latin alphabet character
 *
 * @example
 * // Parse 11 February 2014 from middle-endian format:
 * var result = parse('02/11/2014', 'MM/dd/yyyy', new Date())
 * //=> Tue Feb 11 2014 00:00:00
 *
 * @example
 * // Parse 28th of February in Esperanto locale in the context of 2010 year:
 * import eo from 'date-fns/locale/eo'
 * var result = parse('28-a de februaro', "do 'de' MMMM", new Date(2010, 0, 1), {
 *   locale: eo
 * })
 * //=> Sun Feb 28 2010 00:00:00
 */

function parse(dirtyDateString, dirtyFormatString, dirtyReferenceDate, dirtyOptions) {
  utils.requiredArgs(3, arguments);
  var dateString = String(dirtyDateString);
  var formatString = String(dirtyFormatString);
  var options = dirtyOptions || {};
  var locale = options.locale || utils.locale;

  if (!locale.match) {
    throw new RangeError('locale must contain match property');
  }

  var localeFirstWeekContainsDate = locale.options && locale.options.firstWeekContainsDate;
  var defaultFirstWeekContainsDate = localeFirstWeekContainsDate == null ? 1 : utils.toInteger(localeFirstWeekContainsDate);
  var firstWeekContainsDate = options.firstWeekContainsDate == null ? defaultFirstWeekContainsDate : utils.toInteger(options.firstWeekContainsDate); // Test if weekStartsOn is between 1 and 7 _and_ is not NaN

  if (!(firstWeekContainsDate >= 1 && firstWeekContainsDate <= 7)) {
    throw new RangeError('firstWeekContainsDate must be between 1 and 7 inclusively');
  }

  var localeWeekStartsOn = locale.options && locale.options.weekStartsOn;
  var defaultWeekStartsOn = localeWeekStartsOn == null ? 0 : utils.toInteger(localeWeekStartsOn);
  var weekStartsOn = options.weekStartsOn == null ? defaultWeekStartsOn : utils.toInteger(options.weekStartsOn); // Test if weekStartsOn is between 0 and 6 _and_ is not NaN

  if (!(weekStartsOn >= 0 && weekStartsOn <= 6)) {
    throw new RangeError('weekStartsOn must be between 0 and 6 inclusively');
  }

  if (formatString === '') {
    if (dateString === '') {
      return utils.toDate(dirtyReferenceDate);
    } else {
      return new Date(NaN);
    }
  }

  var subFnOptions = {
    firstWeekContainsDate: firstWeekContainsDate,
    weekStartsOn: weekStartsOn,
    locale: locale
  }; // If timezone isn't specified, it will be set to the system timezone

  var setters = [{
    priority: TIMEZONE_UNIT_PRIORITY,
    subPriority: -1,
    set: dateToSystemTimezone,
    index: 0
  }];
  var i;
  var tokens = formatString.match(longFormattingTokensRegExp).map(function (substring) {
    var firstCharacter = substring[0];

    if (firstCharacter === 'p' || firstCharacter === 'P') {
      var longFormatter = utils.longFormatters[firstCharacter];
      return longFormatter(substring, locale.formatLong, subFnOptions);
    }

    return substring;
  }).join('').match(formattingTokensRegExp);
  var usedTokens = [];

  for (i = 0; i < tokens.length; i++) {
    var token = tokens[i];

    if (!options.useAdditionalWeekYearTokens && utils.isProtectedWeekYearToken(token)) {
      utils.throwProtectedError(token, formatString, dirtyDateString);
    }

    if (!options.useAdditionalDayOfYearTokens && utils.isProtectedDayOfYearToken(token)) {
      utils.throwProtectedError(token, formatString, dirtyDateString);
    }

    var firstCharacter = token[0];
    var parser = parsers[firstCharacter];

    if (parser) {
      var incompatibleTokens = parser.incompatibleTokens;

      if (Array.isArray(incompatibleTokens)) {
        var incompatibleToken = void 0;

        for (var _i = 0; _i < usedTokens.length; _i++) {
          var usedToken = usedTokens[_i].token;

          if (incompatibleTokens.indexOf(usedToken) !== -1 || usedToken === firstCharacter) {
            incompatibleToken = usedTokens[_i];
            break;
          }
        }

        if (incompatibleToken) {
          throw new RangeError("The format string mustn't contain `".concat(incompatibleToken.fullToken, "` and `").concat(token, "` at the same time"));
        }
      } else if (parser.incompatibleTokens === '*' && usedTokens.length) {
        throw new RangeError("The format string mustn't contain `".concat(token, "` and any other token at the same time"));
      }

      usedTokens.push({
        token: firstCharacter,
        fullToken: token
      });
      var parseResult = parser.parse(dateString, token, locale.match, subFnOptions);

      if (!parseResult) {
        return new Date(NaN);
      }

      setters.push({
        priority: parser.priority,
        subPriority: parser.subPriority || 0,
        set: parser.set,
        validate: parser.validate,
        value: parseResult.value,
        index: setters.length
      });
      dateString = parseResult.rest;
    } else {
      if (firstCharacter.match(unescapedLatinCharacterRegExp)) {
        throw new RangeError('Format string contains an unescaped latin alphabet character `' + firstCharacter + '`');
      } // Replace two single quote characters with one single quote character


      if (token === "''") {
        token = "'";
      } else if (firstCharacter === "'") {
        token = cleanEscapedString(token);
      } // Cut token from string, or, if string doesn't match the token, return Invalid Date


      if (dateString.indexOf(token) === 0) {
        dateString = dateString.slice(token.length);
      } else {
        return new Date(NaN);
      }
    }
  } // Check if the remaining input contains something other than whitespace


  if (dateString.length > 0 && notWhitespaceRegExp.test(dateString)) {
    return new Date(NaN);
  }

  var uniquePrioritySetters = setters.map(function (setter) {
    return setter.priority;
  }).sort(function (a, b) {
    return b - a;
  }).filter(function (priority, index, array) {
    return array.indexOf(priority) === index;
  }).map(function (priority) {
    return setters.filter(function (setter) {
      return setter.priority === priority;
    }).sort(function (a, b) {
      return b.subPriority - a.subPriority;
    });
  }).map(function (setterArray) {
    return setterArray[0];
  });
  var date = utils.toDate(dirtyReferenceDate);

  if (isNaN(date)) {
    return new Date(NaN);
  } // Convert the date in system timezone to the same date in UTC+00:00 timezone.
  // This ensures that when UTC functions will be implemented, locales will be compatible with them.
  // See an issue about UTC functions: https://github.com/date-fns/date-fns/issues/37


  var utcDate = utils.subMilliseconds(date, utils.getTimezoneOffsetInMilliseconds(date));
  var flags = {};

  for (i = 0; i < uniquePrioritySetters.length; i++) {
    var setter = uniquePrioritySetters[i];

    if (setter.validate && !setter.validate(utcDate, setter.value, subFnOptions)) {
      return new Date(NaN);
    }

    var result = setter.set(utcDate, flags, setter.value, subFnOptions); // Result is tuple (date, flags)

    if (result[0]) {
      utcDate = result[0];
      assign(flags, result[1]); // Result is date
    } else {
      utcDate = result;
    }
  }

  return utcDate;
}

function dateToSystemTimezone(date, flags) {
  if (flags.timestampIsSet) {
    return date;
  }

  var convertedDate = new Date(0);
  convertedDate.setFullYear(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate());
  convertedDate.setHours(date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds(), date.getUTCMilliseconds());
  return convertedDate;
}

function cleanEscapedString(input) {
  return input.match(escapedStringRegExp)[1].replace(doubleQuoteRegExp, "'");
}

/**
 * @name isSameMonth
 * @category Month Helpers
 * @summary Are the given dates in the same month (and year)?
 *
 * @description
 * Are the given dates in the same month (and year)?
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} dateLeft - the first date to check
 * @param {Date|Number} dateRight - the second date to check
 * @returns {Boolean} the dates are in the same month (and year)
 * @throws {TypeError} 2 arguments required
 *
 * @example
 * // Are 2 September 2014 and 25 September 2014 in the same month?
 * var result = isSameMonth(new Date(2014, 8, 2), new Date(2014, 8, 25))
 * //=> true
 *
 * @example
 * // Are 2 September 2014 and 25 September 2015 in the same month?
 * var result = isSameMonth(new Date(2014, 8, 2), new Date(2015, 8, 25))
 * //=> false
 */

function isSameMonth(dirtyDateLeft, dirtyDateRight) {
  utils.requiredArgs(2, arguments);
  var dateLeft = utils.toDate(dirtyDateLeft);
  var dateRight = utils.toDate(dirtyDateRight);
  return dateLeft.getFullYear() === dateRight.getFullYear() && dateLeft.getMonth() === dateRight.getMonth();
}

/**
 * @name isSameYear
 * @category Year Helpers
 * @summary Are the given dates in the same year?
 *
 * @description
 * Are the given dates in the same year?
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} dateLeft - the first date to check
 * @param {Date|Number} dateRight - the second date to check
 * @returns {Boolean} the dates are in the same year
 * @throws {TypeError} 2 arguments required
 *
 * @example
 * // Are 2 September 2014 and 25 September 2014 in the same year?
 * var result = isSameYear(new Date(2014, 8, 2), new Date(2014, 8, 25))
 * //=> true
 */

function isSameYear(dirtyDateLeft, dirtyDateRight) {
  utils.requiredArgs(2, arguments);
  var dateLeft = utils.toDate(dirtyDateLeft);
  var dateRight = utils.toDate(dirtyDateRight);
  return dateLeft.getFullYear() === dateRight.getFullYear();
}

/**
 * @name isWithinInterval
 * @category Interval Helpers
 * @summary Is the given date within the interval?
 *
 * @description
 * Is the given date within the interval? (Including start and end.)
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * - The function was renamed from `isWithinRange` to `isWithinInterval`.
 *   This change was made to mirror the use of the word "interval" in standard ISO 8601:2004 terminology:
 *
 *   ```
 *   2.1.3
 *   time interval
 *   part of the time axis limited by two instants
 *   ```
 *
 *   Also, this function now accepts an object with `start` and `end` properties
 *   instead of two arguments as an interval.
 *   This function now throws `RangeError` if the start of the interval is after its end
 *   or if any date in the interval is `Invalid Date`.
 *
 *   ```javascript
 *   // Before v2.0.0
 *
 *   isWithinRange(
 *     new Date(2014, 0, 3),
 *     new Date(2014, 0, 1), new Date(2014, 0, 7)
 *   )
 *
 *   // v2.0.0 onward
 *
 *   isWithinInterval(
 *     new Date(2014, 0, 3),
 *     { start: new Date(2014, 0, 1), end: new Date(2014, 0, 7) }
 *   )
 *   ```
 *
 * @param {Date|Number} date - the date to check
 * @param {Interval} interval - the interval to check
 * @returns {Boolean} the date is within the interval
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} The start of an interval cannot be after its end
 * @throws {RangeError} Date in interval cannot be `Invalid Date`
 *
 * @example
 * // For the date within the interval:
 * isWithinInterval(new Date(2014, 0, 3), {
 *   start: new Date(2014, 0, 1),
 *   end: new Date(2014, 0, 7)
 * })
 * //=> true
 *
 * @example
 * // For the date outside of the interval:
 * isWithinInterval(new Date(2014, 0, 10), {
 *   start: new Date(2014, 0, 1),
 *   end: new Date(2014, 0, 7)
 * })
 * //=> false
 *
 * @example
 * // For date equal to interval start:
 * isWithinInterval(date, { start, end: date }) // => true
 *
 * @example
 * // For date equal to interval end:
 * isWithinInterval(date, { start: date, end }) // => true
 */
function isWithinInterval(dirtyDate, interval) {
  utils.requiredArgs(2, arguments);
  var time = utils.toDate(dirtyDate).getTime();
  var startTime = utils.toDate(interval.start).getTime();
  var endTime = utils.toDate(interval.end).getTime(); // Throw an exception if start date is after end date or if any date is `Invalid Date`

  if (!(startTime <= endTime)) {
    throw new RangeError('Invalid interval');
  }

  return time >= startTime && time <= endTime;
}

/**
 * @name setDay
 * @category Weekday Helpers
 * @summary Set the day of the week to the given date.
 *
 * @description
 * Set the day of the week to the given date.
 *
 * ### v2.0.0 breaking changes:
 *
 * - [Changes that are common for the whole library](https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#Common-Changes).
 *
 * @param {Date|Number} date - the date to be changed
 * @param {Number} day - the day of the week of the new date
 * @param {Object} [options] - an object with options.
 * @param {Locale} [options.locale=defaultLocale] - the locale object. See [Locale]{@link https://date-fns.org/docs/Locale}
 * @param {0|1|2|3|4|5|6} [options.weekStartsOn=0] - the index of the first day of the week (0 - Sunday)
 * @returns {Date} the new date with the day of the week set
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} `options.weekStartsOn` must be between 0 and 6
 *
 * @example
 * // Set week day to Sunday, with the default weekStartsOn of Sunday:
 * var result = setDay(new Date(2014, 8, 1), 0)
 * //=> Sun Aug 31 2014 00:00:00
 *
 * @example
 * // Set week day to Sunday, with a weekStartsOn of Monday:
 * var result = setDay(new Date(2014, 8, 1), 0, { weekStartsOn: 1 })
 * //=> Sun Sep 07 2014 00:00:00
 */
function setDay(dirtyDate, dirtyDay, dirtyOptions) {
  utils.requiredArgs(2, arguments);
  var options = dirtyOptions || {};
  var locale = options.locale;
  var localeWeekStartsOn = locale && locale.options && locale.options.weekStartsOn;
  var defaultWeekStartsOn = localeWeekStartsOn == null ? 0 : utils.toInteger(localeWeekStartsOn);
  var weekStartsOn = options.weekStartsOn == null ? defaultWeekStartsOn : utils.toInteger(options.weekStartsOn); // Test if weekStartsOn is between 0 and 6 _and_ is not NaN

  if (!(weekStartsOn >= 0 && weekStartsOn <= 6)) {
    throw new RangeError('weekStartsOn must be between 0 and 6 inclusively');
  }

  var date = utils.toDate(dirtyDate);
  var day = utils.toInteger(dirtyDay);
  var currentDay = date.getDay();
  var remainder = day % 7;
  var dayIndex = (remainder + 7) % 7;
  var delta = 7 - weekStartsOn;
  var diff = day < 0 || day > 6 ? day - (currentDay + delta) % 7 : (dayIndex + delta) % 7 - (currentDay + delta) % 7;
  return utils.addDays(date, diff);
}

const monthHeaderCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bci-core-month-header{display:flex;justify-content:space-between;align-items:center;color:#000000;text-align:center;border-bottom:1px solid #8a9097}.bci-core-month-header .month-header{display:flex;justify-content:center;align-items:center;height:48px;min-width:2em;font-size:16px;font-weight:400;margin-left:14px;color:#000000;cursor:pointer;outline:none}.bci-core-month-header .month-header:hover,.bci-core-month-header .month-header:focus{color:#007bc0}.bci-core-month-header .month-header:active{color:#00629a}.bci-core-month-header .month-header p{margin:0}.bci-core-month-header .month-header .open-arrow{margin-left:4px}.bci-core-month-header .month-header .open-arrow>i{display:inline-block;width:24px;aspect-ratio:1}.bci-core-month-header .month-header.opened{color:#007bc0}.bci-core-month-header .month-header.opened:hover,.bci-core-month-header .month-header.opened:focus{color:#00629a}.bci-core-month-header .month-header.opened .open-arrow{transform:rotate(180deg)}.bci-core-month-header div{display:flex}.bci-core-month-header span:not(.open-arrow){display:grid;place-items:center;width:48px;aspect-ratio:1;color:#000000;outline:none}.bci-core-month-header span:not(.open-arrow):not(.disabled){cursor:pointer}.bci-core-month-header span:not(.open-arrow):not(.disabled):hover,.bci-core-month-header span:not(.open-arrow):not(.disabled):focus{color:#007bc0}.bci-core-month-header span:not(.open-arrow):not(.disabled):active{color:#00629a}.bci-core-month-header span:not(.open-arrow).disabled{pointer-events:none;color:#c1c7cc}";

const MonthHeader = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.arrowClicked = index.createEvent(this, "arrowClicked", 7);
    this.toggleShowYearMonthSelection = index.createEvent(this, "toggleShowYearMonthSelection", 7);
  }
  handleMonthChange(newValue) {
    this.setFormattedMonth(newValue);
  }
  handleYearChange(newValue) {
    this.displayedYear = newValue;
  }
  componentWillLoad() {
    this.setFormattedMonth(this.month);
    this.displayedYear = this.year;
  }
  arrowClickHandler(arrowDirection) {
    this.arrowClicked.emit(arrowDirection);
  }
  handleKeyboardClick(event) {
    if ((event.key !== 'Enter' && event.key !== ' ') || event.defaultPrevented)
      return;
    event.target.click();
  }
  render() {
    return (index.h("div", { class: "bci-core-month-header" }, index.h("div", { "data-test": "webcore.webcomponents.month-header.yearMonthSelection", class: {
        'month-header': true,
        opened: !!this.yearMonthSelectionStep,
      }, tabIndex: 0, role: "button", "aria-label": `${!!this.yearMonthSelectionStep ? 'Close' : 'Open'} the Year/Month Selection`, onKeyDown: (ev) => this.handleKeyboardClick(ev), onClick: () => {
        this.toggleShowYearMonthSelection.emit({ newState: !!this.yearMonthSelectionStep ? false : true });
      } }, index.h("p", null, `${this.displayedMonth} ${this.displayedYear}` || 'Month Header'), index.h("span", { class: "open-arrow" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down" }))), index.h("div", null, index.h("span", { class: { 'left-arrow': true, disabled: this.yearMonthSelectionStep === 'month' }, role: "button", tabIndex: 0, "aria-label": `Show previous ${!!this.yearMonthSelectionStep ? '20 years' : 'month'}`, onKeyDown: (ev) => this.handleKeyboardClick(ev), onClick: () => this.arrowClickHandler({ direction: 'prev' }) }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-back-left", "data-test": "webcore.webcomponents.month-header.month-change-handler.span" })), index.h("span", { class: { 'right-arrow': true, disabled: this.yearMonthSelectionStep === 'month' }, role: "button", tabIndex: 0, "aria-label": `Show next ${!!this.yearMonthSelectionStep ? '20 years' : 'month'}`, onKeyDown: (ev) => this.handleKeyboardClick(ev), onClick: () => this.arrowClickHandler({ direction: 'next' }) }, index.h("i", { "data-test": "webcore.webcomponents.bci-core-month-header-month-change-header.next", class: "bosch-ic bosch-ic-clickable bosch-ic-forward-right" })))));
  }
  setFormattedMonth(value) {
    const locale = { locale: utils.LOCALE_MAP[utils.state.language || utils.DEFAULT_LOCALE.language] };
    this.displayedMonth = utils.format(utils.setMonth(new Date(), value), 'MMMM', locale);
  }
  static get watchers() { return {
    "month": ["handleMonthChange"],
    "year": ["handleYearChange"]
  }; }
};
MonthHeader.style = monthHeaderCss;

const weekHeaderCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */:host .bci-core-datepicker-week-container{display:flex;justify-content:space-between;padding:15px 5px 10px 5px}:host .bci-core-datepicker-week-container .bci-core-datepicker-week-header{font-size:12px;width:40px;line-height:18px;font-weight:700}";

const WeekHeader = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
  }
  getWeekDays() {
    const locale = { locale: utils.LOCALE_MAP[utils.state.language || utils.DEFAULT_LOCALE.language] };
    const weekdays = [];
    for (let i = 0; i < 7; i++) {
      weekdays.push(utils.format(setDay(new Date(), i), 'EEE', locale));
    }
    return weekdays;
  }
  render() {
    const weekDays = this.getWeekDays();
    if (!this.sundayFirst) {
      const sunday = weekDays.shift();
      weekDays.push(sunday);
    }
    return (index.h("div", { "data-test": "webcore.webcomponents.week-header.bci-week-header.day", class: "bci-core-datepicker-week-container" }, weekDays.map((day) => (index.h("th", { class: "bci-core-datepicker-week-header" }, index.h("span", null, day))))));
  }
};
WeekHeader.style = weekHeaderCss;

const weekdaysCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bci-core-weekdays-table{width:calc(100% - 10px);border-spacing:0;border-collapse:collapse;margin:0 5px 5px 5px}.bci-core-weekdays-table td{position:relative;height:0;line-height:0;text-align:center;outline:0;cursor:pointer;padding-top:8px;padding-bottom:32px}.bci-core-weekdays-table td .bci-core-date{color:#000000;font-size:16px;font-weight:400;text-align:center;display:flex;line-height:1;border-width:1px;border-style:solid;border-color:transparent;border-radius:50%;width:32px;height:32px;position:absolute;left:4px;top:4px;align-items:center;justify-content:center;box-sizing:border-box}.bci-core-weekdays-table td.bci-core-disabled{cursor:default}.bci-core-weekdays-table td.bci-core-disabled .bci-core-date{background:transparent;color:#c1c7cc}.bci-core-weekdays-table td.bci-core-disabled.bci-core-today .bci-core-date{border:1px solid #c1c7cc}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected .bci-core-date{background-color:#c1c7cc;color:#8a9097}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range{border-radius:0px}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range .bci-core-date{background-color:transparent}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range.bci-core-first-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;border-radius:16px 0 0 16px}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range.bci-core-last-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;width:32px;border-radius:0 16px 16px 0}.bci-core-weekdays-table td.bci-core-disabled.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range)::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#c1c7cc;border-radius:0}.bci-core-weekdays-table td.bci-core-week-days{cursor:pointer}.bci-core-weekdays-table td.bci-core-week-days .bci-core-date:hover{background-color:#e0e2e5}.bci-core-weekdays-table td.bci-core-week-days .bci-core-date:active{background-color:#c1c7cc}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date{background:transparent;border:1px solid #000000;color:#000000}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date:hover{background-color:#e0e2e5}.bci-core-weekdays-table td.bci-core-week-days.bci-core-today .bci-core-date:active{background-color:#c1c7cc}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date{background-color:#007bc0;color:#ffffff}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date:hover{background-color:#00629a}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected .bci-core-date:active{background-color:#004975}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range{border-radius:0px}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date{background-color:transparent}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date:hover{background-color:#00629a}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range .bci-core-date:active{background-color:#004975}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range.bci-core-first-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;border-radius:16px 0 0 16px}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range.bci-core-last-date-range::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;width:32px;border-radius:0 16px 16px 0}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range)::before{content:\"\";display:block;width:41px;height:32px;position:absolute;left:4px;top:4px;background-color:#007bc0;border-radius:0}.bci-core-weekdays-table td.bci-core-week-days.bci-core-selected.bci-core-range:not(.bci-core-first-date-range):not(.bci-core-last-date-range).bci-core-last-week-day::before{width:32px}.bci-core-weekdays-table .bci-core-empty{visibility:hidden}.bci-core-weekdays-table .bci-core-empty:hover{cursor:not-allowed}";

const Weekdays = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.singleDateSelected = index.createEvent(this, "singleDateSelected", 7);
    this.rangeDateSelected = index.createEvent(this, "rangeDateSelected", 7);
  }
  singleDateSelectedHandler(input) {
    this.singleDateSelected.emit(input);
  }
  rangeDateSelectedHandler(input) {
    this.rangeDateSelected.emit(input);
  }
  render() {
    const cells = [];
    const offset = this.offset % 7;
    for (let blank = 0; blank < offset; blank++) {
      cells.push(this.emptyDateCell());
    }
    for (let day = 1; day <= this.daysInMonth; day++) {
      /** Whether the day is selected  */
      let selected;
      /** Whether the day is within the selected range */
      let selectedRange;
      /** Whether the day is the first day of the selected range */
      let selectedFirstDateRange;
      /** Whether the day is the last day of the selected range */
      let selectedLastDateRange;
      const maxDate = this.dateRestrictionObj.maximumDate;
      const minDate = this.dateRestrictionObj.minimumDate;
      const currentDate = this.timeZone === undefined
        ? utils.createDate(new Date(), this.year, this.month, day)
        : utils.createDate(new Date(new Date().toLocaleString('en-US', { timeZone: this.timeZone })), this.year, this.month, day);
      const todaysDate = utils.isSameDay(currentDate, parse(this.todaysDate, this.formatDate, new Date()));
      // disabled cells
      if ((!utils.isSameDay(minDate, currentDate) && isAfter(minDate, currentDate)) ||
        (!utils.isSameDay(maxDate, currentDate) && isBefore(maxDate, currentDate))) {
        cells.push(index.h("td", { class: {
            'bci-core-disabled': true,
            'bci-core-today': todaysDate,
          } }, index.h("div", { class: "bci-core-date" }, day)));
        continue;
      }
      // clickable cells
      if (this.datesObj.startDate && this.datesObj.endDate) {
        // if startDate and endDate are selected
        const startDate = parse(this.datesObj.startDate, this.formatDate, new Date());
        const endDate = parse(this.datesObj.endDate, this.formatDate, new Date());
        if (isAfter(endDate, startDate) && isWithinInterval(currentDate, { start: startDate, end: endDate })) {
          selected = true;
          selectedRange = true;
        }
        else {
          selected = false;
        }
        if (utils.isSameDay(currentDate, startDate)) {
          selectedFirstDateRange = true;
          selected = true;
        }
        if (utils.isSameDay(currentDate, endDate)) {
          selectedLastDateRange = true;
          selected = true;
          selectedRange = true;
        }
        if (utils.isSameDay(currentDate, startDate) && utils.isSameDay(startDate, endDate)) {
          selected = true;
          selectedFirstDateRange = false;
          selectedLastDateRange = false;
          selectedRange = false;
        }
      }
      else {
        // single date picker
        if (this.datesObj.startDate || this.selectedDate) {
          const selectedDate = parse(this.datesObj.startDate || this.selectedDate, this.formatDate, new Date());
          selected = getDate(selectedDate) === day && getMonth(selectedDate) === this.month && getYear(selectedDate) === this.year;
        }
      }
      const classes = {
        'bci-core-week-days': true,
        'bci-core-selected': selected,
        'bci-core-range': selectedRange,
        'bci-core-first-date-range': selectedFirstDateRange,
        'bci-core-last-date-range': selectedLastDateRange,
        'bci-core-today': selected ? false : todaysDate,
        'bci-core-last-week-day': (cells.length + 1) % 7 === 0,
      };
      cells.push(index.h("td", { "data-test": "webcore.webcomponents.weekdays.bci-weekdays.selected-day", class: classes, onClick: () => this.rangeDate ? this.rangeDateSelectedHandler({ selectedDay: day }) : this.singleDateSelectedHandler({ selectedDay: day }) }, index.h("div", { class: "bci-core-date" }, day)));
    }
    while (cells.length % 7 !== 0) {
      cells.push(this.emptyDateCell());
    }
    const table = cells.reduce((acc, curr, index) => {
      const weekNumber = Math.floor(index / 7);
      acc[weekNumber].push(curr);
      return acc;
    }, [[], [], [], [], [], []]);
    return (index.h("table", { class: "bci-core-weekdays-table" }, index.h("tbody", { class: "bci-core-week-days-container" }, table.sort().map((row, index$1) => (index.h("tr", { key: index$1 }, row.map((cell) => cell)))))));
  }
  emptyDateCell() {
    return (index.h("td", { class: "bci-core-empty" }, index.h("div", { class: "bci-core-date" })));
  }
};
Weekdays.style = weekdaysCss;

const datepickerCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:var(--bci-global-font-family);font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bosch-ic,.Bosch-Ic{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic:before,.Bosch-Ic:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-large,.Bosch-Ic-Large{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-large:before,.Bosch-Ic-Large:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:28px;vertical-align:middle}.bosch-ic-medium,.Bosch-Ic-Medium{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-medium:before,.Bosch-Ic-Medium:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-normal,.Bosch-Ic-Normal{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-normal:before,.Bosch-Ic-Normal:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:20px;vertical-align:middle}.bosch-ic-small,.Bosch-Ic-Small{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-small:before,.Bosch-Ic-Small:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:16px;vertical-align:middle}.Bosch-Ic-Disabled,.bosch-ic-disabled,.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic[disabled],.bosch-ic[disabled],.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover{cursor:not-allowed;color:#d0d4d8}.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic-Disabled:hover:hover,.bosch-ic-disabled:hover:hover,.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover,.Bosch-Ic[disabled]:hover:hover,.bosch-ic[disabled]:hover:hover{color:#d0d4d8}.bosch-ic-clickable,.Bosch-Ic-Clickable{cursor:pointer}.bosch-ic-clickable:disabled,.bosch-ic-clickable.disabled,.Bosch-Ic-Clickable:disabled,.Bosch-Ic-Clickable.disabled{color:#a4abb3;cursor:not-allowed}.bosch-ic-activated,.Bosch-Ic-Activated{color:#007bc0}.bosch-ic-3d-print:before{content:\"\\e085\"}.bosch-ic-a-to-b:before{content:\"\\e086\"}.bosch-ic-add:before{content:\"\\e087\"}.bosch-ic-address-consumer-data:before{content:\"\\e088\"}.bosch-ic-adress-book:before{content:\"\\e089\"}.bosch-ic-agility:before{content:\"\\e08a\"}.bosch-ic-alarm-clock:before{content:\"\\e08b\"}.bosch-ic-ambulance:before{content:\"\\e08c\"}.bosch-ic-analysis:before{content:\"\\e08d\"}.bosch-ic-anchor:before{content:\"\\e08e\"}.bosch-ic-arrow-down:before{content:\"\\e08f\"}.bosch-ic-arrow-left:before{content:\"\\e090\"}.bosch-ic-arrow-left-down:before{content:\"\\e091\"}.bosch-ic-arrow-left-up:before{content:\"\\e092\"}.bosch-ic-arrow-right:before{content:\"\\e093\"}.bosch-ic-arrow-right-down:before{content:\"\\e094\"}.bosch-ic-arrow-right-up:before{content:\"\\e095\"}.bosch-ic-arrow-turn-left:before{content:\"\\e096\"}.bosch-ic-arrow-turn-right:before{content:\"\\e097\"}.bosch-ic-arrow-up:before{content:\"\\e098\"}.bosch-ic-arrows:before{content:\"\\e099\"}.bosch-ic-arrows-x-y-z:before{content:\"\\e09a\"}.bosch-ic-artificial-intelligence:before{content:\"\\e09b\"}.bosch-ic-assembly-line:before{content:\"\\e09c\"}.bosch-ic-atom:before{content:\"\\e09d\"}.bosch-ic-augmented-reality:before{content:\"\\e09e\"}.bosch-ic-autobod:before{content:\"\\e09f\"}.bosch-ic-back-left:before{content:\"\\e0a0\"}.bosch-ic-back-left-double:before{content:\"\\e0a1\"}.bosch-ic-bar-chart:before{content:\"\\e0a2\"}.bosch-ic-bar-chart-double:before{content:\"\\e0a3\"}.bosch-ic-battery-0:before{content:\"\\e0a4\"}.bosch-ic-battery-1:before{content:\"\\e0a5\"}.bosch-ic-battery-2:before{content:\"\\e0a6\"}.bosch-ic-battery-3:before{content:\"\\e0a7\"}.bosch-ic-battery-4:before{content:\"\\e0a8\"}.bosch-ic-battery-fail:before{content:\"\\e0a9\"}.bosch-ic-battery-li-ion:before{content:\"\\e0aa\"}.bosch-ic-battery-loading:before{content:\"\\e0ab\"}.bosch-ic-bed:before{content:\"\\e0ac\"}.bosch-ic-bicycle:before{content:\"\\e0ad\"}.bosch-ic-bicycle-e:before{content:\"\\e0ae\"}.bosch-ic-blog:before{content:\"\\e0af\"}.bosch-ic-bluetooth:before{content:\"\\e0b0\"}.bosch-ic-bluetooth-le:before{content:\"\\e0b1\"}.bosch-ic-board-speaker:before{content:\"\\e0b2\"}.bosch-ic-boiler-connected:before{content:\"\\e0b3\"}.bosch-ic-book:before{content:\"\\e0b4\"}.bosch-ic-bookmark:before{content:\"\\e0b5\"}.bosch-ic-bookmark-add:before{content:\"\\e0b6\"}.bosch-ic-bookmark-check:before{content:\"\\e0b7\"}.bosch-ic-bookmark-delete:before{content:\"\\e0b8\"}.bosch-ic-books:before{content:\"\\e0b9\"}.bosch-ic-bookshelf:before{content:\"\\e0ba\"}.bosch-ic-box:before{content:\"\\e0bb\"}.bosch-ic-box-closed:before{content:\"\\e0bc\"}.bosch-ic-box-delivery:before{content:\"\\e0bd\"}.bosch-ic-box-delivery-connected:before{content:\"\\e0be\"}.bosch-ic-brake-disk:before{content:\"\\e0bf\"}.bosch-ic-briefcase:before{content:\"\\e0c0\"}.bosch-ic-broom:before{content:\"\\e0c1\"}.bosch-ic-broom-cloud-dirt:before{content:\"\\e0c2\"}.bosch-ic-brush:before{content:\"\\e0c3\"}.bosch-ic-bug:before{content:\"\\e0c4\"}.bosch-ic-building:before{content:\"\\e0c5\"}.bosch-ic-cactus:before{content:\"\\e0c6\"}.bosch-ic-calculate:before{content:\"\\e0c7\"}.bosch-ic-calendar:before{content:\"\\e0c8\"}.bosch-ic-calendar-add:before{content:\"\\e0c9\"}.bosch-ic-calendar-settings:before{content:\"\\e0ca\"}.bosch-ic-call:before{content:\"\\e0cb\"}.bosch-ic-call-add:before{content:\"\\e0cc\"}.bosch-ic-call-remove:before{content:\"\\e0cd\"}.bosch-ic-call-sos:before{content:\"\\e0ce\"}.bosch-ic-call-team:before{content:\"\\e0cf\"}.bosch-ic-call-wifi:before{content:\"\\e0d0\"}.bosch-ic-camera:before{content:\"\\e0d1\"}.bosch-ic-camera-switch:before{content:\"\\e0d2\"}.bosch-ic-car:before{content:\"\\e0d3\"}.bosch-ic-car-clock:before{content:\"\\e0d4\"}.bosch-ic-car-connected:before{content:\"\\e0d5\"}.bosch-ic-car-locator:before{content:\"\\e0d6\"}.bosch-ic-car-mechanic:before{content:\"\\e0d7\"}.bosch-ic-car-rental:before{content:\"\\e0d8\"}.bosch-ic-car-seat-connected:before{content:\"\\e0d9\"}.bosch-ic-car-side:before{content:\"\\e0da\"}.bosch-ic-car-side-user:before{content:\"\\e0db\"}.bosch-ic-cart:before{content:\"\\e0dc\"}.bosch-ic-certificate:before{content:\"\\e0dd\"}.bosch-ic-certificate-pki:before{content:\"\\e0de\"}.bosch-ic-change:before{content:\"\\e0df\"}.bosch-ic-chart-bar:before{content:\"\\e0e0\"}.bosch-ic-chart-check:before{content:\"\\e0e1\"}.bosch-ic-chart-dummy:before{content:\"\\e0e2\"}.bosch-ic-chart-line:before{content:\"\\e0e3\"}.bosch-ic-chat:before{content:\"\\e0e4\"}.bosch-ic-chat-add:before{content:\"\\e0e5\"}.bosch-ic-chat-question-answer:before{content:\"\\e0e6\"}.bosch-ic-chat-remove:before{content:\"\\e0e7\"}.bosch-ic-checklist:before{content:\"\\e0e8\"}.bosch-ic-checkmark:before{content:\"\\e0e9\"}.bosch-ic-chip:before{content:\"\\e0ea\"}.bosch-ic-circle-segment:before{content:\"\\e0eb\"}.bosch-ic-clear-all:before{content:\"\\e0ec\"}.bosch-ic-clipboard:before{content:\"\\e0ed\"}.bosch-ic-clipboard-list:before{content:\"\\e0ee\"}.bosch-ic-clock:before{content:\"\\e0ef\"}.bosch-ic-close:before{content:\"\\e0f0\"}.bosch-ic-close-all:before{content:\"\\e0f1\"}.bosch-ic-cloud:before{content:\"\\e0f2\"}.bosch-ic-cloud-co2:before{content:\"\\e0f3\"}.bosch-ic-cloud-download:before{content:\"\\e0f4\"}.bosch-ic-cloud-nox:before{content:\"\\e0f5\"}.bosch-ic-cloud-upload:before{content:\"\\e0f6\"}.bosch-ic-club-arm-chair:before{content:\"\\e0f7\"}.bosch-ic-coffee-break:before{content:\"\\e0f8\"}.bosch-ic-coin-currency:before{content:\"\\e0f9\"}.bosch-ic-coin-dollar:before{content:\"\\e0fa\"}.bosch-ic-colorpicker:before{content:\"\\e0fb\"}.bosch-ic-colors:before{content:\"\\e0fc\"}.bosch-ic-command:before{content:\"\\e0fd\"}.bosch-ic-communicator:before{content:\"\\e0fe\"}.bosch-ic-compare:before{content:\"\\e0ff\"}.bosch-ic-components:before{content:\"\\e100\"}.bosch-ic-components-available:before{content:\"\\e101\"}.bosch-ic-components-reservation:before{content:\"\\e102\"}.bosch-ic-concierge-bell:before{content:\"\\e103\"}.bosch-ic-configuration:before{content:\"\\e104\"}.bosch-ic-confirmation:before{content:\"\\e105\"}.bosch-ic-connection-off:before{content:\"\\e106\"}.bosch-ic-connection-on:before{content:\"\\e107\"}.bosch-ic-connectivity:before{content:\"\\e108\"}.bosch-ic-construction:before{content:\"\\e109\"}.bosch-ic-consumer-goods:before{content:\"\\e10a\"}.bosch-ic-contract:before{content:\"\\e10b\"}.bosch-ic-copy:before{content:\"\\e10c\"}.bosch-ic-cube:before{content:\"\\e10d\"}.bosch-ic-cube-arrows:before{content:\"\\e10e\"}.bosch-ic-cube-filled:before{content:\"\\e10f\"}.bosch-ic-cube-shock:before{content:\"\\e110\"}.bosch-ic-cube-stacked:before{content:\"\\e111\"}.bosch-ic-customerservice:before{content:\"\\e112\"}.bosch-ic-cutlery:before{content:\"\\e113\"}.bosch-ic-damper:before{content:\"\\e114\"}.bosch-ic-danger-flash:before{content:\"\\e115\"}.bosch-ic-data-loss:before{content:\"\\e116\"}.bosch-ic-dealer-details:before{content:\"\\e117\"}.bosch-ic-delete:before{content:\"\\e118\"}.bosch-ic-delivery:before{content:\"\\e119\"}.bosch-ic-denied:before{content:\"\\e11a\"}.bosch-ic-desktop:before{content:\"\\e11b\"}.bosch-ic-desktop-application-2d:before{content:\"\\e11c\"}.bosch-ic-desktop-application-3d:before{content:\"\\e11d\"}.bosch-ic-development:before{content:\"\\e11e\"}.bosch-ic-devices:before{content:\"\\e11f\"}.bosch-ic-devices-mobile:before{content:\"\\e120\"}.bosch-ic-directory:before{content:\"\\e121\"}.bosch-ic-dna:before{content:\"\\e122\"}.bosch-ic-do-not-disturb:before{content:\"\\e123\"}.bosch-ic-document:before{content:\"\\e124\"}.bosch-ic-document-add:before{content:\"\\e125\"}.bosch-ic-document-arrow-down:before{content:\"\\e126\"}.bosch-ic-document-arrow-up:before{content:\"\\e127\"}.bosch-ic-document-audio:before{content:\"\\e128\"}.bosch-ic-document-check:before{content:\"\\e129\"}.bosch-ic-document-cloud:before{content:\"\\e12a\"}.bosch-ic-document-copy:before{content:\"\\e12b\"}.bosch-ic-document-delete:before{content:\"\\e12c\"}.bosch-ic-document-doc:before{content:\"\\e12d\"}.bosch-ic-document-edit:before{content:\"\\e12e\"}.bosch-ic-document-error:before{content:\"\\e12f\"}.bosch-ic-document-locked:before{content:\"\\e130\"}.bosch-ic-document-log:before{content:\"\\e131\"}.bosch-ic-document-one:before{content:\"\\e132\"}.bosch-ic-document-pdf:before{content:\"\\e133\"}.bosch-ic-document-plain:before{content:\"\\e134\"}.bosch-ic-document-plain-add:before{content:\"\\e135\"}.bosch-ic-document-plain-delete:before{content:\"\\e136\"}.bosch-ic-document-ppt:before{content:\"\\e137\"}.bosch-ic-document-pub:before{content:\"\\e138\"}.bosch-ic-document-rtf:before{content:\"\\e139\"}.bosch-ic-document-save-to:before{content:\"\\e13a\"}.bosch-ic-document-search:before{content:\"\\e13b\"}.bosch-ic-document-settings:before{content:\"\\e13c\"}.bosch-ic-document-test:before{content:\"\\e13d\"}.bosch-ic-document-text:before{content:\"\\e13e\"}.bosch-ic-document-txt:before{content:\"\\e13f\"}.bosch-ic-document-vsd:before{content:\"\\e140\"}.bosch-ic-document-xls:before{content:\"\\e141\"}.bosch-ic-document-xml:before{content:\"\\e142\"}.bosch-ic-document-zip:before{content:\"\\e143\"}.bosch-ic-dome:before{content:\"\\e144\"}.bosch-ic-door:before{content:\"\\e145\"}.bosch-ic-door-sensor:before{content:\"\\e146\"}.bosch-ic-down:before{content:\"\\e147\"}.bosch-ic-down-double:before{content:\"\\e148\"}.bosch-ic-download:before{content:\"\\e149\"}.bosch-ic-drag-handle:before{content:\"\\e14a\"}.bosch-ic-dragdrop:before{content:\"\\e14b\"}.bosch-ic-drop:before{content:\"\\e14c\"}.bosch-ic-e-charging:before{content:\"\\e14d\"}.bosch-ic-e-mobility:before{content:\"\\e14e\"}.bosch-ic-ear-off-disabled-light:before{content:\"\\e14f\"}.bosch-ic-ear-on:before{content:\"\\e150\"}.bosch-ic-eco-system:before{content:\"\\e151\"}.bosch-ic-edit:before{content:\"\\e152\"}.bosch-ic-education:before{content:\"\\e153\"}.bosch-ic-efficiency:before{content:\"\\e154\"}.bosch-ic-elevator:before{content:\"\\e155\"}.bosch-ic-elevator-alarm:before{content:\"\\e156\"}.bosch-ic-elevator-cloud:before{content:\"\\e157\"}.bosch-ic-elevator-headset:before{content:\"\\e158\"}.bosch-ic-elevator-service:before{content:\"\\e159\"}.bosch-ic-emergency-exit:before{content:\"\\e15a\"}.bosch-ic-emoji-happy:before{content:\"\\e15b\"}.bosch-ic-emoji-neutral:before{content:\"\\e15c\"}.bosch-ic-emoji-sad:before{content:\"\\e15d\"}.bosch-ic-emoji-super-happy:before{content:\"\\e15e\"}.bosch-ic-emoji-very-sad:before{content:\"\\e15f\"}.bosch-ic-eu-energy-label:before{content:\"\\e160\"}.bosch-ic-excavator:before{content:\"\\e161\"}.bosch-ic-exit:before{content:\"\\e163\"}.bosch-ic-expansion-arrows:before{content:\"\\e164\"}.bosch-ic-explosion:before{content:\"\\e165\"}.bosch-ic-export:before{content:\"\\e166\"}.bosch-ic-externallink:before{content:\"\\e167\"}.bosch-ic-fast-forward:before{content:\"\\e168\"}.bosch-ic-faucet:before{content:\"\\e169\"}.bosch-ic-favorites:before{content:\"\\e16a\"}.bosch-ic-fax:before{content:\"\\e16b\"}.bosch-ic-female:before{content:\"\\e16c\"}.bosch-ic-film:before{content:\"\\e16d\"}.bosch-ic-filter:before{content:\"\\e16e\"}.bosch-ic-filter-success:before{content:\"\\e16f\"}.bosch-ic-fingerprint:before{content:\"\\e170\"}.bosch-ic-fire:before{content:\"\\e171\"}.bosch-ic-fire-emergency:before{content:\"\\e172\"}.bosch-ic-fireworks:before{content:\"\\e173\"}.bosch-ic-first-aid:before{content:\"\\e174\"}.bosch-ic-first-aid-cross:before{content:\"\\e175\"}.bosch-ic-fit-to-sceen:before{content:\"\\e176\"}.bosch-ic-flag:before{content:\"\\e177\"}.bosch-ic-flash:before{content:\"\\e178\"}.bosch-ic-flask:before{content:\"\\e179\"}.bosch-ic-flexpress:before{content:\"\\e17a\"}.bosch-ic-folder:before{content:\"\\e17b\"}.bosch-ic-folder-download:before{content:\"\\e17c\"}.bosch-ic-folder-open:before{content:\"\\e17d\"}.bosch-ic-folder-upload:before{content:\"\\e17e\"}.bosch-ic-fork-lift:before{content:\"\\e17f\"}.bosch-ic-fork-lift-locator:before{content:\"\\e180\"}.bosch-ic-forward-right:before{content:\"\\e181\"}.bosch-ic-forward-right-double:before{content:\"\\e182\"}.bosch-ic-full-empty:before{content:\"\\e183\"}.bosch-ic-fullscreen:before{content:\"\\e184\"}.bosch-ic-fullscreen-arrows:before{content:\"\\e185\"}.bosch-ic-fullscreen-exit:before{content:\"\\e186\"}.bosch-ic-g-arrow-down:before{content:\"\\e187\"}.bosch-ic-g-arrow-up:before{content:\"\\e188\"}.bosch-ic-generator:before{content:\"\\e189\"}.bosch-ic-gift:before{content:\"\\e18a\"}.bosch-ic-glance:before{content:\"\\e18b\"}.bosch-ic-glas-plate:before{content:\"\\e18c\"}.bosch-ic-globe:before{content:\"\\e18d\"}.bosch-ic-globe-arrow:before{content:\"\\e18e\"}.bosch-ic-glossary:before{content:\"\\e18f\"}.bosch-ic-grid-view:before{content:\"\\e190\"}.bosch-ic-hammer:before{content:\"\\e191\"}.bosch-ic-hand:before{content:\"\\e192\"}.bosch-ic-hand-motion:before{content:\"\\e193\"}.bosch-ic-hand-motion-off-disabled-light:before{content:\"\\e194\"}.bosch-ic-handlewithcare:before{content:\"\\e195\"}.bosch-ic-handover-add:before{content:\"\\e196\"}.bosch-ic-handover-check:before{content:\"\\e197\"}.bosch-ic-handover-remove:before{content:\"\\e198\"}.bosch-ic-hanger:before{content:\"\\e199\"}.bosch-ic-health:before{content:\"\\e19a\"}.bosch-ic-heart:before{content:\"\\e19b\"}.bosch-ic-history:before{content:\"\\e19c\"}.bosch-ic-home:before{content:\"\\e19d\"}.bosch-ic-home-locator:before{content:\"\\e19e\"}.bosch-ic-hourglass:before{content:\"\\e19f\"}.bosch-ic-hydrant:before{content:\"\\e1a0\"}.bosch-ic-ice:before{content:\"\\e1a1\"}.bosch-ic-imagery:before{content:\"\\e1a2\"}.bosch-ic-imagery-add:before{content:\"\\e1a3\"}.bosch-ic-imagery-remove:before{content:\"\\e1a4\"}.bosch-ic-import:before{content:\"\\e1a5\"}.bosch-ic-imprint:before{content:\"\\e1a6\"}.bosch-ic-impulse:before{content:\"\\e1a7\"}.bosch-ic-industry:before{content:\"\\e1a8\"}.bosch-ic-industry-clock:before{content:\"\\e1a9\"}.bosch-ic-industry-connected:before{content:\"\\e1aa\"}.bosch-ic-info:before{content:\"\\e1ab\"}.bosch-ic-info-i:before{content:\"\\e1ac\"}.bosch-ic-interval:before{content:\"\\e1ad\"}.bosch-ic-it-device:before{content:\"\\e1ae\"}.bosch-ic-jewel:before{content:\"\\e1af\"}.bosch-ic-keyboard:before{content:\"\\e1b0\"}.bosch-ic-label:before{content:\"\\e1b1\"}.bosch-ic-laptop:before{content:\"\\e1b2\"}.bosch-ic-laser:before{content:\"\\e1b3\"}.bosch-ic-layout:before{content:\"\\e1b4\"}.bosch-ic-leaf:before{content:\"\\e1b5\"}.bosch-ic-led:before{content:\"\\e1b6\"}.bosch-ic-brick:before{content:\"\\e1b7\"}.bosch-ic-brick-slanting-1:before{content:\"\\e1b8\"}.bosch-ic-brick-slanting-2:before{content:\"\\e1b9\"}.bosch-ic-less-minimize:before{content:\"\\e1ba\"}.bosch-ic-lightbulb:before{content:\"\\e1bb\"}.bosch-ic-lightbulb-off:before{content:\"\\e1bc\"}.bosch-ic-line-chart:before{content:\"\\e1bd\"}.bosch-ic-link:before{content:\"\\e1be\"}.bosch-ic-link-broken:before{content:\"\\e1bf\"}.bosch-ic-list-view:before{content:\"\\e1c0\"}.bosch-ic-list-view-mobile:before{content:\"\\e1c1\"}.bosch-ic-livechat:before{content:\"\\e1c2\"}.bosch-ic-locator:before{content:\"\\e1c3\"}.bosch-ic-locator-ip-connected:before{content:\"\\e1c4\"}.bosch-ic-locator-ip-disconnected:before{content:\"\\e1c5\"}.bosch-ic-locator-spot:before{content:\"\\e1c6\"}.bosch-ic-locator-spot-check:before{content:\"\\e1c7\"}.bosch-ic-lock-closed:before{content:\"\\e1c8\"}.bosch-ic-lock-open:before{content:\"\\e1c9\"}.bosch-ic-login:before{content:\"\\e1ca\"}.bosch-ic-logistics:before{content:\"\\e1cb\"}.bosch-ic-logout:before{content:\"\\e1cc\"}.bosch-ic-machine:before{content:\"\\e1cd\"}.bosch-ic-magnet:before{content:\"\\e1ce\"}.bosch-ic-mail:before{content:\"\\e1cf\"}.bosch-ic-mail-forward:before{content:\"\\e1d0\"}.bosch-ic-mail-open:before{content:\"\\e1d1\"}.bosch-ic-mail-out:before{content:\"\\e1d2\"}.bosch-ic-mail-reply:before{content:\"\\e1d3\"}.bosch-ic-male:before{content:\"\\e1d4\"}.bosch-ic-map:before{content:\"\\e1d5\"}.bosch-ic-material-add:before{content:\"\\e1d6\"}.bosch-ic-material-check:before{content:\"\\e1d7\"}.bosch-ic-material-remove:before{content:\"\\e1d8\"}.bosch-ic-maximize:before{content:\"\\e1d9\"}.bosch-ic-mechanic:before{content:\"\\e1da\"}.bosch-ic-megaphone:before{content:\"\\e1db\"}.bosch-ic-message:before{content:\"\\e1dc\"}.bosch-ic-mic:before{content:\"\\e1dd\"}.bosch-ic-microphone:before{content:\"\\e1de\"}.bosch-ic-microphone-classic:before{content:\"\\e1df\"}.bosch-ic-minimize:before{content:\"\\e1e0\"}.bosch-ic-money-currency:before{content:\"\\e1e1\"}.bosch-ic-money-dollar:before{content:\"\\e1e2\"}.bosch-ic-moneybag-currency:before{content:\"\\e1e3\"}.bosch-ic-moneybag-dollar:before{content:\"\\e1e4\"}.bosch-ic-monument:before{content:\"\\e1e5\"}.bosch-ic-moon:before{content:\"\\e1e6\"}.bosch-ic-motorcycle:before{content:\"\\e1e7\"}.bosch-ic-motorcycle-side-car:before{content:\"\\e1e8\"}.bosch-ic-motorcycle-side:before{content:\"\\e1e9\"}.bosch-ic-mouse:before{content:\"\\e1ea\"}.bosch-ic-mouse-left:before{content:\"\\e1eb\"}.bosch-ic-mouse-right:before{content:\"\\e1ec\"}.bosch-ic-mouth:before{content:\"\\e1ed\"}.bosch-ic-my-product:before{content:\"\\e1ee\"}.bosch-ic-new-way-work:before{content:\"\\e1ef\"}.bosch-ic-newsletter:before{content:\"\\e1f0\"}.bosch-ic-newspaper:before{content:\"\\e1f1\"}.bosch-ic-nose:before{content:\"\\e1f2\"}.bosch-ic-notepad:before{content:\"\\e1f3\"}.bosch-ic-notepad-edit:before{content:\"\\e1f4\"}.bosch-ic-notification:before{content:\"\\e1f5\"}.bosch-ic-notification-active:before{content:\"\\e1f6\"}.bosch-ic-notification-add:before{content:\"\\e1f7\"}.bosch-ic-notification-remove:before{content:\"\\e1f8\"}.bosch-ic-oil-car:before{content:\"\\e1f9\"}.bosch-ic-omnichannel:before{content:\"\\e1fa\"}.bosch-ic-options:before{content:\"\\e1fb\"}.bosch-ic-origami-boat:before{content:\"\\e1fc\"}.bosch-ic-pallete-car:before{content:\"\\e1fd\"}.bosch-ic-pallete-car-connected:before{content:\"\\e1fe\"}.bosch-ic-panel:before{content:\"\\e1ff\"}.bosch-ic-paperclip:before{content:\"\\e200\"}.bosch-ic-paperplane:before{content:\"\\e201\"}.bosch-ic-parking:before{content:\"\\e202\"}.bosch-ic-pause:before{content:\"\\e203\"}.bosch-ic-payment:before{content:\"\\e204\"}.bosch-ic-people:before{content:\"\\e205\"}.bosch-ic-petrol-station:before{content:\"\\e206\"}.bosch-ic-piggybank:before{content:\"\\e207\"}.bosch-ic-pin-classic:before{content:\"\\e208\"}.bosch-ic-pin-modern:before{content:\"\\e209\"}.bosch-ic-plane-side:before{content:\"\\e20a\"}.bosch-ic-play:before{content:\"\\e20b\"}.bosch-ic-plug-12v:before{content:\"\\e20c\"}.bosch-ic-podium-speaker:before{content:\"\\e20d\"}.bosch-ic-police:before{content:\"\\e20e\"}.bosch-ic-poop:before{content:\"\\e20f\"}.bosch-ic-post-it:before{content:\"\\e210\"}.bosch-ic-postal-code:before{content:\"\\e211\"}.bosch-ic-power:before{content:\"\\e212\"}.bosch-ic-print:before{content:\"\\e213\"}.bosch-ic-prototyping:before{content:\"\\e214\"}.bosch-ic-puzzle:before{content:\"\\e215\"}.bosch-ic-quad:before{content:\"\\e216\"}.bosch-ic-question:before{content:\"\\e217\"}.bosch-ic-radar:before{content:\"\\e218\"}.bosch-ic-radio:before{content:\"\\e219\"}.bosch-ic-radiotower:before{content:\"\\e21a\"}.bosch-ic-redirect:before{content:\"\\e21b\"}.bosch-ic-redo:before{content:\"\\e21c\"}.bosch-ic-reference:before{content:\"\\e21d\"}.bosch-ic-refresh:before{content:\"\\e21e\"}.bosch-ic-refresh-cloud:before{content:\"\\e21f\"}.bosch-ic-registration:before{content:\"\\e220\"}.bosch-ic-remote:before{content:\"\\e221\"}.bosch-ic-reset:before{content:\"\\e222\"}.bosch-ic-resolution:before{content:\"\\e223\"}.bosch-ic-robot:before{content:\"\\e224\"}.bosch-ic-robot-connected:before{content:\"\\e225\"}.bosch-ic-robothead:before{content:\"\\e226\"}.bosch-ic-rocket:before{content:\"\\e227\"}.bosch-ic-rotation:before{content:\"\\e228\"}.bosch-ic-rotation-360:before{content:\"\\e229\"}.bosch-ic-rotation-x-left:before{content:\"\\e22a\"}.bosch-ic-rotation-x-right:before{content:\"\\e22b\"}.bosch-ic-rotation-y-down:before{content:\"\\e22c\"}.bosch-ic-rotation-y-up:before{content:\"\\e22d\"}.bosch-ic-route:before{content:\"\\e22e\"}.bosch-ic-ruler:before{content:\"\\e22f\"}.bosch-ic-ruler-pen:before{content:\"\\e230\"}.bosch-ic-satellite:before{content:\"\\e231\"}.bosch-ic-save:before{content:\"\\e232\"}.bosch-ic-scale:before{content:\"\\e233\"}.bosch-ic-scissors:before{content:\"\\e234\"}.bosch-ic-search:before{content:\"\\e235\"}.bosch-ic-security:before{content:\"\\e236\"}.bosch-ic-security-check:before{content:\"\\e237\"}.bosch-ic-security-user:before{content:\"\\e238\"}.bosch-ic-sensor:before{content:\"\\e239\"}.bosch-ic-server:before{content:\"\\e23a\"}.bosch-ic-server-expansion:before{content:\"\\e23b\"}.bosch-ic-server-rate:before{content:\"\\e23c\"}.bosch-ic-service-agent:before{content:\"\\e23d\"}.bosch-ic-service-time:before{content:\"\\e23e\"}.bosch-ic-settings:before{content:\"\\e23f\"}.bosch-ic-settings-arrows:before{content:\"\\e240\"}.bosch-ic-settings-connected:before{content:\"\\e241\"}.bosch-ic-share:before{content:\"\\e242\"}.bosch-ic-shield-stop:before{content:\"\\e243\"}.bosch-ic-ship:before{content:\"\\e244\"}.bosch-ic-ship-side:before{content:\"\\e245\"}.bosch-ic-shoppingcart:before{content:\"\\e246\"}.bosch-ic-shoppingcart-add:before{content:\"\\e247\"}.bosch-ic-shoppingcart-switch:before{content:\"\\e248\"}.bosch-ic-signal-full:before{content:\"\\e249\"}.bosch-ic-signal-half:before{content:\"\\e24a\"}.bosch-ic-signal-lost:before{content:\"\\e24b\"}.bosch-ic-signpost:before{content:\"\\e24c\"}.bosch-ic-simcard:before{content:\"\\e24d\"}.bosch-ic-skyscraper:before{content:\"\\e24e\"}.bosch-ic-smartcity:before{content:\"\\e24f\"}.bosch-ic-smarthome:before{content:\"\\e250\"}.bosch-ic-smartphone:before{content:\"\\e251\"}.bosch-ic-smartphone-acoustic:before{content:\"\\e252\"}.bosch-ic-smartphone-arrow-right:before{content:\"\\e253\"}.bosch-ic-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.bosch-ic-smartphone-clock:before{content:\"\\e255\"}.bosch-ic-smartphone-cloud:before{content:\"\\e256\"}.bosch-ic-smartphone-loading-wheel:before{content:\"\\e257\"}.bosch-ic-smartphone-smashed:before{content:\"\\e258\"}.bosch-ic-smartphone-vibration:before{content:\"\\e259\"}.bosch-ic-smartphone-z:before{content:\"\\e25a\"}.bosch-ic-smartwatch-connected:before{content:\"\\e25b\"}.bosch-ic-spare-parts:before{content:\"\\e25c\"}.bosch-ic-spark-plug:before{content:\"\\e25d\"}.bosch-ic-speaker-acoustic:before{content:\"\\e25e\"}.bosch-ic-speedometer:before{content:\"\\e25f\"}.bosch-ic-stadium:before{content:\"\\e260\"}.bosch-ic-stairs:before{content:\"\\e261\"}.bosch-ic-stamp:before{content:\"\\e262\"}.bosch-ic-standby:before{content:\"\\e263\"}.bosch-ic-steering-wheel:before{content:\"\\e264\"}.bosch-ic-steering-wheel-connected:before{content:\"\\e265\"}.bosch-ic-steering-wheel-hands:before{content:\"\\e266\"}.bosch-ic-steps:before{content:\"\\e267\"}.bosch-ic-stethoscope:before{content:\"\\e268\"}.bosch-ic-stop:before{content:\"\\e269\"}.bosch-ic-stopwatch:before{content:\"\\e26a\"}.bosch-ic-storage-local:before{content:\"\\e26b\"}.bosch-ic-structure:before{content:\"\\e26c\"}.bosch-ic-subtitles:before{content:\"\\e26d\"}.bosch-ic-subtitles-off-light:before{content:\"\\e26e\"}.bosch-ic-summary:before{content:\"\\e26f\"}.bosch-ic-sun:before{content:\"\\e270\"}.bosch-ic-sun-half-filled:before{content:\"\\e271\"}.bosch-ic-sunshade-table:before{content:\"\\e272\"}.bosch-ic-surveillance:before{content:\"\\e273\"}.bosch-ic-table-chairs:before{content:\"\\e274\"}.bosch-ic-tablet:before{content:\"\\e275\"}.bosch-ic-team:before{content:\"\\e276\"}.bosch-ic-team-lightbulb:before{content:\"\\e277\"}.bosch-ic-technical-service:before{content:\"\\e278\"}.bosch-ic-temperature:before{content:\"\\e279\"}.bosch-ic-temperature-high:before{content:\"\\e27a\"}.bosch-ic-temperature-low:before{content:\"\\e27b\"}.bosch-ic-theater:before{content:\"\\e27c\"}.bosch-ic-thumb-down:before{content:\"\\e27d\"}.bosch-ic-thumb-up:before{content:\"\\e27e\"}.bosch-ic-ticket:before{content:\"\\e27f\"}.bosch-ic-touch:before{content:\"\\e280\"}.bosch-ic-touch-sos:before{content:\"\\e281\"}.bosch-ic-towing-truck:before{content:\"\\e282\"}.bosch-ic-train:before{content:\"\\e283\"}.bosch-ic-train-side:before{content:\"\\e284\"}.bosch-ic-transport-movements:before{content:\"\\e285\"}.bosch-ic-transscript:before{content:\"\\e286\"}.bosch-ic-transscript-off:before{content:\"\\e287\"}.bosch-ic-transscript-on:before{content:\"\\e288\"}.bosch-ic-tree:before{content:\"\\e289\"}.bosch-ic-tricycle:before{content:\"\\e28a\"}.bosch-ic-trolley-empty:before{content:\"\\e28b\"}.bosch-ic-trolley-filled-box:before{content:\"\\e28c\"}.bosch-ic-trolly-filled-files:before{content:\"\\e28d\"}.bosch-ic-trophy:before{content:\"\\e28e\"}.bosch-ic-truck:before{content:\"\\e28f\"}.bosch-ic-tuktuk:before{content:\"\\e290\"}.bosch-ic-tune:before{content:\"\\e291\"}.bosch-ic-typography:before{content:\"\\e292\"}.bosch-ic-ui:before{content:\"\\e293\"}.bosch-ic-umbrella:before{content:\"\\e294\"}.bosch-ic-undo:before{content:\"\\e295\"}.bosch-ic-up:before{content:\"\\e296\"}.bosch-ic-up-double:before{content:\"\\e297\"}.bosch-ic-up-down:before{content:\"\\e298\"}.bosch-ic-upload:before{content:\"\\e299\"}.bosch-ic-usb:before{content:\"\\e29a\"}.bosch-ic-user:before{content:\"\\e29b\"}.bosch-ic-user-advanced:before{content:\"\\e29c\"}.bosch-ic-user-beginner:before{content:\"\\e29d\"}.bosch-ic-user-desktop:before{content:\"\\e29e\"}.bosch-ic-user-down:before{content:\"\\e29f\"}.bosch-ic-user-falling:before{content:\"\\e2a0\"}.bosch-ic-user-hand:before{content:\"\\e2a1\"}.bosch-ic-user-hand-stop:before{content:\"\\e2a2\"}.bosch-ic-user-professional:before{content:\"\\e2a3\"}.bosch-ic-user-run:before{content:\"\\e2a4\"}.bosch-ic-user-share:before{content:\"\\e2a5\"}.bosch-ic-user-voice:before{content:\"\\e2a6\"}.bosch-ic-user-walk:before{content:\"\\e2a7\"}.bosch-ic-video:before{content:\"\\e2a8\"}.bosch-ic-video-disabled-light:before{content:\"\\e2a9\"}.bosch-ic-video-record:before{content:\"\\e2aa\"}.bosch-ic-virtual-reality:before{content:\"\\e2ab\"}.bosch-ic-volume-disabled:before{content:\"\\e2ac\"}.bosch-ic-volume-high:before{content:\"\\e2ad\"}.bosch-ic-volume-low:before{content:\"\\e2ae\"}.bosch-ic-volume-off-light:before{content:\"\\e2af\"}.bosch-ic-wand:before{content:\"\\e2b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-watch-off-disabled-light:before{content:\"\\e2b2\"}.bosch-ic-watch-on:before{content:\"\\e2b3\"}.bosch-ic-water-shower:before{content:\"\\e2b4\"}.bosch-ic-welcome:before{content:\"\\e2b5\"}.bosch-ic-wellness:before{content:\"\\e2b6\"}.bosch-ic-wheelchair:before{content:\"\\e2b7\"}.bosch-ic-whistle:before{content:\"\\e2b8\"}.bosch-ic-wide-angled-arrow:before{content:\"\\e2b9\"}.bosch-ic-wifi:before{content:\"\\e2ba\"}.bosch-ic-window-analysis:before{content:\"\\e2bb\"}.bosch-ic-window-browser:before{content:\"\\e2bc\"}.bosch-ic-window-console:before{content:\"\\e2bd\"}.bosch-ic-window-gaussian-view:before{content:\"\\e2be\"}.bosch-ic-window-info:before{content:\"\\e2bf\"}.bosch-ic-window-new:before{content:\"\\e2c0\"}.bosch-ic-window-resize:before{content:\"\\e2c1\"}.bosch-ic-window-scaling-view:before{content:\"\\e2c2\"}.bosch-ic-window-shuffle:before{content:\"\\e2c3\"}.bosch-ic-window-statistic:before{content:\"\\e2c4\"}.bosch-ic-wishlist:before{content:\"\\e2c5\"}.bosch-ic-work-order:before{content:\"\\e2c6\"}.bosch-ic-worldwideweb:before{content:\"\\e2c7\"}.bosch-ic-wrench:before{content:\"\\e2c8\"}.bosch-ic-zoom-in:before{content:\"\\e2c9\"}.bosch-ic-zoom-out:before{content:\"\\e2ca\"}.bosch-ic-scooter:before{content:\"\\e2cb\"}.bosch-ic-cars-traffic:before{content:\"\\e2cc\"}.bosch-ic-arm-chair:before{content:\"\\e2cd\"}.bosch-ic-car-phone:before{content:\"\\e2ce\"}.bosch-ic-box-lightbulb:before{content:\"\\e2cf\"}.bosch-ic-chair-officedesk:before{content:\"\\e2d0\"}.bosch-ic-calendar-clock:before{content:\"\\e2d1\"}.bosch-ic-document-cv:before{content:\"\\e2d2\"}.bosch-ic-user-run-bag:before{content:\"\\e2d3\"}.bosch-ic-brake-disk-spray:before{content:\"\\e2d4\"}.bosch-ic-circle-abs:before{content:\"\\e2d5\"}.bosch-ic-bright-down:before{content:\"\\e2d6\"}.bosch-ic-bright-up:before{content:\"\\e2d7\"}.bosch-ic-fan:before{content:\"\\e2d8\"}.bosch-ic-sun-moon:before{content:\"\\e2d9\"}.bosch-ic-lightbulb-attention:before{content:\"\\e2da\"}.bosch-ic-settings-attention:before{content:\"\\e2db\"}.bosch-ic-danger-hazards:before{content:\"\\e2dc\"}.bosch-ic-windshield-spray:before{content:\"\\e2dd\"}.bosch-ic-windshield-heat:before{content:\"\\e2de\"}.bosch-ic-light-drop:before{content:\"\\e2df\"}.bosch-ic-starter:before{content:\"\\e2e0\"}.bosch-ic-turbo:before{content:\"\\e2e1\"}.bosch-ic-settings-lock:before{content:\"\\e2e2\"}.bosch-ic-settings-n:before{content:\"\\e2e3\"}.bosch-ic-light-beam-high:before{content:\"\\e2e4\"}.bosch-ic-light-beam-low:before{content:\"\\e2e5\"}.bosch-ic-glas:before{content:\"\\e2e6\"}.bosch-ic-shirt-ice:before{content:\"\\e2e7\"}.bosch-ic-helmet-fan:before{content:\"\\e2e8\"}.bosch-ic-notification-arrow:before{content:\"\\e2e9\"}.bosch-ic-steering-wheel-arrow:before{content:\"\\e2ea\"}.bosch-ic-circle-arrow:before{content:\"\\e2eb\"}.bosch-ic-tempomat-pit:before{content:\"\\e2ec\"}.bosch-ic-radiator-fan-spray:before{content:\"\\e2ed\"}.bosch-ic-boost-down:before{content:\"\\e2ee\"}.bosch-ic-boost-up:before{content:\"\\e2ef\"}.bosch-ic-circle-lock:before{content:\"\\e2f0\"}.bosch-ic-mp-down:before{content:\"\\e2f1\"}.bosch-ic-mp-up:before{content:\"\\e2f2\"}.bosch-ic-tc-down:before{content:\"\\e2f3\"}.bosch-ic-tc-up:before{content:\"\\e2f4\"}.bosch-ic-danger-arrow-attention:before{content:\"\\e2f5\"}.bosch-ic-user-remove:before{content:\"\\e2f6\"}.bosch-ic-user-add:before{content:\"\\e2f7\"}.bosch-ic-post-it-collection:before{content:\"\\e2f8\"}.bosch-ic-horn:before{content:\"\\e2f9\"}.bosch-ic-desktop-notification:before{content:\"\\e2fa\"}.bosch-ic-router:before{content:\"\\e2fb\"}.bosch-ic-berry:before{content:\"\\e2fc\"}.bosch-ic-chat-language:before{content:\"\\e2fd\"}.bosch-ic-game-controller:before{content:\"\\e2fe\"}.bosch-ic-user-brain:before{content:\"\\e2ff\"}.bosch-ic-mouth-finger:before{content:\"\\e300\"}.bosch-ic-excavator-shovel:before{content:\"\\e301\"}.bosch-ic-ear-in:before{content:\"\\e302\"}.bosch-ic-ear-out:before{content:\"\\e303\"}.bosch-ic-lion:before{content:\"\\e304\"}.bosch-ic-car-convertible:before{content:\"\\e305\"}.bosch-ic-car-sports:before{content:\"\\e306\"}.bosch-ic-bear:before{content:\"\\e307\"}.bosch-ic-pot:before{content:\"\\e308\"}.bosch-ic-bed-double:before{content:\"\\e309\"}.bosch-ic-air-hot:before{content:\"\\e30a\"}.bosch-ic-air-ice:before{content:\"\\e30b\"}.bosch-ic-air-leaf:before{content:\"\\e30c\"}.bosch-ic-air-purifier-0:before{content:\"\\e30d\"}.bosch-ic-air-purifier-1:before{content:\"\\e30e\"}.bosch-ic-air-purifier-2:before{content:\"\\e30f\"}.bosch-ic-air-purifier-3:before{content:\"\\e310\"}.bosch-ic-air:before{content:\"\\e311\"}.bosch-ic-fan-off-disabled-bold:before{content:\"\\e312\"}.bosch-ic-ice-off-disabled-bold:before{content:\"\\e313\"}.bosch-ic-faucet-boost:before{content:\"\\e314\"}.bosch-ic-faucet-mode:before{content:\"\\e315\"}.bosch-ic-faucet-off-disabled-bold:before{content:\"\\e316\"}.bosch-ic-faucet-schedule:before{content:\"\\e317\"}.bosch-ic-fuel-cell:before{content:\"\\e318\"}.bosch-ic-heating-floor-off-disabled-bold:before{content:\"\\e319\"}.bosch-ic-heating-floor:before{content:\"\\e31a\"}.bosch-ic-heating-radiator-boost:before{content:\"\\e31b\"}.bosch-ic-heating-radiator-mode:before{content:\"\\e31c\"}.bosch-ic-heating-radiator-off-disabled-bold:before{content:\"\\e31d\"}.bosch-ic-heating-radiator-schedule:before{content:\"\\e31e\"}.bosch-ic-leaf-simple:before{content:\"\\e31f\"}.bosch-ic-oil-barrel:before{content:\"\\e320\"}.bosch-ic-water-bathtub:before{content:\"\\e321\"}.bosch-ic-water-dishwash:before{content:\"\\e322\"}.bosch-ic-water-handwash:before{content:\"\\e323\"}.bosch-ic-water-hot:before{content:\"\\e324\"}.bosch-ic-water-ice:before{content:\"\\e325\"}.bosch-ic-water-leaf:before{content:\"\\e326\"}.bosch-ic-water-ladder:before{content:\"\\e327\"}.bosch-ic-water-purifier-0:before{content:\"\\e328\"}.bosch-ic-water-purifier-1:before{content:\"\\e329\"}.bosch-ic-water-purifier-2:before{content:\"\\e32a\"}.bosch-ic-water-purifier-3:before{content:\"\\e32b\"}.bosch-ic-water-shower-boost:before{content:\"\\e32c\"}.bosch-ic-water-shower-mode:before{content:\"\\e32d\"}.bosch-ic-water-shower-off-disabled-bold:before{content:\"\\e32e\"}.bosch-ic-water-shower-schedule:before{content:\"\\e32f\"}.bosch-ic-water:before{content:\"\\e330\"}.bosch-ic-chart-bar-drop:before{content:\"\\e331\"}.bosch-ic-chart-bar-fire:before{content:\"\\e332\"}.bosch-ic-chart-bar-flash:before{content:\"\\e333\"}.bosch-ic-chart-bar-oil:before{content:\"\\e334\"}.bosch-ic-drop-minus:before{content:\"\\e335\"}.bosch-ic-drop-plus:before{content:\"\\e336\"}.bosch-ic-fan-minus:before{content:\"\\e337\"}.bosch-ic-fan-plus:before{content:\"\\e338\"}.bosch-ic-home-drops-0:before{content:\"\\e339\"}.bosch-ic-home-drops-1:before{content:\"\\e33a\"}.bosch-ic-home-drops-2:before{content:\"\\e33b\"}.bosch-ic-home-drops-3:before{content:\"\\e33c\"}.bosch-ic-home-temperature-in:before{content:\"\\e33d\"}.bosch-ic-home-temperature-out:before{content:\"\\e33e\"}.bosch-ic-home-user-in:before{content:\"\\e33f\"}.bosch-ic-ice-minus:before{content:\"\\e340\"}.bosch-ic-ice-plus:before{content:\"\\e341\"}.bosch-ic-chimney-sweeper:before{content:\"\\e342\"}.bosch-ic-menu-up:before{content:\"\\e343\"}.bosch-ic-security-ice:before{content:\"\\e344\"}.bosch-ic-security-legionella:before{content:\"\\e345\"}.bosch-ic-trolley-case:before{content:\"\\e346\"}.bosch-ic-sun-minus:before{content:\"\\e347\"}.bosch-ic-sun-plug:before{content:\"\\e348\"}.bosch-ic-sun-plus:before{content:\"\\e349\"}.bosch-ic-touch-pit:before{content:\"\\e34a\"}.bosch-ic-temperature-current:before{content:\"\\e34b\"}.bosch-ic-temperature-set:before{content:\"\\e34c\"}.bosch-ic-teddy-bear:before{content:\"\\e34d\"}.bosch-ic-document-svg:before{content:\"\\e34e\"}.bosch-ic-back-menu:before{content:\"\\e34f\"}.bosch-ic-car-arrows:before{content:\"\\e350\"}.bosch-ic-wrench-cloud:before{content:\"\\e351\"}.bosch-ic-weather-cloud-sun:before{content:\"\\e352\"}.bosch-ic-weather-cloudy:before{content:\"\\e353\"}.bosch-ic-weather-gale:before{content:\"\\e354\"}.bosch-ic-weather-mist:before{content:\"\\e355\"}.bosch-ic-weather-moon-cloud:before{content:\"\\e356\"}.bosch-ic-weather-rain-snow:before{content:\"\\e357\"}.bosch-ic-weather-rain-thunder:before{content:\"\\e358\"}.bosch-ic-weather-rain:before{content:\"\\e359\"}.bosch-ic-weather-snow:before{content:\"\\e35a\"}.bosch-ic-weather-thunder:before{content:\"\\e35b\"}.bosch-ic-weather-wind:before{content:\"\\e35c\"}.bosch-ic-plane-start:before{content:\"\\e35d\"}.bosch-ic-plane-land:before{content:\"\\e35e\"}.bosch-ic-graph-power:before{content:\"\\e35f\"}.bosch-ic-floorplan:before{content:\"\\e360\"}.bosch-ic-clock-pause:before{content:\"\\e361\"}.bosch-ic-clock-start:before{content:\"\\e362\"}.bosch-ic-clock-stop:before{content:\"\\e363\"}.bosch-ic-computer-retro:before{content:\"\\e364\"}.bosch-ic-smartphone-retro:before{content:\"\\e365\"}.bosch-ic-wood:before{content:\"\\e366\"}.bosch-ic-solid-fuel:before{content:\"\\e367\"}.bosch-ic-appliance-fan:before{content:\"\\e368\"}.bosch-ic-appliance-flame:before{content:\"\\e369\"}.bosch-ic-appliance-ice:before{content:\"\\e36a\"}.bosch-ic-appliance-oil:before{content:\"\\e36b\"}.bosch-ic-appliance-solid-fuel:before{content:\"\\e36c\"}.bosch-ic-appliance-sun-ice:before{content:\"\\e36d\"}.bosch-ic-appliance-sun:before{content:\"\\e36e\"}.bosch-ic-appliance-wood:before{content:\"\\e36f\"}.bosch-ic-appliance-air-leaf:before{content:\"\\e370\"}.bosch-ic-appliance-water-air:before{content:\"\\e371\"}.bosch-ic-appliance-water-leaf:before{content:\"\\e372\"}.bosch-ic-appliance-water-oil:before{content:\"\\e373\"}.bosch-ic-appliance-water:before{content:\"\\e374\"}.bosch-ic-appliance-heat-pump-air:before{content:\"\\e375\"}.bosch-ic-appliance-heat-pump-ground:before{content:\"\\e376\"}.bosch-ic-appliance-heat-pump-water:before{content:\"\\e377\"}.bosch-ic-appliance-plug-air:before{content:\"\\e378\"}.bosch-ic-appliance-plug-fuel-cell:before{content:\"\\e379\"}.bosch-ic-appliance-plug-leaf:before{content:\"\\e37a\"}.bosch-ic-appliance-plug-water-air:before{content:\"\\e37b\"}.bosch-ic-thermostat-connected-c:before{content:\"\\e37c\"}.bosch-ic-thermostat-connected-f:before{content:\"\\e37d\"}.bosch-ic-image-gallery:before{content:\"\\e37e\"}.bosch-ic-arrows-caution:before{content:\"\\e37f\"}.bosch-ic-arrows-diagonal-disabled-bold:before{content:\"\\e380\"}.bosch-ic-back-left-end:before{content:\"\\e381\"}.bosch-ic-back-right-end:before{content:\"\\e382\"}.bosch-ic-bar-chart-search:before{content:\"\\e383\"}.bosch-ic-batch:before{content:\"\\e384\"}.bosch-ic-calendar-remove:before{content:\"\\e385\"}.bosch-ic-call-bluetooth:before{content:\"\\e386\"}.bosch-ic-call-deny:before{content:\"\\e387\"}.bosch-ic-call-settings:before{content:\"\\e388\"}.bosch-ic-caution-sign-clock:before{content:\"\\e389\"}.bosch-ic-caution-sign-list:before{content:\"\\e38a\"}.bosch-ic-core-data:before{content:\"\\e38b\"}.bosch-ic-cursor-connected:before{content:\"\\e38c\"}.bosch-ic-cursor:before{content:\"\\e38d\"}.bosch-ic-dmc-code:before{content:\"\\e38e\"}.bosch-ic-find-id:before{content:\"\\e38f\"}.bosch-ic-globe-search:before{content:\"\\e390\"}.bosch-ic-interface:before{content:\"\\e391\"}.bosch-ic-material-add-manually:before{content:\"\\e392\"}.bosch-ic-material-list:before{content:\"\\e393\"}.bosch-ic-material-remove-manually:before{content:\"\\e394\"}.bosch-ic-material:before{content:\"\\e395\"}.bosch-ic-menu-checkmark:before{content:\"\\e396\"}.bosch-ic-qr-code:before{content:\"\\e397\"}.bosch-ic-sort-alphabetically:before{content:\"\\e398\"}.bosch-ic-unpin-classic-disabled-bold:before{content:\"\\e399\"}.bosch-ic-unpin-modern-disabled-bold:before{content:\"\\e39a\"}.bosch-ic-window-search:before{content:\"\\e39b\"}.bosch-ic-cloud-clock:before{content:\"\\e39c\"}.bosch-ic-concierge:before{content:\"\\e39d\"}.bosch-ic-heating-radiator:before{content:\"\\e39e\"}.bosch-ic-home-user-out:before{content:\"\\e39f\"}.bosch-ic-hot:before{content:\"\\e3a0\"}.bosch-ic-screen-service:before{content:\"\\e3a1\"}.bosch-ic-sun-ice:before{content:\"\\e3a2\"}.bosch-ic-sun-off-disabled-bold:before{content:\"\\e3a3\"}.bosch-ic-tractor:before{content:\"\\e3a4\"}.bosch-ic-video-recorder-digital:before{content:\"\\e3a5\"}.bosch-ic-windshield-wiper:before{content:\"\\e3a6\"}.bosch-ic-weather-snow-thunder:before{content:\"\\e3a7\"}.bosch-ic-electricity-pylon:before{content:\"\\e3a8\"}.bosch-ic-box-questionmark:before{content:\"\\e3a9\"}.bosch-ic-rainbow:before{content:\"\\e3aa\"}.bosch-ic-medal:before{content:\"\\e3ab\"}.bosch-ic-wheel:before{content:\"\\e3ac\"}.bosch-ic-borescope-connected:before{content:\"\\e3ad\"}.bosch-ic-filter-delete:before{content:\"\\e3ae\"}.bosch-ic-filter-add:before{content:\"\\e3af\"}.bosch-ic-man:before{content:\"\\e3b0\"}.bosch-ic-woman:before{content:\"\\e3b1\"}.bosch-ic-fridge:before{content:\"\\e3b2\"}.bosch-ic-battery-car:before{content:\"\\e3b3\"}.bosch-ic-bag-escalator:before{content:\"\\e3b4\"}.bosch-ic-shopping-bags:before{content:\"\\e3b5\"}.bosch-ic-car-steering-left:before{content:\"\\e3b6\"}.bosch-ic-car-steering-right:before{content:\"\\e3b7\"}.bosch-ic-thumb-up-down:before{content:\"\\e3b8\"}.bosch-ic-user-run-detection:before{content:\"\\e3b9\"}.bosch-ic-bank:before{content:\"\\e3ba\"}.bosch-ic-transportation:before{content:\"\\e3bb\"}.bosch-ic-government:before{content:\"\\e3bc\"}.bosch-ic-user-music:before{content:\"\\e3bd\"}.bosch-ic-bus:before{content:\"\\e3be\"}.bosch-ic-car-euro:before{content:\"\\e3bf\"}.bosch-ic-bag:before{content:\"\\e3c0\"}.bosch-ic-plate-tetrapack:before{content:\"\\e3c1\"}.bosch-ic-hospital:before{content:\"\\e3c2\"}.bosch-ic-desktop-structure:before{content:\"\\e3c3\"}.bosch-ic-frog-head:before{content:\"\\e3c4\"}.bosch-ic-sportboat:before{content:\"\\e3c5\"}.bosch-ic-yacht:before{content:\"\\e3c6\"}.bosch-ic-leaf-tilia:before{content:\"\\e3c7\"}.bosch-ic-music:before{content:\"\\e3c8\"}.bosch-ic-oil-lamp:before{content:\"\\e3c9\"}.bosch-ic-windshield-wiper-rear:before{content:\"\\e3ca\"}.bosch-ic-dongle-connected:before{content:\"\\e3cb\"}.bosch-ic-slate:before{content:\"\\e3cc\"}.bosch-ic-appliance-heat-pump-exhaust:before{content:\"\\e3cd\"}.bosch-ic-alert-list:before{content:\"\\e3ce\"}.bosch-ic-arrow-left-end-frame:before{content:\"\\e3cf\"}.bosch-ic-arrow-right-end-frame:before{content:\"\\e3d0\"}.bosch-ic-shape-ellipse:before{content:\"\\e3d1\"}.bosch-ic-shape-rectangle:before{content:\"\\e3d2\"}.bosch-ic-wishlist-add:before{content:\"\\e3d3\"}.bosch-ic-smartphone-disabled-light:before{content:\"\\e3d4\"}.bosch-ic-airbag:before{content:\"\\e3d5\"}.bosch-ic-compass:before{content:\"\\e3d6\"}.bosch-ic-turbine-wind:before{content:\"\\e3d7\"}.bosch-ic-arrows-diagonal-disabled-light:before{content:\"\\e3d8\"}.bosch-ic-ear-off-disabled-bold:before{content:\"\\e3d9\"}.bosch-ic-fan-off-disabled-light:before{content:\"\\e3da\"}.bosch-ic-faucet-off-disabled-light:before{content:\"\\e3db\"}.bosch-ic-hand-motion-off-disabled-bold:before{content:\"\\e3dc\"}.bosch-ic-heating-floor-off-disabled-light:before{content:\"\\e3dd\"}.bosch-ic-heating-radiator-off-disabled-light:before{content:\"\\e3de\"}.bosch-ic-ice-off-disabled-light:before{content:\"\\e3df\"}.bosch-ic-smartphone-disabled-bold:before{content:\"\\e3e0\"}.bosch-ic-subtitles-off-bold:before{content:\"\\e3e1\"}.bosch-ic-sun-off-disabled-light:before{content:\"\\e3e2\"}.bosch-ic-video-disabled-bold:before{content:\"\\e3e3\"}.bosch-ic-volume-off-bold:before{content:\"\\e3e4\"}.bosch-ic-watch-off-disabled-bold:before{content:\"\\e3e5\"}.bosch-ic-water-shower-off-disabled-light:before{content:\"\\e3e6\"}.bosch-ic-3d-frame-play:before{content:\"\\e3e7\"}.bosch-ic-alert-error-checkmark:before{content:\"\\e3e8\"}.bosch-ic-arrow-up-frame-check:before{content:\"\\e3e9\"}.bosch-ic-arrow-up-frame-error:before{content:\"\\e3ea\"}.bosch-ic-arrows-checkmark:before{content:\"\\e3eb\"}.bosch-ic-assembly-line-settings:before{content:\"\\e3ec\"}.bosch-ic-augmented-reality-player:before{content:\"\\e3ed\"}.bosch-ic-bar-chart-kappa:before{content:\"\\e3ef\"}.bosch-ic-bar-chart-search-detail:before{content:\"\\e3f0\"}.bosch-ic-barcode-checkmark:before{content:\"\\e3f1\"}.bosch-ic-barcode-scan-bulb:before{content:\"\\e3f2\"}.bosch-ic-barcode:before{content:\"\\e3f3\"}.bosch-ic-binary:before{content:\"\\e3f4\"}.bosch-ic-book-lightbulb:before{content:\"\\e3f5\"}.bosch-ic-box-arrow-down:before{content:\"\\e3f6\"}.bosch-ic-box-delivery-checkmark:before{content:\"\\e3f7\"}.bosch-ic-globe-checkmark:before{content:\"\\e3f8\"}.bosch-ic-calendar-boxes:before{content:\"\\e3f9\"}.bosch-ic-calendar-service:before{content:\"\\e3fa\"}.bosch-ic-camera-processor:before{content:\"\\e3fb\"}.bosch-ic-camera-settings:before{content:\"\\e3fc\"}.bosch-ic-chart-line-checkmark:before{content:\"\\e3fd\"}.bosch-ic-checklist-stack:before{content:\"\\e3fe\"}.bosch-ic-checkmark-frame-minimum:before{content:\"\\e3ff\"}.bosch-ic-checkmark-frame-tripple:before{content:\"\\e400\"}.bosch-ic-clipboard-checkmark:before{content:\"\\e401\"}.bosch-ic-clipboard-list-parts:before{content:\"\\e402\"}.bosch-ic-clipboard-settings:before{content:\"\\e403\"}.bosch-ic-computer-tower-box:before{content:\"\\e404\"}.bosch-ic-connectivity-lock:before{content:\"\\e405\"}.bosch-ic-counter-current:before{content:\"\\e406\"}.bosch-ic-counter-target:before{content:\"\\e407\"}.bosch-ic-cube-checkmark:before{content:\"\\e408\"}.bosch-ic-cube-play:before{content:\"\\e409\"}.bosch-ic-timeline-settings:before{content:\"\\e40a\"}.bosch-ic-timeline-view:before{content:\"\\e40b\"}.bosch-ic-timeline:before{content:\"\\e40c\"}.bosch-ic-delivery-checkmark:before{content:\"\\e40d\"}.bosch-ic-desktop-application-3d-play:before{content:\"\\e40e\"}.bosch-ic-desktop-apps-download:before{content:\"\\e40f\"}.bosch-ic-desktop-dashboard:before{content:\"\\e410\"}.bosch-ic-desktop-graph-checkmark:before{content:\"\\e411\"}.bosch-ic-desktop-graph-search:before{content:\"\\e412\"}.bosch-ic-desktop-graph:before{content:\"\\e413\"}.bosch-ic-development-kit:before{content:\"\\e414\"}.bosch-ic-directory-cloud-settings:before{content:\"\\e415\"}.bosch-ic-directory-cloud:before{content:\"\\e416\"}.bosch-ic-directory-settings:before{content:\"\\e417\"}.bosch-ic-document-clock-cycle:before{content:\"\\e418\"}.bosch-ic-document-code-stack:before{content:\"\\e419\"}.bosch-ic-document-dat-edit:before{content:\"\\e41a\"}.bosch-ic-document-ohd-arrow-down:before{content:\"\\e41b\"}.bosch-ic-dolphin:before{content:\"\\e41c\"}.bosch-ic-energy-management:before{content:\"\\e41d\"}.bosch-ic-eraser:before{content:\"\\e41e\"}.bosch-ic-folder-broom:before{content:\"\\e41f\"}.bosch-ic-folder-check-broom:before{content:\"\\e420\"}.bosch-ic-folder-checkmark:before{content:\"\\e421\"}.bosch-ic-folder-oes-check:before{content:\"\\e422\"}.bosch-ic-folder-open-service:before{content:\"\\e423\"}.bosch-ic-folder-reload:before{content:\"\\e424\"}.bosch-ic-hand-command:before{content:\"\\e425\"}.bosch-ic-heating-start:before{content:\"\\e426\"}.bosch-ic-hierarchy-refresh:before{content:\"\\e427\"}.bosch-ic-hierarchy-search:before{content:\"\\e428\"}.bosch-ic-hierarchy-settings:before{content:\"\\e429\"}.bosch-ic-hierarchy:before{content:\"\\e42a\"}.bosch-ic-hmi-desktop-settings:before{content:\"\\e42b\"}.bosch-ic-human-machine-interface:before{content:\"\\e42c\"}.bosch-ic-keys-user-access:before{content:\"\\e42d\"}.bosch-ic-keys:before{content:\"\\e42e\"}.bosch-ic-label-edit:before{content:\"\\e42f\"}.bosch-ic-line-arrow-checkmark:before{content:\"\\e430\"}.bosch-ic-link-checkmark:before{content:\"\\e431\"}.bosch-ic-lock-closed-checkmark:before{content:\"\\e432\"}.bosch-ic-lock-open-checkmark:before{content:\"\\e433\"}.bosch-ic-material-search:before{content:\"\\e434\"}.bosch-ic-objects-search:before{content:\"\\e435\"}.bosch-ic-objects:before{content:\"\\e436\"}.bosch-ic-plan-a-to-b:before{content:\"\\e437\"}.bosch-ic-plan-chart:before{content:\"\\e438\"}.bosch-ic-print-send:before{content:\"\\e439\"}.bosch-ic-refresh-analysis:before{content:\"\\e43a\"}.bosch-ic-remote-desktop-add:before{content:\"\\e43b\"}.bosch-ic-rfid-checkmark:before{content:\"\\e43c\"}.bosch-ic-rfid-tag:before{content:\"\\e43d\"}.bosch-ic-sequence-checkmark:before{content:\"\\e43e\"}.bosch-ic-server-arrow-up:before{content:\"\\e43f\"}.bosch-ic-server-arrows:before{content:\"\\e440\"}.bosch-ic-server-settings:before{content:\"\\e441\"}.bosch-ic-server-share:before{content:\"\\e442\"}.bosch-ic-settings-editor:before{content:\"\\e443\"}.bosch-ic-survey-checkmark:before{content:\"\\e444\"}.bosch-ic-target-dart:before{content:\"\\e445\"}.bosch-ic-target-view:before{content:\"\\e446\"}.bosch-ic-target:before{content:\"\\e447\"}.bosch-ic-thickness-selection:before{content:\"\\e448\"}.bosch-ic-toolbox-checkmark:before{content:\"\\e449\"}.bosch-ic-toolbox:before{content:\"\\e44a\"}.bosch-ic-translate:before{content:\"\\e44b\"}.bosch-ic-tune-vertical-checkmark:before{content:\"\\e44c\"}.bosch-ic-wand-user-add:before{content:\"\\e44d\"}.bosch-ic-wand-user-settings:before{content:\"\\e44e\"}.bosch-ic-wand-user:before{content:\"\\e44f\"}.bosch-ic-window-globe:before{content:\"\\e450\"}.bosch-ic-wrench-change:before{content:\"\\e451\"}.bosch-ic-clock-24-7:before{content:\"\\e452\"}.bosch-ic-eagle:before{content:\"\\e453\"}.bosch-ic-fluorescent-light:before{content:\"\\e454\"}.bosch-ic-home-cloud-0:before{content:\"\\e455\"}.bosch-ic-home-cloud-1:before{content:\"\\e456\"}.bosch-ic-home-cloud-2:before{content:\"\\e457\"}.bosch-ic-home-cloud-3:before{content:\"\\e458\"}.bosch-ic-home-shadow:before{content:\"\\e459\"}.bosch-ic-motorcycle-sport:before{content:\"\\e45a\"}.bosch-ic-smartphone-connected:before{content:\"\\e45b\"}.bosch-ic-usb-stick:before{content:\"\\e45c\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-timeline-clock:before{content:\"\\e45e\"}.bosch-ic-alarm-fire:before{content:\"\\e45f\"}.bosch-ic-emergency-front:before{content:\"\\e460\"}.bosch-ic-transporter-front:before{content:\"\\e461\"}.bosch-ic-truck-front:before{content:\"\\e462\"}.bosch-ic-unpin-classic-disabled-light:before{content:\"\\e463\"}.bosch-ic-unpin-modern-disabled-light:before{content:\"\\e464\"}.bosch-ic-smartphone-ban:before{content:\"\\e465\"}.bosch-ic-helmet:before{content:\"\\e466\"}.bosch-ic-gate-closed:before{content:\"\\e467\"}.bosch-ic-fence:before{content:\"\\e468\"}.bosch-ic-desktop-apps:before{content:\"\\e469\"}.bosch-ic-candle:before{content:\"\\e46a\"}.bosch-ic-abort-frame:before{content:\"\\e886\"}.bosch-ic-arrow-down-frame:before{content:\"\\e887\"}.bosch-ic-arrow-left-down-frame:before{content:\"\\e888\"}.bosch-ic-arrow-left-frame:before{content:\"\\e889\"}.bosch-ic-arrow-left-up-frame:before{content:\"\\e88a\"}.bosch-ic-arrow-right-down-frame:before{content:\"\\e88b\"}.bosch-ic-arrow-right-frame:before{content:\"\\e88c\"}.bosch-ic-arrow-right-up-frame:before{content:\"\\e88d\"}.bosch-ic-arrow-up-frame:before{content:\"\\e88e\"}.bosch-ic-asiapacific-frame:before{content:\"\\e88f\"}.bosch-ic-cash-frame:before{content:\"\\e890\"}.bosch-ic-checkmark-frame:before{content:\"\\e891\"}.bosch-ic-copyright-frame:before{content:\"\\e892\"}.bosch-ic-download-frame:before{content:\"\\e893\"}.bosch-ic-europe-frame:before{content:\"\\e894\"}.bosch-ic-lock-closed-frame:before{content:\"\\e895\"}.bosch-ic-lock-open-frame:before{content:\"\\e896\"}.bosch-ic-my-brand-frame:before{content:\"\\e897\"}.bosch-ic-northamerica-frame:before{content:\"\\e898\"}.bosch-ic-problem-frame:before{content:\"\\e899\"}.bosch-ic-promotion-frame:before{content:\"\\e89a\"}.bosch-ic-question-frame:before{content:\"\\e89b\"}.bosch-ic-share-frame:before{content:\"\\e89c\"}.bosch-ic-southamerica-frame:before{content:\"\\e89d\"}.bosch-ic-start-play-frame:before{content:\"\\e89e\"}.bosch-ic-upload-frame:before{content:\"\\e89f\"}.bosch-ic-world-frame:before{content:\"\\e8a0\"}.bosch-ic-add-frame:before{content:\"\\e8a1\"}.bosch-ic-call-deny-frame:before{content:\"\\e8a2\"}.bosch-ic-call-frame:before{content:\"\\e8a3\"}.bosch-ic-fast-backward-frame:before{content:\"\\e8a4\"}.bosch-ic-fast-forward-frame:before{content:\"\\e8a5\"}.bosch-ic-skip-backward-frame:before{content:\"\\e8a6\"}.bosch-ic-skip-fast-backward-frame:before{content:\"\\e8a7\"}.bosch-ic-skip-fast-forward-frame:before{content:\"\\e8a8\"}.bosch-ic-skip-forward-frame:before{content:\"\\e8a9\"}.bosch-ic-subtract-frame:before{content:\"\\e8aa\"}.bosch-ic-facebook-frame:before{content:\"\\ea87\"}.bosch-ic-googleplus-frame:before{content:\"\\ea88\"}.bosch-ic-instagram-frame:before{content:\"\\ea89\"}.bosch-ic-lineapp-frame:before{content:\"\\ea8a\"}.bosch-ic-linkedin-frame:before{content:\"\\ea8b\"}.bosch-ic-pinterest-frame:before{content:\"\\ea8c\"}.bosch-ic-snapchat-frame:before{content:\"\\ea8d\"}.bosch-ic-tumblr-frame:before{content:\"\\ea8e\"}.bosch-ic-twitter-frame:before{content:\"\\ea8f\"}.bosch-ic-vimeo-frame:before{content:\"\\ea90\"}.bosch-ic-wechat-frame:before{content:\"\\ea91\"}.bosch-ic-weibo-frame:before{content:\"\\ea92\"}.bosch-ic-whatsapp-frame:before{content:\"\\ea93\"}.bosch-ic-xing-frame:before{content:\"\\ea94\"}.bosch-ic-youku-frame:before{content:\"\\ea95\"}.bosch-ic-youtube-frame:before{content:\"\\ea96\"}.bosch-ic-vk-frame:before{content:\"\\ea97\"}.bosch-ic-skype-frame:before{content:\"\\ea98\"}.bosch-ic-alert-error-filled:before{content:\"\\eb08\"}.bosch-ic-alert-error:before{content:\"\\eb09\"}.bosch-ic-alert-success-filled:before{content:\"\\eb0a\"}.bosch-ic-alert-success:before{content:\"\\eb0b\"}.bosch-ic-alert-warning-filled:before{content:\"\\eb0c\"}.bosch-ic-alert-warning:before{content:\"\\eb0d\"}.bosch-ic-chart-oee:before{content:\"\\e46b\"}.bosch-ic-cycletime-eighth:before{content:\"\\e46c\"}.bosch-ic-cycletime-quarter:before{content:\"\\e46d\"}.bosch-ic-gateway:before{content:\"\\e46e\"}.bosch-ic-heatmap:before{content:\"\\e46f\"}.bosch-ic-laptop-checklist:before{content:\"\\e470\"}.bosch-ic-molecule:before{content:\"\\e471\"}.bosch-ic-robot-settings:before{content:\"\\e472\"}.bosch-ic-sensor-cable:before{content:\"\\e473\"}.bosch-ic-tank-layers:before{content:\"\\e474\"}.bosch-ic-window-settings:before{content:\"\\e475\"}.bosch-ic-heatmap-independent:before{content:\"\\e476\"}.bosch-ic-dumble:before{content:\"\\e477\"}.bosch-ic-truck-wash:before{content:\"\\e478\"}.bosch-ic-smart-sensor:before{content:\"\\e479\"}.bosch-ic-audio-add:before{content:\"\\e47a\"}.bosch-ic-audio-checked:before{content:\"\\e47b\"}.bosch-ic-audio-play:before{content:\"\\e47c\"}.bosch-ic-audio-remove:before{content:\"\\e47d\"}.bosch-ic-image-gallery-checked:before{content:\"\\e47e\"}.bosch-ic-imagery-checked:before{content:\"\\e47f\"}.bosch-ic-image-gallery-remove:before{content:\"\\e480\"}.bosch-ic-recording:before{content:\"\\e481\"}.bosch-ic-stop-frame:before{content:\"\\e482\"}.bosch-ic-roof:before{content:\"\\e483\"}.bosch-ic-projection-interactive:before{content:\"\\e484\"}.bosch-ic-bridge:before{content:\"\\e485\"}.bosch-ic-box-arrows:before{content:\"\\e486\"}.bosch-ic-chart-line-fit:before{content:\"\\e487\"}.bosch-ic-chart-line-compress:before{content:\"\\e488\"}.bosch-ic-ciss:before{content:\"\\e489\"}.bosch-ic-scd:before{content:\"\\e48a\"}.bosch-ic-vci:before{content:\"\\e48b\"}.bosch-ic-access-controller:before{content:\"\\e48c\"}.bosch-ic-card-reader:before{content:\"\\e48d\"}.bosch-ic-table:before{content:\"\\e48e\"}.bosch-ic-replace:before{content:\"\\e48f\"}.bosch-ic-chart-topflop-organized-horizontal:before{content:\"\\e490\"}.bosch-ic-chart-topflop-random-horizontal:before{content:\"\\e491\"}.bosch-ic-chart-topflop-organized-vertical:before{content:\"\\e492\"}.bosch-ic-chart-topflop-random-vertical:before{content:\"\\e493\"}.bosch-ic-ois:before{content:\"\\e494\"}.bosch-ic-cloud-refresh-locked:before{content:\"\\e495\"}.bosch-ic-car-bluetooth:before{content:\"\\e496\"}.bosch-ic-camera-trainer:before{content:\"\\e497\"}.bosch-ic-magnifier-moving:before{content:\"\\e498\"}.bosch-ic-absolute-orientation:before{content:\"\\e499\"}.bosch-ic-accelerometer:before{content:\"\\e49a\"}.bosch-ic-imu:before{content:\"\\e49b\"}.bosch-ic-gyroscope:before{content:\"\\e49c\"}.bosch-ic-magneotmeter:before{content:\"\\e49d\"}.bosch-ic-battery-car-1:before{content:\"\\e49e\"}.bosch-ic-battery-car-2:before{content:\"\\e49f\"}.bosch-ic-battery-car-3:before{content:\"\\e4a0\"}.bosch-ic-battery-car-charging:before{content:\"\\e4a1\"}.bosch-ic-battery-car-defective:before{content:\"\\e4a2\"}.bosch-ic-truck-crane:before{content:\"\\e4a3\"}.bosch-ic-train-diesel:before{content:\"\\e4a4\"}.bosch-ic-snowmobile:before{content:\"\\e4a5\"}.bosch-ic-riding-mower:before{content:\"\\e4a6\"}.bosch-ic-motorcycle-small:before{content:\"\\e4a7\"}.bosch-ic-mining-truck:before{content:\"\\e4a8\"}.bosch-ic-lawnmower:before{content:\"\\e4a9\"}.bosch-ic-jetski:before{content:\"\\e4aa\"}.bosch-ic-golf-caddy:before{content:\"\\e4ab\"}.bosch-ic-generator-electric:before{content:\"\\e4ac\"}.bosch-ic-garbage-truck:before{content:\"\\e4ad\"}.bosch-ic-fork-lift-big:before{content:\"\\e4ae\"}.bosch-ic-engine-small:before{content:\"\\e4af\"}.bosch-ic-engine-big:before{content:\"\\e4b0\"}.bosch-ic-coach:before{content:\"\\e4b1\"}.bosch-ic-chainsaw:before{content:\"\\e4b2\"}.bosch-ic-car-van:before{content:\"\\e4b3\"}.bosch-ic-car-suv:before{content:\"\\e4b4\"}.bosch-ic-car-race:before{content:\"\\e4b5\"}.bosch-ic-car-old:before{content:\"\\e4b6\"}.bosch-ic-car-camping-van:before{content:\"\\e4b7\"}.bosch-ic-car-camper:before{content:\"\\e4b8\"}.bosch-ic-bus-side:before{content:\"\\e4b9\"}.bosch-ic-book-info:before{content:\"\\e4ba\"}.bosch-ic-gears:before{content:\"\\e4bb\"}.bosch-ic-shark:before{content:\"\\e4bc\"}.bosch-ic-mountaintop:before{content:\"\\e4bd\"}.bosch-ic-baby:before{content:\"\\e4be\"}.bosch-ic-coffee-bean:before{content:\"\\e4bf\"}.bosch-ic-coffee-beans:before{content:\"\\e4c0\"}.bosch-ic-skull:before{content:\"\\e4c1\"}.bosch-ic-stroller:before{content:\"\\e4c2\"}.bosch-ic-glasses:before{content:\"\\e4c3\"}.bosch-ic-detergent:before{content:\"\\e4c4\"}.bosch-ic-washing-machine:before{content:\"\\e4c5\"}.bosch-ic-sdcard:before{content:\"\\e4c6\"}.bosch-ic-water-connectivity:before{content:\"\\e4c7\"}.bosch-ic-server-connectivity:before{content:\"\\e4c8\"}.bosch-ic-presentation-speaker:before{content:\"\\e4c9\"}.bosch-ic-chart-bar-arrow:before{content:\"\\e4ca\"}.bosch-ic-book-connectivity:before{content:\"\\e4cb\"}.bosch-ic-route-focus:before{content:\"\\e4cc\"}.bosch-ic-bicycle-helmet:before{content:\"\\e4cd\"}.bosch-ic-cookie:before{content:\"\\e4ce\"}.bosch-ic-glass-wine:before{content:\"\\e4cf\"}.bosch-ic-cup-small:before{content:\"\\e4d0\"}.bosch-ic-cup-large:before{content:\"\\e4d1\"}.bosch-ic-bowl:before{content:\"\\e4d2\"}.bosch-ic-glass-small:before{content:\"\\e4d3\"}.bosch-ic-ice-bottle:before{content:\"\\e4d4\"}.bosch-ic-cup-plate-drop:before{content:\"\\e4d5\"}.bosch-ic-plate:before{content:\"\\e4d6\"}.bosch-ic-cup-plate:before{content:\"\\e4d7\"}.bosch-ic-bubbles:before{content:\"\\e4d8\"}.bosch-ic-button-start-stop:before{content:\"\\e4d9\"}.bosch-ic-elevator-freight-max:before{content:\"\\e4da\"}.bosch-ic-elevator-freight:before{content:\"\\e4db\"}.bosch-ic-elevator-persons:before{content:\"\\e4dc\"}.bosch-ic-embed:before{content:\"\\e4dd\"}.bosch-ic-female-male:before{content:\"\\e4de\"}.bosch-ic-fork-knife-spoon:before{content:\"\\e4df\"}.bosch-ic-global-entrepreneur:before{content:\"\\e4e0\"}.bosch-ic-arrows-left-right-checkmark:before{content:\"\\e4e1\"}.bosch-ic-automatic-mode:before{content:\"\\e4e2\"}.bosch-ic-babybottle:before{content:\"\\e4e3\"}.bosch-ic-barcode-scan:before{content:\"\\e4e4\"}.bosch-ic-barcode-scanner:before{content:\"\\e4e5\"}.bosch-ic-cash-frame-rupee:before{content:\"\\e4e6\"}.bosch-ic-chart-heat-curve:before{content:\"\\e4e7\"}.bosch-ic-compressor:before{content:\"\\e4e8\"}.bosch-ic-cubes:before{content:\"\\e4e9\"}.bosch-ic-highway:before{content:\"\\e4ea\"}.bosch-ic-industry-workers:before{content:\"\\e4eb\"}.bosch-ic-industry-workers-connected:before{content:\"\\e4ec\"}.bosch-ic-machine-station-control:before{content:\"\\e4ed\"}.bosch-ic-map-city:before{content:\"\\e4ee\"}.bosch-ic-ordernumber:before{content:\"\\e4ef\"}.bosch-ic-shirt-business:before{content:\"\\e4f0\"}.bosch-ic-shirt-casual:before{content:\"\\e4f1\"}.bosch-ic-suitcase:before{content:\"\\e4f2\"}.bosch-ic-weather-cloud-sun-sensor:before{content:\"\\e4f3\"}.bosch-ic-window-code:before{content:\"\\e4f4\"}.bosch-ic-wrench-warning:before{content:\"\\e4f5\"}.bosch-ic-arrow-circle-a:before{content:\"\\e4f6\"}.bosch-ic-arrows-agility:before{content:\"\\e4f7\"}.bosch-ic-arrows-collapse-object:before{content:\"\\e4f8\"}.bosch-ic-arrows-expand-object:before{content:\"\\e4f9\"}.bosch-ic-arrows-left-right:before{content:\"\\e4fa\"}.bosch-ic-fan-heat:before{content:\"\\e4fb\"}.bosch-ic-battery-car-arrows-circle:before{content:\"\\e4fc\"}.bosch-ic-battery-car-clock:before{content:\"\\e4fd\"}.bosch-ic-battery-vibration:before{content:\"\\e4fe\"}.bosch-ic-belt-elastic:before{content:\"\\e4ff\"}.bosch-ic-bicycle-e-arrow-away:before{content:\"\\e500\"}.bosch-ic-bicycle-e-arrow-towards:before{content:\"\\e501\"}.bosch-ic-brake-decoupled-power:before{content:\"\\e502\"}.bosch-ic-brake-i-booster:before{content:\"\\e503\"}.bosch-ic-brake-system-esp:before{content:\"\\e504\"}.bosch-ic-brake-system-integrated:before{content:\"\\e505\"}.bosch-ic-brake-vacuum-booster:before{content:\"\\e506\"}.bosch-ic-capacity:before{content:\"\\e507\"}.bosch-ic-car-beam-length:before{content:\"\\e508\"}.bosch-ic-car-smartphone-locator:before{content:\"\\e509\"}.bosch-ic-chart-pie:before{content:\"\\e50a\"}.bosch-ic-chess-jumper:before{content:\"\\e50b\"}.bosch-ic-clock-time-off:before{content:\"\\e50c\"}.bosch-ic-clock-time-on:before{content:\"\\e50d\"}.bosch-ic-coin:before{content:\"\\e50e\"}.bosch-ic-connection-off-settings:before{content:\"\\e50f\"}.bosch-ic-connection-on-settings:before{content:\"\\e510\"}.bosch-ic-control-unit:before{content:\"\\e511\"}.bosch-ic-customer:before{content:\"\\e512\"}.bosch-ic-engine:before{content:\"\\e513\"}.bosch-ic-filter-air:before{content:\"\\e514\"}.bosch-ic-filter-cabin-carbon:before{content:\"\\e515\"}.bosch-ic-filter-cabin:before{content:\"\\e516\"}.bosch-ic-filter-cooling-water:before{content:\"\\e517\"}.bosch-ic-filter-denox-cover:before{content:\"\\e518\"}.bosch-ic-filter-denox:before{content:\"\\e519\"}.bosch-ic-filter-element-compensation-sealing:before{content:\"\\e51a\"}.bosch-ic-filter-element-compensation:before{content:\"\\e51b\"}.bosch-ic-filter-element:before{content:\"\\e51c\"}.bosch-ic-filter-oil:before{content:\"\\e51d\"}.bosch-ic-filter-petrol:before{content:\"\\e51e\"}.bosch-ic-filter-prefilter:before{content:\"\\e51f\"}.bosch-ic-fireplace:before{content:\"\\e520\"}.bosch-ic-flash-plus:before{content:\"\\e521\"}.bosch-ic-globe-worldwideweb:before{content:\"\\e522\"}.bosch-ic-glow-control-unit:before{content:\"\\e523\"}.bosch-ic-glow-plug:before{content:\"\\e524\"}.bosch-ic-hand-cloth:before{content:\"\\e525\"}.bosch-ic-leaf-plus:before{content:\"\\e526\"}.bosch-ic-machine-warning:before{content:\"\\e527\"}.bosch-ic-measuring-jug:before{content:\"\\e528\"}.bosch-ic-plug-play:before{content:\"\\e529\"}.bosch-ic-refresh-checkmark:before{content:\"\\e52a\"}.bosch-ic-replacement-filament:before{content:\"\\e52b\"}.bosch-ic-screw:before{content:\"\\e52c\"}.bosch-ic-seat-belt:before{content:\"\\e52d\"}.bosch-ic-steering-wheel-hands-automated:before{content:\"\\e52e\"}.bosch-ic-sun-ice-parking:before{content:\"\\e52f\"}.bosch-ic-syringe-change-needle:before{content:\"\\e530\"}.bosch-ic-syringe-change:before{content:\"\\e531\"}.bosch-ic-syringe:before{content:\"\\e532\"}.bosch-ic-thread-diameter:before{content:\"\\e533\"}.bosch-ic-tool-extraction:before{content:\"\\e534\"}.bosch-ic-transporter-car-front:before{content:\"\\e535\"}.bosch-ic-truck-car-motorcycle:before{content:\"\\e536\"}.bosch-ic-truck-car:before{content:\"\\e537\"}.bosch-ic-truck-logistics-box:before{content:\"\\e538\"}.bosch-ic-truck-transporter:before{content:\"\\e539\"}.bosch-ic-watch-on-warning:before{content:\"\\e53a\"}.bosch-ic-wrench-calibrate:before{content:\"\\e53b\"}.bosch-ic-wrench-dummy:before{content:\"\\e53c\"}.bosch-ic-wrench-tightening-torque:before{content:\"\\e53d\"}.bosch-ic-wrench-width-across-flats:before{content:\"\\e53e\"}.bosch-ic-user-arm-bandage:before{content:\"\\e53f\"}.bosch-ic-user-eye-blind:before{content:\"\\e540\"}.bosch-ic-user-ear-deaf:before{content:\"\\e541\"}.bosch-ic-user-car-distracted:before{content:\"\\e542\"}.bosch-ic-user-mouth-mute:before{content:\"\\e543\"}.bosch-ic-user-ear-blocked:before{content:\"\\e544\"}.bosch-ic-user-bavarian:before{content:\"\\e545\"}.bosch-ic-user-ear-loud:before{content:\"\\e546\"}.bosch-ic-user-walk-crutch:before{content:\"\\e547\"}.bosch-ic-user-mouth-speechless:before{content:\"\\e548\"}.bosch-ic-user-arm-baby:before{content:\"\\e549\"}.bosch-ic-user-arm-missing:before{content:\"\\e54a\"}.bosch-ic-user-eye-eyepatch:before{content:\"\\e54b\"}.bosch-ic-user-strapped:before{content:\"\\e54c\"}.bosch-ic-cleaning-agent:before{content:\"\\e54d\"}.bosch-ic-cup-plate-dirty:before{content:\"\\e54e\"}.bosch-ic-cup-plate-private:before{content:\"\\e54f\"}.bosch-ic-rubbish-apple:before{content:\"\\e550\"}.bosch-ic-rubbish-fish:before{content:\"\\e551\"}.bosch-ic-rubbish-organic-bin:before{content:\"\\e552\"}.bosch-ic-rubbish-paper:before{content:\"\\e553\"}.bosch-ic-rubbish-paper-bin:before{content:\"\\e554\"}.bosch-ic-rubbish-residual-bin:before{content:\"\\e555\"}.bosch-ic-shower:before{content:\"\\e556\"}.bosch-ic-shower-female:before{content:\"\\e557\"}.bosch-ic-shower-male:before{content:\"\\e558\"}.bosch-ic-wardrobe-female:before{content:\"\\e559\"}.bosch-ic-wardrobe-male:before{content:\"\\e55a\"}.bosch-ic-car-top:before{content:\"\\e55b\"}.bosch-ic-label-info:before{content:\"\\e55c\"}.bosch-ic-perspective-view:before{content:\"\\e55d\"}.bosch-ic-road:before{content:\"\\e55e\"}.bosch-ic-screen-split-horizontal:before{content:\"\\e55f\"}.bosch-ic-screen-split-vertical:before{content:\"\\e560\"}.bosch-ic-screenshot-frame:before{content:\"\\e561\"}.bosch-ic-sensor-lidar:before{content:\"\\e562\"}.bosch-ic-tram:before{content:\"\\e563\"}.bosch-ic-video-360-degrees:before{content:\"\\e564\"}.bosch-ic-belt-groove-drives:before{content:\"\\e565\"}.bosch-ic-bottle-champagne:before{content:\"\\e566\"}.bosch-ic-box-falling:before{content:\"\\e567\"}.bosch-ic-box-falling-off-disabled-bold:before{content:\"\\e568\"}.bosch-ic-box-falling-off-disabled-light:before{content:\"\\e569\"}.bosch-ic-brake-pedal:before{content:\"\\e56a\"}.bosch-ic-car-front-light-inside:before{content:\"\\e56b\"}.bosch-ic-car-licence-plate-light:before{content:\"\\e56c\"}.bosch-ic-car-side-light-back:before{content:\"\\e56d\"}.bosch-ic-car-side-light-front:before{content:\"\\e56e\"}.bosch-ic-light-fog-warning:before{content:\"\\e56f\"}.bosch-ic-prism:before{content:\"\\e570\"}.bosch-ic-turnstile:before{content:\"\\e571\"}.bosch-ic-turnstile-open-in-and-outbound:before{content:\"\\e572\"}.bosch-ic-turnstile-open-inbound:before{content:\"\\e573\"}.bosch-ic-turnstile-open-outbound:before{content:\"\\e574\"}.bosch-ic-uwb:before{content:\"\\e575\"}.bosch-ic-heat-reduce-active:before{content:\"\\e576\"}.bosch-ic-heat-reduce-passive:before{content:\"\\e577\"}.bosch-ic-hourglass-add:before{content:\"\\e578\"}.bosch-ic-light-fog:before{content:\"\\e579\"}.bosch-ic-car-turn-signal:before{content:\"\\e57a\"}.bosch-ic-car-turn-signal-left:before{content:\"\\e57b\"}.bosch-ic-car-turn-signal-right:before{content:\"\\e57c\"}.bosch-ic-light-rear:before{content:\"\\e57d\"}.bosch-ic-car-top-turn-signal-front:before{content:\"\\e57e\"}.bosch-ic-car-top-turn-signal-back:before{content:\"\\e57f\"}.bosch-ic-ccu-cm:before{content:\"\\e580\"}.bosch-ic-lung:before{content:\"\\e581\"}.bosch-ic-head-speaking-voice:before{content:\"\\e582\"}.bosch-ic-car-motorcycle:before{content:\"\\e583\"}.bosch-ic-car-turn-signal-side:before{content:\"\\e584\"}.bosch-ic-car-light-parking:before{content:\"\\e585\"}.bosch-ic-phoenix:before{content:\"\\e586\"}.bosch-ic-sensor-air-quality:before{content:\"\\e587\"}.bosch-ic-weather-station:before{content:\"\\e588\"}.bosch-ic-watering-can:before{content:\"\\e589\"}.bosch-ic-propeller:before{content:\"\\e58a\"}.bosch-ic-temperature-fluctuating:before{content:\"\\e58b\"}.bosch-ic-stove-top:before{content:\"\\e58c\"}.bosch-ic-socket:before{content:\"\\e58d\"}.bosch-ic-siren:before{content:\"\\e58e\"}.bosch-ic-sensor-water:before{content:\"\\e58f\"}.bosch-ic-user-run-sensor-motion:before{content:\"\\e590\"}.bosch-ic-robot-lawnmower:before{content:\"\\e591\"}.bosch-ic-robot-cleaning:before{content:\"\\e592\"}.bosch-ic-oven:before{content:\"\\e593\"}.bosch-ic-loudspeaker:before{content:\"\\e594\"}.bosch-ic-keypad:before{content:\"\\e595\"}.bosch-ic-shape-circle-square:before{content:\"\\e596\"}.bosch-ic-garage:before{content:\"\\e597\"}.bosch-ic-fume-hood:before{content:\"\\e598\"}.bosch-ic-freezer:before{content:\"\\e599\"}.bosch-ic-food-processor:before{content:\"\\e59a\"}.bosch-ic-floorplan-rooms:before{content:\"\\e59b\"}.bosch-ic-dryer-tumble:before{content:\"\\e59c\"}.bosch-ic-dryer:before{content:\"\\e59d\"}.bosch-ic-door-window:before{content:\"\\e59e\"}.bosch-ic-door-lock:before{content:\"\\e59f\"}.bosch-ic-door-bell:before{content:\"\\e5a0\"}.bosch-ic-dishwasher:before{content:\"\\e5a1\"}.bosch-ic-dimmer:before{content:\"\\e5a2\"}.bosch-ic-detector-smoke:before{content:\"\\e5a3\"}.bosch-ic-detector-presence:before{content:\"\\e5a4\"}.bosch-ic-cooking:before{content:\"\\e5a5\"}.bosch-ic-coffee-maschine:before{content:\"\\e5a6\"}.bosch-ic-camera-indoor:before{content:\"\\e5a7\"}.bosch-ic-broom-clean:before{content:\"\\e5a8\"}.bosch-ic-blinds:before{content:\"\\e5a9\"}.bosch-ic-bicycle-e-flash:before{content:\"\\e5aa\"}.bosch-ic-finger-domino-trigger:before{content:\"\\e5ab\"}.bosch-ic-automation-start:before{content:\"\\e5ac\"}.bosch-ic-automation:before{content:\"\\e5ad\"}.bosch-ic-sensor-multifunctional:before{content:\"\\e5ae\"}.bosch-ic-plane-top:before{content:\"\\e5af\"}.bosch-ic-shape-square-overlapping:before{content:\"\\e5b0\"}.bosch-ic-square-overlapping:before{content:\"\\e5b1\"}.bosch-ic-battery-car-14v:before{content:\"\\e5b2\"}.bosch-ic-calibration:before{content:\"\\e5b3\"}.bosch-ic-structure-can-bus:before{content:\"\\e5b4\"}.bosch-ic-gears-direction-arrows:before{content:\"\\e5b5\"}.bosch-ic-car-limousine:before{content:\"\\e5b6\"}.bosch-ic-molecule-gas:before{content:\"\\e5b7\"}.bosch-ic-money-euro:before{content:\"\\e5b8\"}.bosch-ic-revolution-rpm:before{content:\"\\e5b9\"}.bosch-ic-rikscha-e:before{content:\"\\e5ba\"}.bosch-ic-sensor-circle:before{content:\"\\e5bb\"}.bosch-ic-speedometer-high:before{content:\"\\e5bc\"}.bosch-ic-steering-wheel-arrows:before{content:\"\\e5bd\"}.bosch-ic-tablet-checklist:before{content:\"\\e5be\"}.bosch-ic-petrol-charging-station-hybrid:before{content:\"\\e5bf\"}.bosch-ic-weather-cloud-sun-rain-snow:before{content:\"\\e5c0\"}.bosch-ic-light-half:before{content:\"\\e5c1\"}.bosch-ic-video-sensor:before{content:\"\\e5c2\"}.bosch-ic-binary-desktop:before{content:\"\\e5c3\"}.bosch-ic-truck-delivery:before{content:\"\\e5c4\"}.bosch-ic-van-delivery:before{content:\"\\e5c5\"}.bosch-ic-fullscreen-user:before{content:\"\\e5c6\"}.bosch-ic-user-helmet-motorcycle:before{content:\"\\e5c7\"}.bosch-ic-user-hand-disabled-light:before{content:\"\\e5c8\"}.bosch-ic-user-hand-disabled-bold:before{content:\"\\e5c9\"}.bosch-ic-link-connected:before{content:\"\\e5ca\"}.bosch-ic-gate-open:before{content:\"\\e5cb\"}.bosch-ic-box-checkmark:before{content:\"\\e5cc\"}.bosch-ic-box-place:before{content:\"\\e5cd\"}.bosch-ic-logistics-dock:before{content:\"\\e5ce\"}.bosch-ic-barcode-scan-logistics:before{content:\"\\e5cf\"}.bosch-ic-locator-tracking:before{content:\"\\e5d0\"}.bosch-ic-car-light-trunk:before{content:\"\\e5d1\"}.bosch-ic-car-light-glove-compartment:before{content:\"\\e5d2\"}.bosch-ic-car-light-dashboard:before{content:\"\\e5d3\"}.bosch-ic-fire-leaf:before{content:\"\\e5d4\"}.bosch-ic-laptop-hacker:before{content:\"\\e5d5\"}.bosch-ic-hands-drops:before{content:\"\\e5d6\"}.bosch-ic-team-disabled-bold:before{content:\"\\e5d7\"}.bosch-ic-team-disabled-light:before{content:\"\\e5d8\"}.bosch-ic-user-head-arm:before{content:\"\\e5d9\"}.bosch-ic-user-head-hand-disabled-bold:before{content:\"\\e5da\"}.bosch-ic-user-head-hand-disabled-light:before{content:\"\\e5db\"}.bosch-ic-welcome-disabled-bold:before{content:\"\\e5dc\"}.bosch-ic-welcome-disabled-light:before{content:\"\\e5dd\"}.bosch-ic-female-male-arrows:before{content:\"\\e5de\"}.bosch-ic-broccoli:before{content:\"\\e5df\"}.bosch-ic-chicken:before{content:\"\\e5e0\"}.bosch-ic-cow:before{content:\"\\e5e1\"}.bosch-ic-deer:before{content:\"\\e5e2\"}.bosch-ic-fish:before{content:\"\\e5e3\"}.bosch-ic-garlic:before{content:\"\\e5e4\"}.bosch-ic-hotpepper:before{content:\"\\e5e5\"}.bosch-ic-pig:before{content:\"\\e5e6\"}.bosch-ic-rabbit:before{content:\"\\e5e7\"}.bosch-ic-sheep:before{content:\"\\e5e8\"}.bosch-ic-shell:before{content:\"\\e5e9\"}.bosch-ic-shrimp:before{content:\"\\e5ea\"}.bosch-ic-car-seat:before{content:\"\\e5eb\"}.bosch-ic-car-seat-steering-wheel:before{content:\"\\e5ec\"}.bosch-ic-car-side-arrow-back:before{content:\"\\e5ed\"}.bosch-ic-car-side-arrow-front:before{content:\"\\e5ee\"}.bosch-ic-car-steering-left-disabled-bold:before{content:\"\\e5ef\"}.bosch-ic-car-steering-left-disabled-light:before{content:\"\\e5f0\"}.bosch-ic-car-steering-left-lle:before{content:\"\\e5f1\"}.bosch-ic-car-steering-left-lle-disabled-bold:before{content:\"\\e5f2\"}.bosch-ic-car-steering-left-lle-disabled-light:before{content:\"\\e5f3\"}.bosch-ic-car-steering-right-disabled-bold:before{content:\"\\e5f4\"}.bosch-ic-car-steering-right-disabled-light:before{content:\"\\e5f5\"}.bosch-ic-car-steering-right-rle:before{content:\"\\e5f6\"}.bosch-ic-car-steering-right-rle-disabled-bold:before{content:\"\\e5f7\"}.bosch-ic-car-steering-right-rle-disabled-light:before{content:\"\\e5f8\"}.bosch-ic-cars-traffic-left-right:before{content:\"\\e5f9\"}.bosch-ic-configuration-wrench:before{content:\"\\e5fa\"}.bosch-ic-hook:before{content:\"\\e5fb\"}.bosch-ic-hook-arrows:before{content:\"\\e5fc\"}.bosch-ic-hook-arrows-wiperblade:before{content:\"\\e5fd\"}.bosch-ic-hook-disabled-bold:before{content:\"\\e5fe\"}.bosch-ic-hook-disabled-light:before{content:\"\\e5ff\"}.bosch-ic-info-i-frame-add:before{content:\"\\e600\"}.bosch-ic-search-number:before{content:\"\\e601\"}.bosch-ic-search-number-add:before{content:\"\\e602\"}.bosch-ic-set:before{content:\"\\e603\"}.bosch-ic-speedometer-speed-high:before{content:\"\\e604\"}.bosch-ic-windshield-wiper-clean:before{content:\"\\e605\"}.bosch-ic-windshield-wiper-double:before{content:\"\\e606\"}.bosch-ic-wiper-adapter:before{content:\"\\e607\"}.bosch-ic-wiper-blade:before{content:\"\\e608\"}.bosch-ic-wiper-blade-arrow:before{content:\"\\e609\"}.bosch-ic-wiper-blade-eco:before{content:\"\\e60a\"}.bosch-ic-wiper-blade-mm:before{content:\"\\e60b\"}.bosch-ic-wiper-blade-mm-arrows:before{content:\"\\e60c\"}.bosch-ic-wiper-blade-spoiler:before{content:\"\\e60d\"}.bosch-ic-wiper-blade-twin:before{content:\"\\e60e\"}.bosch-ic-wiper-flatblade:before{content:\"\\e60f\"}.bosch-ic-wiper-flatblade-aeroeco:before{content:\"\\e610\"}.bosch-ic-wiper-flatblade-aerofit:before{content:\"\\e611\"}.bosch-ic-wiper-flatblade-aerotwin:before{content:\"\\e612\"}.bosch-ic-wiper-flatblade-arrow:before{content:\"\\e613\"}.bosch-ic-wiper-flatblade-mm:before{content:\"\\e614\"}.bosch-ic-wiper-flatblade-mm-arrows:before{content:\"\\e615\"}.bosch-ic-wiper-rubber:before{content:\"\\e616\"}.bosch-ic-address-control-system:before{content:\"\\e617\"}.bosch-ic-balcony:before{content:\"\\e618\"}.bosch-ic-battery-car-45-degree-arrow:before{content:\"\\e619\"}.bosch-ic-boiler-warning:before{content:\"\\e61a\"}.bosch-ic-book-wrench:before{content:\"\\e61b\"}.bosch-ic-car-driver-passenger-arrows:before{content:\"\\e61c\"}.bosch-ic-car-licence-plate:before{content:\"\\e61d\"}.bosch-ic-car-side-arrow-back-tires:before{content:\"\\e61e\"}.bosch-ic-car-side-arrow-front-tires:before{content:\"\\e61f\"}.bosch-ic-car-side-half-arrow-back:before{content:\"\\e620\"}.bosch-ic-car-steering-left-arrow:before{content:\"\\e621\"}.bosch-ic-car-steering-left-passenger-arrow:before{content:\"\\e622\"}.bosch-ic-car-steering-right-arrow:before{content:\"\\e623\"}.bosch-ic-car-steering-right-passenger-arrow:before{content:\"\\e624\"}.bosch-ic-chess-board-transparency:before{content:\"\\e625\"}.bosch-ic-cloud-locked:before{content:\"\\e626\"}.bosch-ic-conference-system:before{content:\"\\e627\"}.bosch-ic-cycletime-quarter-arrows-interval:before{content:\"\\e628\"}.bosch-ic-desktop-connectivity:before{content:\"\\e629\"}.bosch-ic-hierarchy-checkmark:before{content:\"\\e62a\"}.bosch-ic-home-office:before{content:\"\\e62b\"}.bosch-ic-home-semi-detached-house:before{content:\"\\e62c\"}.bosch-ic-kitchen:before{content:\"\\e62d\"}.bosch-ic-lambda-arrows-length:before{content:\"\\e62e\"}.bosch-ic-lambda-oxygen:before{content:\"\\e62f\"}.bosch-ic-lambda-sensor-cores:before{content:\"\\e630\"}.bosch-ic-panel-control:before{content:\"\\e631\"}.bosch-ic-prison:before{content:\"\\e632\"}.bosch-ic-prison-cell:before{content:\"\\e633\"}.bosch-ic-radio-broadcasting:before{content:\"\\e634\"}.bosch-ic-robot-body:before{content:\"\\e635\"}.bosch-ic-robot-body-arrow:before{content:\"\\e636\"}.bosch-ic-skyscraper-danger:before{content:\"\\e637\"}.bosch-ic-skyscraper-flat:before{content:\"\\e638\"}.bosch-ic-skyscraper-flat-floor-low:before{content:\"\\e639\"}.bosch-ic-skyscraper-flat-floor-middle:before{content:\"\\e63a\"}.bosch-ic-skyscraper-flat-floor-top:before{content:\"\\e63b\"}.bosch-ic-skyscraper-flat-semi-detached:before{content:\"\\e63c\"}.bosch-ic-skyscraper-locked:before{content:\"\\e63d\"}.bosch-ic-skyscraper-settings:before{content:\"\\e63e\"}.bosch-ic-team-signpost:before{content:\"\\e63f\"}.bosch-ic-toaster-hand-warning:before{content:\"\\e640\"}.bosch-ic-user-arrows-heart:before{content:\"\\e641\"}.bosch-ic-user-graduate-chat:before{content:\"\\e642\"}.bosch-ic-user-head-mask:before{content:\"\\e643\"}.bosch-ic-warehouse:before{content:\"\\e644\"}.bosch-ic-world-paperplane:before{content:\"\\e645\"}.bosch-ic-aperture:before{content:\"\\e646\"}.bosch-ic-hook-wiperblade-arrows:before{content:\"\\e647\"}.bosch-ic-image-remove:before{content:\"\\e648\"}.bosch-ic-image-checked:before{content:\"\\e649\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-apartment-in-house-terraced:before{content:\"\\e64b\"}.bosch-ic-apartment-in-house:before{content:\"\\e64c\"}.bosch-ic-appliance-heat-pump-leaf-add:before{content:\"\\e64d\"}.bosch-ic-appliance-heat-pump-leaf-reload:before{content:\"\\e64e\"}.bosch-ic-appliance-heat-pump-oil-refresh:before{content:\"\\e64f\"}.bosch-ic-appliance-water-drop:before{content:\"\\e650\"}.bosch-ic-appliance-water-sun:before{content:\"\\e651\"}.bosch-ic-brake-pedal-thumb-up:before{content:\"\\e652\"}.bosch-ic-call-plus:before{content:\"\\e653\"}.bosch-ic-battery-car-3-plug:before{content:\"\\e654\"}.bosch-ic-cloud-share:before{content:\"\\e655\"}.bosch-ic-crankshaft:before{content:\"\\e656\"}.bosch-ic-detector-motion:before{content:\"\\e657\"}.bosch-ic-device-analog:before{content:\"\\e658\"}.bosch-ic-device-digital:before{content:\"\\e659\"}.bosch-ic-externallink-landscape:before{content:\"\\e65a\"}.bosch-ic-gears-interlocked-arrow:before{content:\"\\e65b\"}.bosch-ic-gearshift-reverse-gear:before{content:\"\\e65c\"}.bosch-ic-glove:before{content:\"\\e65d\"}.bosch-ic-hand-warranty:before{content:\"\\e65e\"}.bosch-ic-heating-floor-radiator:before{content:\"\\e65f\"}.bosch-ic-home-appliance-heat-pump-air-in:before{content:\"\\e660\"}.bosch-ic-home-appliance-heat-pump-air-out:before{content:\"\\e661\"}.bosch-ic-lightbulb-gear:before{content:\"\\e662\"}.bosch-ic-line-open-closed:before{content:\"\\e663\"}.bosch-ic-road-disabled-bold:before{content:\"\\e664\"}.bosch-ic-road-disabled-light:before{content:\"\\e665\"}.bosch-ic-shower-drop-1:before{content:\"\\e666\"}.bosch-ic-shower-drop-2:before{content:\"\\e667\"}.bosch-ic-shower-drop-3:before{content:\"\\e668\"}.bosch-ic-shower-drop-4:before{content:\"\\e669\"}.bosch-ic-sinus-curve:before{content:\"\\e66a\"}.bosch-ic-team-3:before{content:\"\\e66b\"}.bosch-ic-team-4:before{content:\"\\e66c\"}.bosch-ic-team-5:before{content:\"\\e66d\"}.bosch-ic-test-tube:before{content:\"\\e66e\"}.bosch-ic-truck-car-motorcycle-arrows:before{content:\"\\e66f\"}.bosch-ic-weather-cloud-dust:before{content:\"\\e670\"}.bosch-ic-windshield-wiper-double-parallel-left:before{content:\"\\e671\"}.bosch-ic-windshield-wiper-double-parallel-right:before{content:\"\\e672\"}.bosch-ic-world-pin:before{content:\"\\e673\"}.bosch-ic-sign-warning-slope:before{content:\"\\e674\"}.bosch-ic-facebook:before{content:\"\\ea99\"}.bosch-ic-google-frame:before{content:\"\\ea9a\"}.bosch-ic-google:before{content:\"\\ea9b\"}.bosch-ic-googleplus:before{content:\"\\ea9c\"}.bosch-ic-instagram:before{content:\"\\ea9d\"}.bosch-ic-lineapp:before{content:\"\\ea9e\"}.bosch-ic-linkedin:before{content:\"\\ea9f\"}.bosch-ic-pinterest:before{content:\"\\eaa0\"}.bosch-ic-skype:before{content:\"\\eaa1\"}.bosch-ic-snapchat:before{content:\"\\eaa2\"}.bosch-ic-tumblr:before{content:\"\\eaa3\"}.bosch-ic-twitter:before{content:\"\\eaa4\"}.bosch-ic-vimeo:before{content:\"\\eaa5\"}.bosch-ic-vk:before{content:\"\\eaa6\"}.bosch-ic-wechat:before{content:\"\\eaa7\"}.bosch-ic-weibo:before{content:\"\\eaa8\"}.bosch-ic-whatsapp:before{content:\"\\eaa9\"}.bosch-ic-xing:before{content:\"\\eaaa\"}.bosch-ic-youku:before{content:\"\\eaab\"}.bosch-ic-youtube:before{content:\"\\eaac\"}.bosch-ic-alert-info-filled:before{content:\"\\eb0e\"}.bosch-ic-alert-info:before{content:\"\\eb0f\"}.bosch-ic-info-i-frame:before{content:\"\\e8ab\"}.bosch-ic-stop-frame1:before{content:\"\\e8ac\"}.bosch-ic-arrow-up-frame-error1:before{content:\"\\e8ad\"}.bosch-ic-arrow-up-frame-check1:before{content:\"\\e8ae\"}.bosch-ic-arrow-left-end-frame1:before{content:\"\\e8af\"}.bosch-ic-arrow-right-end-frame1:before{content:\"\\e8b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-warranty:before{content:\"\\e2b1\"}.bosch-ic-bond-laser:before{content:\"\\e675\"}.bosch-ic-bond-ribbon-thick:before{content:\"\\e676\"}.bosch-ic-bond-ribbon-thin:before{content:\"\\e677\"}.bosch-ic-bond-wire-thick:before{content:\"\\e678\"}.bosch-ic-bond-wire-thin:before{content:\"\\e679\"}.bosch-ic-clamp-connection-insulation:before{content:\"\\e67a\"}.bosch-ic-clamp-connection-stamping-grid:before{content:\"\\e67b\"}.bosch-ic-clinching:before{content:\"\\e67c\"}.bosch-ic-connector:before{content:\"\\e67d\"}.bosch-ic-connector-potting:before{content:\"\\e67e\"}.bosch-ic-element-damping:before{content:\"\\e67f\"}.bosch-ic-element-pressure-compensation:before{content:\"\\e680\"}.bosch-ic-flanging:before{content:\"\\e681\"}.bosch-ic-gap-pad:before{content:\"\\e682\"}.bosch-ic-heat-sink:before{content:\"\\e683\"}.bosch-ic-hook-clipsing:before{content:\"\\e684\"}.bosch-ic-interconnection-board-to-board:before{content:\"\\e685\"}.bosch-ic-labyrinth-geometry:before{content:\"\\e686\"}.bosch-ic-lacquering-pcb:before{content:\"\\e687\"}.bosch-ic-magnetic-rotor:before{content:\"\\e688\"}.bosch-ic-magnetic-switch:before{content:\"\\e689\"}.bosch-ic-pin-pressfit:before{content:\"\\e68a\"}.bosch-ic-power-contact:before{content:\"\\e68b\"}.bosch-ic-rivet:before{content:\"\\e68c\"}.bosch-ic-seal-ring:before{content:\"\\e68d\"}.bosch-ic-sealing-cipg:before{content:\"\\e68e\"}.bosch-ic-sealing-solid:before{content:\"\\e68f\"}.bosch-ic-sealing-tongue-groove:before{content:\"\\e690\"}.bosch-ic-shrinking-arrows-circle:before{content:\"\\e691\"}.bosch-ic-solder-selective:before{content:\"\\e692\"}.bosch-ic-spring-mechanical:before{content:\"\\e693\"}.bosch-ic-staking-hot-air:before{content:\"\\e694\"}.bosch-ic-stitching:before{content:\"\\e695\"}.bosch-ic-thermal-inlay:before{content:\"\\e696\"}.bosch-ic-thermal-via:before{content:\"\\e697\"}.bosch-ic-tube:before{content:\"\\e698\"}.bosch-ic-tube-gapfiller:before{content:\"\\e699\"}.bosch-ic-tube-gelling:before{content:\"\\e69a\"}.bosch-ic-tube-gluing:before{content:\"\\e69b\"}.bosch-ic-tube-thermal-adhesive:before{content:\"\\e69c\"}.bosch-ic-tube-underfill:before{content:\"\\e69d\"}.bosch-ic-welding-laser:before{content:\"\\e69e\"}.bosch-ic-welding-resistance:before{content:\"\\e69f\"}.bosch-ic-welding-ultrasonic:before{content:\"\\e6a0\"}.bosch-ic-air-condition-cassette:before{content:\"\\e6a1\"}.bosch-ic-air-condition-ceiling:before{content:\"\\e6a2\"}.bosch-ic-air-condition-convertible:before{content:\"\\e6a3\"}.bosch-ic-air-condition-duct:before{content:\"\\e6a4\"}.bosch-ic-air-condition-floor:before{content:\"\\e6a5\"}.bosch-ic-air-condition-outdoor:before{content:\"\\e6a6\"}.bosch-ic-air-condition-outdoor-hp:before{content:\"\\e6a7\"}.bosch-ic-air-condition-outdoor-hr:before{content:\"\\e6a8\"}.bosch-ic-air-condition-swing-horizontal:before{content:\"\\e6a9\"}.bosch-ic-air-condition-swing-vertical:before{content:\"\\e6aa\"}.bosch-ic-air-condition-wall:before{content:\"\\e6ab\"}.bosch-ic-angle-curvature-motorcycle:before{content:\"\\e6ac\"}.bosch-ic-appliance-hot:before{content:\"\\e6ad\"}.bosch-ic-appliance-water-hot:before{content:\"\\e6ae\"}.bosch-ic-appliances-hot:before{content:\"\\e6af\"}.bosch-ic-arrows-left-right-a:before{content:\"\\e6b0\"}.bosch-ic-arrows-left-right-exchange:before{content:\"\\e6b1\"}.bosch-ic-arrows-left-right-exchange-unit:before{content:\"\\e6b2\"}.bosch-ic-arrows-left-right-ice:before{content:\"\\e6b3\"}.bosch-ic-assembly-line-robot:before{content:\"\\e6b4\"}.bosch-ic-back-left-small:before{content:\"\\e6b5\"}.bosch-ic-bacteria:before{content:\"\\e6b6\"}.bosch-ic-battery-car-drop-leakage-proof:before{content:\"\\e6b7\"}.bosch-ic-battery-car-hourglass:before{content:\"\\e6b8\"}.bosch-ic-brake-disk-arrow:before{content:\"\\e6b9\"}.bosch-ic-brake-disk-compound:before{content:\"\\e6ba\"}.bosch-ic-building-religious-church:before{content:\"\\e6bb\"}.bosch-ic-calendar-sheet:before{content:\"\\e6bc\"}.bosch-ic-caliper:before{content:\"\\e6bd\"}.bosch-ic-car-middle-arrow:before{content:\"\\e6be\"}.bosch-ic-checkmark-bold:before{content:\"\\e6bf\"}.bosch-ic-close-small:before{content:\"\\e6c0\"}.bosch-ic-data-center:before{content:\"\\e6c1\"}.bosch-ic-dot:before{content:\"\\e6c2\"}.bosch-ic-down-small:before{content:\"\\e6c3\"}.bosch-ic-e-call:before{content:\"\\e6c4\"}.bosch-ic-fire-extinguisher:before{content:\"\\e6c5\"}.bosch-ic-fire-off-bold:before{content:\"\\e6c6\"}.bosch-ic-fire-off-light:before{content:\"\\e6c7\"}.bosch-ic-forward-right-small:before{content:\"\\e6c8\"}.bosch-ic-health-warning:before{content:\"\\e6c9\"}.bosch-ic-ignition-coil-pencil:before{content:\"\\e6ca\"}.bosch-ic-ignition-condensor:before{content:\"\\e6cb\"}.bosch-ic-ignition-contact-set:before{content:\"\\e6cc\"}.bosch-ic-ignition-distributor-cap:before{content:\"\\e6cd\"}.bosch-ic-ignition-distributor-rotor:before{content:\"\\e6ce\"}.bosch-ic-ignition-module:before{content:\"\\e6cf\"}.bosch-ic-less-minimize-bold:before{content:\"\\e6d0\"}.bosch-ic-leaf-plus-boost:before{content:\"\\e6d1\"}.bosch-ic-sensor-dynamic:before{content:\"\\e6d2\"}.bosch-ic-sensor-radar:before{content:\"\\e6d3\"}.bosch-ic-up-small:before{content:\"\\e6d4\"}.bosch-ic-water-ladder-off-bold:before{content:\"\\e6d5\"}.bosch-ic-water-ladder-off-light:before{content:\"\\e6d6\"}.bosch-ic-nose-desodorize:before{content:\"\\e6d7\"}.bosch-ic-nose-fine-dust:before{content:\"\\e6d8\"}.bosch-ic-smartcity-camera:before{content:\"\\e6d9\"}.bosch-ic-heating-underfloor:before{content:\"\\e6da\"}.bosch-ic-drill-bit:before{content:\"\\e6db\"}.bosch-ic-parameter-process:before{content:\"\\e6dc\"}.bosch-ic-car-side-plug-flash:before{content:\"\\e6dd\"}.bosch-ic-car-side-plug-engine-flash:before{content:\"\\e6de\"}.bosch-ic-car-side-engine-flash:before{content:\"\\e6df\"}.bosch-ic-car-side-hydrogen-flash:before{content:\"\\e6e0\"}.bosch-ic-car-side-engine:before{content:\"\\e6e1\"}.bosch-ic-tunnel:before{content:\"\\e6e2\"}.bosch-ic-stadium-sport:before{content:\"\\e6e3\"}.bosch-ic-refinery-oil:before{content:\"\\e6e4\"}.bosch-ic-power-plants:before{content:\"\\e6e5\"}.bosch-ic-hammer-chisel:before{content:\"\\e6e6\"}.bosch-ic-glass-cocktail:before{content:\"\\e6e7\"}.bosch-ic-traffic-light:before{content:\"\\e6e8\"}.bosch-ic-vehicle-off-road:before{content:\"\\e6e9\"}.bosch-ic-tablet-finger:before{content:\"\\e6ea\"}.bosch-ic-scooter-kick:before{content:\"\\e6eb\"}.bosch-ic-handlebar-motorcycle:before{content:\"\\e6ec\"}.bosch-ic-display-motorcycle:before{content:\"\\e6ed\"}.bosch-ic-case-label-product:before{content:\"\\e6ee\"}.bosch-ic-case-label:before{content:\"\\e6ef\"}.bosch-ic-product-falling:before{content:\"\\e6f0\"}.bosch-ic-product-checkmark:before{content:\"\\e6f1\"}.bosch-ic-product-broken:before{content:\"\\e6f2\"}.bosch-ic-lightbulb-halogen-rays-bright:before{content:\"\\e6f3\"}.bosch-ic-lightbulb-halogen-rays:before{content:\"\\e6f4\"}.bosch-ic-lamp-gas-discharge-rays:before{content:\"\\e6f5\"}.bosch-ic-home-lightbulb-off-disabled-bold:before{content:\"\\e6f6\"}.bosch-ic-home-lightbulb-off-disabled-light:before{content:\"\\e6f7\"}.bosch-ic-fork-lift-boxes:before{content:\"\\e6f8\"}.bosch-ic-harbour-ship-cargo:before{content:\"\\e6f9\"}.bosch-ic-rollercoaster:before{content:\"\\e6fa\"}.bosch-ic-sound-off-light:before{content:\"\\e6fb\"}.bosch-ic-sound-off-bold:before{content:\"\\e6fc\"}.bosch-ic-car-door:before{content:\"\\e6fd\"}.bosch-ic-car-door-tailgate:before{content:\"\\e6fe\"}.bosch-ic-car-door-split-rear:before{content:\"\\e6ff\"}.bosch-ic-car-door-rear:before{content:\"\\e700\"}.bosch-ic-drill-driver-cordless:before{content:\"\\e701\"}.bosch-ic-virus:before{content:\"\\e702\"}.bosch-ic-ice-temperature:before{content:\"\\e703\"}.bosch-ic-sensor-ultrasonic:before{content:\"\\e704\"}.bosch-ic-user-artificial-intelligence:before{content:\"\\e705\"}.bosch-ic-detector-leak:before{content:\"\\e706\"}.bosch-ic-car-seat-add:before{content:\"\\e707\"}.bosch-ic-car-shuttle-robo:before{content:\"\\e708\"}.bosch-ic-caravan:before{content:\"\\e709\"}.bosch-ic-user-water-swimming:before{content:\"\\e70a\"}.bosch-ic-square-add:before{content:\"\\e70b\"}.bosch-ic-map-zoom-in:before{content:\"\\e70c\"}.bosch-ic-map-arrow-pop-out:before{content:\"\\e70d\"}.bosch-ic-home-user-in-error:before{content:\"\\e70e\"}.bosch-ic-home-arrow-right-in-denied:before{content:\"\\e70f\"}.bosch-ic-home-arrow-right-in-check:before{content:\"\\e710\"}.bosch-ic-home-arrow-right-in:before{content:\"\\e711\"}.bosch-ic-home-arrow-left-out:before{content:\"\\e712\"}.bosch-ic-file-error:before{content:\"\\e713\"}.bosch-ic-dealer-details-missing:before{content:\"\\e714\"}.bosch-ic-dealer-details-check:before{content:\"\\e715\"}.bosch-ic-components-add:before{content:\"\\e716\"}.bosch-ic-machine-cell:before{content:\"\\e717\"}.bosch-ic-wheel-rim-clean:before{content:\"\\e718\"}.bosch-ic-gas-can:before{content:\"\\e719\"}.bosch-ic-steps-locator:before{content:\"\\e71a\"}.bosch-ic-panel-control-manual:before{content:\"\\e71b\"}.bosch-ic-socket-europe:before{content:\"\\e71c\"}.bosch-ic-tripod:before{content:\"\\e71d\"}.bosch-ic-document-dxf:before{content:\"\\e71e\"}.bosch-ic-document-dwg:before{content:\"\\e71f\"}.bosch-ic-hook-wiperblade:before{content:\"\\e720\"}.bosch-ic-traffic-cone:before{content:\"\\e721\"}.bosch-ic-folder-locked:before{content:\"\\e722\"}.bosch-ic-package-self-service:before{content:\"\\e723\"}.bosch-ic-drop-ice:before{content:\"\\e724\"}.bosch-ic-lamp-gas-discharge:before{content:\"\\e725\"}.bosch-ic-home-basement:before{content:\"\\e726\"}.bosch-ic-link-edit:before{content:\"\\e727\"}.bosch-ic-table-header-add:before{content:\"\\e728\"}.bosch-ic-table-column-add-after:before{content:\"\\e729\"}.bosch-ic-table-row-add-above:before{content:\"\\e72a\"}.bosch-ic-table-column-add-before:before{content:\"\\e72b\"}.bosch-ic-table-row-add-below:before{content:\"\\e72c\"}.bosch-ic-table-cells-split:before{content:\"\\e72d\"}.bosch-ic-table-cells-merge:before{content:\"\\e72e\"}.bosch-ic-table-row-delete:before{content:\"\\e72f\"}.bosch-ic-text-indent-arrow:before{content:\"\\e730\"}.bosch-ic-text-outdent-arrow:before{content:\"\\e731\"}.bosch-ic-car-side-flash:before{content:\"\\e732\"}.bosch-ic-battery-plug-flash:before{content:\"\\e733\"}.bosch-ic-car-chip:before{content:\"\\e734\"}.bosch-ic-car-cloud-upload:before{content:\"\\e735\"}.bosch-ic-circle-parking:before{content:\"\\e736\"}.bosch-ic-engine-battery-flash:before{content:\"\\e737\"}.bosch-ic-engine-battery-flash-plug:before{content:\"\\e738\"}.bosch-ic-engine-combustion:before{content:\"\\e739\"}.bosch-ic-user-head-scan:before{content:\"\\e73a\"}.bosch-ic-bluetooth-off-disabled-bold:before{content:\"\\e73b\"}.bosch-ic-bluetooth-off-disabled-light:before{content:\"\\e73c\"}.bosch-ic-communicator-off-disabled-bold:before{content:\"\\e73d\"}.bosch-ic-communicator-off-disabled-light:before{content:\"\\e73e\"}.bosch-ic-connectivity-off-disabled-bold:before{content:\"\\e73f\"}.bosch-ic-connectivity-off-disabled-light:before{content:\"\\e740\"}.bosch-ic-globe-off-disabled-bold:before{content:\"\\e741\"}.bosch-ic-globe-off-disabled-light:before{content:\"\\e742\"}.bosch-ic-lamp-led:before{content:\"\\e743\"}.bosch-ic-wifi-off-disabled-bold:before{content:\"\\e744\"}.bosch-ic-wifi-off-disabled-light:before{content:\"\\e745\"}.bosch-ic-wheel-vehicle:before{content:\"\\e746\"}.bosch-ic-ignition-cable:before{content:\"\\e747\"}.bosch-ic-ignition-coil:before{content:\"\\e748\"}.bosch-ic-ignition-distributor:before{content:\"\\e749\"}.bosch-ic-microphone-classic-off-disabled-light:before{content:\"\\e74a\"}.bosch-ic-microphone-classic-off-disabled-bold:before{content:\"\\e74b\"}.bosch-ic-cloud-refresh:before{content:\"\\e74c\"}.bosch-ic-ignition-coil-module:before{content:\"\\e74d\"}.bosch-ic-bluetooth-frame:before{content:\"\\e8b1\"}.bosch-ic-bluetooth-frame-error:before{content:\"\\e8b2\"}.bosch-ic-ece-ccc-frame:before{content:\"\\e8b3\"}.bosch-ic-ms-teams:before{content:\"\\eaad\"}.bosch-ic-ms-teams-frame:before{content:\"\\eaae\"}.bosch-ic-mold:before{content:\"\\f0000\"}.bosch-ic-battery-car-cranking-power-high:before{content:\"\\f0001\"}.bosch-ic-brake-disk-spray-can-coated:before{content:\"\\f0002\"}.bosch-ic-car-top-arrows-left-front-right-back:before{content:\"\\f0003\"}.bosch-ic-car-top-arrows-left-back-right-front:before{content:\"\\f0004\"}.bosch-ic-car-top-arrows-front:before{content:\"\\f0005\"}.bosch-ic-car-top-arrows-back-front-left-right:before{content:\"\\f0006\"}.bosch-ic-car-top-arrows-back:before{content:\"\\f0007\"}.bosch-ic-car-top-arrow-right:before{content:\"\\f0008\"}.bosch-ic-car-top-arrow-left:before{content:\"\\f0009\"}.bosch-ic-car-top-arrow-front-right:before{content:\"\\f000a\"}.bosch-ic-car-top-arrow-front-left:before{content:\"\\f000b\"}.bosch-ic-car-top-arrow-back-right:before{content:\"\\f000c\"}.bosch-ic-car-top-arrow-back-left:before{content:\"\\f000d\"}.bosch-ic-watermelon:before{content:\"\\f000e\"}.bosch-ic-wind-vane-moderate:before{content:\"\\f000f\"}.bosch-ic-radioactive:before{content:\"\\f0010\"}.bosch-ic-heating-underfloor-off-bold:before{content:\"\\f0011\"}.bosch-ic-heating-underfloor-off-light:before{content:\"\\f0012\"}.bosch-ic-goat:before{content:\"\\f0013\"}.bosch-ic-fire-hose:before{content:\"\\f0014\"}.bosch-ic-user-run-arrow:before{content:\"\\f0015\"}.bosch-ic-store:before{content:\"\\f0016\"}.bosch-ic-user-work-dig:before{content:\"\\f0017\"}.bosch-ic-car-top-arrow-angle-right:before{content:\"\\f0018\"}.bosch-ic-control-unit-ecu:before{content:\"\\f0019\"}.bosch-ic-cigarette:before{content:\"\\f001a\"}.bosch-ic-structure-line:before{content:\"\\f001b\"}.bosch-ic-product-database-table:before{content:\"\\f001c\"}.bosch-ic-bicycle-e-remote-led:before{content:\"\\f001d\"}.bosch-ic-bicycle-e-drive-unit:before{content:\"\\f001e\"}.bosch-ic-bicycle-e-drive-unit-individual-support:before{content:\"\\f001f\"}.bosch-ic-nm-arrow-circle:before{content:\"\\f0020\"}.bosch-ic-tree-path:before{content:\"\\f0021\"}.bosch-ic-bicycle-e-wheel-arrows:before{content:\"\\f0022\"}.bosch-ic-tune-vertical:before{content:\"\\f0023\"}.bosch-ic-sound-professional:before{content:\"\\f0024\"}.bosch-ic-plane-top-headset:before{content:\"\\f0025\"}.bosch-ic-desktop-user-headset:before{content:\"\\f0026\"}.bosch-ic-user-service:before{content:\"\\f0027\"}.bosch-ic-audio-commercial:before{content:\"\\f0028\"}.bosch-ic-loudspeaker-portable-note:before{content:\"\\f0029\"}.bosch-ic-audio-loudspeaker-public:before{content:\"\\f002a\"}.bosch-ic-items-lost-questionmark:before{content:\"\\f002b\"}.bosch-ic-hand-pull:before{content:\"\\f002c\"}.bosch-ic-steering-wheel-hand-horn:before{content:\"\\f002d\"}.bosch-ic-car-chassis:before{content:\"\\f002e\"}.bosch-ic-steering-system:before{content:\"\\f002f\"}.bosch-ic-wheel-suspension:before{content:\"\\f0030\"}.bosch-ic-car-body-side:before{content:\"\\f0031\"}.bosch-ic-address-consumer-data-upright:before{content:\"\\f0032\"}.bosch-ic-weather-wind-rain:before{content:\"\\f0033\"}.bosch-ic-weather-rain-warning:before{content:\"\\f0034\"}.bosch-ic-user-check:before{content:\"\\f0035\"}.bosch-ic-user-warning:before{content:\"\\f0036\"}.bosch-ic-black-white-stars-circle:before{content:\"\\f0037\"}.bosch-ic-team-warning:before{content:\"\\f0038\"}.bosch-ic-black-white-circle:before{content:\"\\f0039\"}.bosch-ic-suitcase-small:before{content:\"\\f003a\"}.bosch-ic-suitcase-small-remove:before{content:\"\\f003b\"}.bosch-ic-home-arrow-out:before{content:\"\\f003c\"}.bosch-ic-home-arrow-in:before{content:\"\\f003d\"}.bosch-ic-document-copy-check:before{content:\"\\f003e\"}.bosch-ic-ring-infrared:before{content:\"\\f003f\"}.bosch-ic-spirit-level:before{content:\"\\f0040\"}.bosch-ic-drill-driver-check:before{content:\"\\f0041\"}.bosch-ic-desktop-led:before{content:\"\\f0042\"}.bosch-ic-desktop-black-white:before{content:\"\\f0043\"}.bosch-ic-desktop-warning:before{content:\"\\f0044\"}.bosch-ic-fire-wall:before{content:\"\\f0045\"}.bosch-ic-onroad-car-top:before{content:\"\\f0046\"}.bosch-ic-onroad-suv-top:before{content:\"\\f0047\"}.bosch-ic-onroad-truck-top:before{content:\"\\f0048\"}.bosch-ic-weather-rain-ip44:before{content:\"\\f0049\"}.bosch-ic-weather-rain-ip65:before{content:\"\\f004a\"}.bosch-ic-weather-rain-ip66:before{content:\"\\f004b\"}.bosch-ic-weather-rain-ip68:before{content:\"\\f004c\"}.bosch-ic-weather-rain-ip67:before{content:\"\\f004d\"}.bosch-ic-user-masking:before{content:\"\\f004e\"}.bosch-ic-address-consumer-data-dual:before{content:\"\\f004f\"}.bosch-ic-user-framed:before{content:\"\\f0050\"}.bosch-ic-user-counting-1:before{content:\"\\f0051\"}.bosch-ic-square-kx-uhd:before{content:\"\\f0052\"}.bosch-ic-square-qvga:before{content:\"\\f0053\"}.bosch-ic-square-vga:before{content:\"\\f0054\"}.bosch-ic-square-xmp:before{content:\"\\f0055\"}.bosch-ic-square-xxxp:before{content:\"\\f0056\"}.bosch-ic-angle-arrows-across:before{content:\"\\f0057\"}.bosch-ic-angle:before{content:\"\\f0058\"}.bosch-ic-angle-arrow-up:before{content:\"\\f0059\"}.bosch-ic-angle-view-wide:before{content:\"\\f005a\"}.bosch-ic-chat-language-selection:before{content:\"\\f005b\"}.bosch-ic-smartphone-app:before{content:\"\\f005c\"}.bosch-ic-commissioning-remote:before{content:\"\\f005d\"}.bosch-ic-user-run-detection-pir:before{content:\"\\f005e\"}.bosch-ic-field-in-motion:before{content:\"\\f005f\"}.bosch-ic-building-integration-system:before{content:\"\\f0060\"}.bosch-ic-desktop-mamangement:before{content:\"\\f0061\"}.bosch-ic-desktop-management-camera:before{content:\"\\f0062\"}.bosch-ic-screwdriver-check:before{content:\"\\f0063\"}.bosch-ic-screwdriver:before{content:\"\\f0064\"}.bosch-ic-desktop-hand-touch:before{content:\"\\f0065\"}.bosch-ic-user-robot-head:before{content:\"\\f0066\"}.bosch-ic-gear-circles-elements:before{content:\"\\f0067\"}.bosch-ic-gears-3:before{content:\"\\f0068\"}.bosch-ic-door-address-consumer-data:before{content:\"\\f0069\"}.bosch-ic-lock-closed-128bit:before{content:\"\\f006a\"}.bosch-ic-lock-closed-connected-wps:before{content:\"\\f006b\"}.bosch-ic-battery-arrow-right:before{content:\"\\f006c\"}.bosch-ic-connector-bnc:before{content:\"\\f006d\"}.bosch-ic-tower-connection-cdma:before{content:\"\\f006e\"}.bosch-ic-tower-connection-gprs:before{content:\"\\f006f\"}.bosch-ic-tritech:before{content:\"\\f0070\"}.bosch-ic-quad-sensor-two:before{content:\"\\f0071\"}.bosch-ic-dynamic-range-hdr:before{content:\"\\f0072\"}.bosch-ic-dynamic-range-wdr:before{content:\"\\f0073\"}.bosch-ic-battery-thumb-up-arrow-right:before{content:\"\\f0074\"}.bosch-ic-screwdriver-thumb-up:before{content:\"\\f0075\"}.bosch-ic-thumb-up-arrow-up:before{content:\"\\f0076\"}.bosch-ic-core-data-sql:before{content:\"\\f0077\"}.bosch-ic-frequency-869:before{content:\"\\f0078\"}.bosch-ic-coins:before{content:\"\\f0079\"}.bosch-ic-frames-60:before{content:\"\\f007a\"}.bosch-ic-hand-card:before{content:\"\\f007b\"}.bosch-ic-microphone-plugabble:before{content:\"\\f007c\"}.bosch-ic-video-h264:before{content:\"\\f007d\"}.bosch-ic-video-hd:before{content:\"\\f007e\"}.bosch-ic-video-mpeg:before{content:\"\\f007f\"}.bosch-ic-video-hd-4k:before{content:\"\\f0080\"}.bosch-ic-video-hd-720:before{content:\"\\f0081\"}.bosch-ic-lasso:before{content:\"\\f0082\"}.bosch-ic-lasso-remove:before{content:\"\\f0083\"}.bosch-ic-battery-car-3-clock:before{content:\"\\f0084\"}.bosch-ic-battery-car-agm:before{content:\"\\f0085\"}.bosch-ic-battery-car-flash:before{content:\"\\f0086\"}.bosch-ic-filter-binary:before{content:\"\\f0087\"}.bosch-ic-speedometer-bitrate-reduce:before{content:\"\\f0088\"}.bosch-ic-voting:before{content:\"\\f0089\"}.bosch-ic-voting-dual:before{content:\"\\f008a\"}.bosch-ic-sound-pressure-level-90:before{content:\"\\f008b\"}.bosch-ic-sound-pressure-level-88:before{content:\"\\f008c\"}.bosch-ic-sound-pressure-level-80:before{content:\"\\f008d\"}.bosch-ic-power-rated-20:before{content:\"\\f008e\"}.bosch-ic-labyrinth-geometry-integration:before{content:\"\\f008f\"}.bosch-ic-head-speaking-voice-2:before{content:\"\\f0090\"}.bosch-ic-volume-microphone:before{content:\"\\f0091\"}.bosch-ic-user-walk-route:before{content:\"\\f0092\"}.bosch-ic-user-walk-line-crossing:before{content:\"\\f0093\"}.bosch-ic-user-walk-line-count:before{content:\"\\f0094\"}.bosch-ic-fullscreen-arrows-circle:before{content:\"\\f0095\"}.bosch-ic-field-arrows-counter-flow:before{content:\"\\f0096\"}.bosch-ic-field-arrows-directed-flow:before{content:\"\\f0097\"}.bosch-ic-user-birds-eye-count-12:before{content:\"\\f0098\"}.bosch-ic-field-user-count-12:before{content:\"\\f0099\"}.bosch-ic-field-user-falling:before{content:\"\\f009a\"}.bosch-ic-field-user-loitering-arrow:before{content:\"\\f009b\"}.bosch-ic-field-user-similarity:before{content:\"\\f009c\"}.bosch-ic-field-user-walk:before{content:\"\\f009d\"}.bosch-ic-field-user-walk-arrow-behind:before{content:\"\\f009e\"}.bosch-ic-field-user-walk-arrow-in-front:before{content:\"\\f009f\"}.bosch-ic-field-users:before{content:\"\\f00a0\"}.bosch-ic-shock-resistant:before{content:\"\\f00a1\"}.bosch-ic-bus-sdi:before{content:\"\\f00a2\"}.bosch-ic-bus-sdi2:before{content:\"\\f00a3\"}.bosch-ic-folder-ip-net:before{content:\"\\f00a4\"}.bosch-ic-globe-iscsi:before{content:\"\\f00a5\"}.bosch-ic-offline-www:before{content:\"\\f00a6\"}.bosch-ic-opc:before{content:\"\\f00a7\"}.bosch-ic-osdp:before{content:\"\\f00a8\"}.bosch-ic-settings-connected-ptrz:before{content:\"\\f00a9\"}.bosch-ic-raid:before{content:\"\\f00aa\"}.bosch-ic-lsn:before{content:\"\\f00ab\"}.bosch-ic-pointbus:before{content:\"\\f00ac\"}.bosch-ic-male-count-12:before{content:\"\\f00ad\"}.bosch-ic-circle-point-record:before{content:\"\\f00ae\"}.bosch-ic-plug-poe:before{content:\"\\f00af\"}.bosch-ic-ir-distribution:before{content:\"\\f00b0\"}.bosch-ic-ir-range:before{content:\"\\f00b1\"}.bosch-ic-connectivity-it:before{content:\"\\f00b2\"}.bosch-ic-music-note-plus:before{content:\"\\f00b3\"}.bosch-ic-music-note-plus-two:before{content:\"\\f00b4\"}.bosch-ic-chat-language-chn:before{content:\"\\f00b5\"}.bosch-ic-microphone-high-directive:before{content:\"\\f00b6\"}.bosch-ic-document-copy-rg-gf:before{content:\"\\f00b7\"}.bosch-ic-document-copy-cdq:before{content:\"\\f00b8\"}.bosch-ic-document-copy-cgp:before{content:\"\\f00b9\"}.bosch-ic-folder-add:before{content:\"\\f00ba\"}.bosch-ic-user-tie:before{content:\"\\f00bb\"}.bosch-ic-pill:before{content:\"\\f00bc\"}.bosch-ic-sonotrode:before{content:\"\\f00bd\"}.bosch-ic-user-thumb-up:before{content:\"\\f00be\"}.bosch-ic-architecture:before{content:\"\\f00bf\"}.bosch-ic-clamping-area-arrows:before{content:\"\\f00c0\"}.bosch-ic-clamping-rib:before{content:\"\\f00c1\"}.bosch-ic-connection-plug-in:before{content:\"\\f00c2\"}.bosch-ic-connection-plug-in-electrical:before{content:\"\\f00c3\"}.bosch-ic-connection-plug-in-socket:before{content:\"\\f00c4\"}.bosch-ic-connector-shield-emc:before{content:\"\\f00c5\"}.bosch-ic-foil-flex:before{content:\"\\f00c6\"}.bosch-ic-grid-fan:before{content:\"\\f00c7\"}.bosch-ic-screw-nut:before{content:\"\\f00c8\"}.bosch-ic-sealing-groove:before{content:\"\\f00c9\"}.bosch-ic-sealing-tongue-groove-flash:before{content:\"\\f00ca\"}.bosch-ic-solder:before{content:\"\\f00cb\"}.bosch-ic-solder-selective-ssl:before{content:\"\\f00cc\"}.bosch-ic-solder-selective-thrs:before{content:\"\\f00cd\"}.bosch-ic-tuner-box:before{content:\"\\f00ce\"}.bosch-ic-water-cooling-arrow-up:before{content:\"\\f00cf\"}.bosch-ic-generic-device-connected:before{content:\"\\f00d0\"}.bosch-ic-home-sun-flash:before{content:\"\\f00d1\"}.bosch-ic-home-sun-wrench:before{content:\"\\f00d2\"}.bosch-ic-temperature-lock:before{content:\"\\f00d3\"}.bosch-ic-solder-method-smd:before{content:\"\\f00d4\"}.bosch-ic-pins:before{content:\"\\f00d5\"}.bosch-ic-connector-housing:before{content:\"\\f00d6\"}.bosch-ic-antenna-cover-arrow-up:before{content:\"\\f00d7\"}.bosch-ic-angle-open-horizontal-130:before{content:\"\\f00d8\"}.bosch-ic-angle-open-vertical-130:before{content:\"\\f00d9\"}.bosch-ic-angle-open-vertical-90:before{content:\"\\f00da\"}.bosch-ic-angle-open-horizontal-145:before{content:\"\\f00db\"}.bosch-ic-steering-system-steer-by-wire:before{content:\"\\f00dc\"}.bosch-ic-tune-vertical-music-note:before{content:\"\\f00dd\"}.bosch-ic-hammer-movement:before{content:\"\\f00de\"}.bosch-ic-drill-drive-percent-35:before{content:\"\\f00df\"}.bosch-ic-country-switzerland:before{content:\"\\f00e0\"}.bosch-ic-speedlimit-revoking:before{content:\"\\f00e1\"}.bosch-ic-speedlimit-80-revoking:before{content:\"\\f00e2\"}.bosch-ic-speedlimit-80-dirty:before{content:\"\\f00e3\"}.bosch-ic-battery-car-mf:before{content:\"\\f00e4\"}.bosch-ic-access-controller-2:before{content:\"\\f00e5\"}.bosch-ic-access-controller-arrow-up:before{content:\"\\f00e6\"}.bosch-ic-access-controller-points-set:before{content:\"\\f00e7\"}.bosch-ic-analog-value:before{content:\"\\f00e8\"}.bosch-ic-analog-value-arrow-in:before{content:\"\\f00e9\"}.bosch-ic-analog-value-arrow-out:before{content:\"\\f00ea\"}.bosch-ic-binary-value:before{content:\"\\f00eb\"}.bosch-ic-binary-value-arrow-in:before{content:\"\\f00ec\"}.bosch-ic-binary-value-arrow-out:before{content:\"\\f00ed\"}.bosch-ic-calendar-date-single:before{content:\"\\f00ee\"}.bosch-ic-calendar-date-weekend:before{content:\"\\f00ef\"}.bosch-ic-calendar-view:before{content:\"\\f00f0\"}.bosch-ic-chart-average:before{content:\"\\f00f1\"}.bosch-ic-chart-line-arrows-up:before{content:\"\\f00f2\"}.bosch-ic-configuration-points-set:before{content:\"\\f00f3\"}.bosch-ic-data-points-set:before{content:\"\\f00f4\"}.bosch-ic-document-points-set:before{content:\"\\f00f5\"}.bosch-ic-multistate-value:before{content:\"\\f00f6\"}.bosch-ic-multistate-value-arrow-in:before{content:\"\\f00f7\"}.bosch-ic-multistate-value-arrow-out:before{content:\"\\f00f8\"}.bosch-ic-structure-bacnet:before{content:\"\\f00f9\"}.bosch-ic-structure-sbus:before{content:\"\\f00fa\"}.bosch-ic-team-arrows-in:before{content:\"\\f00fb\"}.bosch-ic-import-points-set:before{content:\"\\f00fc\"}.bosch-ic-image-points-set:before{content:\"\\f00fd\"}.bosch-ic-circuit-hydraulic:before{content:\"\\f00fe\"}.bosch-ic-structure-area:before{content:\"\\f00ff\"}.bosch-ic-structure-calculate:before{content:\"\\f0100\"}.bosch-ic-structure-clipboard:before{content:\"\\f0101\"}.bosch-ic-structure-collection:before{content:\"\\f0102\"}.bosch-ic-structure-components:before{content:\"\\f0103\"}.bosch-ic-structure-device:before{content:\"\\f0104\"}.bosch-ic-structure-dragdrop:before{content:\"\\f0105\"}.bosch-ic-structure-floor:before{content:\"\\f0106\"}.bosch-ic-structure-organization:before{content:\"\\f0107\"}.bosch-ic-structure-questionmark:before{content:\"\\f0108\"}.bosch-ic-structure-settings:before{content:\"\\f0109\"}.bosch-ic-structure-skyscraper:before{content:\"\\f010a\"}.bosch-ic-structure-window-console:before{content:\"\\f010b\"}.bosch-ic-structure-wrench:before{content:\"\\f010c\"}.bosch-ic-car-pickup:before{content:\"\\f010d\"}.bosch-ic-desktop-team:before{content:\"\\f010e\"}.bosch-ic-keyboard-arrow-enter-return:before{content:\"\\f010f\"}.bosch-ic-keyboard-arrow-left:before{content:\"\\f0110\"}.bosch-ic-keyboard-arrow-right-tab:before{content:\"\\f0111\"}.bosch-ic-keyboard-capslock:before{content:\"\\f0112\"}.bosch-ic-keyboard-shift:before{content:\"\\f0113\"}.bosch-ic-keyboard-space:before{content:\"\\f0114\"}.bosch-ic-bluetooth-le-off-disabled-light:before{content:\"\\f0115\"}.bosch-ic-structure-label-edit:before{content:\"\\f0116\"}.bosch-ic-structure-server:before{content:\"\\f0117\"}.bosch-ic-truck-box:before{content:\"\\f0118\"}.bosch-ic-imagery-off-disabled-light:before{content:\"\\f0119\"}.bosch-ic-imagery-off-disabled-bold:before{content:\"\\f011a\"}.bosch-ic-plan-floorplan-edit:before{content:\"\\f011b\"}.bosch-ic-desktop-building-ruler-pen:before{content:\"\\f011c\"}.bosch-ic-conference-system-connected:before{content:\"\\f011d\"}.bosch-ic-receiver-pocket-headphones:before{content:\"\\f011e\"}.bosch-ic-conference-system-microphone:before{content:\"\\f011f\"}.bosch-ic-user-carry-box:before{content:\"\\f0120\"}.bosch-ic-user-handcart:before{content:\"\\f0121\"}.bosch-ic-user-carry-box-2:before{content:\"\\f0122\"}.bosch-ic-locator-error:before{content:\"\\f0123\"}.bosch-ic-locator-offline:before{content:\"\\f0124\"}.bosch-ic-locator-refresh:before{content:\"\\f0125\"}.bosch-ic-chart-bar-plug-arrow-up:before{content:\"\\f0126\"}.bosch-ic-clipboard-search-leaf:before{content:\"\\f0127\"}.bosch-ic-handover-leaf:before{content:\"\\f0128\"}.bosch-ic-home-sun-plug:before{content:\"\\f0129\"}.bosch-ic-lightbulb-plug-leaf:before{content:\"\\f012a\"}.bosch-ic-wheel-water-plug:before{content:\"\\f012b\"}.bosch-ic-windmill-plug:before{content:\"\\f012c\"}.bosch-ic-windmill-plug-sun-water:before{content:\"\\f012d\"}.bosch-ic-scale-co2:before{content:\"\\f012e\"}.bosch-ic-certificat-co2:before{content:\"\\f012f\"}.bosch-ic-car-crash:before{content:\"\\f0130\"}.bosch-ic-headphones:before{content:\"\\f0131\"}.bosch-ic-music-note:before{content:\"\\f0132\"}.bosch-ic-clef-treble:before{content:\"\\f0133\"}.bosch-ic-clef-bass:before{content:\"\\f0134\"}.bosch-ic-volume:before{content:\"\\f0135\"}.bosch-ic-volume-increase:before{content:\"\\f0136\"}.bosch-ic-head-speach-speaking:before{content:\"\\f0137\"}.bosch-ic-head-speaking-request:before{content:\"\\f0138\"}.bosch-ic-tetragon-priority:before{content:\"\\f0139\"}.bosch-ic-head-respond-speaking:before{content:\"\\f013a\"}.bosch-ic-head-respond-request:before{content:\"\\f013b\"}.bosch-ic-hand-stop:before{content:\"\\f013c\"}.bosch-ic-layers-stack:before{content:\"\\f013d\"}.bosch-ic-puzzle-binary:before{content:\"\\f013e\"}.bosch-ic-sdcard-micro:before{content:\"\\f013f\"}.bosch-ic-reader-card:before{content:\"\\f0140\"}.bosch-ic-reader-user-recognition-face:before{content:\"\\f0141\"}.bosch-ic-detector-point-type:before{content:\"\\f0142\"}.bosch-ic-manual-call-point:before{content:\"\\f0143\"}.bosch-ic-gun-magnifier:before{content:\"\\f0144\"}.bosch-ic-interface-module:before{content:\"\\f0145\"}.bosch-ic-fire-detector-automatic:before{content:\"\\f0146\"}.bosch-ic-fire-brigade-emergency:before{content:\"\\f0147\"}.bosch-ic-volume-siren-device:before{content:\"\\f0148\"}.bosch-ic-user-reception:before{content:\"\\f0149\"}.bosch-ic-shredder-machine:before{content:\"\\f014a\"}.bosch-ic-badge-clean:before{content:\"\\f014b\"}.bosch-ic-parking-assistance:before{content:\"\\f014c\"}.bosch-ic-laptop-questionmark:before{content:\"\\f014d\"}.bosch-ic-security-lock:before{content:\"\\f014e\"}.bosch-ic-battery-car-12v:before{content:\"\\f014f\"}.bosch-ic-car-far-propagation:before{content:\"\\f0150\"}.bosch-ic-car-wide-propagation:before{content:\"\\f0151\"}.bosch-ic-ear-sound:before{content:\"\\f0152\"}.bosch-ic-umbrella-drops:before{content:\"\\f0153\"}.bosch-ic-horn-arrows-in:before{content:\"\\f0154\"}.bosch-ic-video-bell:before{content:\"\\f0155\"}.bosch-ic-steering-wheel-arrows-up:before{content:\"\\f0156\"}.bosch-ic-circle-full:before{content:\"\\f0157\"}.bosch-ic-circle-half:before{content:\"\\f0158\"}.bosch-ic-circle-quarter:before{content:\"\\f0159\"}.bosch-ic-circle-quarter-three:before{content:\"\\f015a\"}.bosch-ic-appliance-flame-arrow:before{content:\"\\f015b\"}.bosch-ic-appliance-flame-arrow-bold:before{content:\"\\f015c\"}.bosch-ic-appliance-flame-disabled-bold:before{content:\"\\f015d\"}.bosch-ic-appliance-flame-disabled-light:before{content:\"\\f015e\"}.bosch-ic-appliance-flame-plus:before{content:\"\\f015f\"}.bosch-ic-appliance-flame-plus-bold:before{content:\"\\f0160\"}.bosch-ic-flash-minus-bold:before{content:\"\\f0161\"}.bosch-ic-flash-minus:before{content:\"\\f0162\"}.bosch-ic-flash-off:before{content:\"\\f0163\"}.bosch-ic-flash-plus-bold:before{content:\"\\f0164\"}.bosch-ic-flash-smart-grid:before{content:\"\\f0165\"}.bosch-ic-signal-high:before{content:\"\\f0166\"}.bosch-ic-signal-low:before{content:\"\\f0167\"}.bosch-ic-signal-very-low:before{content:\"\\f0168\"}.bosch-ic-5g:before{content:\"\\f0169\"}.bosch-ic-sun-photovoltaic:before{content:\"\\f016a\"}.bosch-ic-inverter:before{content:\"\\f016b\"}.bosch-ic-car-arrow-down:before{content:\"\\f016c\"}.bosch-ic-car-distance:before{content:\"\\f016d\"}.bosch-ic-car-sensor:before{content:\"\\f016e\"}.bosch-ic-car-sensor-environment:before{content:\"\\f016f\"}.bosch-ic-car-sensor-front:before{content:\"\\f0170\"}.bosch-ic-streets-locator:before{content:\"\\f0171\"}.bosch-ic-truck-distance:before{content:\"\\f0172\"}.bosch-ic-warning-user-walk:before{content:\"\\f0173\"}.bosch-ic-parking-check:before{content:\"\\f0174\"}.bosch-ic-dremel:before{content:\"\\f0175\"}.bosch-ic-allergens:before{content:\"\\f0176\"}.bosch-ic-money-currency-coins:before{content:\"\\f0177\"}.bosch-ic-hydrogen-small:before{content:\"\\f0178\"}.bosch-ic-hydrogen-large:before{content:\"\\f0179\"}.bosch-ic-shoppingcart-remove:before{content:\"\\f017a\"}.bosch-ic-chimney-sweeper-disabled-light:before{content:\"\\f017b\"}.bosch-ic-chimney-sweeper-disabled-bold:before{content:\"\\f017c\"}.bosch-ic-car-waves:before{content:\"\\f017d\"}.bosch-ic-360-degrees-arrow:before{content:\"\\f017e\"}.bosch-ic-clock-info:before{content:\"\\f017f\"}.bosch-ic-truck-front-disabled-bold:before{content:\"\\f0180\"}.bosch-ic-truck-front-disabled-light:before{content:\"\\f0181\"}.bosch-ic-box-closed-list:before{content:\"\\f0182\"}.bosch-ic-battery-car-exchange:before{content:\"\\f0183\"}.bosch-ic-guitar:before{content:\"\\f0184\"}.bosch-ic-actuator-by-wire:before{content:\"\\f0185\"}.bosch-ic-guitar-electric:before{content:\"\\f0186\"}.bosch-ic-format-align-center:before{content:\"\\f0187\"}.bosch-ic-format-align-justify:before{content:\"\\f0188\"}.bosch-ic-format-align-right:before{content:\"\\f0189\"}.bosch-ic-format-checklist:before{content:\"\\f018a\"}.bosch-ic-format-italic:before{content:\"\\f018b\"}.bosch-ic-format-ordered-list:before{content:\"\\f018c\"}.bosch-ic-format-strike:before{content:\"\\f018d\"}.bosch-ic-format-underline:before{content:\"\\f018e\"}.bosch-ic-format-align-left:before{content:\"\\f018f\"}.bosch-ic-format-fill-drop:before{content:\"\\f0190\"}.bosch-ic-format-color-a:before{content:\"\\f0191\"}.bosch-ic-format-bold-b:before{content:\"\\f0192\"}.bosch-ic-fly:before{content:\"\\f0193\"}.bosch-ic-paw:before{content:\"\\f0194\"}.bosch-ic-desktop-video-response-fast:before{content:\"\\f0195\"}.bosch-ic-user-walk-weather-rain:before{content:\"\\f0196\"}.bosch-ic-brake-pedal-plug:before{content:\"\\f0197\"}.bosch-ic-detector-motion-range-down-view:before{content:\"\\f0198\"}.bosch-ic-detector-motion-range-long:before{content:\"\\f0199\"}.bosch-ic-monitor-tiled:before{content:\"\\f019a\"}.bosch-ic-radar-doppler:before{content:\"\\f019b\"}.bosch-ic-radioactive-disabled-light:before{content:\"\\f019c\"}.bosch-ic-radioactive-disabled-bold:before{content:\"\\f019d\"}.bosch-ic-bug-disabled-light:before{content:\"\\f019e\"}.bosch-ic-bug-disabled-bold:before{content:\"\\f019f\"}.bosch-ic-fly-disabled-light:before{content:\"\\f01a0\"}.bosch-ic-fly-disabled-bold:before{content:\"\\f01a1\"}.bosch-ic-puzzle-2:before{content:\"\\f01a2\"}.bosch-ic-camera-lenses-multi-images:before{content:\"\\f01a3\"}.bosch-ic-detector-spray:before{content:\"\\f01a4\"}.bosch-ic-weather-rain-ip56:before{content:\"\\f01a5\"}.bosch-ic-document-log-start:before{content:\"\\f01a6\"}.bosch-ic-document-log-stop:before{content:\"\\f01a7\"}.bosch-ic-document-3d-pdf:before{content:\"\\f01a8\"}.bosch-ic-document-ifc:before{content:\"\\f01a9\"}.bosch-ic-document-rfa:before{content:\"\\f01aa\"}.bosch-ic-document-stp:before{content:\"\\f01ab\"}.bosch-ic-recycle-arrows-pap20-frame:before{content:\"\\e8b4\"}.bosch-ic-umbrella-drops-frame:before{content:\"\\e8b5\"}.bosch-ic-arrows-this-side-up-frame:before{content:\"\\e8b6\"}.bosch-ic-glass-wine-frame:before{content:\"\\e8b7\"}.bosch-ic-pb-frame:before{content:\"\\e8b8\"}.bosch-ic-pb-frame-disabled-bold:before{content:\"\\e8b9\"}.bosch-ic-pb-frame-disabled-light:before{content:\"\\e8ba\"}.bosch-ic-mode-frame:before{content:\"\\e8bb\"}.bosch-ic-uv-frame-disabled-bold:before{content:\"\\e8bc\"}.bosch-ic-uv-frame-disabled-light:before{content:\"\\e8bd\"}.bosch-ic-fan-frame:before{content:\"\\e8be\"}.bosch-ic-flash-frame:before{content:\"\\e8bf\"}.bosch-ic-kakao:before{content:\"\\eaaf\"}.bosch-ic-kakao-frame:before{content:\"\\eab0\"}.bosch-ic-naver:before{content:\"\\eab1\"}.bosch-ic-naver-frame:before{content:\"\\eab2\"}.bosch-ic-tencent:before{content:\"\\eab3\"}.bosch-ic-tencent-frame:before{content:\"\\eab4\"}.bosch-ic-wordpress:before{content:\"\\eab5\"}.bosch-ic-wordpress-frame:before{content:\"\\eab6\"}.bosch-ic-mini-program-frame:before{content:\"\\eab7\"}.bosch-ic-mini-program:before{content:\"\\eab8\"}.bosch-ic-tiktok:before{content:\"\\eab9\"}.bosch-ic-tiktok-frame:before{content:\"\\eaba\"}.bosch-ic-glassdoor:before{content:\"\\eabb\"}.bosch-ic-glassdoor-frame:before{content:\"\\eabc\"}.bosch-ic-git-hub:before{content:\"\\eabd\"}.bosch-ic-git-hub-frame:before{content:\"\\eabe\"}.bosch-ic-online-www:before{content:\"\\f01ac\"}.bosch-ic-bluetooth-connected:before{content:\"\\f01ad\"}.bosch-ic-connected-tools:before{content:\"\\f01ae\"}.bosch-ic-badge-arrow-kickback:before{content:\"\\f01af\"}.bosch-ic-copy-paste:before{content:\"\\f01b0\"}.bosch-ic-volleyball:before{content:\"\\f01b1\"}.bosch-ic-car-top-trailer:before{content:\"\\f01b2\"}.bosch-ic-mountain:before{content:\"\\f01b3\"}.bosch-ic-battery-car-oil-lamp:before{content:\"\\f01b4\"}.bosch-ic-battery-car-efb:before{content:\"\\f01b5\"}.bosch-ic-battery-car-ams:before{content:\"\\f01b6\"}.bosch-ic-battery-car-ag:before{content:\"\\f01b7\"}.bosch-ic-display-fuel-saving:before{content:\"\\f01b8\"}.bosch-ic-circle-parking-arrows:before{content:\"\\f01b9\"}.bosch-ic-unicorn-head:before{content:\"\\f01ba\"}.bosch-ic-car-parking-automated:before{content:\"\\f01bb\"}.bosch-ic-lunchboard:before{content:\"\\f01bc\"}.bosch-ic-tetrapack:before{content:\"\\f01bd\"}.bosch-ic-recycle-arrows:before{content:\"\\f01be\"}.bosch-ic-stairs-arrow-up:before{content:\"\\f01bf\"}.bosch-ic-stairs-arrow-down:before{content:\"\\f01c0\"}.bosch-ic-industry-car-arrow:before{content:\"\\f01c1\"}.bosch-ic-sensor-throttle-position:before{content:\"\\f01c2\"}.bosch-ic-sensor-knock:before{content:\"\\f01c3\"}.bosch-ic-sensor-air-mass-meter:before{content:\"\\f01c4\"}.bosch-ic-sensor-pressure:before{content:\"\\f01c5\"}.bosch-ic-sensor-temperature:before{content:\"\\f01c6\"}.bosch-ic-sensor-speed-rpm:before{content:\"\\f01c7\"}.bosch-ic-desktop-user-software-clock:before{content:\"\\f01c8\"}.bosch-ic-book-service-wrench-add:before{content:\"\\f01c9\"}.bosch-ic-book-service-wrench:before{content:\"\\f01ca\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-light:before{content:\"\\f01cb\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-bold:before{content:\"\\f01cc\"}.bosch-ic-circle-parking-disabled-bold:before{content:\"\\f01cd\"}.bosch-ic-circle-parking-disabled-light:before{content:\"\\f01ce\"}.bosch-ic-interval-30sec:before{content:\"\\f01cf\"}.bosch-ic-truck-front-light-inside:before{content:\"\\f01d0\"}.bosch-ic-software-tex:before{content:\"\\f01d1\"}.bosch-ic-fire-arrow-circle:before{content:\"\\f01d2\"}.bosch-ic-heating-underfloor-circle-global-off:before{content:\"\\f01d3\"}.bosch-ic-heating-radiator-circle-global-off:before{content:\"\\f01d4\"}.bosch-ic-faucet-circle-global-off:before{content:\"\\f01d5\"}.bosch-ic-moon-stars-starlight:before{content:\"\\f01d6\"}.bosch-ic-tube-thumb-up:before{content:\"\\f01d7\"}.bosch-ic-detector-microwave:before{content:\"\\f01d8\"}.bosch-ic-user-headset-thumb-up:before{content:\"\\f01d9\"}.bosch-ic-cloud-battery:before{content:\"\\f01da\"}.bosch-ic-temperature-arrow-up-increase:before{content:\"\\f01db\"}.bosch-ic-temperature-arrow-down-decrease:before{content:\"\\f01dc\"}.bosch-ic-moneybag-dollar-chart-double:before{content:\"\\f01dd\"}.bosch-ic-locator-remove:before{content:\"\\f01de\"}.bosch-ic-square-remove:before{content:\"\\f01df\"}.bosch-ic-sun-hdr:before{content:\"\\f01e0\"}.bosch-ic-moon-stars-circle:before{content:\"\\f01e1\"}.bosch-ic-temperature-user-detection:before{content:\"\\f01e2\"}.bosch-ic-wrench-thumb-up:before{content:\"\\f01e3\"}.bosch-ic-ring-infrared-ir:before{content:\"\\f01e4\"}.bosch-ic-play-back-arrow:before{content:\"\\f01e5\"}.bosch-ic-ignition-off:before{content:\"\\f01e6\"}.bosch-ic-arrow-up-down:before{content:\"\\f01e7\"}.bosch-ic-arrow-left-right:before{content:\"\\f01e8\"}.bosch-ic-number-0-bold:before{content:\"\\f01e9\"}.bosch-ic-number-1-bold:before{content:\"\\f01ea\"}.bosch-ic-number-2-bold:before{content:\"\\f01eb\"}.bosch-ic-number-3-bold:before{content:\"\\f01ec\"}.bosch-ic-number-4-bold:before{content:\"\\f01ed\"}.bosch-ic-number-5-bold:before{content:\"\\f01ee\"}.bosch-ic-number-6-bold:before{content:\"\\f01ef\"}.bosch-ic-number-7-bold:before{content:\"\\f01f0\"}.bosch-ic-number-8-bold:before{content:\"\\f01f1\"}.bosch-ic-number-9-bold:before{content:\"\\f01f2\"}.bosch-ic-e-charging-plug-dc:before{content:\"\\f01f3\"}.bosch-ic-e-charging-plug-ac-dc:before{content:\"\\f01f4\"}.bosch-ic-angle-view-wide-monitor:before{content:\"\\f01f5\"}.bosch-ic-settings-range-ptrz:before{content:\"\\f01f6\"}.bosch-ic-puzzle-video-integration:before{content:\"\\f01f7\"}.bosch-ic-number-0-regular:before{content:\"\\f01f8\"}.bosch-ic-number-1-regular:before{content:\"\\f01f9\"}.bosch-ic-number-2-regular:before{content:\"\\f01fa\"}.bosch-ic-number-3-regular:before{content:\"\\f01fb\"}.bosch-ic-number-4-regular:before{content:\"\\f01fc\"}.bosch-ic-number-5-regular:before{content:\"\\f01fd\"}.bosch-ic-number-6-regular:before{content:\"\\f01fe\"}.bosch-ic-number-7-regular:before{content:\"\\f01ff\"}.bosch-ic-number-8-regular:before{content:\"\\f0200\"}.bosch-ic-number-9-regular:before{content:\"\\f0201\"}.bosch-ic-import-beep:before{content:\"\\f0202\"}.bosch-ic-file-questionmark:before{content:\"\\f0203\"}.bosch-ic-file-dollar:before{content:\"\\f0204\"}.bosch-ic-document-reporting:before{content:\"\\f0205\"}.bosch-ic-box-questionmark1:before{content:\"\\f0206\"}.bosch-ic-chat-faq:before{content:\"\\f0207\"}.bosch-ic-wishlist-filled:before{content:\"\\f0208\"}.bosch-ic-wishlist-half-filled:before{content:\"\\f0209\"}.bosch-ic-glove-work:before{content:\"\\f020a\"}.bosch-ic-lightbulb-halogen-hand-disabled-bold:before{content:\"\\f020b\"}.bosch-ic-lightbulb-halogen-hand-disabled-light:before{content:\"\\f020c\"}.bosch-ic-lightbulb-halogen-cracked-disabled-bold:before{content:\"\\f020d\"}.bosch-ic-lightbulb-halogen-cracked-disabled-light:before{content:\"\\f020e\"}.bosch-ic-lightbulb-halogen-cracked:before{content:\"\\f020f\"}.bosch-ic-drops:before{content:\"\\f0210\"}.bosch-ic-drops-disabled-bold:before{content:\"\\f0211\"}.bosch-ic-drops-disabled-light:before{content:\"\\f0212\"}.bosch-ic-temperature-heat:before{content:\"\\f0213\"}.bosch-ic-allen-key:before{content:\"\\f0214\"}.bosch-ic-aperture-closed:before{content:\"\\f0215\"}.bosch-ic-appliance-heat-pump-leaf-connected:before{content:\"\\f0216\"}.bosch-ic-badge-flash:before{content:\"\\f0217\"}.bosch-ic-battery-car-1-plug-warning:before{content:\"\\f0218\"}.bosch-ic-battery-car-3-automatic-mode:before{content:\"\\f0219\"}.bosch-ic-battery-car-3-ice:before{content:\"\\f021a\"}.bosch-ic-battery-car-memory-function:before{content:\"\\f021b\"}.bosch-ic-battery-polarity-reverse-warning:before{content:\"\\f021c\"}.bosch-ic-book-questionmark:before{content:\"\\f021d\"}.bosch-ic-book-service-wrench-refresh:before{content:\"\\f021e\"}.bosch-ic-box-arrow-up:before{content:\"\\f021f\"}.bosch-ic-box-arrows-up-down:before{content:\"\\f0220\"}.bosch-ic-camera-optic-sensor:before{content:\"\\f0221\"}.bosch-ic-car-cloud-software:before{content:\"\\f0222\"}.bosch-ic-car-gear-technology:before{content:\"\\f0223\"}.bosch-ic-car-street:before{content:\"\\f0224\"}.bosch-ic-chip-sensor:before{content:\"\\f0225\"}.bosch-ic-circle-blank:before{content:\"\\f0226\"}.bosch-ic-detector-point-type-replace:before{content:\"\\f0227\"}.bosch-ic-document-x:before{content:\"\\f0228\"}.bosch-ic-drag-handle-arrows-up-down:before{content:\"\\f0229\"}.bosch-ic-electrolyzer:before{content:\"\\f022a\"}.bosch-ic-exhaust-pipe:before{content:\"\\f022b\"}.bosch-ic-flower-focus-near:before{content:\"\\f022c\"}.bosch-ic-fuel-pump-electric:before{content:\"\\f022d\"}.bosch-ic-gear-dollar:before{content:\"\\f022e\"}.bosch-ic-gloves-work:before{content:\"\\f022f\"}.bosch-ic-handlewithcare-globe:before{content:\"\\f0230\"}.bosch-ic-handlewithcare-user:before{content:\"\\f0231\"}.bosch-ic-hard-disk:before{content:\"\\f0232\"}.bosch-ic-hierarchy-child:before{content:\"\\f0233\"}.bosch-ic-hierarchy-parent:before{content:\"\\f0234\"}.bosch-ic-hydraulic-connection-leaking-drops:before{content:\"\\f0235\"}.bosch-ic-hydraulic-connection-off:before{content:\"\\f0236\"}.bosch-ic-hydraulic-connection-on:before{content:\"\\f0237\"}.bosch-ic-light-beam:before{content:\"\\f0238\"}.bosch-ic-map5000:before{content:\"\\f0239\"}.bosch-ic-module-anti-blocking-system:before{content:\"\\f023a\"}.bosch-ic-module-ecg:before{content:\"\\f023b\"}.bosch-ic-module-ecg-suction:before{content:\"\\f023c\"}.bosch-ic-module-navigation:before{content:\"\\f023d\"}.bosch-ic-power-supply-unit-plug:before{content:\"\\f023e\"}.bosch-ic-security-street:before{content:\"\\f023f\"}.bosch-ic-server-single:before{content:\"\\f0240\"}.bosch-ic-skip-backward:before{content:\"\\f0241\"}.bosch-ic-skip-forward:before{content:\"\\f0242\"}.bosch-ic-speedometer-instrument-cluster:before{content:\"\\f0243\"}.bosch-ic-system-radar-connected:before{content:\"\\f0244\"}.bosch-ic-truck-dump:before{content:\"\\f0245\"}.bosch-ic-truck-loader:before{content:\"\\f0246\"}.bosch-ic-user-head-twin-digital:before{content:\"\\f0247\"}.bosch-ic-wind-vane-strong:before{content:\"\\f0248\"}.bosch-ic-wind-vane-weak:before{content:\"\\f0249\"}.bosch-ic-windshield-crack:before{content:\"\\f024a\"}.bosch-ic-common-rail:before{content:\"\\f024b\"}.bosch-ic-common-rail-injector-cri1-cri2:before{content:\"\\f024c\"}.bosch-ic-common-rail-injector-crii3:before{content:\"\\f024d\"}.bosch-ic-common-rail-injector-cril3:before{content:\"\\f024e\"}.bosch-ic-common-rail-injector-crin4-2:before{content:\"\\f024f\"}.bosch-ic-common-rail-injector-crin123:before{content:\"\\f0250\"}.bosch-ic-conventional-pump-a:before{content:\"\\f0251\"}.bosch-ic-conventional-pump-b:before{content:\"\\f0252\"}.bosch-ic-edc-inline-injection-pump-pes6p:before{content:\"\\f0253\"}.bosch-ic-edc-inline-injection-pump-pes6h:before{content:\"\\f0254\"}.bosch-ic-nozzle-holder:before{content:\"\\f0255\"}.bosch-ic-unit-injector:before{content:\"\\f0256\"}.bosch-ic-qr-code-scan:before{content:\"\\f0257\"}.bosch-ic-car-mechanic-plug:before{content:\"\\f0258\"}.bosch-ic-copy-machine:before{content:\"\\f0259\"}.bosch-ic-dispenser-paper-towel:before{content:\"\\f025a\"}.bosch-ic-hand-drops-soap:before{content:\"\\f025b\"}.bosch-ic-door-automatic:before{content:\"\\f025c\"}.bosch-ic-door-automatic-slide:before{content:\"\\f025d\"}.bosch-ic-door-pull:before{content:\"\\f025e\"}.bosch-ic-door-push:before{content:\"\\f025f\"}.bosch-ic-floor-load-max:before{content:\"\\f0260\"}.bosch-ic-lockers:before{content:\"\\f0261\"}.bosch-ic-rubbish-trash-bin:before{content:\"\\f0262\"}.bosch-ic-rubbish-waste-bin:before{content:\"\\f0263\"}.bosch-ic-rubbish-waste-container:before{content:\"\\f0264\"}.bosch-ic-sugar:before{content:\"\\f0265\"}.bosch-ic-rubbish-glass-bin:before{content:\"\\f0266\"}.bosch-ic-car-seat-steering-wheel-stowable:before{content:\"\\f0267\"}.bosch-ic-car-seat-steering-wheel-standard:before{content:\"\\f0268\"}.bosch-ic-car-seat-steering-wheel-extended:before{content:\"\\f0269\"}.bosch-ic-device-bridge:before{content:\"\\f026a\"}.bosch-ic-screw-cross-slot:before{content:\"\\f026b\"}.bosch-ic-screw-hexagon-head:before{content:\"\\f026c\"}.bosch-ic-screw-hexagon-internal:before{content:\"\\f026d\"}.bosch-ic-screw-slotted:before{content:\"\\f026e\"}.bosch-ic-screw-torx:before{content:\"\\f026f\"}.bosch-ic-tape:before{content:\"\\f0270\"}.bosch-ic-tape-teflon:before{content:\"\\f0271\"}.bosch-ic-security-fire:before{content:\"\\f0272\"}.bosch-ic-led-fault:before{content:\"\\f0273\"}.bosch-ic-led-alarm:before{content:\"\\f0274\"}.bosch-ic-firepanel:before{content:\"\\f0275\"}.bosch-ic-fire-output-disabled:before{content:\"\\f0276\"}.bosch-ic-user-fire-alarm:before{content:\"\\f0277\"}.bosch-ic-car-desktop-graph:before{content:\"\\f0278\"}.bosch-ic-desktop-management-statistics:before{content:\"\\f0279\"}.bosch-ic-document-key:before{content:\"\\f027a\"}.bosch-ic-options-vertical:before{content:\"\\f027b\"}.bosch-ic-euro-arrows:before{content:\"\\f027c\"}.bosch-ic-monkey:before{content:\"\\f027d\"}.bosch-ic-appliance-resistance:before{content:\"\\f027e\"}.bosch-ic-pipe-cutter:before{content:\"\\f027f\"}.bosch-ic-flash-arrow:before{content:\"\\f0280\"}.bosch-ic-flash-arrow-disabled-light:before{content:\"\\f0281\"}.bosch-ic-heating-ceiling:before{content:\"\\f0282\"}.bosch-ic-heating-ceiling-off-light:before{content:\"\\f0283\"}.bosch-ic-heating-ceiling-off-bold:before{content:\"\\f0284\"}.bosch-ic-battery-analysis:before{content:\"\\f0285\"}.bosch-ic-battery-ice:before{content:\"\\f0286\"}.bosch-ic-battery-refresh:before{content:\"\\f0287\"}.bosch-ic-device-ids:before{content:\"\\f0288\"}.bosch-ic-device-thermostat-bcc50-72:before{content:\"\\f0289\"}.bosch-ic-device-thermostat-bcc100-72:before{content:\"\\f028a\"}.bosch-ic-silos-gas-natural:before{content:\"\\f028b\"}.bosch-ic-silos-gas-leaf-biogas:before{content:\"\\f028c\"}.bosch-ic-circle-decentralized:before{content:\"\\f028d\"}.bosch-ic-molecules-h2:before{content:\"\\f028e\"}.bosch-ic-megawatt-range-arrow-scalable:before{content:\"\\f028f\"}.bosch-ic-fan-arrows:before{content:\"\\f0290\"}.bosch-ic-recycle-arrows-pvc3:before{content:\"\\f0291\"}.bosch-ic-recycle-arrows-ps6:before{content:\"\\f0292\"}.bosch-ic-recycle-arrows-pp5:before{content:\"\\f0293\"}.bosch-ic-recycle-arrows-pet1:before{content:\"\\f0294\"}.bosch-ic-recycle-arrows-pap22:before{content:\"\\f0295\"}.bosch-ic-recycle-arrows-pap21:before{content:\"\\f0296\"}.bosch-ic-recycle-arrows-pap20:before{content:\"\\f0297\"}.bosch-ic-recycle-arrows-oth7:before{content:\"\\f0298\"}.bosch-ic-recycle-arrows-ldpe4:before{content:\"\\f0299\"}.bosch-ic-recycle-arrows-hdpe2:before{content:\"\\f029a\"}.bosch-ic-car-petrol-plug:before{content:\"\\f029b\"}.bosch-ic-user-diverse:before{content:\"\\f029c\"}.bosch-ic-fire-emergency-run-map:before{content:\"\\f029d\"}.bosch-ic-automatic-mode-fan:before{content:\"\\f029e\"}.bosch-ic-air-condition-swing-pac:before{content:\"\\f029f\"}.bosch-ic-user-fire-alarm-check:before{content:\"\\f02a0\"}.bosch-ic-handlewithcare-team:before{content:\"\\f02a1\"}.bosch-ic-settings-user-dollar:before{content:\"\\f02a2\"}.bosch-ic-skyscraper-connected:before{content:\"\\f02a3\"}.bosch-ic-security-ac:before{content:\"\\f02a4\"}.bosch-ic-windshield-glare-sun:before{content:\"\\f02a5\"}.bosch-ic-user-flag:before{content:\"\\f02a6\"}.bosch-ic-hammer-legal:before{content:\"\\f02a7\"}.bosch-ic-handover-star:before{content:\"\\f02a8\"}.bosch-ic-laptop-wrench:before{content:\"\\f02a9\"}/*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.Bosch-IC-find-part:before{content:\"\\e38f\"}.Bosch-IC-next-station:before{content:\"\\e8a9\"}.Bosch-Ic-HMI:before{content:\"\\e293\"}.Bosch-Ic-add-outline:before{content:\"\\e8a1\"}.Bosch-Ic-apas:before{content:\"\\e224\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-Ic-cash:before{content:\"\\e890\"}.Bosch-Ic-pinterest:before{content:\"\\ea8c\"}.Bosch-Ic-problem:before{content:\"\\e899\"}.Bosch-Ic-promotion:before{content:\"\\e89a\"}.Bosch-Ic-arrow-diagonal:before{content:\"\\e095\"}.Bosch-Ic-arrow-down-filled:before{content:\"\\e887\"}.Bosch-Ic-arrow-down-circle:before{content:\"\\e887\"}.Bosch-Ic-arrow-end-left:before{content:\"\\e0a1\"}.Bosch-Ic-arrow-end-right:before{content:\"\\e182\"}.Bosch-Ic-arrow-left-down-filled:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-filled:before{content:\"\\e889\"}.Bosch-Ic-arrow-left-up-filled:before{content:\"\\e88a\"}.Bosch-Ic-arrow-right-down-filled:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-filled:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-up-filled:before{content:\"\\e88d\"}.Bosch-Ic-arrow-up-filled:before{content:\"\\e88e\"}.Bosch-Ic-batch:before{content:\"\\e384\"}.Bosch-Ic-calculator:before{content:\"\\e0c7\"}.Bosch-Ic-calendar-delete:before{content:\"\\e385\"}.Bosch-Ic-call-bluetooth:before{content:\"\\e386\"}.Bosch-Ic-call-deny-outline:before{content:\"\\e8a2\"}.Bosch-Ic-call-deny:before{content:\"\\e387\"}.Bosch-Ic-call-pick-up-outline:before{content:\"\\e8a3\"}.Bosch-Ic-call-pick-up:before{content:\"\\e0cb\"}.Bosch-Ic-call-settings:before{content:\"\\e388\"}.Bosch-Ic-clear-all-annotations:before{content:\"\\e0ec\"}.Bosch-Ic-connection:before{content:\"\\e107\"}.Bosch-Ic-core-data:before{content:\"\\e38b\"}.Bosch-Ic-curve-scaling-view:before{content:\"\\e2bb\"}.Bosch-Ic-dashboard:before{content:\"\\e190\"}.Bosch-Ic-delete-material-manually:before{content:\"\\e394\"}.Bosch-Ic-dmc-code:before{content:\"\\e38e\"}.Bosch-Ic-document-to-svg:before{content:\"\\e34e\"}.Bosch-Ic-emergency-order:before{content:\"\\e12f\"}.Bosch-Ic-fast-backwards:before{content:\"\\e8a4\"}.Bosch-Ic-fast-forwards:before{content:\"\\e8a5\"}.Bosch-Ic-favorite:before{content:\"\\e2c5\"}.Bosch-Ic-go-to:before{content:\"\\e1ca\"}.Bosch-Ic-gps:before{content:\"\\e1c6\"}.Bosch-Ic-home-filled:before{content:\"\\e19d\"}.Bosch-Ic-input:before{content:\"\\e1ca\"}.Bosch-Ic-interface:before{content:\"\\e391\"}.Bosch-Ic-list:before{content:\"\\e1c1\"}.Bosch-Ic-malfunction-history:before{content:\"\\e389\"}.Bosch-Ic-malfunction-order:before{content:\"\\e38a\"}.Bosch-Ic-material-list:before{content:\"\\e393\"}.Bosch-Ic-material:before{content:\"\\e395\"}.Bosch-Ic-maximize-window:before{content:\"\\e269\"}.Bosch-Ic-menu-tree:before{content:\"\\e396\"}.Bosch-Ic-minimize-window:before{content:\"\\e1ba\"}.Bosch-Ic-notification-hint:before{content:\"\\e899\"}.Bosch-Ic-notification-question:before{content:\"\\e89b\"}.Bosch-Ic-notification-info:before{content:\"\\eb0f\"}.Bosch-Ic-notification-error:before{content:\"\\eb08\"}.Bosch-Ic-notification-error-outline:before{content:\"\\eb09\"}.Bosch-Ic-notification-outline:before{content:\"\\eb0d\"}.Bosch-Ic-notification-success:before{content:\"\\eb0a\"}.Bosch-Ic-notification-success-outline:before{content:\"\\eb0b\"}.Bosch-Ic-notification-warning:before{content:\"\\eb0c\"}.Bosch-Ic-notification:before{content:\"\\e1f5\"}.Bosch-Ic-output:before{content:\"\\e1cc\"}.Bosch-Ic-paste:before{content:\"\\e0ed\"}.Bosch-Ic-qr-code:before{content:\"\\e397\"}.Bosch-Ic-remote-cursor:before{content:\"\\e38c\"}.Bosch-Ic-scale:before{content:\"\\e167\"}.Bosch-Ic-search-chart:before{content:\"\\e383\"}.Bosch-Ic-search-web:before{content:\"\\e390\"}.Bosch-Ic-search-window:before{content:\"\\e39b\"}.Bosch-Ic-shape-ellipse:before{content:\"\\e3d1\"}.Bosch-Ic-shape-rectangle:before{content:\"\\e269\"}.Bosch-Ic-show-more-vertical:before{content:\"\\e1fb\"}.Bosch-Ic-signal:before{content:\"\\e249\"}.Bosch-Ic-skip-back:before{content:\"\\e8a6\"}.Bosch-Ic-skip-backward:before{content:\"\\e8a7\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-Ic-skip-for:before{content:\"\\e8a9\"}.Bosch-Ic-skip-forward:before{content:\"\\e8a8\"}.Bosch-Ic-sort-alphabetically:before{content:\"\\e398\"}.Bosch-Ic-switch-language:before{content:\"\\e18d\"}.Bosch-Ic-snyc-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-problem:before{content:\"\\e37f\"}.Bosch-Ic-sync:before{content:\"\\e21e\"}.Bosch-Ic-test:before{content:\"\\e105\"}.Bosch-Ic-to-order:before{content:\"\\e127\"}.Bosch-Ic-to-raw:before{content:\"\\e126\"}.Bosch-Ic-transport-orders:before{content:\"\\e0ee\"}.Bosch-Ic-unpin:before{content:\"\\e463\"}.Bosch-Ic-user-filled:before{content:\"\\e29b\"}.Bosch-Ic-video-play:before{content:\"\\e20b\"}.Bosch-Ic-weblink:before{content:\"\\e1be\"}.Bosch-Ic-warning-outline:before{content:\"\\eb0d\"}.Bosch-Ic-add-material-manually:before{content:\"\\e392\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-IC-3d-print:before{content:\"\\e085\"}.Bosch-Ic-tours:before{content:\"\\e086\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-IC-agility:before{content:\"\\e08a\"}.Bosch-Ic-clock-stop:before{content:\"\\e08b\"}.Bosch-IC-ambulance:before{content:\"\\e08c\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-Ic-arrow-down:before{content:\"\\e08f\"}.Bosch-Ic-arrow-left:before{content:\"\\e090\"}.Bosch-Ic-arrow-left-down:before{content:\"\\e091\"}.Bosch-Ic-arrow-left-up:before{content:\"\\e092\"}.Bosch-Ic-arrow-right:before{content:\"\\e093\"}.Bosch-Ic-arrow-right-down:before{content:\"\\e094\"}.Bosch-Ic-arrow-right-up:before{content:\"\\e095\"}.Bosch-IC-arrow-turn-left:before{content:\"\\e096\"}.Bosch-IC-arrow-turn-right:before{content:\"\\e097\"}.Bosch-Ic-arrow-up:before{content:\"\\e098\"}.Bosch-IC-arrows:before{content:\"\\e099\"}.Bosch-IC-arrows-x-y-z:before{content:\"\\e09a\"}.Bosch-IC-artificial-intelligence:before{content:\"\\e09b\"}.Bosch-IC-assembly-line:before{content:\"\\e09c\"}.Bosch-IC-atom:before{content:\"\\e09d\"}.Bosch-IC-augmented-reality:before{content:\"\\e09e\"}.Bosch-Ic-autobod:before{content:\"\\e09f\"}.Bosch-Ic-arrow-expand-left:before{content:\"\\e0a0\"}.Bosch-Ic-arrow-expand-double-left:before{content:\"\\e0a1\"}.Bosch-IC-bar-chart:before{content:\"\\e0a2\"}.Bosch-IC-bar-chart-double:before{content:\"\\e0a3\"}.Bosch-Ic-battery-empty:before{content:\"\\e0a4\"}.Bosch-Ic-battery-25:before{content:\"\\e0a5\"}.Bosch-Ic-battery-50:before{content:\"\\e0a6\"}.Bosch-Ic-battery-75:before{content:\"\\e0a7\"}.Bosch-Ic-battery-100:before{content:\"\\e0a8\"}.Bosch-Ic-battery-error:before{content:\"\\e0a9\"}.Bosch-IC-battery_Li-Ion:before{content:\"\\e0aa\"}.Bosch-Ic-battery-charging:before{content:\"\\e0ab\"}.Bosch-Ic-help:before{content:\"\\e89b\"}.Bosch-IC-bed:before{content:\"\\e0ac\"}.Bosch-IC-bicycle:before{content:\"\\e0ad\"}.Bosch-IC-bicycle-e:before{content:\"\\e0ae\"}.Bosch-IC-blog:before{content:\"\\e0af\"}.Bosch-IC-bluetooth:before{content:\"\\e0b0\"}.Bosch-IC-bluetooth-LE:before{content:\"\\e0b1\"}.Bosch-IC-board-speaker:before{content:\"\\e0b2\"}.Bosch-IC-boiler-connected:before{content:\"\\e0b3\"}.Bosch-IC-book:before{content:\"\\e0b4\"}.Bosch-Ic-bookmark:before{content:\"\\e0b5\"}.Bosch-Ic-bookmark-add:before{content:\"\\e0b6\"}.Bosch-Ic-bookmark-check:before{content:\"\\e0b7\"}.Bosch-Ic-bookmark-delete:before{content:\"\\e0b8\"}.Bosch-IC-books:before{content:\"\\e0b9\"}.Bosch-IC-bookshelf:before{content:\"\\e0ba\"}.Bosch-Ic-box:before{content:\"\\e0bb\"}.Bosch-IC-box-closed:before{content:\"\\e0bc\"}.Bosch-IC-box-delivery:before{content:\"\\e0bd\"}.Bosch-IC-box-delivery-connected:before{content:\"\\e0be\"}.Bosch-IC-brake-disk:before{content:\"\\e0bf\"}.Bosch-IC-briefcase:before{content:\"\\e0c0\"}.Bosch-IC-broom:before{content:\"\\e0c1\"}.Bosch-IC-broom-cloud-dirt:before{content:\"\\e0c2\"}.Bosch-Ic-brush:before{content:\"\\e0c3\"}.Bosch-Ic-bug:before{content:\"\\e0c4\"}.Bosch-IC-building:before{content:\"\\e0c5\"}.Bosch-IC-cactus:before{content:\"\\e0c6\"}.Bosch-IC-calculate:before{content:\"\\e0c7\"}.Bosch-Ic-calendar:before{content:\"\\e0c8\"}.Bosch-Ic-calendar-add:before{content:\"\\e0c9\"}.Bosch-IC-calendar-settings:before{content:\"\\e0ca\"}.Bosch-Ic-call:before{content:\"\\e0cb\"}.Bosch-IC-call-add:before{content:\"\\e0cc\"}.Bosch-IC-call-remove:before{content:\"\\e0cd\"}.Bosch-IC-call-sos:before{content:\"\\e0ce\"}.Bosch-IC-call-team:before{content:\"\\e0cf\"}.Bosch-Ic-call-wifi:before{content:\"\\e0d0\"}.Bosch-Ic-camera:before{content:\"\\e0d1\"}.Bosch-Ic-camera-switch:before{content:\"\\e0d2\"}.Bosch-IC-car:before{content:\"\\e0d3\"}.Bosch-IC-car-clock:before{content:\"\\e0d4\"}.Bosch-IC-car-connected:before{content:\"\\e0d5\"}.Bosch-IC-car-locator:before{content:\"\\e0d6\"}.Bosch-IC-car-mechanic:before{content:\"\\e0d7\"}.Bosch-IC-car-rental:before{content:\"\\e0d8\"}.Bosch-IC-car-seat-connected:before{content:\"\\e0d9\"}.Bosch-IC-car-side:before{content:\"\\e0da\"}.Bosch-IC-car-side-user:before{content:\"\\e0db\"}.Bosch-Ic-milkrun-cart:before{content:\"\\e0dc\"}.Bosch-Ic-certificate:before{content:\"\\e0dd\"}.Bosch-IC-certificate-pki:before{content:\"\\e0de\"}.Bosch-IC-change:before{content:\"\\e0df\"}.Bosch-Ic-chart:before{content:\"\\e0e0\"}.Bosch-IC-chart-check:before{content:\"\\e0e1\"}.Bosch-IC-chart-dummy:before{content:\"\\e0e2\"}.Bosch-Ic-diagram:before{content:\"\\e0e3\"}.Bosch-Ic-chat:before{content:\"\\e0e4\"}.Bosch-IC-chat-add:before{content:\"\\e0e5\"}.Bosch-IC-chat-question-answer:before{content:\"\\e0e6\"}.Bosch-IC-chat-remove:before{content:\"\\e0e7\"}.Bosch-Ic-checklist:before{content:\"\\e0e8\"}.Bosch-Ic-checkmark:before{content:\"\\e0e9\"}.Bosch-Ic-chip:before{content:\"\\e0ea\"}.Bosch-IC-circle-segment:before{content:\"\\e0eb\"}.Bosch-IC-clear-all:before{content:\"\\e0ec\"}.Bosch-Ic-clipboard:before{content:\"\\e0ed\"}.Bosch-Ic-clipboard-list:before{content:\"\\e0ee\"}.Bosch-Ic-clock:before{content:\"\\e0ef\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-IC-close-all:before{content:\"\\e0f1\"}.Bosch-Ic-cloud:before{content:\"\\e0f2\"}.Bosch-IC-cloud-co2:before{content:\"\\e0f3\"}.Bosch-Ic-cloud-download:before{content:\"\\e0f4\"}.Bosch-IC-cloud-nox:before{content:\"\\e0f5\"}.Bosch-Ic-cloud-upload:before{content:\"\\e0f6\"}.Bosch-IC-club-arm-chair:before{content:\"\\e0f7\"}.Bosch-IC-coffee-break:before{content:\"\\e0f8\"}.Bosch-IC-coin-currency:before{content:\"\\e0f9\"}.Bosch-Ic-currency:before{content:\"\\e0fa\"}.Bosch-Ic-color-picker:before{content:\"\\e0fb\"}.Bosch-IC-colors:before{content:\"\\e0fc\"}.Bosch-Ic-command:before{content:\"\\e0fd\"}.Bosch-IC-communicator:before{content:\"\\e0fe\"}.Bosch-Ic-compare:before{content:\"\\e0ff\"}.Bosch-Ic-components:before{content:\"\\e100\"}.Bosch-Ic-components-available:before{content:\"\\e101\"}.Bosch-Ic-components-reservation:before{content:\"\\e102\"}.Bosch-IC-concierge-bell:before{content:\"\\e103\"}.Bosch-IC-configuration:before{content:\"\\e104\"}.Bosch-Ic-confirmation:before{content:\"\\e105\"}.Bosch-Ic-unconnected:before{content:\"\\e106\"}.Bosch-Ic-connected:before{content:\"\\e107\"}.Bosch-IC-connectivity:before{content:\"\\e108\"}.Bosch-Ic-builder:before{content:\"\\e109\"}.Bosch-IC-consumer-goods:before{content:\"\\e10a\"}.Bosch-IC-contract:before{content:\"\\e10b\"}.Bosch-Ic-copy:before{content:\"\\e10c\"}.Bosch-IC-cube:before{content:\"\\e10d\"}.Bosch-IC-cube-arrows:before{content:\"\\e10e\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-IC-cube-shock:before{content:\"\\e110\"}.Bosch-IC-cube-stacked:before{content:\"\\e111\"}.Bosch-Ic-customerservice:before{content:\"\\e112\"}.Bosch-IC-cutlery:before{content:\"\\e113\"}.Bosch-IC-damper:before{content:\"\\e114\"}.Bosch-IC-danger-flash:before{content:\"\\e115\"}.Bosch-IC-data-loss:before{content:\"\\e116\"}.Bosch-Ic-dealer:before{content:\"\\e117\"}.Bosch-Ic-delete:before{content:\"\\e118\"}.Bosch-Ic-delivery:before{content:\"\\e119\"}.Bosch-IC-denied:before{content:\"\\e11a\"}.Bosch-Ic-desktop:before{content:\"\\e11b\"}.Bosch-IC-desktop-application-2d:before{content:\"\\e11c\"}.Bosch-IC-desktop-application-3d:before{content:\"\\e11d\"}.Bosch-IC-development:before{content:\"\\e11e\"}.Bosch-Ic-devices:before{content:\"\\e11f\"}.Bosch-IC-devices-mobile:before{content:\"\\e120\"}.Bosch-Ic-directory:before{content:\"\\e121\"}.Bosch-Ic-dna:before{content:\"\\e122\"}.Bosch-Ic-do-not-disturb:before{content:\"\\e123\"}.Bosch-Ic-document:before{content:\"\\e124\"}.Bosch-Ic-document-add:before{content:\"\\e125\"}.Bosch-IC-document-arrow-down:before{content:\"\\e126\"}.Bosch-IC-document-arrow-up:before{content:\"\\e127\"}.Bosch-Ic-document-audio:before{content:\"\\e128\"}.Bosch-IC-document-check:before{content:\"\\e129\"}.Bosch-Ic-document-to-cloud:before{content:\"\\e12a\"}.Bosch-Ic-document-copy:before{content:\"\\e12b\"}.Bosch-Ic-document-delete:before{content:\"\\e12c\"}.Bosch-Ic-document-to-doc:before{content:\"\\e12d\"}.Bosch-Ic-document-edit:before{content:\"\\e12e\"}.Bosch-Ic-document-error:before{content:\"\\e12f\"}.Bosch-Ic-document-locked:before{content:\"\\e130\"}.Bosch-Ic-log-file:before{content:\"\\e131\"}.Bosch-Ic-document-to-one:before{content:\"\\e132\"}.Bosch-Ic-document-to-pdf:before{content:\"\\e133\"}.Bosch-Ic-document-plain:before{content:\"\\e134\"}.Bosch-Ic-document-plain-add:before{content:\"\\e135\"}.Bosch-Ic-document-plain-delete:before{content:\"\\e136\"}.Bosch-Ic-document-to-ppt:before{content:\"\\e137\"}.Bosch-Ic-document-to-pub:before{content:\"\\e138\"}.Bosch-Ic-document-to-rtf:before{content:\"\\e139\"}.Bosch-Ic-document-save-to:before{content:\"\\e13a\"}.Bosch-Ic-search-document:before{content:\"\\e13b\"}.Bosch-Ic-document-settings:before{content:\"\\e13c\"}.Bosch-Ic-document-test:before{content:\"\\e13d\"}.Bosch-Ic-document-text:before{content:\"\\e13e\"}.Bosch-Ic-document-to-txt:before{content:\"\\e13f\"}.Bosch-Ic-document-to-vsd:before{content:\"\\e140\"}.Bosch-Ic-document-to-xls:before{content:\"\\e141\"}.Bosch-Ic-document-to-xml:before{content:\"\\e142\"}.Bosch-Ic-document-to-zip:before{content:\"\\e143\"}.Bosch-IC-dome:before{content:\"\\e144\"}.Bosch-IC-door:before{content:\"\\e145\"}.Bosch-IC-door-sensor:before{content:\"\\e146\"}.Bosch-Ic-arrow-expand-down:before{content:\"\\e147\"}.Bosch-Ic-arrow-expand-double-down:before{content:\"\\e148\"}.Bosch-Ic-arrow-left-circle:before{content:\"\\e889\"}.Bosch-Ic-arrow-right-circle:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-down-circle:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-up-circle:before{content:\"\\e88d\"}.Bosch-Ic-arrow-left-down-circle:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-up-circle:before{content:\"\\e88a\"}.Bosch-Ic-arrow-up-circle:before{content:\"\\e88e\"}.Bosch-Ic-download:before{content:\"\\e149\"}.Bosch-IC-drag-handle:before{content:\"\\e14a\"}.Bosch-IC-dragdrop:before{content:\"\\e14b\"}.Bosch-IC-drop:before{content:\"\\e14c\"}.Bosch-IC-e-Charging:before{content:\"\\e14d\"}.Bosch-IC-e-mobility:before{content:\"\\e14e\"}.Bosch-IC-ear-off:before{content:\"\\e14f\"}.Bosch-IC-ear-on:before{content:\"\\e150\"}.Bosch-IC-eco-system:before{content:\"\\e151\"}.Bosch-Ic-edit:before{content:\"\\e152\"}.Bosch-IC-education:before{content:\"\\e153\"}.Bosch-IC-efficiency:before{content:\"\\e154\"}.Bosch-IC-elevator:before{content:\"\\e155\"}.Bosch-IC-elevator-alarm:before{content:\"\\e156\"}.Bosch-IC-elevator-cloud:before{content:\"\\e157\"}.Bosch-IC-elevator-headset:before{content:\"\\e158\"}.Bosch-IC-elevator-service:before{content:\"\\e159\"}.Bosch-IC-emergency-exit:before{content:\"\\e15a\"}.Bosch-IC-emoji-happy:before{content:\"\\e15b\"}.Bosch-IC-emoji-neutral:before{content:\"\\e15c\"}.Bosch-IC-emoji-sad:before{content:\"\\e15d\"}.Bosch-IC-emoji-super-happy:before{content:\"\\e15e\"}.Bosch-IC-emoji-very-sad:before{content:\"\\e15f\"}.Bosch-IC-EU-energy-label:before{content:\"\\e160\"}.Bosch-IC-excavator:before{content:\"\\e161\"}.Bosch-Ic-exit:before{content:\"\\e163\"}.Bosch-IC-expansion-arrows:before{content:\"\\e164\"}.Bosch-IC-explosion:before{content:\"\\e165\"}.Bosch-Ic-export:before{content:\"\\e166\"}.Bosch-IC-externallink:before{content:\"\\e167\"}.Bosch-IC-fast-forward:before{content:\"\\e168\"}.Bosch-IC-faucet:before{content:\"\\e169\"}.Bosch-IC-favorites:before{content:\"\\e16a\"}.Bosch-IC-fax:before{content:\"\\e16b\"}.Bosch-IC-female:before{content:\"\\e16c\"}.Bosch-IC-film:before{content:\"\\e16d\"}.Bosch-Ic-filter:before{content:\"\\e16e\"}.Bosch-IC-filter-success:before{content:\"\\e16f\"}.Bosch-IC-fingerprint:before{content:\"\\e170\"}.Bosch-IC-fire:before{content:\"\\e171\"}.Bosch-IC-fire-emergency:before{content:\"\\e172\"}.Bosch-IC-fireworks:before{content:\"\\e173\"}.Bosch-Ic-first-aid:before{content:\"\\e174\"}.Bosch-IC-first-aid-cross:before{content:\"\\e175\"}.Bosch-IC-fit-to-sceen:before{content:\"\\e176\"}.Bosch-Ic-flag:before{content:\"\\e177\"}.Bosch-Ic-danger-flash-outline:before{content:\"\\e178\"}.Bosch-IC-flask:before{content:\"\\e179\"}.Bosch-Ic-flexpress:before{content:\"\\e17a\"}.Bosch-Ic-twitter:before{content:\"\\ea8f\"}.Bosch-Ic-folder:before{content:\"\\e17b\"}.Bosch-Ic-download-from-folder:before{content:\"\\e17c\"}.Bosch-Ic-download-frame:before{content:\"\\e893\"}.Bosch-IC-folder-open:before{content:\"\\e17d\"}.Bosch-Ic-upload-to-folder:before{content:\"\\e17e\"}.Bosch-Ic-fork-lift:before{content:\"\\e17f\"}.Bosch-IC-fork-lift-locator:before{content:\"\\e180\"}.Bosch-Ic-arrow-expand-right:before{content:\"\\e181\"}.Bosch-Ic-arrow-expand-double-right:before{content:\"\\e182\"}.Bosch-IC-full-empty:before{content:\"\\e183\"}.Bosch-Ic-fullscreen:before{content:\"\\e184\"}.Bosch-IC-fullscreen-arrows:before{content:\"\\e185\"}.Bosch-Ic-fullscreen-exit:before{content:\"\\e186\"}.Bosch-IC-g-arrow-down:before{content:\"\\e187\"}.Bosch-IC-g-arrow-up:before{content:\"\\e188\"}.Bosch-IC-generator:before{content:\"\\e189\"}.Bosch-IC-gift:before{content:\"\\e18a\"}.Bosch-IC-glance:before{content:\"\\e18b\"}.Bosch-IC-glas-plate:before{content:\"\\e18c\"}.Bosch-IC-globe:before{content:\"\\e18d\"}.Bosch-IC-globe-arrow:before{content:\"\\e18e\"}.Bosch-IC-glossary:before{content:\"\\e18f\"}.Bosch-Ic-grid:before{content:\"\\e190\"}.Bosch-IC-hammer:before{content:\"\\e191\"}.Bosch-Ic-hand:before{content:\"\\e192\"}.Bosch-IC-hand-motion:before{content:\"\\e193\"}.Bosch-IC-hand-motion-off:before{content:\"\\e194\"}.Bosch-IC-handlewithcare:before{content:\"\\e195\"}.Bosch-Ic-add-material-manually:before{content:\"\\e196\"}.Bosch-Ic-handover-check:before{content:\"\\e197\"}.Bosch-Ic-remove-from-handover:before{content:\"\\e198\"}.Bosch-IC-hanger:before{content:\"\\e199\"}.Bosch-Ic-health:before{content:\"\\e19a\"}.Bosch-IC-heart:before{content:\"\\e19b\"}.Bosch-Ic-history:before{content:\"\\e19c\"}.Bosch-Ic-home-outline:before{content:\"\\e19d\"}.Bosch-IC-home-locator:before{content:\"\\e19e\"}.Bosch-Ic-sandclock:before{content:\"\\e19f\"}.Bosch-IC-hydrant:before{content:\"\\e1a0\"}.Bosch-IC-ice:before{content:\"\\e1a1\"}.Bosch-Ic-image:before{content:\"\\e1a2\"}.Bosch-IC-imagery-add:before{content:\"\\e1a3\"}.Bosch-IC-imagery-remove:before{content:\"\\e1a4\"}.Bosch-Ic-import:before{content:\"\\e1a5\"}.Bosch-Ic-imprint:before{content:\"\\e1a6\"}.Bosch-IC-impulse:before{content:\"\\e1a7\"}.Bosch-Ic-fabric:before{content:\"\\e1a8\"}.Bosch-IC-industry-clock:before{content:\"\\e1a9\"}.Bosch-IC-industry-connected:before{content:\"\\e1aa\"}.Bosch-Ic-information:before{content:\"\\e1ac\"}.Bosch-Ic-information-tooltip:before{content:\"\\e1ac\"}.Bosch-IC-interval:before{content:\"\\e1ad\"}.Bosch-IC-it-device:before{content:\"\\e1ae\"}.Bosch-IC-jewel:before{content:\"\\e1af\"}.Bosch-IC-keyboard:before{content:\"\\e1b0\"}.Bosch-IC-label:before{content:\"\\e1b1\"}.Bosch-Ic-laptop-01:before{content:\"\\e1b2\"}.Bosch-IC-laser:before{content:\"\\e1b3\"}.Bosch-IC-layout:before{content:\"\\e1b4\"}.Bosch-IC-leaf:before{content:\"\\e1b5\"}.Bosch-IC-led:before{content:\"\\e1b6\"}.Bosch-Ic-world:before{content:\"\\e8a0\"}.Bosch-Ic-facebook:before{content:\"\\ea87\"}.Bosch-Ic-checkmark-outline:before{content:\"\\e891\"}.Bosch-Ic-youtube:before{content:\"\\ea96\"}.Bosch-Ic-share:before{content:\"\\e242\"}.Bosch-Ic-user:before{content:\"\\e29b\"}.Bosch-Ic-upload_outline:before{content:\"\\e89f\"}.Bosch-Ic-download_outline:before{content:\"\\e893\"}.Bosch-Ic-wishlist-add:before{content:\"\\e3d3\"}.Bosch-Ic-youtube-frame:before{content:\"\\ea96\"}.Bosch-Ic-less-minimize:before{content:\"\\e1ba\"}.Bosch-IC-lightbulb:before{content:\"\\e1bb\"}.bosch-Ic-innovation:before{content:\"\\e1bc\"}.Bosch-IC-line-chart:before{content:\"\\e1bd\"}.Bosch-Ic-link:before{content:\"\\e1be\"}.Bosch-IC-link-broken:before{content:\"\\e1bf\"}.Bosch-Ic-list-view:before{content:\"\\e1c0\"}.Bosch-Ic-menu:before{content:\"\\e1c1\"}.Bosch-IC-livechat:before{content:\"\\e1c2\"}.Bosch-Ic-location-pin:before{content:\"\\e1c3\"}.Bosch-IC-locator-ip-connected:before{content:\"\\e1c4\"}.Bosch-IC-locator-ip-disconnected:before{content:\"\\e1c5\"}.Bosch-IC-locator-spot:before{content:\"\\e1c6\"}.Bosch-IC-locator-spot-check:before{content:\"\\e1c7\"}.Bosch-Ic-lock-closed:before{content:\"\\e1c8\"}.Bosch-Ic-lock-open:before{content:\"\\e1c9\"}.Bosch-Ic-log-in:before{content:\"\\e1ca\"}.Bosch-Ic-logistics:before{content:\"\\e1cb\"}.Bosch-Ic-log-out:before{content:\"\\e1cc\"}.Bosch-IC-machine:before{content:\"\\e1cd\"}.Bosch-Ic-magnet:before{content:\"\\e1ce\"}.Bosch-Ic-mail:before{content:\"\\e1cf\"}.Bosch-Ic-mail-forward:before{content:\"\\e1d0\"}.Bosch-Ic-mail-open:before{content:\"\\e1d1\"}.Bosch-Ic-mail-out:before{content:\"\\e1d2\"}.Bosch-Ic-mail-answer:before{content:\"\\e1d3\"}.Bosch-IC-male:before{content:\"\\e1d4\"}.Bosch-Ic-map:before{content:\"\\e1d5\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-IC-material-check:before{content:\"\\e1d7\"}.Bosch-Ic-remove-material:before{content:\"\\e1d8\"}.Bosch-IC-maximize:before{content:\"\\e1d9\"}.Bosch-IC-mechanic:before{content:\"\\e1da\"}.Bosch-IC-megaphone:before{content:\"\\e1db\"}.Bosch-Ic-message:before{content:\"\\e1dc\"}.Bosch-IC-mic:before{content:\"\\e1dd\"}.Bosch-Ic-microphone:before{content:\"\\e1de\"}.Bosch-IC-microphone-classic:before{content:\"\\e1df\"}.Bosch-IC-minimize:before{content:\"\\e1e0\"}.Bosch-IC-money-currency:before{content:\"\\e1e1\"}.Bosch-IC-money-dollar:before{content:\"\\e1e2\"}.Bosch-IC-moneybag-currency:before{content:\"\\e1e3\"}.Bosch-IC-moneybag-dollar:before{content:\"\\e1e4\"}.Bosch-IC-monument:before{content:\"\\e1e5\"}.Bosch-IC-moon:before{content:\"\\e1e6\"}.Bosch-IC-mouse:before{content:\"\\e1ea\"}.Bosch-IC-mouse-left:before{content:\"\\e1eb\"}.Bosch-IC-mouse-right:before{content:\"\\e1ec\"}.Bosch-IC-mouth:before{content:\"\\e1ed\"}.Bosch-Ic-my-product:before{content:\"\\e1ee\"}.Bosch-IC-new-way-work:before{content:\"\\e1ef\"}.Bosch-Ic-mail-sent:before{content:\"\\e1f0\"}.Bosch-Ic-news:before{content:\"\\e1f1\"}.Bosch-IC-nose:before{content:\"\\e1f2\"}.Bosch-Ic-notes:before{content:\"\\e1f3\"}.Bosch-Ic-notes-edit:before{content:\"\\e1f4\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-IC-notification-active:before{content:\"\\e1f6\"}.Bosch-IC-notification-add:before{content:\"\\e1f7\"}.Bosch-IC-notification-remove:before{content:\"\\e1f8\"}.Bosch-Ic-oil-trailer:before{content:\"\\e1f9\"}.Bosch-IC-omnichannel:before{content:\"\\e1fa\"}.Bosch-Ic-show-more-horizontal:before{content:\"\\e1fb\"}.Bosch-IC-origami-boat:before{content:\"\\e1fc\"}.Bosch-Ic-palette-trailer:before{content:\"\\e1fd\"}.Bosch-IC-pallete-car-connected:before{content:\"\\e1fe\"}.Bosch-IC-panel:before{content:\"\\e1ff\"}.Bosch-Ic-paperclip:before{content:\"\\e200\"}.Bosch-IC-paperplane:before{content:\"\\e201\"}.Bosch-IC-parking:before{content:\"\\e202\"}.Bosch-Ic-pause:before{content:\"\\e203\"}.Bosch-Ic-creditcard:before{content:\"\\e204\"}.Bosch-IC-people:before{content:\"\\e205\"}.Bosch-IC-petrol-station:before{content:\"\\e206\"}.Bosch-IC-piggybank:before{content:\"\\e207\"}.Bosch-Ic-pin:before{content:\"\\e208\"}.Bosch-IC-pin-modern:before{content:\"\\e209\"}.Bosch-IC-plane-side:before{content:\"\\e20a\"}.Bosch-Ic-play:before{content:\"\\e20b\"}.Bosch-IC-plug-12V:before{content:\"\\e20c\"}.Bosch-IC-podium-speaker:before{content:\"\\e20d\"}.Bosch-IC-police:before{content:\"\\e20e\"}.Bosch-IC-poop:before{content:\"\\e20f\"}.Bosch-IC-post-it:before{content:\"\\e210\"}.Bosch-IC-postal-code:before{content:\"\\e211\"}.Bosch-IC-power:before{content:\"\\e212\"}.Bosch-Ic-print:before{content:\"\\e213\"}.Bosch-IC-prototyping:before{content:\"\\e214\"}.Bosch-IC-puzzle:before{content:\"\\e215\"}.Bosch-IC-quad:before{content:\"\\e216\"}.Bosch-IC-question:before{content:\"\\e217\"}.Bosch-IC-radar:before{content:\"\\e218\"}.Bosch-IC-radio:before{content:\"\\e219\"}.Bosch-Ic-radio:before{content:\"\\e21a\"}.Bosch-IC-redirect:before{content:\"\\e21b\"}.Bosch-Ic-redo:before{content:\"\\e21c\"}.Bosch-IC-reference:before{content:\"\\e21d\"}.Bosch-Ic-refresh:before{content:\"\\e21e\"}.Bosch-IC-refresh-cloud:before{content:\"\\e21f\"}.Bosch-Ic-registration:before{content:\"\\e220\"}.Bosch-IC-remote:before{content:\"\\e221\"}.Bosch-Ic-reset:before{content:\"\\e222\"}.Bosch-IC-resolution:before{content:\"\\e223\"}.Bosch-Ic-robot-automation:before{content:\"\\e224\"}.Bosch-IC-robot-connected:before{content:\"\\e225\"}.Bosch-IC-robothead:before{content:\"\\e226\"}.Bosch-IC-rocket:before{content:\"\\e227\"}.Bosch-Ic-rotation:before{content:\"\\e228\"}.Bosch-Ic-rotate-360-degrees:before{content:\"\\e229\"}.Bosch-Ic-rotate-clockwise:before{content:\"\\e22a\"}.Bosch-Ic-rotate-counterclockwise:before{content:\"\\e22b\"}.Bosch-IC-rotation-y-down:before{content:\"\\e22c\"}.Bosch-IC-rotation-y-up:before{content:\"\\e22d\"}.Bosch-Ic-navigation:before{content:\"\\e22e\"}.Bosch-Ic-measure:before{content:\"\\e22f\"}.Bosch-IC-ruler-pen:before{content:\"\\e230\"}.Bosch-Ic-satelite:before{content:\"\\e231\"}.Bosch-Ic-save:before{content:\"\\e232\"}.Bosch-IC-scale:before{content:\"\\e233\"}.Bosch-Ic-cut:before{content:\"\\e234\"}.Bosch-Ic-search:before{content:\"\\e235\"}.Bosch-IC-security:before{content:\"\\e236\"}.Bosch-IC-security-check:before{content:\"\\e237\"}.Bosch-IC-security-user:before{content:\"\\e238\"}.Bosch-Ic-sensor:before{content:\"\\e239\"}.Bosch-Ic-server:before{content:\"\\e23a\"}.Bosch-IC-server-expansion:before{content:\"\\e23b\"}.Bosch-IC-server-rate:before{content:\"\\e23c\"}.Bosch-IC-service-agent:before{content:\"\\e23d\"}.Bosch-IC-service-time:before{content:\"\\e23e\"}.Bosch-Ic-settings:before{content:\"\\e23f\"}.Bosch-IC-settings-arrows:before{content:\"\\e240\"}.Bosch-IC-settings-connected:before{content:\"\\e241\"}.Bosch-Ic-share-outline:before{content:\"\\e242\"}.Bosch-IC-shield-stop:before{content:\"\\e243\"}.Bosch-IC-ship:before{content:\"\\e244\"}.Bosch-IC-ship-side:before{content:\"\\e245\"}.Bosch-Ic-shoppingcart:before{content:\"\\e246\"}.Bosch-IC-shoppingcart-add:before{content:\"\\e247\"}.Bosch-IC-shoppingcart-switch:before{content:\"\\e248\"}.Bosch-IC-signal-full:before{content:\"\\e249\"}.Bosch-IC-signal-half:before{content:\"\\e24a\"}.Bosch-IC-signal-lost:before{content:\"\\e24b\"}.Bosch-IC-signpost:before{content:\"\\e24c\"}.Bosch-IC-simcard:before{content:\"\\e24d\"}.Bosch-IC-skyscraper:before{content:\"\\e24e\"}.Bosch-IC-smartcity:before{content:\"\\e24f\"}.Bosch-IC-smarthome:before{content:\"\\e250\"}.Bosch-Ic-phone:before{content:\"\\e251\"}.Bosch-IC-smartphone-acoustic:before{content:\"\\e252\"}.Bosch-IC-smartphone-arrow-right:before{content:\"\\e253\"}.Bosch-IC-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.Bosch-IC-smartphone-clock:before{content:\"\\e255\"}.Bosch-IC-smartphone-cloud:before{content:\"\\e256\"}.Bosch-IC-smartphone-loading-wheel:before{content:\"\\e257\"}.Bosch-IC-smartphone-smashed:before{content:\"\\e258\"}.Bosch-IC-smartphone-vibration:before{content:\"\\e259\"}.Bosch-IC-smartphone-z:before{content:\"\\e25a\"}.Bosch-Ic-spareparts:before{content:\"\\e25c\"}.Bosch-IC-spark-plug:before{content:\"\\e25d\"}.Bosch-IC-speaker-acoustic:before{content:\"\\e25e\"}.Bosch-IC-speedometer:before{content:\"\\e25f\"}.Bosch-IC-stadium:before{content:\"\\e260\"}.Bosch-IC-stairs:before{content:\"\\e261\"}.Bosch-IC-stamp:before{content:\"\\e262\"}.Bosch-IC-standby:before{content:\"\\e263\"}.Bosch-IC-steering-wheel:before{content:\"\\e264\"}.Bosch-IC-steering-wheel-connected:before{content:\"\\e265\"}.Bosch-IC-steering-wheel-hands:before{content:\"\\e266\"}.Bosch-IC-steps:before{content:\"\\e267\"}.Bosch-IC-stethoscope:before{content:\"\\e268\"}.Bosch-Ic-stop:before{content:\"\\e269\"}.Bosch-IC-stopwatch:before{content:\"\\e26a\"}.Bosch-IC-storage-local:before{content:\"\\e26b\"}.Bosch-IC-structure:before{content:\"\\e26c\"}.Bosch-IC-subtitles:before{content:\"\\e26d\"}.Bosch-Ic-summary:before{content:\"\\e26f\"}.Bosch-IC-sun:before{content:\"\\e270\"}.Bosch-IC-sun-half-filled:before{content:\"\\e271\"}.Bosch-IC-sunshade-table:before{content:\"\\e272\"}.Bosch-IC-surveillance:before{content:\"\\e273\"}.Bosch-IC-table-chairs:before{content:\"\\e274\"}.Bosch-Ic-tablet:before{content:\"\\e275\"}.Bosch-IC-team:before{content:\"\\e276\"}.Bosch-IC-team-lightbulb:before{content:\"\\e277\"}.Bosch-IC-technical-service:before{content:\"\\e278\"}.Bosch-Ic-temperature:before{content:\"\\e279\"}.Bosch-IC-temperature-high:before{content:\"\\e27a\"}.Bosch-IC-temperature-low:before{content:\"\\e27b\"}.Bosch-IC-theater:before{content:\"\\e27c\"}.Bosch-Ic-dislike:before{content:\"\\e27d\"}.Bosch-Ic-like:before{content:\"\\e27e\"}.Bosch-Ic-ticket:before{content:\"\\e27f\"}.Bosch-IC-touch:before{content:\"\\e280\"}.Bosch-IC-touch-sos:before{content:\"\\e281\"}.Bosch-IC-towing-truck:before{content:\"\\e282\"}.Bosch-IC-train:before{content:\"\\e283\"}.Bosch-IC-train-side:before{content:\"\\e284\"}.Bosch-Ic-transport-movements:before{content:\"\\e285\"}.Bosch-IC-transscript:before{content:\"\\e286\"}.Bosch-IC-transscript-off:before{content:\"\\e287\"}.Bosch-IC-transscript-on:before{content:\"\\e288\"}.Bosch-IC-tree:before{content:\"\\e289\"}.Bosch-IC-tricycle:before{content:\"\\e28a\"}.Bosch-Ic-default-trailer:before{content:\"\\e28b\"}.Bosch-Ic-taxi-trailer:before{content:\"\\e28c\"}.Bosch-Ic-shelve-trailer:before{content:\"\\e28d\"}.Bosch-IC-trophy:before{content:\"\\e28e\"}.Bosch-IC-truck:before{content:\"\\e28f\"}.Bosch-IC-tuktuk:before{content:\"\\e290\"}.Bosch-Ic-tune:before{content:\"\\e291\"}.Bosch-IC-typography:before{content:\"\\e292\"}.Bosch-IC-ui:before{content:\"\\e293\"}.Bosch-IC-umbrella:before{content:\"\\e294\"}.Bosch-Ic-undo:before{content:\"\\e295\"}.Bosch-Ic-arrow-expand-up:before{content:\"\\e296\"}.Bosch-Ic-arrow-expand-double-up:before{content:\"\\e297\"}.Bosch-Ic-sort:before{content:\"\\e298\"}.Bosch-Ic-upload:before{content:\"\\e299\"}.Bosch-IC-usb:before{content:\"\\e29a\"}.Bosch-Ic-user-outline:before{content:\"\\e29b\"}.Bosch-IC-user-advanced:before{content:\"\\e29c\"}.Bosch-IC-user-beginner:before{content:\"\\e29d\"}.Bosch-IC-user-desktop:before{content:\"\\e29e\"}.Bosch-IC-user-down:before{content:\"\\e29f\"}.Bosch-IC-user-falling:before{content:\"\\e2a0\"}.Bosch-IC-user-hand:before{content:\"\\e2a1\"}.Bosch-IC-user-hand-stop:before{content:\"\\e2a2\"}.Bosch-IC-user-professional:before{content:\"\\e2a3\"}.Bosch-IC-user-run:before{content:\"\\e2a4\"}.Bosch-IC-user-share:before{content:\"\\e2a5\"}.Bosch-IC-user-voice:before{content:\"\\e2a6\"}.Bosch-IC-user-walk:before{content:\"\\e2a7\"}.Bosch-Ic-videocam:before{content:\"\\e2a8\"}.Bosch-Ic-videocam-disabled:before{content:\"\\e2a9\"}.Bosch-IC-video-record:before{content:\"\\e2aa\"}.Bosch-IC-virtual-reality:before{content:\"\\e2ab\"}.Bosch-Ic-volume-disabled:before{content:\"\\e2ac\"}.Bosch-Ic-volume-high:before{content:\"\\e2ad\"}.Bosch-Ic-volume-low:before{content:\"\\e2ae\"}.Bosch-Ic-volume-off:before{content:\"\\e2af\"}.Bosch-Ic-magic-wand:before{content:\"\\e2b0\"}.Bosch-Ic-hidden-project:before{content:\"\\e2b2\"}.Bosch-Ic-show:before{content:\"\\e2b3\"}.Bosch-IC-water-shower:before{content:\"\\e2b4\"}.Bosch-IC-welcome:before{content:\"\\e2b5\"}.Bosch-IC-wellness:before{content:\"\\e2b6\"}.Bosch-IC-wheelchair:before{content:\"\\e2b7\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-IC-wide-angled-arrow:before{content:\"\\e2b9\"}.Bosch-Ic-wifi:before{content:\"\\e2ba\"}.Bosch-Ic-window-analysis:before{content:\"\\e2bb\"}.Bosch-Ic-window-browser:before{content:\"\\e2bc\"}.Bosch-Ic-window-console:before{content:\"\\e2bd\"}.Bosch-Ic-gaussian-view:before{content:\"\\e2be\"}.Bosch-Ic-window-info:before{content:\"\\e2bf\"}.Bosch-IC-window-new:before{content:\"\\e2c0\"}.Bosch-IC-window-resize:before{content:\"\\e2c1\"}.Bosch-Ic-general-scaling-view:before{content:\"\\e2c2\"}.Bosch-Ic-restore-window:before{content:\"\\e2c3\"}.Bosch-Ic-window-statistic:before{content:\"\\e2c4\"}.Bosch-Ic-wishlist:before{content:\"\\e2c5\"}.Bosch-Ic-work-orders:before{content:\"\\e2c6\"}.Bosch-IC-worldwideweb:before{content:\"\\e2c7\"}.Bosch-Ic-wrench:before{content:\"\\e2c8\"}.Bosch-Ic-zoom-in:before{content:\"\\e2c9\"}.Bosch-Ic-zoom-out:before{content:\"\\e2ca\"}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */body{margin:0;display:flex;flex-direction:column}main,*{font-family:var(--bci-global-font-family)}::selection,::-moz-selection{background-color:#007bc0}a::-moz-selection{color:#ffffff}.lead{margin-bottom:24px;font-size:18px;font-weight:300;line-height:1.4}@media (min-width: 768px){.lead{font-size:24px}}.bci-core-datepicker{display:flex;flex-direction:column;position:relative;font-size:14px;width:292px}.bci-core-datepicker .bci-core-hide{display:none}.bci-core-datepicker .bci-core-datepicker-container{background:#ffffff;overflow-y:auto;overflow-x:hidden;white-space:normal;text-align:center;visibility:visible}.bci-core-datepicker .bci-core-datepicker-container .bci-core-month-container{display:inline-block;width:100%}";

const Datepicker = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
  }
  async componentWillLoad() {
    await utils.setLocale();
    if (!this.formatDate) {
      this.formatDate = utils.state.dateFormat;
    }
  }
  render() {
    const currentMonth = this.selectedMonth;
    const isoWeekday = getISODay(utils.createDate(new Date(), this.selectedYear, currentMonth, 0));
    const offset = this.sundayFirst ? isoWeekday + 1 : isoWeekday;
    const minimumDate = new Date(this.minDate);
    const maximumDate = new Date(this.maxDate);
    const dateRestrictionObj = {
      minimumDate,
      maximumDate,
    };
    return (index.h("div", { class: "bci-core-datepicker" }, index.h("div", { class: "bci-core-datepicker-container" }, index.h("div", { class: "bci-core-month-container" }, index.h("bci-week-header", { "data-test": "webcore.webcomponents.bci-datepicker.sunday-first", "sunday-first": this.sundayFirst }), index.h("bci-weekdays", { "data-test": "webcore.webcomponents.bci-datepicker.select-date", datesObj: this.datesObj, daysInMonth: utils.getDaysInMonth(utils.createDate(new Date(), this.selectedYear, currentMonth)), lastDayOfMonth: endOfMonth(utils.createDate(new Date(), this.selectedYear, this.selectedMonth, 0)), dateRestrictionObj: dateRestrictionObj, month: currentMonth, rangeDate: this.rangeDate, offset: offset, selectedDate: this.selectedDate, todaysDate: this.today, year: this.selectedYear, formatDate: this.formatDate, timeZone: this.timeZone })))));
  }
};
Datepicker.style = datepickerCss;

var toInteger_1 = _commonjsHelpers.createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = toInteger;

function toInteger(dirtyNumber) {
  if (dirtyNumber === null || dirtyNumber === true || dirtyNumber === false) {
    return NaN;
  }

  var number = Number(dirtyNumber);

  if (isNaN(number)) {
    return number;
  }

  return number < 0 ? Math.ceil(number) : Math.floor(number);
}

module.exports = exports.default;
});

const toInteger = /*@__PURE__*/_commonjsHelpers.getDefaultExportFromCjs(toInteger_1);

var getTimezoneOffsetInMilliseconds_1 = _commonjsHelpers.createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getTimezoneOffsetInMilliseconds;

/**
 * Google Chrome as of 67.0.3396.87 introduced timezones with offset that includes seconds.
 * They usually appear for dates that denote time before the timezones were introduced
 * (e.g. for 'Europe/Prague' timezone the offset is GMT+00:57:44 before 1 October 1891
 * and GMT+01:00:00 after that date)
 *
 * Date#getTimezoneOffset returns the offset in minutes and would return 57 for the example above,
 * which would lead to incorrect calculations.
 *
 * This function returns the timezone offset in milliseconds that takes seconds in account.
 */
function getTimezoneOffsetInMilliseconds(date) {
  var utcDate = new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()));
  utcDate.setUTCFullYear(date.getFullYear());
  return date.getTime() - utcDate.getTime();
}

module.exports = exports.default;
});

const getTimezoneOffsetInMilliseconds = /*@__PURE__*/_commonjsHelpers.getDefaultExportFromCjs(getTimezoneOffsetInMilliseconds_1);

/**
 * Returns the [year, month, day, hour, minute, seconds] tokens of the provided
 * `date` as it will be rendered in the `timeZone`.
 */
function tzTokenizeDate(date, timeZone) {
  var dtf = getDateTimeFormat(timeZone);
  return dtf.formatToParts ? partsOffset(dtf, date) : hackyOffset(dtf, date)
}

var typeToPos = {
  year: 0,
  month: 1,
  day: 2,
  hour: 3,
  minute: 4,
  second: 5,
};

function partsOffset(dtf, date) {
  try {
    var formatted = dtf.formatToParts(date);
    var filled = [];
    for (var i = 0; i < formatted.length; i++) {
      var pos = typeToPos[formatted[i].type];

      if (pos >= 0) {
        filled[pos] = parseInt(formatted[i].value, 10);
      }
    }
    return filled
  } catch (error) {
    if (error instanceof RangeError) {
      return [NaN]
    }
    throw error
  }
}

function hackyOffset(dtf, date) {
  var formatted = dtf.format(date).replace(/\u200E/g, '');
  var parsed = /(\d+)\/(\d+)\/(\d+),? (\d+):(\d+):(\d+)/.exec(formatted);
  // var [, fMonth, fDay, fYear, fHour, fMinute, fSecond] = parsed
  // return [fYear, fMonth, fDay, fHour, fMinute, fSecond]
  return [parsed[3], parsed[1], parsed[2], parsed[4], parsed[5], parsed[6]]
}

// Get a cached Intl.DateTimeFormat instance for the IANA `timeZone`. This can be used
// to get deterministic local date/time output according to the `en-US` locale which
// can be used to extract local time parts as necessary.
var dtfCache = {};
function getDateTimeFormat(timeZone) {
  if (!dtfCache[timeZone]) {
    // New browsers use `hourCycle`, IE and Chrome <73 does not support it and uses `hour12`
    var testDateFormatted = new Intl.DateTimeFormat('en-US', {
      hour12: false,
      timeZone: 'America/New_York',
      year: 'numeric',
      month: '2-digit',
      day: '2-digit',
      hour: '2-digit',
      minute: '2-digit',
      second: '2-digit',
    }).format(new Date('2014-06-25T04:00:00.123Z'));
    var hourCycleSupported =
      testDateFormatted === '06/25/2014, 00:00:00' ||
      testDateFormatted === '‎06‎/‎25‎/‎2014‎ ‎00‎:‎00‎:‎00';

    dtfCache[timeZone] = hourCycleSupported
      ? new Intl.DateTimeFormat('en-US', {
          hour12: false,
          timeZone: timeZone,
          year: 'numeric',
          month: '2-digit',
          day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
        })
      : new Intl.DateTimeFormat('en-US', {
          hourCycle: 'h23',
          timeZone: timeZone,
          year: 'numeric',
          month: '2-digit',
          day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
        });
  }
  return dtfCache[timeZone]
}

/**
 * Use instead of `new Date(Date.UTC(...))` to support years below 100 which doesn't work
 * otherwise due to the nature of the
 * [`Date` constructor](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date#interpretation_of_two-digit_years.
 *
 * For `Date.UTC(...)`, use `newDateUTC(...).getTime()`.
 */
function newDateUTC(fullYear, month, day, hour, minute, second, millisecond) {
  var utcDate = new Date(0);
  utcDate.setUTCFullYear(fullYear, month, day);
  utcDate.setUTCHours(hour, minute, second, millisecond);
  return utcDate
}

var MILLISECONDS_IN_HOUR$1 = 3600000;
var MILLISECONDS_IN_MINUTE$1 = 60000;

var patterns$1 = {
  timezone: /([Z+-].*)$/,
  timezoneZ: /^(Z)$/,
  timezoneHH: /^([+-]\d{2})$/,
  timezoneHHMM: /^([+-]\d{2}):?(\d{2})$/,
};

// Parse various time zone offset formats to an offset in milliseconds
function tzParseTimezone(timezoneString, date, isUtcDate) {
  var token;
  var absoluteOffset;

  // Empty string
  if (timezoneString === '') {
    return 0
  }

  // Z
  token = patterns$1.timezoneZ.exec(timezoneString);
  if (token) {
    return 0
  }

  var hours;

  // ±hh
  token = patterns$1.timezoneHH.exec(timezoneString);
  if (token) {
    hours = parseInt(token[1], 10);

    if (!validateTimezone(hours)) {
      return NaN
    }

    return -(hours * MILLISECONDS_IN_HOUR$1)
  }

  // ±hh:mm or ±hhmm
  token = patterns$1.timezoneHHMM.exec(timezoneString);
  if (token) {
    hours = parseInt(token[1], 10);
    var minutes = parseInt(token[2], 10);

    if (!validateTimezone(hours, minutes)) {
      return NaN
    }

    absoluteOffset = Math.abs(hours) * MILLISECONDS_IN_HOUR$1 + minutes * MILLISECONDS_IN_MINUTE$1;
    return hours > 0 ? -absoluteOffset : absoluteOffset
  }

  // IANA time zone
  if (isValidTimezoneIANAString(timezoneString)) {
    date = new Date(date || Date.now());
    var utcDate = isUtcDate ? date : toUtcDate(date);

    var offset = calcOffset(utcDate, timezoneString);

    var fixedOffset = isUtcDate ? offset : fixOffset(date, offset, timezoneString);

    return -fixedOffset
  }

  return NaN
}

function toUtcDate(date) {
  return newDateUTC(
    date.getFullYear(),
    date.getMonth(),
    date.getDate(),
    date.getHours(),
    date.getMinutes(),
    date.getSeconds(),
    date.getMilliseconds()
  )
}

function calcOffset(date, timezoneString) {
  var tokens = tzTokenizeDate(date, timezoneString);

  // ms dropped because it's not provided by tzTokenizeDate
  var asUTC = newDateUTC(
    tokens[0],
    tokens[1] - 1,
    tokens[2],
    tokens[3] % 24,
    tokens[4],
    tokens[5],
    0
  ).getTime();

  var asTS = date.getTime();
  var over = asTS % 1000;
  asTS -= over >= 0 ? over : 1000 + over;
  return asUTC - asTS
}

function fixOffset(date, offset, timezoneString) {
  var localTS = date.getTime();

  // Our UTC time is just a guess because our offset is just a guess
  var utcGuess = localTS - offset;

  // Test whether the zone matches the offset for this ts
  var o2 = calcOffset(new Date(utcGuess), timezoneString);

  // If so, offset didn't change, and we're done
  if (offset === o2) {
    return offset
  }

  // If not, change the ts by the difference in the offset
  utcGuess -= o2 - offset;

  // If that gives us the local time we want, we're done
  var o3 = calcOffset(new Date(utcGuess), timezoneString);
  if (o2 === o3) {
    return o2
  }

  // If it's different, we're in a hole time. The offset has changed, but we don't adjust the time
  return Math.max(o2, o3)
}

function validateTimezone(hours, minutes) {
  return -23 <= hours && hours <= 23 && (minutes == null || (0 <= minutes && minutes <= 59))
}

var validIANATimezoneCache = {};
function isValidTimezoneIANAString(timeZoneString) {
  if (validIANATimezoneCache[timeZoneString]) return true
  try {
    validIANATimezoneCache[timeZoneString] = true;
    return true
  } catch (error) {
    return false
  }
}

/** Regex to identify the presence of a time zone specifier in a date string */
var tzPattern = /(Z|[+-]\d{2}(?::?\d{2})?| UTC| [a-zA-Z]+\/[a-zA-Z_]+(?:\/[a-zA-Z_]+)?)$/;

var MILLISECONDS_IN_HOUR = 3600000;
var MILLISECONDS_IN_MINUTE = 60000;
var DEFAULT_ADDITIONAL_DIGITS = 2;

var patterns = {
  dateTimePattern: /^([0-9W+-]+)(T| )(.*)/,
  datePattern: /^([0-9W+-]+)(.*)/,
  plainTime: /:/,

  // year tokens
  YY: /^(\d{2})$/,
  YYY: [
    /^([+-]\d{2})$/, // 0 additional digits
    /^([+-]\d{3})$/, // 1 additional digit
    /^([+-]\d{4})$/, // 2 additional digits
  ],
  YYYY: /^(\d{4})/,
  YYYYY: [
    /^([+-]\d{4})/, // 0 additional digits
    /^([+-]\d{5})/, // 1 additional digit
    /^([+-]\d{6})/, // 2 additional digits
  ],

  // date tokens
  MM: /^-(\d{2})$/,
  DDD: /^-?(\d{3})$/,
  MMDD: /^-?(\d{2})-?(\d{2})$/,
  Www: /^-?W(\d{2})$/,
  WwwD: /^-?W(\d{2})-?(\d{1})$/,

  HH: /^(\d{2}([.,]\d*)?)$/,
  HHMM: /^(\d{2}):?(\d{2}([.,]\d*)?)$/,
  HHMMSS: /^(\d{2}):?(\d{2}):?(\d{2}([.,]\d*)?)$/,

  // time zone tokens (to identify the presence of a tz)
  timeZone: tzPattern,
};

/**
 * @name toDate
 * @category Common Helpers
 * @summary Convert the given argument to an instance of Date.
 *
 * @description
 * Convert the given argument to an instance of Date.
 *
 * If the argument is an instance of Date, the function returns its clone.
 *
 * If the argument is a number, it is treated as a timestamp.
 *
 * If an argument is a string, the function tries to parse it.
 * Function accepts complete ISO 8601 formats as well as partial implementations.
 * ISO 8601: http://en.wikipedia.org/wiki/ISO_8601
 * If the function cannot parse the string or the values are invalid, it returns Invalid Date.
 *
 * If the argument is none of the above, the function returns Invalid Date.
 *
 * **Note**: *all* Date arguments passed to any *date-fns* function is processed by `toDate`.
 * All *date-fns* functions will throw `RangeError` if `options.additionalDigits` is not 0, 1, 2 or undefined.
 *
 * @param {Date|String|Number} argument - the value to convert
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - the additional number of digits in the extended year format
 * @param {String} [options.timeZone=''] - used to specify the IANA time zone offset of a date String.
 * @returns {Date} the parsed date in the local time zone
 * @throws {TypeError} 1 argument required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // Convert string '2014-02-11T11:30:30' to date:
 * var result = toDate('2014-02-11T11:30:30')
 * //=> Tue Feb 11 2014 11:30:30
 *
 * @example
 * // Convert string '+02014101' to date,
 * // if the additional number of digits in the extended year format is 1:
 * var result = toDate('+02014101', {additionalDigits: 1})
 * //=> Fri Apr 11 2014 00:00:00
 */
function toDate(argument, dirtyOptions) {
  if (arguments.length < 1) {
    throw new TypeError('1 argument required, but only ' + arguments.length + ' present')
  }

  if (argument === null) {
    return new Date(NaN)
  }

  var options = dirtyOptions || {};

  var additionalDigits =
    options.additionalDigits == null
      ? DEFAULT_ADDITIONAL_DIGITS
      : toInteger(options.additionalDigits);
  if (additionalDigits !== 2 && additionalDigits !== 1 && additionalDigits !== 0) {
    throw new RangeError('additionalDigits must be 0, 1 or 2')
  }

  // Clone the date
  if (
    argument instanceof Date ||
    (typeof argument === 'object' && Object.prototype.toString.call(argument) === '[object Date]')
  ) {
    // Prevent the date to lose the milliseconds when passed to new Date() in IE10
    return new Date(argument.getTime())
  } else if (
    typeof argument === 'number' ||
    Object.prototype.toString.call(argument) === '[object Number]'
  ) {
    return new Date(argument)
  } else if (
    !(
      typeof argument === 'string' || Object.prototype.toString.call(argument) === '[object String]'
    )
  ) {
    return new Date(NaN)
  }

  var dateStrings = splitDateString(argument);

  var parseYearResult = parseYear(dateStrings.date, additionalDigits);
  var year = parseYearResult.year;
  var restDateString = parseYearResult.restDateString;

  var date = parseDate(restDateString, year);

  if (isNaN(date)) {
    return new Date(NaN)
  }

  if (date) {
    var timestamp = date.getTime();
    var time = 0;
    var offset;

    if (dateStrings.time) {
      time = parseTime(dateStrings.time);

      if (isNaN(time)) {
        return new Date(NaN)
      }
    }

    if (dateStrings.timeZone || options.timeZone) {
      offset = tzParseTimezone(dateStrings.timeZone || options.timeZone, new Date(timestamp + time));
      if (isNaN(offset)) {
        return new Date(NaN)
      }
    } else {
      // get offset accurate to hour in time zones that change offset
      offset = getTimezoneOffsetInMilliseconds(new Date(timestamp + time));
      offset = getTimezoneOffsetInMilliseconds(new Date(timestamp + time + offset));
    }

    return new Date(timestamp + time + offset)
  } else {
    return new Date(NaN)
  }
}

function splitDateString(dateString) {
  var dateStrings = {};
  var parts = patterns.dateTimePattern.exec(dateString);
  var timeString;

  if (!parts) {
    parts = patterns.datePattern.exec(dateString);
    if (parts) {
      dateStrings.date = parts[1];
      timeString = parts[2];
    } else {
      dateStrings.date = null;
      timeString = dateString;
    }
  } else {
    dateStrings.date = parts[1];
    timeString = parts[3];
  }

  if (timeString) {
    var token = patterns.timeZone.exec(timeString);
    if (token) {
      dateStrings.time = timeString.replace(token[1], '');
      dateStrings.timeZone = token[1].trim();
    } else {
      dateStrings.time = timeString;
    }
  }

  return dateStrings
}

function parseYear(dateString, additionalDigits) {
  var patternYYY = patterns.YYY[additionalDigits];
  var patternYYYYY = patterns.YYYYY[additionalDigits];

  var token;

  // YYYY or ±YYYYY
  token = patterns.YYYY.exec(dateString) || patternYYYYY.exec(dateString);
  if (token) {
    var yearString = token[1];
    return {
      year: parseInt(yearString, 10),
      restDateString: dateString.slice(yearString.length),
    }
  }

  // YY or ±YYY
  token = patterns.YY.exec(dateString) || patternYYY.exec(dateString);
  if (token) {
    var centuryString = token[1];
    return {
      year: parseInt(centuryString, 10) * 100,
      restDateString: dateString.slice(centuryString.length),
    }
  }

  // Invalid ISO-formatted year
  return {
    year: null,
  }
}

function parseDate(dateString, year) {
  // Invalid ISO-formatted year
  if (year === null) {
    return null
  }

  var token;
  var date;
  var month;
  var week;

  // YYYY
  if (dateString.length === 0) {
    date = new Date(0);
    date.setUTCFullYear(year);
    return date
  }

  // YYYY-MM
  token = patterns.MM.exec(dateString);
  if (token) {
    date = new Date(0);
    month = parseInt(token[1], 10) - 1;

    if (!validateDate(year, month)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, month);
    return date
  }

  // YYYY-DDD or YYYYDDD
  token = patterns.DDD.exec(dateString);
  if (token) {
    date = new Date(0);
    var dayOfYear = parseInt(token[1], 10);

    if (!validateDayOfYearDate(year, dayOfYear)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, 0, dayOfYear);
    return date
  }

  // yyyy-MM-dd or YYYYMMDD
  token = patterns.MMDD.exec(dateString);
  if (token) {
    date = new Date(0);
    month = parseInt(token[1], 10) - 1;
    var day = parseInt(token[2], 10);

    if (!validateDate(year, month, day)) {
      return new Date(NaN)
    }

    date.setUTCFullYear(year, month, day);
    return date
  }

  // YYYY-Www or YYYYWww
  token = patterns.Www.exec(dateString);
  if (token) {
    week = parseInt(token[1], 10) - 1;

    if (!validateWeekDate(year, week)) {
      return new Date(NaN)
    }

    return dayOfISOWeekYear(year, week)
  }

  // YYYY-Www-D or YYYYWwwD
  token = patterns.WwwD.exec(dateString);
  if (token) {
    week = parseInt(token[1], 10) - 1;
    var dayOfWeek = parseInt(token[2], 10) - 1;

    if (!validateWeekDate(year, week, dayOfWeek)) {
      return new Date(NaN)
    }

    return dayOfISOWeekYear(year, week, dayOfWeek)
  }

  // Invalid ISO-formatted date
  return null
}

function parseTime(timeString) {
  var token;
  var hours;
  var minutes;

  // hh
  token = patterns.HH.exec(timeString);
  if (token) {
    hours = parseFloat(token[1].replace(',', '.'));

    if (!validateTime(hours)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR
  }

  // hh:mm or hhmm
  token = patterns.HHMM.exec(timeString);
  if (token) {
    hours = parseInt(token[1], 10);
    minutes = parseFloat(token[2].replace(',', '.'));

    if (!validateTime(hours, minutes)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR + minutes * MILLISECONDS_IN_MINUTE
  }

  // hh:mm:ss or hhmmss
  token = patterns.HHMMSS.exec(timeString);
  if (token) {
    hours = parseInt(token[1], 10);
    minutes = parseInt(token[2], 10);
    var seconds = parseFloat(token[3].replace(',', '.'));

    if (!validateTime(hours, minutes, seconds)) {
      return NaN
    }

    return (hours % 24) * MILLISECONDS_IN_HOUR + minutes * MILLISECONDS_IN_MINUTE + seconds * 1000
  }

  // Invalid ISO-formatted time
  return null
}

function dayOfISOWeekYear(isoWeekYear, week, day) {
  week = week || 0;
  day = day || 0;
  var date = new Date(0);
  date.setUTCFullYear(isoWeekYear, 0, 4);
  var fourthOfJanuaryDay = date.getUTCDay() || 7;
  var diff = week * 7 + day + 1 - fourthOfJanuaryDay;
  date.setUTCDate(date.getUTCDate() + diff);
  return date
}

// Validation functions

var DAYS_IN_MONTH = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
var DAYS_IN_MONTH_LEAP_YEAR = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

function isLeapYearIndex(year) {
  return year % 400 === 0 || (year % 4 === 0 && year % 100 !== 0)
}

function validateDate(year, month, date) {
  if (month < 0 || month > 11) {
    return false
  }

  if (date != null) {
    if (date < 1) {
      return false
    }

    var isLeapYear = isLeapYearIndex(year);
    if (isLeapYear && date > DAYS_IN_MONTH_LEAP_YEAR[month]) {
      return false
    }
    if (!isLeapYear && date > DAYS_IN_MONTH[month]) {
      return false
    }
  }

  return true
}

function validateDayOfYearDate(year, dayOfYear) {
  if (dayOfYear < 1) {
    return false
  }

  var isLeapYear = isLeapYearIndex(year);
  if (isLeapYear && dayOfYear > 366) {
    return false
  }
  if (!isLeapYear && dayOfYear > 365) {
    return false
  }

  return true
}

function validateWeekDate(year, week, day) {
  if (week < 0 || week > 52) {
    return false
  }

  if (day != null && (day < 0 || day > 6)) {
    return false
  }

  return true
}

function validateTime(hours, minutes, seconds) {
  if (hours != null && (hours < 0 || hours >= 25)) {
    return false
  }

  if (minutes != null && (minutes < 0 || minutes >= 60)) {
    return false
  }

  if (seconds != null && (seconds < 0 || seconds >= 60)) {
    return false
  }

  return true
}

var assign_1 = _commonjsHelpers.createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = assign;

function assign(target, dirtyObject) {
  if (target == null) {
    throw new TypeError('assign requires that input parameter not be null or undefined');
  }

  dirtyObject = dirtyObject || {};

  for (var property in dirtyObject) {
    if (Object.prototype.hasOwnProperty.call(dirtyObject, property)) {
      target[property] = dirtyObject[property];
    }
  }

  return target;
}

module.exports = exports.default;
});

var cloneObject_1 = _commonjsHelpers.createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cloneObject;

var _index = _interopRequireDefault(assign_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function cloneObject(dirtyObject) {
  return (0, _index.default)({}, dirtyObject);
}

module.exports = exports.default;
});

const cloneObject = /*@__PURE__*/_commonjsHelpers.getDefaultExportFromCjs(cloneObject_1);

/**
 * @name utcToZonedTime
 * @category Time Zone Helpers
 * @summary Get a date/time representing local time in a given time zone from the UTC date
 *
 * @description
 * Returns a date instance with values representing the local time in the time zone
 * specified of the UTC time from the date provided. In other words, when the new date
 * is formatted it will show the equivalent hours in the target time zone regardless
 * of the current system time zone.
 *
 * @param {Date|String|Number} date - the date with the relevant UTC time
 * @param {String} timeZone - the time zone to get local time for, can be an offset or IANA time zone
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - passed to `toDate`. See [toDate]{@link https://date-fns.org/docs/toDate}
 * @returns {Date} the new date with the equivalent time in the time zone
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // In June 10am UTC is 6am in New York (-04:00)
 * const result = utcToZonedTime('2014-06-25T10:00:00.000Z', 'America/New_York')
 * //=> Jun 25 2014 06:00:00
 */
function utcToZonedTime(dirtyDate, timeZone, options) {
  var date = toDate(dirtyDate, options);

  var offsetMilliseconds = tzParseTimezone(timeZone, date, true);

  var d = new Date(date.getTime() - offsetMilliseconds);

  return new Date(
    d.getUTCFullYear(),
    d.getUTCMonth(),
    d.getUTCDate(),
    d.getUTCHours(),
    d.getUTCMinutes(),
    d.getUTCSeconds(),
    d.getUTCMilliseconds()
  )
}

/**
 * @name zonedTimeToUtc
 * @category Time Zone Helpers
 * @summary Get the UTC date/time from a date representing local time in a given time zone
 *
 * @description
 * Returns a date instance with the UTC time of the provided date of which the values
 * represented the local time in the time zone specified. In other words, if the input
 * date represented local time in time time zone, the timestamp of the output date will
 * give the equivalent UTC of that local time regardless of the current system time zone.
 *
 * @param {Date|String|Number} date - the date with values representing the local time
 * @param {String} timeZone - the time zone of this local time, can be an offset or IANA time zone
 * @param {OptionsWithTZ} [options] - the object with options. See [Options]{@link https://date-fns.org/docs/Options}
 * @param {0|1|2} [options.additionalDigits=2] - passed to `toDate`. See [toDate]{@link https://date-fns.org/docs/toDate}
 * @returns {Date} the new date with the equivalent time in the time zone
 * @throws {TypeError} 2 arguments required
 * @throws {RangeError} `options.additionalDigits` must be 0, 1 or 2
 *
 * @example
 * // In June 10am in Los Angeles is 5pm UTC
 * const result = zonedTimeToUtc(new Date(2014, 5, 25, 10, 0, 0), 'America/Los_Angeles')
 * //=> 2014-06-25T17:00:00.000Z
 */
function zonedTimeToUtc(date, timeZone, options) {
  if (typeof date === 'string' && !date.match(tzPattern)) {
    var extendedOptions = cloneObject(options);
    extendedOptions.timeZone = timeZone;
    return toDate(date, extendedOptions)
  }

  var d = toDate(date, options);

  var utc = newDateUTC(
    d.getFullYear(),
    d.getMonth(),
    d.getDate(),
    d.getHours(),
    d.getMinutes(),
    d.getSeconds(),
    d.getMilliseconds()
  ).getTime();

  var offsetMilliseconds = tzParseTimezone(timeZone, new Date(utc));

  return new Date(utc + offsetMilliseconds)
}

/**
 * is used in the formattingServiceFactory if no valid is provided
 */
const defaultFactoryConfig = {
  dateFormat: utils.DEFAULT_LOCALE.dateFormat,
  timeFormat: utils.DEFAULT_LOCALE.timeFormat,
};
/**
 * Uses the defaultFactoryConfig if nothing was provided
 * @param config
 * @returns
 */
function formattingServiceFactory(config = {}) {
  const { dateFormat, timeFormat, timeZone } = Object.assign(Object.assign({}, defaultFactoryConfig), config);
  return new FormattingService(dateFormat, timeFormat, timeZone);
}
/**
 * Used to format date values
 */
class FormattingService {
  constructor(dateFormat, timeFormat, timeZone) {
    this.dateFormat = dateFormat;
    this.timeFormat = timeFormat;
    this.timeZone = timeZone;
  }
  /**
   * Returns date in any format with utc timezone set by property time-zone or with local browser timezone
   * @param date
   * @returns
   */
  formatDate(date) {
    if (!date) {
      return '';
    }
    if (typeof date === 'number') {
      return this.formatNumber(date, this.dateFormat);
    }
    if (typeof date === 'string') {
      return utils.format(this.formatDateToDate(date), this.dateFormat);
    }
    return utils.format(date, this.dateFormat);
  }
  formatDateToDate(date) {
    return parse(date, this.dateFormat, new Date());
  }
  /**
   * Returns time in any format with utc timezone set by property time-zone or with local browser timezone
   * @param time
   * @returns
   */
  formatTime(time) {
    if (!time) {
      return '';
    }
    if (typeof time === 'number') {
      return this.formatNumber(time, this.timeFormat);
    }
    return utils.format(time, this.timeFormat);
  }
  /**
   * Formats a date/time range for display
   * @param {string} [dateStart] - The start date
   * @param {string} [dateEnd] - The end date
   * @param {string} [timeStart] - The start time
   * @param {string} [timeEnd] - The end time
   * @returns {string} The formatted date/time range string
   */
  formatRange(dateStart, dateEnd, timeStart, timeEnd) {
    let value = '';
    if (dateStart) {
      value += dateStart;
    }
    if (dateStart && timeStart) {
      value += ' ';
    }
    if (timeStart) {
      value += timeStart;
    }
    if ((dateStart && dateEnd && dateEnd !== dateStart) || (timeStart && timeEnd && timeEnd !== timeStart)) {
      value += ' – ';
    }
    if (dateEnd && dateEnd !== dateStart) {
      value += dateEnd;
    }
    if (dateEnd && timeEnd && dateEnd !== dateStart) {
      value += ' ';
    }
    if (timeEnd && (timeStart !== timeEnd || dateStart !== dateEnd)) {
      value += timeEnd;
    }
    return value;
  }
  formatDateWithOptionalTime(date, time) {
    const dateString = time ? date + ' ' + time : date;
    const format = time ? this.dateFormat + ' ' + this.timeFormat : this.dateFormat;
    return parse(dateString, format, new Date());
  }
  timestampToZonedTime(timestamp) {
    if (this.timeZone) {
      return utcToZonedTime(fromUnixTime(timestamp), this.timeZone);
    }
    return fromUnixTime(timestamp);
  }
  formatNumber(timestamp, formatting) {
    return utils.format(this.timestampToZonedTime(timestamp / 1000), formatting);
  }
}

/**
 * This config is used when no config was provided to the dateServiceFactory function
 */
const defaultDateConfig = {
  formatService: formattingServiceFactory(),
  rangeDateTime: false,
  hideSeconds: false,
  minDate: 0,
  maxDate: 0,
};
const FORMAT_BLOCKS = {
  dd: /(0[1-9]|[12][0-9]|3[01])/,
  MM: /(0[1-9]|1[0-2])/,
  yy: /([0-9]{2})/,
  d: /([1-9]|[12][0-9]|3[01])/,
  M: /([1-9]|1[0-2])/,
  y: /([12][0-9]{3})/,
  h: /(0?[0-9]|1[0-2])/,
  H: /(0?[0-9]|[1][0-9]|2[0-3])/,
  mm: /([0-5][0-9])/,
  ss: /([0-5][0-9])/,
  ' a': /( [AaPp][Mm])/,
};
/** All the date formats that can be parsed from a string */
const SUPPORTED_DATE_FORMATS_DEFAULT = [
  'dd.MM.y',
  'MM/dd/y',
  'dd/MM/y',
  'y-MM-dd',
  'd.M.y',
  'M/d/y',
  'd/M/y',
  'y-M-d',
  'dd.MM.yy',
  'MM/dd/yy',
  'dd/MM/yy',
  'yy-MM-dd',
  'd.M.yy',
  'M/d/yy',
  'd/M/yy',
  'yy-M-d',
];
/** All the date formats that can be parsed from a string. Prioritising "/"-Formats, where Days are before Months */
const SUPPORTED_DATE_FORMATS_START_WITH_DAY = [
  'dd.MM.y',
  'dd/MM/y',
  'dd.MM.yy',
  'dd/MM/yy',
  'MM/dd/y',
  'MM/dd/yy',
  'y-MM-dd',
  'd.M.y',
  'd/M/y',
  'M/d/y',
  'y-M-d',
  'yy-MM-dd',
  'd.M.yy',
  'd/M/yy',
  'M/d/yy',
  'yy-M-d',
];
/** All the time formats that can be parsed from a string */
const SUPPORTED_TIME_FORMATS = ['h:mm:ss a', 'H:mm:ss', 'h:mm a', 'H:mm', 'h a'];
/**
 * Uses the defaultDateConfig if no value is provided
 * @param config
 * @returns
 */
function dateServiceFactory(config = {}) {
  const { hideSeconds, rangeDateTime, minDate, maxDate, formatService } = Object.assign(Object.assign({}, defaultDateConfig), config);
  return new DateService(hideSeconds, rangeDateTime, minDate, maxDate, formatService);
}
/**
 * Used to manipulate date values
 */
class DateService {
  constructor(hideSeconds, rangeDateTime, minDate, maxDate, formatService) {
    this.hideSeconds = hideSeconds;
    this.rangeDateTime = rangeDateTime;
    this.minDate = minDate;
    this.maxDate = maxDate;
    this.formatService = formatService;
  }
  getTodayValue(timeZone = undefined) {
    return timeZone === undefined ? new Date() : new Date(new Date().toLocaleString('en-US', { timeZone: timeZone }));
  }
  getDataFromInput(input) {
    const dates = this.getDateFromDateInput(input);
    const times = this.getTimeFromDateInput(input);
    const correctedData = this.correctInputData(Object.assign(Object.assign({}, dates), times));
    return Object.assign(Object.assign({}, correctedData), { valid: true });
  }
  /**
   * check whether the input contains valid date and time values and that they are also in the correct time order.
   * @param input
   * @returns
   */
  validateInput(input) {
    const dateStart = utils.startOfDay(this.formatService.formatDateToDate(input.dateStart || ''));
    const dateEnd = utils.startOfDay(this.formatService.formatDateToDate(input.dateEnd || ''));
    if (isAfter(dateStart, endOfDay(this.maxDate)) ||
      isBefore(dateStart, utils.startOfDay(this.minDate)) ||
      isAfter(dateEnd, endOfDay(this.maxDate)) ||
      isBefore(dateEnd, utils.startOfDay(this.minDate)) ||
      isAfter(dateStart, dateEnd)) {
      return { valid: false, timeEnd: null, dateEnd: null };
    }
    else {
      return { valid: true };
    }
  }
  /**
   * Corrects date values, puts them within allowed ranges and swaps values if necessary.
   * @param dateObj dateStart, dateEnd, timeStart, timeEnd
   * @returns corrected dateStart, dateEnd, timeStart, timeEnd
   */
  correctInputData(dateObj) {
    const { dateStart, dateEnd, timeStart, timeEnd } = dateObj;
    let parsedDateStart, parsedDateEnd;
    // Case with only one date and two times
    if (!!dateStart && !!timeStart && !!timeEnd && !dateEnd) {
      parsedDateStart = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeStart));
      parsedDateEnd = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeEnd));
      if (isBefore(parsedDateEnd, parsedDateStart)) {
        return Object.assign({}, this.createInputData(parsedDateEnd, parsedDateStart));
      }
    }
    // Case when nothing valid was entered into the input
    else if (!dateStart || !dateEnd) {
      return dateObj;
    }
    // Base case
    else {
      parsedDateStart = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateStart, timeStart));
      parsedDateEnd = this.dateWithinAllowedRange(this.formatService.formatDateWithOptionalTime(dateEnd, timeEnd));
      if (isBefore(parsedDateEnd, parsedDateStart)) {
        return Object.assign({}, this.createInputData(parsedDateEnd, parsedDateStart));
      }
    }
    return Object.assign({}, this.createInputData(parsedDateStart, parsedDateEnd));
  }
  dateWithinAllowedRange(date) {
    return clamp(date, {
      start: this.formatService.timestampToZonedTime(this.minDate / 1000),
      end: this.maxDate === 0 ? Infinity : this.formatService.timestampToZonedTime(this.maxDate / 1000),
    });
  }
  /**
   * Creates valid InputData object from two dates.
   * @param dateStart Date to infer dateStart and timeStart from
   * @param dateEnd Date to infer dateEnd and timeEnd from
   */
  createInputData(dateStart, dateEnd) {
    return {
      dateStart: this.formatService.formatDate(dateStart),
      dateEnd: this.formatService.formatDate(dateEnd) || null,
      timeStart: this.formatService.formatTime(dateStart),
      timeEnd: this.formatService.formatTime(dateEnd) || null,
    };
  }
  /**
   * Returns utc timestamp created by timezone from property time-zone or with local browser timezone
   * @param date string parsed with formatDate
   * @param time string in format 00:00
   */
  toSelectionTimestamp(date, time) {
    if (!date || !this.formatService.timeZone) {
      return undefined;
    }
    const dateTime = this.formatService.formatDateWithOptionalTime(date, time);
    return getUnixTime(zonedTimeToUtc(dateTime, this.formatService.timeZone)) * 1000;
  }
  getDateFromDateInput(input) {
    const matches = this.getPartFromDateInput(input, 'dateStart', 'dateEnd', this.DATE_FORMATS);
    const result = { dateStart: null, dateEnd: null };
    result.dateStart = matches.dateStart ? this.formatService.formatDate(matches.dateStart) : null;
    result.dateEnd = matches.dateEnd ? this.formatService.formatDate(matches.dateEnd) : null;
    return result;
  }
  getTimeFromDateInput(input) {
    const matches = this.getPartFromDateInput(input, 'timeStart', 'timeEnd', this.TIME_FORMATS);
    const result = { timeStart: null, timeEnd: null };
    result.timeStart = matches.timeStart ? this.formatService.formatTime(matches.timeStart) : null;
    result.timeEnd = matches.timeEnd ? this.formatService.formatTime(matches.timeEnd) : null;
    return result;
  }
  getPartFromDateInput(input, startKey, endKey, tests) {
    var _a, _b;
    let match;
    let countMatches = 0;
    const dateObj = {
      [startKey]: null,
      [endKey]: null,
    };
    const result = {
      [startKey]: null,
      [endKey]: null,
    };
    Object.keys(tests).forEach((key) => {
      var _a;
      const pattern = tests[key];
      // Match time input
      while ((match = pattern.exec(input)) !== null) {
        // This is necessary to avoid infinite loops with zero-width matches
        if (match.index === pattern.lastIndex) {
          pattern.lastIndex++;
        }
        if (match[0] !== null && match.index !== ((_a = dateObj[startKey]) === null || _a === void 0 ? void 0 : _a.atIndex)) {
          countMatches++;
          if (countMatches >= 3)
            break;
          if (this.rangeDateTime && countMatches === 2) {
            dateObj[endKey] = { match: match[0], fromFormat: key, atIndex: match.index };
          }
          else {
            // only match first date
            if (countMatches >= 2)
              break;
            dateObj[startKey] = { match: match[0], fromFormat: key, atIndex: match.index };
          }
        }
      }
    });
    // parse to a normal date as the matches can be from different formats
    if (!!dateObj[endKey]) {
      const { match, fromFormat } = dateObj[endKey];
      const normalizedDate = parse(match, fromFormat, new Date());
      result[endKey] = normalizedDate;
    }
    if (!!dateObj[startKey]) {
      const { match, fromFormat } = dateObj[startKey];
      const normalizedDate = parse(match, fromFormat, new Date());
      result[startKey] = normalizedDate;
    }
    // Swap dates if match found later is before other match in the input string.
    // This happens if match before is from a format that gets searched later.
    if (((_a = dateObj[startKey]) === null || _a === void 0 ? void 0 : _a.atIndex) > ((_b = dateObj[endKey]) === null || _b === void 0 ? void 0 : _b.atIndex)) {
      const temp = result[startKey];
      result[startKey] = result[endKey];
      result[endKey] = temp;
    }
    return result;
  }
  get DATE_FORMATS() {
    var _a, _b;
    const formats = {};
    const DATE_FORMATS = ((_a = this.formatService.dateFormat) === null || _a === void 0 ? void 0 : _a.toLowerCase().startsWith('dd/')) || ((_b = this.formatService.dateFormat) === null || _b === void 0 ? void 0 : _b.toLowerCase().startsWith('d/'))
      ? SUPPORTED_DATE_FORMATS_START_WITH_DAY
      : SUPPORTED_DATE_FORMATS_DEFAULT;
    DATE_FORMATS.forEach((format) => {
      let regex = format;
      ['.', '/', '-'].forEach((symbol) => {
        regex = regex.replace(new RegExp(`[${symbol}]`, 'g'), `[${symbol}]`);
      });
      Object.keys(FORMAT_BLOCKS).forEach((block) => {
        regex = regex.replace(new RegExp(block, 'g'), FORMAT_BLOCKS[block].source);
      });
      formats[format] = new RegExp(regex, 'g');
    });
    return formats;
  }
  get TIME_FORMATS() {
    const formats = {};
    SUPPORTED_TIME_FORMATS.map((format) => {
      let regex = format;
      [':'].forEach((symbol) => {
        regex = regex.replace(new RegExp(`[${symbol}]`, 'g'), `[${symbol}]`);
      });
      Object.keys(FORMAT_BLOCKS).forEach((block) => {
        regex = regex.replace(new RegExp(block, 'g'), FORMAT_BLOCKS[block].source);
      });
      formats[format] = new RegExp(regex, 'g');
    });
    return formats;
  }
}

const dialogCss = "/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:\"Bosch-Sans\";font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.clickable-element{transition:0.4s;cursor:pointer}.clickable-element:active,.clickable-element.active{color:#007bc0}.clickable-element:active:hover,.clickable-element:hover .clickable-element:focus{color:#9dc9ff}.clickable-element:disabled,.clickable-element.disabled{color:#a4abb3}.clickable-element:disabled:active,.clickable-element:disabled.active,.clickable-element:disabled:active:hover,.clickable-element:disabled:hover,.clickable-element.disabled:active,.clickable-element.disabled.active,.clickable-element.disabled:active:hover,.clickable-element.disabled:hover{color:#a4abb3}button.button-primary,button.button-default,input.button-primary,input.button-default{color:#ffffff;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;background-color:#007bc0;background-size:100% 200%;border-width:0;border-color:transparent;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-primary:hover,button.button-primary:focus-visible,button.button-default:hover,button.button-default:focus-visible,input.button-primary:hover,input.button-primary:focus-visible,input.button-default:hover,input.button-default:focus-visible{background-color:#00629a;color:#ffffff;outline:0}button.button-primary:active,button.button-default:active,input.button-primary:active,input.button-default:active{background-color:#004975;color:#ffffff;outline:0}button.button-primary:disabled,button.button-primary.disabled,button.button-default:disabled,button.button-default.disabled,input.button-primary:disabled,input.button-primary.disabled,input.button-default:disabled,input.button-default.disabled{background-color:#c1c7cc;color:#656a6f;background-image:none;cursor:default}button.button-primary.button-lg,button.button-default.button-lg,input.button-primary.button-lg,input.button-default.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-sm,button.button-default.button-sm,input.button-primary.button-sm,input.button-default.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-xs,button.button-default.button-xs,input.button-primary.button-xs,input.button-default.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary:not(.mat-icon-button):has(mat-icon),button.button-default:not(.mat-icon-button):has(mat-icon),input.button-primary:not(.mat-icon-button):has(mat-icon),input.button-default:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-primary.mat-icon-button,button.button-default.mat-icon-button,input.button-primary.mat-icon-button,input.button-default.mat-icon-button{padding:12px}button.button-secondary,input.button-secondary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 15px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-width:1px;border-color:#007bc0;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:hover,button.button-secondary:focus-visible,input.button-secondary:hover,input.button-secondary:focus-visible{border-color:#004975;outline:0;color:#004975;background-color:#d1e4ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:active,input.button-secondary:active{border-color:#004975;color:#004975;outline:0;background-color:#9dc9ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:disabled,button.button-secondary.disabled,input.button-secondary:disabled,input.button-secondary.disabled{color:#979ea4;border-color:#979ea4;background-color:transparent;cursor:default}button.button-secondary.button-lg,input.button-secondary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-sm,input.button-secondary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-xs,input.button-secondary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary:not(.mat-icon-button):has(mat-icon),input.button-secondary:not(.mat-icon-button):has(mat-icon){padding:0 15px 0 13px}button.button-secondary.mat-icon-button,input.button-secondary.mat-icon-button{padding:11px}button.button-tertiary,input.button-tertiary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-tertiary:hover,button.button-tertiary:focus-visible,input.button-tertiary:hover,input.button-tertiary:focus-visible{outline:0;color:#00629a;background-color:#d1e4ff}button.button-tertiary:active,input.button-tertiary:active{color:#004975;background-color:#9dc9ff;outline:0}button.button-tertiary:disabled,button.button-tertiary.disabled,input.button-tertiary:disabled,input.button-tertiary.disabled{color:#979ea4;background-color:transparent;cursor:default}button.button-tertiary.button-lg,input.button-tertiary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-sm,input.button-tertiary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-xs,input.button-tertiary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary:not(.mat-icon-button):has(mat-icon),input.button-tertiary:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-tertiary.mat-icon-button,input.button-tertiary.mat-icon-button{padding:12px}button.button-integrated,input.button-integrated{color:#000000;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-integrated:hover,button.button-integrated:focus-visible,input.button-integrated:hover,input.button-integrated:focus-visible{color:#007bc0;outline:0}button.button-integrated:active,input.button-integrated:active{color:#00629a;outline:0}button.button-integrated:disabled,button.button-integrated.disabled,input.button-integrated:disabled,input.button-integrated.disabled{color:#979ea4 !important;background-color:transparent;cursor:default}button.button-integrated:not(.mat-icon-button):has(mat-icon),input.button-integrated:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-integrated.mat-icon-button,input.button-integrated.mat-icon-button{padding:12px}button.button-link,input.button-link{color:#007bc0;background-color:transparent;border:none;-webkit-box-shadow:none;box-shadow:none;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0 16px 0 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link::after,input.button-link::after{position:absolute;font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:18px;content:\"\\e181\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}button.button-link:hover,button.button-link:focus-visible,input.button-link:hover,input.button-link:focus-visible{color:#00629a;text-decoration:underline;outline:0}button.button-link:active,input.button-link:active{color:#004975;text-decoration:underline}button.button-link.disabled,button.button-link[disabled],button.button-link fieldset[disabled],input.button-link.disabled,input.button-link[disabled],input.button-link fieldset[disabled]{color:#c1c7cc;text-decoration:none;-webkit-transition:none;-o-transition:none;transition:none}button.button-link.disabled::after,button.button-link[disabled]::after,button.button-link fieldset[disabled]::after,input.button-link.disabled::after,input.button-link[disabled]::after,input.button-link fieldset[disabled]::after{-webkit-transform:translate(0, 0);-ms-transform:translate(0, 0);-o-transform:translate(0, 0);transform:translate(0, 0)}button.button-link.button-lg,input.button-link.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-lg::after,input.button-link.button-lg::after{font-size:22px}button.button-link.button-sm,input.button-link.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:14px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-sm::after,input.button-link.button-sm::after{font-size:14px}button.button-link.no-arrow::after,input.button-link.no-arrow::after{content:\"\"}button.button-link:not(.mat-icon-button):has(mat-icon),input.button-link:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-link.mat-icon-button,input.button-link.mat-icon-button{padding:12px}button.button-block,input[type=submit].button-block,input[type=reset].button-block,input[type=button].button-block{display:block;width:100%}/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bci-core-date-time-picker-container{background:#ffffff;display:flex;flex-flow:column nowrap;outline:none;box-shadow:0px 0px 8px rgba(0, 0, 0, 0.25)}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container{display:flex}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.hidden{display:none}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.year-month{height:292px;min-width:292px}.bci-core-date-time-picker-container .bci-core-date-time-picker-sub-container.year-month.has-time{width:584px}.bci-core-date-time-picker-container .bci-core-save{color:#005587;text-align:end;border-top:1px solid #8a9097;font-size:16px;padding:5px 10px 5px 0px;height:29px}.bci-core-date-time-picker-container .bci-core-save .bosch-ic-clickable{position:relative;top:4px}.bci-core-date-time-picker-container .bci-core-datepicker-selection{text-align:right;border-top:1px solid #8a9097;display:flex;justify-content:space-between;align-items:center;padding:0 10px;height:68px}.bci-core-date-time-picker-container .bci-core-datepicker-selection .close{margin-left:auto}@media (max-width: 767px){.bci-core-date-time-picker-sub-container{flex-flow:column nowrap;width:292px !important}}@media (min-width: 768px){.bci-core-date-time-picker-sub-container{flex-flow:row nowrap}.bci-core-date-time-picker-sub-container>bci-datepicker{order:1;flex:1}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container{order:2;flex:1}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container:not(.no-border){border-left:1px solid #8a9097}.bci-core-date-time-picker-sub-container .bci-core-time-picker-container bci-timepicker .bci-core-timepicker-container{padding-right:8px}.bci-core-date-time-picker-sub-container>.bci-core-save{order:3;flex:2}}";

const DateTimeDialog = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.selectionChanged = index.createEvent(this, "selectionChanged", 7);
    this.showTemplates = false;
    /**
     * Case null: Year/Month Selection closed.
     * Case 'year': Year Selection active.
     * Case 'month': Month Selection active.
     */
    this.yearMonthSelectionStep = null;
    /** Tracks how often the user switched pages in the year selection */
    this.yearSelectionPageOffset = 0;
    /** Object to keep track of the currently selected date range */
    this.datesObj = {
      startDate: null,
      endDate: null,
    };
    /**
     * Optional - inject the FormattingService instance
     */
    this.formatService = formattingServiceFactory();
    /**
     * Optional - inject the DateService instance
     */
    this.dateService = dateServiceFactory({ formatService: this.formatService });
  }
  emitSelectionChanged(event) {
    this.selectionChanged.emit(event);
  }
  onTemplateUpdate({ detail: { firstEmit, timeStamp, id } }) {
    const event = {
      reason: 'frame',
      timestampStart: timeStamp.timestampStart,
      timestampEnd: timeStamp.timestampEnd,
      firstEmit,
      id,
    };
    this.selectionChanged.emit(event);
  }
  timeSelectedHandler({ detail: data }) {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    let { timeStart, timeEnd } = this.datePickerInputObj;
    if (data.timepickerId === 'timeStart_' + this.dateTimePickerId) {
      timeStart = data.updatedTime;
    }
    if (data.timepickerId === 'timeEnd_' + this.dateTimePickerId) {
      timeEnd = data.updatedTime;
    }
    const event = {
      reason: 'time',
      timeStart,
      timeEnd,
    };
    this.selectionChanged.emit(event);
  }
  showTimeRange() {
    this.showTemplates = false;
  }
  /**
   * Clears internal timer and resets selected template
   */
  async close() {
    var _a;
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
  }
  /**
   * Closes the Year/Month Selection
   */
  async closeYearMonthSelection() {
    this.yearMonthSelectionStep = null;
  }
  showTemplate() {
    this.showTemplates = true;
  }
  toggleShowYearMonthSelectionHandler(event) {
    // null means the yearMonthSelection is closed
    this.yearMonthSelectionStep = event.detail.newState ? 'year' : null;
    this.yearSelectionPageOffset = 0;
  }
  arrowClickedHandler(event) {
    if (!!this.yearMonthSelectionStep && this.yearMonthSelectionStep === 'year') {
      switch (event.detail.direction) {
        case 'next':
          this.yearSelectionPageOffset++;
          break;
        case 'prev':
          this.yearSelectionPageOffset--;
          break;
      }
      return;
    }
    this.handleMonthChangeOnArrowClick(event.detail);
  }
  handleMonthChangeOnArrowClick(detail) {
    let newYear = this.dateObj.calendarDateYear;
    let newMonth;
    switch (detail.direction) {
      case 'next':
        newMonth = (this.dateObj.calendarDateMonth + 1) % 12;
        if (newMonth === 0) {
          newYear = this.dateObj.calendarDateYear + 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
      case 'prev':
        newMonth = this.dateObj.calendarDateMonth - 1;
        if (newMonth === -1) {
          newMonth = 11;
          newYear = this.dateObj.calendarDateYear - 1;
        }
        this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth, calendarDateYear: newYear });
        break;
    }
  }
  yearMonthSelectedHandler(event) {
    if (!this.yearMonthSelectionStep)
      return;
    if (this.yearMonthSelectionStep === 'year') {
      const newYear = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: newYear });
      this.yearMonthSelectionStep = 'month';
    }
    else if (this.yearMonthSelectionStep === 'month') {
      const newMonth = event.detail.selectedYearMonth;
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: newMonth });
      this.yearMonthSelectionStep = null;
    }
  }
  singleDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = utils.createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDate: utils.format(newDate, this.datePickerConfig.formatDate), selectedDate: utils.format(newDate, this.datePickerConfig.formatDate) });
    const dateSelectedEvent = {
      reason: 'date',
      startDate: this.dateObj.selectedDate,
      endDate: null,
      rangeCompleted: false,
    };
    this.emitSelectionChanged(dateSelectedEvent);
  }
  rangeDateSelectedHandler(event) {
    var _a;
    // reset the templates element's state as there is no longer a template selected
    (_a = this.templatesElement) === null || _a === void 0 ? void 0 : _a.reset();
    const selectedDay = event.detail.selectedDay;
    const newDate = utils.createDate(new Date(), this.dateObj.calendarDateYear, this.dateObj.calendarDateMonth, selectedDay);
    let rangeComplete = false;
    // reset on subsequent clicks after first and second dates have been set
    if (this.datesObj.startDate && this.datesObj.endDate && this.datesObj.startDate !== this.datesObj.endDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: utils.format(newDate, this.datePickerConfig.formatDate), endDate: utils.format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (!this.datesObj.startDate) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: utils.format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = false;
    }
    else if (isBefore(newDate, parse(this.datesObj.startDate, this.datePickerConfig.formatDate, new Date()))) {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: this.datesObj.startDate, startDate: utils.format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    else {
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { endDate: utils.format(newDate, this.datePickerConfig.formatDate) });
      rangeComplete = true;
    }
    const dateSelectedEvent = Object.assign(Object.assign({}, this.datesObj), { reason: 'date', rangeCompleted: rangeComplete });
    this.emitSelectionChanged(dateSelectedEvent);
  }
  userInputDate(newValue) {
    if (!newValue)
      return;
    if (newValue.dateStart) {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateMonth: getMonth(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())), calendarDateYear: getYear(parse(newValue.dateStart, this.datePickerConfig.formatDate, new Date())) });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: newValue.dateStart, endDate: newValue.dateEnd });
    }
    else {
      this.dateObj = Object.assign(Object.assign({}, this.dateObj), { calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)), selectedDate: null });
      this.datesObj = Object.assign(Object.assign({}, this.datesObj), { startDate: null, endDate: null });
    }
  }
  async componentWillLoad() {
    this.dateObj = {
      calendarDate: utils.format(utcToZonedTime(new Date(), this.datePickerConfig.timeZone), this.datePickerConfig.formatDate),
      calendarDateYear: getYear(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      calendarDateMonth: getMonth(utcToZonedTime(new Date(), this.datePickerConfig.timeZone)),
      selectedDate: null,
    };
  }
  applyTemplate(option, firstEmit) {
    this.onTemplateUpdate({ detail: { timeStamp: option.calculateTemplate(), firstEmit, id: option.id } });
  }
  render() {
    var _a, _b, _c, _d;
    return (index.h(index.Host, { id: `overlay_${this.dateTimePickerId}` }, index.h("div", { class: "bci-core-date-time-picker-container", tabindex: "-1" }, !this.showTemplates && this.hide !== 'calendar' && (index.h("div", null, index.h("bci-month-header", { "data-test": "webcore.webcomponents.bci-datepicker.year-month", year: this.dateObj.calendarDateYear, month: this.dateObj.calendarDateMonth, yearMonthSelectionStep: this.yearMonthSelectionStep }), index.h("div", { class: `bci-core-date-time-picker-sub-container year-month ${!this.yearMonthSelectionStep ? 'hidden' : ''} ${this.hide !== 'time' ? 'has-time' : ''}` }, index.h("bci-year-month", { selectionStep: this.yearMonthSelectionStep, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, dateObj: this.dateObj, yearSelectionPageOffset: this.yearSelectionPageOffset })))), !this.showTemplates && (index.h("div", { class: `bci-core-date-time-picker-sub-container ${!!this.yearMonthSelectionStep ? 'hidden' : ''} ` }, this.hide !== 'calendar' && (index.h("bci-datepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.dateTimePickerInput", bciDatepickerId: this.dateTimePickerId, minDate: this.datePickerConfig.minDate, maxDate: this.datePickerConfig.maxDate, rangeDate: this.datePickerConfig.rangeDateTime, sundayFirst: this.datePickerConfig.sundayFirst, formatDate: this.datePickerConfig.formatDate, datePickerInputObj: this.datePickerInputObj, timeZone: this.datePickerConfig.timeZone, selectedYear: this.dateObj.calendarDateYear, selectedMonth: this.dateObj.calendarDateMonth, selectedDate: this.dateObj.selectedDate, today: this.formatService.formatDate(this.dateService.getTodayValue(this.datePickerConfig.timeZone)), datesObj: this.datesObj })), this.hide !== 'time' && (index.h("div", { class: { 'bci-core-time-picker-container': true, 'no-border': this.hide === 'calendar' } }, index.h("bci-timepicker", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.starttime", class: { 'full-height': !this.timePickerConfig.rangeDateTime }, timeInput: (_a = this.datePickerInputObj) === null || _a === void 0 ? void 0 : _a.timeStart, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeStart_' + this.dateTimePickerId, rangeTimePicker: false, timeTitle: this.timePickerConfig.rangeDateTime ? utils.translate('startTime') : utils.translate('time') }), index.h("bci-timepicker", { "data-test": "datetime-picker.core-datetime-picker.endtime", class: { hidden: !this.timePickerConfig.rangeDateTime }, timeInput: (_b = this.datePickerInputObj) === null || _b === void 0 ? void 0 : _b.timeEnd, timeFormat: this.timePickerConfig.timeFormat, bciTimepickerId: 'timeEnd_' + this.dateTimePickerId, rangeTimePicker: true, timeTitle: utils.translate('endTime') }))))), !this.showTemplates && (index.h("div", { class: "bci-core-datepicker-selection" }, ((_c = this.templatesList) === null || _c === void 0 ? void 0 : _c.length) > 1 && this.hide !== 'calendar' && (index.h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.showTemplate() }, utils.translate('templates'))), ((_d = this.templatesList) === null || _d === void 0 ? void 0 : _d.length) === 1 && this.hide !== 'calendar' && (index.h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.template", class: "button-secondary no-arrow template", disabled: this.templatesDisabled, onClick: () => this.applyTemplate(this.templatesList[0], true) }, this.templatesList[0].label)), index.h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.close", class: "button-primary no-arrow close", onClick: () => this.toggleDialogHandler() }, utils.translate('close')))), index.h("bci-template", { templatesList: this.templatesList, visible: this.showTemplates && !!this.templatesList, ref: (el) => (this.templatesElement = el) }))));
  }
  static get watchers() { return {
    "datePickerInputObj": ["userInputDate"]
  }; }
};
DateTimeDialog.style = dialogCss;

const timepickerCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:var(--bci-global-font-family);font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */body{margin:0;display:flex;flex-direction:column}main,*{font-family:var(--bci-global-font-family)}::selection,::-moz-selection{background-color:#007bc0}a::-moz-selection{color:#ffffff}.lead{margin-bottom:24px;font-size:18px;font-weight:300;line-height:1.4}@media (min-width: 768px){.lead{font-size:24px}}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bosch-ic,.Bosch-Ic{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic:before,.Bosch-Ic:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-large,.Bosch-Ic-Large{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-large:before,.Bosch-Ic-Large:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:28px;vertical-align:middle}.bosch-ic-medium,.Bosch-Ic-Medium{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-medium:before,.Bosch-Ic-Medium:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-normal,.Bosch-Ic-Normal{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-normal:before,.Bosch-Ic-Normal:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:20px;vertical-align:middle}.bosch-ic-small,.Bosch-Ic-Small{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-small:before,.Bosch-Ic-Small:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:16px;vertical-align:middle}.Bosch-Ic-Disabled,.bosch-ic-disabled,.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic[disabled],.bosch-ic[disabled],.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover{cursor:not-allowed;color:#d0d4d8}.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic-Disabled:hover:hover,.bosch-ic-disabled:hover:hover,.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover,.Bosch-Ic[disabled]:hover:hover,.bosch-ic[disabled]:hover:hover{color:#d0d4d8}.bosch-ic-clickable,.Bosch-Ic-Clickable{cursor:pointer}.bosch-ic-clickable:disabled,.bosch-ic-clickable.disabled,.Bosch-Ic-Clickable:disabled,.Bosch-Ic-Clickable.disabled{color:#a4abb3;cursor:not-allowed}.bosch-ic-activated,.Bosch-Ic-Activated{color:#007bc0}.bosch-ic-3d-print:before{content:\"\\e085\"}.bosch-ic-a-to-b:before{content:\"\\e086\"}.bosch-ic-add:before{content:\"\\e087\"}.bosch-ic-address-consumer-data:before{content:\"\\e088\"}.bosch-ic-adress-book:before{content:\"\\e089\"}.bosch-ic-agility:before{content:\"\\e08a\"}.bosch-ic-alarm-clock:before{content:\"\\e08b\"}.bosch-ic-ambulance:before{content:\"\\e08c\"}.bosch-ic-analysis:before{content:\"\\e08d\"}.bosch-ic-anchor:before{content:\"\\e08e\"}.bosch-ic-arrow-down:before{content:\"\\e08f\"}.bosch-ic-arrow-left:before{content:\"\\e090\"}.bosch-ic-arrow-left-down:before{content:\"\\e091\"}.bosch-ic-arrow-left-up:before{content:\"\\e092\"}.bosch-ic-arrow-right:before{content:\"\\e093\"}.bosch-ic-arrow-right-down:before{content:\"\\e094\"}.bosch-ic-arrow-right-up:before{content:\"\\e095\"}.bosch-ic-arrow-turn-left:before{content:\"\\e096\"}.bosch-ic-arrow-turn-right:before{content:\"\\e097\"}.bosch-ic-arrow-up:before{content:\"\\e098\"}.bosch-ic-arrows:before{content:\"\\e099\"}.bosch-ic-arrows-x-y-z:before{content:\"\\e09a\"}.bosch-ic-artificial-intelligence:before{content:\"\\e09b\"}.bosch-ic-assembly-line:before{content:\"\\e09c\"}.bosch-ic-atom:before{content:\"\\e09d\"}.bosch-ic-augmented-reality:before{content:\"\\e09e\"}.bosch-ic-autobod:before{content:\"\\e09f\"}.bosch-ic-back-left:before{content:\"\\e0a0\"}.bosch-ic-back-left-double:before{content:\"\\e0a1\"}.bosch-ic-bar-chart:before{content:\"\\e0a2\"}.bosch-ic-bar-chart-double:before{content:\"\\e0a3\"}.bosch-ic-battery-0:before{content:\"\\e0a4\"}.bosch-ic-battery-1:before{content:\"\\e0a5\"}.bosch-ic-battery-2:before{content:\"\\e0a6\"}.bosch-ic-battery-3:before{content:\"\\e0a7\"}.bosch-ic-battery-4:before{content:\"\\e0a8\"}.bosch-ic-battery-fail:before{content:\"\\e0a9\"}.bosch-ic-battery-li-ion:before{content:\"\\e0aa\"}.bosch-ic-battery-loading:before{content:\"\\e0ab\"}.bosch-ic-bed:before{content:\"\\e0ac\"}.bosch-ic-bicycle:before{content:\"\\e0ad\"}.bosch-ic-bicycle-e:before{content:\"\\e0ae\"}.bosch-ic-blog:before{content:\"\\e0af\"}.bosch-ic-bluetooth:before{content:\"\\e0b0\"}.bosch-ic-bluetooth-le:before{content:\"\\e0b1\"}.bosch-ic-board-speaker:before{content:\"\\e0b2\"}.bosch-ic-boiler-connected:before{content:\"\\e0b3\"}.bosch-ic-book:before{content:\"\\e0b4\"}.bosch-ic-bookmark:before{content:\"\\e0b5\"}.bosch-ic-bookmark-add:before{content:\"\\e0b6\"}.bosch-ic-bookmark-check:before{content:\"\\e0b7\"}.bosch-ic-bookmark-delete:before{content:\"\\e0b8\"}.bosch-ic-books:before{content:\"\\e0b9\"}.bosch-ic-bookshelf:before{content:\"\\e0ba\"}.bosch-ic-box:before{content:\"\\e0bb\"}.bosch-ic-box-closed:before{content:\"\\e0bc\"}.bosch-ic-box-delivery:before{content:\"\\e0bd\"}.bosch-ic-box-delivery-connected:before{content:\"\\e0be\"}.bosch-ic-brake-disk:before{content:\"\\e0bf\"}.bosch-ic-briefcase:before{content:\"\\e0c0\"}.bosch-ic-broom:before{content:\"\\e0c1\"}.bosch-ic-broom-cloud-dirt:before{content:\"\\e0c2\"}.bosch-ic-brush:before{content:\"\\e0c3\"}.bosch-ic-bug:before{content:\"\\e0c4\"}.bosch-ic-building:before{content:\"\\e0c5\"}.bosch-ic-cactus:before{content:\"\\e0c6\"}.bosch-ic-calculate:before{content:\"\\e0c7\"}.bosch-ic-calendar:before{content:\"\\e0c8\"}.bosch-ic-calendar-add:before{content:\"\\e0c9\"}.bosch-ic-calendar-settings:before{content:\"\\e0ca\"}.bosch-ic-call:before{content:\"\\e0cb\"}.bosch-ic-call-add:before{content:\"\\e0cc\"}.bosch-ic-call-remove:before{content:\"\\e0cd\"}.bosch-ic-call-sos:before{content:\"\\e0ce\"}.bosch-ic-call-team:before{content:\"\\e0cf\"}.bosch-ic-call-wifi:before{content:\"\\e0d0\"}.bosch-ic-camera:before{content:\"\\e0d1\"}.bosch-ic-camera-switch:before{content:\"\\e0d2\"}.bosch-ic-car:before{content:\"\\e0d3\"}.bosch-ic-car-clock:before{content:\"\\e0d4\"}.bosch-ic-car-connected:before{content:\"\\e0d5\"}.bosch-ic-car-locator:before{content:\"\\e0d6\"}.bosch-ic-car-mechanic:before{content:\"\\e0d7\"}.bosch-ic-car-rental:before{content:\"\\e0d8\"}.bosch-ic-car-seat-connected:before{content:\"\\e0d9\"}.bosch-ic-car-side:before{content:\"\\e0da\"}.bosch-ic-car-side-user:before{content:\"\\e0db\"}.bosch-ic-cart:before{content:\"\\e0dc\"}.bosch-ic-certificate:before{content:\"\\e0dd\"}.bosch-ic-certificate-pki:before{content:\"\\e0de\"}.bosch-ic-change:before{content:\"\\e0df\"}.bosch-ic-chart-bar:before{content:\"\\e0e0\"}.bosch-ic-chart-check:before{content:\"\\e0e1\"}.bosch-ic-chart-dummy:before{content:\"\\e0e2\"}.bosch-ic-chart-line:before{content:\"\\e0e3\"}.bosch-ic-chat:before{content:\"\\e0e4\"}.bosch-ic-chat-add:before{content:\"\\e0e5\"}.bosch-ic-chat-question-answer:before{content:\"\\e0e6\"}.bosch-ic-chat-remove:before{content:\"\\e0e7\"}.bosch-ic-checklist:before{content:\"\\e0e8\"}.bosch-ic-checkmark:before{content:\"\\e0e9\"}.bosch-ic-chip:before{content:\"\\e0ea\"}.bosch-ic-circle-segment:before{content:\"\\e0eb\"}.bosch-ic-clear-all:before{content:\"\\e0ec\"}.bosch-ic-clipboard:before{content:\"\\e0ed\"}.bosch-ic-clipboard-list:before{content:\"\\e0ee\"}.bosch-ic-clock:before{content:\"\\e0ef\"}.bosch-ic-close:before{content:\"\\e0f0\"}.bosch-ic-close-all:before{content:\"\\e0f1\"}.bosch-ic-cloud:before{content:\"\\e0f2\"}.bosch-ic-cloud-co2:before{content:\"\\e0f3\"}.bosch-ic-cloud-download:before{content:\"\\e0f4\"}.bosch-ic-cloud-nox:before{content:\"\\e0f5\"}.bosch-ic-cloud-upload:before{content:\"\\e0f6\"}.bosch-ic-club-arm-chair:before{content:\"\\e0f7\"}.bosch-ic-coffee-break:before{content:\"\\e0f8\"}.bosch-ic-coin-currency:before{content:\"\\e0f9\"}.bosch-ic-coin-dollar:before{content:\"\\e0fa\"}.bosch-ic-colorpicker:before{content:\"\\e0fb\"}.bosch-ic-colors:before{content:\"\\e0fc\"}.bosch-ic-command:before{content:\"\\e0fd\"}.bosch-ic-communicator:before{content:\"\\e0fe\"}.bosch-ic-compare:before{content:\"\\e0ff\"}.bosch-ic-components:before{content:\"\\e100\"}.bosch-ic-components-available:before{content:\"\\e101\"}.bosch-ic-components-reservation:before{content:\"\\e102\"}.bosch-ic-concierge-bell:before{content:\"\\e103\"}.bosch-ic-configuration:before{content:\"\\e104\"}.bosch-ic-confirmation:before{content:\"\\e105\"}.bosch-ic-connection-off:before{content:\"\\e106\"}.bosch-ic-connection-on:before{content:\"\\e107\"}.bosch-ic-connectivity:before{content:\"\\e108\"}.bosch-ic-construction:before{content:\"\\e109\"}.bosch-ic-consumer-goods:before{content:\"\\e10a\"}.bosch-ic-contract:before{content:\"\\e10b\"}.bosch-ic-copy:before{content:\"\\e10c\"}.bosch-ic-cube:before{content:\"\\e10d\"}.bosch-ic-cube-arrows:before{content:\"\\e10e\"}.bosch-ic-cube-filled:before{content:\"\\e10f\"}.bosch-ic-cube-shock:before{content:\"\\e110\"}.bosch-ic-cube-stacked:before{content:\"\\e111\"}.bosch-ic-customerservice:before{content:\"\\e112\"}.bosch-ic-cutlery:before{content:\"\\e113\"}.bosch-ic-damper:before{content:\"\\e114\"}.bosch-ic-danger-flash:before{content:\"\\e115\"}.bosch-ic-data-loss:before{content:\"\\e116\"}.bosch-ic-dealer-details:before{content:\"\\e117\"}.bosch-ic-delete:before{content:\"\\e118\"}.bosch-ic-delivery:before{content:\"\\e119\"}.bosch-ic-denied:before{content:\"\\e11a\"}.bosch-ic-desktop:before{content:\"\\e11b\"}.bosch-ic-desktop-application-2d:before{content:\"\\e11c\"}.bosch-ic-desktop-application-3d:before{content:\"\\e11d\"}.bosch-ic-development:before{content:\"\\e11e\"}.bosch-ic-devices:before{content:\"\\e11f\"}.bosch-ic-devices-mobile:before{content:\"\\e120\"}.bosch-ic-directory:before{content:\"\\e121\"}.bosch-ic-dna:before{content:\"\\e122\"}.bosch-ic-do-not-disturb:before{content:\"\\e123\"}.bosch-ic-document:before{content:\"\\e124\"}.bosch-ic-document-add:before{content:\"\\e125\"}.bosch-ic-document-arrow-down:before{content:\"\\e126\"}.bosch-ic-document-arrow-up:before{content:\"\\e127\"}.bosch-ic-document-audio:before{content:\"\\e128\"}.bosch-ic-document-check:before{content:\"\\e129\"}.bosch-ic-document-cloud:before{content:\"\\e12a\"}.bosch-ic-document-copy:before{content:\"\\e12b\"}.bosch-ic-document-delete:before{content:\"\\e12c\"}.bosch-ic-document-doc:before{content:\"\\e12d\"}.bosch-ic-document-edit:before{content:\"\\e12e\"}.bosch-ic-document-error:before{content:\"\\e12f\"}.bosch-ic-document-locked:before{content:\"\\e130\"}.bosch-ic-document-log:before{content:\"\\e131\"}.bosch-ic-document-one:before{content:\"\\e132\"}.bosch-ic-document-pdf:before{content:\"\\e133\"}.bosch-ic-document-plain:before{content:\"\\e134\"}.bosch-ic-document-plain-add:before{content:\"\\e135\"}.bosch-ic-document-plain-delete:before{content:\"\\e136\"}.bosch-ic-document-ppt:before{content:\"\\e137\"}.bosch-ic-document-pub:before{content:\"\\e138\"}.bosch-ic-document-rtf:before{content:\"\\e139\"}.bosch-ic-document-save-to:before{content:\"\\e13a\"}.bosch-ic-document-search:before{content:\"\\e13b\"}.bosch-ic-document-settings:before{content:\"\\e13c\"}.bosch-ic-document-test:before{content:\"\\e13d\"}.bosch-ic-document-text:before{content:\"\\e13e\"}.bosch-ic-document-txt:before{content:\"\\e13f\"}.bosch-ic-document-vsd:before{content:\"\\e140\"}.bosch-ic-document-xls:before{content:\"\\e141\"}.bosch-ic-document-xml:before{content:\"\\e142\"}.bosch-ic-document-zip:before{content:\"\\e143\"}.bosch-ic-dome:before{content:\"\\e144\"}.bosch-ic-door:before{content:\"\\e145\"}.bosch-ic-door-sensor:before{content:\"\\e146\"}.bosch-ic-down:before{content:\"\\e147\"}.bosch-ic-down-double:before{content:\"\\e148\"}.bosch-ic-download:before{content:\"\\e149\"}.bosch-ic-drag-handle:before{content:\"\\e14a\"}.bosch-ic-dragdrop:before{content:\"\\e14b\"}.bosch-ic-drop:before{content:\"\\e14c\"}.bosch-ic-e-charging:before{content:\"\\e14d\"}.bosch-ic-e-mobility:before{content:\"\\e14e\"}.bosch-ic-ear-off-disabled-light:before{content:\"\\e14f\"}.bosch-ic-ear-on:before{content:\"\\e150\"}.bosch-ic-eco-system:before{content:\"\\e151\"}.bosch-ic-edit:before{content:\"\\e152\"}.bosch-ic-education:before{content:\"\\e153\"}.bosch-ic-efficiency:before{content:\"\\e154\"}.bosch-ic-elevator:before{content:\"\\e155\"}.bosch-ic-elevator-alarm:before{content:\"\\e156\"}.bosch-ic-elevator-cloud:before{content:\"\\e157\"}.bosch-ic-elevator-headset:before{content:\"\\e158\"}.bosch-ic-elevator-service:before{content:\"\\e159\"}.bosch-ic-emergency-exit:before{content:\"\\e15a\"}.bosch-ic-emoji-happy:before{content:\"\\e15b\"}.bosch-ic-emoji-neutral:before{content:\"\\e15c\"}.bosch-ic-emoji-sad:before{content:\"\\e15d\"}.bosch-ic-emoji-super-happy:before{content:\"\\e15e\"}.bosch-ic-emoji-very-sad:before{content:\"\\e15f\"}.bosch-ic-eu-energy-label:before{content:\"\\e160\"}.bosch-ic-excavator:before{content:\"\\e161\"}.bosch-ic-exit:before{content:\"\\e163\"}.bosch-ic-expansion-arrows:before{content:\"\\e164\"}.bosch-ic-explosion:before{content:\"\\e165\"}.bosch-ic-export:before{content:\"\\e166\"}.bosch-ic-externallink:before{content:\"\\e167\"}.bosch-ic-fast-forward:before{content:\"\\e168\"}.bosch-ic-faucet:before{content:\"\\e169\"}.bosch-ic-favorites:before{content:\"\\e16a\"}.bosch-ic-fax:before{content:\"\\e16b\"}.bosch-ic-female:before{content:\"\\e16c\"}.bosch-ic-film:before{content:\"\\e16d\"}.bosch-ic-filter:before{content:\"\\e16e\"}.bosch-ic-filter-success:before{content:\"\\e16f\"}.bosch-ic-fingerprint:before{content:\"\\e170\"}.bosch-ic-fire:before{content:\"\\e171\"}.bosch-ic-fire-emergency:before{content:\"\\e172\"}.bosch-ic-fireworks:before{content:\"\\e173\"}.bosch-ic-first-aid:before{content:\"\\e174\"}.bosch-ic-first-aid-cross:before{content:\"\\e175\"}.bosch-ic-fit-to-sceen:before{content:\"\\e176\"}.bosch-ic-flag:before{content:\"\\e177\"}.bosch-ic-flash:before{content:\"\\e178\"}.bosch-ic-flask:before{content:\"\\e179\"}.bosch-ic-flexpress:before{content:\"\\e17a\"}.bosch-ic-folder:before{content:\"\\e17b\"}.bosch-ic-folder-download:before{content:\"\\e17c\"}.bosch-ic-folder-open:before{content:\"\\e17d\"}.bosch-ic-folder-upload:before{content:\"\\e17e\"}.bosch-ic-fork-lift:before{content:\"\\e17f\"}.bosch-ic-fork-lift-locator:before{content:\"\\e180\"}.bosch-ic-forward-right:before{content:\"\\e181\"}.bosch-ic-forward-right-double:before{content:\"\\e182\"}.bosch-ic-full-empty:before{content:\"\\e183\"}.bosch-ic-fullscreen:before{content:\"\\e184\"}.bosch-ic-fullscreen-arrows:before{content:\"\\e185\"}.bosch-ic-fullscreen-exit:before{content:\"\\e186\"}.bosch-ic-g-arrow-down:before{content:\"\\e187\"}.bosch-ic-g-arrow-up:before{content:\"\\e188\"}.bosch-ic-generator:before{content:\"\\e189\"}.bosch-ic-gift:before{content:\"\\e18a\"}.bosch-ic-glance:before{content:\"\\e18b\"}.bosch-ic-glas-plate:before{content:\"\\e18c\"}.bosch-ic-globe:before{content:\"\\e18d\"}.bosch-ic-globe-arrow:before{content:\"\\e18e\"}.bosch-ic-glossary:before{content:\"\\e18f\"}.bosch-ic-grid-view:before{content:\"\\e190\"}.bosch-ic-hammer:before{content:\"\\e191\"}.bosch-ic-hand:before{content:\"\\e192\"}.bosch-ic-hand-motion:before{content:\"\\e193\"}.bosch-ic-hand-motion-off-disabled-light:before{content:\"\\e194\"}.bosch-ic-handlewithcare:before{content:\"\\e195\"}.bosch-ic-handover-add:before{content:\"\\e196\"}.bosch-ic-handover-check:before{content:\"\\e197\"}.bosch-ic-handover-remove:before{content:\"\\e198\"}.bosch-ic-hanger:before{content:\"\\e199\"}.bosch-ic-health:before{content:\"\\e19a\"}.bosch-ic-heart:before{content:\"\\e19b\"}.bosch-ic-history:before{content:\"\\e19c\"}.bosch-ic-home:before{content:\"\\e19d\"}.bosch-ic-home-locator:before{content:\"\\e19e\"}.bosch-ic-hourglass:before{content:\"\\e19f\"}.bosch-ic-hydrant:before{content:\"\\e1a0\"}.bosch-ic-ice:before{content:\"\\e1a1\"}.bosch-ic-imagery:before{content:\"\\e1a2\"}.bosch-ic-imagery-add:before{content:\"\\e1a3\"}.bosch-ic-imagery-remove:before{content:\"\\e1a4\"}.bosch-ic-import:before{content:\"\\e1a5\"}.bosch-ic-imprint:before{content:\"\\e1a6\"}.bosch-ic-impulse:before{content:\"\\e1a7\"}.bosch-ic-industry:before{content:\"\\e1a8\"}.bosch-ic-industry-clock:before{content:\"\\e1a9\"}.bosch-ic-industry-connected:before{content:\"\\e1aa\"}.bosch-ic-info:before{content:\"\\e1ab\"}.bosch-ic-info-i:before{content:\"\\e1ac\"}.bosch-ic-interval:before{content:\"\\e1ad\"}.bosch-ic-it-device:before{content:\"\\e1ae\"}.bosch-ic-jewel:before{content:\"\\e1af\"}.bosch-ic-keyboard:before{content:\"\\e1b0\"}.bosch-ic-label:before{content:\"\\e1b1\"}.bosch-ic-laptop:before{content:\"\\e1b2\"}.bosch-ic-laser:before{content:\"\\e1b3\"}.bosch-ic-layout:before{content:\"\\e1b4\"}.bosch-ic-leaf:before{content:\"\\e1b5\"}.bosch-ic-led:before{content:\"\\e1b6\"}.bosch-ic-brick:before{content:\"\\e1b7\"}.bosch-ic-brick-slanting-1:before{content:\"\\e1b8\"}.bosch-ic-brick-slanting-2:before{content:\"\\e1b9\"}.bosch-ic-less-minimize:before{content:\"\\e1ba\"}.bosch-ic-lightbulb:before{content:\"\\e1bb\"}.bosch-ic-lightbulb-off:before{content:\"\\e1bc\"}.bosch-ic-line-chart:before{content:\"\\e1bd\"}.bosch-ic-link:before{content:\"\\e1be\"}.bosch-ic-link-broken:before{content:\"\\e1bf\"}.bosch-ic-list-view:before{content:\"\\e1c0\"}.bosch-ic-list-view-mobile:before{content:\"\\e1c1\"}.bosch-ic-livechat:before{content:\"\\e1c2\"}.bosch-ic-locator:before{content:\"\\e1c3\"}.bosch-ic-locator-ip-connected:before{content:\"\\e1c4\"}.bosch-ic-locator-ip-disconnected:before{content:\"\\e1c5\"}.bosch-ic-locator-spot:before{content:\"\\e1c6\"}.bosch-ic-locator-spot-check:before{content:\"\\e1c7\"}.bosch-ic-lock-closed:before{content:\"\\e1c8\"}.bosch-ic-lock-open:before{content:\"\\e1c9\"}.bosch-ic-login:before{content:\"\\e1ca\"}.bosch-ic-logistics:before{content:\"\\e1cb\"}.bosch-ic-logout:before{content:\"\\e1cc\"}.bosch-ic-machine:before{content:\"\\e1cd\"}.bosch-ic-magnet:before{content:\"\\e1ce\"}.bosch-ic-mail:before{content:\"\\e1cf\"}.bosch-ic-mail-forward:before{content:\"\\e1d0\"}.bosch-ic-mail-open:before{content:\"\\e1d1\"}.bosch-ic-mail-out:before{content:\"\\e1d2\"}.bosch-ic-mail-reply:before{content:\"\\e1d3\"}.bosch-ic-male:before{content:\"\\e1d4\"}.bosch-ic-map:before{content:\"\\e1d5\"}.bosch-ic-material-add:before{content:\"\\e1d6\"}.bosch-ic-material-check:before{content:\"\\e1d7\"}.bosch-ic-material-remove:before{content:\"\\e1d8\"}.bosch-ic-maximize:before{content:\"\\e1d9\"}.bosch-ic-mechanic:before{content:\"\\e1da\"}.bosch-ic-megaphone:before{content:\"\\e1db\"}.bosch-ic-message:before{content:\"\\e1dc\"}.bosch-ic-mic:before{content:\"\\e1dd\"}.bosch-ic-microphone:before{content:\"\\e1de\"}.bosch-ic-microphone-classic:before{content:\"\\e1df\"}.bosch-ic-minimize:before{content:\"\\e1e0\"}.bosch-ic-money-currency:before{content:\"\\e1e1\"}.bosch-ic-money-dollar:before{content:\"\\e1e2\"}.bosch-ic-moneybag-currency:before{content:\"\\e1e3\"}.bosch-ic-moneybag-dollar:before{content:\"\\e1e4\"}.bosch-ic-monument:before{content:\"\\e1e5\"}.bosch-ic-moon:before{content:\"\\e1e6\"}.bosch-ic-motorcycle:before{content:\"\\e1e7\"}.bosch-ic-motorcycle-side-car:before{content:\"\\e1e8\"}.bosch-ic-motorcycle-side:before{content:\"\\e1e9\"}.bosch-ic-mouse:before{content:\"\\e1ea\"}.bosch-ic-mouse-left:before{content:\"\\e1eb\"}.bosch-ic-mouse-right:before{content:\"\\e1ec\"}.bosch-ic-mouth:before{content:\"\\e1ed\"}.bosch-ic-my-product:before{content:\"\\e1ee\"}.bosch-ic-new-way-work:before{content:\"\\e1ef\"}.bosch-ic-newsletter:before{content:\"\\e1f0\"}.bosch-ic-newspaper:before{content:\"\\e1f1\"}.bosch-ic-nose:before{content:\"\\e1f2\"}.bosch-ic-notepad:before{content:\"\\e1f3\"}.bosch-ic-notepad-edit:before{content:\"\\e1f4\"}.bosch-ic-notification:before{content:\"\\e1f5\"}.bosch-ic-notification-active:before{content:\"\\e1f6\"}.bosch-ic-notification-add:before{content:\"\\e1f7\"}.bosch-ic-notification-remove:before{content:\"\\e1f8\"}.bosch-ic-oil-car:before{content:\"\\e1f9\"}.bosch-ic-omnichannel:before{content:\"\\e1fa\"}.bosch-ic-options:before{content:\"\\e1fb\"}.bosch-ic-origami-boat:before{content:\"\\e1fc\"}.bosch-ic-pallete-car:before{content:\"\\e1fd\"}.bosch-ic-pallete-car-connected:before{content:\"\\e1fe\"}.bosch-ic-panel:before{content:\"\\e1ff\"}.bosch-ic-paperclip:before{content:\"\\e200\"}.bosch-ic-paperplane:before{content:\"\\e201\"}.bosch-ic-parking:before{content:\"\\e202\"}.bosch-ic-pause:before{content:\"\\e203\"}.bosch-ic-payment:before{content:\"\\e204\"}.bosch-ic-people:before{content:\"\\e205\"}.bosch-ic-petrol-station:before{content:\"\\e206\"}.bosch-ic-piggybank:before{content:\"\\e207\"}.bosch-ic-pin-classic:before{content:\"\\e208\"}.bosch-ic-pin-modern:before{content:\"\\e209\"}.bosch-ic-plane-side:before{content:\"\\e20a\"}.bosch-ic-play:before{content:\"\\e20b\"}.bosch-ic-plug-12v:before{content:\"\\e20c\"}.bosch-ic-podium-speaker:before{content:\"\\e20d\"}.bosch-ic-police:before{content:\"\\e20e\"}.bosch-ic-poop:before{content:\"\\e20f\"}.bosch-ic-post-it:before{content:\"\\e210\"}.bosch-ic-postal-code:before{content:\"\\e211\"}.bosch-ic-power:before{content:\"\\e212\"}.bosch-ic-print:before{content:\"\\e213\"}.bosch-ic-prototyping:before{content:\"\\e214\"}.bosch-ic-puzzle:before{content:\"\\e215\"}.bosch-ic-quad:before{content:\"\\e216\"}.bosch-ic-question:before{content:\"\\e217\"}.bosch-ic-radar:before{content:\"\\e218\"}.bosch-ic-radio:before{content:\"\\e219\"}.bosch-ic-radiotower:before{content:\"\\e21a\"}.bosch-ic-redirect:before{content:\"\\e21b\"}.bosch-ic-redo:before{content:\"\\e21c\"}.bosch-ic-reference:before{content:\"\\e21d\"}.bosch-ic-refresh:before{content:\"\\e21e\"}.bosch-ic-refresh-cloud:before{content:\"\\e21f\"}.bosch-ic-registration:before{content:\"\\e220\"}.bosch-ic-remote:before{content:\"\\e221\"}.bosch-ic-reset:before{content:\"\\e222\"}.bosch-ic-resolution:before{content:\"\\e223\"}.bosch-ic-robot:before{content:\"\\e224\"}.bosch-ic-robot-connected:before{content:\"\\e225\"}.bosch-ic-robothead:before{content:\"\\e226\"}.bosch-ic-rocket:before{content:\"\\e227\"}.bosch-ic-rotation:before{content:\"\\e228\"}.bosch-ic-rotation-360:before{content:\"\\e229\"}.bosch-ic-rotation-x-left:before{content:\"\\e22a\"}.bosch-ic-rotation-x-right:before{content:\"\\e22b\"}.bosch-ic-rotation-y-down:before{content:\"\\e22c\"}.bosch-ic-rotation-y-up:before{content:\"\\e22d\"}.bosch-ic-route:before{content:\"\\e22e\"}.bosch-ic-ruler:before{content:\"\\e22f\"}.bosch-ic-ruler-pen:before{content:\"\\e230\"}.bosch-ic-satellite:before{content:\"\\e231\"}.bosch-ic-save:before{content:\"\\e232\"}.bosch-ic-scale:before{content:\"\\e233\"}.bosch-ic-scissors:before{content:\"\\e234\"}.bosch-ic-search:before{content:\"\\e235\"}.bosch-ic-security:before{content:\"\\e236\"}.bosch-ic-security-check:before{content:\"\\e237\"}.bosch-ic-security-user:before{content:\"\\e238\"}.bosch-ic-sensor:before{content:\"\\e239\"}.bosch-ic-server:before{content:\"\\e23a\"}.bosch-ic-server-expansion:before{content:\"\\e23b\"}.bosch-ic-server-rate:before{content:\"\\e23c\"}.bosch-ic-service-agent:before{content:\"\\e23d\"}.bosch-ic-service-time:before{content:\"\\e23e\"}.bosch-ic-settings:before{content:\"\\e23f\"}.bosch-ic-settings-arrows:before{content:\"\\e240\"}.bosch-ic-settings-connected:before{content:\"\\e241\"}.bosch-ic-share:before{content:\"\\e242\"}.bosch-ic-shield-stop:before{content:\"\\e243\"}.bosch-ic-ship:before{content:\"\\e244\"}.bosch-ic-ship-side:before{content:\"\\e245\"}.bosch-ic-shoppingcart:before{content:\"\\e246\"}.bosch-ic-shoppingcart-add:before{content:\"\\e247\"}.bosch-ic-shoppingcart-switch:before{content:\"\\e248\"}.bosch-ic-signal-full:before{content:\"\\e249\"}.bosch-ic-signal-half:before{content:\"\\e24a\"}.bosch-ic-signal-lost:before{content:\"\\e24b\"}.bosch-ic-signpost:before{content:\"\\e24c\"}.bosch-ic-simcard:before{content:\"\\e24d\"}.bosch-ic-skyscraper:before{content:\"\\e24e\"}.bosch-ic-smartcity:before{content:\"\\e24f\"}.bosch-ic-smarthome:before{content:\"\\e250\"}.bosch-ic-smartphone:before{content:\"\\e251\"}.bosch-ic-smartphone-acoustic:before{content:\"\\e252\"}.bosch-ic-smartphone-arrow-right:before{content:\"\\e253\"}.bosch-ic-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.bosch-ic-smartphone-clock:before{content:\"\\e255\"}.bosch-ic-smartphone-cloud:before{content:\"\\e256\"}.bosch-ic-smartphone-loading-wheel:before{content:\"\\e257\"}.bosch-ic-smartphone-smashed:before{content:\"\\e258\"}.bosch-ic-smartphone-vibration:before{content:\"\\e259\"}.bosch-ic-smartphone-z:before{content:\"\\e25a\"}.bosch-ic-smartwatch-connected:before{content:\"\\e25b\"}.bosch-ic-spare-parts:before{content:\"\\e25c\"}.bosch-ic-spark-plug:before{content:\"\\e25d\"}.bosch-ic-speaker-acoustic:before{content:\"\\e25e\"}.bosch-ic-speedometer:before{content:\"\\e25f\"}.bosch-ic-stadium:before{content:\"\\e260\"}.bosch-ic-stairs:before{content:\"\\e261\"}.bosch-ic-stamp:before{content:\"\\e262\"}.bosch-ic-standby:before{content:\"\\e263\"}.bosch-ic-steering-wheel:before{content:\"\\e264\"}.bosch-ic-steering-wheel-connected:before{content:\"\\e265\"}.bosch-ic-steering-wheel-hands:before{content:\"\\e266\"}.bosch-ic-steps:before{content:\"\\e267\"}.bosch-ic-stethoscope:before{content:\"\\e268\"}.bosch-ic-stop:before{content:\"\\e269\"}.bosch-ic-stopwatch:before{content:\"\\e26a\"}.bosch-ic-storage-local:before{content:\"\\e26b\"}.bosch-ic-structure:before{content:\"\\e26c\"}.bosch-ic-subtitles:before{content:\"\\e26d\"}.bosch-ic-subtitles-off-light:before{content:\"\\e26e\"}.bosch-ic-summary:before{content:\"\\e26f\"}.bosch-ic-sun:before{content:\"\\e270\"}.bosch-ic-sun-half-filled:before{content:\"\\e271\"}.bosch-ic-sunshade-table:before{content:\"\\e272\"}.bosch-ic-surveillance:before{content:\"\\e273\"}.bosch-ic-table-chairs:before{content:\"\\e274\"}.bosch-ic-tablet:before{content:\"\\e275\"}.bosch-ic-team:before{content:\"\\e276\"}.bosch-ic-team-lightbulb:before{content:\"\\e277\"}.bosch-ic-technical-service:before{content:\"\\e278\"}.bosch-ic-temperature:before{content:\"\\e279\"}.bosch-ic-temperature-high:before{content:\"\\e27a\"}.bosch-ic-temperature-low:before{content:\"\\e27b\"}.bosch-ic-theater:before{content:\"\\e27c\"}.bosch-ic-thumb-down:before{content:\"\\e27d\"}.bosch-ic-thumb-up:before{content:\"\\e27e\"}.bosch-ic-ticket:before{content:\"\\e27f\"}.bosch-ic-touch:before{content:\"\\e280\"}.bosch-ic-touch-sos:before{content:\"\\e281\"}.bosch-ic-towing-truck:before{content:\"\\e282\"}.bosch-ic-train:before{content:\"\\e283\"}.bosch-ic-train-side:before{content:\"\\e284\"}.bosch-ic-transport-movements:before{content:\"\\e285\"}.bosch-ic-transscript:before{content:\"\\e286\"}.bosch-ic-transscript-off:before{content:\"\\e287\"}.bosch-ic-transscript-on:before{content:\"\\e288\"}.bosch-ic-tree:before{content:\"\\e289\"}.bosch-ic-tricycle:before{content:\"\\e28a\"}.bosch-ic-trolley-empty:before{content:\"\\e28b\"}.bosch-ic-trolley-filled-box:before{content:\"\\e28c\"}.bosch-ic-trolly-filled-files:before{content:\"\\e28d\"}.bosch-ic-trophy:before{content:\"\\e28e\"}.bosch-ic-truck:before{content:\"\\e28f\"}.bosch-ic-tuktuk:before{content:\"\\e290\"}.bosch-ic-tune:before{content:\"\\e291\"}.bosch-ic-typography:before{content:\"\\e292\"}.bosch-ic-ui:before{content:\"\\e293\"}.bosch-ic-umbrella:before{content:\"\\e294\"}.bosch-ic-undo:before{content:\"\\e295\"}.bosch-ic-up:before{content:\"\\e296\"}.bosch-ic-up-double:before{content:\"\\e297\"}.bosch-ic-up-down:before{content:\"\\e298\"}.bosch-ic-upload:before{content:\"\\e299\"}.bosch-ic-usb:before{content:\"\\e29a\"}.bosch-ic-user:before{content:\"\\e29b\"}.bosch-ic-user-advanced:before{content:\"\\e29c\"}.bosch-ic-user-beginner:before{content:\"\\e29d\"}.bosch-ic-user-desktop:before{content:\"\\e29e\"}.bosch-ic-user-down:before{content:\"\\e29f\"}.bosch-ic-user-falling:before{content:\"\\e2a0\"}.bosch-ic-user-hand:before{content:\"\\e2a1\"}.bosch-ic-user-hand-stop:before{content:\"\\e2a2\"}.bosch-ic-user-professional:before{content:\"\\e2a3\"}.bosch-ic-user-run:before{content:\"\\e2a4\"}.bosch-ic-user-share:before{content:\"\\e2a5\"}.bosch-ic-user-voice:before{content:\"\\e2a6\"}.bosch-ic-user-walk:before{content:\"\\e2a7\"}.bosch-ic-video:before{content:\"\\e2a8\"}.bosch-ic-video-disabled-light:before{content:\"\\e2a9\"}.bosch-ic-video-record:before{content:\"\\e2aa\"}.bosch-ic-virtual-reality:before{content:\"\\e2ab\"}.bosch-ic-volume-disabled:before{content:\"\\e2ac\"}.bosch-ic-volume-high:before{content:\"\\e2ad\"}.bosch-ic-volume-low:before{content:\"\\e2ae\"}.bosch-ic-volume-off-light:before{content:\"\\e2af\"}.bosch-ic-wand:before{content:\"\\e2b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-watch-off-disabled-light:before{content:\"\\e2b2\"}.bosch-ic-watch-on:before{content:\"\\e2b3\"}.bosch-ic-water-shower:before{content:\"\\e2b4\"}.bosch-ic-welcome:before{content:\"\\e2b5\"}.bosch-ic-wellness:before{content:\"\\e2b6\"}.bosch-ic-wheelchair:before{content:\"\\e2b7\"}.bosch-ic-whistle:before{content:\"\\e2b8\"}.bosch-ic-wide-angled-arrow:before{content:\"\\e2b9\"}.bosch-ic-wifi:before{content:\"\\e2ba\"}.bosch-ic-window-analysis:before{content:\"\\e2bb\"}.bosch-ic-window-browser:before{content:\"\\e2bc\"}.bosch-ic-window-console:before{content:\"\\e2bd\"}.bosch-ic-window-gaussian-view:before{content:\"\\e2be\"}.bosch-ic-window-info:before{content:\"\\e2bf\"}.bosch-ic-window-new:before{content:\"\\e2c0\"}.bosch-ic-window-resize:before{content:\"\\e2c1\"}.bosch-ic-window-scaling-view:before{content:\"\\e2c2\"}.bosch-ic-window-shuffle:before{content:\"\\e2c3\"}.bosch-ic-window-statistic:before{content:\"\\e2c4\"}.bosch-ic-wishlist:before{content:\"\\e2c5\"}.bosch-ic-work-order:before{content:\"\\e2c6\"}.bosch-ic-worldwideweb:before{content:\"\\e2c7\"}.bosch-ic-wrench:before{content:\"\\e2c8\"}.bosch-ic-zoom-in:before{content:\"\\e2c9\"}.bosch-ic-zoom-out:before{content:\"\\e2ca\"}.bosch-ic-scooter:before{content:\"\\e2cb\"}.bosch-ic-cars-traffic:before{content:\"\\e2cc\"}.bosch-ic-arm-chair:before{content:\"\\e2cd\"}.bosch-ic-car-phone:before{content:\"\\e2ce\"}.bosch-ic-box-lightbulb:before{content:\"\\e2cf\"}.bosch-ic-chair-officedesk:before{content:\"\\e2d0\"}.bosch-ic-calendar-clock:before{content:\"\\e2d1\"}.bosch-ic-document-cv:before{content:\"\\e2d2\"}.bosch-ic-user-run-bag:before{content:\"\\e2d3\"}.bosch-ic-brake-disk-spray:before{content:\"\\e2d4\"}.bosch-ic-circle-abs:before{content:\"\\e2d5\"}.bosch-ic-bright-down:before{content:\"\\e2d6\"}.bosch-ic-bright-up:before{content:\"\\e2d7\"}.bosch-ic-fan:before{content:\"\\e2d8\"}.bosch-ic-sun-moon:before{content:\"\\e2d9\"}.bosch-ic-lightbulb-attention:before{content:\"\\e2da\"}.bosch-ic-settings-attention:before{content:\"\\e2db\"}.bosch-ic-danger-hazards:before{content:\"\\e2dc\"}.bosch-ic-windshield-spray:before{content:\"\\e2dd\"}.bosch-ic-windshield-heat:before{content:\"\\e2de\"}.bosch-ic-light-drop:before{content:\"\\e2df\"}.bosch-ic-starter:before{content:\"\\e2e0\"}.bosch-ic-turbo:before{content:\"\\e2e1\"}.bosch-ic-settings-lock:before{content:\"\\e2e2\"}.bosch-ic-settings-n:before{content:\"\\e2e3\"}.bosch-ic-light-beam-high:before{content:\"\\e2e4\"}.bosch-ic-light-beam-low:before{content:\"\\e2e5\"}.bosch-ic-glas:before{content:\"\\e2e6\"}.bosch-ic-shirt-ice:before{content:\"\\e2e7\"}.bosch-ic-helmet-fan:before{content:\"\\e2e8\"}.bosch-ic-notification-arrow:before{content:\"\\e2e9\"}.bosch-ic-steering-wheel-arrow:before{content:\"\\e2ea\"}.bosch-ic-circle-arrow:before{content:\"\\e2eb\"}.bosch-ic-tempomat-pit:before{content:\"\\e2ec\"}.bosch-ic-radiator-fan-spray:before{content:\"\\e2ed\"}.bosch-ic-boost-down:before{content:\"\\e2ee\"}.bosch-ic-boost-up:before{content:\"\\e2ef\"}.bosch-ic-circle-lock:before{content:\"\\e2f0\"}.bosch-ic-mp-down:before{content:\"\\e2f1\"}.bosch-ic-mp-up:before{content:\"\\e2f2\"}.bosch-ic-tc-down:before{content:\"\\e2f3\"}.bosch-ic-tc-up:before{content:\"\\e2f4\"}.bosch-ic-danger-arrow-attention:before{content:\"\\e2f5\"}.bosch-ic-user-remove:before{content:\"\\e2f6\"}.bosch-ic-user-add:before{content:\"\\e2f7\"}.bosch-ic-post-it-collection:before{content:\"\\e2f8\"}.bosch-ic-horn:before{content:\"\\e2f9\"}.bosch-ic-desktop-notification:before{content:\"\\e2fa\"}.bosch-ic-router:before{content:\"\\e2fb\"}.bosch-ic-berry:before{content:\"\\e2fc\"}.bosch-ic-chat-language:before{content:\"\\e2fd\"}.bosch-ic-game-controller:before{content:\"\\e2fe\"}.bosch-ic-user-brain:before{content:\"\\e2ff\"}.bosch-ic-mouth-finger:before{content:\"\\e300\"}.bosch-ic-excavator-shovel:before{content:\"\\e301\"}.bosch-ic-ear-in:before{content:\"\\e302\"}.bosch-ic-ear-out:before{content:\"\\e303\"}.bosch-ic-lion:before{content:\"\\e304\"}.bosch-ic-car-convertible:before{content:\"\\e305\"}.bosch-ic-car-sports:before{content:\"\\e306\"}.bosch-ic-bear:before{content:\"\\e307\"}.bosch-ic-pot:before{content:\"\\e308\"}.bosch-ic-bed-double:before{content:\"\\e309\"}.bosch-ic-air-hot:before{content:\"\\e30a\"}.bosch-ic-air-ice:before{content:\"\\e30b\"}.bosch-ic-air-leaf:before{content:\"\\e30c\"}.bosch-ic-air-purifier-0:before{content:\"\\e30d\"}.bosch-ic-air-purifier-1:before{content:\"\\e30e\"}.bosch-ic-air-purifier-2:before{content:\"\\e30f\"}.bosch-ic-air-purifier-3:before{content:\"\\e310\"}.bosch-ic-air:before{content:\"\\e311\"}.bosch-ic-fan-off-disabled-bold:before{content:\"\\e312\"}.bosch-ic-ice-off-disabled-bold:before{content:\"\\e313\"}.bosch-ic-faucet-boost:before{content:\"\\e314\"}.bosch-ic-faucet-mode:before{content:\"\\e315\"}.bosch-ic-faucet-off-disabled-bold:before{content:\"\\e316\"}.bosch-ic-faucet-schedule:before{content:\"\\e317\"}.bosch-ic-fuel-cell:before{content:\"\\e318\"}.bosch-ic-heating-floor-off-disabled-bold:before{content:\"\\e319\"}.bosch-ic-heating-floor:before{content:\"\\e31a\"}.bosch-ic-heating-radiator-boost:before{content:\"\\e31b\"}.bosch-ic-heating-radiator-mode:before{content:\"\\e31c\"}.bosch-ic-heating-radiator-off-disabled-bold:before{content:\"\\e31d\"}.bosch-ic-heating-radiator-schedule:before{content:\"\\e31e\"}.bosch-ic-leaf-simple:before{content:\"\\e31f\"}.bosch-ic-oil-barrel:before{content:\"\\e320\"}.bosch-ic-water-bathtub:before{content:\"\\e321\"}.bosch-ic-water-dishwash:before{content:\"\\e322\"}.bosch-ic-water-handwash:before{content:\"\\e323\"}.bosch-ic-water-hot:before{content:\"\\e324\"}.bosch-ic-water-ice:before{content:\"\\e325\"}.bosch-ic-water-leaf:before{content:\"\\e326\"}.bosch-ic-water-ladder:before{content:\"\\e327\"}.bosch-ic-water-purifier-0:before{content:\"\\e328\"}.bosch-ic-water-purifier-1:before{content:\"\\e329\"}.bosch-ic-water-purifier-2:before{content:\"\\e32a\"}.bosch-ic-water-purifier-3:before{content:\"\\e32b\"}.bosch-ic-water-shower-boost:before{content:\"\\e32c\"}.bosch-ic-water-shower-mode:before{content:\"\\e32d\"}.bosch-ic-water-shower-off-disabled-bold:before{content:\"\\e32e\"}.bosch-ic-water-shower-schedule:before{content:\"\\e32f\"}.bosch-ic-water:before{content:\"\\e330\"}.bosch-ic-chart-bar-drop:before{content:\"\\e331\"}.bosch-ic-chart-bar-fire:before{content:\"\\e332\"}.bosch-ic-chart-bar-flash:before{content:\"\\e333\"}.bosch-ic-chart-bar-oil:before{content:\"\\e334\"}.bosch-ic-drop-minus:before{content:\"\\e335\"}.bosch-ic-drop-plus:before{content:\"\\e336\"}.bosch-ic-fan-minus:before{content:\"\\e337\"}.bosch-ic-fan-plus:before{content:\"\\e338\"}.bosch-ic-home-drops-0:before{content:\"\\e339\"}.bosch-ic-home-drops-1:before{content:\"\\e33a\"}.bosch-ic-home-drops-2:before{content:\"\\e33b\"}.bosch-ic-home-drops-3:before{content:\"\\e33c\"}.bosch-ic-home-temperature-in:before{content:\"\\e33d\"}.bosch-ic-home-temperature-out:before{content:\"\\e33e\"}.bosch-ic-home-user-in:before{content:\"\\e33f\"}.bosch-ic-ice-minus:before{content:\"\\e340\"}.bosch-ic-ice-plus:before{content:\"\\e341\"}.bosch-ic-chimney-sweeper:before{content:\"\\e342\"}.bosch-ic-menu-up:before{content:\"\\e343\"}.bosch-ic-security-ice:before{content:\"\\e344\"}.bosch-ic-security-legionella:before{content:\"\\e345\"}.bosch-ic-trolley-case:before{content:\"\\e346\"}.bosch-ic-sun-minus:before{content:\"\\e347\"}.bosch-ic-sun-plug:before{content:\"\\e348\"}.bosch-ic-sun-plus:before{content:\"\\e349\"}.bosch-ic-touch-pit:before{content:\"\\e34a\"}.bosch-ic-temperature-current:before{content:\"\\e34b\"}.bosch-ic-temperature-set:before{content:\"\\e34c\"}.bosch-ic-teddy-bear:before{content:\"\\e34d\"}.bosch-ic-document-svg:before{content:\"\\e34e\"}.bosch-ic-back-menu:before{content:\"\\e34f\"}.bosch-ic-car-arrows:before{content:\"\\e350\"}.bosch-ic-wrench-cloud:before{content:\"\\e351\"}.bosch-ic-weather-cloud-sun:before{content:\"\\e352\"}.bosch-ic-weather-cloudy:before{content:\"\\e353\"}.bosch-ic-weather-gale:before{content:\"\\e354\"}.bosch-ic-weather-mist:before{content:\"\\e355\"}.bosch-ic-weather-moon-cloud:before{content:\"\\e356\"}.bosch-ic-weather-rain-snow:before{content:\"\\e357\"}.bosch-ic-weather-rain-thunder:before{content:\"\\e358\"}.bosch-ic-weather-rain:before{content:\"\\e359\"}.bosch-ic-weather-snow:before{content:\"\\e35a\"}.bosch-ic-weather-thunder:before{content:\"\\e35b\"}.bosch-ic-weather-wind:before{content:\"\\e35c\"}.bosch-ic-plane-start:before{content:\"\\e35d\"}.bosch-ic-plane-land:before{content:\"\\e35e\"}.bosch-ic-graph-power:before{content:\"\\e35f\"}.bosch-ic-floorplan:before{content:\"\\e360\"}.bosch-ic-clock-pause:before{content:\"\\e361\"}.bosch-ic-clock-start:before{content:\"\\e362\"}.bosch-ic-clock-stop:before{content:\"\\e363\"}.bosch-ic-computer-retro:before{content:\"\\e364\"}.bosch-ic-smartphone-retro:before{content:\"\\e365\"}.bosch-ic-wood:before{content:\"\\e366\"}.bosch-ic-solid-fuel:before{content:\"\\e367\"}.bosch-ic-appliance-fan:before{content:\"\\e368\"}.bosch-ic-appliance-flame:before{content:\"\\e369\"}.bosch-ic-appliance-ice:before{content:\"\\e36a\"}.bosch-ic-appliance-oil:before{content:\"\\e36b\"}.bosch-ic-appliance-solid-fuel:before{content:\"\\e36c\"}.bosch-ic-appliance-sun-ice:before{content:\"\\e36d\"}.bosch-ic-appliance-sun:before{content:\"\\e36e\"}.bosch-ic-appliance-wood:before{content:\"\\e36f\"}.bosch-ic-appliance-air-leaf:before{content:\"\\e370\"}.bosch-ic-appliance-water-air:before{content:\"\\e371\"}.bosch-ic-appliance-water-leaf:before{content:\"\\e372\"}.bosch-ic-appliance-water-oil:before{content:\"\\e373\"}.bosch-ic-appliance-water:before{content:\"\\e374\"}.bosch-ic-appliance-heat-pump-air:before{content:\"\\e375\"}.bosch-ic-appliance-heat-pump-ground:before{content:\"\\e376\"}.bosch-ic-appliance-heat-pump-water:before{content:\"\\e377\"}.bosch-ic-appliance-plug-air:before{content:\"\\e378\"}.bosch-ic-appliance-plug-fuel-cell:before{content:\"\\e379\"}.bosch-ic-appliance-plug-leaf:before{content:\"\\e37a\"}.bosch-ic-appliance-plug-water-air:before{content:\"\\e37b\"}.bosch-ic-thermostat-connected-c:before{content:\"\\e37c\"}.bosch-ic-thermostat-connected-f:before{content:\"\\e37d\"}.bosch-ic-image-gallery:before{content:\"\\e37e\"}.bosch-ic-arrows-caution:before{content:\"\\e37f\"}.bosch-ic-arrows-diagonal-disabled-bold:before{content:\"\\e380\"}.bosch-ic-back-left-end:before{content:\"\\e381\"}.bosch-ic-back-right-end:before{content:\"\\e382\"}.bosch-ic-bar-chart-search:before{content:\"\\e383\"}.bosch-ic-batch:before{content:\"\\e384\"}.bosch-ic-calendar-remove:before{content:\"\\e385\"}.bosch-ic-call-bluetooth:before{content:\"\\e386\"}.bosch-ic-call-deny:before{content:\"\\e387\"}.bosch-ic-call-settings:before{content:\"\\e388\"}.bosch-ic-caution-sign-clock:before{content:\"\\e389\"}.bosch-ic-caution-sign-list:before{content:\"\\e38a\"}.bosch-ic-core-data:before{content:\"\\e38b\"}.bosch-ic-cursor-connected:before{content:\"\\e38c\"}.bosch-ic-cursor:before{content:\"\\e38d\"}.bosch-ic-dmc-code:before{content:\"\\e38e\"}.bosch-ic-find-id:before{content:\"\\e38f\"}.bosch-ic-globe-search:before{content:\"\\e390\"}.bosch-ic-interface:before{content:\"\\e391\"}.bosch-ic-material-add-manually:before{content:\"\\e392\"}.bosch-ic-material-list:before{content:\"\\e393\"}.bosch-ic-material-remove-manually:before{content:\"\\e394\"}.bosch-ic-material:before{content:\"\\e395\"}.bosch-ic-menu-checkmark:before{content:\"\\e396\"}.bosch-ic-qr-code:before{content:\"\\e397\"}.bosch-ic-sort-alphabetically:before{content:\"\\e398\"}.bosch-ic-unpin-classic-disabled-bold:before{content:\"\\e399\"}.bosch-ic-unpin-modern-disabled-bold:before{content:\"\\e39a\"}.bosch-ic-window-search:before{content:\"\\e39b\"}.bosch-ic-cloud-clock:before{content:\"\\e39c\"}.bosch-ic-concierge:before{content:\"\\e39d\"}.bosch-ic-heating-radiator:before{content:\"\\e39e\"}.bosch-ic-home-user-out:before{content:\"\\e39f\"}.bosch-ic-hot:before{content:\"\\e3a0\"}.bosch-ic-screen-service:before{content:\"\\e3a1\"}.bosch-ic-sun-ice:before{content:\"\\e3a2\"}.bosch-ic-sun-off-disabled-bold:before{content:\"\\e3a3\"}.bosch-ic-tractor:before{content:\"\\e3a4\"}.bosch-ic-video-recorder-digital:before{content:\"\\e3a5\"}.bosch-ic-windshield-wiper:before{content:\"\\e3a6\"}.bosch-ic-weather-snow-thunder:before{content:\"\\e3a7\"}.bosch-ic-electricity-pylon:before{content:\"\\e3a8\"}.bosch-ic-box-questionmark:before{content:\"\\e3a9\"}.bosch-ic-rainbow:before{content:\"\\e3aa\"}.bosch-ic-medal:before{content:\"\\e3ab\"}.bosch-ic-wheel:before{content:\"\\e3ac\"}.bosch-ic-borescope-connected:before{content:\"\\e3ad\"}.bosch-ic-filter-delete:before{content:\"\\e3ae\"}.bosch-ic-filter-add:before{content:\"\\e3af\"}.bosch-ic-man:before{content:\"\\e3b0\"}.bosch-ic-woman:before{content:\"\\e3b1\"}.bosch-ic-fridge:before{content:\"\\e3b2\"}.bosch-ic-battery-car:before{content:\"\\e3b3\"}.bosch-ic-bag-escalator:before{content:\"\\e3b4\"}.bosch-ic-shopping-bags:before{content:\"\\e3b5\"}.bosch-ic-car-steering-left:before{content:\"\\e3b6\"}.bosch-ic-car-steering-right:before{content:\"\\e3b7\"}.bosch-ic-thumb-up-down:before{content:\"\\e3b8\"}.bosch-ic-user-run-detection:before{content:\"\\e3b9\"}.bosch-ic-bank:before{content:\"\\e3ba\"}.bosch-ic-transportation:before{content:\"\\e3bb\"}.bosch-ic-government:before{content:\"\\e3bc\"}.bosch-ic-user-music:before{content:\"\\e3bd\"}.bosch-ic-bus:before{content:\"\\e3be\"}.bosch-ic-car-euro:before{content:\"\\e3bf\"}.bosch-ic-bag:before{content:\"\\e3c0\"}.bosch-ic-plate-tetrapack:before{content:\"\\e3c1\"}.bosch-ic-hospital:before{content:\"\\e3c2\"}.bosch-ic-desktop-structure:before{content:\"\\e3c3\"}.bosch-ic-frog-head:before{content:\"\\e3c4\"}.bosch-ic-sportboat:before{content:\"\\e3c5\"}.bosch-ic-yacht:before{content:\"\\e3c6\"}.bosch-ic-leaf-tilia:before{content:\"\\e3c7\"}.bosch-ic-music:before{content:\"\\e3c8\"}.bosch-ic-oil-lamp:before{content:\"\\e3c9\"}.bosch-ic-windshield-wiper-rear:before{content:\"\\e3ca\"}.bosch-ic-dongle-connected:before{content:\"\\e3cb\"}.bosch-ic-slate:before{content:\"\\e3cc\"}.bosch-ic-appliance-heat-pump-exhaust:before{content:\"\\e3cd\"}.bosch-ic-alert-list:before{content:\"\\e3ce\"}.bosch-ic-arrow-left-end-frame:before{content:\"\\e3cf\"}.bosch-ic-arrow-right-end-frame:before{content:\"\\e3d0\"}.bosch-ic-shape-ellipse:before{content:\"\\e3d1\"}.bosch-ic-shape-rectangle:before{content:\"\\e3d2\"}.bosch-ic-wishlist-add:before{content:\"\\e3d3\"}.bosch-ic-smartphone-disabled-light:before{content:\"\\e3d4\"}.bosch-ic-airbag:before{content:\"\\e3d5\"}.bosch-ic-compass:before{content:\"\\e3d6\"}.bosch-ic-turbine-wind:before{content:\"\\e3d7\"}.bosch-ic-arrows-diagonal-disabled-light:before{content:\"\\e3d8\"}.bosch-ic-ear-off-disabled-bold:before{content:\"\\e3d9\"}.bosch-ic-fan-off-disabled-light:before{content:\"\\e3da\"}.bosch-ic-faucet-off-disabled-light:before{content:\"\\e3db\"}.bosch-ic-hand-motion-off-disabled-bold:before{content:\"\\e3dc\"}.bosch-ic-heating-floor-off-disabled-light:before{content:\"\\e3dd\"}.bosch-ic-heating-radiator-off-disabled-light:before{content:\"\\e3de\"}.bosch-ic-ice-off-disabled-light:before{content:\"\\e3df\"}.bosch-ic-smartphone-disabled-bold:before{content:\"\\e3e0\"}.bosch-ic-subtitles-off-bold:before{content:\"\\e3e1\"}.bosch-ic-sun-off-disabled-light:before{content:\"\\e3e2\"}.bosch-ic-video-disabled-bold:before{content:\"\\e3e3\"}.bosch-ic-volume-off-bold:before{content:\"\\e3e4\"}.bosch-ic-watch-off-disabled-bold:before{content:\"\\e3e5\"}.bosch-ic-water-shower-off-disabled-light:before{content:\"\\e3e6\"}.bosch-ic-3d-frame-play:before{content:\"\\e3e7\"}.bosch-ic-alert-error-checkmark:before{content:\"\\e3e8\"}.bosch-ic-arrow-up-frame-check:before{content:\"\\e3e9\"}.bosch-ic-arrow-up-frame-error:before{content:\"\\e3ea\"}.bosch-ic-arrows-checkmark:before{content:\"\\e3eb\"}.bosch-ic-assembly-line-settings:before{content:\"\\e3ec\"}.bosch-ic-augmented-reality-player:before{content:\"\\e3ed\"}.bosch-ic-bar-chart-kappa:before{content:\"\\e3ef\"}.bosch-ic-bar-chart-search-detail:before{content:\"\\e3f0\"}.bosch-ic-barcode-checkmark:before{content:\"\\e3f1\"}.bosch-ic-barcode-scan-bulb:before{content:\"\\e3f2\"}.bosch-ic-barcode:before{content:\"\\e3f3\"}.bosch-ic-binary:before{content:\"\\e3f4\"}.bosch-ic-book-lightbulb:before{content:\"\\e3f5\"}.bosch-ic-box-arrow-down:before{content:\"\\e3f6\"}.bosch-ic-box-delivery-checkmark:before{content:\"\\e3f7\"}.bosch-ic-globe-checkmark:before{content:\"\\e3f8\"}.bosch-ic-calendar-boxes:before{content:\"\\e3f9\"}.bosch-ic-calendar-service:before{content:\"\\e3fa\"}.bosch-ic-camera-processor:before{content:\"\\e3fb\"}.bosch-ic-camera-settings:before{content:\"\\e3fc\"}.bosch-ic-chart-line-checkmark:before{content:\"\\e3fd\"}.bosch-ic-checklist-stack:before{content:\"\\e3fe\"}.bosch-ic-checkmark-frame-minimum:before{content:\"\\e3ff\"}.bosch-ic-checkmark-frame-tripple:before{content:\"\\e400\"}.bosch-ic-clipboard-checkmark:before{content:\"\\e401\"}.bosch-ic-clipboard-list-parts:before{content:\"\\e402\"}.bosch-ic-clipboard-settings:before{content:\"\\e403\"}.bosch-ic-computer-tower-box:before{content:\"\\e404\"}.bosch-ic-connectivity-lock:before{content:\"\\e405\"}.bosch-ic-counter-current:before{content:\"\\e406\"}.bosch-ic-counter-target:before{content:\"\\e407\"}.bosch-ic-cube-checkmark:before{content:\"\\e408\"}.bosch-ic-cube-play:before{content:\"\\e409\"}.bosch-ic-timeline-settings:before{content:\"\\e40a\"}.bosch-ic-timeline-view:before{content:\"\\e40b\"}.bosch-ic-timeline:before{content:\"\\e40c\"}.bosch-ic-delivery-checkmark:before{content:\"\\e40d\"}.bosch-ic-desktop-application-3d-play:before{content:\"\\e40e\"}.bosch-ic-desktop-apps-download:before{content:\"\\e40f\"}.bosch-ic-desktop-dashboard:before{content:\"\\e410\"}.bosch-ic-desktop-graph-checkmark:before{content:\"\\e411\"}.bosch-ic-desktop-graph-search:before{content:\"\\e412\"}.bosch-ic-desktop-graph:before{content:\"\\e413\"}.bosch-ic-development-kit:before{content:\"\\e414\"}.bosch-ic-directory-cloud-settings:before{content:\"\\e415\"}.bosch-ic-directory-cloud:before{content:\"\\e416\"}.bosch-ic-directory-settings:before{content:\"\\e417\"}.bosch-ic-document-clock-cycle:before{content:\"\\e418\"}.bosch-ic-document-code-stack:before{content:\"\\e419\"}.bosch-ic-document-dat-edit:before{content:\"\\e41a\"}.bosch-ic-document-ohd-arrow-down:before{content:\"\\e41b\"}.bosch-ic-dolphin:before{content:\"\\e41c\"}.bosch-ic-energy-management:before{content:\"\\e41d\"}.bosch-ic-eraser:before{content:\"\\e41e\"}.bosch-ic-folder-broom:before{content:\"\\e41f\"}.bosch-ic-folder-check-broom:before{content:\"\\e420\"}.bosch-ic-folder-checkmark:before{content:\"\\e421\"}.bosch-ic-folder-oes-check:before{content:\"\\e422\"}.bosch-ic-folder-open-service:before{content:\"\\e423\"}.bosch-ic-folder-reload:before{content:\"\\e424\"}.bosch-ic-hand-command:before{content:\"\\e425\"}.bosch-ic-heating-start:before{content:\"\\e426\"}.bosch-ic-hierarchy-refresh:before{content:\"\\e427\"}.bosch-ic-hierarchy-search:before{content:\"\\e428\"}.bosch-ic-hierarchy-settings:before{content:\"\\e429\"}.bosch-ic-hierarchy:before{content:\"\\e42a\"}.bosch-ic-hmi-desktop-settings:before{content:\"\\e42b\"}.bosch-ic-human-machine-interface:before{content:\"\\e42c\"}.bosch-ic-keys-user-access:before{content:\"\\e42d\"}.bosch-ic-keys:before{content:\"\\e42e\"}.bosch-ic-label-edit:before{content:\"\\e42f\"}.bosch-ic-line-arrow-checkmark:before{content:\"\\e430\"}.bosch-ic-link-checkmark:before{content:\"\\e431\"}.bosch-ic-lock-closed-checkmark:before{content:\"\\e432\"}.bosch-ic-lock-open-checkmark:before{content:\"\\e433\"}.bosch-ic-material-search:before{content:\"\\e434\"}.bosch-ic-objects-search:before{content:\"\\e435\"}.bosch-ic-objects:before{content:\"\\e436\"}.bosch-ic-plan-a-to-b:before{content:\"\\e437\"}.bosch-ic-plan-chart:before{content:\"\\e438\"}.bosch-ic-print-send:before{content:\"\\e439\"}.bosch-ic-refresh-analysis:before{content:\"\\e43a\"}.bosch-ic-remote-desktop-add:before{content:\"\\e43b\"}.bosch-ic-rfid-checkmark:before{content:\"\\e43c\"}.bosch-ic-rfid-tag:before{content:\"\\e43d\"}.bosch-ic-sequence-checkmark:before{content:\"\\e43e\"}.bosch-ic-server-arrow-up:before{content:\"\\e43f\"}.bosch-ic-server-arrows:before{content:\"\\e440\"}.bosch-ic-server-settings:before{content:\"\\e441\"}.bosch-ic-server-share:before{content:\"\\e442\"}.bosch-ic-settings-editor:before{content:\"\\e443\"}.bosch-ic-survey-checkmark:before{content:\"\\e444\"}.bosch-ic-target-dart:before{content:\"\\e445\"}.bosch-ic-target-view:before{content:\"\\e446\"}.bosch-ic-target:before{content:\"\\e447\"}.bosch-ic-thickness-selection:before{content:\"\\e448\"}.bosch-ic-toolbox-checkmark:before{content:\"\\e449\"}.bosch-ic-toolbox:before{content:\"\\e44a\"}.bosch-ic-translate:before{content:\"\\e44b\"}.bosch-ic-tune-vertical-checkmark:before{content:\"\\e44c\"}.bosch-ic-wand-user-add:before{content:\"\\e44d\"}.bosch-ic-wand-user-settings:before{content:\"\\e44e\"}.bosch-ic-wand-user:before{content:\"\\e44f\"}.bosch-ic-window-globe:before{content:\"\\e450\"}.bosch-ic-wrench-change:before{content:\"\\e451\"}.bosch-ic-clock-24-7:before{content:\"\\e452\"}.bosch-ic-eagle:before{content:\"\\e453\"}.bosch-ic-fluorescent-light:before{content:\"\\e454\"}.bosch-ic-home-cloud-0:before{content:\"\\e455\"}.bosch-ic-home-cloud-1:before{content:\"\\e456\"}.bosch-ic-home-cloud-2:before{content:\"\\e457\"}.bosch-ic-home-cloud-3:before{content:\"\\e458\"}.bosch-ic-home-shadow:before{content:\"\\e459\"}.bosch-ic-motorcycle-sport:before{content:\"\\e45a\"}.bosch-ic-smartphone-connected:before{content:\"\\e45b\"}.bosch-ic-usb-stick:before{content:\"\\e45c\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-timeline-clock:before{content:\"\\e45e\"}.bosch-ic-alarm-fire:before{content:\"\\e45f\"}.bosch-ic-emergency-front:before{content:\"\\e460\"}.bosch-ic-transporter-front:before{content:\"\\e461\"}.bosch-ic-truck-front:before{content:\"\\e462\"}.bosch-ic-unpin-classic-disabled-light:before{content:\"\\e463\"}.bosch-ic-unpin-modern-disabled-light:before{content:\"\\e464\"}.bosch-ic-smartphone-ban:before{content:\"\\e465\"}.bosch-ic-helmet:before{content:\"\\e466\"}.bosch-ic-gate-closed:before{content:\"\\e467\"}.bosch-ic-fence:before{content:\"\\e468\"}.bosch-ic-desktop-apps:before{content:\"\\e469\"}.bosch-ic-candle:before{content:\"\\e46a\"}.bosch-ic-abort-frame:before{content:\"\\e886\"}.bosch-ic-arrow-down-frame:before{content:\"\\e887\"}.bosch-ic-arrow-left-down-frame:before{content:\"\\e888\"}.bosch-ic-arrow-left-frame:before{content:\"\\e889\"}.bosch-ic-arrow-left-up-frame:before{content:\"\\e88a\"}.bosch-ic-arrow-right-down-frame:before{content:\"\\e88b\"}.bosch-ic-arrow-right-frame:before{content:\"\\e88c\"}.bosch-ic-arrow-right-up-frame:before{content:\"\\e88d\"}.bosch-ic-arrow-up-frame:before{content:\"\\e88e\"}.bosch-ic-asiapacific-frame:before{content:\"\\e88f\"}.bosch-ic-cash-frame:before{content:\"\\e890\"}.bosch-ic-checkmark-frame:before{content:\"\\e891\"}.bosch-ic-copyright-frame:before{content:\"\\e892\"}.bosch-ic-download-frame:before{content:\"\\e893\"}.bosch-ic-europe-frame:before{content:\"\\e894\"}.bosch-ic-lock-closed-frame:before{content:\"\\e895\"}.bosch-ic-lock-open-frame:before{content:\"\\e896\"}.bosch-ic-my-brand-frame:before{content:\"\\e897\"}.bosch-ic-northamerica-frame:before{content:\"\\e898\"}.bosch-ic-problem-frame:before{content:\"\\e899\"}.bosch-ic-promotion-frame:before{content:\"\\e89a\"}.bosch-ic-question-frame:before{content:\"\\e89b\"}.bosch-ic-share-frame:before{content:\"\\e89c\"}.bosch-ic-southamerica-frame:before{content:\"\\e89d\"}.bosch-ic-start-play-frame:before{content:\"\\e89e\"}.bosch-ic-upload-frame:before{content:\"\\e89f\"}.bosch-ic-world-frame:before{content:\"\\e8a0\"}.bosch-ic-add-frame:before{content:\"\\e8a1\"}.bosch-ic-call-deny-frame:before{content:\"\\e8a2\"}.bosch-ic-call-frame:before{content:\"\\e8a3\"}.bosch-ic-fast-backward-frame:before{content:\"\\e8a4\"}.bosch-ic-fast-forward-frame:before{content:\"\\e8a5\"}.bosch-ic-skip-backward-frame:before{content:\"\\e8a6\"}.bosch-ic-skip-fast-backward-frame:before{content:\"\\e8a7\"}.bosch-ic-skip-fast-forward-frame:before{content:\"\\e8a8\"}.bosch-ic-skip-forward-frame:before{content:\"\\e8a9\"}.bosch-ic-subtract-frame:before{content:\"\\e8aa\"}.bosch-ic-facebook-frame:before{content:\"\\ea87\"}.bosch-ic-googleplus-frame:before{content:\"\\ea88\"}.bosch-ic-instagram-frame:before{content:\"\\ea89\"}.bosch-ic-lineapp-frame:before{content:\"\\ea8a\"}.bosch-ic-linkedin-frame:before{content:\"\\ea8b\"}.bosch-ic-pinterest-frame:before{content:\"\\ea8c\"}.bosch-ic-snapchat-frame:before{content:\"\\ea8d\"}.bosch-ic-tumblr-frame:before{content:\"\\ea8e\"}.bosch-ic-twitter-frame:before{content:\"\\ea8f\"}.bosch-ic-vimeo-frame:before{content:\"\\ea90\"}.bosch-ic-wechat-frame:before{content:\"\\ea91\"}.bosch-ic-weibo-frame:before{content:\"\\ea92\"}.bosch-ic-whatsapp-frame:before{content:\"\\ea93\"}.bosch-ic-xing-frame:before{content:\"\\ea94\"}.bosch-ic-youku-frame:before{content:\"\\ea95\"}.bosch-ic-youtube-frame:before{content:\"\\ea96\"}.bosch-ic-vk-frame:before{content:\"\\ea97\"}.bosch-ic-skype-frame:before{content:\"\\ea98\"}.bosch-ic-alert-error-filled:before{content:\"\\eb08\"}.bosch-ic-alert-error:before{content:\"\\eb09\"}.bosch-ic-alert-success-filled:before{content:\"\\eb0a\"}.bosch-ic-alert-success:before{content:\"\\eb0b\"}.bosch-ic-alert-warning-filled:before{content:\"\\eb0c\"}.bosch-ic-alert-warning:before{content:\"\\eb0d\"}.bosch-ic-chart-oee:before{content:\"\\e46b\"}.bosch-ic-cycletime-eighth:before{content:\"\\e46c\"}.bosch-ic-cycletime-quarter:before{content:\"\\e46d\"}.bosch-ic-gateway:before{content:\"\\e46e\"}.bosch-ic-heatmap:before{content:\"\\e46f\"}.bosch-ic-laptop-checklist:before{content:\"\\e470\"}.bosch-ic-molecule:before{content:\"\\e471\"}.bosch-ic-robot-settings:before{content:\"\\e472\"}.bosch-ic-sensor-cable:before{content:\"\\e473\"}.bosch-ic-tank-layers:before{content:\"\\e474\"}.bosch-ic-window-settings:before{content:\"\\e475\"}.bosch-ic-heatmap-independent:before{content:\"\\e476\"}.bosch-ic-dumble:before{content:\"\\e477\"}.bosch-ic-truck-wash:before{content:\"\\e478\"}.bosch-ic-smart-sensor:before{content:\"\\e479\"}.bosch-ic-audio-add:before{content:\"\\e47a\"}.bosch-ic-audio-checked:before{content:\"\\e47b\"}.bosch-ic-audio-play:before{content:\"\\e47c\"}.bosch-ic-audio-remove:before{content:\"\\e47d\"}.bosch-ic-image-gallery-checked:before{content:\"\\e47e\"}.bosch-ic-imagery-checked:before{content:\"\\e47f\"}.bosch-ic-image-gallery-remove:before{content:\"\\e480\"}.bosch-ic-recording:before{content:\"\\e481\"}.bosch-ic-stop-frame:before{content:\"\\e482\"}.bosch-ic-roof:before{content:\"\\e483\"}.bosch-ic-projection-interactive:before{content:\"\\e484\"}.bosch-ic-bridge:before{content:\"\\e485\"}.bosch-ic-box-arrows:before{content:\"\\e486\"}.bosch-ic-chart-line-fit:before{content:\"\\e487\"}.bosch-ic-chart-line-compress:before{content:\"\\e488\"}.bosch-ic-ciss:before{content:\"\\e489\"}.bosch-ic-scd:before{content:\"\\e48a\"}.bosch-ic-vci:before{content:\"\\e48b\"}.bosch-ic-access-controller:before{content:\"\\e48c\"}.bosch-ic-card-reader:before{content:\"\\e48d\"}.bosch-ic-table:before{content:\"\\e48e\"}.bosch-ic-replace:before{content:\"\\e48f\"}.bosch-ic-chart-topflop-organized-horizontal:before{content:\"\\e490\"}.bosch-ic-chart-topflop-random-horizontal:before{content:\"\\e491\"}.bosch-ic-chart-topflop-organized-vertical:before{content:\"\\e492\"}.bosch-ic-chart-topflop-random-vertical:before{content:\"\\e493\"}.bosch-ic-ois:before{content:\"\\e494\"}.bosch-ic-cloud-refresh-locked:before{content:\"\\e495\"}.bosch-ic-car-bluetooth:before{content:\"\\e496\"}.bosch-ic-camera-trainer:before{content:\"\\e497\"}.bosch-ic-magnifier-moving:before{content:\"\\e498\"}.bosch-ic-absolute-orientation:before{content:\"\\e499\"}.bosch-ic-accelerometer:before{content:\"\\e49a\"}.bosch-ic-imu:before{content:\"\\e49b\"}.bosch-ic-gyroscope:before{content:\"\\e49c\"}.bosch-ic-magneotmeter:before{content:\"\\e49d\"}.bosch-ic-battery-car-1:before{content:\"\\e49e\"}.bosch-ic-battery-car-2:before{content:\"\\e49f\"}.bosch-ic-battery-car-3:before{content:\"\\e4a0\"}.bosch-ic-battery-car-charging:before{content:\"\\e4a1\"}.bosch-ic-battery-car-defective:before{content:\"\\e4a2\"}.bosch-ic-truck-crane:before{content:\"\\e4a3\"}.bosch-ic-train-diesel:before{content:\"\\e4a4\"}.bosch-ic-snowmobile:before{content:\"\\e4a5\"}.bosch-ic-riding-mower:before{content:\"\\e4a6\"}.bosch-ic-motorcycle-small:before{content:\"\\e4a7\"}.bosch-ic-mining-truck:before{content:\"\\e4a8\"}.bosch-ic-lawnmower:before{content:\"\\e4a9\"}.bosch-ic-jetski:before{content:\"\\e4aa\"}.bosch-ic-golf-caddy:before{content:\"\\e4ab\"}.bosch-ic-generator-electric:before{content:\"\\e4ac\"}.bosch-ic-garbage-truck:before{content:\"\\e4ad\"}.bosch-ic-fork-lift-big:before{content:\"\\e4ae\"}.bosch-ic-engine-small:before{content:\"\\e4af\"}.bosch-ic-engine-big:before{content:\"\\e4b0\"}.bosch-ic-coach:before{content:\"\\e4b1\"}.bosch-ic-chainsaw:before{content:\"\\e4b2\"}.bosch-ic-car-van:before{content:\"\\e4b3\"}.bosch-ic-car-suv:before{content:\"\\e4b4\"}.bosch-ic-car-race:before{content:\"\\e4b5\"}.bosch-ic-car-old:before{content:\"\\e4b6\"}.bosch-ic-car-camping-van:before{content:\"\\e4b7\"}.bosch-ic-car-camper:before{content:\"\\e4b8\"}.bosch-ic-bus-side:before{content:\"\\e4b9\"}.bosch-ic-book-info:before{content:\"\\e4ba\"}.bosch-ic-gears:before{content:\"\\e4bb\"}.bosch-ic-shark:before{content:\"\\e4bc\"}.bosch-ic-mountaintop:before{content:\"\\e4bd\"}.bosch-ic-baby:before{content:\"\\e4be\"}.bosch-ic-coffee-bean:before{content:\"\\e4bf\"}.bosch-ic-coffee-beans:before{content:\"\\e4c0\"}.bosch-ic-skull:before{content:\"\\e4c1\"}.bosch-ic-stroller:before{content:\"\\e4c2\"}.bosch-ic-glasses:before{content:\"\\e4c3\"}.bosch-ic-detergent:before{content:\"\\e4c4\"}.bosch-ic-washing-machine:before{content:\"\\e4c5\"}.bosch-ic-sdcard:before{content:\"\\e4c6\"}.bosch-ic-water-connectivity:before{content:\"\\e4c7\"}.bosch-ic-server-connectivity:before{content:\"\\e4c8\"}.bosch-ic-presentation-speaker:before{content:\"\\e4c9\"}.bosch-ic-chart-bar-arrow:before{content:\"\\e4ca\"}.bosch-ic-book-connectivity:before{content:\"\\e4cb\"}.bosch-ic-route-focus:before{content:\"\\e4cc\"}.bosch-ic-bicycle-helmet:before{content:\"\\e4cd\"}.bosch-ic-cookie:before{content:\"\\e4ce\"}.bosch-ic-glass-wine:before{content:\"\\e4cf\"}.bosch-ic-cup-small:before{content:\"\\e4d0\"}.bosch-ic-cup-large:before{content:\"\\e4d1\"}.bosch-ic-bowl:before{content:\"\\e4d2\"}.bosch-ic-glass-small:before{content:\"\\e4d3\"}.bosch-ic-ice-bottle:before{content:\"\\e4d4\"}.bosch-ic-cup-plate-drop:before{content:\"\\e4d5\"}.bosch-ic-plate:before{content:\"\\e4d6\"}.bosch-ic-cup-plate:before{content:\"\\e4d7\"}.bosch-ic-bubbles:before{content:\"\\e4d8\"}.bosch-ic-button-start-stop:before{content:\"\\e4d9\"}.bosch-ic-elevator-freight-max:before{content:\"\\e4da\"}.bosch-ic-elevator-freight:before{content:\"\\e4db\"}.bosch-ic-elevator-persons:before{content:\"\\e4dc\"}.bosch-ic-embed:before{content:\"\\e4dd\"}.bosch-ic-female-male:before{content:\"\\e4de\"}.bosch-ic-fork-knife-spoon:before{content:\"\\e4df\"}.bosch-ic-global-entrepreneur:before{content:\"\\e4e0\"}.bosch-ic-arrows-left-right-checkmark:before{content:\"\\e4e1\"}.bosch-ic-automatic-mode:before{content:\"\\e4e2\"}.bosch-ic-babybottle:before{content:\"\\e4e3\"}.bosch-ic-barcode-scan:before{content:\"\\e4e4\"}.bosch-ic-barcode-scanner:before{content:\"\\e4e5\"}.bosch-ic-cash-frame-rupee:before{content:\"\\e4e6\"}.bosch-ic-chart-heat-curve:before{content:\"\\e4e7\"}.bosch-ic-compressor:before{content:\"\\e4e8\"}.bosch-ic-cubes:before{content:\"\\e4e9\"}.bosch-ic-highway:before{content:\"\\e4ea\"}.bosch-ic-industry-workers:before{content:\"\\e4eb\"}.bosch-ic-industry-workers-connected:before{content:\"\\e4ec\"}.bosch-ic-machine-station-control:before{content:\"\\e4ed\"}.bosch-ic-map-city:before{content:\"\\e4ee\"}.bosch-ic-ordernumber:before{content:\"\\e4ef\"}.bosch-ic-shirt-business:before{content:\"\\e4f0\"}.bosch-ic-shirt-casual:before{content:\"\\e4f1\"}.bosch-ic-suitcase:before{content:\"\\e4f2\"}.bosch-ic-weather-cloud-sun-sensor:before{content:\"\\e4f3\"}.bosch-ic-window-code:before{content:\"\\e4f4\"}.bosch-ic-wrench-warning:before{content:\"\\e4f5\"}.bosch-ic-arrow-circle-a:before{content:\"\\e4f6\"}.bosch-ic-arrows-agility:before{content:\"\\e4f7\"}.bosch-ic-arrows-collapse-object:before{content:\"\\e4f8\"}.bosch-ic-arrows-expand-object:before{content:\"\\e4f9\"}.bosch-ic-arrows-left-right:before{content:\"\\e4fa\"}.bosch-ic-fan-heat:before{content:\"\\e4fb\"}.bosch-ic-battery-car-arrows-circle:before{content:\"\\e4fc\"}.bosch-ic-battery-car-clock:before{content:\"\\e4fd\"}.bosch-ic-battery-vibration:before{content:\"\\e4fe\"}.bosch-ic-belt-elastic:before{content:\"\\e4ff\"}.bosch-ic-bicycle-e-arrow-away:before{content:\"\\e500\"}.bosch-ic-bicycle-e-arrow-towards:before{content:\"\\e501\"}.bosch-ic-brake-decoupled-power:before{content:\"\\e502\"}.bosch-ic-brake-i-booster:before{content:\"\\e503\"}.bosch-ic-brake-system-esp:before{content:\"\\e504\"}.bosch-ic-brake-system-integrated:before{content:\"\\e505\"}.bosch-ic-brake-vacuum-booster:before{content:\"\\e506\"}.bosch-ic-capacity:before{content:\"\\e507\"}.bosch-ic-car-beam-length:before{content:\"\\e508\"}.bosch-ic-car-smartphone-locator:before{content:\"\\e509\"}.bosch-ic-chart-pie:before{content:\"\\e50a\"}.bosch-ic-chess-jumper:before{content:\"\\e50b\"}.bosch-ic-clock-time-off:before{content:\"\\e50c\"}.bosch-ic-clock-time-on:before{content:\"\\e50d\"}.bosch-ic-coin:before{content:\"\\e50e\"}.bosch-ic-connection-off-settings:before{content:\"\\e50f\"}.bosch-ic-connection-on-settings:before{content:\"\\e510\"}.bosch-ic-control-unit:before{content:\"\\e511\"}.bosch-ic-customer:before{content:\"\\e512\"}.bosch-ic-engine:before{content:\"\\e513\"}.bosch-ic-filter-air:before{content:\"\\e514\"}.bosch-ic-filter-cabin-carbon:before{content:\"\\e515\"}.bosch-ic-filter-cabin:before{content:\"\\e516\"}.bosch-ic-filter-cooling-water:before{content:\"\\e517\"}.bosch-ic-filter-denox-cover:before{content:\"\\e518\"}.bosch-ic-filter-denox:before{content:\"\\e519\"}.bosch-ic-filter-element-compensation-sealing:before{content:\"\\e51a\"}.bosch-ic-filter-element-compensation:before{content:\"\\e51b\"}.bosch-ic-filter-element:before{content:\"\\e51c\"}.bosch-ic-filter-oil:before{content:\"\\e51d\"}.bosch-ic-filter-petrol:before{content:\"\\e51e\"}.bosch-ic-filter-prefilter:before{content:\"\\e51f\"}.bosch-ic-fireplace:before{content:\"\\e520\"}.bosch-ic-flash-plus:before{content:\"\\e521\"}.bosch-ic-globe-worldwideweb:before{content:\"\\e522\"}.bosch-ic-glow-control-unit:before{content:\"\\e523\"}.bosch-ic-glow-plug:before{content:\"\\e524\"}.bosch-ic-hand-cloth:before{content:\"\\e525\"}.bosch-ic-leaf-plus:before{content:\"\\e526\"}.bosch-ic-machine-warning:before{content:\"\\e527\"}.bosch-ic-measuring-jug:before{content:\"\\e528\"}.bosch-ic-plug-play:before{content:\"\\e529\"}.bosch-ic-refresh-checkmark:before{content:\"\\e52a\"}.bosch-ic-replacement-filament:before{content:\"\\e52b\"}.bosch-ic-screw:before{content:\"\\e52c\"}.bosch-ic-seat-belt:before{content:\"\\e52d\"}.bosch-ic-steering-wheel-hands-automated:before{content:\"\\e52e\"}.bosch-ic-sun-ice-parking:before{content:\"\\e52f\"}.bosch-ic-syringe-change-needle:before{content:\"\\e530\"}.bosch-ic-syringe-change:before{content:\"\\e531\"}.bosch-ic-syringe:before{content:\"\\e532\"}.bosch-ic-thread-diameter:before{content:\"\\e533\"}.bosch-ic-tool-extraction:before{content:\"\\e534\"}.bosch-ic-transporter-car-front:before{content:\"\\e535\"}.bosch-ic-truck-car-motorcycle:before{content:\"\\e536\"}.bosch-ic-truck-car:before{content:\"\\e537\"}.bosch-ic-truck-logistics-box:before{content:\"\\e538\"}.bosch-ic-truck-transporter:before{content:\"\\e539\"}.bosch-ic-watch-on-warning:before{content:\"\\e53a\"}.bosch-ic-wrench-calibrate:before{content:\"\\e53b\"}.bosch-ic-wrench-dummy:before{content:\"\\e53c\"}.bosch-ic-wrench-tightening-torque:before{content:\"\\e53d\"}.bosch-ic-wrench-width-across-flats:before{content:\"\\e53e\"}.bosch-ic-user-arm-bandage:before{content:\"\\e53f\"}.bosch-ic-user-eye-blind:before{content:\"\\e540\"}.bosch-ic-user-ear-deaf:before{content:\"\\e541\"}.bosch-ic-user-car-distracted:before{content:\"\\e542\"}.bosch-ic-user-mouth-mute:before{content:\"\\e543\"}.bosch-ic-user-ear-blocked:before{content:\"\\e544\"}.bosch-ic-user-bavarian:before{content:\"\\e545\"}.bosch-ic-user-ear-loud:before{content:\"\\e546\"}.bosch-ic-user-walk-crutch:before{content:\"\\e547\"}.bosch-ic-user-mouth-speechless:before{content:\"\\e548\"}.bosch-ic-user-arm-baby:before{content:\"\\e549\"}.bosch-ic-user-arm-missing:before{content:\"\\e54a\"}.bosch-ic-user-eye-eyepatch:before{content:\"\\e54b\"}.bosch-ic-user-strapped:before{content:\"\\e54c\"}.bosch-ic-cleaning-agent:before{content:\"\\e54d\"}.bosch-ic-cup-plate-dirty:before{content:\"\\e54e\"}.bosch-ic-cup-plate-private:before{content:\"\\e54f\"}.bosch-ic-rubbish-apple:before{content:\"\\e550\"}.bosch-ic-rubbish-fish:before{content:\"\\e551\"}.bosch-ic-rubbish-organic-bin:before{content:\"\\e552\"}.bosch-ic-rubbish-paper:before{content:\"\\e553\"}.bosch-ic-rubbish-paper-bin:before{content:\"\\e554\"}.bosch-ic-rubbish-residual-bin:before{content:\"\\e555\"}.bosch-ic-shower:before{content:\"\\e556\"}.bosch-ic-shower-female:before{content:\"\\e557\"}.bosch-ic-shower-male:before{content:\"\\e558\"}.bosch-ic-wardrobe-female:before{content:\"\\e559\"}.bosch-ic-wardrobe-male:before{content:\"\\e55a\"}.bosch-ic-car-top:before{content:\"\\e55b\"}.bosch-ic-label-info:before{content:\"\\e55c\"}.bosch-ic-perspective-view:before{content:\"\\e55d\"}.bosch-ic-road:before{content:\"\\e55e\"}.bosch-ic-screen-split-horizontal:before{content:\"\\e55f\"}.bosch-ic-screen-split-vertical:before{content:\"\\e560\"}.bosch-ic-screenshot-frame:before{content:\"\\e561\"}.bosch-ic-sensor-lidar:before{content:\"\\e562\"}.bosch-ic-tram:before{content:\"\\e563\"}.bosch-ic-video-360-degrees:before{content:\"\\e564\"}.bosch-ic-belt-groove-drives:before{content:\"\\e565\"}.bosch-ic-bottle-champagne:before{content:\"\\e566\"}.bosch-ic-box-falling:before{content:\"\\e567\"}.bosch-ic-box-falling-off-disabled-bold:before{content:\"\\e568\"}.bosch-ic-box-falling-off-disabled-light:before{content:\"\\e569\"}.bosch-ic-brake-pedal:before{content:\"\\e56a\"}.bosch-ic-car-front-light-inside:before{content:\"\\e56b\"}.bosch-ic-car-licence-plate-light:before{content:\"\\e56c\"}.bosch-ic-car-side-light-back:before{content:\"\\e56d\"}.bosch-ic-car-side-light-front:before{content:\"\\e56e\"}.bosch-ic-light-fog-warning:before{content:\"\\e56f\"}.bosch-ic-prism:before{content:\"\\e570\"}.bosch-ic-turnstile:before{content:\"\\e571\"}.bosch-ic-turnstile-open-in-and-outbound:before{content:\"\\e572\"}.bosch-ic-turnstile-open-inbound:before{content:\"\\e573\"}.bosch-ic-turnstile-open-outbound:before{content:\"\\e574\"}.bosch-ic-uwb:before{content:\"\\e575\"}.bosch-ic-heat-reduce-active:before{content:\"\\e576\"}.bosch-ic-heat-reduce-passive:before{content:\"\\e577\"}.bosch-ic-hourglass-add:before{content:\"\\e578\"}.bosch-ic-light-fog:before{content:\"\\e579\"}.bosch-ic-car-turn-signal:before{content:\"\\e57a\"}.bosch-ic-car-turn-signal-left:before{content:\"\\e57b\"}.bosch-ic-car-turn-signal-right:before{content:\"\\e57c\"}.bosch-ic-light-rear:before{content:\"\\e57d\"}.bosch-ic-car-top-turn-signal-front:before{content:\"\\e57e\"}.bosch-ic-car-top-turn-signal-back:before{content:\"\\e57f\"}.bosch-ic-ccu-cm:before{content:\"\\e580\"}.bosch-ic-lung:before{content:\"\\e581\"}.bosch-ic-head-speaking-voice:before{content:\"\\e582\"}.bosch-ic-car-motorcycle:before{content:\"\\e583\"}.bosch-ic-car-turn-signal-side:before{content:\"\\e584\"}.bosch-ic-car-light-parking:before{content:\"\\e585\"}.bosch-ic-phoenix:before{content:\"\\e586\"}.bosch-ic-sensor-air-quality:before{content:\"\\e587\"}.bosch-ic-weather-station:before{content:\"\\e588\"}.bosch-ic-watering-can:before{content:\"\\e589\"}.bosch-ic-propeller:before{content:\"\\e58a\"}.bosch-ic-temperature-fluctuating:before{content:\"\\e58b\"}.bosch-ic-stove-top:before{content:\"\\e58c\"}.bosch-ic-socket:before{content:\"\\e58d\"}.bosch-ic-siren:before{content:\"\\e58e\"}.bosch-ic-sensor-water:before{content:\"\\e58f\"}.bosch-ic-user-run-sensor-motion:before{content:\"\\e590\"}.bosch-ic-robot-lawnmower:before{content:\"\\e591\"}.bosch-ic-robot-cleaning:before{content:\"\\e592\"}.bosch-ic-oven:before{content:\"\\e593\"}.bosch-ic-loudspeaker:before{content:\"\\e594\"}.bosch-ic-keypad:before{content:\"\\e595\"}.bosch-ic-shape-circle-square:before{content:\"\\e596\"}.bosch-ic-garage:before{content:\"\\e597\"}.bosch-ic-fume-hood:before{content:\"\\e598\"}.bosch-ic-freezer:before{content:\"\\e599\"}.bosch-ic-food-processor:before{content:\"\\e59a\"}.bosch-ic-floorplan-rooms:before{content:\"\\e59b\"}.bosch-ic-dryer-tumble:before{content:\"\\e59c\"}.bosch-ic-dryer:before{content:\"\\e59d\"}.bosch-ic-door-window:before{content:\"\\e59e\"}.bosch-ic-door-lock:before{content:\"\\e59f\"}.bosch-ic-door-bell:before{content:\"\\e5a0\"}.bosch-ic-dishwasher:before{content:\"\\e5a1\"}.bosch-ic-dimmer:before{content:\"\\e5a2\"}.bosch-ic-detector-smoke:before{content:\"\\e5a3\"}.bosch-ic-detector-presence:before{content:\"\\e5a4\"}.bosch-ic-cooking:before{content:\"\\e5a5\"}.bosch-ic-coffee-maschine:before{content:\"\\e5a6\"}.bosch-ic-camera-indoor:before{content:\"\\e5a7\"}.bosch-ic-broom-clean:before{content:\"\\e5a8\"}.bosch-ic-blinds:before{content:\"\\e5a9\"}.bosch-ic-bicycle-e-flash:before{content:\"\\e5aa\"}.bosch-ic-finger-domino-trigger:before{content:\"\\e5ab\"}.bosch-ic-automation-start:before{content:\"\\e5ac\"}.bosch-ic-automation:before{content:\"\\e5ad\"}.bosch-ic-sensor-multifunctional:before{content:\"\\e5ae\"}.bosch-ic-plane-top:before{content:\"\\e5af\"}.bosch-ic-shape-square-overlapping:before{content:\"\\e5b0\"}.bosch-ic-square-overlapping:before{content:\"\\e5b1\"}.bosch-ic-battery-car-14v:before{content:\"\\e5b2\"}.bosch-ic-calibration:before{content:\"\\e5b3\"}.bosch-ic-structure-can-bus:before{content:\"\\e5b4\"}.bosch-ic-gears-direction-arrows:before{content:\"\\e5b5\"}.bosch-ic-car-limousine:before{content:\"\\e5b6\"}.bosch-ic-molecule-gas:before{content:\"\\e5b7\"}.bosch-ic-money-euro:before{content:\"\\e5b8\"}.bosch-ic-revolution-rpm:before{content:\"\\e5b9\"}.bosch-ic-rikscha-e:before{content:\"\\e5ba\"}.bosch-ic-sensor-circle:before{content:\"\\e5bb\"}.bosch-ic-speedometer-high:before{content:\"\\e5bc\"}.bosch-ic-steering-wheel-arrows:before{content:\"\\e5bd\"}.bosch-ic-tablet-checklist:before{content:\"\\e5be\"}.bosch-ic-petrol-charging-station-hybrid:before{content:\"\\e5bf\"}.bosch-ic-weather-cloud-sun-rain-snow:before{content:\"\\e5c0\"}.bosch-ic-light-half:before{content:\"\\e5c1\"}.bosch-ic-video-sensor:before{content:\"\\e5c2\"}.bosch-ic-binary-desktop:before{content:\"\\e5c3\"}.bosch-ic-truck-delivery:before{content:\"\\e5c4\"}.bosch-ic-van-delivery:before{content:\"\\e5c5\"}.bosch-ic-fullscreen-user:before{content:\"\\e5c6\"}.bosch-ic-user-helmet-motorcycle:before{content:\"\\e5c7\"}.bosch-ic-user-hand-disabled-light:before{content:\"\\e5c8\"}.bosch-ic-user-hand-disabled-bold:before{content:\"\\e5c9\"}.bosch-ic-link-connected:before{content:\"\\e5ca\"}.bosch-ic-gate-open:before{content:\"\\e5cb\"}.bosch-ic-box-checkmark:before{content:\"\\e5cc\"}.bosch-ic-box-place:before{content:\"\\e5cd\"}.bosch-ic-logistics-dock:before{content:\"\\e5ce\"}.bosch-ic-barcode-scan-logistics:before{content:\"\\e5cf\"}.bosch-ic-locator-tracking:before{content:\"\\e5d0\"}.bosch-ic-car-light-trunk:before{content:\"\\e5d1\"}.bosch-ic-car-light-glove-compartment:before{content:\"\\e5d2\"}.bosch-ic-car-light-dashboard:before{content:\"\\e5d3\"}.bosch-ic-fire-leaf:before{content:\"\\e5d4\"}.bosch-ic-laptop-hacker:before{content:\"\\e5d5\"}.bosch-ic-hands-drops:before{content:\"\\e5d6\"}.bosch-ic-team-disabled-bold:before{content:\"\\e5d7\"}.bosch-ic-team-disabled-light:before{content:\"\\e5d8\"}.bosch-ic-user-head-arm:before{content:\"\\e5d9\"}.bosch-ic-user-head-hand-disabled-bold:before{content:\"\\e5da\"}.bosch-ic-user-head-hand-disabled-light:before{content:\"\\e5db\"}.bosch-ic-welcome-disabled-bold:before{content:\"\\e5dc\"}.bosch-ic-welcome-disabled-light:before{content:\"\\e5dd\"}.bosch-ic-female-male-arrows:before{content:\"\\e5de\"}.bosch-ic-broccoli:before{content:\"\\e5df\"}.bosch-ic-chicken:before{content:\"\\e5e0\"}.bosch-ic-cow:before{content:\"\\e5e1\"}.bosch-ic-deer:before{content:\"\\e5e2\"}.bosch-ic-fish:before{content:\"\\e5e3\"}.bosch-ic-garlic:before{content:\"\\e5e4\"}.bosch-ic-hotpepper:before{content:\"\\e5e5\"}.bosch-ic-pig:before{content:\"\\e5e6\"}.bosch-ic-rabbit:before{content:\"\\e5e7\"}.bosch-ic-sheep:before{content:\"\\e5e8\"}.bosch-ic-shell:before{content:\"\\e5e9\"}.bosch-ic-shrimp:before{content:\"\\e5ea\"}.bosch-ic-car-seat:before{content:\"\\e5eb\"}.bosch-ic-car-seat-steering-wheel:before{content:\"\\e5ec\"}.bosch-ic-car-side-arrow-back:before{content:\"\\e5ed\"}.bosch-ic-car-side-arrow-front:before{content:\"\\e5ee\"}.bosch-ic-car-steering-left-disabled-bold:before{content:\"\\e5ef\"}.bosch-ic-car-steering-left-disabled-light:before{content:\"\\e5f0\"}.bosch-ic-car-steering-left-lle:before{content:\"\\e5f1\"}.bosch-ic-car-steering-left-lle-disabled-bold:before{content:\"\\e5f2\"}.bosch-ic-car-steering-left-lle-disabled-light:before{content:\"\\e5f3\"}.bosch-ic-car-steering-right-disabled-bold:before{content:\"\\e5f4\"}.bosch-ic-car-steering-right-disabled-light:before{content:\"\\e5f5\"}.bosch-ic-car-steering-right-rle:before{content:\"\\e5f6\"}.bosch-ic-car-steering-right-rle-disabled-bold:before{content:\"\\e5f7\"}.bosch-ic-car-steering-right-rle-disabled-light:before{content:\"\\e5f8\"}.bosch-ic-cars-traffic-left-right:before{content:\"\\e5f9\"}.bosch-ic-configuration-wrench:before{content:\"\\e5fa\"}.bosch-ic-hook:before{content:\"\\e5fb\"}.bosch-ic-hook-arrows:before{content:\"\\e5fc\"}.bosch-ic-hook-arrows-wiperblade:before{content:\"\\e5fd\"}.bosch-ic-hook-disabled-bold:before{content:\"\\e5fe\"}.bosch-ic-hook-disabled-light:before{content:\"\\e5ff\"}.bosch-ic-info-i-frame-add:before{content:\"\\e600\"}.bosch-ic-search-number:before{content:\"\\e601\"}.bosch-ic-search-number-add:before{content:\"\\e602\"}.bosch-ic-set:before{content:\"\\e603\"}.bosch-ic-speedometer-speed-high:before{content:\"\\e604\"}.bosch-ic-windshield-wiper-clean:before{content:\"\\e605\"}.bosch-ic-windshield-wiper-double:before{content:\"\\e606\"}.bosch-ic-wiper-adapter:before{content:\"\\e607\"}.bosch-ic-wiper-blade:before{content:\"\\e608\"}.bosch-ic-wiper-blade-arrow:before{content:\"\\e609\"}.bosch-ic-wiper-blade-eco:before{content:\"\\e60a\"}.bosch-ic-wiper-blade-mm:before{content:\"\\e60b\"}.bosch-ic-wiper-blade-mm-arrows:before{content:\"\\e60c\"}.bosch-ic-wiper-blade-spoiler:before{content:\"\\e60d\"}.bosch-ic-wiper-blade-twin:before{content:\"\\e60e\"}.bosch-ic-wiper-flatblade:before{content:\"\\e60f\"}.bosch-ic-wiper-flatblade-aeroeco:before{content:\"\\e610\"}.bosch-ic-wiper-flatblade-aerofit:before{content:\"\\e611\"}.bosch-ic-wiper-flatblade-aerotwin:before{content:\"\\e612\"}.bosch-ic-wiper-flatblade-arrow:before{content:\"\\e613\"}.bosch-ic-wiper-flatblade-mm:before{content:\"\\e614\"}.bosch-ic-wiper-flatblade-mm-arrows:before{content:\"\\e615\"}.bosch-ic-wiper-rubber:before{content:\"\\e616\"}.bosch-ic-address-control-system:before{content:\"\\e617\"}.bosch-ic-balcony:before{content:\"\\e618\"}.bosch-ic-battery-car-45-degree-arrow:before{content:\"\\e619\"}.bosch-ic-boiler-warning:before{content:\"\\e61a\"}.bosch-ic-book-wrench:before{content:\"\\e61b\"}.bosch-ic-car-driver-passenger-arrows:before{content:\"\\e61c\"}.bosch-ic-car-licence-plate:before{content:\"\\e61d\"}.bosch-ic-car-side-arrow-back-tires:before{content:\"\\e61e\"}.bosch-ic-car-side-arrow-front-tires:before{content:\"\\e61f\"}.bosch-ic-car-side-half-arrow-back:before{content:\"\\e620\"}.bosch-ic-car-steering-left-arrow:before{content:\"\\e621\"}.bosch-ic-car-steering-left-passenger-arrow:before{content:\"\\e622\"}.bosch-ic-car-steering-right-arrow:before{content:\"\\e623\"}.bosch-ic-car-steering-right-passenger-arrow:before{content:\"\\e624\"}.bosch-ic-chess-board-transparency:before{content:\"\\e625\"}.bosch-ic-cloud-locked:before{content:\"\\e626\"}.bosch-ic-conference-system:before{content:\"\\e627\"}.bosch-ic-cycletime-quarter-arrows-interval:before{content:\"\\e628\"}.bosch-ic-desktop-connectivity:before{content:\"\\e629\"}.bosch-ic-hierarchy-checkmark:before{content:\"\\e62a\"}.bosch-ic-home-office:before{content:\"\\e62b\"}.bosch-ic-home-semi-detached-house:before{content:\"\\e62c\"}.bosch-ic-kitchen:before{content:\"\\e62d\"}.bosch-ic-lambda-arrows-length:before{content:\"\\e62e\"}.bosch-ic-lambda-oxygen:before{content:\"\\e62f\"}.bosch-ic-lambda-sensor-cores:before{content:\"\\e630\"}.bosch-ic-panel-control:before{content:\"\\e631\"}.bosch-ic-prison:before{content:\"\\e632\"}.bosch-ic-prison-cell:before{content:\"\\e633\"}.bosch-ic-radio-broadcasting:before{content:\"\\e634\"}.bosch-ic-robot-body:before{content:\"\\e635\"}.bosch-ic-robot-body-arrow:before{content:\"\\e636\"}.bosch-ic-skyscraper-danger:before{content:\"\\e637\"}.bosch-ic-skyscraper-flat:before{content:\"\\e638\"}.bosch-ic-skyscraper-flat-floor-low:before{content:\"\\e639\"}.bosch-ic-skyscraper-flat-floor-middle:before{content:\"\\e63a\"}.bosch-ic-skyscraper-flat-floor-top:before{content:\"\\e63b\"}.bosch-ic-skyscraper-flat-semi-detached:before{content:\"\\e63c\"}.bosch-ic-skyscraper-locked:before{content:\"\\e63d\"}.bosch-ic-skyscraper-settings:before{content:\"\\e63e\"}.bosch-ic-team-signpost:before{content:\"\\e63f\"}.bosch-ic-toaster-hand-warning:before{content:\"\\e640\"}.bosch-ic-user-arrows-heart:before{content:\"\\e641\"}.bosch-ic-user-graduate-chat:before{content:\"\\e642\"}.bosch-ic-user-head-mask:before{content:\"\\e643\"}.bosch-ic-warehouse:before{content:\"\\e644\"}.bosch-ic-world-paperplane:before{content:\"\\e645\"}.bosch-ic-aperture:before{content:\"\\e646\"}.bosch-ic-hook-wiperblade-arrows:before{content:\"\\e647\"}.bosch-ic-image-remove:before{content:\"\\e648\"}.bosch-ic-image-checked:before{content:\"\\e649\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-apartment-in-house-terraced:before{content:\"\\e64b\"}.bosch-ic-apartment-in-house:before{content:\"\\e64c\"}.bosch-ic-appliance-heat-pump-leaf-add:before{content:\"\\e64d\"}.bosch-ic-appliance-heat-pump-leaf-reload:before{content:\"\\e64e\"}.bosch-ic-appliance-heat-pump-oil-refresh:before{content:\"\\e64f\"}.bosch-ic-appliance-water-drop:before{content:\"\\e650\"}.bosch-ic-appliance-water-sun:before{content:\"\\e651\"}.bosch-ic-brake-pedal-thumb-up:before{content:\"\\e652\"}.bosch-ic-call-plus:before{content:\"\\e653\"}.bosch-ic-battery-car-3-plug:before{content:\"\\e654\"}.bosch-ic-cloud-share:before{content:\"\\e655\"}.bosch-ic-crankshaft:before{content:\"\\e656\"}.bosch-ic-detector-motion:before{content:\"\\e657\"}.bosch-ic-device-analog:before{content:\"\\e658\"}.bosch-ic-device-digital:before{content:\"\\e659\"}.bosch-ic-externallink-landscape:before{content:\"\\e65a\"}.bosch-ic-gears-interlocked-arrow:before{content:\"\\e65b\"}.bosch-ic-gearshift-reverse-gear:before{content:\"\\e65c\"}.bosch-ic-glove:before{content:\"\\e65d\"}.bosch-ic-hand-warranty:before{content:\"\\e65e\"}.bosch-ic-heating-floor-radiator:before{content:\"\\e65f\"}.bosch-ic-home-appliance-heat-pump-air-in:before{content:\"\\e660\"}.bosch-ic-home-appliance-heat-pump-air-out:before{content:\"\\e661\"}.bosch-ic-lightbulb-gear:before{content:\"\\e662\"}.bosch-ic-line-open-closed:before{content:\"\\e663\"}.bosch-ic-road-disabled-bold:before{content:\"\\e664\"}.bosch-ic-road-disabled-light:before{content:\"\\e665\"}.bosch-ic-shower-drop-1:before{content:\"\\e666\"}.bosch-ic-shower-drop-2:before{content:\"\\e667\"}.bosch-ic-shower-drop-3:before{content:\"\\e668\"}.bosch-ic-shower-drop-4:before{content:\"\\e669\"}.bosch-ic-sinus-curve:before{content:\"\\e66a\"}.bosch-ic-team-3:before{content:\"\\e66b\"}.bosch-ic-team-4:before{content:\"\\e66c\"}.bosch-ic-team-5:before{content:\"\\e66d\"}.bosch-ic-test-tube:before{content:\"\\e66e\"}.bosch-ic-truck-car-motorcycle-arrows:before{content:\"\\e66f\"}.bosch-ic-weather-cloud-dust:before{content:\"\\e670\"}.bosch-ic-windshield-wiper-double-parallel-left:before{content:\"\\e671\"}.bosch-ic-windshield-wiper-double-parallel-right:before{content:\"\\e672\"}.bosch-ic-world-pin:before{content:\"\\e673\"}.bosch-ic-sign-warning-slope:before{content:\"\\e674\"}.bosch-ic-facebook:before{content:\"\\ea99\"}.bosch-ic-google-frame:before{content:\"\\ea9a\"}.bosch-ic-google:before{content:\"\\ea9b\"}.bosch-ic-googleplus:before{content:\"\\ea9c\"}.bosch-ic-instagram:before{content:\"\\ea9d\"}.bosch-ic-lineapp:before{content:\"\\ea9e\"}.bosch-ic-linkedin:before{content:\"\\ea9f\"}.bosch-ic-pinterest:before{content:\"\\eaa0\"}.bosch-ic-skype:before{content:\"\\eaa1\"}.bosch-ic-snapchat:before{content:\"\\eaa2\"}.bosch-ic-tumblr:before{content:\"\\eaa3\"}.bosch-ic-twitter:before{content:\"\\eaa4\"}.bosch-ic-vimeo:before{content:\"\\eaa5\"}.bosch-ic-vk:before{content:\"\\eaa6\"}.bosch-ic-wechat:before{content:\"\\eaa7\"}.bosch-ic-weibo:before{content:\"\\eaa8\"}.bosch-ic-whatsapp:before{content:\"\\eaa9\"}.bosch-ic-xing:before{content:\"\\eaaa\"}.bosch-ic-youku:before{content:\"\\eaab\"}.bosch-ic-youtube:before{content:\"\\eaac\"}.bosch-ic-alert-info-filled:before{content:\"\\eb0e\"}.bosch-ic-alert-info:before{content:\"\\eb0f\"}.bosch-ic-info-i-frame:before{content:\"\\e8ab\"}.bosch-ic-stop-frame1:before{content:\"\\e8ac\"}.bosch-ic-arrow-up-frame-error1:before{content:\"\\e8ad\"}.bosch-ic-arrow-up-frame-check1:before{content:\"\\e8ae\"}.bosch-ic-arrow-left-end-frame1:before{content:\"\\e8af\"}.bosch-ic-arrow-right-end-frame1:before{content:\"\\e8b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-warranty:before{content:\"\\e2b1\"}.bosch-ic-bond-laser:before{content:\"\\e675\"}.bosch-ic-bond-ribbon-thick:before{content:\"\\e676\"}.bosch-ic-bond-ribbon-thin:before{content:\"\\e677\"}.bosch-ic-bond-wire-thick:before{content:\"\\e678\"}.bosch-ic-bond-wire-thin:before{content:\"\\e679\"}.bosch-ic-clamp-connection-insulation:before{content:\"\\e67a\"}.bosch-ic-clamp-connection-stamping-grid:before{content:\"\\e67b\"}.bosch-ic-clinching:before{content:\"\\e67c\"}.bosch-ic-connector:before{content:\"\\e67d\"}.bosch-ic-connector-potting:before{content:\"\\e67e\"}.bosch-ic-element-damping:before{content:\"\\e67f\"}.bosch-ic-element-pressure-compensation:before{content:\"\\e680\"}.bosch-ic-flanging:before{content:\"\\e681\"}.bosch-ic-gap-pad:before{content:\"\\e682\"}.bosch-ic-heat-sink:before{content:\"\\e683\"}.bosch-ic-hook-clipsing:before{content:\"\\e684\"}.bosch-ic-interconnection-board-to-board:before{content:\"\\e685\"}.bosch-ic-labyrinth-geometry:before{content:\"\\e686\"}.bosch-ic-lacquering-pcb:before{content:\"\\e687\"}.bosch-ic-magnetic-rotor:before{content:\"\\e688\"}.bosch-ic-magnetic-switch:before{content:\"\\e689\"}.bosch-ic-pin-pressfit:before{content:\"\\e68a\"}.bosch-ic-power-contact:before{content:\"\\e68b\"}.bosch-ic-rivet:before{content:\"\\e68c\"}.bosch-ic-seal-ring:before{content:\"\\e68d\"}.bosch-ic-sealing-cipg:before{content:\"\\e68e\"}.bosch-ic-sealing-solid:before{content:\"\\e68f\"}.bosch-ic-sealing-tongue-groove:before{content:\"\\e690\"}.bosch-ic-shrinking-arrows-circle:before{content:\"\\e691\"}.bosch-ic-solder-selective:before{content:\"\\e692\"}.bosch-ic-spring-mechanical:before{content:\"\\e693\"}.bosch-ic-staking-hot-air:before{content:\"\\e694\"}.bosch-ic-stitching:before{content:\"\\e695\"}.bosch-ic-thermal-inlay:before{content:\"\\e696\"}.bosch-ic-thermal-via:before{content:\"\\e697\"}.bosch-ic-tube:before{content:\"\\e698\"}.bosch-ic-tube-gapfiller:before{content:\"\\e699\"}.bosch-ic-tube-gelling:before{content:\"\\e69a\"}.bosch-ic-tube-gluing:before{content:\"\\e69b\"}.bosch-ic-tube-thermal-adhesive:before{content:\"\\e69c\"}.bosch-ic-tube-underfill:before{content:\"\\e69d\"}.bosch-ic-welding-laser:before{content:\"\\e69e\"}.bosch-ic-welding-resistance:before{content:\"\\e69f\"}.bosch-ic-welding-ultrasonic:before{content:\"\\e6a0\"}.bosch-ic-air-condition-cassette:before{content:\"\\e6a1\"}.bosch-ic-air-condition-ceiling:before{content:\"\\e6a2\"}.bosch-ic-air-condition-convertible:before{content:\"\\e6a3\"}.bosch-ic-air-condition-duct:before{content:\"\\e6a4\"}.bosch-ic-air-condition-floor:before{content:\"\\e6a5\"}.bosch-ic-air-condition-outdoor:before{content:\"\\e6a6\"}.bosch-ic-air-condition-outdoor-hp:before{content:\"\\e6a7\"}.bosch-ic-air-condition-outdoor-hr:before{content:\"\\e6a8\"}.bosch-ic-air-condition-swing-horizontal:before{content:\"\\e6a9\"}.bosch-ic-air-condition-swing-vertical:before{content:\"\\e6aa\"}.bosch-ic-air-condition-wall:before{content:\"\\e6ab\"}.bosch-ic-angle-curvature-motorcycle:before{content:\"\\e6ac\"}.bosch-ic-appliance-hot:before{content:\"\\e6ad\"}.bosch-ic-appliance-water-hot:before{content:\"\\e6ae\"}.bosch-ic-appliances-hot:before{content:\"\\e6af\"}.bosch-ic-arrows-left-right-a:before{content:\"\\e6b0\"}.bosch-ic-arrows-left-right-exchange:before{content:\"\\e6b1\"}.bosch-ic-arrows-left-right-exchange-unit:before{content:\"\\e6b2\"}.bosch-ic-arrows-left-right-ice:before{content:\"\\e6b3\"}.bosch-ic-assembly-line-robot:before{content:\"\\e6b4\"}.bosch-ic-back-left-small:before{content:\"\\e6b5\"}.bosch-ic-bacteria:before{content:\"\\e6b6\"}.bosch-ic-battery-car-drop-leakage-proof:before{content:\"\\e6b7\"}.bosch-ic-battery-car-hourglass:before{content:\"\\e6b8\"}.bosch-ic-brake-disk-arrow:before{content:\"\\e6b9\"}.bosch-ic-brake-disk-compound:before{content:\"\\e6ba\"}.bosch-ic-building-religious-church:before{content:\"\\e6bb\"}.bosch-ic-calendar-sheet:before{content:\"\\e6bc\"}.bosch-ic-caliper:before{content:\"\\e6bd\"}.bosch-ic-car-middle-arrow:before{content:\"\\e6be\"}.bosch-ic-checkmark-bold:before{content:\"\\e6bf\"}.bosch-ic-close-small:before{content:\"\\e6c0\"}.bosch-ic-data-center:before{content:\"\\e6c1\"}.bosch-ic-dot:before{content:\"\\e6c2\"}.bosch-ic-down-small:before{content:\"\\e6c3\"}.bosch-ic-e-call:before{content:\"\\e6c4\"}.bosch-ic-fire-extinguisher:before{content:\"\\e6c5\"}.bosch-ic-fire-off-bold:before{content:\"\\e6c6\"}.bosch-ic-fire-off-light:before{content:\"\\e6c7\"}.bosch-ic-forward-right-small:before{content:\"\\e6c8\"}.bosch-ic-health-warning:before{content:\"\\e6c9\"}.bosch-ic-ignition-coil-pencil:before{content:\"\\e6ca\"}.bosch-ic-ignition-condensor:before{content:\"\\e6cb\"}.bosch-ic-ignition-contact-set:before{content:\"\\e6cc\"}.bosch-ic-ignition-distributor-cap:before{content:\"\\e6cd\"}.bosch-ic-ignition-distributor-rotor:before{content:\"\\e6ce\"}.bosch-ic-ignition-module:before{content:\"\\e6cf\"}.bosch-ic-less-minimize-bold:before{content:\"\\e6d0\"}.bosch-ic-leaf-plus-boost:before{content:\"\\e6d1\"}.bosch-ic-sensor-dynamic:before{content:\"\\e6d2\"}.bosch-ic-sensor-radar:before{content:\"\\e6d3\"}.bosch-ic-up-small:before{content:\"\\e6d4\"}.bosch-ic-water-ladder-off-bold:before{content:\"\\e6d5\"}.bosch-ic-water-ladder-off-light:before{content:\"\\e6d6\"}.bosch-ic-nose-desodorize:before{content:\"\\e6d7\"}.bosch-ic-nose-fine-dust:before{content:\"\\e6d8\"}.bosch-ic-smartcity-camera:before{content:\"\\e6d9\"}.bosch-ic-heating-underfloor:before{content:\"\\e6da\"}.bosch-ic-drill-bit:before{content:\"\\e6db\"}.bosch-ic-parameter-process:before{content:\"\\e6dc\"}.bosch-ic-car-side-plug-flash:before{content:\"\\e6dd\"}.bosch-ic-car-side-plug-engine-flash:before{content:\"\\e6de\"}.bosch-ic-car-side-engine-flash:before{content:\"\\e6df\"}.bosch-ic-car-side-hydrogen-flash:before{content:\"\\e6e0\"}.bosch-ic-car-side-engine:before{content:\"\\e6e1\"}.bosch-ic-tunnel:before{content:\"\\e6e2\"}.bosch-ic-stadium-sport:before{content:\"\\e6e3\"}.bosch-ic-refinery-oil:before{content:\"\\e6e4\"}.bosch-ic-power-plants:before{content:\"\\e6e5\"}.bosch-ic-hammer-chisel:before{content:\"\\e6e6\"}.bosch-ic-glass-cocktail:before{content:\"\\e6e7\"}.bosch-ic-traffic-light:before{content:\"\\e6e8\"}.bosch-ic-vehicle-off-road:before{content:\"\\e6e9\"}.bosch-ic-tablet-finger:before{content:\"\\e6ea\"}.bosch-ic-scooter-kick:before{content:\"\\e6eb\"}.bosch-ic-handlebar-motorcycle:before{content:\"\\e6ec\"}.bosch-ic-display-motorcycle:before{content:\"\\e6ed\"}.bosch-ic-case-label-product:before{content:\"\\e6ee\"}.bosch-ic-case-label:before{content:\"\\e6ef\"}.bosch-ic-product-falling:before{content:\"\\e6f0\"}.bosch-ic-product-checkmark:before{content:\"\\e6f1\"}.bosch-ic-product-broken:before{content:\"\\e6f2\"}.bosch-ic-lightbulb-halogen-rays-bright:before{content:\"\\e6f3\"}.bosch-ic-lightbulb-halogen-rays:before{content:\"\\e6f4\"}.bosch-ic-lamp-gas-discharge-rays:before{content:\"\\e6f5\"}.bosch-ic-home-lightbulb-off-disabled-bold:before{content:\"\\e6f6\"}.bosch-ic-home-lightbulb-off-disabled-light:before{content:\"\\e6f7\"}.bosch-ic-fork-lift-boxes:before{content:\"\\e6f8\"}.bosch-ic-harbour-ship-cargo:before{content:\"\\e6f9\"}.bosch-ic-rollercoaster:before{content:\"\\e6fa\"}.bosch-ic-sound-off-light:before{content:\"\\e6fb\"}.bosch-ic-sound-off-bold:before{content:\"\\e6fc\"}.bosch-ic-car-door:before{content:\"\\e6fd\"}.bosch-ic-car-door-tailgate:before{content:\"\\e6fe\"}.bosch-ic-car-door-split-rear:before{content:\"\\e6ff\"}.bosch-ic-car-door-rear:before{content:\"\\e700\"}.bosch-ic-drill-driver-cordless:before{content:\"\\e701\"}.bosch-ic-virus:before{content:\"\\e702\"}.bosch-ic-ice-temperature:before{content:\"\\e703\"}.bosch-ic-sensor-ultrasonic:before{content:\"\\e704\"}.bosch-ic-user-artificial-intelligence:before{content:\"\\e705\"}.bosch-ic-detector-leak:before{content:\"\\e706\"}.bosch-ic-car-seat-add:before{content:\"\\e707\"}.bosch-ic-car-shuttle-robo:before{content:\"\\e708\"}.bosch-ic-caravan:before{content:\"\\e709\"}.bosch-ic-user-water-swimming:before{content:\"\\e70a\"}.bosch-ic-square-add:before{content:\"\\e70b\"}.bosch-ic-map-zoom-in:before{content:\"\\e70c\"}.bosch-ic-map-arrow-pop-out:before{content:\"\\e70d\"}.bosch-ic-home-user-in-error:before{content:\"\\e70e\"}.bosch-ic-home-arrow-right-in-denied:before{content:\"\\e70f\"}.bosch-ic-home-arrow-right-in-check:before{content:\"\\e710\"}.bosch-ic-home-arrow-right-in:before{content:\"\\e711\"}.bosch-ic-home-arrow-left-out:before{content:\"\\e712\"}.bosch-ic-file-error:before{content:\"\\e713\"}.bosch-ic-dealer-details-missing:before{content:\"\\e714\"}.bosch-ic-dealer-details-check:before{content:\"\\e715\"}.bosch-ic-components-add:before{content:\"\\e716\"}.bosch-ic-machine-cell:before{content:\"\\e717\"}.bosch-ic-wheel-rim-clean:before{content:\"\\e718\"}.bosch-ic-gas-can:before{content:\"\\e719\"}.bosch-ic-steps-locator:before{content:\"\\e71a\"}.bosch-ic-panel-control-manual:before{content:\"\\e71b\"}.bosch-ic-socket-europe:before{content:\"\\e71c\"}.bosch-ic-tripod:before{content:\"\\e71d\"}.bosch-ic-document-dxf:before{content:\"\\e71e\"}.bosch-ic-document-dwg:before{content:\"\\e71f\"}.bosch-ic-hook-wiperblade:before{content:\"\\e720\"}.bosch-ic-traffic-cone:before{content:\"\\e721\"}.bosch-ic-folder-locked:before{content:\"\\e722\"}.bosch-ic-package-self-service:before{content:\"\\e723\"}.bosch-ic-drop-ice:before{content:\"\\e724\"}.bosch-ic-lamp-gas-discharge:before{content:\"\\e725\"}.bosch-ic-home-basement:before{content:\"\\e726\"}.bosch-ic-link-edit:before{content:\"\\e727\"}.bosch-ic-table-header-add:before{content:\"\\e728\"}.bosch-ic-table-column-add-after:before{content:\"\\e729\"}.bosch-ic-table-row-add-above:before{content:\"\\e72a\"}.bosch-ic-table-column-add-before:before{content:\"\\e72b\"}.bosch-ic-table-row-add-below:before{content:\"\\e72c\"}.bosch-ic-table-cells-split:before{content:\"\\e72d\"}.bosch-ic-table-cells-merge:before{content:\"\\e72e\"}.bosch-ic-table-row-delete:before{content:\"\\e72f\"}.bosch-ic-text-indent-arrow:before{content:\"\\e730\"}.bosch-ic-text-outdent-arrow:before{content:\"\\e731\"}.bosch-ic-car-side-flash:before{content:\"\\e732\"}.bosch-ic-battery-plug-flash:before{content:\"\\e733\"}.bosch-ic-car-chip:before{content:\"\\e734\"}.bosch-ic-car-cloud-upload:before{content:\"\\e735\"}.bosch-ic-circle-parking:before{content:\"\\e736\"}.bosch-ic-engine-battery-flash:before{content:\"\\e737\"}.bosch-ic-engine-battery-flash-plug:before{content:\"\\e738\"}.bosch-ic-engine-combustion:before{content:\"\\e739\"}.bosch-ic-user-head-scan:before{content:\"\\e73a\"}.bosch-ic-bluetooth-off-disabled-bold:before{content:\"\\e73b\"}.bosch-ic-bluetooth-off-disabled-light:before{content:\"\\e73c\"}.bosch-ic-communicator-off-disabled-bold:before{content:\"\\e73d\"}.bosch-ic-communicator-off-disabled-light:before{content:\"\\e73e\"}.bosch-ic-connectivity-off-disabled-bold:before{content:\"\\e73f\"}.bosch-ic-connectivity-off-disabled-light:before{content:\"\\e740\"}.bosch-ic-globe-off-disabled-bold:before{content:\"\\e741\"}.bosch-ic-globe-off-disabled-light:before{content:\"\\e742\"}.bosch-ic-lamp-led:before{content:\"\\e743\"}.bosch-ic-wifi-off-disabled-bold:before{content:\"\\e744\"}.bosch-ic-wifi-off-disabled-light:before{content:\"\\e745\"}.bosch-ic-wheel-vehicle:before{content:\"\\e746\"}.bosch-ic-ignition-cable:before{content:\"\\e747\"}.bosch-ic-ignition-coil:before{content:\"\\e748\"}.bosch-ic-ignition-distributor:before{content:\"\\e749\"}.bosch-ic-microphone-classic-off-disabled-light:before{content:\"\\e74a\"}.bosch-ic-microphone-classic-off-disabled-bold:before{content:\"\\e74b\"}.bosch-ic-cloud-refresh:before{content:\"\\e74c\"}.bosch-ic-ignition-coil-module:before{content:\"\\e74d\"}.bosch-ic-bluetooth-frame:before{content:\"\\e8b1\"}.bosch-ic-bluetooth-frame-error:before{content:\"\\e8b2\"}.bosch-ic-ece-ccc-frame:before{content:\"\\e8b3\"}.bosch-ic-ms-teams:before{content:\"\\eaad\"}.bosch-ic-ms-teams-frame:before{content:\"\\eaae\"}.bosch-ic-mold:before{content:\"\\f0000\"}.bosch-ic-battery-car-cranking-power-high:before{content:\"\\f0001\"}.bosch-ic-brake-disk-spray-can-coated:before{content:\"\\f0002\"}.bosch-ic-car-top-arrows-left-front-right-back:before{content:\"\\f0003\"}.bosch-ic-car-top-arrows-left-back-right-front:before{content:\"\\f0004\"}.bosch-ic-car-top-arrows-front:before{content:\"\\f0005\"}.bosch-ic-car-top-arrows-back-front-left-right:before{content:\"\\f0006\"}.bosch-ic-car-top-arrows-back:before{content:\"\\f0007\"}.bosch-ic-car-top-arrow-right:before{content:\"\\f0008\"}.bosch-ic-car-top-arrow-left:before{content:\"\\f0009\"}.bosch-ic-car-top-arrow-front-right:before{content:\"\\f000a\"}.bosch-ic-car-top-arrow-front-left:before{content:\"\\f000b\"}.bosch-ic-car-top-arrow-back-right:before{content:\"\\f000c\"}.bosch-ic-car-top-arrow-back-left:before{content:\"\\f000d\"}.bosch-ic-watermelon:before{content:\"\\f000e\"}.bosch-ic-wind-vane-moderate:before{content:\"\\f000f\"}.bosch-ic-radioactive:before{content:\"\\f0010\"}.bosch-ic-heating-underfloor-off-bold:before{content:\"\\f0011\"}.bosch-ic-heating-underfloor-off-light:before{content:\"\\f0012\"}.bosch-ic-goat:before{content:\"\\f0013\"}.bosch-ic-fire-hose:before{content:\"\\f0014\"}.bosch-ic-user-run-arrow:before{content:\"\\f0015\"}.bosch-ic-store:before{content:\"\\f0016\"}.bosch-ic-user-work-dig:before{content:\"\\f0017\"}.bosch-ic-car-top-arrow-angle-right:before{content:\"\\f0018\"}.bosch-ic-control-unit-ecu:before{content:\"\\f0019\"}.bosch-ic-cigarette:before{content:\"\\f001a\"}.bosch-ic-structure-line:before{content:\"\\f001b\"}.bosch-ic-product-database-table:before{content:\"\\f001c\"}.bosch-ic-bicycle-e-remote-led:before{content:\"\\f001d\"}.bosch-ic-bicycle-e-drive-unit:before{content:\"\\f001e\"}.bosch-ic-bicycle-e-drive-unit-individual-support:before{content:\"\\f001f\"}.bosch-ic-nm-arrow-circle:before{content:\"\\f0020\"}.bosch-ic-tree-path:before{content:\"\\f0021\"}.bosch-ic-bicycle-e-wheel-arrows:before{content:\"\\f0022\"}.bosch-ic-tune-vertical:before{content:\"\\f0023\"}.bosch-ic-sound-professional:before{content:\"\\f0024\"}.bosch-ic-plane-top-headset:before{content:\"\\f0025\"}.bosch-ic-desktop-user-headset:before{content:\"\\f0026\"}.bosch-ic-user-service:before{content:\"\\f0027\"}.bosch-ic-audio-commercial:before{content:\"\\f0028\"}.bosch-ic-loudspeaker-portable-note:before{content:\"\\f0029\"}.bosch-ic-audio-loudspeaker-public:before{content:\"\\f002a\"}.bosch-ic-items-lost-questionmark:before{content:\"\\f002b\"}.bosch-ic-hand-pull:before{content:\"\\f002c\"}.bosch-ic-steering-wheel-hand-horn:before{content:\"\\f002d\"}.bosch-ic-car-chassis:before{content:\"\\f002e\"}.bosch-ic-steering-system:before{content:\"\\f002f\"}.bosch-ic-wheel-suspension:before{content:\"\\f0030\"}.bosch-ic-car-body-side:before{content:\"\\f0031\"}.bosch-ic-address-consumer-data-upright:before{content:\"\\f0032\"}.bosch-ic-weather-wind-rain:before{content:\"\\f0033\"}.bosch-ic-weather-rain-warning:before{content:\"\\f0034\"}.bosch-ic-user-check:before{content:\"\\f0035\"}.bosch-ic-user-warning:before{content:\"\\f0036\"}.bosch-ic-black-white-stars-circle:before{content:\"\\f0037\"}.bosch-ic-team-warning:before{content:\"\\f0038\"}.bosch-ic-black-white-circle:before{content:\"\\f0039\"}.bosch-ic-suitcase-small:before{content:\"\\f003a\"}.bosch-ic-suitcase-small-remove:before{content:\"\\f003b\"}.bosch-ic-home-arrow-out:before{content:\"\\f003c\"}.bosch-ic-home-arrow-in:before{content:\"\\f003d\"}.bosch-ic-document-copy-check:before{content:\"\\f003e\"}.bosch-ic-ring-infrared:before{content:\"\\f003f\"}.bosch-ic-spirit-level:before{content:\"\\f0040\"}.bosch-ic-drill-driver-check:before{content:\"\\f0041\"}.bosch-ic-desktop-led:before{content:\"\\f0042\"}.bosch-ic-desktop-black-white:before{content:\"\\f0043\"}.bosch-ic-desktop-warning:before{content:\"\\f0044\"}.bosch-ic-fire-wall:before{content:\"\\f0045\"}.bosch-ic-onroad-car-top:before{content:\"\\f0046\"}.bosch-ic-onroad-suv-top:before{content:\"\\f0047\"}.bosch-ic-onroad-truck-top:before{content:\"\\f0048\"}.bosch-ic-weather-rain-ip44:before{content:\"\\f0049\"}.bosch-ic-weather-rain-ip65:before{content:\"\\f004a\"}.bosch-ic-weather-rain-ip66:before{content:\"\\f004b\"}.bosch-ic-weather-rain-ip68:before{content:\"\\f004c\"}.bosch-ic-weather-rain-ip67:before{content:\"\\f004d\"}.bosch-ic-user-masking:before{content:\"\\f004e\"}.bosch-ic-address-consumer-data-dual:before{content:\"\\f004f\"}.bosch-ic-user-framed:before{content:\"\\f0050\"}.bosch-ic-user-counting-1:before{content:\"\\f0051\"}.bosch-ic-square-kx-uhd:before{content:\"\\f0052\"}.bosch-ic-square-qvga:before{content:\"\\f0053\"}.bosch-ic-square-vga:before{content:\"\\f0054\"}.bosch-ic-square-xmp:before{content:\"\\f0055\"}.bosch-ic-square-xxxp:before{content:\"\\f0056\"}.bosch-ic-angle-arrows-across:before{content:\"\\f0057\"}.bosch-ic-angle:before{content:\"\\f0058\"}.bosch-ic-angle-arrow-up:before{content:\"\\f0059\"}.bosch-ic-angle-view-wide:before{content:\"\\f005a\"}.bosch-ic-chat-language-selection:before{content:\"\\f005b\"}.bosch-ic-smartphone-app:before{content:\"\\f005c\"}.bosch-ic-commissioning-remote:before{content:\"\\f005d\"}.bosch-ic-user-run-detection-pir:before{content:\"\\f005e\"}.bosch-ic-field-in-motion:before{content:\"\\f005f\"}.bosch-ic-building-integration-system:before{content:\"\\f0060\"}.bosch-ic-desktop-mamangement:before{content:\"\\f0061\"}.bosch-ic-desktop-management-camera:before{content:\"\\f0062\"}.bosch-ic-screwdriver-check:before{content:\"\\f0063\"}.bosch-ic-screwdriver:before{content:\"\\f0064\"}.bosch-ic-desktop-hand-touch:before{content:\"\\f0065\"}.bosch-ic-user-robot-head:before{content:\"\\f0066\"}.bosch-ic-gear-circles-elements:before{content:\"\\f0067\"}.bosch-ic-gears-3:before{content:\"\\f0068\"}.bosch-ic-door-address-consumer-data:before{content:\"\\f0069\"}.bosch-ic-lock-closed-128bit:before{content:\"\\f006a\"}.bosch-ic-lock-closed-connected-wps:before{content:\"\\f006b\"}.bosch-ic-battery-arrow-right:before{content:\"\\f006c\"}.bosch-ic-connector-bnc:before{content:\"\\f006d\"}.bosch-ic-tower-connection-cdma:before{content:\"\\f006e\"}.bosch-ic-tower-connection-gprs:before{content:\"\\f006f\"}.bosch-ic-tritech:before{content:\"\\f0070\"}.bosch-ic-quad-sensor-two:before{content:\"\\f0071\"}.bosch-ic-dynamic-range-hdr:before{content:\"\\f0072\"}.bosch-ic-dynamic-range-wdr:before{content:\"\\f0073\"}.bosch-ic-battery-thumb-up-arrow-right:before{content:\"\\f0074\"}.bosch-ic-screwdriver-thumb-up:before{content:\"\\f0075\"}.bosch-ic-thumb-up-arrow-up:before{content:\"\\f0076\"}.bosch-ic-core-data-sql:before{content:\"\\f0077\"}.bosch-ic-frequency-869:before{content:\"\\f0078\"}.bosch-ic-coins:before{content:\"\\f0079\"}.bosch-ic-frames-60:before{content:\"\\f007a\"}.bosch-ic-hand-card:before{content:\"\\f007b\"}.bosch-ic-microphone-plugabble:before{content:\"\\f007c\"}.bosch-ic-video-h264:before{content:\"\\f007d\"}.bosch-ic-video-hd:before{content:\"\\f007e\"}.bosch-ic-video-mpeg:before{content:\"\\f007f\"}.bosch-ic-video-hd-4k:before{content:\"\\f0080\"}.bosch-ic-video-hd-720:before{content:\"\\f0081\"}.bosch-ic-lasso:before{content:\"\\f0082\"}.bosch-ic-lasso-remove:before{content:\"\\f0083\"}.bosch-ic-battery-car-3-clock:before{content:\"\\f0084\"}.bosch-ic-battery-car-agm:before{content:\"\\f0085\"}.bosch-ic-battery-car-flash:before{content:\"\\f0086\"}.bosch-ic-filter-binary:before{content:\"\\f0087\"}.bosch-ic-speedometer-bitrate-reduce:before{content:\"\\f0088\"}.bosch-ic-voting:before{content:\"\\f0089\"}.bosch-ic-voting-dual:before{content:\"\\f008a\"}.bosch-ic-sound-pressure-level-90:before{content:\"\\f008b\"}.bosch-ic-sound-pressure-level-88:before{content:\"\\f008c\"}.bosch-ic-sound-pressure-level-80:before{content:\"\\f008d\"}.bosch-ic-power-rated-20:before{content:\"\\f008e\"}.bosch-ic-labyrinth-geometry-integration:before{content:\"\\f008f\"}.bosch-ic-head-speaking-voice-2:before{content:\"\\f0090\"}.bosch-ic-volume-microphone:before{content:\"\\f0091\"}.bosch-ic-user-walk-route:before{content:\"\\f0092\"}.bosch-ic-user-walk-line-crossing:before{content:\"\\f0093\"}.bosch-ic-user-walk-line-count:before{content:\"\\f0094\"}.bosch-ic-fullscreen-arrows-circle:before{content:\"\\f0095\"}.bosch-ic-field-arrows-counter-flow:before{content:\"\\f0096\"}.bosch-ic-field-arrows-directed-flow:before{content:\"\\f0097\"}.bosch-ic-user-birds-eye-count-12:before{content:\"\\f0098\"}.bosch-ic-field-user-count-12:before{content:\"\\f0099\"}.bosch-ic-field-user-falling:before{content:\"\\f009a\"}.bosch-ic-field-user-loitering-arrow:before{content:\"\\f009b\"}.bosch-ic-field-user-similarity:before{content:\"\\f009c\"}.bosch-ic-field-user-walk:before{content:\"\\f009d\"}.bosch-ic-field-user-walk-arrow-behind:before{content:\"\\f009e\"}.bosch-ic-field-user-walk-arrow-in-front:before{content:\"\\f009f\"}.bosch-ic-field-users:before{content:\"\\f00a0\"}.bosch-ic-shock-resistant:before{content:\"\\f00a1\"}.bosch-ic-bus-sdi:before{content:\"\\f00a2\"}.bosch-ic-bus-sdi2:before{content:\"\\f00a3\"}.bosch-ic-folder-ip-net:before{content:\"\\f00a4\"}.bosch-ic-globe-iscsi:before{content:\"\\f00a5\"}.bosch-ic-offline-www:before{content:\"\\f00a6\"}.bosch-ic-opc:before{content:\"\\f00a7\"}.bosch-ic-osdp:before{content:\"\\f00a8\"}.bosch-ic-settings-connected-ptrz:before{content:\"\\f00a9\"}.bosch-ic-raid:before{content:\"\\f00aa\"}.bosch-ic-lsn:before{content:\"\\f00ab\"}.bosch-ic-pointbus:before{content:\"\\f00ac\"}.bosch-ic-male-count-12:before{content:\"\\f00ad\"}.bosch-ic-circle-point-record:before{content:\"\\f00ae\"}.bosch-ic-plug-poe:before{content:\"\\f00af\"}.bosch-ic-ir-distribution:before{content:\"\\f00b0\"}.bosch-ic-ir-range:before{content:\"\\f00b1\"}.bosch-ic-connectivity-it:before{content:\"\\f00b2\"}.bosch-ic-music-note-plus:before{content:\"\\f00b3\"}.bosch-ic-music-note-plus-two:before{content:\"\\f00b4\"}.bosch-ic-chat-language-chn:before{content:\"\\f00b5\"}.bosch-ic-microphone-high-directive:before{content:\"\\f00b6\"}.bosch-ic-document-copy-rg-gf:before{content:\"\\f00b7\"}.bosch-ic-document-copy-cdq:before{content:\"\\f00b8\"}.bosch-ic-document-copy-cgp:before{content:\"\\f00b9\"}.bosch-ic-folder-add:before{content:\"\\f00ba\"}.bosch-ic-user-tie:before{content:\"\\f00bb\"}.bosch-ic-pill:before{content:\"\\f00bc\"}.bosch-ic-sonotrode:before{content:\"\\f00bd\"}.bosch-ic-user-thumb-up:before{content:\"\\f00be\"}.bosch-ic-architecture:before{content:\"\\f00bf\"}.bosch-ic-clamping-area-arrows:before{content:\"\\f00c0\"}.bosch-ic-clamping-rib:before{content:\"\\f00c1\"}.bosch-ic-connection-plug-in:before{content:\"\\f00c2\"}.bosch-ic-connection-plug-in-electrical:before{content:\"\\f00c3\"}.bosch-ic-connection-plug-in-socket:before{content:\"\\f00c4\"}.bosch-ic-connector-shield-emc:before{content:\"\\f00c5\"}.bosch-ic-foil-flex:before{content:\"\\f00c6\"}.bosch-ic-grid-fan:before{content:\"\\f00c7\"}.bosch-ic-screw-nut:before{content:\"\\f00c8\"}.bosch-ic-sealing-groove:before{content:\"\\f00c9\"}.bosch-ic-sealing-tongue-groove-flash:before{content:\"\\f00ca\"}.bosch-ic-solder:before{content:\"\\f00cb\"}.bosch-ic-solder-selective-ssl:before{content:\"\\f00cc\"}.bosch-ic-solder-selective-thrs:before{content:\"\\f00cd\"}.bosch-ic-tuner-box:before{content:\"\\f00ce\"}.bosch-ic-water-cooling-arrow-up:before{content:\"\\f00cf\"}.bosch-ic-generic-device-connected:before{content:\"\\f00d0\"}.bosch-ic-home-sun-flash:before{content:\"\\f00d1\"}.bosch-ic-home-sun-wrench:before{content:\"\\f00d2\"}.bosch-ic-temperature-lock:before{content:\"\\f00d3\"}.bosch-ic-solder-method-smd:before{content:\"\\f00d4\"}.bosch-ic-pins:before{content:\"\\f00d5\"}.bosch-ic-connector-housing:before{content:\"\\f00d6\"}.bosch-ic-antenna-cover-arrow-up:before{content:\"\\f00d7\"}.bosch-ic-angle-open-horizontal-130:before{content:\"\\f00d8\"}.bosch-ic-angle-open-vertical-130:before{content:\"\\f00d9\"}.bosch-ic-angle-open-vertical-90:before{content:\"\\f00da\"}.bosch-ic-angle-open-horizontal-145:before{content:\"\\f00db\"}.bosch-ic-steering-system-steer-by-wire:before{content:\"\\f00dc\"}.bosch-ic-tune-vertical-music-note:before{content:\"\\f00dd\"}.bosch-ic-hammer-movement:before{content:\"\\f00de\"}.bosch-ic-drill-drive-percent-35:before{content:\"\\f00df\"}.bosch-ic-country-switzerland:before{content:\"\\f00e0\"}.bosch-ic-speedlimit-revoking:before{content:\"\\f00e1\"}.bosch-ic-speedlimit-80-revoking:before{content:\"\\f00e2\"}.bosch-ic-speedlimit-80-dirty:before{content:\"\\f00e3\"}.bosch-ic-battery-car-mf:before{content:\"\\f00e4\"}.bosch-ic-access-controller-2:before{content:\"\\f00e5\"}.bosch-ic-access-controller-arrow-up:before{content:\"\\f00e6\"}.bosch-ic-access-controller-points-set:before{content:\"\\f00e7\"}.bosch-ic-analog-value:before{content:\"\\f00e8\"}.bosch-ic-analog-value-arrow-in:before{content:\"\\f00e9\"}.bosch-ic-analog-value-arrow-out:before{content:\"\\f00ea\"}.bosch-ic-binary-value:before{content:\"\\f00eb\"}.bosch-ic-binary-value-arrow-in:before{content:\"\\f00ec\"}.bosch-ic-binary-value-arrow-out:before{content:\"\\f00ed\"}.bosch-ic-calendar-date-single:before{content:\"\\f00ee\"}.bosch-ic-calendar-date-weekend:before{content:\"\\f00ef\"}.bosch-ic-calendar-view:before{content:\"\\f00f0\"}.bosch-ic-chart-average:before{content:\"\\f00f1\"}.bosch-ic-chart-line-arrows-up:before{content:\"\\f00f2\"}.bosch-ic-configuration-points-set:before{content:\"\\f00f3\"}.bosch-ic-data-points-set:before{content:\"\\f00f4\"}.bosch-ic-document-points-set:before{content:\"\\f00f5\"}.bosch-ic-multistate-value:before{content:\"\\f00f6\"}.bosch-ic-multistate-value-arrow-in:before{content:\"\\f00f7\"}.bosch-ic-multistate-value-arrow-out:before{content:\"\\f00f8\"}.bosch-ic-structure-bacnet:before{content:\"\\f00f9\"}.bosch-ic-structure-sbus:before{content:\"\\f00fa\"}.bosch-ic-team-arrows-in:before{content:\"\\f00fb\"}.bosch-ic-import-points-set:before{content:\"\\f00fc\"}.bosch-ic-image-points-set:before{content:\"\\f00fd\"}.bosch-ic-circuit-hydraulic:before{content:\"\\f00fe\"}.bosch-ic-structure-area:before{content:\"\\f00ff\"}.bosch-ic-structure-calculate:before{content:\"\\f0100\"}.bosch-ic-structure-clipboard:before{content:\"\\f0101\"}.bosch-ic-structure-collection:before{content:\"\\f0102\"}.bosch-ic-structure-components:before{content:\"\\f0103\"}.bosch-ic-structure-device:before{content:\"\\f0104\"}.bosch-ic-structure-dragdrop:before{content:\"\\f0105\"}.bosch-ic-structure-floor:before{content:\"\\f0106\"}.bosch-ic-structure-organization:before{content:\"\\f0107\"}.bosch-ic-structure-questionmark:before{content:\"\\f0108\"}.bosch-ic-structure-settings:before{content:\"\\f0109\"}.bosch-ic-structure-skyscraper:before{content:\"\\f010a\"}.bosch-ic-structure-window-console:before{content:\"\\f010b\"}.bosch-ic-structure-wrench:before{content:\"\\f010c\"}.bosch-ic-car-pickup:before{content:\"\\f010d\"}.bosch-ic-desktop-team:before{content:\"\\f010e\"}.bosch-ic-keyboard-arrow-enter-return:before{content:\"\\f010f\"}.bosch-ic-keyboard-arrow-left:before{content:\"\\f0110\"}.bosch-ic-keyboard-arrow-right-tab:before{content:\"\\f0111\"}.bosch-ic-keyboard-capslock:before{content:\"\\f0112\"}.bosch-ic-keyboard-shift:before{content:\"\\f0113\"}.bosch-ic-keyboard-space:before{content:\"\\f0114\"}.bosch-ic-bluetooth-le-off-disabled-light:before{content:\"\\f0115\"}.bosch-ic-structure-label-edit:before{content:\"\\f0116\"}.bosch-ic-structure-server:before{content:\"\\f0117\"}.bosch-ic-truck-box:before{content:\"\\f0118\"}.bosch-ic-imagery-off-disabled-light:before{content:\"\\f0119\"}.bosch-ic-imagery-off-disabled-bold:before{content:\"\\f011a\"}.bosch-ic-plan-floorplan-edit:before{content:\"\\f011b\"}.bosch-ic-desktop-building-ruler-pen:before{content:\"\\f011c\"}.bosch-ic-conference-system-connected:before{content:\"\\f011d\"}.bosch-ic-receiver-pocket-headphones:before{content:\"\\f011e\"}.bosch-ic-conference-system-microphone:before{content:\"\\f011f\"}.bosch-ic-user-carry-box:before{content:\"\\f0120\"}.bosch-ic-user-handcart:before{content:\"\\f0121\"}.bosch-ic-user-carry-box-2:before{content:\"\\f0122\"}.bosch-ic-locator-error:before{content:\"\\f0123\"}.bosch-ic-locator-offline:before{content:\"\\f0124\"}.bosch-ic-locator-refresh:before{content:\"\\f0125\"}.bosch-ic-chart-bar-plug-arrow-up:before{content:\"\\f0126\"}.bosch-ic-clipboard-search-leaf:before{content:\"\\f0127\"}.bosch-ic-handover-leaf:before{content:\"\\f0128\"}.bosch-ic-home-sun-plug:before{content:\"\\f0129\"}.bosch-ic-lightbulb-plug-leaf:before{content:\"\\f012a\"}.bosch-ic-wheel-water-plug:before{content:\"\\f012b\"}.bosch-ic-windmill-plug:before{content:\"\\f012c\"}.bosch-ic-windmill-plug-sun-water:before{content:\"\\f012d\"}.bosch-ic-scale-co2:before{content:\"\\f012e\"}.bosch-ic-certificat-co2:before{content:\"\\f012f\"}.bosch-ic-car-crash:before{content:\"\\f0130\"}.bosch-ic-headphones:before{content:\"\\f0131\"}.bosch-ic-music-note:before{content:\"\\f0132\"}.bosch-ic-clef-treble:before{content:\"\\f0133\"}.bosch-ic-clef-bass:before{content:\"\\f0134\"}.bosch-ic-volume:before{content:\"\\f0135\"}.bosch-ic-volume-increase:before{content:\"\\f0136\"}.bosch-ic-head-speach-speaking:before{content:\"\\f0137\"}.bosch-ic-head-speaking-request:before{content:\"\\f0138\"}.bosch-ic-tetragon-priority:before{content:\"\\f0139\"}.bosch-ic-head-respond-speaking:before{content:\"\\f013a\"}.bosch-ic-head-respond-request:before{content:\"\\f013b\"}.bosch-ic-hand-stop:before{content:\"\\f013c\"}.bosch-ic-layers-stack:before{content:\"\\f013d\"}.bosch-ic-puzzle-binary:before{content:\"\\f013e\"}.bosch-ic-sdcard-micro:before{content:\"\\f013f\"}.bosch-ic-reader-card:before{content:\"\\f0140\"}.bosch-ic-reader-user-recognition-face:before{content:\"\\f0141\"}.bosch-ic-detector-point-type:before{content:\"\\f0142\"}.bosch-ic-manual-call-point:before{content:\"\\f0143\"}.bosch-ic-gun-magnifier:before{content:\"\\f0144\"}.bosch-ic-interface-module:before{content:\"\\f0145\"}.bosch-ic-fire-detector-automatic:before{content:\"\\f0146\"}.bosch-ic-fire-brigade-emergency:before{content:\"\\f0147\"}.bosch-ic-volume-siren-device:before{content:\"\\f0148\"}.bosch-ic-user-reception:before{content:\"\\f0149\"}.bosch-ic-shredder-machine:before{content:\"\\f014a\"}.bosch-ic-badge-clean:before{content:\"\\f014b\"}.bosch-ic-parking-assistance:before{content:\"\\f014c\"}.bosch-ic-laptop-questionmark:before{content:\"\\f014d\"}.bosch-ic-security-lock:before{content:\"\\f014e\"}.bosch-ic-battery-car-12v:before{content:\"\\f014f\"}.bosch-ic-car-far-propagation:before{content:\"\\f0150\"}.bosch-ic-car-wide-propagation:before{content:\"\\f0151\"}.bosch-ic-ear-sound:before{content:\"\\f0152\"}.bosch-ic-umbrella-drops:before{content:\"\\f0153\"}.bosch-ic-horn-arrows-in:before{content:\"\\f0154\"}.bosch-ic-video-bell:before{content:\"\\f0155\"}.bosch-ic-steering-wheel-arrows-up:before{content:\"\\f0156\"}.bosch-ic-circle-full:before{content:\"\\f0157\"}.bosch-ic-circle-half:before{content:\"\\f0158\"}.bosch-ic-circle-quarter:before{content:\"\\f0159\"}.bosch-ic-circle-quarter-three:before{content:\"\\f015a\"}.bosch-ic-appliance-flame-arrow:before{content:\"\\f015b\"}.bosch-ic-appliance-flame-arrow-bold:before{content:\"\\f015c\"}.bosch-ic-appliance-flame-disabled-bold:before{content:\"\\f015d\"}.bosch-ic-appliance-flame-disabled-light:before{content:\"\\f015e\"}.bosch-ic-appliance-flame-plus:before{content:\"\\f015f\"}.bosch-ic-appliance-flame-plus-bold:before{content:\"\\f0160\"}.bosch-ic-flash-minus-bold:before{content:\"\\f0161\"}.bosch-ic-flash-minus:before{content:\"\\f0162\"}.bosch-ic-flash-off:before{content:\"\\f0163\"}.bosch-ic-flash-plus-bold:before{content:\"\\f0164\"}.bosch-ic-flash-smart-grid:before{content:\"\\f0165\"}.bosch-ic-signal-high:before{content:\"\\f0166\"}.bosch-ic-signal-low:before{content:\"\\f0167\"}.bosch-ic-signal-very-low:before{content:\"\\f0168\"}.bosch-ic-5g:before{content:\"\\f0169\"}.bosch-ic-sun-photovoltaic:before{content:\"\\f016a\"}.bosch-ic-inverter:before{content:\"\\f016b\"}.bosch-ic-car-arrow-down:before{content:\"\\f016c\"}.bosch-ic-car-distance:before{content:\"\\f016d\"}.bosch-ic-car-sensor:before{content:\"\\f016e\"}.bosch-ic-car-sensor-environment:before{content:\"\\f016f\"}.bosch-ic-car-sensor-front:before{content:\"\\f0170\"}.bosch-ic-streets-locator:before{content:\"\\f0171\"}.bosch-ic-truck-distance:before{content:\"\\f0172\"}.bosch-ic-warning-user-walk:before{content:\"\\f0173\"}.bosch-ic-parking-check:before{content:\"\\f0174\"}.bosch-ic-dremel:before{content:\"\\f0175\"}.bosch-ic-allergens:before{content:\"\\f0176\"}.bosch-ic-money-currency-coins:before{content:\"\\f0177\"}.bosch-ic-hydrogen-small:before{content:\"\\f0178\"}.bosch-ic-hydrogen-large:before{content:\"\\f0179\"}.bosch-ic-shoppingcart-remove:before{content:\"\\f017a\"}.bosch-ic-chimney-sweeper-disabled-light:before{content:\"\\f017b\"}.bosch-ic-chimney-sweeper-disabled-bold:before{content:\"\\f017c\"}.bosch-ic-car-waves:before{content:\"\\f017d\"}.bosch-ic-360-degrees-arrow:before{content:\"\\f017e\"}.bosch-ic-clock-info:before{content:\"\\f017f\"}.bosch-ic-truck-front-disabled-bold:before{content:\"\\f0180\"}.bosch-ic-truck-front-disabled-light:before{content:\"\\f0181\"}.bosch-ic-box-closed-list:before{content:\"\\f0182\"}.bosch-ic-battery-car-exchange:before{content:\"\\f0183\"}.bosch-ic-guitar:before{content:\"\\f0184\"}.bosch-ic-actuator-by-wire:before{content:\"\\f0185\"}.bosch-ic-guitar-electric:before{content:\"\\f0186\"}.bosch-ic-format-align-center:before{content:\"\\f0187\"}.bosch-ic-format-align-justify:before{content:\"\\f0188\"}.bosch-ic-format-align-right:before{content:\"\\f0189\"}.bosch-ic-format-checklist:before{content:\"\\f018a\"}.bosch-ic-format-italic:before{content:\"\\f018b\"}.bosch-ic-format-ordered-list:before{content:\"\\f018c\"}.bosch-ic-format-strike:before{content:\"\\f018d\"}.bosch-ic-format-underline:before{content:\"\\f018e\"}.bosch-ic-format-align-left:before{content:\"\\f018f\"}.bosch-ic-format-fill-drop:before{content:\"\\f0190\"}.bosch-ic-format-color-a:before{content:\"\\f0191\"}.bosch-ic-format-bold-b:before{content:\"\\f0192\"}.bosch-ic-fly:before{content:\"\\f0193\"}.bosch-ic-paw:before{content:\"\\f0194\"}.bosch-ic-desktop-video-response-fast:before{content:\"\\f0195\"}.bosch-ic-user-walk-weather-rain:before{content:\"\\f0196\"}.bosch-ic-brake-pedal-plug:before{content:\"\\f0197\"}.bosch-ic-detector-motion-range-down-view:before{content:\"\\f0198\"}.bosch-ic-detector-motion-range-long:before{content:\"\\f0199\"}.bosch-ic-monitor-tiled:before{content:\"\\f019a\"}.bosch-ic-radar-doppler:before{content:\"\\f019b\"}.bosch-ic-radioactive-disabled-light:before{content:\"\\f019c\"}.bosch-ic-radioactive-disabled-bold:before{content:\"\\f019d\"}.bosch-ic-bug-disabled-light:before{content:\"\\f019e\"}.bosch-ic-bug-disabled-bold:before{content:\"\\f019f\"}.bosch-ic-fly-disabled-light:before{content:\"\\f01a0\"}.bosch-ic-fly-disabled-bold:before{content:\"\\f01a1\"}.bosch-ic-puzzle-2:before{content:\"\\f01a2\"}.bosch-ic-camera-lenses-multi-images:before{content:\"\\f01a3\"}.bosch-ic-detector-spray:before{content:\"\\f01a4\"}.bosch-ic-weather-rain-ip56:before{content:\"\\f01a5\"}.bosch-ic-document-log-start:before{content:\"\\f01a6\"}.bosch-ic-document-log-stop:before{content:\"\\f01a7\"}.bosch-ic-document-3d-pdf:before{content:\"\\f01a8\"}.bosch-ic-document-ifc:before{content:\"\\f01a9\"}.bosch-ic-document-rfa:before{content:\"\\f01aa\"}.bosch-ic-document-stp:before{content:\"\\f01ab\"}.bosch-ic-recycle-arrows-pap20-frame:before{content:\"\\e8b4\"}.bosch-ic-umbrella-drops-frame:before{content:\"\\e8b5\"}.bosch-ic-arrows-this-side-up-frame:before{content:\"\\e8b6\"}.bosch-ic-glass-wine-frame:before{content:\"\\e8b7\"}.bosch-ic-pb-frame:before{content:\"\\e8b8\"}.bosch-ic-pb-frame-disabled-bold:before{content:\"\\e8b9\"}.bosch-ic-pb-frame-disabled-light:before{content:\"\\e8ba\"}.bosch-ic-mode-frame:before{content:\"\\e8bb\"}.bosch-ic-uv-frame-disabled-bold:before{content:\"\\e8bc\"}.bosch-ic-uv-frame-disabled-light:before{content:\"\\e8bd\"}.bosch-ic-fan-frame:before{content:\"\\e8be\"}.bosch-ic-flash-frame:before{content:\"\\e8bf\"}.bosch-ic-kakao:before{content:\"\\eaaf\"}.bosch-ic-kakao-frame:before{content:\"\\eab0\"}.bosch-ic-naver:before{content:\"\\eab1\"}.bosch-ic-naver-frame:before{content:\"\\eab2\"}.bosch-ic-tencent:before{content:\"\\eab3\"}.bosch-ic-tencent-frame:before{content:\"\\eab4\"}.bosch-ic-wordpress:before{content:\"\\eab5\"}.bosch-ic-wordpress-frame:before{content:\"\\eab6\"}.bosch-ic-mini-program-frame:before{content:\"\\eab7\"}.bosch-ic-mini-program:before{content:\"\\eab8\"}.bosch-ic-tiktok:before{content:\"\\eab9\"}.bosch-ic-tiktok-frame:before{content:\"\\eaba\"}.bosch-ic-glassdoor:before{content:\"\\eabb\"}.bosch-ic-glassdoor-frame:before{content:\"\\eabc\"}.bosch-ic-git-hub:before{content:\"\\eabd\"}.bosch-ic-git-hub-frame:before{content:\"\\eabe\"}.bosch-ic-online-www:before{content:\"\\f01ac\"}.bosch-ic-bluetooth-connected:before{content:\"\\f01ad\"}.bosch-ic-connected-tools:before{content:\"\\f01ae\"}.bosch-ic-badge-arrow-kickback:before{content:\"\\f01af\"}.bosch-ic-copy-paste:before{content:\"\\f01b0\"}.bosch-ic-volleyball:before{content:\"\\f01b1\"}.bosch-ic-car-top-trailer:before{content:\"\\f01b2\"}.bosch-ic-mountain:before{content:\"\\f01b3\"}.bosch-ic-battery-car-oil-lamp:before{content:\"\\f01b4\"}.bosch-ic-battery-car-efb:before{content:\"\\f01b5\"}.bosch-ic-battery-car-ams:before{content:\"\\f01b6\"}.bosch-ic-battery-car-ag:before{content:\"\\f01b7\"}.bosch-ic-display-fuel-saving:before{content:\"\\f01b8\"}.bosch-ic-circle-parking-arrows:before{content:\"\\f01b9\"}.bosch-ic-unicorn-head:before{content:\"\\f01ba\"}.bosch-ic-car-parking-automated:before{content:\"\\f01bb\"}.bosch-ic-lunchboard:before{content:\"\\f01bc\"}.bosch-ic-tetrapack:before{content:\"\\f01bd\"}.bosch-ic-recycle-arrows:before{content:\"\\f01be\"}.bosch-ic-stairs-arrow-up:before{content:\"\\f01bf\"}.bosch-ic-stairs-arrow-down:before{content:\"\\f01c0\"}.bosch-ic-industry-car-arrow:before{content:\"\\f01c1\"}.bosch-ic-sensor-throttle-position:before{content:\"\\f01c2\"}.bosch-ic-sensor-knock:before{content:\"\\f01c3\"}.bosch-ic-sensor-air-mass-meter:before{content:\"\\f01c4\"}.bosch-ic-sensor-pressure:before{content:\"\\f01c5\"}.bosch-ic-sensor-temperature:before{content:\"\\f01c6\"}.bosch-ic-sensor-speed-rpm:before{content:\"\\f01c7\"}.bosch-ic-desktop-user-software-clock:before{content:\"\\f01c8\"}.bosch-ic-book-service-wrench-add:before{content:\"\\f01c9\"}.bosch-ic-book-service-wrench:before{content:\"\\f01ca\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-light:before{content:\"\\f01cb\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-bold:before{content:\"\\f01cc\"}.bosch-ic-circle-parking-disabled-bold:before{content:\"\\f01cd\"}.bosch-ic-circle-parking-disabled-light:before{content:\"\\f01ce\"}.bosch-ic-interval-30sec:before{content:\"\\f01cf\"}.bosch-ic-truck-front-light-inside:before{content:\"\\f01d0\"}.bosch-ic-software-tex:before{content:\"\\f01d1\"}.bosch-ic-fire-arrow-circle:before{content:\"\\f01d2\"}.bosch-ic-heating-underfloor-circle-global-off:before{content:\"\\f01d3\"}.bosch-ic-heating-radiator-circle-global-off:before{content:\"\\f01d4\"}.bosch-ic-faucet-circle-global-off:before{content:\"\\f01d5\"}.bosch-ic-moon-stars-starlight:before{content:\"\\f01d6\"}.bosch-ic-tube-thumb-up:before{content:\"\\f01d7\"}.bosch-ic-detector-microwave:before{content:\"\\f01d8\"}.bosch-ic-user-headset-thumb-up:before{content:\"\\f01d9\"}.bosch-ic-cloud-battery:before{content:\"\\f01da\"}.bosch-ic-temperature-arrow-up-increase:before{content:\"\\f01db\"}.bosch-ic-temperature-arrow-down-decrease:before{content:\"\\f01dc\"}.bosch-ic-moneybag-dollar-chart-double:before{content:\"\\f01dd\"}.bosch-ic-locator-remove:before{content:\"\\f01de\"}.bosch-ic-square-remove:before{content:\"\\f01df\"}.bosch-ic-sun-hdr:before{content:\"\\f01e0\"}.bosch-ic-moon-stars-circle:before{content:\"\\f01e1\"}.bosch-ic-temperature-user-detection:before{content:\"\\f01e2\"}.bosch-ic-wrench-thumb-up:before{content:\"\\f01e3\"}.bosch-ic-ring-infrared-ir:before{content:\"\\f01e4\"}.bosch-ic-play-back-arrow:before{content:\"\\f01e5\"}.bosch-ic-ignition-off:before{content:\"\\f01e6\"}.bosch-ic-arrow-up-down:before{content:\"\\f01e7\"}.bosch-ic-arrow-left-right:before{content:\"\\f01e8\"}.bosch-ic-number-0-bold:before{content:\"\\f01e9\"}.bosch-ic-number-1-bold:before{content:\"\\f01ea\"}.bosch-ic-number-2-bold:before{content:\"\\f01eb\"}.bosch-ic-number-3-bold:before{content:\"\\f01ec\"}.bosch-ic-number-4-bold:before{content:\"\\f01ed\"}.bosch-ic-number-5-bold:before{content:\"\\f01ee\"}.bosch-ic-number-6-bold:before{content:\"\\f01ef\"}.bosch-ic-number-7-bold:before{content:\"\\f01f0\"}.bosch-ic-number-8-bold:before{content:\"\\f01f1\"}.bosch-ic-number-9-bold:before{content:\"\\f01f2\"}.bosch-ic-e-charging-plug-dc:before{content:\"\\f01f3\"}.bosch-ic-e-charging-plug-ac-dc:before{content:\"\\f01f4\"}.bosch-ic-angle-view-wide-monitor:before{content:\"\\f01f5\"}.bosch-ic-settings-range-ptrz:before{content:\"\\f01f6\"}.bosch-ic-puzzle-video-integration:before{content:\"\\f01f7\"}.bosch-ic-number-0-regular:before{content:\"\\f01f8\"}.bosch-ic-number-1-regular:before{content:\"\\f01f9\"}.bosch-ic-number-2-regular:before{content:\"\\f01fa\"}.bosch-ic-number-3-regular:before{content:\"\\f01fb\"}.bosch-ic-number-4-regular:before{content:\"\\f01fc\"}.bosch-ic-number-5-regular:before{content:\"\\f01fd\"}.bosch-ic-number-6-regular:before{content:\"\\f01fe\"}.bosch-ic-number-7-regular:before{content:\"\\f01ff\"}.bosch-ic-number-8-regular:before{content:\"\\f0200\"}.bosch-ic-number-9-regular:before{content:\"\\f0201\"}.bosch-ic-import-beep:before{content:\"\\f0202\"}.bosch-ic-file-questionmark:before{content:\"\\f0203\"}.bosch-ic-file-dollar:before{content:\"\\f0204\"}.bosch-ic-document-reporting:before{content:\"\\f0205\"}.bosch-ic-box-questionmark1:before{content:\"\\f0206\"}.bosch-ic-chat-faq:before{content:\"\\f0207\"}.bosch-ic-wishlist-filled:before{content:\"\\f0208\"}.bosch-ic-wishlist-half-filled:before{content:\"\\f0209\"}.bosch-ic-glove-work:before{content:\"\\f020a\"}.bosch-ic-lightbulb-halogen-hand-disabled-bold:before{content:\"\\f020b\"}.bosch-ic-lightbulb-halogen-hand-disabled-light:before{content:\"\\f020c\"}.bosch-ic-lightbulb-halogen-cracked-disabled-bold:before{content:\"\\f020d\"}.bosch-ic-lightbulb-halogen-cracked-disabled-light:before{content:\"\\f020e\"}.bosch-ic-lightbulb-halogen-cracked:before{content:\"\\f020f\"}.bosch-ic-drops:before{content:\"\\f0210\"}.bosch-ic-drops-disabled-bold:before{content:\"\\f0211\"}.bosch-ic-drops-disabled-light:before{content:\"\\f0212\"}.bosch-ic-temperature-heat:before{content:\"\\f0213\"}.bosch-ic-allen-key:before{content:\"\\f0214\"}.bosch-ic-aperture-closed:before{content:\"\\f0215\"}.bosch-ic-appliance-heat-pump-leaf-connected:before{content:\"\\f0216\"}.bosch-ic-badge-flash:before{content:\"\\f0217\"}.bosch-ic-battery-car-1-plug-warning:before{content:\"\\f0218\"}.bosch-ic-battery-car-3-automatic-mode:before{content:\"\\f0219\"}.bosch-ic-battery-car-3-ice:before{content:\"\\f021a\"}.bosch-ic-battery-car-memory-function:before{content:\"\\f021b\"}.bosch-ic-battery-polarity-reverse-warning:before{content:\"\\f021c\"}.bosch-ic-book-questionmark:before{content:\"\\f021d\"}.bosch-ic-book-service-wrench-refresh:before{content:\"\\f021e\"}.bosch-ic-box-arrow-up:before{content:\"\\f021f\"}.bosch-ic-box-arrows-up-down:before{content:\"\\f0220\"}.bosch-ic-camera-optic-sensor:before{content:\"\\f0221\"}.bosch-ic-car-cloud-software:before{content:\"\\f0222\"}.bosch-ic-car-gear-technology:before{content:\"\\f0223\"}.bosch-ic-car-street:before{content:\"\\f0224\"}.bosch-ic-chip-sensor:before{content:\"\\f0225\"}.bosch-ic-circle-blank:before{content:\"\\f0226\"}.bosch-ic-detector-point-type-replace:before{content:\"\\f0227\"}.bosch-ic-document-x:before{content:\"\\f0228\"}.bosch-ic-drag-handle-arrows-up-down:before{content:\"\\f0229\"}.bosch-ic-electrolyzer:before{content:\"\\f022a\"}.bosch-ic-exhaust-pipe:before{content:\"\\f022b\"}.bosch-ic-flower-focus-near:before{content:\"\\f022c\"}.bosch-ic-fuel-pump-electric:before{content:\"\\f022d\"}.bosch-ic-gear-dollar:before{content:\"\\f022e\"}.bosch-ic-gloves-work:before{content:\"\\f022f\"}.bosch-ic-handlewithcare-globe:before{content:\"\\f0230\"}.bosch-ic-handlewithcare-user:before{content:\"\\f0231\"}.bosch-ic-hard-disk:before{content:\"\\f0232\"}.bosch-ic-hierarchy-child:before{content:\"\\f0233\"}.bosch-ic-hierarchy-parent:before{content:\"\\f0234\"}.bosch-ic-hydraulic-connection-leaking-drops:before{content:\"\\f0235\"}.bosch-ic-hydraulic-connection-off:before{content:\"\\f0236\"}.bosch-ic-hydraulic-connection-on:before{content:\"\\f0237\"}.bosch-ic-light-beam:before{content:\"\\f0238\"}.bosch-ic-map5000:before{content:\"\\f0239\"}.bosch-ic-module-anti-blocking-system:before{content:\"\\f023a\"}.bosch-ic-module-ecg:before{content:\"\\f023b\"}.bosch-ic-module-ecg-suction:before{content:\"\\f023c\"}.bosch-ic-module-navigation:before{content:\"\\f023d\"}.bosch-ic-power-supply-unit-plug:before{content:\"\\f023e\"}.bosch-ic-security-street:before{content:\"\\f023f\"}.bosch-ic-server-single:before{content:\"\\f0240\"}.bosch-ic-skip-backward:before{content:\"\\f0241\"}.bosch-ic-skip-forward:before{content:\"\\f0242\"}.bosch-ic-speedometer-instrument-cluster:before{content:\"\\f0243\"}.bosch-ic-system-radar-connected:before{content:\"\\f0244\"}.bosch-ic-truck-dump:before{content:\"\\f0245\"}.bosch-ic-truck-loader:before{content:\"\\f0246\"}.bosch-ic-user-head-twin-digital:before{content:\"\\f0247\"}.bosch-ic-wind-vane-strong:before{content:\"\\f0248\"}.bosch-ic-wind-vane-weak:before{content:\"\\f0249\"}.bosch-ic-windshield-crack:before{content:\"\\f024a\"}.bosch-ic-common-rail:before{content:\"\\f024b\"}.bosch-ic-common-rail-injector-cri1-cri2:before{content:\"\\f024c\"}.bosch-ic-common-rail-injector-crii3:before{content:\"\\f024d\"}.bosch-ic-common-rail-injector-cril3:before{content:\"\\f024e\"}.bosch-ic-common-rail-injector-crin4-2:before{content:\"\\f024f\"}.bosch-ic-common-rail-injector-crin123:before{content:\"\\f0250\"}.bosch-ic-conventional-pump-a:before{content:\"\\f0251\"}.bosch-ic-conventional-pump-b:before{content:\"\\f0252\"}.bosch-ic-edc-inline-injection-pump-pes6p:before{content:\"\\f0253\"}.bosch-ic-edc-inline-injection-pump-pes6h:before{content:\"\\f0254\"}.bosch-ic-nozzle-holder:before{content:\"\\f0255\"}.bosch-ic-unit-injector:before{content:\"\\f0256\"}.bosch-ic-qr-code-scan:before{content:\"\\f0257\"}.bosch-ic-car-mechanic-plug:before{content:\"\\f0258\"}.bosch-ic-copy-machine:before{content:\"\\f0259\"}.bosch-ic-dispenser-paper-towel:before{content:\"\\f025a\"}.bosch-ic-hand-drops-soap:before{content:\"\\f025b\"}.bosch-ic-door-automatic:before{content:\"\\f025c\"}.bosch-ic-door-automatic-slide:before{content:\"\\f025d\"}.bosch-ic-door-pull:before{content:\"\\f025e\"}.bosch-ic-door-push:before{content:\"\\f025f\"}.bosch-ic-floor-load-max:before{content:\"\\f0260\"}.bosch-ic-lockers:before{content:\"\\f0261\"}.bosch-ic-rubbish-trash-bin:before{content:\"\\f0262\"}.bosch-ic-rubbish-waste-bin:before{content:\"\\f0263\"}.bosch-ic-rubbish-waste-container:before{content:\"\\f0264\"}.bosch-ic-sugar:before{content:\"\\f0265\"}.bosch-ic-rubbish-glass-bin:before{content:\"\\f0266\"}.bosch-ic-car-seat-steering-wheel-stowable:before{content:\"\\f0267\"}.bosch-ic-car-seat-steering-wheel-standard:before{content:\"\\f0268\"}.bosch-ic-car-seat-steering-wheel-extended:before{content:\"\\f0269\"}.bosch-ic-device-bridge:before{content:\"\\f026a\"}.bosch-ic-screw-cross-slot:before{content:\"\\f026b\"}.bosch-ic-screw-hexagon-head:before{content:\"\\f026c\"}.bosch-ic-screw-hexagon-internal:before{content:\"\\f026d\"}.bosch-ic-screw-slotted:before{content:\"\\f026e\"}.bosch-ic-screw-torx:before{content:\"\\f026f\"}.bosch-ic-tape:before{content:\"\\f0270\"}.bosch-ic-tape-teflon:before{content:\"\\f0271\"}.bosch-ic-security-fire:before{content:\"\\f0272\"}.bosch-ic-led-fault:before{content:\"\\f0273\"}.bosch-ic-led-alarm:before{content:\"\\f0274\"}.bosch-ic-firepanel:before{content:\"\\f0275\"}.bosch-ic-fire-output-disabled:before{content:\"\\f0276\"}.bosch-ic-user-fire-alarm:before{content:\"\\f0277\"}.bosch-ic-car-desktop-graph:before{content:\"\\f0278\"}.bosch-ic-desktop-management-statistics:before{content:\"\\f0279\"}.bosch-ic-document-key:before{content:\"\\f027a\"}.bosch-ic-options-vertical:before{content:\"\\f027b\"}.bosch-ic-euro-arrows:before{content:\"\\f027c\"}.bosch-ic-monkey:before{content:\"\\f027d\"}.bosch-ic-appliance-resistance:before{content:\"\\f027e\"}.bosch-ic-pipe-cutter:before{content:\"\\f027f\"}.bosch-ic-flash-arrow:before{content:\"\\f0280\"}.bosch-ic-flash-arrow-disabled-light:before{content:\"\\f0281\"}.bosch-ic-heating-ceiling:before{content:\"\\f0282\"}.bosch-ic-heating-ceiling-off-light:before{content:\"\\f0283\"}.bosch-ic-heating-ceiling-off-bold:before{content:\"\\f0284\"}.bosch-ic-battery-analysis:before{content:\"\\f0285\"}.bosch-ic-battery-ice:before{content:\"\\f0286\"}.bosch-ic-battery-refresh:before{content:\"\\f0287\"}.bosch-ic-device-ids:before{content:\"\\f0288\"}.bosch-ic-device-thermostat-bcc50-72:before{content:\"\\f0289\"}.bosch-ic-device-thermostat-bcc100-72:before{content:\"\\f028a\"}.bosch-ic-silos-gas-natural:before{content:\"\\f028b\"}.bosch-ic-silos-gas-leaf-biogas:before{content:\"\\f028c\"}.bosch-ic-circle-decentralized:before{content:\"\\f028d\"}.bosch-ic-molecules-h2:before{content:\"\\f028e\"}.bosch-ic-megawatt-range-arrow-scalable:before{content:\"\\f028f\"}.bosch-ic-fan-arrows:before{content:\"\\f0290\"}.bosch-ic-recycle-arrows-pvc3:before{content:\"\\f0291\"}.bosch-ic-recycle-arrows-ps6:before{content:\"\\f0292\"}.bosch-ic-recycle-arrows-pp5:before{content:\"\\f0293\"}.bosch-ic-recycle-arrows-pet1:before{content:\"\\f0294\"}.bosch-ic-recycle-arrows-pap22:before{content:\"\\f0295\"}.bosch-ic-recycle-arrows-pap21:before{content:\"\\f0296\"}.bosch-ic-recycle-arrows-pap20:before{content:\"\\f0297\"}.bosch-ic-recycle-arrows-oth7:before{content:\"\\f0298\"}.bosch-ic-recycle-arrows-ldpe4:before{content:\"\\f0299\"}.bosch-ic-recycle-arrows-hdpe2:before{content:\"\\f029a\"}.bosch-ic-car-petrol-plug:before{content:\"\\f029b\"}.bosch-ic-user-diverse:before{content:\"\\f029c\"}.bosch-ic-fire-emergency-run-map:before{content:\"\\f029d\"}.bosch-ic-automatic-mode-fan:before{content:\"\\f029e\"}.bosch-ic-air-condition-swing-pac:before{content:\"\\f029f\"}.bosch-ic-user-fire-alarm-check:before{content:\"\\f02a0\"}.bosch-ic-handlewithcare-team:before{content:\"\\f02a1\"}.bosch-ic-settings-user-dollar:before{content:\"\\f02a2\"}.bosch-ic-skyscraper-connected:before{content:\"\\f02a3\"}.bosch-ic-security-ac:before{content:\"\\f02a4\"}.bosch-ic-windshield-glare-sun:before{content:\"\\f02a5\"}.bosch-ic-user-flag:before{content:\"\\f02a6\"}.bosch-ic-hammer-legal:before{content:\"\\f02a7\"}.bosch-ic-handover-star:before{content:\"\\f02a8\"}.bosch-ic-laptop-wrench:before{content:\"\\f02a9\"}/*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.Bosch-IC-find-part:before{content:\"\\e38f\"}.Bosch-IC-next-station:before{content:\"\\e8a9\"}.Bosch-Ic-HMI:before{content:\"\\e293\"}.Bosch-Ic-add-outline:before{content:\"\\e8a1\"}.Bosch-Ic-apas:before{content:\"\\e224\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-Ic-cash:before{content:\"\\e890\"}.Bosch-Ic-pinterest:before{content:\"\\ea8c\"}.Bosch-Ic-problem:before{content:\"\\e899\"}.Bosch-Ic-promotion:before{content:\"\\e89a\"}.Bosch-Ic-arrow-diagonal:before{content:\"\\e095\"}.Bosch-Ic-arrow-down-filled:before{content:\"\\e887\"}.Bosch-Ic-arrow-down-circle:before{content:\"\\e887\"}.Bosch-Ic-arrow-end-left:before{content:\"\\e0a1\"}.Bosch-Ic-arrow-end-right:before{content:\"\\e182\"}.Bosch-Ic-arrow-left-down-filled:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-filled:before{content:\"\\e889\"}.Bosch-Ic-arrow-left-up-filled:before{content:\"\\e88a\"}.Bosch-Ic-arrow-right-down-filled:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-filled:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-up-filled:before{content:\"\\e88d\"}.Bosch-Ic-arrow-up-filled:before{content:\"\\e88e\"}.Bosch-Ic-batch:before{content:\"\\e384\"}.Bosch-Ic-calculator:before{content:\"\\e0c7\"}.Bosch-Ic-calendar-delete:before{content:\"\\e385\"}.Bosch-Ic-call-bluetooth:before{content:\"\\e386\"}.Bosch-Ic-call-deny-outline:before{content:\"\\e8a2\"}.Bosch-Ic-call-deny:before{content:\"\\e387\"}.Bosch-Ic-call-pick-up-outline:before{content:\"\\e8a3\"}.Bosch-Ic-call-pick-up:before{content:\"\\e0cb\"}.Bosch-Ic-call-settings:before{content:\"\\e388\"}.Bosch-Ic-clear-all-annotations:before{content:\"\\e0ec\"}.Bosch-Ic-connection:before{content:\"\\e107\"}.Bosch-Ic-core-data:before{content:\"\\e38b\"}.Bosch-Ic-curve-scaling-view:before{content:\"\\e2bb\"}.Bosch-Ic-dashboard:before{content:\"\\e190\"}.Bosch-Ic-delete-material-manually:before{content:\"\\e394\"}.Bosch-Ic-dmc-code:before{content:\"\\e38e\"}.Bosch-Ic-document-to-svg:before{content:\"\\e34e\"}.Bosch-Ic-emergency-order:before{content:\"\\e12f\"}.Bosch-Ic-fast-backwards:before{content:\"\\e8a4\"}.Bosch-Ic-fast-forwards:before{content:\"\\e8a5\"}.Bosch-Ic-favorite:before{content:\"\\e2c5\"}.Bosch-Ic-go-to:before{content:\"\\e1ca\"}.Bosch-Ic-gps:before{content:\"\\e1c6\"}.Bosch-Ic-home-filled:before{content:\"\\e19d\"}.Bosch-Ic-input:before{content:\"\\e1ca\"}.Bosch-Ic-interface:before{content:\"\\e391\"}.Bosch-Ic-list:before{content:\"\\e1c1\"}.Bosch-Ic-malfunction-history:before{content:\"\\e389\"}.Bosch-Ic-malfunction-order:before{content:\"\\e38a\"}.Bosch-Ic-material-list:before{content:\"\\e393\"}.Bosch-Ic-material:before{content:\"\\e395\"}.Bosch-Ic-maximize-window:before{content:\"\\e269\"}.Bosch-Ic-menu-tree:before{content:\"\\e396\"}.Bosch-Ic-minimize-window:before{content:\"\\e1ba\"}.Bosch-Ic-notification-hint:before{content:\"\\e899\"}.Bosch-Ic-notification-question:before{content:\"\\e89b\"}.Bosch-Ic-notification-info:before{content:\"\\eb0f\"}.Bosch-Ic-notification-error:before{content:\"\\eb08\"}.Bosch-Ic-notification-error-outline:before{content:\"\\eb09\"}.Bosch-Ic-notification-outline:before{content:\"\\eb0d\"}.Bosch-Ic-notification-success:before{content:\"\\eb0a\"}.Bosch-Ic-notification-success-outline:before{content:\"\\eb0b\"}.Bosch-Ic-notification-warning:before{content:\"\\eb0c\"}.Bosch-Ic-notification:before{content:\"\\e1f5\"}.Bosch-Ic-output:before{content:\"\\e1cc\"}.Bosch-Ic-paste:before{content:\"\\e0ed\"}.Bosch-Ic-qr-code:before{content:\"\\e397\"}.Bosch-Ic-remote-cursor:before{content:\"\\e38c\"}.Bosch-Ic-scale:before{content:\"\\e167\"}.Bosch-Ic-search-chart:before{content:\"\\e383\"}.Bosch-Ic-search-web:before{content:\"\\e390\"}.Bosch-Ic-search-window:before{content:\"\\e39b\"}.Bosch-Ic-shape-ellipse:before{content:\"\\e3d1\"}.Bosch-Ic-shape-rectangle:before{content:\"\\e269\"}.Bosch-Ic-show-more-vertical:before{content:\"\\e1fb\"}.Bosch-Ic-signal:before{content:\"\\e249\"}.Bosch-Ic-skip-back:before{content:\"\\e8a6\"}.Bosch-Ic-skip-backward:before{content:\"\\e8a7\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-Ic-skip-for:before{content:\"\\e8a9\"}.Bosch-Ic-skip-forward:before{content:\"\\e8a8\"}.Bosch-Ic-sort-alphabetically:before{content:\"\\e398\"}.Bosch-Ic-switch-language:before{content:\"\\e18d\"}.Bosch-Ic-snyc-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-problem:before{content:\"\\e37f\"}.Bosch-Ic-sync:before{content:\"\\e21e\"}.Bosch-Ic-test:before{content:\"\\e105\"}.Bosch-Ic-to-order:before{content:\"\\e127\"}.Bosch-Ic-to-raw:before{content:\"\\e126\"}.Bosch-Ic-transport-orders:before{content:\"\\e0ee\"}.Bosch-Ic-unpin:before{content:\"\\e463\"}.Bosch-Ic-user-filled:before{content:\"\\e29b\"}.Bosch-Ic-video-play:before{content:\"\\e20b\"}.Bosch-Ic-weblink:before{content:\"\\e1be\"}.Bosch-Ic-warning-outline:before{content:\"\\eb0d\"}.Bosch-Ic-add-material-manually:before{content:\"\\e392\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-IC-3d-print:before{content:\"\\e085\"}.Bosch-Ic-tours:before{content:\"\\e086\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-IC-agility:before{content:\"\\e08a\"}.Bosch-Ic-clock-stop:before{content:\"\\e08b\"}.Bosch-IC-ambulance:before{content:\"\\e08c\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-Ic-arrow-down:before{content:\"\\e08f\"}.Bosch-Ic-arrow-left:before{content:\"\\e090\"}.Bosch-Ic-arrow-left-down:before{content:\"\\e091\"}.Bosch-Ic-arrow-left-up:before{content:\"\\e092\"}.Bosch-Ic-arrow-right:before{content:\"\\e093\"}.Bosch-Ic-arrow-right-down:before{content:\"\\e094\"}.Bosch-Ic-arrow-right-up:before{content:\"\\e095\"}.Bosch-IC-arrow-turn-left:before{content:\"\\e096\"}.Bosch-IC-arrow-turn-right:before{content:\"\\e097\"}.Bosch-Ic-arrow-up:before{content:\"\\e098\"}.Bosch-IC-arrows:before{content:\"\\e099\"}.Bosch-IC-arrows-x-y-z:before{content:\"\\e09a\"}.Bosch-IC-artificial-intelligence:before{content:\"\\e09b\"}.Bosch-IC-assembly-line:before{content:\"\\e09c\"}.Bosch-IC-atom:before{content:\"\\e09d\"}.Bosch-IC-augmented-reality:before{content:\"\\e09e\"}.Bosch-Ic-autobod:before{content:\"\\e09f\"}.Bosch-Ic-arrow-expand-left:before{content:\"\\e0a0\"}.Bosch-Ic-arrow-expand-double-left:before{content:\"\\e0a1\"}.Bosch-IC-bar-chart:before{content:\"\\e0a2\"}.Bosch-IC-bar-chart-double:before{content:\"\\e0a3\"}.Bosch-Ic-battery-empty:before{content:\"\\e0a4\"}.Bosch-Ic-battery-25:before{content:\"\\e0a5\"}.Bosch-Ic-battery-50:before{content:\"\\e0a6\"}.Bosch-Ic-battery-75:before{content:\"\\e0a7\"}.Bosch-Ic-battery-100:before{content:\"\\e0a8\"}.Bosch-Ic-battery-error:before{content:\"\\e0a9\"}.Bosch-IC-battery_Li-Ion:before{content:\"\\e0aa\"}.Bosch-Ic-battery-charging:before{content:\"\\e0ab\"}.Bosch-Ic-help:before{content:\"\\e89b\"}.Bosch-IC-bed:before{content:\"\\e0ac\"}.Bosch-IC-bicycle:before{content:\"\\e0ad\"}.Bosch-IC-bicycle-e:before{content:\"\\e0ae\"}.Bosch-IC-blog:before{content:\"\\e0af\"}.Bosch-IC-bluetooth:before{content:\"\\e0b0\"}.Bosch-IC-bluetooth-LE:before{content:\"\\e0b1\"}.Bosch-IC-board-speaker:before{content:\"\\e0b2\"}.Bosch-IC-boiler-connected:before{content:\"\\e0b3\"}.Bosch-IC-book:before{content:\"\\e0b4\"}.Bosch-Ic-bookmark:before{content:\"\\e0b5\"}.Bosch-Ic-bookmark-add:before{content:\"\\e0b6\"}.Bosch-Ic-bookmark-check:before{content:\"\\e0b7\"}.Bosch-Ic-bookmark-delete:before{content:\"\\e0b8\"}.Bosch-IC-books:before{content:\"\\e0b9\"}.Bosch-IC-bookshelf:before{content:\"\\e0ba\"}.Bosch-Ic-box:before{content:\"\\e0bb\"}.Bosch-IC-box-closed:before{content:\"\\e0bc\"}.Bosch-IC-box-delivery:before{content:\"\\e0bd\"}.Bosch-IC-box-delivery-connected:before{content:\"\\e0be\"}.Bosch-IC-brake-disk:before{content:\"\\e0bf\"}.Bosch-IC-briefcase:before{content:\"\\e0c0\"}.Bosch-IC-broom:before{content:\"\\e0c1\"}.Bosch-IC-broom-cloud-dirt:before{content:\"\\e0c2\"}.Bosch-Ic-brush:before{content:\"\\e0c3\"}.Bosch-Ic-bug:before{content:\"\\e0c4\"}.Bosch-IC-building:before{content:\"\\e0c5\"}.Bosch-IC-cactus:before{content:\"\\e0c6\"}.Bosch-IC-calculate:before{content:\"\\e0c7\"}.Bosch-Ic-calendar:before{content:\"\\e0c8\"}.Bosch-Ic-calendar-add:before{content:\"\\e0c9\"}.Bosch-IC-calendar-settings:before{content:\"\\e0ca\"}.Bosch-Ic-call:before{content:\"\\e0cb\"}.Bosch-IC-call-add:before{content:\"\\e0cc\"}.Bosch-IC-call-remove:before{content:\"\\e0cd\"}.Bosch-IC-call-sos:before{content:\"\\e0ce\"}.Bosch-IC-call-team:before{content:\"\\e0cf\"}.Bosch-Ic-call-wifi:before{content:\"\\e0d0\"}.Bosch-Ic-camera:before{content:\"\\e0d1\"}.Bosch-Ic-camera-switch:before{content:\"\\e0d2\"}.Bosch-IC-car:before{content:\"\\e0d3\"}.Bosch-IC-car-clock:before{content:\"\\e0d4\"}.Bosch-IC-car-connected:before{content:\"\\e0d5\"}.Bosch-IC-car-locator:before{content:\"\\e0d6\"}.Bosch-IC-car-mechanic:before{content:\"\\e0d7\"}.Bosch-IC-car-rental:before{content:\"\\e0d8\"}.Bosch-IC-car-seat-connected:before{content:\"\\e0d9\"}.Bosch-IC-car-side:before{content:\"\\e0da\"}.Bosch-IC-car-side-user:before{content:\"\\e0db\"}.Bosch-Ic-milkrun-cart:before{content:\"\\e0dc\"}.Bosch-Ic-certificate:before{content:\"\\e0dd\"}.Bosch-IC-certificate-pki:before{content:\"\\e0de\"}.Bosch-IC-change:before{content:\"\\e0df\"}.Bosch-Ic-chart:before{content:\"\\e0e0\"}.Bosch-IC-chart-check:before{content:\"\\e0e1\"}.Bosch-IC-chart-dummy:before{content:\"\\e0e2\"}.Bosch-Ic-diagram:before{content:\"\\e0e3\"}.Bosch-Ic-chat:before{content:\"\\e0e4\"}.Bosch-IC-chat-add:before{content:\"\\e0e5\"}.Bosch-IC-chat-question-answer:before{content:\"\\e0e6\"}.Bosch-IC-chat-remove:before{content:\"\\e0e7\"}.Bosch-Ic-checklist:before{content:\"\\e0e8\"}.Bosch-Ic-checkmark:before{content:\"\\e0e9\"}.Bosch-Ic-chip:before{content:\"\\e0ea\"}.Bosch-IC-circle-segment:before{content:\"\\e0eb\"}.Bosch-IC-clear-all:before{content:\"\\e0ec\"}.Bosch-Ic-clipboard:before{content:\"\\e0ed\"}.Bosch-Ic-clipboard-list:before{content:\"\\e0ee\"}.Bosch-Ic-clock:before{content:\"\\e0ef\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-IC-close-all:before{content:\"\\e0f1\"}.Bosch-Ic-cloud:before{content:\"\\e0f2\"}.Bosch-IC-cloud-co2:before{content:\"\\e0f3\"}.Bosch-Ic-cloud-download:before{content:\"\\e0f4\"}.Bosch-IC-cloud-nox:before{content:\"\\e0f5\"}.Bosch-Ic-cloud-upload:before{content:\"\\e0f6\"}.Bosch-IC-club-arm-chair:before{content:\"\\e0f7\"}.Bosch-IC-coffee-break:before{content:\"\\e0f8\"}.Bosch-IC-coin-currency:before{content:\"\\e0f9\"}.Bosch-Ic-currency:before{content:\"\\e0fa\"}.Bosch-Ic-color-picker:before{content:\"\\e0fb\"}.Bosch-IC-colors:before{content:\"\\e0fc\"}.Bosch-Ic-command:before{content:\"\\e0fd\"}.Bosch-IC-communicator:before{content:\"\\e0fe\"}.Bosch-Ic-compare:before{content:\"\\e0ff\"}.Bosch-Ic-components:before{content:\"\\e100\"}.Bosch-Ic-components-available:before{content:\"\\e101\"}.Bosch-Ic-components-reservation:before{content:\"\\e102\"}.Bosch-IC-concierge-bell:before{content:\"\\e103\"}.Bosch-IC-configuration:before{content:\"\\e104\"}.Bosch-Ic-confirmation:before{content:\"\\e105\"}.Bosch-Ic-unconnected:before{content:\"\\e106\"}.Bosch-Ic-connected:before{content:\"\\e107\"}.Bosch-IC-connectivity:before{content:\"\\e108\"}.Bosch-Ic-builder:before{content:\"\\e109\"}.Bosch-IC-consumer-goods:before{content:\"\\e10a\"}.Bosch-IC-contract:before{content:\"\\e10b\"}.Bosch-Ic-copy:before{content:\"\\e10c\"}.Bosch-IC-cube:before{content:\"\\e10d\"}.Bosch-IC-cube-arrows:before{content:\"\\e10e\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-IC-cube-shock:before{content:\"\\e110\"}.Bosch-IC-cube-stacked:before{content:\"\\e111\"}.Bosch-Ic-customerservice:before{content:\"\\e112\"}.Bosch-IC-cutlery:before{content:\"\\e113\"}.Bosch-IC-damper:before{content:\"\\e114\"}.Bosch-IC-danger-flash:before{content:\"\\e115\"}.Bosch-IC-data-loss:before{content:\"\\e116\"}.Bosch-Ic-dealer:before{content:\"\\e117\"}.Bosch-Ic-delete:before{content:\"\\e118\"}.Bosch-Ic-delivery:before{content:\"\\e119\"}.Bosch-IC-denied:before{content:\"\\e11a\"}.Bosch-Ic-desktop:before{content:\"\\e11b\"}.Bosch-IC-desktop-application-2d:before{content:\"\\e11c\"}.Bosch-IC-desktop-application-3d:before{content:\"\\e11d\"}.Bosch-IC-development:before{content:\"\\e11e\"}.Bosch-Ic-devices:before{content:\"\\e11f\"}.Bosch-IC-devices-mobile:before{content:\"\\e120\"}.Bosch-Ic-directory:before{content:\"\\e121\"}.Bosch-Ic-dna:before{content:\"\\e122\"}.Bosch-Ic-do-not-disturb:before{content:\"\\e123\"}.Bosch-Ic-document:before{content:\"\\e124\"}.Bosch-Ic-document-add:before{content:\"\\e125\"}.Bosch-IC-document-arrow-down:before{content:\"\\e126\"}.Bosch-IC-document-arrow-up:before{content:\"\\e127\"}.Bosch-Ic-document-audio:before{content:\"\\e128\"}.Bosch-IC-document-check:before{content:\"\\e129\"}.Bosch-Ic-document-to-cloud:before{content:\"\\e12a\"}.Bosch-Ic-document-copy:before{content:\"\\e12b\"}.Bosch-Ic-document-delete:before{content:\"\\e12c\"}.Bosch-Ic-document-to-doc:before{content:\"\\e12d\"}.Bosch-Ic-document-edit:before{content:\"\\e12e\"}.Bosch-Ic-document-error:before{content:\"\\e12f\"}.Bosch-Ic-document-locked:before{content:\"\\e130\"}.Bosch-Ic-log-file:before{content:\"\\e131\"}.Bosch-Ic-document-to-one:before{content:\"\\e132\"}.Bosch-Ic-document-to-pdf:before{content:\"\\e133\"}.Bosch-Ic-document-plain:before{content:\"\\e134\"}.Bosch-Ic-document-plain-add:before{content:\"\\e135\"}.Bosch-Ic-document-plain-delete:before{content:\"\\e136\"}.Bosch-Ic-document-to-ppt:before{content:\"\\e137\"}.Bosch-Ic-document-to-pub:before{content:\"\\e138\"}.Bosch-Ic-document-to-rtf:before{content:\"\\e139\"}.Bosch-Ic-document-save-to:before{content:\"\\e13a\"}.Bosch-Ic-search-document:before{content:\"\\e13b\"}.Bosch-Ic-document-settings:before{content:\"\\e13c\"}.Bosch-Ic-document-test:before{content:\"\\e13d\"}.Bosch-Ic-document-text:before{content:\"\\e13e\"}.Bosch-Ic-document-to-txt:before{content:\"\\e13f\"}.Bosch-Ic-document-to-vsd:before{content:\"\\e140\"}.Bosch-Ic-document-to-xls:before{content:\"\\e141\"}.Bosch-Ic-document-to-xml:before{content:\"\\e142\"}.Bosch-Ic-document-to-zip:before{content:\"\\e143\"}.Bosch-IC-dome:before{content:\"\\e144\"}.Bosch-IC-door:before{content:\"\\e145\"}.Bosch-IC-door-sensor:before{content:\"\\e146\"}.Bosch-Ic-arrow-expand-down:before{content:\"\\e147\"}.Bosch-Ic-arrow-expand-double-down:before{content:\"\\e148\"}.Bosch-Ic-arrow-left-circle:before{content:\"\\e889\"}.Bosch-Ic-arrow-right-circle:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-down-circle:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-up-circle:before{content:\"\\e88d\"}.Bosch-Ic-arrow-left-down-circle:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-up-circle:before{content:\"\\e88a\"}.Bosch-Ic-arrow-up-circle:before{content:\"\\e88e\"}.Bosch-Ic-download:before{content:\"\\e149\"}.Bosch-IC-drag-handle:before{content:\"\\e14a\"}.Bosch-IC-dragdrop:before{content:\"\\e14b\"}.Bosch-IC-drop:before{content:\"\\e14c\"}.Bosch-IC-e-Charging:before{content:\"\\e14d\"}.Bosch-IC-e-mobility:before{content:\"\\e14e\"}.Bosch-IC-ear-off:before{content:\"\\e14f\"}.Bosch-IC-ear-on:before{content:\"\\e150\"}.Bosch-IC-eco-system:before{content:\"\\e151\"}.Bosch-Ic-edit:before{content:\"\\e152\"}.Bosch-IC-education:before{content:\"\\e153\"}.Bosch-IC-efficiency:before{content:\"\\e154\"}.Bosch-IC-elevator:before{content:\"\\e155\"}.Bosch-IC-elevator-alarm:before{content:\"\\e156\"}.Bosch-IC-elevator-cloud:before{content:\"\\e157\"}.Bosch-IC-elevator-headset:before{content:\"\\e158\"}.Bosch-IC-elevator-service:before{content:\"\\e159\"}.Bosch-IC-emergency-exit:before{content:\"\\e15a\"}.Bosch-IC-emoji-happy:before{content:\"\\e15b\"}.Bosch-IC-emoji-neutral:before{content:\"\\e15c\"}.Bosch-IC-emoji-sad:before{content:\"\\e15d\"}.Bosch-IC-emoji-super-happy:before{content:\"\\e15e\"}.Bosch-IC-emoji-very-sad:before{content:\"\\e15f\"}.Bosch-IC-EU-energy-label:before{content:\"\\e160\"}.Bosch-IC-excavator:before{content:\"\\e161\"}.Bosch-Ic-exit:before{content:\"\\e163\"}.Bosch-IC-expansion-arrows:before{content:\"\\e164\"}.Bosch-IC-explosion:before{content:\"\\e165\"}.Bosch-Ic-export:before{content:\"\\e166\"}.Bosch-IC-externallink:before{content:\"\\e167\"}.Bosch-IC-fast-forward:before{content:\"\\e168\"}.Bosch-IC-faucet:before{content:\"\\e169\"}.Bosch-IC-favorites:before{content:\"\\e16a\"}.Bosch-IC-fax:before{content:\"\\e16b\"}.Bosch-IC-female:before{content:\"\\e16c\"}.Bosch-IC-film:before{content:\"\\e16d\"}.Bosch-Ic-filter:before{content:\"\\e16e\"}.Bosch-IC-filter-success:before{content:\"\\e16f\"}.Bosch-IC-fingerprint:before{content:\"\\e170\"}.Bosch-IC-fire:before{content:\"\\e171\"}.Bosch-IC-fire-emergency:before{content:\"\\e172\"}.Bosch-IC-fireworks:before{content:\"\\e173\"}.Bosch-Ic-first-aid:before{content:\"\\e174\"}.Bosch-IC-first-aid-cross:before{content:\"\\e175\"}.Bosch-IC-fit-to-sceen:before{content:\"\\e176\"}.Bosch-Ic-flag:before{content:\"\\e177\"}.Bosch-Ic-danger-flash-outline:before{content:\"\\e178\"}.Bosch-IC-flask:before{content:\"\\e179\"}.Bosch-Ic-flexpress:before{content:\"\\e17a\"}.Bosch-Ic-twitter:before{content:\"\\ea8f\"}.Bosch-Ic-folder:before{content:\"\\e17b\"}.Bosch-Ic-download-from-folder:before{content:\"\\e17c\"}.Bosch-Ic-download-frame:before{content:\"\\e893\"}.Bosch-IC-folder-open:before{content:\"\\e17d\"}.Bosch-Ic-upload-to-folder:before{content:\"\\e17e\"}.Bosch-Ic-fork-lift:before{content:\"\\e17f\"}.Bosch-IC-fork-lift-locator:before{content:\"\\e180\"}.Bosch-Ic-arrow-expand-right:before{content:\"\\e181\"}.Bosch-Ic-arrow-expand-double-right:before{content:\"\\e182\"}.Bosch-IC-full-empty:before{content:\"\\e183\"}.Bosch-Ic-fullscreen:before{content:\"\\e184\"}.Bosch-IC-fullscreen-arrows:before{content:\"\\e185\"}.Bosch-Ic-fullscreen-exit:before{content:\"\\e186\"}.Bosch-IC-g-arrow-down:before{content:\"\\e187\"}.Bosch-IC-g-arrow-up:before{content:\"\\e188\"}.Bosch-IC-generator:before{content:\"\\e189\"}.Bosch-IC-gift:before{content:\"\\e18a\"}.Bosch-IC-glance:before{content:\"\\e18b\"}.Bosch-IC-glas-plate:before{content:\"\\e18c\"}.Bosch-IC-globe:before{content:\"\\e18d\"}.Bosch-IC-globe-arrow:before{content:\"\\e18e\"}.Bosch-IC-glossary:before{content:\"\\e18f\"}.Bosch-Ic-grid:before{content:\"\\e190\"}.Bosch-IC-hammer:before{content:\"\\e191\"}.Bosch-Ic-hand:before{content:\"\\e192\"}.Bosch-IC-hand-motion:before{content:\"\\e193\"}.Bosch-IC-hand-motion-off:before{content:\"\\e194\"}.Bosch-IC-handlewithcare:before{content:\"\\e195\"}.Bosch-Ic-add-material-manually:before{content:\"\\e196\"}.Bosch-Ic-handover-check:before{content:\"\\e197\"}.Bosch-Ic-remove-from-handover:before{content:\"\\e198\"}.Bosch-IC-hanger:before{content:\"\\e199\"}.Bosch-Ic-health:before{content:\"\\e19a\"}.Bosch-IC-heart:before{content:\"\\e19b\"}.Bosch-Ic-history:before{content:\"\\e19c\"}.Bosch-Ic-home-outline:before{content:\"\\e19d\"}.Bosch-IC-home-locator:before{content:\"\\e19e\"}.Bosch-Ic-sandclock:before{content:\"\\e19f\"}.Bosch-IC-hydrant:before{content:\"\\e1a0\"}.Bosch-IC-ice:before{content:\"\\e1a1\"}.Bosch-Ic-image:before{content:\"\\e1a2\"}.Bosch-IC-imagery-add:before{content:\"\\e1a3\"}.Bosch-IC-imagery-remove:before{content:\"\\e1a4\"}.Bosch-Ic-import:before{content:\"\\e1a5\"}.Bosch-Ic-imprint:before{content:\"\\e1a6\"}.Bosch-IC-impulse:before{content:\"\\e1a7\"}.Bosch-Ic-fabric:before{content:\"\\e1a8\"}.Bosch-IC-industry-clock:before{content:\"\\e1a9\"}.Bosch-IC-industry-connected:before{content:\"\\e1aa\"}.Bosch-Ic-information:before{content:\"\\e1ac\"}.Bosch-Ic-information-tooltip:before{content:\"\\e1ac\"}.Bosch-IC-interval:before{content:\"\\e1ad\"}.Bosch-IC-it-device:before{content:\"\\e1ae\"}.Bosch-IC-jewel:before{content:\"\\e1af\"}.Bosch-IC-keyboard:before{content:\"\\e1b0\"}.Bosch-IC-label:before{content:\"\\e1b1\"}.Bosch-Ic-laptop-01:before{content:\"\\e1b2\"}.Bosch-IC-laser:before{content:\"\\e1b3\"}.Bosch-IC-layout:before{content:\"\\e1b4\"}.Bosch-IC-leaf:before{content:\"\\e1b5\"}.Bosch-IC-led:before{content:\"\\e1b6\"}.Bosch-Ic-world:before{content:\"\\e8a0\"}.Bosch-Ic-facebook:before{content:\"\\ea87\"}.Bosch-Ic-checkmark-outline:before{content:\"\\e891\"}.Bosch-Ic-youtube:before{content:\"\\ea96\"}.Bosch-Ic-share:before{content:\"\\e242\"}.Bosch-Ic-user:before{content:\"\\e29b\"}.Bosch-Ic-upload_outline:before{content:\"\\e89f\"}.Bosch-Ic-download_outline:before{content:\"\\e893\"}.Bosch-Ic-wishlist-add:before{content:\"\\e3d3\"}.Bosch-Ic-youtube-frame:before{content:\"\\ea96\"}.Bosch-Ic-less-minimize:before{content:\"\\e1ba\"}.Bosch-IC-lightbulb:before{content:\"\\e1bb\"}.bosch-Ic-innovation:before{content:\"\\e1bc\"}.Bosch-IC-line-chart:before{content:\"\\e1bd\"}.Bosch-Ic-link:before{content:\"\\e1be\"}.Bosch-IC-link-broken:before{content:\"\\e1bf\"}.Bosch-Ic-list-view:before{content:\"\\e1c0\"}.Bosch-Ic-menu:before{content:\"\\e1c1\"}.Bosch-IC-livechat:before{content:\"\\e1c2\"}.Bosch-Ic-location-pin:before{content:\"\\e1c3\"}.Bosch-IC-locator-ip-connected:before{content:\"\\e1c4\"}.Bosch-IC-locator-ip-disconnected:before{content:\"\\e1c5\"}.Bosch-IC-locator-spot:before{content:\"\\e1c6\"}.Bosch-IC-locator-spot-check:before{content:\"\\e1c7\"}.Bosch-Ic-lock-closed:before{content:\"\\e1c8\"}.Bosch-Ic-lock-open:before{content:\"\\e1c9\"}.Bosch-Ic-log-in:before{content:\"\\e1ca\"}.Bosch-Ic-logistics:before{content:\"\\e1cb\"}.Bosch-Ic-log-out:before{content:\"\\e1cc\"}.Bosch-IC-machine:before{content:\"\\e1cd\"}.Bosch-Ic-magnet:before{content:\"\\e1ce\"}.Bosch-Ic-mail:before{content:\"\\e1cf\"}.Bosch-Ic-mail-forward:before{content:\"\\e1d0\"}.Bosch-Ic-mail-open:before{content:\"\\e1d1\"}.Bosch-Ic-mail-out:before{content:\"\\e1d2\"}.Bosch-Ic-mail-answer:before{content:\"\\e1d3\"}.Bosch-IC-male:before{content:\"\\e1d4\"}.Bosch-Ic-map:before{content:\"\\e1d5\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-IC-material-check:before{content:\"\\e1d7\"}.Bosch-Ic-remove-material:before{content:\"\\e1d8\"}.Bosch-IC-maximize:before{content:\"\\e1d9\"}.Bosch-IC-mechanic:before{content:\"\\e1da\"}.Bosch-IC-megaphone:before{content:\"\\e1db\"}.Bosch-Ic-message:before{content:\"\\e1dc\"}.Bosch-IC-mic:before{content:\"\\e1dd\"}.Bosch-Ic-microphone:before{content:\"\\e1de\"}.Bosch-IC-microphone-classic:before{content:\"\\e1df\"}.Bosch-IC-minimize:before{content:\"\\e1e0\"}.Bosch-IC-money-currency:before{content:\"\\e1e1\"}.Bosch-IC-money-dollar:before{content:\"\\e1e2\"}.Bosch-IC-moneybag-currency:before{content:\"\\e1e3\"}.Bosch-IC-moneybag-dollar:before{content:\"\\e1e4\"}.Bosch-IC-monument:before{content:\"\\e1e5\"}.Bosch-IC-moon:before{content:\"\\e1e6\"}.Bosch-IC-mouse:before{content:\"\\e1ea\"}.Bosch-IC-mouse-left:before{content:\"\\e1eb\"}.Bosch-IC-mouse-right:before{content:\"\\e1ec\"}.Bosch-IC-mouth:before{content:\"\\e1ed\"}.Bosch-Ic-my-product:before{content:\"\\e1ee\"}.Bosch-IC-new-way-work:before{content:\"\\e1ef\"}.Bosch-Ic-mail-sent:before{content:\"\\e1f0\"}.Bosch-Ic-news:before{content:\"\\e1f1\"}.Bosch-IC-nose:before{content:\"\\e1f2\"}.Bosch-Ic-notes:before{content:\"\\e1f3\"}.Bosch-Ic-notes-edit:before{content:\"\\e1f4\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-IC-notification-active:before{content:\"\\e1f6\"}.Bosch-IC-notification-add:before{content:\"\\e1f7\"}.Bosch-IC-notification-remove:before{content:\"\\e1f8\"}.Bosch-Ic-oil-trailer:before{content:\"\\e1f9\"}.Bosch-IC-omnichannel:before{content:\"\\e1fa\"}.Bosch-Ic-show-more-horizontal:before{content:\"\\e1fb\"}.Bosch-IC-origami-boat:before{content:\"\\e1fc\"}.Bosch-Ic-palette-trailer:before{content:\"\\e1fd\"}.Bosch-IC-pallete-car-connected:before{content:\"\\e1fe\"}.Bosch-IC-panel:before{content:\"\\e1ff\"}.Bosch-Ic-paperclip:before{content:\"\\e200\"}.Bosch-IC-paperplane:before{content:\"\\e201\"}.Bosch-IC-parking:before{content:\"\\e202\"}.Bosch-Ic-pause:before{content:\"\\e203\"}.Bosch-Ic-creditcard:before{content:\"\\e204\"}.Bosch-IC-people:before{content:\"\\e205\"}.Bosch-IC-petrol-station:before{content:\"\\e206\"}.Bosch-IC-piggybank:before{content:\"\\e207\"}.Bosch-Ic-pin:before{content:\"\\e208\"}.Bosch-IC-pin-modern:before{content:\"\\e209\"}.Bosch-IC-plane-side:before{content:\"\\e20a\"}.Bosch-Ic-play:before{content:\"\\e20b\"}.Bosch-IC-plug-12V:before{content:\"\\e20c\"}.Bosch-IC-podium-speaker:before{content:\"\\e20d\"}.Bosch-IC-police:before{content:\"\\e20e\"}.Bosch-IC-poop:before{content:\"\\e20f\"}.Bosch-IC-post-it:before{content:\"\\e210\"}.Bosch-IC-postal-code:before{content:\"\\e211\"}.Bosch-IC-power:before{content:\"\\e212\"}.Bosch-Ic-print:before{content:\"\\e213\"}.Bosch-IC-prototyping:before{content:\"\\e214\"}.Bosch-IC-puzzle:before{content:\"\\e215\"}.Bosch-IC-quad:before{content:\"\\e216\"}.Bosch-IC-question:before{content:\"\\e217\"}.Bosch-IC-radar:before{content:\"\\e218\"}.Bosch-IC-radio:before{content:\"\\e219\"}.Bosch-Ic-radio:before{content:\"\\e21a\"}.Bosch-IC-redirect:before{content:\"\\e21b\"}.Bosch-Ic-redo:before{content:\"\\e21c\"}.Bosch-IC-reference:before{content:\"\\e21d\"}.Bosch-Ic-refresh:before{content:\"\\e21e\"}.Bosch-IC-refresh-cloud:before{content:\"\\e21f\"}.Bosch-Ic-registration:before{content:\"\\e220\"}.Bosch-IC-remote:before{content:\"\\e221\"}.Bosch-Ic-reset:before{content:\"\\e222\"}.Bosch-IC-resolution:before{content:\"\\e223\"}.Bosch-Ic-robot-automation:before{content:\"\\e224\"}.Bosch-IC-robot-connected:before{content:\"\\e225\"}.Bosch-IC-robothead:before{content:\"\\e226\"}.Bosch-IC-rocket:before{content:\"\\e227\"}.Bosch-Ic-rotation:before{content:\"\\e228\"}.Bosch-Ic-rotate-360-degrees:before{content:\"\\e229\"}.Bosch-Ic-rotate-clockwise:before{content:\"\\e22a\"}.Bosch-Ic-rotate-counterclockwise:before{content:\"\\e22b\"}.Bosch-IC-rotation-y-down:before{content:\"\\e22c\"}.Bosch-IC-rotation-y-up:before{content:\"\\e22d\"}.Bosch-Ic-navigation:before{content:\"\\e22e\"}.Bosch-Ic-measure:before{content:\"\\e22f\"}.Bosch-IC-ruler-pen:before{content:\"\\e230\"}.Bosch-Ic-satelite:before{content:\"\\e231\"}.Bosch-Ic-save:before{content:\"\\e232\"}.Bosch-IC-scale:before{content:\"\\e233\"}.Bosch-Ic-cut:before{content:\"\\e234\"}.Bosch-Ic-search:before{content:\"\\e235\"}.Bosch-IC-security:before{content:\"\\e236\"}.Bosch-IC-security-check:before{content:\"\\e237\"}.Bosch-IC-security-user:before{content:\"\\e238\"}.Bosch-Ic-sensor:before{content:\"\\e239\"}.Bosch-Ic-server:before{content:\"\\e23a\"}.Bosch-IC-server-expansion:before{content:\"\\e23b\"}.Bosch-IC-server-rate:before{content:\"\\e23c\"}.Bosch-IC-service-agent:before{content:\"\\e23d\"}.Bosch-IC-service-time:before{content:\"\\e23e\"}.Bosch-Ic-settings:before{content:\"\\e23f\"}.Bosch-IC-settings-arrows:before{content:\"\\e240\"}.Bosch-IC-settings-connected:before{content:\"\\e241\"}.Bosch-Ic-share-outline:before{content:\"\\e242\"}.Bosch-IC-shield-stop:before{content:\"\\e243\"}.Bosch-IC-ship:before{content:\"\\e244\"}.Bosch-IC-ship-side:before{content:\"\\e245\"}.Bosch-Ic-shoppingcart:before{content:\"\\e246\"}.Bosch-IC-shoppingcart-add:before{content:\"\\e247\"}.Bosch-IC-shoppingcart-switch:before{content:\"\\e248\"}.Bosch-IC-signal-full:before{content:\"\\e249\"}.Bosch-IC-signal-half:before{content:\"\\e24a\"}.Bosch-IC-signal-lost:before{content:\"\\e24b\"}.Bosch-IC-signpost:before{content:\"\\e24c\"}.Bosch-IC-simcard:before{content:\"\\e24d\"}.Bosch-IC-skyscraper:before{content:\"\\e24e\"}.Bosch-IC-smartcity:before{content:\"\\e24f\"}.Bosch-IC-smarthome:before{content:\"\\e250\"}.Bosch-Ic-phone:before{content:\"\\e251\"}.Bosch-IC-smartphone-acoustic:before{content:\"\\e252\"}.Bosch-IC-smartphone-arrow-right:before{content:\"\\e253\"}.Bosch-IC-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.Bosch-IC-smartphone-clock:before{content:\"\\e255\"}.Bosch-IC-smartphone-cloud:before{content:\"\\e256\"}.Bosch-IC-smartphone-loading-wheel:before{content:\"\\e257\"}.Bosch-IC-smartphone-smashed:before{content:\"\\e258\"}.Bosch-IC-smartphone-vibration:before{content:\"\\e259\"}.Bosch-IC-smartphone-z:before{content:\"\\e25a\"}.Bosch-Ic-spareparts:before{content:\"\\e25c\"}.Bosch-IC-spark-plug:before{content:\"\\e25d\"}.Bosch-IC-speaker-acoustic:before{content:\"\\e25e\"}.Bosch-IC-speedometer:before{content:\"\\e25f\"}.Bosch-IC-stadium:before{content:\"\\e260\"}.Bosch-IC-stairs:before{content:\"\\e261\"}.Bosch-IC-stamp:before{content:\"\\e262\"}.Bosch-IC-standby:before{content:\"\\e263\"}.Bosch-IC-steering-wheel:before{content:\"\\e264\"}.Bosch-IC-steering-wheel-connected:before{content:\"\\e265\"}.Bosch-IC-steering-wheel-hands:before{content:\"\\e266\"}.Bosch-IC-steps:before{content:\"\\e267\"}.Bosch-IC-stethoscope:before{content:\"\\e268\"}.Bosch-Ic-stop:before{content:\"\\e269\"}.Bosch-IC-stopwatch:before{content:\"\\e26a\"}.Bosch-IC-storage-local:before{content:\"\\e26b\"}.Bosch-IC-structure:before{content:\"\\e26c\"}.Bosch-IC-subtitles:before{content:\"\\e26d\"}.Bosch-Ic-summary:before{content:\"\\e26f\"}.Bosch-IC-sun:before{content:\"\\e270\"}.Bosch-IC-sun-half-filled:before{content:\"\\e271\"}.Bosch-IC-sunshade-table:before{content:\"\\e272\"}.Bosch-IC-surveillance:before{content:\"\\e273\"}.Bosch-IC-table-chairs:before{content:\"\\e274\"}.Bosch-Ic-tablet:before{content:\"\\e275\"}.Bosch-IC-team:before{content:\"\\e276\"}.Bosch-IC-team-lightbulb:before{content:\"\\e277\"}.Bosch-IC-technical-service:before{content:\"\\e278\"}.Bosch-Ic-temperature:before{content:\"\\e279\"}.Bosch-IC-temperature-high:before{content:\"\\e27a\"}.Bosch-IC-temperature-low:before{content:\"\\e27b\"}.Bosch-IC-theater:before{content:\"\\e27c\"}.Bosch-Ic-dislike:before{content:\"\\e27d\"}.Bosch-Ic-like:before{content:\"\\e27e\"}.Bosch-Ic-ticket:before{content:\"\\e27f\"}.Bosch-IC-touch:before{content:\"\\e280\"}.Bosch-IC-touch-sos:before{content:\"\\e281\"}.Bosch-IC-towing-truck:before{content:\"\\e282\"}.Bosch-IC-train:before{content:\"\\e283\"}.Bosch-IC-train-side:before{content:\"\\e284\"}.Bosch-Ic-transport-movements:before{content:\"\\e285\"}.Bosch-IC-transscript:before{content:\"\\e286\"}.Bosch-IC-transscript-off:before{content:\"\\e287\"}.Bosch-IC-transscript-on:before{content:\"\\e288\"}.Bosch-IC-tree:before{content:\"\\e289\"}.Bosch-IC-tricycle:before{content:\"\\e28a\"}.Bosch-Ic-default-trailer:before{content:\"\\e28b\"}.Bosch-Ic-taxi-trailer:before{content:\"\\e28c\"}.Bosch-Ic-shelve-trailer:before{content:\"\\e28d\"}.Bosch-IC-trophy:before{content:\"\\e28e\"}.Bosch-IC-truck:before{content:\"\\e28f\"}.Bosch-IC-tuktuk:before{content:\"\\e290\"}.Bosch-Ic-tune:before{content:\"\\e291\"}.Bosch-IC-typography:before{content:\"\\e292\"}.Bosch-IC-ui:before{content:\"\\e293\"}.Bosch-IC-umbrella:before{content:\"\\e294\"}.Bosch-Ic-undo:before{content:\"\\e295\"}.Bosch-Ic-arrow-expand-up:before{content:\"\\e296\"}.Bosch-Ic-arrow-expand-double-up:before{content:\"\\e297\"}.Bosch-Ic-sort:before{content:\"\\e298\"}.Bosch-Ic-upload:before{content:\"\\e299\"}.Bosch-IC-usb:before{content:\"\\e29a\"}.Bosch-Ic-user-outline:before{content:\"\\e29b\"}.Bosch-IC-user-advanced:before{content:\"\\e29c\"}.Bosch-IC-user-beginner:before{content:\"\\e29d\"}.Bosch-IC-user-desktop:before{content:\"\\e29e\"}.Bosch-IC-user-down:before{content:\"\\e29f\"}.Bosch-IC-user-falling:before{content:\"\\e2a0\"}.Bosch-IC-user-hand:before{content:\"\\e2a1\"}.Bosch-IC-user-hand-stop:before{content:\"\\e2a2\"}.Bosch-IC-user-professional:before{content:\"\\e2a3\"}.Bosch-IC-user-run:before{content:\"\\e2a4\"}.Bosch-IC-user-share:before{content:\"\\e2a5\"}.Bosch-IC-user-voice:before{content:\"\\e2a6\"}.Bosch-IC-user-walk:before{content:\"\\e2a7\"}.Bosch-Ic-videocam:before{content:\"\\e2a8\"}.Bosch-Ic-videocam-disabled:before{content:\"\\e2a9\"}.Bosch-IC-video-record:before{content:\"\\e2aa\"}.Bosch-IC-virtual-reality:before{content:\"\\e2ab\"}.Bosch-Ic-volume-disabled:before{content:\"\\e2ac\"}.Bosch-Ic-volume-high:before{content:\"\\e2ad\"}.Bosch-Ic-volume-low:before{content:\"\\e2ae\"}.Bosch-Ic-volume-off:before{content:\"\\e2af\"}.Bosch-Ic-magic-wand:before{content:\"\\e2b0\"}.Bosch-Ic-hidden-project:before{content:\"\\e2b2\"}.Bosch-Ic-show:before{content:\"\\e2b3\"}.Bosch-IC-water-shower:before{content:\"\\e2b4\"}.Bosch-IC-welcome:before{content:\"\\e2b5\"}.Bosch-IC-wellness:before{content:\"\\e2b6\"}.Bosch-IC-wheelchair:before{content:\"\\e2b7\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-IC-wide-angled-arrow:before{content:\"\\e2b9\"}.Bosch-Ic-wifi:before{content:\"\\e2ba\"}.Bosch-Ic-window-analysis:before{content:\"\\e2bb\"}.Bosch-Ic-window-browser:before{content:\"\\e2bc\"}.Bosch-Ic-window-console:before{content:\"\\e2bd\"}.Bosch-Ic-gaussian-view:before{content:\"\\e2be\"}.Bosch-Ic-window-info:before{content:\"\\e2bf\"}.Bosch-IC-window-new:before{content:\"\\e2c0\"}.Bosch-IC-window-resize:before{content:\"\\e2c1\"}.Bosch-Ic-general-scaling-view:before{content:\"\\e2c2\"}.Bosch-Ic-restore-window:before{content:\"\\e2c3\"}.Bosch-Ic-window-statistic:before{content:\"\\e2c4\"}.Bosch-Ic-wishlist:before{content:\"\\e2c5\"}.Bosch-Ic-work-orders:before{content:\"\\e2c6\"}.Bosch-IC-worldwideweb:before{content:\"\\e2c7\"}.Bosch-Ic-wrench:before{content:\"\\e2c8\"}.Bosch-Ic-zoom-in:before{content:\"\\e2c9\"}.Bosch-Ic-zoom-out:before{content:\"\\e2ca\"}bci-timepicker{display:block;display:flex;flex-direction:column;justify-content:stretch}bci-timepicker.full-height{height:100%}bci-timepicker:nth-of-type(2){border-top:1px solid #8a9097}.bci-core-timepicker-title{flex-wrap:nowrap;font-size:12px;line-height:1.5;padding-top:16px;padding-left:10px;font-weight:700}.bci-core-timepicker-container{display:flex;justify-content:center;padding:8px;margin:auto 0}.bci-core-timepicker-container .bosch-ic-clickable{color:#000000}.bci-core-timepicker-container .bosch-ic-clickable:hover{color:#007bc0}.bci-core-timepicker-container .bosch-ic-clickable:active{color:#00629a}.bci-core-input-container{display:flex;flex-direction:column;align-items:center}.bci-core-input-container.divider{align-self:center;padding:0 10px 7px 10px;font-size:16px;font-weight:400}.bci-core-input-container .arrow-button{color:#000000}.bci-core-input-container input::-webkit-outer-spin-button,.bci-core-input-container input::-webkit-inner-spin-button{-webkit-appearance:none;margin:0}.bci-core-input-container input{height:48px;width:48px;padding:0px 1px 0px 1px;text-align:center;font-size:16px;line-height:24px;font-weight:400;color:#000000;background-color:#e0e2e5;border:0;border-bottom:1px solid black;outline:none;appearance:textfield}.bci-core-input-container input:hover{background-color:#c1c7cc}.bci-core-input-container input:active{background-color:#a4abb3}.bci-core-input-container input:focus,.bci-core-input-container input:focus-within{background-color:#d1e4ff}.bci-core-input-container.no-seconds input{width:73px}.bci-core-input-container.no-secondsandmerdiam input{width:123px}.hidden{display:none}";

var TimeDirection;
(function (TimeDirection) {
  TimeDirection[TimeDirection["UP"] = 0] = "UP";
  TimeDirection[TimeDirection["DOWN"] = 1] = "DOWN";
})(TimeDirection || (TimeDirection = {}));
var MeridiemValue;
(function (MeridiemValue) {
  MeridiemValue["AM"] = "AM";
  MeridiemValue["PM"] = "PM";
})(MeridiemValue || (MeridiemValue = {}));
const Timepicker = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.updatedTime = index.createEvent(this, "updatedTime", 7);
    this.showMeridiem = true;
    this.meridiem = MeridiemValue.AM;
  }
  handleTimeInput(newValue) {
    this.hour = !!newValue
      ? utils.format(parse(newValue, this.timeFormat, new Date()), utils.timeFormatHasMeridiem(this.timeFormat) ? 'hh' : 'HH')
      : utils.timeFormatHasMeridiem(this.timeFormat)
        ? null
        : null;
    this.hour = this.formatTime(this.hour);
    this.minutes = !!newValue ? utils.format(parse(newValue, this.timeFormat, new Date()), 'mm') : null;
    this.minutes = this.formatTime(this.minutes);
    this.seconds = !!newValue ? utils.format(parse(newValue, this.timeFormat, new Date()), 'ss') : null;
    this.seconds = this.formatTime(this.seconds);
    this.showMeridiem = !utils.timeFormatHasMeridiem(this.timeFormat);
    this.hideSeconds = !utils.isHideSeconds(this.timeFormat);
    this.meridiem = (newValue === null || newValue === void 0 ? void 0 : newValue.includes(MeridiemValue.PM)) ? MeridiemValue.PM : MeridiemValue.AM;
  }
  componentDidLoad() {
    if (isNaN(Date.parse(this.timeInput))) {
      this.handleTimeInput(this.timeInput);
    }
  }
  getUpdatedTime() {
    if (this.hour || this.minutes || this.seconds) {
      const newTime = `${!!this.hour ? this.hour : utils.timeFormatHasMeridiem(this.timeFormat) ? '01' : '00'}:${this.minutes || '00'}:${this.seconds || '00'}${utils.timeFormatHasMeridiem(this.timeFormat) ? ' ' + this.meridiem : ''}`;
      return utils.format(parse(newTime, utils.timeFormatHasMeridiem(this.timeFormat) ? 'hh:mm:ss a' : 'HH:mm:ss', new Date()), this.timeFormat);
    }
    else {
      return null;
    }
  }
  emitUpdatedTime(isDelete) {
    this.updatedTime.emit({
      updatedTime: isDelete ? null : this.getUpdatedTime(),
      timepickerId: this.bciTimepickerId,
    });
  }
  updateTimeOnType(ev, hmsmr, maxValue, minValue = 0) {
    let newTime;
    newTime = ev.target.value;
    if (!newTime || isNaN(newTime) || newTime === '') {
      newTime = null;
    }
    else {
      newTime = parseInt(newTime, 10);
      newTime = Math.min(newTime, maxValue);
      newTime = Math.max(newTime, minValue);
      newTime = this.formatTime(newTime);
    }
    switch (hmsmr) {
      case 'h':
        this.hour = !!newTime ? String(newTime) : newTime;
        this.hourInput.value = `${newTime}`;
        break;
      case 'm':
        this.minutes = !!newTime ? String(newTime) : newTime;
        this.minutesInput.value = `${newTime}`;
        break;
      case 's':
        this.seconds = !!newTime ? String(newTime) : newTime;
        this.secondsInput.value = `${newTime}`;
        break;
    }
    if (!newTime && typeof newTime !== 'number') {
      this.emitUpdatedTime(true);
    }
    else {
      this.emitUpdatedTime();
    }
  }
  handleInputType(ev, hmsmr, maxValue, minValue = 0) {
    let newTime = String(ev.target.value);
    /** Wheter the event is triggered by pressing number-keys on the keyboard.
     * Otherwise this handler is triggered by pressing ArrowUp/ArrowDown keys. */
    const isInputEvent = ev instanceof InputEvent;
    // automatically move to next input if input is valid
    const newTimeAsInt = parseInt(newTime, 10);
    if (isInputEvent) {
      if (newTime.length === 2 && newTimeAsInt >= minValue && newTimeAsInt <= maxValue) {
        this.selectNextInput(hmsmr);
      }
    }
    else {
      ev.target.select();
    }
    // prevent negative values
    newTime = newTime.replace('-', '');
    // prevent more than 2 digits
    if (newTime.length > 2) {
      newTime = newTime.substring(0, 2);
    }
    switch (hmsmr) {
      case 'h':
        this.hourInput.value = `${newTime}`;
        break;
      case 'm':
        this.minutesInput.value = `${newTime}`;
        break;
      case 's':
        this.secondsInput.value = `${newTime}`;
        break;
    }
  }
  handleKeyPressed(ev, hmsmr, maxValue, minValue = 0) {
    if (ev.key === 'Enter') {
      this.updateTimeOnType(ev, hmsmr, maxValue, minValue);
      this.selectNextInput(hmsmr);
    }
  }
  selectNextInput(hmsmr) {
    switch (hmsmr) {
      case 'h':
        this.minutesInput.select();
        break;
      case 'm':
        this.secondsInput.select();
        break;
      case 's':
        if (utils.timeFormatHasMeridiem(this.timeFormat))
          this.meridiemInput.focus();
        break;
    }
  }
  formatTime(value) {
    if (!value && typeof value !== 'number')
      return null;
    return String(value).padStart(2, '0');
  }
  updateTimeOnClick(direction, value, maxValue, minValue = 0) {
    if (!value) {
      value = direction === TimeDirection.UP ? minValue : maxValue;
      return this.formatTime(value);
    }
    value = parseInt(value, 10);
    if (direction === TimeDirection.UP) {
      value = value === maxValue ? minValue : value + 1;
    }
    if (direction === TimeDirection.DOWN) {
      value = value === minValue ? maxValue : value - 1;
    }
    return this.formatTime(value);
  }
  updateMeridiemOnClick() {
    this.meridiem = this.meridiem === MeridiemValue.AM ? MeridiemValue.PM : MeridiemValue.AM;
  }
  render() {
    return (index.h(index.Host, null, index.h("div", { class: "bci-core-timepicker-title" }, this.timeTitle), index.h("div", { class: "bci-core-timepicker-container" }, index.h("div", { class: {
        'bci-core-input-container': true,
        rangeTimePicker: this.rangeTimePicker,
        'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
        'no-secondsandmerdiam': this.hideSeconds && this.showMeridiem,
      } }, index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
        this.hour = this.updateTimeOnClick(TimeDirection.UP, this.hour, utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.hour-up.span" })), index.h("input", { "data-test": "webcore.webcomponents.timepicker.update-time-on-type", type: "number", placeholder: "HH", step: 1, min: utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0, max: utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, pattern: "[012][0-9]*", "data-hours": "true", onFocus: () => this.hourInput.select(), onInput: (ev) => this.handleInputType(ev, 'h', utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), onKeyPress: (ev) => this.handleKeyPressed(ev, 'h', utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), onBlur: (ev) => this.updateTimeOnType(ev, 'h', utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0), value: this.hour, ref: (el) => (this.hourInput = el) }), index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
        this.hour = this.updateTimeOnClick(TimeDirection.DOWN, this.hour, utils.timeFormatHasMeridiem(this.timeFormat) ? 12 : 23, utils.timeFormatHasMeridiem(this.timeFormat) ? 1 : 0);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.hour-down.span" }))), index.h("div", { class: "bci-core-input-container divider" }, index.h("span", null, ":")), index.h("div", { class: {
        'bci-core-input-container': true,
        rangeTimePicker: this.rangeTimePicker,
        'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
        'no-secondsandmerdiam': this.hideSeconds && this.showMeridiem,
      } }, index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
        this.minutes = this.updateTimeOnClick(TimeDirection.UP, this.minutes, 59);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.minute-up.span" })), index.h("input", { "data-test": "webcore.webcomponents.timepicker.update-time-on-type-minute", min: 0, max: 59, placeholder: "MM", type: "number", step: 1, pattern: "[0-5][0-9]*", "data-minutes": "true", onFocus: () => this.minutesInput.select(), onInput: (ev) => this.handleInputType(ev, 'm', 59), onKeyPress: (ev) => this.handleKeyPressed(ev, 'm', 59), onBlur: (ev) => this.updateTimeOnType(ev, 'm', 59), value: this.minutes, ref: (el) => (this.minutesInput = el) }), index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
        this.minutes = this.updateTimeOnClick(TimeDirection.DOWN, this.minutes, 59);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.minute-down.span" }))), index.h("div", { class: {
        'bci-core-input-container': true,
        divider: true,
        hidden: this.hideSeconds,
      } }, index.h("span", null, ":")), index.h("div", { class: {
        'bci-core-input-container': true,
        hidden: this.hideSeconds,
        rangeTimePicker: this.rangeTimePicker,
        'no-seconds': (this.hideSeconds && !this.showMeridiem) || (!this.hideSeconds && this.timeFormat === 'HH:mm:ss'),
      } }, index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
        this.seconds = this.updateTimeOnClick(TimeDirection.UP, this.seconds, 59);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.second-up.span" })), index.h("input", { "data-test": "webcore.webcomponents.timepicker.bci-core-timepicker-seconds-input", type: "number", placeholder: "SS", step: 1, min: 0, max: 59, pattern: "[0-5][0-9]*", "data-seconds": "true", onFocus: () => this.secondsInput.select(), onInput: (ev) => this.handleInputType(ev, 's', 59), onKeyPress: (ev) => this.handleKeyPressed(ev, 's', 59), onBlur: (ev) => this.updateTimeOnType(ev, 's', 59), value: this.seconds, ref: (el) => (this.secondsInput = el) }), index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
        this.seconds = this.updateTimeOnClick(TimeDirection.DOWN, this.seconds, 59);
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.second-down.span" }))), index.h("div", { class: {
        'bci-core-input-container': true,
        divider: true,
        hidden: this.showMeridiem,
      } }), index.h("div", { class: {
        'bci-core-input-container': true,
        'no-seconds': this.hideSeconds && !this.showMeridiem,
        hidden: this.showMeridiem,
        rangeTimePicker: this.rangeTimePicker,
      } }, index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-up", onClick: () => {
        this.updateMeridiemOnClick();
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.meridiem-up.span" })), index.h("input", { "data-test": "webcore.webcomponents.timepicker.bci-core-timepicker-meridiem-input", type: "text", readonly: "readonly", "data-seconds": "true", onKeyDown: (ev) => {
        if (['ArrowDown', 'ArrowUp'].includes(ev.key)) {
          this.updateMeridiemOnClick();
          this.emitUpdatedTime();
        }
      }, value: this.meridiem, ref: (el) => (this.meridiemInput = el) }), index.h("span", { class: "arrow-button" }, index.h("i", { class: "bosch-ic bosch-ic-clickable bosch-ic-down", onClick: () => {
        this.updateMeridiemOnClick();
        this.emitUpdatedTime();
      }, "data-test": "webcore.webcomponents.timepicker.update-time.meridiem-down.span" }))))));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "timeInput": ["handleTimeInput"]
  }; }
};
Timepicker.style = timepickerCss;

const datetimePickerCss = "/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:var(--bci-global-font-family);font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */body{margin:0;display:flex;flex-direction:column}main,*{font-family:var(--bci-global-font-family)}::selection,::-moz-selection{background-color:#007bc0}a::-moz-selection{color:#ffffff}.lead{margin-bottom:24px;font-size:18px;font-weight:300;line-height:1.4}@media (min-width: 768px){.lead{font-size:24px}}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bosch-ic,.Bosch-Ic{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic:before,.Bosch-Ic:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-large,.Bosch-Ic-Large{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-large:before,.Bosch-Ic-Large:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:28px;vertical-align:middle}.bosch-ic-medium,.Bosch-Ic-Medium{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-medium:before,.Bosch-Ic-Medium:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-normal,.Bosch-Ic-Normal{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-normal:before,.Bosch-Ic-Normal:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:20px;vertical-align:middle}.bosch-ic-small,.Bosch-Ic-Small{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-small:before,.Bosch-Ic-Small:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:16px;vertical-align:middle}.Bosch-Ic-Disabled,.bosch-ic-disabled,.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic[disabled],.bosch-ic[disabled],.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover{cursor:not-allowed;color:#d0d4d8}.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic-Disabled:hover:hover,.bosch-ic-disabled:hover:hover,.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover,.Bosch-Ic[disabled]:hover:hover,.bosch-ic[disabled]:hover:hover{color:#d0d4d8}.bosch-ic-clickable,.Bosch-Ic-Clickable{cursor:pointer}.bosch-ic-clickable:disabled,.bosch-ic-clickable.disabled,.Bosch-Ic-Clickable:disabled,.Bosch-Ic-Clickable.disabled{color:#a4abb3;cursor:not-allowed}.bosch-ic-activated,.Bosch-Ic-Activated{color:#007bc0}.bosch-ic-3d-print:before{content:\"\\e085\"}.bosch-ic-a-to-b:before{content:\"\\e086\"}.bosch-ic-add:before{content:\"\\e087\"}.bosch-ic-address-consumer-data:before{content:\"\\e088\"}.bosch-ic-adress-book:before{content:\"\\e089\"}.bosch-ic-agility:before{content:\"\\e08a\"}.bosch-ic-alarm-clock:before{content:\"\\e08b\"}.bosch-ic-ambulance:before{content:\"\\e08c\"}.bosch-ic-analysis:before{content:\"\\e08d\"}.bosch-ic-anchor:before{content:\"\\e08e\"}.bosch-ic-arrow-down:before{content:\"\\e08f\"}.bosch-ic-arrow-left:before{content:\"\\e090\"}.bosch-ic-arrow-left-down:before{content:\"\\e091\"}.bosch-ic-arrow-left-up:before{content:\"\\e092\"}.bosch-ic-arrow-right:before{content:\"\\e093\"}.bosch-ic-arrow-right-down:before{content:\"\\e094\"}.bosch-ic-arrow-right-up:before{content:\"\\e095\"}.bosch-ic-arrow-turn-left:before{content:\"\\e096\"}.bosch-ic-arrow-turn-right:before{content:\"\\e097\"}.bosch-ic-arrow-up:before{content:\"\\e098\"}.bosch-ic-arrows:before{content:\"\\e099\"}.bosch-ic-arrows-x-y-z:before{content:\"\\e09a\"}.bosch-ic-artificial-intelligence:before{content:\"\\e09b\"}.bosch-ic-assembly-line:before{content:\"\\e09c\"}.bosch-ic-atom:before{content:\"\\e09d\"}.bosch-ic-augmented-reality:before{content:\"\\e09e\"}.bosch-ic-autobod:before{content:\"\\e09f\"}.bosch-ic-back-left:before{content:\"\\e0a0\"}.bosch-ic-back-left-double:before{content:\"\\e0a1\"}.bosch-ic-bar-chart:before{content:\"\\e0a2\"}.bosch-ic-bar-chart-double:before{content:\"\\e0a3\"}.bosch-ic-battery-0:before{content:\"\\e0a4\"}.bosch-ic-battery-1:before{content:\"\\e0a5\"}.bosch-ic-battery-2:before{content:\"\\e0a6\"}.bosch-ic-battery-3:before{content:\"\\e0a7\"}.bosch-ic-battery-4:before{content:\"\\e0a8\"}.bosch-ic-battery-fail:before{content:\"\\e0a9\"}.bosch-ic-battery-li-ion:before{content:\"\\e0aa\"}.bosch-ic-battery-loading:before{content:\"\\e0ab\"}.bosch-ic-bed:before{content:\"\\e0ac\"}.bosch-ic-bicycle:before{content:\"\\e0ad\"}.bosch-ic-bicycle-e:before{content:\"\\e0ae\"}.bosch-ic-blog:before{content:\"\\e0af\"}.bosch-ic-bluetooth:before{content:\"\\e0b0\"}.bosch-ic-bluetooth-le:before{content:\"\\e0b1\"}.bosch-ic-board-speaker:before{content:\"\\e0b2\"}.bosch-ic-boiler-connected:before{content:\"\\e0b3\"}.bosch-ic-book:before{content:\"\\e0b4\"}.bosch-ic-bookmark:before{content:\"\\e0b5\"}.bosch-ic-bookmark-add:before{content:\"\\e0b6\"}.bosch-ic-bookmark-check:before{content:\"\\e0b7\"}.bosch-ic-bookmark-delete:before{content:\"\\e0b8\"}.bosch-ic-books:before{content:\"\\e0b9\"}.bosch-ic-bookshelf:before{content:\"\\e0ba\"}.bosch-ic-box:before{content:\"\\e0bb\"}.bosch-ic-box-closed:before{content:\"\\e0bc\"}.bosch-ic-box-delivery:before{content:\"\\e0bd\"}.bosch-ic-box-delivery-connected:before{content:\"\\e0be\"}.bosch-ic-brake-disk:before{content:\"\\e0bf\"}.bosch-ic-briefcase:before{content:\"\\e0c0\"}.bosch-ic-broom:before{content:\"\\e0c1\"}.bosch-ic-broom-cloud-dirt:before{content:\"\\e0c2\"}.bosch-ic-brush:before{content:\"\\e0c3\"}.bosch-ic-bug:before{content:\"\\e0c4\"}.bosch-ic-building:before{content:\"\\e0c5\"}.bosch-ic-cactus:before{content:\"\\e0c6\"}.bosch-ic-calculate:before{content:\"\\e0c7\"}.bosch-ic-calendar:before{content:\"\\e0c8\"}.bosch-ic-calendar-add:before{content:\"\\e0c9\"}.bosch-ic-calendar-settings:before{content:\"\\e0ca\"}.bosch-ic-call:before{content:\"\\e0cb\"}.bosch-ic-call-add:before{content:\"\\e0cc\"}.bosch-ic-call-remove:before{content:\"\\e0cd\"}.bosch-ic-call-sos:before{content:\"\\e0ce\"}.bosch-ic-call-team:before{content:\"\\e0cf\"}.bosch-ic-call-wifi:before{content:\"\\e0d0\"}.bosch-ic-camera:before{content:\"\\e0d1\"}.bosch-ic-camera-switch:before{content:\"\\e0d2\"}.bosch-ic-car:before{content:\"\\e0d3\"}.bosch-ic-car-clock:before{content:\"\\e0d4\"}.bosch-ic-car-connected:before{content:\"\\e0d5\"}.bosch-ic-car-locator:before{content:\"\\e0d6\"}.bosch-ic-car-mechanic:before{content:\"\\e0d7\"}.bosch-ic-car-rental:before{content:\"\\e0d8\"}.bosch-ic-car-seat-connected:before{content:\"\\e0d9\"}.bosch-ic-car-side:before{content:\"\\e0da\"}.bosch-ic-car-side-user:before{content:\"\\e0db\"}.bosch-ic-cart:before{content:\"\\e0dc\"}.bosch-ic-certificate:before{content:\"\\e0dd\"}.bosch-ic-certificate-pki:before{content:\"\\e0de\"}.bosch-ic-change:before{content:\"\\e0df\"}.bosch-ic-chart-bar:before{content:\"\\e0e0\"}.bosch-ic-chart-check:before{content:\"\\e0e1\"}.bosch-ic-chart-dummy:before{content:\"\\e0e2\"}.bosch-ic-chart-line:before{content:\"\\e0e3\"}.bosch-ic-chat:before{content:\"\\e0e4\"}.bosch-ic-chat-add:before{content:\"\\e0e5\"}.bosch-ic-chat-question-answer:before{content:\"\\e0e6\"}.bosch-ic-chat-remove:before{content:\"\\e0e7\"}.bosch-ic-checklist:before{content:\"\\e0e8\"}.bosch-ic-checkmark:before{content:\"\\e0e9\"}.bosch-ic-chip:before{content:\"\\e0ea\"}.bosch-ic-circle-segment:before{content:\"\\e0eb\"}.bosch-ic-clear-all:before{content:\"\\e0ec\"}.bosch-ic-clipboard:before{content:\"\\e0ed\"}.bosch-ic-clipboard-list:before{content:\"\\e0ee\"}.bosch-ic-clock:before{content:\"\\e0ef\"}.bosch-ic-close:before{content:\"\\e0f0\"}.bosch-ic-close-all:before{content:\"\\e0f1\"}.bosch-ic-cloud:before{content:\"\\e0f2\"}.bosch-ic-cloud-co2:before{content:\"\\e0f3\"}.bosch-ic-cloud-download:before{content:\"\\e0f4\"}.bosch-ic-cloud-nox:before{content:\"\\e0f5\"}.bosch-ic-cloud-upload:before{content:\"\\e0f6\"}.bosch-ic-club-arm-chair:before{content:\"\\e0f7\"}.bosch-ic-coffee-break:before{content:\"\\e0f8\"}.bosch-ic-coin-currency:before{content:\"\\e0f9\"}.bosch-ic-coin-dollar:before{content:\"\\e0fa\"}.bosch-ic-colorpicker:before{content:\"\\e0fb\"}.bosch-ic-colors:before{content:\"\\e0fc\"}.bosch-ic-command:before{content:\"\\e0fd\"}.bosch-ic-communicator:before{content:\"\\e0fe\"}.bosch-ic-compare:before{content:\"\\e0ff\"}.bosch-ic-components:before{content:\"\\e100\"}.bosch-ic-components-available:before{content:\"\\e101\"}.bosch-ic-components-reservation:before{content:\"\\e102\"}.bosch-ic-concierge-bell:before{content:\"\\e103\"}.bosch-ic-configuration:before{content:\"\\e104\"}.bosch-ic-confirmation:before{content:\"\\e105\"}.bosch-ic-connection-off:before{content:\"\\e106\"}.bosch-ic-connection-on:before{content:\"\\e107\"}.bosch-ic-connectivity:before{content:\"\\e108\"}.bosch-ic-construction:before{content:\"\\e109\"}.bosch-ic-consumer-goods:before{content:\"\\e10a\"}.bosch-ic-contract:before{content:\"\\e10b\"}.bosch-ic-copy:before{content:\"\\e10c\"}.bosch-ic-cube:before{content:\"\\e10d\"}.bosch-ic-cube-arrows:before{content:\"\\e10e\"}.bosch-ic-cube-filled:before{content:\"\\e10f\"}.bosch-ic-cube-shock:before{content:\"\\e110\"}.bosch-ic-cube-stacked:before{content:\"\\e111\"}.bosch-ic-customerservice:before{content:\"\\e112\"}.bosch-ic-cutlery:before{content:\"\\e113\"}.bosch-ic-damper:before{content:\"\\e114\"}.bosch-ic-danger-flash:before{content:\"\\e115\"}.bosch-ic-data-loss:before{content:\"\\e116\"}.bosch-ic-dealer-details:before{content:\"\\e117\"}.bosch-ic-delete:before{content:\"\\e118\"}.bosch-ic-delivery:before{content:\"\\e119\"}.bosch-ic-denied:before{content:\"\\e11a\"}.bosch-ic-desktop:before{content:\"\\e11b\"}.bosch-ic-desktop-application-2d:before{content:\"\\e11c\"}.bosch-ic-desktop-application-3d:before{content:\"\\e11d\"}.bosch-ic-development:before{content:\"\\e11e\"}.bosch-ic-devices:before{content:\"\\e11f\"}.bosch-ic-devices-mobile:before{content:\"\\e120\"}.bosch-ic-directory:before{content:\"\\e121\"}.bosch-ic-dna:before{content:\"\\e122\"}.bosch-ic-do-not-disturb:before{content:\"\\e123\"}.bosch-ic-document:before{content:\"\\e124\"}.bosch-ic-document-add:before{content:\"\\e125\"}.bosch-ic-document-arrow-down:before{content:\"\\e126\"}.bosch-ic-document-arrow-up:before{content:\"\\e127\"}.bosch-ic-document-audio:before{content:\"\\e128\"}.bosch-ic-document-check:before{content:\"\\e129\"}.bosch-ic-document-cloud:before{content:\"\\e12a\"}.bosch-ic-document-copy:before{content:\"\\e12b\"}.bosch-ic-document-delete:before{content:\"\\e12c\"}.bosch-ic-document-doc:before{content:\"\\e12d\"}.bosch-ic-document-edit:before{content:\"\\e12e\"}.bosch-ic-document-error:before{content:\"\\e12f\"}.bosch-ic-document-locked:before{content:\"\\e130\"}.bosch-ic-document-log:before{content:\"\\e131\"}.bosch-ic-document-one:before{content:\"\\e132\"}.bosch-ic-document-pdf:before{content:\"\\e133\"}.bosch-ic-document-plain:before{content:\"\\e134\"}.bosch-ic-document-plain-add:before{content:\"\\e135\"}.bosch-ic-document-plain-delete:before{content:\"\\e136\"}.bosch-ic-document-ppt:before{content:\"\\e137\"}.bosch-ic-document-pub:before{content:\"\\e138\"}.bosch-ic-document-rtf:before{content:\"\\e139\"}.bosch-ic-document-save-to:before{content:\"\\e13a\"}.bosch-ic-document-search:before{content:\"\\e13b\"}.bosch-ic-document-settings:before{content:\"\\e13c\"}.bosch-ic-document-test:before{content:\"\\e13d\"}.bosch-ic-document-text:before{content:\"\\e13e\"}.bosch-ic-document-txt:before{content:\"\\e13f\"}.bosch-ic-document-vsd:before{content:\"\\e140\"}.bosch-ic-document-xls:before{content:\"\\e141\"}.bosch-ic-document-xml:before{content:\"\\e142\"}.bosch-ic-document-zip:before{content:\"\\e143\"}.bosch-ic-dome:before{content:\"\\e144\"}.bosch-ic-door:before{content:\"\\e145\"}.bosch-ic-door-sensor:before{content:\"\\e146\"}.bosch-ic-down:before{content:\"\\e147\"}.bosch-ic-down-double:before{content:\"\\e148\"}.bosch-ic-download:before{content:\"\\e149\"}.bosch-ic-drag-handle:before{content:\"\\e14a\"}.bosch-ic-dragdrop:before{content:\"\\e14b\"}.bosch-ic-drop:before{content:\"\\e14c\"}.bosch-ic-e-charging:before{content:\"\\e14d\"}.bosch-ic-e-mobility:before{content:\"\\e14e\"}.bosch-ic-ear-off-disabled-light:before{content:\"\\e14f\"}.bosch-ic-ear-on:before{content:\"\\e150\"}.bosch-ic-eco-system:before{content:\"\\e151\"}.bosch-ic-edit:before{content:\"\\e152\"}.bosch-ic-education:before{content:\"\\e153\"}.bosch-ic-efficiency:before{content:\"\\e154\"}.bosch-ic-elevator:before{content:\"\\e155\"}.bosch-ic-elevator-alarm:before{content:\"\\e156\"}.bosch-ic-elevator-cloud:before{content:\"\\e157\"}.bosch-ic-elevator-headset:before{content:\"\\e158\"}.bosch-ic-elevator-service:before{content:\"\\e159\"}.bosch-ic-emergency-exit:before{content:\"\\e15a\"}.bosch-ic-emoji-happy:before{content:\"\\e15b\"}.bosch-ic-emoji-neutral:before{content:\"\\e15c\"}.bosch-ic-emoji-sad:before{content:\"\\e15d\"}.bosch-ic-emoji-super-happy:before{content:\"\\e15e\"}.bosch-ic-emoji-very-sad:before{content:\"\\e15f\"}.bosch-ic-eu-energy-label:before{content:\"\\e160\"}.bosch-ic-excavator:before{content:\"\\e161\"}.bosch-ic-exit:before{content:\"\\e163\"}.bosch-ic-expansion-arrows:before{content:\"\\e164\"}.bosch-ic-explosion:before{content:\"\\e165\"}.bosch-ic-export:before{content:\"\\e166\"}.bosch-ic-externallink:before{content:\"\\e167\"}.bosch-ic-fast-forward:before{content:\"\\e168\"}.bosch-ic-faucet:before{content:\"\\e169\"}.bosch-ic-favorites:before{content:\"\\e16a\"}.bosch-ic-fax:before{content:\"\\e16b\"}.bosch-ic-female:before{content:\"\\e16c\"}.bosch-ic-film:before{content:\"\\e16d\"}.bosch-ic-filter:before{content:\"\\e16e\"}.bosch-ic-filter-success:before{content:\"\\e16f\"}.bosch-ic-fingerprint:before{content:\"\\e170\"}.bosch-ic-fire:before{content:\"\\e171\"}.bosch-ic-fire-emergency:before{content:\"\\e172\"}.bosch-ic-fireworks:before{content:\"\\e173\"}.bosch-ic-first-aid:before{content:\"\\e174\"}.bosch-ic-first-aid-cross:before{content:\"\\e175\"}.bosch-ic-fit-to-sceen:before{content:\"\\e176\"}.bosch-ic-flag:before{content:\"\\e177\"}.bosch-ic-flash:before{content:\"\\e178\"}.bosch-ic-flask:before{content:\"\\e179\"}.bosch-ic-flexpress:before{content:\"\\e17a\"}.bosch-ic-folder:before{content:\"\\e17b\"}.bosch-ic-folder-download:before{content:\"\\e17c\"}.bosch-ic-folder-open:before{content:\"\\e17d\"}.bosch-ic-folder-upload:before{content:\"\\e17e\"}.bosch-ic-fork-lift:before{content:\"\\e17f\"}.bosch-ic-fork-lift-locator:before{content:\"\\e180\"}.bosch-ic-forward-right:before{content:\"\\e181\"}.bosch-ic-forward-right-double:before{content:\"\\e182\"}.bosch-ic-full-empty:before{content:\"\\e183\"}.bosch-ic-fullscreen:before{content:\"\\e184\"}.bosch-ic-fullscreen-arrows:before{content:\"\\e185\"}.bosch-ic-fullscreen-exit:before{content:\"\\e186\"}.bosch-ic-g-arrow-down:before{content:\"\\e187\"}.bosch-ic-g-arrow-up:before{content:\"\\e188\"}.bosch-ic-generator:before{content:\"\\e189\"}.bosch-ic-gift:before{content:\"\\e18a\"}.bosch-ic-glance:before{content:\"\\e18b\"}.bosch-ic-glas-plate:before{content:\"\\e18c\"}.bosch-ic-globe:before{content:\"\\e18d\"}.bosch-ic-globe-arrow:before{content:\"\\e18e\"}.bosch-ic-glossary:before{content:\"\\e18f\"}.bosch-ic-grid-view:before{content:\"\\e190\"}.bosch-ic-hammer:before{content:\"\\e191\"}.bosch-ic-hand:before{content:\"\\e192\"}.bosch-ic-hand-motion:before{content:\"\\e193\"}.bosch-ic-hand-motion-off-disabled-light:before{content:\"\\e194\"}.bosch-ic-handlewithcare:before{content:\"\\e195\"}.bosch-ic-handover-add:before{content:\"\\e196\"}.bosch-ic-handover-check:before{content:\"\\e197\"}.bosch-ic-handover-remove:before{content:\"\\e198\"}.bosch-ic-hanger:before{content:\"\\e199\"}.bosch-ic-health:before{content:\"\\e19a\"}.bosch-ic-heart:before{content:\"\\e19b\"}.bosch-ic-history:before{content:\"\\e19c\"}.bosch-ic-home:before{content:\"\\e19d\"}.bosch-ic-home-locator:before{content:\"\\e19e\"}.bosch-ic-hourglass:before{content:\"\\e19f\"}.bosch-ic-hydrant:before{content:\"\\e1a0\"}.bosch-ic-ice:before{content:\"\\e1a1\"}.bosch-ic-imagery:before{content:\"\\e1a2\"}.bosch-ic-imagery-add:before{content:\"\\e1a3\"}.bosch-ic-imagery-remove:before{content:\"\\e1a4\"}.bosch-ic-import:before{content:\"\\e1a5\"}.bosch-ic-imprint:before{content:\"\\e1a6\"}.bosch-ic-impulse:before{content:\"\\e1a7\"}.bosch-ic-industry:before{content:\"\\e1a8\"}.bosch-ic-industry-clock:before{content:\"\\e1a9\"}.bosch-ic-industry-connected:before{content:\"\\e1aa\"}.bosch-ic-info:before{content:\"\\e1ab\"}.bosch-ic-info-i:before{content:\"\\e1ac\"}.bosch-ic-interval:before{content:\"\\e1ad\"}.bosch-ic-it-device:before{content:\"\\e1ae\"}.bosch-ic-jewel:before{content:\"\\e1af\"}.bosch-ic-keyboard:before{content:\"\\e1b0\"}.bosch-ic-label:before{content:\"\\e1b1\"}.bosch-ic-laptop:before{content:\"\\e1b2\"}.bosch-ic-laser:before{content:\"\\e1b3\"}.bosch-ic-layout:before{content:\"\\e1b4\"}.bosch-ic-leaf:before{content:\"\\e1b5\"}.bosch-ic-led:before{content:\"\\e1b6\"}.bosch-ic-brick:before{content:\"\\e1b7\"}.bosch-ic-brick-slanting-1:before{content:\"\\e1b8\"}.bosch-ic-brick-slanting-2:before{content:\"\\e1b9\"}.bosch-ic-less-minimize:before{content:\"\\e1ba\"}.bosch-ic-lightbulb:before{content:\"\\e1bb\"}.bosch-ic-lightbulb-off:before{content:\"\\e1bc\"}.bosch-ic-line-chart:before{content:\"\\e1bd\"}.bosch-ic-link:before{content:\"\\e1be\"}.bosch-ic-link-broken:before{content:\"\\e1bf\"}.bosch-ic-list-view:before{content:\"\\e1c0\"}.bosch-ic-list-view-mobile:before{content:\"\\e1c1\"}.bosch-ic-livechat:before{content:\"\\e1c2\"}.bosch-ic-locator:before{content:\"\\e1c3\"}.bosch-ic-locator-ip-connected:before{content:\"\\e1c4\"}.bosch-ic-locator-ip-disconnected:before{content:\"\\e1c5\"}.bosch-ic-locator-spot:before{content:\"\\e1c6\"}.bosch-ic-locator-spot-check:before{content:\"\\e1c7\"}.bosch-ic-lock-closed:before{content:\"\\e1c8\"}.bosch-ic-lock-open:before{content:\"\\e1c9\"}.bosch-ic-login:before{content:\"\\e1ca\"}.bosch-ic-logistics:before{content:\"\\e1cb\"}.bosch-ic-logout:before{content:\"\\e1cc\"}.bosch-ic-machine:before{content:\"\\e1cd\"}.bosch-ic-magnet:before{content:\"\\e1ce\"}.bosch-ic-mail:before{content:\"\\e1cf\"}.bosch-ic-mail-forward:before{content:\"\\e1d0\"}.bosch-ic-mail-open:before{content:\"\\e1d1\"}.bosch-ic-mail-out:before{content:\"\\e1d2\"}.bosch-ic-mail-reply:before{content:\"\\e1d3\"}.bosch-ic-male:before{content:\"\\e1d4\"}.bosch-ic-map:before{content:\"\\e1d5\"}.bosch-ic-material-add:before{content:\"\\e1d6\"}.bosch-ic-material-check:before{content:\"\\e1d7\"}.bosch-ic-material-remove:before{content:\"\\e1d8\"}.bosch-ic-maximize:before{content:\"\\e1d9\"}.bosch-ic-mechanic:before{content:\"\\e1da\"}.bosch-ic-megaphone:before{content:\"\\e1db\"}.bosch-ic-message:before{content:\"\\e1dc\"}.bosch-ic-mic:before{content:\"\\e1dd\"}.bosch-ic-microphone:before{content:\"\\e1de\"}.bosch-ic-microphone-classic:before{content:\"\\e1df\"}.bosch-ic-minimize:before{content:\"\\e1e0\"}.bosch-ic-money-currency:before{content:\"\\e1e1\"}.bosch-ic-money-dollar:before{content:\"\\e1e2\"}.bosch-ic-moneybag-currency:before{content:\"\\e1e3\"}.bosch-ic-moneybag-dollar:before{content:\"\\e1e4\"}.bosch-ic-monument:before{content:\"\\e1e5\"}.bosch-ic-moon:before{content:\"\\e1e6\"}.bosch-ic-motorcycle:before{content:\"\\e1e7\"}.bosch-ic-motorcycle-side-car:before{content:\"\\e1e8\"}.bosch-ic-motorcycle-side:before{content:\"\\e1e9\"}.bosch-ic-mouse:before{content:\"\\e1ea\"}.bosch-ic-mouse-left:before{content:\"\\e1eb\"}.bosch-ic-mouse-right:before{content:\"\\e1ec\"}.bosch-ic-mouth:before{content:\"\\e1ed\"}.bosch-ic-my-product:before{content:\"\\e1ee\"}.bosch-ic-new-way-work:before{content:\"\\e1ef\"}.bosch-ic-newsletter:before{content:\"\\e1f0\"}.bosch-ic-newspaper:before{content:\"\\e1f1\"}.bosch-ic-nose:before{content:\"\\e1f2\"}.bosch-ic-notepad:before{content:\"\\e1f3\"}.bosch-ic-notepad-edit:before{content:\"\\e1f4\"}.bosch-ic-notification:before{content:\"\\e1f5\"}.bosch-ic-notification-active:before{content:\"\\e1f6\"}.bosch-ic-notification-add:before{content:\"\\e1f7\"}.bosch-ic-notification-remove:before{content:\"\\e1f8\"}.bosch-ic-oil-car:before{content:\"\\e1f9\"}.bosch-ic-omnichannel:before{content:\"\\e1fa\"}.bosch-ic-options:before{content:\"\\e1fb\"}.bosch-ic-origami-boat:before{content:\"\\e1fc\"}.bosch-ic-pallete-car:before{content:\"\\e1fd\"}.bosch-ic-pallete-car-connected:before{content:\"\\e1fe\"}.bosch-ic-panel:before{content:\"\\e1ff\"}.bosch-ic-paperclip:before{content:\"\\e200\"}.bosch-ic-paperplane:before{content:\"\\e201\"}.bosch-ic-parking:before{content:\"\\e202\"}.bosch-ic-pause:before{content:\"\\e203\"}.bosch-ic-payment:before{content:\"\\e204\"}.bosch-ic-people:before{content:\"\\e205\"}.bosch-ic-petrol-station:before{content:\"\\e206\"}.bosch-ic-piggybank:before{content:\"\\e207\"}.bosch-ic-pin-classic:before{content:\"\\e208\"}.bosch-ic-pin-modern:before{content:\"\\e209\"}.bosch-ic-plane-side:before{content:\"\\e20a\"}.bosch-ic-play:before{content:\"\\e20b\"}.bosch-ic-plug-12v:before{content:\"\\e20c\"}.bosch-ic-podium-speaker:before{content:\"\\e20d\"}.bosch-ic-police:before{content:\"\\e20e\"}.bosch-ic-poop:before{content:\"\\e20f\"}.bosch-ic-post-it:before{content:\"\\e210\"}.bosch-ic-postal-code:before{content:\"\\e211\"}.bosch-ic-power:before{content:\"\\e212\"}.bosch-ic-print:before{content:\"\\e213\"}.bosch-ic-prototyping:before{content:\"\\e214\"}.bosch-ic-puzzle:before{content:\"\\e215\"}.bosch-ic-quad:before{content:\"\\e216\"}.bosch-ic-question:before{content:\"\\e217\"}.bosch-ic-radar:before{content:\"\\e218\"}.bosch-ic-radio:before{content:\"\\e219\"}.bosch-ic-radiotower:before{content:\"\\e21a\"}.bosch-ic-redirect:before{content:\"\\e21b\"}.bosch-ic-redo:before{content:\"\\e21c\"}.bosch-ic-reference:before{content:\"\\e21d\"}.bosch-ic-refresh:before{content:\"\\e21e\"}.bosch-ic-refresh-cloud:before{content:\"\\e21f\"}.bosch-ic-registration:before{content:\"\\e220\"}.bosch-ic-remote:before{content:\"\\e221\"}.bosch-ic-reset:before{content:\"\\e222\"}.bosch-ic-resolution:before{content:\"\\e223\"}.bosch-ic-robot:before{content:\"\\e224\"}.bosch-ic-robot-connected:before{content:\"\\e225\"}.bosch-ic-robothead:before{content:\"\\e226\"}.bosch-ic-rocket:before{content:\"\\e227\"}.bosch-ic-rotation:before{content:\"\\e228\"}.bosch-ic-rotation-360:before{content:\"\\e229\"}.bosch-ic-rotation-x-left:before{content:\"\\e22a\"}.bosch-ic-rotation-x-right:before{content:\"\\e22b\"}.bosch-ic-rotation-y-down:before{content:\"\\e22c\"}.bosch-ic-rotation-y-up:before{content:\"\\e22d\"}.bosch-ic-route:before{content:\"\\e22e\"}.bosch-ic-ruler:before{content:\"\\e22f\"}.bosch-ic-ruler-pen:before{content:\"\\e230\"}.bosch-ic-satellite:before{content:\"\\e231\"}.bosch-ic-save:before{content:\"\\e232\"}.bosch-ic-scale:before{content:\"\\e233\"}.bosch-ic-scissors:before{content:\"\\e234\"}.bosch-ic-search:before{content:\"\\e235\"}.bosch-ic-security:before{content:\"\\e236\"}.bosch-ic-security-check:before{content:\"\\e237\"}.bosch-ic-security-user:before{content:\"\\e238\"}.bosch-ic-sensor:before{content:\"\\e239\"}.bosch-ic-server:before{content:\"\\e23a\"}.bosch-ic-server-expansion:before{content:\"\\e23b\"}.bosch-ic-server-rate:before{content:\"\\e23c\"}.bosch-ic-service-agent:before{content:\"\\e23d\"}.bosch-ic-service-time:before{content:\"\\e23e\"}.bosch-ic-settings:before{content:\"\\e23f\"}.bosch-ic-settings-arrows:before{content:\"\\e240\"}.bosch-ic-settings-connected:before{content:\"\\e241\"}.bosch-ic-share:before{content:\"\\e242\"}.bosch-ic-shield-stop:before{content:\"\\e243\"}.bosch-ic-ship:before{content:\"\\e244\"}.bosch-ic-ship-side:before{content:\"\\e245\"}.bosch-ic-shoppingcart:before{content:\"\\e246\"}.bosch-ic-shoppingcart-add:before{content:\"\\e247\"}.bosch-ic-shoppingcart-switch:before{content:\"\\e248\"}.bosch-ic-signal-full:before{content:\"\\e249\"}.bosch-ic-signal-half:before{content:\"\\e24a\"}.bosch-ic-signal-lost:before{content:\"\\e24b\"}.bosch-ic-signpost:before{content:\"\\e24c\"}.bosch-ic-simcard:before{content:\"\\e24d\"}.bosch-ic-skyscraper:before{content:\"\\e24e\"}.bosch-ic-smartcity:before{content:\"\\e24f\"}.bosch-ic-smarthome:before{content:\"\\e250\"}.bosch-ic-smartphone:before{content:\"\\e251\"}.bosch-ic-smartphone-acoustic:before{content:\"\\e252\"}.bosch-ic-smartphone-arrow-right:before{content:\"\\e253\"}.bosch-ic-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.bosch-ic-smartphone-clock:before{content:\"\\e255\"}.bosch-ic-smartphone-cloud:before{content:\"\\e256\"}.bosch-ic-smartphone-loading-wheel:before{content:\"\\e257\"}.bosch-ic-smartphone-smashed:before{content:\"\\e258\"}.bosch-ic-smartphone-vibration:before{content:\"\\e259\"}.bosch-ic-smartphone-z:before{content:\"\\e25a\"}.bosch-ic-smartwatch-connected:before{content:\"\\e25b\"}.bosch-ic-spare-parts:before{content:\"\\e25c\"}.bosch-ic-spark-plug:before{content:\"\\e25d\"}.bosch-ic-speaker-acoustic:before{content:\"\\e25e\"}.bosch-ic-speedometer:before{content:\"\\e25f\"}.bosch-ic-stadium:before{content:\"\\e260\"}.bosch-ic-stairs:before{content:\"\\e261\"}.bosch-ic-stamp:before{content:\"\\e262\"}.bosch-ic-standby:before{content:\"\\e263\"}.bosch-ic-steering-wheel:before{content:\"\\e264\"}.bosch-ic-steering-wheel-connected:before{content:\"\\e265\"}.bosch-ic-steering-wheel-hands:before{content:\"\\e266\"}.bosch-ic-steps:before{content:\"\\e267\"}.bosch-ic-stethoscope:before{content:\"\\e268\"}.bosch-ic-stop:before{content:\"\\e269\"}.bosch-ic-stopwatch:before{content:\"\\e26a\"}.bosch-ic-storage-local:before{content:\"\\e26b\"}.bosch-ic-structure:before{content:\"\\e26c\"}.bosch-ic-subtitles:before{content:\"\\e26d\"}.bosch-ic-subtitles-off-light:before{content:\"\\e26e\"}.bosch-ic-summary:before{content:\"\\e26f\"}.bosch-ic-sun:before{content:\"\\e270\"}.bosch-ic-sun-half-filled:before{content:\"\\e271\"}.bosch-ic-sunshade-table:before{content:\"\\e272\"}.bosch-ic-surveillance:before{content:\"\\e273\"}.bosch-ic-table-chairs:before{content:\"\\e274\"}.bosch-ic-tablet:before{content:\"\\e275\"}.bosch-ic-team:before{content:\"\\e276\"}.bosch-ic-team-lightbulb:before{content:\"\\e277\"}.bosch-ic-technical-service:before{content:\"\\e278\"}.bosch-ic-temperature:before{content:\"\\e279\"}.bosch-ic-temperature-high:before{content:\"\\e27a\"}.bosch-ic-temperature-low:before{content:\"\\e27b\"}.bosch-ic-theater:before{content:\"\\e27c\"}.bosch-ic-thumb-down:before{content:\"\\e27d\"}.bosch-ic-thumb-up:before{content:\"\\e27e\"}.bosch-ic-ticket:before{content:\"\\e27f\"}.bosch-ic-touch:before{content:\"\\e280\"}.bosch-ic-touch-sos:before{content:\"\\e281\"}.bosch-ic-towing-truck:before{content:\"\\e282\"}.bosch-ic-train:before{content:\"\\e283\"}.bosch-ic-train-side:before{content:\"\\e284\"}.bosch-ic-transport-movements:before{content:\"\\e285\"}.bosch-ic-transscript:before{content:\"\\e286\"}.bosch-ic-transscript-off:before{content:\"\\e287\"}.bosch-ic-transscript-on:before{content:\"\\e288\"}.bosch-ic-tree:before{content:\"\\e289\"}.bosch-ic-tricycle:before{content:\"\\e28a\"}.bosch-ic-trolley-empty:before{content:\"\\e28b\"}.bosch-ic-trolley-filled-box:before{content:\"\\e28c\"}.bosch-ic-trolly-filled-files:before{content:\"\\e28d\"}.bosch-ic-trophy:before{content:\"\\e28e\"}.bosch-ic-truck:before{content:\"\\e28f\"}.bosch-ic-tuktuk:before{content:\"\\e290\"}.bosch-ic-tune:before{content:\"\\e291\"}.bosch-ic-typography:before{content:\"\\e292\"}.bosch-ic-ui:before{content:\"\\e293\"}.bosch-ic-umbrella:before{content:\"\\e294\"}.bosch-ic-undo:before{content:\"\\e295\"}.bosch-ic-up:before{content:\"\\e296\"}.bosch-ic-up-double:before{content:\"\\e297\"}.bosch-ic-up-down:before{content:\"\\e298\"}.bosch-ic-upload:before{content:\"\\e299\"}.bosch-ic-usb:before{content:\"\\e29a\"}.bosch-ic-user:before{content:\"\\e29b\"}.bosch-ic-user-advanced:before{content:\"\\e29c\"}.bosch-ic-user-beginner:before{content:\"\\e29d\"}.bosch-ic-user-desktop:before{content:\"\\e29e\"}.bosch-ic-user-down:before{content:\"\\e29f\"}.bosch-ic-user-falling:before{content:\"\\e2a0\"}.bosch-ic-user-hand:before{content:\"\\e2a1\"}.bosch-ic-user-hand-stop:before{content:\"\\e2a2\"}.bosch-ic-user-professional:before{content:\"\\e2a3\"}.bosch-ic-user-run:before{content:\"\\e2a4\"}.bosch-ic-user-share:before{content:\"\\e2a5\"}.bosch-ic-user-voice:before{content:\"\\e2a6\"}.bosch-ic-user-walk:before{content:\"\\e2a7\"}.bosch-ic-video:before{content:\"\\e2a8\"}.bosch-ic-video-disabled-light:before{content:\"\\e2a9\"}.bosch-ic-video-record:before{content:\"\\e2aa\"}.bosch-ic-virtual-reality:before{content:\"\\e2ab\"}.bosch-ic-volume-disabled:before{content:\"\\e2ac\"}.bosch-ic-volume-high:before{content:\"\\e2ad\"}.bosch-ic-volume-low:before{content:\"\\e2ae\"}.bosch-ic-volume-off-light:before{content:\"\\e2af\"}.bosch-ic-wand:before{content:\"\\e2b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-watch-off-disabled-light:before{content:\"\\e2b2\"}.bosch-ic-watch-on:before{content:\"\\e2b3\"}.bosch-ic-water-shower:before{content:\"\\e2b4\"}.bosch-ic-welcome:before{content:\"\\e2b5\"}.bosch-ic-wellness:before{content:\"\\e2b6\"}.bosch-ic-wheelchair:before{content:\"\\e2b7\"}.bosch-ic-whistle:before{content:\"\\e2b8\"}.bosch-ic-wide-angled-arrow:before{content:\"\\e2b9\"}.bosch-ic-wifi:before{content:\"\\e2ba\"}.bosch-ic-window-analysis:before{content:\"\\e2bb\"}.bosch-ic-window-browser:before{content:\"\\e2bc\"}.bosch-ic-window-console:before{content:\"\\e2bd\"}.bosch-ic-window-gaussian-view:before{content:\"\\e2be\"}.bosch-ic-window-info:before{content:\"\\e2bf\"}.bosch-ic-window-new:before{content:\"\\e2c0\"}.bosch-ic-window-resize:before{content:\"\\e2c1\"}.bosch-ic-window-scaling-view:before{content:\"\\e2c2\"}.bosch-ic-window-shuffle:before{content:\"\\e2c3\"}.bosch-ic-window-statistic:before{content:\"\\e2c4\"}.bosch-ic-wishlist:before{content:\"\\e2c5\"}.bosch-ic-work-order:before{content:\"\\e2c6\"}.bosch-ic-worldwideweb:before{content:\"\\e2c7\"}.bosch-ic-wrench:before{content:\"\\e2c8\"}.bosch-ic-zoom-in:before{content:\"\\e2c9\"}.bosch-ic-zoom-out:before{content:\"\\e2ca\"}.bosch-ic-scooter:before{content:\"\\e2cb\"}.bosch-ic-cars-traffic:before{content:\"\\e2cc\"}.bosch-ic-arm-chair:before{content:\"\\e2cd\"}.bosch-ic-car-phone:before{content:\"\\e2ce\"}.bosch-ic-box-lightbulb:before{content:\"\\e2cf\"}.bosch-ic-chair-officedesk:before{content:\"\\e2d0\"}.bosch-ic-calendar-clock:before{content:\"\\e2d1\"}.bosch-ic-document-cv:before{content:\"\\e2d2\"}.bosch-ic-user-run-bag:before{content:\"\\e2d3\"}.bosch-ic-brake-disk-spray:before{content:\"\\e2d4\"}.bosch-ic-circle-abs:before{content:\"\\e2d5\"}.bosch-ic-bright-down:before{content:\"\\e2d6\"}.bosch-ic-bright-up:before{content:\"\\e2d7\"}.bosch-ic-fan:before{content:\"\\e2d8\"}.bosch-ic-sun-moon:before{content:\"\\e2d9\"}.bosch-ic-lightbulb-attention:before{content:\"\\e2da\"}.bosch-ic-settings-attention:before{content:\"\\e2db\"}.bosch-ic-danger-hazards:before{content:\"\\e2dc\"}.bosch-ic-windshield-spray:before{content:\"\\e2dd\"}.bosch-ic-windshield-heat:before{content:\"\\e2de\"}.bosch-ic-light-drop:before{content:\"\\e2df\"}.bosch-ic-starter:before{content:\"\\e2e0\"}.bosch-ic-turbo:before{content:\"\\e2e1\"}.bosch-ic-settings-lock:before{content:\"\\e2e2\"}.bosch-ic-settings-n:before{content:\"\\e2e3\"}.bosch-ic-light-beam-high:before{content:\"\\e2e4\"}.bosch-ic-light-beam-low:before{content:\"\\e2e5\"}.bosch-ic-glas:before{content:\"\\e2e6\"}.bosch-ic-shirt-ice:before{content:\"\\e2e7\"}.bosch-ic-helmet-fan:before{content:\"\\e2e8\"}.bosch-ic-notification-arrow:before{content:\"\\e2e9\"}.bosch-ic-steering-wheel-arrow:before{content:\"\\e2ea\"}.bosch-ic-circle-arrow:before{content:\"\\e2eb\"}.bosch-ic-tempomat-pit:before{content:\"\\e2ec\"}.bosch-ic-radiator-fan-spray:before{content:\"\\e2ed\"}.bosch-ic-boost-down:before{content:\"\\e2ee\"}.bosch-ic-boost-up:before{content:\"\\e2ef\"}.bosch-ic-circle-lock:before{content:\"\\e2f0\"}.bosch-ic-mp-down:before{content:\"\\e2f1\"}.bosch-ic-mp-up:before{content:\"\\e2f2\"}.bosch-ic-tc-down:before{content:\"\\e2f3\"}.bosch-ic-tc-up:before{content:\"\\e2f4\"}.bosch-ic-danger-arrow-attention:before{content:\"\\e2f5\"}.bosch-ic-user-remove:before{content:\"\\e2f6\"}.bosch-ic-user-add:before{content:\"\\e2f7\"}.bosch-ic-post-it-collection:before{content:\"\\e2f8\"}.bosch-ic-horn:before{content:\"\\e2f9\"}.bosch-ic-desktop-notification:before{content:\"\\e2fa\"}.bosch-ic-router:before{content:\"\\e2fb\"}.bosch-ic-berry:before{content:\"\\e2fc\"}.bosch-ic-chat-language:before{content:\"\\e2fd\"}.bosch-ic-game-controller:before{content:\"\\e2fe\"}.bosch-ic-user-brain:before{content:\"\\e2ff\"}.bosch-ic-mouth-finger:before{content:\"\\e300\"}.bosch-ic-excavator-shovel:before{content:\"\\e301\"}.bosch-ic-ear-in:before{content:\"\\e302\"}.bosch-ic-ear-out:before{content:\"\\e303\"}.bosch-ic-lion:before{content:\"\\e304\"}.bosch-ic-car-convertible:before{content:\"\\e305\"}.bosch-ic-car-sports:before{content:\"\\e306\"}.bosch-ic-bear:before{content:\"\\e307\"}.bosch-ic-pot:before{content:\"\\e308\"}.bosch-ic-bed-double:before{content:\"\\e309\"}.bosch-ic-air-hot:before{content:\"\\e30a\"}.bosch-ic-air-ice:before{content:\"\\e30b\"}.bosch-ic-air-leaf:before{content:\"\\e30c\"}.bosch-ic-air-purifier-0:before{content:\"\\e30d\"}.bosch-ic-air-purifier-1:before{content:\"\\e30e\"}.bosch-ic-air-purifier-2:before{content:\"\\e30f\"}.bosch-ic-air-purifier-3:before{content:\"\\e310\"}.bosch-ic-air:before{content:\"\\e311\"}.bosch-ic-fan-off-disabled-bold:before{content:\"\\e312\"}.bosch-ic-ice-off-disabled-bold:before{content:\"\\e313\"}.bosch-ic-faucet-boost:before{content:\"\\e314\"}.bosch-ic-faucet-mode:before{content:\"\\e315\"}.bosch-ic-faucet-off-disabled-bold:before{content:\"\\e316\"}.bosch-ic-faucet-schedule:before{content:\"\\e317\"}.bosch-ic-fuel-cell:before{content:\"\\e318\"}.bosch-ic-heating-floor-off-disabled-bold:before{content:\"\\e319\"}.bosch-ic-heating-floor:before{content:\"\\e31a\"}.bosch-ic-heating-radiator-boost:before{content:\"\\e31b\"}.bosch-ic-heating-radiator-mode:before{content:\"\\e31c\"}.bosch-ic-heating-radiator-off-disabled-bold:before{content:\"\\e31d\"}.bosch-ic-heating-radiator-schedule:before{content:\"\\e31e\"}.bosch-ic-leaf-simple:before{content:\"\\e31f\"}.bosch-ic-oil-barrel:before{content:\"\\e320\"}.bosch-ic-water-bathtub:before{content:\"\\e321\"}.bosch-ic-water-dishwash:before{content:\"\\e322\"}.bosch-ic-water-handwash:before{content:\"\\e323\"}.bosch-ic-water-hot:before{content:\"\\e324\"}.bosch-ic-water-ice:before{content:\"\\e325\"}.bosch-ic-water-leaf:before{content:\"\\e326\"}.bosch-ic-water-ladder:before{content:\"\\e327\"}.bosch-ic-water-purifier-0:before{content:\"\\e328\"}.bosch-ic-water-purifier-1:before{content:\"\\e329\"}.bosch-ic-water-purifier-2:before{content:\"\\e32a\"}.bosch-ic-water-purifier-3:before{content:\"\\e32b\"}.bosch-ic-water-shower-boost:before{content:\"\\e32c\"}.bosch-ic-water-shower-mode:before{content:\"\\e32d\"}.bosch-ic-water-shower-off-disabled-bold:before{content:\"\\e32e\"}.bosch-ic-water-shower-schedule:before{content:\"\\e32f\"}.bosch-ic-water:before{content:\"\\e330\"}.bosch-ic-chart-bar-drop:before{content:\"\\e331\"}.bosch-ic-chart-bar-fire:before{content:\"\\e332\"}.bosch-ic-chart-bar-flash:before{content:\"\\e333\"}.bosch-ic-chart-bar-oil:before{content:\"\\e334\"}.bosch-ic-drop-minus:before{content:\"\\e335\"}.bosch-ic-drop-plus:before{content:\"\\e336\"}.bosch-ic-fan-minus:before{content:\"\\e337\"}.bosch-ic-fan-plus:before{content:\"\\e338\"}.bosch-ic-home-drops-0:before{content:\"\\e339\"}.bosch-ic-home-drops-1:before{content:\"\\e33a\"}.bosch-ic-home-drops-2:before{content:\"\\e33b\"}.bosch-ic-home-drops-3:before{content:\"\\e33c\"}.bosch-ic-home-temperature-in:before{content:\"\\e33d\"}.bosch-ic-home-temperature-out:before{content:\"\\e33e\"}.bosch-ic-home-user-in:before{content:\"\\e33f\"}.bosch-ic-ice-minus:before{content:\"\\e340\"}.bosch-ic-ice-plus:before{content:\"\\e341\"}.bosch-ic-chimney-sweeper:before{content:\"\\e342\"}.bosch-ic-menu-up:before{content:\"\\e343\"}.bosch-ic-security-ice:before{content:\"\\e344\"}.bosch-ic-security-legionella:before{content:\"\\e345\"}.bosch-ic-trolley-case:before{content:\"\\e346\"}.bosch-ic-sun-minus:before{content:\"\\e347\"}.bosch-ic-sun-plug:before{content:\"\\e348\"}.bosch-ic-sun-plus:before{content:\"\\e349\"}.bosch-ic-touch-pit:before{content:\"\\e34a\"}.bosch-ic-temperature-current:before{content:\"\\e34b\"}.bosch-ic-temperature-set:before{content:\"\\e34c\"}.bosch-ic-teddy-bear:before{content:\"\\e34d\"}.bosch-ic-document-svg:before{content:\"\\e34e\"}.bosch-ic-back-menu:before{content:\"\\e34f\"}.bosch-ic-car-arrows:before{content:\"\\e350\"}.bosch-ic-wrench-cloud:before{content:\"\\e351\"}.bosch-ic-weather-cloud-sun:before{content:\"\\e352\"}.bosch-ic-weather-cloudy:before{content:\"\\e353\"}.bosch-ic-weather-gale:before{content:\"\\e354\"}.bosch-ic-weather-mist:before{content:\"\\e355\"}.bosch-ic-weather-moon-cloud:before{content:\"\\e356\"}.bosch-ic-weather-rain-snow:before{content:\"\\e357\"}.bosch-ic-weather-rain-thunder:before{content:\"\\e358\"}.bosch-ic-weather-rain:before{content:\"\\e359\"}.bosch-ic-weather-snow:before{content:\"\\e35a\"}.bosch-ic-weather-thunder:before{content:\"\\e35b\"}.bosch-ic-weather-wind:before{content:\"\\e35c\"}.bosch-ic-plane-start:before{content:\"\\e35d\"}.bosch-ic-plane-land:before{content:\"\\e35e\"}.bosch-ic-graph-power:before{content:\"\\e35f\"}.bosch-ic-floorplan:before{content:\"\\e360\"}.bosch-ic-clock-pause:before{content:\"\\e361\"}.bosch-ic-clock-start:before{content:\"\\e362\"}.bosch-ic-clock-stop:before{content:\"\\e363\"}.bosch-ic-computer-retro:before{content:\"\\e364\"}.bosch-ic-smartphone-retro:before{content:\"\\e365\"}.bosch-ic-wood:before{content:\"\\e366\"}.bosch-ic-solid-fuel:before{content:\"\\e367\"}.bosch-ic-appliance-fan:before{content:\"\\e368\"}.bosch-ic-appliance-flame:before{content:\"\\e369\"}.bosch-ic-appliance-ice:before{content:\"\\e36a\"}.bosch-ic-appliance-oil:before{content:\"\\e36b\"}.bosch-ic-appliance-solid-fuel:before{content:\"\\e36c\"}.bosch-ic-appliance-sun-ice:before{content:\"\\e36d\"}.bosch-ic-appliance-sun:before{content:\"\\e36e\"}.bosch-ic-appliance-wood:before{content:\"\\e36f\"}.bosch-ic-appliance-air-leaf:before{content:\"\\e370\"}.bosch-ic-appliance-water-air:before{content:\"\\e371\"}.bosch-ic-appliance-water-leaf:before{content:\"\\e372\"}.bosch-ic-appliance-water-oil:before{content:\"\\e373\"}.bosch-ic-appliance-water:before{content:\"\\e374\"}.bosch-ic-appliance-heat-pump-air:before{content:\"\\e375\"}.bosch-ic-appliance-heat-pump-ground:before{content:\"\\e376\"}.bosch-ic-appliance-heat-pump-water:before{content:\"\\e377\"}.bosch-ic-appliance-plug-air:before{content:\"\\e378\"}.bosch-ic-appliance-plug-fuel-cell:before{content:\"\\e379\"}.bosch-ic-appliance-plug-leaf:before{content:\"\\e37a\"}.bosch-ic-appliance-plug-water-air:before{content:\"\\e37b\"}.bosch-ic-thermostat-connected-c:before{content:\"\\e37c\"}.bosch-ic-thermostat-connected-f:before{content:\"\\e37d\"}.bosch-ic-image-gallery:before{content:\"\\e37e\"}.bosch-ic-arrows-caution:before{content:\"\\e37f\"}.bosch-ic-arrows-diagonal-disabled-bold:before{content:\"\\e380\"}.bosch-ic-back-left-end:before{content:\"\\e381\"}.bosch-ic-back-right-end:before{content:\"\\e382\"}.bosch-ic-bar-chart-search:before{content:\"\\e383\"}.bosch-ic-batch:before{content:\"\\e384\"}.bosch-ic-calendar-remove:before{content:\"\\e385\"}.bosch-ic-call-bluetooth:before{content:\"\\e386\"}.bosch-ic-call-deny:before{content:\"\\e387\"}.bosch-ic-call-settings:before{content:\"\\e388\"}.bosch-ic-caution-sign-clock:before{content:\"\\e389\"}.bosch-ic-caution-sign-list:before{content:\"\\e38a\"}.bosch-ic-core-data:before{content:\"\\e38b\"}.bosch-ic-cursor-connected:before{content:\"\\e38c\"}.bosch-ic-cursor:before{content:\"\\e38d\"}.bosch-ic-dmc-code:before{content:\"\\e38e\"}.bosch-ic-find-id:before{content:\"\\e38f\"}.bosch-ic-globe-search:before{content:\"\\e390\"}.bosch-ic-interface:before{content:\"\\e391\"}.bosch-ic-material-add-manually:before{content:\"\\e392\"}.bosch-ic-material-list:before{content:\"\\e393\"}.bosch-ic-material-remove-manually:before{content:\"\\e394\"}.bosch-ic-material:before{content:\"\\e395\"}.bosch-ic-menu-checkmark:before{content:\"\\e396\"}.bosch-ic-qr-code:before{content:\"\\e397\"}.bosch-ic-sort-alphabetically:before{content:\"\\e398\"}.bosch-ic-unpin-classic-disabled-bold:before{content:\"\\e399\"}.bosch-ic-unpin-modern-disabled-bold:before{content:\"\\e39a\"}.bosch-ic-window-search:before{content:\"\\e39b\"}.bosch-ic-cloud-clock:before{content:\"\\e39c\"}.bosch-ic-concierge:before{content:\"\\e39d\"}.bosch-ic-heating-radiator:before{content:\"\\e39e\"}.bosch-ic-home-user-out:before{content:\"\\e39f\"}.bosch-ic-hot:before{content:\"\\e3a0\"}.bosch-ic-screen-service:before{content:\"\\e3a1\"}.bosch-ic-sun-ice:before{content:\"\\e3a2\"}.bosch-ic-sun-off-disabled-bold:before{content:\"\\e3a3\"}.bosch-ic-tractor:before{content:\"\\e3a4\"}.bosch-ic-video-recorder-digital:before{content:\"\\e3a5\"}.bosch-ic-windshield-wiper:before{content:\"\\e3a6\"}.bosch-ic-weather-snow-thunder:before{content:\"\\e3a7\"}.bosch-ic-electricity-pylon:before{content:\"\\e3a8\"}.bosch-ic-box-questionmark:before{content:\"\\e3a9\"}.bosch-ic-rainbow:before{content:\"\\e3aa\"}.bosch-ic-medal:before{content:\"\\e3ab\"}.bosch-ic-wheel:before{content:\"\\e3ac\"}.bosch-ic-borescope-connected:before{content:\"\\e3ad\"}.bosch-ic-filter-delete:before{content:\"\\e3ae\"}.bosch-ic-filter-add:before{content:\"\\e3af\"}.bosch-ic-man:before{content:\"\\e3b0\"}.bosch-ic-woman:before{content:\"\\e3b1\"}.bosch-ic-fridge:before{content:\"\\e3b2\"}.bosch-ic-battery-car:before{content:\"\\e3b3\"}.bosch-ic-bag-escalator:before{content:\"\\e3b4\"}.bosch-ic-shopping-bags:before{content:\"\\e3b5\"}.bosch-ic-car-steering-left:before{content:\"\\e3b6\"}.bosch-ic-car-steering-right:before{content:\"\\e3b7\"}.bosch-ic-thumb-up-down:before{content:\"\\e3b8\"}.bosch-ic-user-run-detection:before{content:\"\\e3b9\"}.bosch-ic-bank:before{content:\"\\e3ba\"}.bosch-ic-transportation:before{content:\"\\e3bb\"}.bosch-ic-government:before{content:\"\\e3bc\"}.bosch-ic-user-music:before{content:\"\\e3bd\"}.bosch-ic-bus:before{content:\"\\e3be\"}.bosch-ic-car-euro:before{content:\"\\e3bf\"}.bosch-ic-bag:before{content:\"\\e3c0\"}.bosch-ic-plate-tetrapack:before{content:\"\\e3c1\"}.bosch-ic-hospital:before{content:\"\\e3c2\"}.bosch-ic-desktop-structure:before{content:\"\\e3c3\"}.bosch-ic-frog-head:before{content:\"\\e3c4\"}.bosch-ic-sportboat:before{content:\"\\e3c5\"}.bosch-ic-yacht:before{content:\"\\e3c6\"}.bosch-ic-leaf-tilia:before{content:\"\\e3c7\"}.bosch-ic-music:before{content:\"\\e3c8\"}.bosch-ic-oil-lamp:before{content:\"\\e3c9\"}.bosch-ic-windshield-wiper-rear:before{content:\"\\e3ca\"}.bosch-ic-dongle-connected:before{content:\"\\e3cb\"}.bosch-ic-slate:before{content:\"\\e3cc\"}.bosch-ic-appliance-heat-pump-exhaust:before{content:\"\\e3cd\"}.bosch-ic-alert-list:before{content:\"\\e3ce\"}.bosch-ic-arrow-left-end-frame:before{content:\"\\e3cf\"}.bosch-ic-arrow-right-end-frame:before{content:\"\\e3d0\"}.bosch-ic-shape-ellipse:before{content:\"\\e3d1\"}.bosch-ic-shape-rectangle:before{content:\"\\e3d2\"}.bosch-ic-wishlist-add:before{content:\"\\e3d3\"}.bosch-ic-smartphone-disabled-light:before{content:\"\\e3d4\"}.bosch-ic-airbag:before{content:\"\\e3d5\"}.bosch-ic-compass:before{content:\"\\e3d6\"}.bosch-ic-turbine-wind:before{content:\"\\e3d7\"}.bosch-ic-arrows-diagonal-disabled-light:before{content:\"\\e3d8\"}.bosch-ic-ear-off-disabled-bold:before{content:\"\\e3d9\"}.bosch-ic-fan-off-disabled-light:before{content:\"\\e3da\"}.bosch-ic-faucet-off-disabled-light:before{content:\"\\e3db\"}.bosch-ic-hand-motion-off-disabled-bold:before{content:\"\\e3dc\"}.bosch-ic-heating-floor-off-disabled-light:before{content:\"\\e3dd\"}.bosch-ic-heating-radiator-off-disabled-light:before{content:\"\\e3de\"}.bosch-ic-ice-off-disabled-light:before{content:\"\\e3df\"}.bosch-ic-smartphone-disabled-bold:before{content:\"\\e3e0\"}.bosch-ic-subtitles-off-bold:before{content:\"\\e3e1\"}.bosch-ic-sun-off-disabled-light:before{content:\"\\e3e2\"}.bosch-ic-video-disabled-bold:before{content:\"\\e3e3\"}.bosch-ic-volume-off-bold:before{content:\"\\e3e4\"}.bosch-ic-watch-off-disabled-bold:before{content:\"\\e3e5\"}.bosch-ic-water-shower-off-disabled-light:before{content:\"\\e3e6\"}.bosch-ic-3d-frame-play:before{content:\"\\e3e7\"}.bosch-ic-alert-error-checkmark:before{content:\"\\e3e8\"}.bosch-ic-arrow-up-frame-check:before{content:\"\\e3e9\"}.bosch-ic-arrow-up-frame-error:before{content:\"\\e3ea\"}.bosch-ic-arrows-checkmark:before{content:\"\\e3eb\"}.bosch-ic-assembly-line-settings:before{content:\"\\e3ec\"}.bosch-ic-augmented-reality-player:before{content:\"\\e3ed\"}.bosch-ic-bar-chart-kappa:before{content:\"\\e3ef\"}.bosch-ic-bar-chart-search-detail:before{content:\"\\e3f0\"}.bosch-ic-barcode-checkmark:before{content:\"\\e3f1\"}.bosch-ic-barcode-scan-bulb:before{content:\"\\e3f2\"}.bosch-ic-barcode:before{content:\"\\e3f3\"}.bosch-ic-binary:before{content:\"\\e3f4\"}.bosch-ic-book-lightbulb:before{content:\"\\e3f5\"}.bosch-ic-box-arrow-down:before{content:\"\\e3f6\"}.bosch-ic-box-delivery-checkmark:before{content:\"\\e3f7\"}.bosch-ic-globe-checkmark:before{content:\"\\e3f8\"}.bosch-ic-calendar-boxes:before{content:\"\\e3f9\"}.bosch-ic-calendar-service:before{content:\"\\e3fa\"}.bosch-ic-camera-processor:before{content:\"\\e3fb\"}.bosch-ic-camera-settings:before{content:\"\\e3fc\"}.bosch-ic-chart-line-checkmark:before{content:\"\\e3fd\"}.bosch-ic-checklist-stack:before{content:\"\\e3fe\"}.bosch-ic-checkmark-frame-minimum:before{content:\"\\e3ff\"}.bosch-ic-checkmark-frame-tripple:before{content:\"\\e400\"}.bosch-ic-clipboard-checkmark:before{content:\"\\e401\"}.bosch-ic-clipboard-list-parts:before{content:\"\\e402\"}.bosch-ic-clipboard-settings:before{content:\"\\e403\"}.bosch-ic-computer-tower-box:before{content:\"\\e404\"}.bosch-ic-connectivity-lock:before{content:\"\\e405\"}.bosch-ic-counter-current:before{content:\"\\e406\"}.bosch-ic-counter-target:before{content:\"\\e407\"}.bosch-ic-cube-checkmark:before{content:\"\\e408\"}.bosch-ic-cube-play:before{content:\"\\e409\"}.bosch-ic-timeline-settings:before{content:\"\\e40a\"}.bosch-ic-timeline-view:before{content:\"\\e40b\"}.bosch-ic-timeline:before{content:\"\\e40c\"}.bosch-ic-delivery-checkmark:before{content:\"\\e40d\"}.bosch-ic-desktop-application-3d-play:before{content:\"\\e40e\"}.bosch-ic-desktop-apps-download:before{content:\"\\e40f\"}.bosch-ic-desktop-dashboard:before{content:\"\\e410\"}.bosch-ic-desktop-graph-checkmark:before{content:\"\\e411\"}.bosch-ic-desktop-graph-search:before{content:\"\\e412\"}.bosch-ic-desktop-graph:before{content:\"\\e413\"}.bosch-ic-development-kit:before{content:\"\\e414\"}.bosch-ic-directory-cloud-settings:before{content:\"\\e415\"}.bosch-ic-directory-cloud:before{content:\"\\e416\"}.bosch-ic-directory-settings:before{content:\"\\e417\"}.bosch-ic-document-clock-cycle:before{content:\"\\e418\"}.bosch-ic-document-code-stack:before{content:\"\\e419\"}.bosch-ic-document-dat-edit:before{content:\"\\e41a\"}.bosch-ic-document-ohd-arrow-down:before{content:\"\\e41b\"}.bosch-ic-dolphin:before{content:\"\\e41c\"}.bosch-ic-energy-management:before{content:\"\\e41d\"}.bosch-ic-eraser:before{content:\"\\e41e\"}.bosch-ic-folder-broom:before{content:\"\\e41f\"}.bosch-ic-folder-check-broom:before{content:\"\\e420\"}.bosch-ic-folder-checkmark:before{content:\"\\e421\"}.bosch-ic-folder-oes-check:before{content:\"\\e422\"}.bosch-ic-folder-open-service:before{content:\"\\e423\"}.bosch-ic-folder-reload:before{content:\"\\e424\"}.bosch-ic-hand-command:before{content:\"\\e425\"}.bosch-ic-heating-start:before{content:\"\\e426\"}.bosch-ic-hierarchy-refresh:before{content:\"\\e427\"}.bosch-ic-hierarchy-search:before{content:\"\\e428\"}.bosch-ic-hierarchy-settings:before{content:\"\\e429\"}.bosch-ic-hierarchy:before{content:\"\\e42a\"}.bosch-ic-hmi-desktop-settings:before{content:\"\\e42b\"}.bosch-ic-human-machine-interface:before{content:\"\\e42c\"}.bosch-ic-keys-user-access:before{content:\"\\e42d\"}.bosch-ic-keys:before{content:\"\\e42e\"}.bosch-ic-label-edit:before{content:\"\\e42f\"}.bosch-ic-line-arrow-checkmark:before{content:\"\\e430\"}.bosch-ic-link-checkmark:before{content:\"\\e431\"}.bosch-ic-lock-closed-checkmark:before{content:\"\\e432\"}.bosch-ic-lock-open-checkmark:before{content:\"\\e433\"}.bosch-ic-material-search:before{content:\"\\e434\"}.bosch-ic-objects-search:before{content:\"\\e435\"}.bosch-ic-objects:before{content:\"\\e436\"}.bosch-ic-plan-a-to-b:before{content:\"\\e437\"}.bosch-ic-plan-chart:before{content:\"\\e438\"}.bosch-ic-print-send:before{content:\"\\e439\"}.bosch-ic-refresh-analysis:before{content:\"\\e43a\"}.bosch-ic-remote-desktop-add:before{content:\"\\e43b\"}.bosch-ic-rfid-checkmark:before{content:\"\\e43c\"}.bosch-ic-rfid-tag:before{content:\"\\e43d\"}.bosch-ic-sequence-checkmark:before{content:\"\\e43e\"}.bosch-ic-server-arrow-up:before{content:\"\\e43f\"}.bosch-ic-server-arrows:before{content:\"\\e440\"}.bosch-ic-server-settings:before{content:\"\\e441\"}.bosch-ic-server-share:before{content:\"\\e442\"}.bosch-ic-settings-editor:before{content:\"\\e443\"}.bosch-ic-survey-checkmark:before{content:\"\\e444\"}.bosch-ic-target-dart:before{content:\"\\e445\"}.bosch-ic-target-view:before{content:\"\\e446\"}.bosch-ic-target:before{content:\"\\e447\"}.bosch-ic-thickness-selection:before{content:\"\\e448\"}.bosch-ic-toolbox-checkmark:before{content:\"\\e449\"}.bosch-ic-toolbox:before{content:\"\\e44a\"}.bosch-ic-translate:before{content:\"\\e44b\"}.bosch-ic-tune-vertical-checkmark:before{content:\"\\e44c\"}.bosch-ic-wand-user-add:before{content:\"\\e44d\"}.bosch-ic-wand-user-settings:before{content:\"\\e44e\"}.bosch-ic-wand-user:before{content:\"\\e44f\"}.bosch-ic-window-globe:before{content:\"\\e450\"}.bosch-ic-wrench-change:before{content:\"\\e451\"}.bosch-ic-clock-24-7:before{content:\"\\e452\"}.bosch-ic-eagle:before{content:\"\\e453\"}.bosch-ic-fluorescent-light:before{content:\"\\e454\"}.bosch-ic-home-cloud-0:before{content:\"\\e455\"}.bosch-ic-home-cloud-1:before{content:\"\\e456\"}.bosch-ic-home-cloud-2:before{content:\"\\e457\"}.bosch-ic-home-cloud-3:before{content:\"\\e458\"}.bosch-ic-home-shadow:before{content:\"\\e459\"}.bosch-ic-motorcycle-sport:before{content:\"\\e45a\"}.bosch-ic-smartphone-connected:before{content:\"\\e45b\"}.bosch-ic-usb-stick:before{content:\"\\e45c\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-timeline-clock:before{content:\"\\e45e\"}.bosch-ic-alarm-fire:before{content:\"\\e45f\"}.bosch-ic-emergency-front:before{content:\"\\e460\"}.bosch-ic-transporter-front:before{content:\"\\e461\"}.bosch-ic-truck-front:before{content:\"\\e462\"}.bosch-ic-unpin-classic-disabled-light:before{content:\"\\e463\"}.bosch-ic-unpin-modern-disabled-light:before{content:\"\\e464\"}.bosch-ic-smartphone-ban:before{content:\"\\e465\"}.bosch-ic-helmet:before{content:\"\\e466\"}.bosch-ic-gate-closed:before{content:\"\\e467\"}.bosch-ic-fence:before{content:\"\\e468\"}.bosch-ic-desktop-apps:before{content:\"\\e469\"}.bosch-ic-candle:before{content:\"\\e46a\"}.bosch-ic-abort-frame:before{content:\"\\e886\"}.bosch-ic-arrow-down-frame:before{content:\"\\e887\"}.bosch-ic-arrow-left-down-frame:before{content:\"\\e888\"}.bosch-ic-arrow-left-frame:before{content:\"\\e889\"}.bosch-ic-arrow-left-up-frame:before{content:\"\\e88a\"}.bosch-ic-arrow-right-down-frame:before{content:\"\\e88b\"}.bosch-ic-arrow-right-frame:before{content:\"\\e88c\"}.bosch-ic-arrow-right-up-frame:before{content:\"\\e88d\"}.bosch-ic-arrow-up-frame:before{content:\"\\e88e\"}.bosch-ic-asiapacific-frame:before{content:\"\\e88f\"}.bosch-ic-cash-frame:before{content:\"\\e890\"}.bosch-ic-checkmark-frame:before{content:\"\\e891\"}.bosch-ic-copyright-frame:before{content:\"\\e892\"}.bosch-ic-download-frame:before{content:\"\\e893\"}.bosch-ic-europe-frame:before{content:\"\\e894\"}.bosch-ic-lock-closed-frame:before{content:\"\\e895\"}.bosch-ic-lock-open-frame:before{content:\"\\e896\"}.bosch-ic-my-brand-frame:before{content:\"\\e897\"}.bosch-ic-northamerica-frame:before{content:\"\\e898\"}.bosch-ic-problem-frame:before{content:\"\\e899\"}.bosch-ic-promotion-frame:before{content:\"\\e89a\"}.bosch-ic-question-frame:before{content:\"\\e89b\"}.bosch-ic-share-frame:before{content:\"\\e89c\"}.bosch-ic-southamerica-frame:before{content:\"\\e89d\"}.bosch-ic-start-play-frame:before{content:\"\\e89e\"}.bosch-ic-upload-frame:before{content:\"\\e89f\"}.bosch-ic-world-frame:before{content:\"\\e8a0\"}.bosch-ic-add-frame:before{content:\"\\e8a1\"}.bosch-ic-call-deny-frame:before{content:\"\\e8a2\"}.bosch-ic-call-frame:before{content:\"\\e8a3\"}.bosch-ic-fast-backward-frame:before{content:\"\\e8a4\"}.bosch-ic-fast-forward-frame:before{content:\"\\e8a5\"}.bosch-ic-skip-backward-frame:before{content:\"\\e8a6\"}.bosch-ic-skip-fast-backward-frame:before{content:\"\\e8a7\"}.bosch-ic-skip-fast-forward-frame:before{content:\"\\e8a8\"}.bosch-ic-skip-forward-frame:before{content:\"\\e8a9\"}.bosch-ic-subtract-frame:before{content:\"\\e8aa\"}.bosch-ic-facebook-frame:before{content:\"\\ea87\"}.bosch-ic-googleplus-frame:before{content:\"\\ea88\"}.bosch-ic-instagram-frame:before{content:\"\\ea89\"}.bosch-ic-lineapp-frame:before{content:\"\\ea8a\"}.bosch-ic-linkedin-frame:before{content:\"\\ea8b\"}.bosch-ic-pinterest-frame:before{content:\"\\ea8c\"}.bosch-ic-snapchat-frame:before{content:\"\\ea8d\"}.bosch-ic-tumblr-frame:before{content:\"\\ea8e\"}.bosch-ic-twitter-frame:before{content:\"\\ea8f\"}.bosch-ic-vimeo-frame:before{content:\"\\ea90\"}.bosch-ic-wechat-frame:before{content:\"\\ea91\"}.bosch-ic-weibo-frame:before{content:\"\\ea92\"}.bosch-ic-whatsapp-frame:before{content:\"\\ea93\"}.bosch-ic-xing-frame:before{content:\"\\ea94\"}.bosch-ic-youku-frame:before{content:\"\\ea95\"}.bosch-ic-youtube-frame:before{content:\"\\ea96\"}.bosch-ic-vk-frame:before{content:\"\\ea97\"}.bosch-ic-skype-frame:before{content:\"\\ea98\"}.bosch-ic-alert-error-filled:before{content:\"\\eb08\"}.bosch-ic-alert-error:before{content:\"\\eb09\"}.bosch-ic-alert-success-filled:before{content:\"\\eb0a\"}.bosch-ic-alert-success:before{content:\"\\eb0b\"}.bosch-ic-alert-warning-filled:before{content:\"\\eb0c\"}.bosch-ic-alert-warning:before{content:\"\\eb0d\"}.bosch-ic-chart-oee:before{content:\"\\e46b\"}.bosch-ic-cycletime-eighth:before{content:\"\\e46c\"}.bosch-ic-cycletime-quarter:before{content:\"\\e46d\"}.bosch-ic-gateway:before{content:\"\\e46e\"}.bosch-ic-heatmap:before{content:\"\\e46f\"}.bosch-ic-laptop-checklist:before{content:\"\\e470\"}.bosch-ic-molecule:before{content:\"\\e471\"}.bosch-ic-robot-settings:before{content:\"\\e472\"}.bosch-ic-sensor-cable:before{content:\"\\e473\"}.bosch-ic-tank-layers:before{content:\"\\e474\"}.bosch-ic-window-settings:before{content:\"\\e475\"}.bosch-ic-heatmap-independent:before{content:\"\\e476\"}.bosch-ic-dumble:before{content:\"\\e477\"}.bosch-ic-truck-wash:before{content:\"\\e478\"}.bosch-ic-smart-sensor:before{content:\"\\e479\"}.bosch-ic-audio-add:before{content:\"\\e47a\"}.bosch-ic-audio-checked:before{content:\"\\e47b\"}.bosch-ic-audio-play:before{content:\"\\e47c\"}.bosch-ic-audio-remove:before{content:\"\\e47d\"}.bosch-ic-image-gallery-checked:before{content:\"\\e47e\"}.bosch-ic-imagery-checked:before{content:\"\\e47f\"}.bosch-ic-image-gallery-remove:before{content:\"\\e480\"}.bosch-ic-recording:before{content:\"\\e481\"}.bosch-ic-stop-frame:before{content:\"\\e482\"}.bosch-ic-roof:before{content:\"\\e483\"}.bosch-ic-projection-interactive:before{content:\"\\e484\"}.bosch-ic-bridge:before{content:\"\\e485\"}.bosch-ic-box-arrows:before{content:\"\\e486\"}.bosch-ic-chart-line-fit:before{content:\"\\e487\"}.bosch-ic-chart-line-compress:before{content:\"\\e488\"}.bosch-ic-ciss:before{content:\"\\e489\"}.bosch-ic-scd:before{content:\"\\e48a\"}.bosch-ic-vci:before{content:\"\\e48b\"}.bosch-ic-access-controller:before{content:\"\\e48c\"}.bosch-ic-card-reader:before{content:\"\\e48d\"}.bosch-ic-table:before{content:\"\\e48e\"}.bosch-ic-replace:before{content:\"\\e48f\"}.bosch-ic-chart-topflop-organized-horizontal:before{content:\"\\e490\"}.bosch-ic-chart-topflop-random-horizontal:before{content:\"\\e491\"}.bosch-ic-chart-topflop-organized-vertical:before{content:\"\\e492\"}.bosch-ic-chart-topflop-random-vertical:before{content:\"\\e493\"}.bosch-ic-ois:before{content:\"\\e494\"}.bosch-ic-cloud-refresh-locked:before{content:\"\\e495\"}.bosch-ic-car-bluetooth:before{content:\"\\e496\"}.bosch-ic-camera-trainer:before{content:\"\\e497\"}.bosch-ic-magnifier-moving:before{content:\"\\e498\"}.bosch-ic-absolute-orientation:before{content:\"\\e499\"}.bosch-ic-accelerometer:before{content:\"\\e49a\"}.bosch-ic-imu:before{content:\"\\e49b\"}.bosch-ic-gyroscope:before{content:\"\\e49c\"}.bosch-ic-magneotmeter:before{content:\"\\e49d\"}.bosch-ic-battery-car-1:before{content:\"\\e49e\"}.bosch-ic-battery-car-2:before{content:\"\\e49f\"}.bosch-ic-battery-car-3:before{content:\"\\e4a0\"}.bosch-ic-battery-car-charging:before{content:\"\\e4a1\"}.bosch-ic-battery-car-defective:before{content:\"\\e4a2\"}.bosch-ic-truck-crane:before{content:\"\\e4a3\"}.bosch-ic-train-diesel:before{content:\"\\e4a4\"}.bosch-ic-snowmobile:before{content:\"\\e4a5\"}.bosch-ic-riding-mower:before{content:\"\\e4a6\"}.bosch-ic-motorcycle-small:before{content:\"\\e4a7\"}.bosch-ic-mining-truck:before{content:\"\\e4a8\"}.bosch-ic-lawnmower:before{content:\"\\e4a9\"}.bosch-ic-jetski:before{content:\"\\e4aa\"}.bosch-ic-golf-caddy:before{content:\"\\e4ab\"}.bosch-ic-generator-electric:before{content:\"\\e4ac\"}.bosch-ic-garbage-truck:before{content:\"\\e4ad\"}.bosch-ic-fork-lift-big:before{content:\"\\e4ae\"}.bosch-ic-engine-small:before{content:\"\\e4af\"}.bosch-ic-engine-big:before{content:\"\\e4b0\"}.bosch-ic-coach:before{content:\"\\e4b1\"}.bosch-ic-chainsaw:before{content:\"\\e4b2\"}.bosch-ic-car-van:before{content:\"\\e4b3\"}.bosch-ic-car-suv:before{content:\"\\e4b4\"}.bosch-ic-car-race:before{content:\"\\e4b5\"}.bosch-ic-car-old:before{content:\"\\e4b6\"}.bosch-ic-car-camping-van:before{content:\"\\e4b7\"}.bosch-ic-car-camper:before{content:\"\\e4b8\"}.bosch-ic-bus-side:before{content:\"\\e4b9\"}.bosch-ic-book-info:before{content:\"\\e4ba\"}.bosch-ic-gears:before{content:\"\\e4bb\"}.bosch-ic-shark:before{content:\"\\e4bc\"}.bosch-ic-mountaintop:before{content:\"\\e4bd\"}.bosch-ic-baby:before{content:\"\\e4be\"}.bosch-ic-coffee-bean:before{content:\"\\e4bf\"}.bosch-ic-coffee-beans:before{content:\"\\e4c0\"}.bosch-ic-skull:before{content:\"\\e4c1\"}.bosch-ic-stroller:before{content:\"\\e4c2\"}.bosch-ic-glasses:before{content:\"\\e4c3\"}.bosch-ic-detergent:before{content:\"\\e4c4\"}.bosch-ic-washing-machine:before{content:\"\\e4c5\"}.bosch-ic-sdcard:before{content:\"\\e4c6\"}.bosch-ic-water-connectivity:before{content:\"\\e4c7\"}.bosch-ic-server-connectivity:before{content:\"\\e4c8\"}.bosch-ic-presentation-speaker:before{content:\"\\e4c9\"}.bosch-ic-chart-bar-arrow:before{content:\"\\e4ca\"}.bosch-ic-book-connectivity:before{content:\"\\e4cb\"}.bosch-ic-route-focus:before{content:\"\\e4cc\"}.bosch-ic-bicycle-helmet:before{content:\"\\e4cd\"}.bosch-ic-cookie:before{content:\"\\e4ce\"}.bosch-ic-glass-wine:before{content:\"\\e4cf\"}.bosch-ic-cup-small:before{content:\"\\e4d0\"}.bosch-ic-cup-large:before{content:\"\\e4d1\"}.bosch-ic-bowl:before{content:\"\\e4d2\"}.bosch-ic-glass-small:before{content:\"\\e4d3\"}.bosch-ic-ice-bottle:before{content:\"\\e4d4\"}.bosch-ic-cup-plate-drop:before{content:\"\\e4d5\"}.bosch-ic-plate:before{content:\"\\e4d6\"}.bosch-ic-cup-plate:before{content:\"\\e4d7\"}.bosch-ic-bubbles:before{content:\"\\e4d8\"}.bosch-ic-button-start-stop:before{content:\"\\e4d9\"}.bosch-ic-elevator-freight-max:before{content:\"\\e4da\"}.bosch-ic-elevator-freight:before{content:\"\\e4db\"}.bosch-ic-elevator-persons:before{content:\"\\e4dc\"}.bosch-ic-embed:before{content:\"\\e4dd\"}.bosch-ic-female-male:before{content:\"\\e4de\"}.bosch-ic-fork-knife-spoon:before{content:\"\\e4df\"}.bosch-ic-global-entrepreneur:before{content:\"\\e4e0\"}.bosch-ic-arrows-left-right-checkmark:before{content:\"\\e4e1\"}.bosch-ic-automatic-mode:before{content:\"\\e4e2\"}.bosch-ic-babybottle:before{content:\"\\e4e3\"}.bosch-ic-barcode-scan:before{content:\"\\e4e4\"}.bosch-ic-barcode-scanner:before{content:\"\\e4e5\"}.bosch-ic-cash-frame-rupee:before{content:\"\\e4e6\"}.bosch-ic-chart-heat-curve:before{content:\"\\e4e7\"}.bosch-ic-compressor:before{content:\"\\e4e8\"}.bosch-ic-cubes:before{content:\"\\e4e9\"}.bosch-ic-highway:before{content:\"\\e4ea\"}.bosch-ic-industry-workers:before{content:\"\\e4eb\"}.bosch-ic-industry-workers-connected:before{content:\"\\e4ec\"}.bosch-ic-machine-station-control:before{content:\"\\e4ed\"}.bosch-ic-map-city:before{content:\"\\e4ee\"}.bosch-ic-ordernumber:before{content:\"\\e4ef\"}.bosch-ic-shirt-business:before{content:\"\\e4f0\"}.bosch-ic-shirt-casual:before{content:\"\\e4f1\"}.bosch-ic-suitcase:before{content:\"\\e4f2\"}.bosch-ic-weather-cloud-sun-sensor:before{content:\"\\e4f3\"}.bosch-ic-window-code:before{content:\"\\e4f4\"}.bosch-ic-wrench-warning:before{content:\"\\e4f5\"}.bosch-ic-arrow-circle-a:before{content:\"\\e4f6\"}.bosch-ic-arrows-agility:before{content:\"\\e4f7\"}.bosch-ic-arrows-collapse-object:before{content:\"\\e4f8\"}.bosch-ic-arrows-expand-object:before{content:\"\\e4f9\"}.bosch-ic-arrows-left-right:before{content:\"\\e4fa\"}.bosch-ic-fan-heat:before{content:\"\\e4fb\"}.bosch-ic-battery-car-arrows-circle:before{content:\"\\e4fc\"}.bosch-ic-battery-car-clock:before{content:\"\\e4fd\"}.bosch-ic-battery-vibration:before{content:\"\\e4fe\"}.bosch-ic-belt-elastic:before{content:\"\\e4ff\"}.bosch-ic-bicycle-e-arrow-away:before{content:\"\\e500\"}.bosch-ic-bicycle-e-arrow-towards:before{content:\"\\e501\"}.bosch-ic-brake-decoupled-power:before{content:\"\\e502\"}.bosch-ic-brake-i-booster:before{content:\"\\e503\"}.bosch-ic-brake-system-esp:before{content:\"\\e504\"}.bosch-ic-brake-system-integrated:before{content:\"\\e505\"}.bosch-ic-brake-vacuum-booster:before{content:\"\\e506\"}.bosch-ic-capacity:before{content:\"\\e507\"}.bosch-ic-car-beam-length:before{content:\"\\e508\"}.bosch-ic-car-smartphone-locator:before{content:\"\\e509\"}.bosch-ic-chart-pie:before{content:\"\\e50a\"}.bosch-ic-chess-jumper:before{content:\"\\e50b\"}.bosch-ic-clock-time-off:before{content:\"\\e50c\"}.bosch-ic-clock-time-on:before{content:\"\\e50d\"}.bosch-ic-coin:before{content:\"\\e50e\"}.bosch-ic-connection-off-settings:before{content:\"\\e50f\"}.bosch-ic-connection-on-settings:before{content:\"\\e510\"}.bosch-ic-control-unit:before{content:\"\\e511\"}.bosch-ic-customer:before{content:\"\\e512\"}.bosch-ic-engine:before{content:\"\\e513\"}.bosch-ic-filter-air:before{content:\"\\e514\"}.bosch-ic-filter-cabin-carbon:before{content:\"\\e515\"}.bosch-ic-filter-cabin:before{content:\"\\e516\"}.bosch-ic-filter-cooling-water:before{content:\"\\e517\"}.bosch-ic-filter-denox-cover:before{content:\"\\e518\"}.bosch-ic-filter-denox:before{content:\"\\e519\"}.bosch-ic-filter-element-compensation-sealing:before{content:\"\\e51a\"}.bosch-ic-filter-element-compensation:before{content:\"\\e51b\"}.bosch-ic-filter-element:before{content:\"\\e51c\"}.bosch-ic-filter-oil:before{content:\"\\e51d\"}.bosch-ic-filter-petrol:before{content:\"\\e51e\"}.bosch-ic-filter-prefilter:before{content:\"\\e51f\"}.bosch-ic-fireplace:before{content:\"\\e520\"}.bosch-ic-flash-plus:before{content:\"\\e521\"}.bosch-ic-globe-worldwideweb:before{content:\"\\e522\"}.bosch-ic-glow-control-unit:before{content:\"\\e523\"}.bosch-ic-glow-plug:before{content:\"\\e524\"}.bosch-ic-hand-cloth:before{content:\"\\e525\"}.bosch-ic-leaf-plus:before{content:\"\\e526\"}.bosch-ic-machine-warning:before{content:\"\\e527\"}.bosch-ic-measuring-jug:before{content:\"\\e528\"}.bosch-ic-plug-play:before{content:\"\\e529\"}.bosch-ic-refresh-checkmark:before{content:\"\\e52a\"}.bosch-ic-replacement-filament:before{content:\"\\e52b\"}.bosch-ic-screw:before{content:\"\\e52c\"}.bosch-ic-seat-belt:before{content:\"\\e52d\"}.bosch-ic-steering-wheel-hands-automated:before{content:\"\\e52e\"}.bosch-ic-sun-ice-parking:before{content:\"\\e52f\"}.bosch-ic-syringe-change-needle:before{content:\"\\e530\"}.bosch-ic-syringe-change:before{content:\"\\e531\"}.bosch-ic-syringe:before{content:\"\\e532\"}.bosch-ic-thread-diameter:before{content:\"\\e533\"}.bosch-ic-tool-extraction:before{content:\"\\e534\"}.bosch-ic-transporter-car-front:before{content:\"\\e535\"}.bosch-ic-truck-car-motorcycle:before{content:\"\\e536\"}.bosch-ic-truck-car:before{content:\"\\e537\"}.bosch-ic-truck-logistics-box:before{content:\"\\e538\"}.bosch-ic-truck-transporter:before{content:\"\\e539\"}.bosch-ic-watch-on-warning:before{content:\"\\e53a\"}.bosch-ic-wrench-calibrate:before{content:\"\\e53b\"}.bosch-ic-wrench-dummy:before{content:\"\\e53c\"}.bosch-ic-wrench-tightening-torque:before{content:\"\\e53d\"}.bosch-ic-wrench-width-across-flats:before{content:\"\\e53e\"}.bosch-ic-user-arm-bandage:before{content:\"\\e53f\"}.bosch-ic-user-eye-blind:before{content:\"\\e540\"}.bosch-ic-user-ear-deaf:before{content:\"\\e541\"}.bosch-ic-user-car-distracted:before{content:\"\\e542\"}.bosch-ic-user-mouth-mute:before{content:\"\\e543\"}.bosch-ic-user-ear-blocked:before{content:\"\\e544\"}.bosch-ic-user-bavarian:before{content:\"\\e545\"}.bosch-ic-user-ear-loud:before{content:\"\\e546\"}.bosch-ic-user-walk-crutch:before{content:\"\\e547\"}.bosch-ic-user-mouth-speechless:before{content:\"\\e548\"}.bosch-ic-user-arm-baby:before{content:\"\\e549\"}.bosch-ic-user-arm-missing:before{content:\"\\e54a\"}.bosch-ic-user-eye-eyepatch:before{content:\"\\e54b\"}.bosch-ic-user-strapped:before{content:\"\\e54c\"}.bosch-ic-cleaning-agent:before{content:\"\\e54d\"}.bosch-ic-cup-plate-dirty:before{content:\"\\e54e\"}.bosch-ic-cup-plate-private:before{content:\"\\e54f\"}.bosch-ic-rubbish-apple:before{content:\"\\e550\"}.bosch-ic-rubbish-fish:before{content:\"\\e551\"}.bosch-ic-rubbish-organic-bin:before{content:\"\\e552\"}.bosch-ic-rubbish-paper:before{content:\"\\e553\"}.bosch-ic-rubbish-paper-bin:before{content:\"\\e554\"}.bosch-ic-rubbish-residual-bin:before{content:\"\\e555\"}.bosch-ic-shower:before{content:\"\\e556\"}.bosch-ic-shower-female:before{content:\"\\e557\"}.bosch-ic-shower-male:before{content:\"\\e558\"}.bosch-ic-wardrobe-female:before{content:\"\\e559\"}.bosch-ic-wardrobe-male:before{content:\"\\e55a\"}.bosch-ic-car-top:before{content:\"\\e55b\"}.bosch-ic-label-info:before{content:\"\\e55c\"}.bosch-ic-perspective-view:before{content:\"\\e55d\"}.bosch-ic-road:before{content:\"\\e55e\"}.bosch-ic-screen-split-horizontal:before{content:\"\\e55f\"}.bosch-ic-screen-split-vertical:before{content:\"\\e560\"}.bosch-ic-screenshot-frame:before{content:\"\\e561\"}.bosch-ic-sensor-lidar:before{content:\"\\e562\"}.bosch-ic-tram:before{content:\"\\e563\"}.bosch-ic-video-360-degrees:before{content:\"\\e564\"}.bosch-ic-belt-groove-drives:before{content:\"\\e565\"}.bosch-ic-bottle-champagne:before{content:\"\\e566\"}.bosch-ic-box-falling:before{content:\"\\e567\"}.bosch-ic-box-falling-off-disabled-bold:before{content:\"\\e568\"}.bosch-ic-box-falling-off-disabled-light:before{content:\"\\e569\"}.bosch-ic-brake-pedal:before{content:\"\\e56a\"}.bosch-ic-car-front-light-inside:before{content:\"\\e56b\"}.bosch-ic-car-licence-plate-light:before{content:\"\\e56c\"}.bosch-ic-car-side-light-back:before{content:\"\\e56d\"}.bosch-ic-car-side-light-front:before{content:\"\\e56e\"}.bosch-ic-light-fog-warning:before{content:\"\\e56f\"}.bosch-ic-prism:before{content:\"\\e570\"}.bosch-ic-turnstile:before{content:\"\\e571\"}.bosch-ic-turnstile-open-in-and-outbound:before{content:\"\\e572\"}.bosch-ic-turnstile-open-inbound:before{content:\"\\e573\"}.bosch-ic-turnstile-open-outbound:before{content:\"\\e574\"}.bosch-ic-uwb:before{content:\"\\e575\"}.bosch-ic-heat-reduce-active:before{content:\"\\e576\"}.bosch-ic-heat-reduce-passive:before{content:\"\\e577\"}.bosch-ic-hourglass-add:before{content:\"\\e578\"}.bosch-ic-light-fog:before{content:\"\\e579\"}.bosch-ic-car-turn-signal:before{content:\"\\e57a\"}.bosch-ic-car-turn-signal-left:before{content:\"\\e57b\"}.bosch-ic-car-turn-signal-right:before{content:\"\\e57c\"}.bosch-ic-light-rear:before{content:\"\\e57d\"}.bosch-ic-car-top-turn-signal-front:before{content:\"\\e57e\"}.bosch-ic-car-top-turn-signal-back:before{content:\"\\e57f\"}.bosch-ic-ccu-cm:before{content:\"\\e580\"}.bosch-ic-lung:before{content:\"\\e581\"}.bosch-ic-head-speaking-voice:before{content:\"\\e582\"}.bosch-ic-car-motorcycle:before{content:\"\\e583\"}.bosch-ic-car-turn-signal-side:before{content:\"\\e584\"}.bosch-ic-car-light-parking:before{content:\"\\e585\"}.bosch-ic-phoenix:before{content:\"\\e586\"}.bosch-ic-sensor-air-quality:before{content:\"\\e587\"}.bosch-ic-weather-station:before{content:\"\\e588\"}.bosch-ic-watering-can:before{content:\"\\e589\"}.bosch-ic-propeller:before{content:\"\\e58a\"}.bosch-ic-temperature-fluctuating:before{content:\"\\e58b\"}.bosch-ic-stove-top:before{content:\"\\e58c\"}.bosch-ic-socket:before{content:\"\\e58d\"}.bosch-ic-siren:before{content:\"\\e58e\"}.bosch-ic-sensor-water:before{content:\"\\e58f\"}.bosch-ic-user-run-sensor-motion:before{content:\"\\e590\"}.bosch-ic-robot-lawnmower:before{content:\"\\e591\"}.bosch-ic-robot-cleaning:before{content:\"\\e592\"}.bosch-ic-oven:before{content:\"\\e593\"}.bosch-ic-loudspeaker:before{content:\"\\e594\"}.bosch-ic-keypad:before{content:\"\\e595\"}.bosch-ic-shape-circle-square:before{content:\"\\e596\"}.bosch-ic-garage:before{content:\"\\e597\"}.bosch-ic-fume-hood:before{content:\"\\e598\"}.bosch-ic-freezer:before{content:\"\\e599\"}.bosch-ic-food-processor:before{content:\"\\e59a\"}.bosch-ic-floorplan-rooms:before{content:\"\\e59b\"}.bosch-ic-dryer-tumble:before{content:\"\\e59c\"}.bosch-ic-dryer:before{content:\"\\e59d\"}.bosch-ic-door-window:before{content:\"\\e59e\"}.bosch-ic-door-lock:before{content:\"\\e59f\"}.bosch-ic-door-bell:before{content:\"\\e5a0\"}.bosch-ic-dishwasher:before{content:\"\\e5a1\"}.bosch-ic-dimmer:before{content:\"\\e5a2\"}.bosch-ic-detector-smoke:before{content:\"\\e5a3\"}.bosch-ic-detector-presence:before{content:\"\\e5a4\"}.bosch-ic-cooking:before{content:\"\\e5a5\"}.bosch-ic-coffee-maschine:before{content:\"\\e5a6\"}.bosch-ic-camera-indoor:before{content:\"\\e5a7\"}.bosch-ic-broom-clean:before{content:\"\\e5a8\"}.bosch-ic-blinds:before{content:\"\\e5a9\"}.bosch-ic-bicycle-e-flash:before{content:\"\\e5aa\"}.bosch-ic-finger-domino-trigger:before{content:\"\\e5ab\"}.bosch-ic-automation-start:before{content:\"\\e5ac\"}.bosch-ic-automation:before{content:\"\\e5ad\"}.bosch-ic-sensor-multifunctional:before{content:\"\\e5ae\"}.bosch-ic-plane-top:before{content:\"\\e5af\"}.bosch-ic-shape-square-overlapping:before{content:\"\\e5b0\"}.bosch-ic-square-overlapping:before{content:\"\\e5b1\"}.bosch-ic-battery-car-14v:before{content:\"\\e5b2\"}.bosch-ic-calibration:before{content:\"\\e5b3\"}.bosch-ic-structure-can-bus:before{content:\"\\e5b4\"}.bosch-ic-gears-direction-arrows:before{content:\"\\e5b5\"}.bosch-ic-car-limousine:before{content:\"\\e5b6\"}.bosch-ic-molecule-gas:before{content:\"\\e5b7\"}.bosch-ic-money-euro:before{content:\"\\e5b8\"}.bosch-ic-revolution-rpm:before{content:\"\\e5b9\"}.bosch-ic-rikscha-e:before{content:\"\\e5ba\"}.bosch-ic-sensor-circle:before{content:\"\\e5bb\"}.bosch-ic-speedometer-high:before{content:\"\\e5bc\"}.bosch-ic-steering-wheel-arrows:before{content:\"\\e5bd\"}.bosch-ic-tablet-checklist:before{content:\"\\e5be\"}.bosch-ic-petrol-charging-station-hybrid:before{content:\"\\e5bf\"}.bosch-ic-weather-cloud-sun-rain-snow:before{content:\"\\e5c0\"}.bosch-ic-light-half:before{content:\"\\e5c1\"}.bosch-ic-video-sensor:before{content:\"\\e5c2\"}.bosch-ic-binary-desktop:before{content:\"\\e5c3\"}.bosch-ic-truck-delivery:before{content:\"\\e5c4\"}.bosch-ic-van-delivery:before{content:\"\\e5c5\"}.bosch-ic-fullscreen-user:before{content:\"\\e5c6\"}.bosch-ic-user-helmet-motorcycle:before{content:\"\\e5c7\"}.bosch-ic-user-hand-disabled-light:before{content:\"\\e5c8\"}.bosch-ic-user-hand-disabled-bold:before{content:\"\\e5c9\"}.bosch-ic-link-connected:before{content:\"\\e5ca\"}.bosch-ic-gate-open:before{content:\"\\e5cb\"}.bosch-ic-box-checkmark:before{content:\"\\e5cc\"}.bosch-ic-box-place:before{content:\"\\e5cd\"}.bosch-ic-logistics-dock:before{content:\"\\e5ce\"}.bosch-ic-barcode-scan-logistics:before{content:\"\\e5cf\"}.bosch-ic-locator-tracking:before{content:\"\\e5d0\"}.bosch-ic-car-light-trunk:before{content:\"\\e5d1\"}.bosch-ic-car-light-glove-compartment:before{content:\"\\e5d2\"}.bosch-ic-car-light-dashboard:before{content:\"\\e5d3\"}.bosch-ic-fire-leaf:before{content:\"\\e5d4\"}.bosch-ic-laptop-hacker:before{content:\"\\e5d5\"}.bosch-ic-hands-drops:before{content:\"\\e5d6\"}.bosch-ic-team-disabled-bold:before{content:\"\\e5d7\"}.bosch-ic-team-disabled-light:before{content:\"\\e5d8\"}.bosch-ic-user-head-arm:before{content:\"\\e5d9\"}.bosch-ic-user-head-hand-disabled-bold:before{content:\"\\e5da\"}.bosch-ic-user-head-hand-disabled-light:before{content:\"\\e5db\"}.bosch-ic-welcome-disabled-bold:before{content:\"\\e5dc\"}.bosch-ic-welcome-disabled-light:before{content:\"\\e5dd\"}.bosch-ic-female-male-arrows:before{content:\"\\e5de\"}.bosch-ic-broccoli:before{content:\"\\e5df\"}.bosch-ic-chicken:before{content:\"\\e5e0\"}.bosch-ic-cow:before{content:\"\\e5e1\"}.bosch-ic-deer:before{content:\"\\e5e2\"}.bosch-ic-fish:before{content:\"\\e5e3\"}.bosch-ic-garlic:before{content:\"\\e5e4\"}.bosch-ic-hotpepper:before{content:\"\\e5e5\"}.bosch-ic-pig:before{content:\"\\e5e6\"}.bosch-ic-rabbit:before{content:\"\\e5e7\"}.bosch-ic-sheep:before{content:\"\\e5e8\"}.bosch-ic-shell:before{content:\"\\e5e9\"}.bosch-ic-shrimp:before{content:\"\\e5ea\"}.bosch-ic-car-seat:before{content:\"\\e5eb\"}.bosch-ic-car-seat-steering-wheel:before{content:\"\\e5ec\"}.bosch-ic-car-side-arrow-back:before{content:\"\\e5ed\"}.bosch-ic-car-side-arrow-front:before{content:\"\\e5ee\"}.bosch-ic-car-steering-left-disabled-bold:before{content:\"\\e5ef\"}.bosch-ic-car-steering-left-disabled-light:before{content:\"\\e5f0\"}.bosch-ic-car-steering-left-lle:before{content:\"\\e5f1\"}.bosch-ic-car-steering-left-lle-disabled-bold:before{content:\"\\e5f2\"}.bosch-ic-car-steering-left-lle-disabled-light:before{content:\"\\e5f3\"}.bosch-ic-car-steering-right-disabled-bold:before{content:\"\\e5f4\"}.bosch-ic-car-steering-right-disabled-light:before{content:\"\\e5f5\"}.bosch-ic-car-steering-right-rle:before{content:\"\\e5f6\"}.bosch-ic-car-steering-right-rle-disabled-bold:before{content:\"\\e5f7\"}.bosch-ic-car-steering-right-rle-disabled-light:before{content:\"\\e5f8\"}.bosch-ic-cars-traffic-left-right:before{content:\"\\e5f9\"}.bosch-ic-configuration-wrench:before{content:\"\\e5fa\"}.bosch-ic-hook:before{content:\"\\e5fb\"}.bosch-ic-hook-arrows:before{content:\"\\e5fc\"}.bosch-ic-hook-arrows-wiperblade:before{content:\"\\e5fd\"}.bosch-ic-hook-disabled-bold:before{content:\"\\e5fe\"}.bosch-ic-hook-disabled-light:before{content:\"\\e5ff\"}.bosch-ic-info-i-frame-add:before{content:\"\\e600\"}.bosch-ic-search-number:before{content:\"\\e601\"}.bosch-ic-search-number-add:before{content:\"\\e602\"}.bosch-ic-set:before{content:\"\\e603\"}.bosch-ic-speedometer-speed-high:before{content:\"\\e604\"}.bosch-ic-windshield-wiper-clean:before{content:\"\\e605\"}.bosch-ic-windshield-wiper-double:before{content:\"\\e606\"}.bosch-ic-wiper-adapter:before{content:\"\\e607\"}.bosch-ic-wiper-blade:before{content:\"\\e608\"}.bosch-ic-wiper-blade-arrow:before{content:\"\\e609\"}.bosch-ic-wiper-blade-eco:before{content:\"\\e60a\"}.bosch-ic-wiper-blade-mm:before{content:\"\\e60b\"}.bosch-ic-wiper-blade-mm-arrows:before{content:\"\\e60c\"}.bosch-ic-wiper-blade-spoiler:before{content:\"\\e60d\"}.bosch-ic-wiper-blade-twin:before{content:\"\\e60e\"}.bosch-ic-wiper-flatblade:before{content:\"\\e60f\"}.bosch-ic-wiper-flatblade-aeroeco:before{content:\"\\e610\"}.bosch-ic-wiper-flatblade-aerofit:before{content:\"\\e611\"}.bosch-ic-wiper-flatblade-aerotwin:before{content:\"\\e612\"}.bosch-ic-wiper-flatblade-arrow:before{content:\"\\e613\"}.bosch-ic-wiper-flatblade-mm:before{content:\"\\e614\"}.bosch-ic-wiper-flatblade-mm-arrows:before{content:\"\\e615\"}.bosch-ic-wiper-rubber:before{content:\"\\e616\"}.bosch-ic-address-control-system:before{content:\"\\e617\"}.bosch-ic-balcony:before{content:\"\\e618\"}.bosch-ic-battery-car-45-degree-arrow:before{content:\"\\e619\"}.bosch-ic-boiler-warning:before{content:\"\\e61a\"}.bosch-ic-book-wrench:before{content:\"\\e61b\"}.bosch-ic-car-driver-passenger-arrows:before{content:\"\\e61c\"}.bosch-ic-car-licence-plate:before{content:\"\\e61d\"}.bosch-ic-car-side-arrow-back-tires:before{content:\"\\e61e\"}.bosch-ic-car-side-arrow-front-tires:before{content:\"\\e61f\"}.bosch-ic-car-side-half-arrow-back:before{content:\"\\e620\"}.bosch-ic-car-steering-left-arrow:before{content:\"\\e621\"}.bosch-ic-car-steering-left-passenger-arrow:before{content:\"\\e622\"}.bosch-ic-car-steering-right-arrow:before{content:\"\\e623\"}.bosch-ic-car-steering-right-passenger-arrow:before{content:\"\\e624\"}.bosch-ic-chess-board-transparency:before{content:\"\\e625\"}.bosch-ic-cloud-locked:before{content:\"\\e626\"}.bosch-ic-conference-system:before{content:\"\\e627\"}.bosch-ic-cycletime-quarter-arrows-interval:before{content:\"\\e628\"}.bosch-ic-desktop-connectivity:before{content:\"\\e629\"}.bosch-ic-hierarchy-checkmark:before{content:\"\\e62a\"}.bosch-ic-home-office:before{content:\"\\e62b\"}.bosch-ic-home-semi-detached-house:before{content:\"\\e62c\"}.bosch-ic-kitchen:before{content:\"\\e62d\"}.bosch-ic-lambda-arrows-length:before{content:\"\\e62e\"}.bosch-ic-lambda-oxygen:before{content:\"\\e62f\"}.bosch-ic-lambda-sensor-cores:before{content:\"\\e630\"}.bosch-ic-panel-control:before{content:\"\\e631\"}.bosch-ic-prison:before{content:\"\\e632\"}.bosch-ic-prison-cell:before{content:\"\\e633\"}.bosch-ic-radio-broadcasting:before{content:\"\\e634\"}.bosch-ic-robot-body:before{content:\"\\e635\"}.bosch-ic-robot-body-arrow:before{content:\"\\e636\"}.bosch-ic-skyscraper-danger:before{content:\"\\e637\"}.bosch-ic-skyscraper-flat:before{content:\"\\e638\"}.bosch-ic-skyscraper-flat-floor-low:before{content:\"\\e639\"}.bosch-ic-skyscraper-flat-floor-middle:before{content:\"\\e63a\"}.bosch-ic-skyscraper-flat-floor-top:before{content:\"\\e63b\"}.bosch-ic-skyscraper-flat-semi-detached:before{content:\"\\e63c\"}.bosch-ic-skyscraper-locked:before{content:\"\\e63d\"}.bosch-ic-skyscraper-settings:before{content:\"\\e63e\"}.bosch-ic-team-signpost:before{content:\"\\e63f\"}.bosch-ic-toaster-hand-warning:before{content:\"\\e640\"}.bosch-ic-user-arrows-heart:before{content:\"\\e641\"}.bosch-ic-user-graduate-chat:before{content:\"\\e642\"}.bosch-ic-user-head-mask:before{content:\"\\e643\"}.bosch-ic-warehouse:before{content:\"\\e644\"}.bosch-ic-world-paperplane:before{content:\"\\e645\"}.bosch-ic-aperture:before{content:\"\\e646\"}.bosch-ic-hook-wiperblade-arrows:before{content:\"\\e647\"}.bosch-ic-image-remove:before{content:\"\\e648\"}.bosch-ic-image-checked:before{content:\"\\e649\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-apartment-in-house-terraced:before{content:\"\\e64b\"}.bosch-ic-apartment-in-house:before{content:\"\\e64c\"}.bosch-ic-appliance-heat-pump-leaf-add:before{content:\"\\e64d\"}.bosch-ic-appliance-heat-pump-leaf-reload:before{content:\"\\e64e\"}.bosch-ic-appliance-heat-pump-oil-refresh:before{content:\"\\e64f\"}.bosch-ic-appliance-water-drop:before{content:\"\\e650\"}.bosch-ic-appliance-water-sun:before{content:\"\\e651\"}.bosch-ic-brake-pedal-thumb-up:before{content:\"\\e652\"}.bosch-ic-call-plus:before{content:\"\\e653\"}.bosch-ic-battery-car-3-plug:before{content:\"\\e654\"}.bosch-ic-cloud-share:before{content:\"\\e655\"}.bosch-ic-crankshaft:before{content:\"\\e656\"}.bosch-ic-detector-motion:before{content:\"\\e657\"}.bosch-ic-device-analog:before{content:\"\\e658\"}.bosch-ic-device-digital:before{content:\"\\e659\"}.bosch-ic-externallink-landscape:before{content:\"\\e65a\"}.bosch-ic-gears-interlocked-arrow:before{content:\"\\e65b\"}.bosch-ic-gearshift-reverse-gear:before{content:\"\\e65c\"}.bosch-ic-glove:before{content:\"\\e65d\"}.bosch-ic-hand-warranty:before{content:\"\\e65e\"}.bosch-ic-heating-floor-radiator:before{content:\"\\e65f\"}.bosch-ic-home-appliance-heat-pump-air-in:before{content:\"\\e660\"}.bosch-ic-home-appliance-heat-pump-air-out:before{content:\"\\e661\"}.bosch-ic-lightbulb-gear:before{content:\"\\e662\"}.bosch-ic-line-open-closed:before{content:\"\\e663\"}.bosch-ic-road-disabled-bold:before{content:\"\\e664\"}.bosch-ic-road-disabled-light:before{content:\"\\e665\"}.bosch-ic-shower-drop-1:before{content:\"\\e666\"}.bosch-ic-shower-drop-2:before{content:\"\\e667\"}.bosch-ic-shower-drop-3:before{content:\"\\e668\"}.bosch-ic-shower-drop-4:before{content:\"\\e669\"}.bosch-ic-sinus-curve:before{content:\"\\e66a\"}.bosch-ic-team-3:before{content:\"\\e66b\"}.bosch-ic-team-4:before{content:\"\\e66c\"}.bosch-ic-team-5:before{content:\"\\e66d\"}.bosch-ic-test-tube:before{content:\"\\e66e\"}.bosch-ic-truck-car-motorcycle-arrows:before{content:\"\\e66f\"}.bosch-ic-weather-cloud-dust:before{content:\"\\e670\"}.bosch-ic-windshield-wiper-double-parallel-left:before{content:\"\\e671\"}.bosch-ic-windshield-wiper-double-parallel-right:before{content:\"\\e672\"}.bosch-ic-world-pin:before{content:\"\\e673\"}.bosch-ic-sign-warning-slope:before{content:\"\\e674\"}.bosch-ic-facebook:before{content:\"\\ea99\"}.bosch-ic-google-frame:before{content:\"\\ea9a\"}.bosch-ic-google:before{content:\"\\ea9b\"}.bosch-ic-googleplus:before{content:\"\\ea9c\"}.bosch-ic-instagram:before{content:\"\\ea9d\"}.bosch-ic-lineapp:before{content:\"\\ea9e\"}.bosch-ic-linkedin:before{content:\"\\ea9f\"}.bosch-ic-pinterest:before{content:\"\\eaa0\"}.bosch-ic-skype:before{content:\"\\eaa1\"}.bosch-ic-snapchat:before{content:\"\\eaa2\"}.bosch-ic-tumblr:before{content:\"\\eaa3\"}.bosch-ic-twitter:before{content:\"\\eaa4\"}.bosch-ic-vimeo:before{content:\"\\eaa5\"}.bosch-ic-vk:before{content:\"\\eaa6\"}.bosch-ic-wechat:before{content:\"\\eaa7\"}.bosch-ic-weibo:before{content:\"\\eaa8\"}.bosch-ic-whatsapp:before{content:\"\\eaa9\"}.bosch-ic-xing:before{content:\"\\eaaa\"}.bosch-ic-youku:before{content:\"\\eaab\"}.bosch-ic-youtube:before{content:\"\\eaac\"}.bosch-ic-alert-info-filled:before{content:\"\\eb0e\"}.bosch-ic-alert-info:before{content:\"\\eb0f\"}.bosch-ic-info-i-frame:before{content:\"\\e8ab\"}.bosch-ic-stop-frame1:before{content:\"\\e8ac\"}.bosch-ic-arrow-up-frame-error1:before{content:\"\\e8ad\"}.bosch-ic-arrow-up-frame-check1:before{content:\"\\e8ae\"}.bosch-ic-arrow-left-end-frame1:before{content:\"\\e8af\"}.bosch-ic-arrow-right-end-frame1:before{content:\"\\e8b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-warranty:before{content:\"\\e2b1\"}.bosch-ic-bond-laser:before{content:\"\\e675\"}.bosch-ic-bond-ribbon-thick:before{content:\"\\e676\"}.bosch-ic-bond-ribbon-thin:before{content:\"\\e677\"}.bosch-ic-bond-wire-thick:before{content:\"\\e678\"}.bosch-ic-bond-wire-thin:before{content:\"\\e679\"}.bosch-ic-clamp-connection-insulation:before{content:\"\\e67a\"}.bosch-ic-clamp-connection-stamping-grid:before{content:\"\\e67b\"}.bosch-ic-clinching:before{content:\"\\e67c\"}.bosch-ic-connector:before{content:\"\\e67d\"}.bosch-ic-connector-potting:before{content:\"\\e67e\"}.bosch-ic-element-damping:before{content:\"\\e67f\"}.bosch-ic-element-pressure-compensation:before{content:\"\\e680\"}.bosch-ic-flanging:before{content:\"\\e681\"}.bosch-ic-gap-pad:before{content:\"\\e682\"}.bosch-ic-heat-sink:before{content:\"\\e683\"}.bosch-ic-hook-clipsing:before{content:\"\\e684\"}.bosch-ic-interconnection-board-to-board:before{content:\"\\e685\"}.bosch-ic-labyrinth-geometry:before{content:\"\\e686\"}.bosch-ic-lacquering-pcb:before{content:\"\\e687\"}.bosch-ic-magnetic-rotor:before{content:\"\\e688\"}.bosch-ic-magnetic-switch:before{content:\"\\e689\"}.bosch-ic-pin-pressfit:before{content:\"\\e68a\"}.bosch-ic-power-contact:before{content:\"\\e68b\"}.bosch-ic-rivet:before{content:\"\\e68c\"}.bosch-ic-seal-ring:before{content:\"\\e68d\"}.bosch-ic-sealing-cipg:before{content:\"\\e68e\"}.bosch-ic-sealing-solid:before{content:\"\\e68f\"}.bosch-ic-sealing-tongue-groove:before{content:\"\\e690\"}.bosch-ic-shrinking-arrows-circle:before{content:\"\\e691\"}.bosch-ic-solder-selective:before{content:\"\\e692\"}.bosch-ic-spring-mechanical:before{content:\"\\e693\"}.bosch-ic-staking-hot-air:before{content:\"\\e694\"}.bosch-ic-stitching:before{content:\"\\e695\"}.bosch-ic-thermal-inlay:before{content:\"\\e696\"}.bosch-ic-thermal-via:before{content:\"\\e697\"}.bosch-ic-tube:before{content:\"\\e698\"}.bosch-ic-tube-gapfiller:before{content:\"\\e699\"}.bosch-ic-tube-gelling:before{content:\"\\e69a\"}.bosch-ic-tube-gluing:before{content:\"\\e69b\"}.bosch-ic-tube-thermal-adhesive:before{content:\"\\e69c\"}.bosch-ic-tube-underfill:before{content:\"\\e69d\"}.bosch-ic-welding-laser:before{content:\"\\e69e\"}.bosch-ic-welding-resistance:before{content:\"\\e69f\"}.bosch-ic-welding-ultrasonic:before{content:\"\\e6a0\"}.bosch-ic-air-condition-cassette:before{content:\"\\e6a1\"}.bosch-ic-air-condition-ceiling:before{content:\"\\e6a2\"}.bosch-ic-air-condition-convertible:before{content:\"\\e6a3\"}.bosch-ic-air-condition-duct:before{content:\"\\e6a4\"}.bosch-ic-air-condition-floor:before{content:\"\\e6a5\"}.bosch-ic-air-condition-outdoor:before{content:\"\\e6a6\"}.bosch-ic-air-condition-outdoor-hp:before{content:\"\\e6a7\"}.bosch-ic-air-condition-outdoor-hr:before{content:\"\\e6a8\"}.bosch-ic-air-condition-swing-horizontal:before{content:\"\\e6a9\"}.bosch-ic-air-condition-swing-vertical:before{content:\"\\e6aa\"}.bosch-ic-air-condition-wall:before{content:\"\\e6ab\"}.bosch-ic-angle-curvature-motorcycle:before{content:\"\\e6ac\"}.bosch-ic-appliance-hot:before{content:\"\\e6ad\"}.bosch-ic-appliance-water-hot:before{content:\"\\e6ae\"}.bosch-ic-appliances-hot:before{content:\"\\e6af\"}.bosch-ic-arrows-left-right-a:before{content:\"\\e6b0\"}.bosch-ic-arrows-left-right-exchange:before{content:\"\\e6b1\"}.bosch-ic-arrows-left-right-exchange-unit:before{content:\"\\e6b2\"}.bosch-ic-arrows-left-right-ice:before{content:\"\\e6b3\"}.bosch-ic-assembly-line-robot:before{content:\"\\e6b4\"}.bosch-ic-back-left-small:before{content:\"\\e6b5\"}.bosch-ic-bacteria:before{content:\"\\e6b6\"}.bosch-ic-battery-car-drop-leakage-proof:before{content:\"\\e6b7\"}.bosch-ic-battery-car-hourglass:before{content:\"\\e6b8\"}.bosch-ic-brake-disk-arrow:before{content:\"\\e6b9\"}.bosch-ic-brake-disk-compound:before{content:\"\\e6ba\"}.bosch-ic-building-religious-church:before{content:\"\\e6bb\"}.bosch-ic-calendar-sheet:before{content:\"\\e6bc\"}.bosch-ic-caliper:before{content:\"\\e6bd\"}.bosch-ic-car-middle-arrow:before{content:\"\\e6be\"}.bosch-ic-checkmark-bold:before{content:\"\\e6bf\"}.bosch-ic-close-small:before{content:\"\\e6c0\"}.bosch-ic-data-center:before{content:\"\\e6c1\"}.bosch-ic-dot:before{content:\"\\e6c2\"}.bosch-ic-down-small:before{content:\"\\e6c3\"}.bosch-ic-e-call:before{content:\"\\e6c4\"}.bosch-ic-fire-extinguisher:before{content:\"\\e6c5\"}.bosch-ic-fire-off-bold:before{content:\"\\e6c6\"}.bosch-ic-fire-off-light:before{content:\"\\e6c7\"}.bosch-ic-forward-right-small:before{content:\"\\e6c8\"}.bosch-ic-health-warning:before{content:\"\\e6c9\"}.bosch-ic-ignition-coil-pencil:before{content:\"\\e6ca\"}.bosch-ic-ignition-condensor:before{content:\"\\e6cb\"}.bosch-ic-ignition-contact-set:before{content:\"\\e6cc\"}.bosch-ic-ignition-distributor-cap:before{content:\"\\e6cd\"}.bosch-ic-ignition-distributor-rotor:before{content:\"\\e6ce\"}.bosch-ic-ignition-module:before{content:\"\\e6cf\"}.bosch-ic-less-minimize-bold:before{content:\"\\e6d0\"}.bosch-ic-leaf-plus-boost:before{content:\"\\e6d1\"}.bosch-ic-sensor-dynamic:before{content:\"\\e6d2\"}.bosch-ic-sensor-radar:before{content:\"\\e6d3\"}.bosch-ic-up-small:before{content:\"\\e6d4\"}.bosch-ic-water-ladder-off-bold:before{content:\"\\e6d5\"}.bosch-ic-water-ladder-off-light:before{content:\"\\e6d6\"}.bosch-ic-nose-desodorize:before{content:\"\\e6d7\"}.bosch-ic-nose-fine-dust:before{content:\"\\e6d8\"}.bosch-ic-smartcity-camera:before{content:\"\\e6d9\"}.bosch-ic-heating-underfloor:before{content:\"\\e6da\"}.bosch-ic-drill-bit:before{content:\"\\e6db\"}.bosch-ic-parameter-process:before{content:\"\\e6dc\"}.bosch-ic-car-side-plug-flash:before{content:\"\\e6dd\"}.bosch-ic-car-side-plug-engine-flash:before{content:\"\\e6de\"}.bosch-ic-car-side-engine-flash:before{content:\"\\e6df\"}.bosch-ic-car-side-hydrogen-flash:before{content:\"\\e6e0\"}.bosch-ic-car-side-engine:before{content:\"\\e6e1\"}.bosch-ic-tunnel:before{content:\"\\e6e2\"}.bosch-ic-stadium-sport:before{content:\"\\e6e3\"}.bosch-ic-refinery-oil:before{content:\"\\e6e4\"}.bosch-ic-power-plants:before{content:\"\\e6e5\"}.bosch-ic-hammer-chisel:before{content:\"\\e6e6\"}.bosch-ic-glass-cocktail:before{content:\"\\e6e7\"}.bosch-ic-traffic-light:before{content:\"\\e6e8\"}.bosch-ic-vehicle-off-road:before{content:\"\\e6e9\"}.bosch-ic-tablet-finger:before{content:\"\\e6ea\"}.bosch-ic-scooter-kick:before{content:\"\\e6eb\"}.bosch-ic-handlebar-motorcycle:before{content:\"\\e6ec\"}.bosch-ic-display-motorcycle:before{content:\"\\e6ed\"}.bosch-ic-case-label-product:before{content:\"\\e6ee\"}.bosch-ic-case-label:before{content:\"\\e6ef\"}.bosch-ic-product-falling:before{content:\"\\e6f0\"}.bosch-ic-product-checkmark:before{content:\"\\e6f1\"}.bosch-ic-product-broken:before{content:\"\\e6f2\"}.bosch-ic-lightbulb-halogen-rays-bright:before{content:\"\\e6f3\"}.bosch-ic-lightbulb-halogen-rays:before{content:\"\\e6f4\"}.bosch-ic-lamp-gas-discharge-rays:before{content:\"\\e6f5\"}.bosch-ic-home-lightbulb-off-disabled-bold:before{content:\"\\e6f6\"}.bosch-ic-home-lightbulb-off-disabled-light:before{content:\"\\e6f7\"}.bosch-ic-fork-lift-boxes:before{content:\"\\e6f8\"}.bosch-ic-harbour-ship-cargo:before{content:\"\\e6f9\"}.bosch-ic-rollercoaster:before{content:\"\\e6fa\"}.bosch-ic-sound-off-light:before{content:\"\\e6fb\"}.bosch-ic-sound-off-bold:before{content:\"\\e6fc\"}.bosch-ic-car-door:before{content:\"\\e6fd\"}.bosch-ic-car-door-tailgate:before{content:\"\\e6fe\"}.bosch-ic-car-door-split-rear:before{content:\"\\e6ff\"}.bosch-ic-car-door-rear:before{content:\"\\e700\"}.bosch-ic-drill-driver-cordless:before{content:\"\\e701\"}.bosch-ic-virus:before{content:\"\\e702\"}.bosch-ic-ice-temperature:before{content:\"\\e703\"}.bosch-ic-sensor-ultrasonic:before{content:\"\\e704\"}.bosch-ic-user-artificial-intelligence:before{content:\"\\e705\"}.bosch-ic-detector-leak:before{content:\"\\e706\"}.bosch-ic-car-seat-add:before{content:\"\\e707\"}.bosch-ic-car-shuttle-robo:before{content:\"\\e708\"}.bosch-ic-caravan:before{content:\"\\e709\"}.bosch-ic-user-water-swimming:before{content:\"\\e70a\"}.bosch-ic-square-add:before{content:\"\\e70b\"}.bosch-ic-map-zoom-in:before{content:\"\\e70c\"}.bosch-ic-map-arrow-pop-out:before{content:\"\\e70d\"}.bosch-ic-home-user-in-error:before{content:\"\\e70e\"}.bosch-ic-home-arrow-right-in-denied:before{content:\"\\e70f\"}.bosch-ic-home-arrow-right-in-check:before{content:\"\\e710\"}.bosch-ic-home-arrow-right-in:before{content:\"\\e711\"}.bosch-ic-home-arrow-left-out:before{content:\"\\e712\"}.bosch-ic-file-error:before{content:\"\\e713\"}.bosch-ic-dealer-details-missing:before{content:\"\\e714\"}.bosch-ic-dealer-details-check:before{content:\"\\e715\"}.bosch-ic-components-add:before{content:\"\\e716\"}.bosch-ic-machine-cell:before{content:\"\\e717\"}.bosch-ic-wheel-rim-clean:before{content:\"\\e718\"}.bosch-ic-gas-can:before{content:\"\\e719\"}.bosch-ic-steps-locator:before{content:\"\\e71a\"}.bosch-ic-panel-control-manual:before{content:\"\\e71b\"}.bosch-ic-socket-europe:before{content:\"\\e71c\"}.bosch-ic-tripod:before{content:\"\\e71d\"}.bosch-ic-document-dxf:before{content:\"\\e71e\"}.bosch-ic-document-dwg:before{content:\"\\e71f\"}.bosch-ic-hook-wiperblade:before{content:\"\\e720\"}.bosch-ic-traffic-cone:before{content:\"\\e721\"}.bosch-ic-folder-locked:before{content:\"\\e722\"}.bosch-ic-package-self-service:before{content:\"\\e723\"}.bosch-ic-drop-ice:before{content:\"\\e724\"}.bosch-ic-lamp-gas-discharge:before{content:\"\\e725\"}.bosch-ic-home-basement:before{content:\"\\e726\"}.bosch-ic-link-edit:before{content:\"\\e727\"}.bosch-ic-table-header-add:before{content:\"\\e728\"}.bosch-ic-table-column-add-after:before{content:\"\\e729\"}.bosch-ic-table-row-add-above:before{content:\"\\e72a\"}.bosch-ic-table-column-add-before:before{content:\"\\e72b\"}.bosch-ic-table-row-add-below:before{content:\"\\e72c\"}.bosch-ic-table-cells-split:before{content:\"\\e72d\"}.bosch-ic-table-cells-merge:before{content:\"\\e72e\"}.bosch-ic-table-row-delete:before{content:\"\\e72f\"}.bosch-ic-text-indent-arrow:before{content:\"\\e730\"}.bosch-ic-text-outdent-arrow:before{content:\"\\e731\"}.bosch-ic-car-side-flash:before{content:\"\\e732\"}.bosch-ic-battery-plug-flash:before{content:\"\\e733\"}.bosch-ic-car-chip:before{content:\"\\e734\"}.bosch-ic-car-cloud-upload:before{content:\"\\e735\"}.bosch-ic-circle-parking:before{content:\"\\e736\"}.bosch-ic-engine-battery-flash:before{content:\"\\e737\"}.bosch-ic-engine-battery-flash-plug:before{content:\"\\e738\"}.bosch-ic-engine-combustion:before{content:\"\\e739\"}.bosch-ic-user-head-scan:before{content:\"\\e73a\"}.bosch-ic-bluetooth-off-disabled-bold:before{content:\"\\e73b\"}.bosch-ic-bluetooth-off-disabled-light:before{content:\"\\e73c\"}.bosch-ic-communicator-off-disabled-bold:before{content:\"\\e73d\"}.bosch-ic-communicator-off-disabled-light:before{content:\"\\e73e\"}.bosch-ic-connectivity-off-disabled-bold:before{content:\"\\e73f\"}.bosch-ic-connectivity-off-disabled-light:before{content:\"\\e740\"}.bosch-ic-globe-off-disabled-bold:before{content:\"\\e741\"}.bosch-ic-globe-off-disabled-light:before{content:\"\\e742\"}.bosch-ic-lamp-led:before{content:\"\\e743\"}.bosch-ic-wifi-off-disabled-bold:before{content:\"\\e744\"}.bosch-ic-wifi-off-disabled-light:before{content:\"\\e745\"}.bosch-ic-wheel-vehicle:before{content:\"\\e746\"}.bosch-ic-ignition-cable:before{content:\"\\e747\"}.bosch-ic-ignition-coil:before{content:\"\\e748\"}.bosch-ic-ignition-distributor:before{content:\"\\e749\"}.bosch-ic-microphone-classic-off-disabled-light:before{content:\"\\e74a\"}.bosch-ic-microphone-classic-off-disabled-bold:before{content:\"\\e74b\"}.bosch-ic-cloud-refresh:before{content:\"\\e74c\"}.bosch-ic-ignition-coil-module:before{content:\"\\e74d\"}.bosch-ic-bluetooth-frame:before{content:\"\\e8b1\"}.bosch-ic-bluetooth-frame-error:before{content:\"\\e8b2\"}.bosch-ic-ece-ccc-frame:before{content:\"\\e8b3\"}.bosch-ic-ms-teams:before{content:\"\\eaad\"}.bosch-ic-ms-teams-frame:before{content:\"\\eaae\"}.bosch-ic-mold:before{content:\"\\f0000\"}.bosch-ic-battery-car-cranking-power-high:before{content:\"\\f0001\"}.bosch-ic-brake-disk-spray-can-coated:before{content:\"\\f0002\"}.bosch-ic-car-top-arrows-left-front-right-back:before{content:\"\\f0003\"}.bosch-ic-car-top-arrows-left-back-right-front:before{content:\"\\f0004\"}.bosch-ic-car-top-arrows-front:before{content:\"\\f0005\"}.bosch-ic-car-top-arrows-back-front-left-right:before{content:\"\\f0006\"}.bosch-ic-car-top-arrows-back:before{content:\"\\f0007\"}.bosch-ic-car-top-arrow-right:before{content:\"\\f0008\"}.bosch-ic-car-top-arrow-left:before{content:\"\\f0009\"}.bosch-ic-car-top-arrow-front-right:before{content:\"\\f000a\"}.bosch-ic-car-top-arrow-front-left:before{content:\"\\f000b\"}.bosch-ic-car-top-arrow-back-right:before{content:\"\\f000c\"}.bosch-ic-car-top-arrow-back-left:before{content:\"\\f000d\"}.bosch-ic-watermelon:before{content:\"\\f000e\"}.bosch-ic-wind-vane-moderate:before{content:\"\\f000f\"}.bosch-ic-radioactive:before{content:\"\\f0010\"}.bosch-ic-heating-underfloor-off-bold:before{content:\"\\f0011\"}.bosch-ic-heating-underfloor-off-light:before{content:\"\\f0012\"}.bosch-ic-goat:before{content:\"\\f0013\"}.bosch-ic-fire-hose:before{content:\"\\f0014\"}.bosch-ic-user-run-arrow:before{content:\"\\f0015\"}.bosch-ic-store:before{content:\"\\f0016\"}.bosch-ic-user-work-dig:before{content:\"\\f0017\"}.bosch-ic-car-top-arrow-angle-right:before{content:\"\\f0018\"}.bosch-ic-control-unit-ecu:before{content:\"\\f0019\"}.bosch-ic-cigarette:before{content:\"\\f001a\"}.bosch-ic-structure-line:before{content:\"\\f001b\"}.bosch-ic-product-database-table:before{content:\"\\f001c\"}.bosch-ic-bicycle-e-remote-led:before{content:\"\\f001d\"}.bosch-ic-bicycle-e-drive-unit:before{content:\"\\f001e\"}.bosch-ic-bicycle-e-drive-unit-individual-support:before{content:\"\\f001f\"}.bosch-ic-nm-arrow-circle:before{content:\"\\f0020\"}.bosch-ic-tree-path:before{content:\"\\f0021\"}.bosch-ic-bicycle-e-wheel-arrows:before{content:\"\\f0022\"}.bosch-ic-tune-vertical:before{content:\"\\f0023\"}.bosch-ic-sound-professional:before{content:\"\\f0024\"}.bosch-ic-plane-top-headset:before{content:\"\\f0025\"}.bosch-ic-desktop-user-headset:before{content:\"\\f0026\"}.bosch-ic-user-service:before{content:\"\\f0027\"}.bosch-ic-audio-commercial:before{content:\"\\f0028\"}.bosch-ic-loudspeaker-portable-note:before{content:\"\\f0029\"}.bosch-ic-audio-loudspeaker-public:before{content:\"\\f002a\"}.bosch-ic-items-lost-questionmark:before{content:\"\\f002b\"}.bosch-ic-hand-pull:before{content:\"\\f002c\"}.bosch-ic-steering-wheel-hand-horn:before{content:\"\\f002d\"}.bosch-ic-car-chassis:before{content:\"\\f002e\"}.bosch-ic-steering-system:before{content:\"\\f002f\"}.bosch-ic-wheel-suspension:before{content:\"\\f0030\"}.bosch-ic-car-body-side:before{content:\"\\f0031\"}.bosch-ic-address-consumer-data-upright:before{content:\"\\f0032\"}.bosch-ic-weather-wind-rain:before{content:\"\\f0033\"}.bosch-ic-weather-rain-warning:before{content:\"\\f0034\"}.bosch-ic-user-check:before{content:\"\\f0035\"}.bosch-ic-user-warning:before{content:\"\\f0036\"}.bosch-ic-black-white-stars-circle:before{content:\"\\f0037\"}.bosch-ic-team-warning:before{content:\"\\f0038\"}.bosch-ic-black-white-circle:before{content:\"\\f0039\"}.bosch-ic-suitcase-small:before{content:\"\\f003a\"}.bosch-ic-suitcase-small-remove:before{content:\"\\f003b\"}.bosch-ic-home-arrow-out:before{content:\"\\f003c\"}.bosch-ic-home-arrow-in:before{content:\"\\f003d\"}.bosch-ic-document-copy-check:before{content:\"\\f003e\"}.bosch-ic-ring-infrared:before{content:\"\\f003f\"}.bosch-ic-spirit-level:before{content:\"\\f0040\"}.bosch-ic-drill-driver-check:before{content:\"\\f0041\"}.bosch-ic-desktop-led:before{content:\"\\f0042\"}.bosch-ic-desktop-black-white:before{content:\"\\f0043\"}.bosch-ic-desktop-warning:before{content:\"\\f0044\"}.bosch-ic-fire-wall:before{content:\"\\f0045\"}.bosch-ic-onroad-car-top:before{content:\"\\f0046\"}.bosch-ic-onroad-suv-top:before{content:\"\\f0047\"}.bosch-ic-onroad-truck-top:before{content:\"\\f0048\"}.bosch-ic-weather-rain-ip44:before{content:\"\\f0049\"}.bosch-ic-weather-rain-ip65:before{content:\"\\f004a\"}.bosch-ic-weather-rain-ip66:before{content:\"\\f004b\"}.bosch-ic-weather-rain-ip68:before{content:\"\\f004c\"}.bosch-ic-weather-rain-ip67:before{content:\"\\f004d\"}.bosch-ic-user-masking:before{content:\"\\f004e\"}.bosch-ic-address-consumer-data-dual:before{content:\"\\f004f\"}.bosch-ic-user-framed:before{content:\"\\f0050\"}.bosch-ic-user-counting-1:before{content:\"\\f0051\"}.bosch-ic-square-kx-uhd:before{content:\"\\f0052\"}.bosch-ic-square-qvga:before{content:\"\\f0053\"}.bosch-ic-square-vga:before{content:\"\\f0054\"}.bosch-ic-square-xmp:before{content:\"\\f0055\"}.bosch-ic-square-xxxp:before{content:\"\\f0056\"}.bosch-ic-angle-arrows-across:before{content:\"\\f0057\"}.bosch-ic-angle:before{content:\"\\f0058\"}.bosch-ic-angle-arrow-up:before{content:\"\\f0059\"}.bosch-ic-angle-view-wide:before{content:\"\\f005a\"}.bosch-ic-chat-language-selection:before{content:\"\\f005b\"}.bosch-ic-smartphone-app:before{content:\"\\f005c\"}.bosch-ic-commissioning-remote:before{content:\"\\f005d\"}.bosch-ic-user-run-detection-pir:before{content:\"\\f005e\"}.bosch-ic-field-in-motion:before{content:\"\\f005f\"}.bosch-ic-building-integration-system:before{content:\"\\f0060\"}.bosch-ic-desktop-mamangement:before{content:\"\\f0061\"}.bosch-ic-desktop-management-camera:before{content:\"\\f0062\"}.bosch-ic-screwdriver-check:before{content:\"\\f0063\"}.bosch-ic-screwdriver:before{content:\"\\f0064\"}.bosch-ic-desktop-hand-touch:before{content:\"\\f0065\"}.bosch-ic-user-robot-head:before{content:\"\\f0066\"}.bosch-ic-gear-circles-elements:before{content:\"\\f0067\"}.bosch-ic-gears-3:before{content:\"\\f0068\"}.bosch-ic-door-address-consumer-data:before{content:\"\\f0069\"}.bosch-ic-lock-closed-128bit:before{content:\"\\f006a\"}.bosch-ic-lock-closed-connected-wps:before{content:\"\\f006b\"}.bosch-ic-battery-arrow-right:before{content:\"\\f006c\"}.bosch-ic-connector-bnc:before{content:\"\\f006d\"}.bosch-ic-tower-connection-cdma:before{content:\"\\f006e\"}.bosch-ic-tower-connection-gprs:before{content:\"\\f006f\"}.bosch-ic-tritech:before{content:\"\\f0070\"}.bosch-ic-quad-sensor-two:before{content:\"\\f0071\"}.bosch-ic-dynamic-range-hdr:before{content:\"\\f0072\"}.bosch-ic-dynamic-range-wdr:before{content:\"\\f0073\"}.bosch-ic-battery-thumb-up-arrow-right:before{content:\"\\f0074\"}.bosch-ic-screwdriver-thumb-up:before{content:\"\\f0075\"}.bosch-ic-thumb-up-arrow-up:before{content:\"\\f0076\"}.bosch-ic-core-data-sql:before{content:\"\\f0077\"}.bosch-ic-frequency-869:before{content:\"\\f0078\"}.bosch-ic-coins:before{content:\"\\f0079\"}.bosch-ic-frames-60:before{content:\"\\f007a\"}.bosch-ic-hand-card:before{content:\"\\f007b\"}.bosch-ic-microphone-plugabble:before{content:\"\\f007c\"}.bosch-ic-video-h264:before{content:\"\\f007d\"}.bosch-ic-video-hd:before{content:\"\\f007e\"}.bosch-ic-video-mpeg:before{content:\"\\f007f\"}.bosch-ic-video-hd-4k:before{content:\"\\f0080\"}.bosch-ic-video-hd-720:before{content:\"\\f0081\"}.bosch-ic-lasso:before{content:\"\\f0082\"}.bosch-ic-lasso-remove:before{content:\"\\f0083\"}.bosch-ic-battery-car-3-clock:before{content:\"\\f0084\"}.bosch-ic-battery-car-agm:before{content:\"\\f0085\"}.bosch-ic-battery-car-flash:before{content:\"\\f0086\"}.bosch-ic-filter-binary:before{content:\"\\f0087\"}.bosch-ic-speedometer-bitrate-reduce:before{content:\"\\f0088\"}.bosch-ic-voting:before{content:\"\\f0089\"}.bosch-ic-voting-dual:before{content:\"\\f008a\"}.bosch-ic-sound-pressure-level-90:before{content:\"\\f008b\"}.bosch-ic-sound-pressure-level-88:before{content:\"\\f008c\"}.bosch-ic-sound-pressure-level-80:before{content:\"\\f008d\"}.bosch-ic-power-rated-20:before{content:\"\\f008e\"}.bosch-ic-labyrinth-geometry-integration:before{content:\"\\f008f\"}.bosch-ic-head-speaking-voice-2:before{content:\"\\f0090\"}.bosch-ic-volume-microphone:before{content:\"\\f0091\"}.bosch-ic-user-walk-route:before{content:\"\\f0092\"}.bosch-ic-user-walk-line-crossing:before{content:\"\\f0093\"}.bosch-ic-user-walk-line-count:before{content:\"\\f0094\"}.bosch-ic-fullscreen-arrows-circle:before{content:\"\\f0095\"}.bosch-ic-field-arrows-counter-flow:before{content:\"\\f0096\"}.bosch-ic-field-arrows-directed-flow:before{content:\"\\f0097\"}.bosch-ic-user-birds-eye-count-12:before{content:\"\\f0098\"}.bosch-ic-field-user-count-12:before{content:\"\\f0099\"}.bosch-ic-field-user-falling:before{content:\"\\f009a\"}.bosch-ic-field-user-loitering-arrow:before{content:\"\\f009b\"}.bosch-ic-field-user-similarity:before{content:\"\\f009c\"}.bosch-ic-field-user-walk:before{content:\"\\f009d\"}.bosch-ic-field-user-walk-arrow-behind:before{content:\"\\f009e\"}.bosch-ic-field-user-walk-arrow-in-front:before{content:\"\\f009f\"}.bosch-ic-field-users:before{content:\"\\f00a0\"}.bosch-ic-shock-resistant:before{content:\"\\f00a1\"}.bosch-ic-bus-sdi:before{content:\"\\f00a2\"}.bosch-ic-bus-sdi2:before{content:\"\\f00a3\"}.bosch-ic-folder-ip-net:before{content:\"\\f00a4\"}.bosch-ic-globe-iscsi:before{content:\"\\f00a5\"}.bosch-ic-offline-www:before{content:\"\\f00a6\"}.bosch-ic-opc:before{content:\"\\f00a7\"}.bosch-ic-osdp:before{content:\"\\f00a8\"}.bosch-ic-settings-connected-ptrz:before{content:\"\\f00a9\"}.bosch-ic-raid:before{content:\"\\f00aa\"}.bosch-ic-lsn:before{content:\"\\f00ab\"}.bosch-ic-pointbus:before{content:\"\\f00ac\"}.bosch-ic-male-count-12:before{content:\"\\f00ad\"}.bosch-ic-circle-point-record:before{content:\"\\f00ae\"}.bosch-ic-plug-poe:before{content:\"\\f00af\"}.bosch-ic-ir-distribution:before{content:\"\\f00b0\"}.bosch-ic-ir-range:before{content:\"\\f00b1\"}.bosch-ic-connectivity-it:before{content:\"\\f00b2\"}.bosch-ic-music-note-plus:before{content:\"\\f00b3\"}.bosch-ic-music-note-plus-two:before{content:\"\\f00b4\"}.bosch-ic-chat-language-chn:before{content:\"\\f00b5\"}.bosch-ic-microphone-high-directive:before{content:\"\\f00b6\"}.bosch-ic-document-copy-rg-gf:before{content:\"\\f00b7\"}.bosch-ic-document-copy-cdq:before{content:\"\\f00b8\"}.bosch-ic-document-copy-cgp:before{content:\"\\f00b9\"}.bosch-ic-folder-add:before{content:\"\\f00ba\"}.bosch-ic-user-tie:before{content:\"\\f00bb\"}.bosch-ic-pill:before{content:\"\\f00bc\"}.bosch-ic-sonotrode:before{content:\"\\f00bd\"}.bosch-ic-user-thumb-up:before{content:\"\\f00be\"}.bosch-ic-architecture:before{content:\"\\f00bf\"}.bosch-ic-clamping-area-arrows:before{content:\"\\f00c0\"}.bosch-ic-clamping-rib:before{content:\"\\f00c1\"}.bosch-ic-connection-plug-in:before{content:\"\\f00c2\"}.bosch-ic-connection-plug-in-electrical:before{content:\"\\f00c3\"}.bosch-ic-connection-plug-in-socket:before{content:\"\\f00c4\"}.bosch-ic-connector-shield-emc:before{content:\"\\f00c5\"}.bosch-ic-foil-flex:before{content:\"\\f00c6\"}.bosch-ic-grid-fan:before{content:\"\\f00c7\"}.bosch-ic-screw-nut:before{content:\"\\f00c8\"}.bosch-ic-sealing-groove:before{content:\"\\f00c9\"}.bosch-ic-sealing-tongue-groove-flash:before{content:\"\\f00ca\"}.bosch-ic-solder:before{content:\"\\f00cb\"}.bosch-ic-solder-selective-ssl:before{content:\"\\f00cc\"}.bosch-ic-solder-selective-thrs:before{content:\"\\f00cd\"}.bosch-ic-tuner-box:before{content:\"\\f00ce\"}.bosch-ic-water-cooling-arrow-up:before{content:\"\\f00cf\"}.bosch-ic-generic-device-connected:before{content:\"\\f00d0\"}.bosch-ic-home-sun-flash:before{content:\"\\f00d1\"}.bosch-ic-home-sun-wrench:before{content:\"\\f00d2\"}.bosch-ic-temperature-lock:before{content:\"\\f00d3\"}.bosch-ic-solder-method-smd:before{content:\"\\f00d4\"}.bosch-ic-pins:before{content:\"\\f00d5\"}.bosch-ic-connector-housing:before{content:\"\\f00d6\"}.bosch-ic-antenna-cover-arrow-up:before{content:\"\\f00d7\"}.bosch-ic-angle-open-horizontal-130:before{content:\"\\f00d8\"}.bosch-ic-angle-open-vertical-130:before{content:\"\\f00d9\"}.bosch-ic-angle-open-vertical-90:before{content:\"\\f00da\"}.bosch-ic-angle-open-horizontal-145:before{content:\"\\f00db\"}.bosch-ic-steering-system-steer-by-wire:before{content:\"\\f00dc\"}.bosch-ic-tune-vertical-music-note:before{content:\"\\f00dd\"}.bosch-ic-hammer-movement:before{content:\"\\f00de\"}.bosch-ic-drill-drive-percent-35:before{content:\"\\f00df\"}.bosch-ic-country-switzerland:before{content:\"\\f00e0\"}.bosch-ic-speedlimit-revoking:before{content:\"\\f00e1\"}.bosch-ic-speedlimit-80-revoking:before{content:\"\\f00e2\"}.bosch-ic-speedlimit-80-dirty:before{content:\"\\f00e3\"}.bosch-ic-battery-car-mf:before{content:\"\\f00e4\"}.bosch-ic-access-controller-2:before{content:\"\\f00e5\"}.bosch-ic-access-controller-arrow-up:before{content:\"\\f00e6\"}.bosch-ic-access-controller-points-set:before{content:\"\\f00e7\"}.bosch-ic-analog-value:before{content:\"\\f00e8\"}.bosch-ic-analog-value-arrow-in:before{content:\"\\f00e9\"}.bosch-ic-analog-value-arrow-out:before{content:\"\\f00ea\"}.bosch-ic-binary-value:before{content:\"\\f00eb\"}.bosch-ic-binary-value-arrow-in:before{content:\"\\f00ec\"}.bosch-ic-binary-value-arrow-out:before{content:\"\\f00ed\"}.bosch-ic-calendar-date-single:before{content:\"\\f00ee\"}.bosch-ic-calendar-date-weekend:before{content:\"\\f00ef\"}.bosch-ic-calendar-view:before{content:\"\\f00f0\"}.bosch-ic-chart-average:before{content:\"\\f00f1\"}.bosch-ic-chart-line-arrows-up:before{content:\"\\f00f2\"}.bosch-ic-configuration-points-set:before{content:\"\\f00f3\"}.bosch-ic-data-points-set:before{content:\"\\f00f4\"}.bosch-ic-document-points-set:before{content:\"\\f00f5\"}.bosch-ic-multistate-value:before{content:\"\\f00f6\"}.bosch-ic-multistate-value-arrow-in:before{content:\"\\f00f7\"}.bosch-ic-multistate-value-arrow-out:before{content:\"\\f00f8\"}.bosch-ic-structure-bacnet:before{content:\"\\f00f9\"}.bosch-ic-structure-sbus:before{content:\"\\f00fa\"}.bosch-ic-team-arrows-in:before{content:\"\\f00fb\"}.bosch-ic-import-points-set:before{content:\"\\f00fc\"}.bosch-ic-image-points-set:before{content:\"\\f00fd\"}.bosch-ic-circuit-hydraulic:before{content:\"\\f00fe\"}.bosch-ic-structure-area:before{content:\"\\f00ff\"}.bosch-ic-structure-calculate:before{content:\"\\f0100\"}.bosch-ic-structure-clipboard:before{content:\"\\f0101\"}.bosch-ic-structure-collection:before{content:\"\\f0102\"}.bosch-ic-structure-components:before{content:\"\\f0103\"}.bosch-ic-structure-device:before{content:\"\\f0104\"}.bosch-ic-structure-dragdrop:before{content:\"\\f0105\"}.bosch-ic-structure-floor:before{content:\"\\f0106\"}.bosch-ic-structure-organization:before{content:\"\\f0107\"}.bosch-ic-structure-questionmark:before{content:\"\\f0108\"}.bosch-ic-structure-settings:before{content:\"\\f0109\"}.bosch-ic-structure-skyscraper:before{content:\"\\f010a\"}.bosch-ic-structure-window-console:before{content:\"\\f010b\"}.bosch-ic-structure-wrench:before{content:\"\\f010c\"}.bosch-ic-car-pickup:before{content:\"\\f010d\"}.bosch-ic-desktop-team:before{content:\"\\f010e\"}.bosch-ic-keyboard-arrow-enter-return:before{content:\"\\f010f\"}.bosch-ic-keyboard-arrow-left:before{content:\"\\f0110\"}.bosch-ic-keyboard-arrow-right-tab:before{content:\"\\f0111\"}.bosch-ic-keyboard-capslock:before{content:\"\\f0112\"}.bosch-ic-keyboard-shift:before{content:\"\\f0113\"}.bosch-ic-keyboard-space:before{content:\"\\f0114\"}.bosch-ic-bluetooth-le-off-disabled-light:before{content:\"\\f0115\"}.bosch-ic-structure-label-edit:before{content:\"\\f0116\"}.bosch-ic-structure-server:before{content:\"\\f0117\"}.bosch-ic-truck-box:before{content:\"\\f0118\"}.bosch-ic-imagery-off-disabled-light:before{content:\"\\f0119\"}.bosch-ic-imagery-off-disabled-bold:before{content:\"\\f011a\"}.bosch-ic-plan-floorplan-edit:before{content:\"\\f011b\"}.bosch-ic-desktop-building-ruler-pen:before{content:\"\\f011c\"}.bosch-ic-conference-system-connected:before{content:\"\\f011d\"}.bosch-ic-receiver-pocket-headphones:before{content:\"\\f011e\"}.bosch-ic-conference-system-microphone:before{content:\"\\f011f\"}.bosch-ic-user-carry-box:before{content:\"\\f0120\"}.bosch-ic-user-handcart:before{content:\"\\f0121\"}.bosch-ic-user-carry-box-2:before{content:\"\\f0122\"}.bosch-ic-locator-error:before{content:\"\\f0123\"}.bosch-ic-locator-offline:before{content:\"\\f0124\"}.bosch-ic-locator-refresh:before{content:\"\\f0125\"}.bosch-ic-chart-bar-plug-arrow-up:before{content:\"\\f0126\"}.bosch-ic-clipboard-search-leaf:before{content:\"\\f0127\"}.bosch-ic-handover-leaf:before{content:\"\\f0128\"}.bosch-ic-home-sun-plug:before{content:\"\\f0129\"}.bosch-ic-lightbulb-plug-leaf:before{content:\"\\f012a\"}.bosch-ic-wheel-water-plug:before{content:\"\\f012b\"}.bosch-ic-windmill-plug:before{content:\"\\f012c\"}.bosch-ic-windmill-plug-sun-water:before{content:\"\\f012d\"}.bosch-ic-scale-co2:before{content:\"\\f012e\"}.bosch-ic-certificat-co2:before{content:\"\\f012f\"}.bosch-ic-car-crash:before{content:\"\\f0130\"}.bosch-ic-headphones:before{content:\"\\f0131\"}.bosch-ic-music-note:before{content:\"\\f0132\"}.bosch-ic-clef-treble:before{content:\"\\f0133\"}.bosch-ic-clef-bass:before{content:\"\\f0134\"}.bosch-ic-volume:before{content:\"\\f0135\"}.bosch-ic-volume-increase:before{content:\"\\f0136\"}.bosch-ic-head-speach-speaking:before{content:\"\\f0137\"}.bosch-ic-head-speaking-request:before{content:\"\\f0138\"}.bosch-ic-tetragon-priority:before{content:\"\\f0139\"}.bosch-ic-head-respond-speaking:before{content:\"\\f013a\"}.bosch-ic-head-respond-request:before{content:\"\\f013b\"}.bosch-ic-hand-stop:before{content:\"\\f013c\"}.bosch-ic-layers-stack:before{content:\"\\f013d\"}.bosch-ic-puzzle-binary:before{content:\"\\f013e\"}.bosch-ic-sdcard-micro:before{content:\"\\f013f\"}.bosch-ic-reader-card:before{content:\"\\f0140\"}.bosch-ic-reader-user-recognition-face:before{content:\"\\f0141\"}.bosch-ic-detector-point-type:before{content:\"\\f0142\"}.bosch-ic-manual-call-point:before{content:\"\\f0143\"}.bosch-ic-gun-magnifier:before{content:\"\\f0144\"}.bosch-ic-interface-module:before{content:\"\\f0145\"}.bosch-ic-fire-detector-automatic:before{content:\"\\f0146\"}.bosch-ic-fire-brigade-emergency:before{content:\"\\f0147\"}.bosch-ic-volume-siren-device:before{content:\"\\f0148\"}.bosch-ic-user-reception:before{content:\"\\f0149\"}.bosch-ic-shredder-machine:before{content:\"\\f014a\"}.bosch-ic-badge-clean:before{content:\"\\f014b\"}.bosch-ic-parking-assistance:before{content:\"\\f014c\"}.bosch-ic-laptop-questionmark:before{content:\"\\f014d\"}.bosch-ic-security-lock:before{content:\"\\f014e\"}.bosch-ic-battery-car-12v:before{content:\"\\f014f\"}.bosch-ic-car-far-propagation:before{content:\"\\f0150\"}.bosch-ic-car-wide-propagation:before{content:\"\\f0151\"}.bosch-ic-ear-sound:before{content:\"\\f0152\"}.bosch-ic-umbrella-drops:before{content:\"\\f0153\"}.bosch-ic-horn-arrows-in:before{content:\"\\f0154\"}.bosch-ic-video-bell:before{content:\"\\f0155\"}.bosch-ic-steering-wheel-arrows-up:before{content:\"\\f0156\"}.bosch-ic-circle-full:before{content:\"\\f0157\"}.bosch-ic-circle-half:before{content:\"\\f0158\"}.bosch-ic-circle-quarter:before{content:\"\\f0159\"}.bosch-ic-circle-quarter-three:before{content:\"\\f015a\"}.bosch-ic-appliance-flame-arrow:before{content:\"\\f015b\"}.bosch-ic-appliance-flame-arrow-bold:before{content:\"\\f015c\"}.bosch-ic-appliance-flame-disabled-bold:before{content:\"\\f015d\"}.bosch-ic-appliance-flame-disabled-light:before{content:\"\\f015e\"}.bosch-ic-appliance-flame-plus:before{content:\"\\f015f\"}.bosch-ic-appliance-flame-plus-bold:before{content:\"\\f0160\"}.bosch-ic-flash-minus-bold:before{content:\"\\f0161\"}.bosch-ic-flash-minus:before{content:\"\\f0162\"}.bosch-ic-flash-off:before{content:\"\\f0163\"}.bosch-ic-flash-plus-bold:before{content:\"\\f0164\"}.bosch-ic-flash-smart-grid:before{content:\"\\f0165\"}.bosch-ic-signal-high:before{content:\"\\f0166\"}.bosch-ic-signal-low:before{content:\"\\f0167\"}.bosch-ic-signal-very-low:before{content:\"\\f0168\"}.bosch-ic-5g:before{content:\"\\f0169\"}.bosch-ic-sun-photovoltaic:before{content:\"\\f016a\"}.bosch-ic-inverter:before{content:\"\\f016b\"}.bosch-ic-car-arrow-down:before{content:\"\\f016c\"}.bosch-ic-car-distance:before{content:\"\\f016d\"}.bosch-ic-car-sensor:before{content:\"\\f016e\"}.bosch-ic-car-sensor-environment:before{content:\"\\f016f\"}.bosch-ic-car-sensor-front:before{content:\"\\f0170\"}.bosch-ic-streets-locator:before{content:\"\\f0171\"}.bosch-ic-truck-distance:before{content:\"\\f0172\"}.bosch-ic-warning-user-walk:before{content:\"\\f0173\"}.bosch-ic-parking-check:before{content:\"\\f0174\"}.bosch-ic-dremel:before{content:\"\\f0175\"}.bosch-ic-allergens:before{content:\"\\f0176\"}.bosch-ic-money-currency-coins:before{content:\"\\f0177\"}.bosch-ic-hydrogen-small:before{content:\"\\f0178\"}.bosch-ic-hydrogen-large:before{content:\"\\f0179\"}.bosch-ic-shoppingcart-remove:before{content:\"\\f017a\"}.bosch-ic-chimney-sweeper-disabled-light:before{content:\"\\f017b\"}.bosch-ic-chimney-sweeper-disabled-bold:before{content:\"\\f017c\"}.bosch-ic-car-waves:before{content:\"\\f017d\"}.bosch-ic-360-degrees-arrow:before{content:\"\\f017e\"}.bosch-ic-clock-info:before{content:\"\\f017f\"}.bosch-ic-truck-front-disabled-bold:before{content:\"\\f0180\"}.bosch-ic-truck-front-disabled-light:before{content:\"\\f0181\"}.bosch-ic-box-closed-list:before{content:\"\\f0182\"}.bosch-ic-battery-car-exchange:before{content:\"\\f0183\"}.bosch-ic-guitar:before{content:\"\\f0184\"}.bosch-ic-actuator-by-wire:before{content:\"\\f0185\"}.bosch-ic-guitar-electric:before{content:\"\\f0186\"}.bosch-ic-format-align-center:before{content:\"\\f0187\"}.bosch-ic-format-align-justify:before{content:\"\\f0188\"}.bosch-ic-format-align-right:before{content:\"\\f0189\"}.bosch-ic-format-checklist:before{content:\"\\f018a\"}.bosch-ic-format-italic:before{content:\"\\f018b\"}.bosch-ic-format-ordered-list:before{content:\"\\f018c\"}.bosch-ic-format-strike:before{content:\"\\f018d\"}.bosch-ic-format-underline:before{content:\"\\f018e\"}.bosch-ic-format-align-left:before{content:\"\\f018f\"}.bosch-ic-format-fill-drop:before{content:\"\\f0190\"}.bosch-ic-format-color-a:before{content:\"\\f0191\"}.bosch-ic-format-bold-b:before{content:\"\\f0192\"}.bosch-ic-fly:before{content:\"\\f0193\"}.bosch-ic-paw:before{content:\"\\f0194\"}.bosch-ic-desktop-video-response-fast:before{content:\"\\f0195\"}.bosch-ic-user-walk-weather-rain:before{content:\"\\f0196\"}.bosch-ic-brake-pedal-plug:before{content:\"\\f0197\"}.bosch-ic-detector-motion-range-down-view:before{content:\"\\f0198\"}.bosch-ic-detector-motion-range-long:before{content:\"\\f0199\"}.bosch-ic-monitor-tiled:before{content:\"\\f019a\"}.bosch-ic-radar-doppler:before{content:\"\\f019b\"}.bosch-ic-radioactive-disabled-light:before{content:\"\\f019c\"}.bosch-ic-radioactive-disabled-bold:before{content:\"\\f019d\"}.bosch-ic-bug-disabled-light:before{content:\"\\f019e\"}.bosch-ic-bug-disabled-bold:before{content:\"\\f019f\"}.bosch-ic-fly-disabled-light:before{content:\"\\f01a0\"}.bosch-ic-fly-disabled-bold:before{content:\"\\f01a1\"}.bosch-ic-puzzle-2:before{content:\"\\f01a2\"}.bosch-ic-camera-lenses-multi-images:before{content:\"\\f01a3\"}.bosch-ic-detector-spray:before{content:\"\\f01a4\"}.bosch-ic-weather-rain-ip56:before{content:\"\\f01a5\"}.bosch-ic-document-log-start:before{content:\"\\f01a6\"}.bosch-ic-document-log-stop:before{content:\"\\f01a7\"}.bosch-ic-document-3d-pdf:before{content:\"\\f01a8\"}.bosch-ic-document-ifc:before{content:\"\\f01a9\"}.bosch-ic-document-rfa:before{content:\"\\f01aa\"}.bosch-ic-document-stp:before{content:\"\\f01ab\"}.bosch-ic-recycle-arrows-pap20-frame:before{content:\"\\e8b4\"}.bosch-ic-umbrella-drops-frame:before{content:\"\\e8b5\"}.bosch-ic-arrows-this-side-up-frame:before{content:\"\\e8b6\"}.bosch-ic-glass-wine-frame:before{content:\"\\e8b7\"}.bosch-ic-pb-frame:before{content:\"\\e8b8\"}.bosch-ic-pb-frame-disabled-bold:before{content:\"\\e8b9\"}.bosch-ic-pb-frame-disabled-light:before{content:\"\\e8ba\"}.bosch-ic-mode-frame:before{content:\"\\e8bb\"}.bosch-ic-uv-frame-disabled-bold:before{content:\"\\e8bc\"}.bosch-ic-uv-frame-disabled-light:before{content:\"\\e8bd\"}.bosch-ic-fan-frame:before{content:\"\\e8be\"}.bosch-ic-flash-frame:before{content:\"\\e8bf\"}.bosch-ic-kakao:before{content:\"\\eaaf\"}.bosch-ic-kakao-frame:before{content:\"\\eab0\"}.bosch-ic-naver:before{content:\"\\eab1\"}.bosch-ic-naver-frame:before{content:\"\\eab2\"}.bosch-ic-tencent:before{content:\"\\eab3\"}.bosch-ic-tencent-frame:before{content:\"\\eab4\"}.bosch-ic-wordpress:before{content:\"\\eab5\"}.bosch-ic-wordpress-frame:before{content:\"\\eab6\"}.bosch-ic-mini-program-frame:before{content:\"\\eab7\"}.bosch-ic-mini-program:before{content:\"\\eab8\"}.bosch-ic-tiktok:before{content:\"\\eab9\"}.bosch-ic-tiktok-frame:before{content:\"\\eaba\"}.bosch-ic-glassdoor:before{content:\"\\eabb\"}.bosch-ic-glassdoor-frame:before{content:\"\\eabc\"}.bosch-ic-git-hub:before{content:\"\\eabd\"}.bosch-ic-git-hub-frame:before{content:\"\\eabe\"}.bosch-ic-online-www:before{content:\"\\f01ac\"}.bosch-ic-bluetooth-connected:before{content:\"\\f01ad\"}.bosch-ic-connected-tools:before{content:\"\\f01ae\"}.bosch-ic-badge-arrow-kickback:before{content:\"\\f01af\"}.bosch-ic-copy-paste:before{content:\"\\f01b0\"}.bosch-ic-volleyball:before{content:\"\\f01b1\"}.bosch-ic-car-top-trailer:before{content:\"\\f01b2\"}.bosch-ic-mountain:before{content:\"\\f01b3\"}.bosch-ic-battery-car-oil-lamp:before{content:\"\\f01b4\"}.bosch-ic-battery-car-efb:before{content:\"\\f01b5\"}.bosch-ic-battery-car-ams:before{content:\"\\f01b6\"}.bosch-ic-battery-car-ag:before{content:\"\\f01b7\"}.bosch-ic-display-fuel-saving:before{content:\"\\f01b8\"}.bosch-ic-circle-parking-arrows:before{content:\"\\f01b9\"}.bosch-ic-unicorn-head:before{content:\"\\f01ba\"}.bosch-ic-car-parking-automated:before{content:\"\\f01bb\"}.bosch-ic-lunchboard:before{content:\"\\f01bc\"}.bosch-ic-tetrapack:before{content:\"\\f01bd\"}.bosch-ic-recycle-arrows:before{content:\"\\f01be\"}.bosch-ic-stairs-arrow-up:before{content:\"\\f01bf\"}.bosch-ic-stairs-arrow-down:before{content:\"\\f01c0\"}.bosch-ic-industry-car-arrow:before{content:\"\\f01c1\"}.bosch-ic-sensor-throttle-position:before{content:\"\\f01c2\"}.bosch-ic-sensor-knock:before{content:\"\\f01c3\"}.bosch-ic-sensor-air-mass-meter:before{content:\"\\f01c4\"}.bosch-ic-sensor-pressure:before{content:\"\\f01c5\"}.bosch-ic-sensor-temperature:before{content:\"\\f01c6\"}.bosch-ic-sensor-speed-rpm:before{content:\"\\f01c7\"}.bosch-ic-desktop-user-software-clock:before{content:\"\\f01c8\"}.bosch-ic-book-service-wrench-add:before{content:\"\\f01c9\"}.bosch-ic-book-service-wrench:before{content:\"\\f01ca\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-light:before{content:\"\\f01cb\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-bold:before{content:\"\\f01cc\"}.bosch-ic-circle-parking-disabled-bold:before{content:\"\\f01cd\"}.bosch-ic-circle-parking-disabled-light:before{content:\"\\f01ce\"}.bosch-ic-interval-30sec:before{content:\"\\f01cf\"}.bosch-ic-truck-front-light-inside:before{content:\"\\f01d0\"}.bosch-ic-software-tex:before{content:\"\\f01d1\"}.bosch-ic-fire-arrow-circle:before{content:\"\\f01d2\"}.bosch-ic-heating-underfloor-circle-global-off:before{content:\"\\f01d3\"}.bosch-ic-heating-radiator-circle-global-off:before{content:\"\\f01d4\"}.bosch-ic-faucet-circle-global-off:before{content:\"\\f01d5\"}.bosch-ic-moon-stars-starlight:before{content:\"\\f01d6\"}.bosch-ic-tube-thumb-up:before{content:\"\\f01d7\"}.bosch-ic-detector-microwave:before{content:\"\\f01d8\"}.bosch-ic-user-headset-thumb-up:before{content:\"\\f01d9\"}.bosch-ic-cloud-battery:before{content:\"\\f01da\"}.bosch-ic-temperature-arrow-up-increase:before{content:\"\\f01db\"}.bosch-ic-temperature-arrow-down-decrease:before{content:\"\\f01dc\"}.bosch-ic-moneybag-dollar-chart-double:before{content:\"\\f01dd\"}.bosch-ic-locator-remove:before{content:\"\\f01de\"}.bosch-ic-square-remove:before{content:\"\\f01df\"}.bosch-ic-sun-hdr:before{content:\"\\f01e0\"}.bosch-ic-moon-stars-circle:before{content:\"\\f01e1\"}.bosch-ic-temperature-user-detection:before{content:\"\\f01e2\"}.bosch-ic-wrench-thumb-up:before{content:\"\\f01e3\"}.bosch-ic-ring-infrared-ir:before{content:\"\\f01e4\"}.bosch-ic-play-back-arrow:before{content:\"\\f01e5\"}.bosch-ic-ignition-off:before{content:\"\\f01e6\"}.bosch-ic-arrow-up-down:before{content:\"\\f01e7\"}.bosch-ic-arrow-left-right:before{content:\"\\f01e8\"}.bosch-ic-number-0-bold:before{content:\"\\f01e9\"}.bosch-ic-number-1-bold:before{content:\"\\f01ea\"}.bosch-ic-number-2-bold:before{content:\"\\f01eb\"}.bosch-ic-number-3-bold:before{content:\"\\f01ec\"}.bosch-ic-number-4-bold:before{content:\"\\f01ed\"}.bosch-ic-number-5-bold:before{content:\"\\f01ee\"}.bosch-ic-number-6-bold:before{content:\"\\f01ef\"}.bosch-ic-number-7-bold:before{content:\"\\f01f0\"}.bosch-ic-number-8-bold:before{content:\"\\f01f1\"}.bosch-ic-number-9-bold:before{content:\"\\f01f2\"}.bosch-ic-e-charging-plug-dc:before{content:\"\\f01f3\"}.bosch-ic-e-charging-plug-ac-dc:before{content:\"\\f01f4\"}.bosch-ic-angle-view-wide-monitor:before{content:\"\\f01f5\"}.bosch-ic-settings-range-ptrz:before{content:\"\\f01f6\"}.bosch-ic-puzzle-video-integration:before{content:\"\\f01f7\"}.bosch-ic-number-0-regular:before{content:\"\\f01f8\"}.bosch-ic-number-1-regular:before{content:\"\\f01f9\"}.bosch-ic-number-2-regular:before{content:\"\\f01fa\"}.bosch-ic-number-3-regular:before{content:\"\\f01fb\"}.bosch-ic-number-4-regular:before{content:\"\\f01fc\"}.bosch-ic-number-5-regular:before{content:\"\\f01fd\"}.bosch-ic-number-6-regular:before{content:\"\\f01fe\"}.bosch-ic-number-7-regular:before{content:\"\\f01ff\"}.bosch-ic-number-8-regular:before{content:\"\\f0200\"}.bosch-ic-number-9-regular:before{content:\"\\f0201\"}.bosch-ic-import-beep:before{content:\"\\f0202\"}.bosch-ic-file-questionmark:before{content:\"\\f0203\"}.bosch-ic-file-dollar:before{content:\"\\f0204\"}.bosch-ic-document-reporting:before{content:\"\\f0205\"}.bosch-ic-box-questionmark1:before{content:\"\\f0206\"}.bosch-ic-chat-faq:before{content:\"\\f0207\"}.bosch-ic-wishlist-filled:before{content:\"\\f0208\"}.bosch-ic-wishlist-half-filled:before{content:\"\\f0209\"}.bosch-ic-glove-work:before{content:\"\\f020a\"}.bosch-ic-lightbulb-halogen-hand-disabled-bold:before{content:\"\\f020b\"}.bosch-ic-lightbulb-halogen-hand-disabled-light:before{content:\"\\f020c\"}.bosch-ic-lightbulb-halogen-cracked-disabled-bold:before{content:\"\\f020d\"}.bosch-ic-lightbulb-halogen-cracked-disabled-light:before{content:\"\\f020e\"}.bosch-ic-lightbulb-halogen-cracked:before{content:\"\\f020f\"}.bosch-ic-drops:before{content:\"\\f0210\"}.bosch-ic-drops-disabled-bold:before{content:\"\\f0211\"}.bosch-ic-drops-disabled-light:before{content:\"\\f0212\"}.bosch-ic-temperature-heat:before{content:\"\\f0213\"}.bosch-ic-allen-key:before{content:\"\\f0214\"}.bosch-ic-aperture-closed:before{content:\"\\f0215\"}.bosch-ic-appliance-heat-pump-leaf-connected:before{content:\"\\f0216\"}.bosch-ic-badge-flash:before{content:\"\\f0217\"}.bosch-ic-battery-car-1-plug-warning:before{content:\"\\f0218\"}.bosch-ic-battery-car-3-automatic-mode:before{content:\"\\f0219\"}.bosch-ic-battery-car-3-ice:before{content:\"\\f021a\"}.bosch-ic-battery-car-memory-function:before{content:\"\\f021b\"}.bosch-ic-battery-polarity-reverse-warning:before{content:\"\\f021c\"}.bosch-ic-book-questionmark:before{content:\"\\f021d\"}.bosch-ic-book-service-wrench-refresh:before{content:\"\\f021e\"}.bosch-ic-box-arrow-up:before{content:\"\\f021f\"}.bosch-ic-box-arrows-up-down:before{content:\"\\f0220\"}.bosch-ic-camera-optic-sensor:before{content:\"\\f0221\"}.bosch-ic-car-cloud-software:before{content:\"\\f0222\"}.bosch-ic-car-gear-technology:before{content:\"\\f0223\"}.bosch-ic-car-street:before{content:\"\\f0224\"}.bosch-ic-chip-sensor:before{content:\"\\f0225\"}.bosch-ic-circle-blank:before{content:\"\\f0226\"}.bosch-ic-detector-point-type-replace:before{content:\"\\f0227\"}.bosch-ic-document-x:before{content:\"\\f0228\"}.bosch-ic-drag-handle-arrows-up-down:before{content:\"\\f0229\"}.bosch-ic-electrolyzer:before{content:\"\\f022a\"}.bosch-ic-exhaust-pipe:before{content:\"\\f022b\"}.bosch-ic-flower-focus-near:before{content:\"\\f022c\"}.bosch-ic-fuel-pump-electric:before{content:\"\\f022d\"}.bosch-ic-gear-dollar:before{content:\"\\f022e\"}.bosch-ic-gloves-work:before{content:\"\\f022f\"}.bosch-ic-handlewithcare-globe:before{content:\"\\f0230\"}.bosch-ic-handlewithcare-user:before{content:\"\\f0231\"}.bosch-ic-hard-disk:before{content:\"\\f0232\"}.bosch-ic-hierarchy-child:before{content:\"\\f0233\"}.bosch-ic-hierarchy-parent:before{content:\"\\f0234\"}.bosch-ic-hydraulic-connection-leaking-drops:before{content:\"\\f0235\"}.bosch-ic-hydraulic-connection-off:before{content:\"\\f0236\"}.bosch-ic-hydraulic-connection-on:before{content:\"\\f0237\"}.bosch-ic-light-beam:before{content:\"\\f0238\"}.bosch-ic-map5000:before{content:\"\\f0239\"}.bosch-ic-module-anti-blocking-system:before{content:\"\\f023a\"}.bosch-ic-module-ecg:before{content:\"\\f023b\"}.bosch-ic-module-ecg-suction:before{content:\"\\f023c\"}.bosch-ic-module-navigation:before{content:\"\\f023d\"}.bosch-ic-power-supply-unit-plug:before{content:\"\\f023e\"}.bosch-ic-security-street:before{content:\"\\f023f\"}.bosch-ic-server-single:before{content:\"\\f0240\"}.bosch-ic-skip-backward:before{content:\"\\f0241\"}.bosch-ic-skip-forward:before{content:\"\\f0242\"}.bosch-ic-speedometer-instrument-cluster:before{content:\"\\f0243\"}.bosch-ic-system-radar-connected:before{content:\"\\f0244\"}.bosch-ic-truck-dump:before{content:\"\\f0245\"}.bosch-ic-truck-loader:before{content:\"\\f0246\"}.bosch-ic-user-head-twin-digital:before{content:\"\\f0247\"}.bosch-ic-wind-vane-strong:before{content:\"\\f0248\"}.bosch-ic-wind-vane-weak:before{content:\"\\f0249\"}.bosch-ic-windshield-crack:before{content:\"\\f024a\"}.bosch-ic-common-rail:before{content:\"\\f024b\"}.bosch-ic-common-rail-injector-cri1-cri2:before{content:\"\\f024c\"}.bosch-ic-common-rail-injector-crii3:before{content:\"\\f024d\"}.bosch-ic-common-rail-injector-cril3:before{content:\"\\f024e\"}.bosch-ic-common-rail-injector-crin4-2:before{content:\"\\f024f\"}.bosch-ic-common-rail-injector-crin123:before{content:\"\\f0250\"}.bosch-ic-conventional-pump-a:before{content:\"\\f0251\"}.bosch-ic-conventional-pump-b:before{content:\"\\f0252\"}.bosch-ic-edc-inline-injection-pump-pes6p:before{content:\"\\f0253\"}.bosch-ic-edc-inline-injection-pump-pes6h:before{content:\"\\f0254\"}.bosch-ic-nozzle-holder:before{content:\"\\f0255\"}.bosch-ic-unit-injector:before{content:\"\\f0256\"}.bosch-ic-qr-code-scan:before{content:\"\\f0257\"}.bosch-ic-car-mechanic-plug:before{content:\"\\f0258\"}.bosch-ic-copy-machine:before{content:\"\\f0259\"}.bosch-ic-dispenser-paper-towel:before{content:\"\\f025a\"}.bosch-ic-hand-drops-soap:before{content:\"\\f025b\"}.bosch-ic-door-automatic:before{content:\"\\f025c\"}.bosch-ic-door-automatic-slide:before{content:\"\\f025d\"}.bosch-ic-door-pull:before{content:\"\\f025e\"}.bosch-ic-door-push:before{content:\"\\f025f\"}.bosch-ic-floor-load-max:before{content:\"\\f0260\"}.bosch-ic-lockers:before{content:\"\\f0261\"}.bosch-ic-rubbish-trash-bin:before{content:\"\\f0262\"}.bosch-ic-rubbish-waste-bin:before{content:\"\\f0263\"}.bosch-ic-rubbish-waste-container:before{content:\"\\f0264\"}.bosch-ic-sugar:before{content:\"\\f0265\"}.bosch-ic-rubbish-glass-bin:before{content:\"\\f0266\"}.bosch-ic-car-seat-steering-wheel-stowable:before{content:\"\\f0267\"}.bosch-ic-car-seat-steering-wheel-standard:before{content:\"\\f0268\"}.bosch-ic-car-seat-steering-wheel-extended:before{content:\"\\f0269\"}.bosch-ic-device-bridge:before{content:\"\\f026a\"}.bosch-ic-screw-cross-slot:before{content:\"\\f026b\"}.bosch-ic-screw-hexagon-head:before{content:\"\\f026c\"}.bosch-ic-screw-hexagon-internal:before{content:\"\\f026d\"}.bosch-ic-screw-slotted:before{content:\"\\f026e\"}.bosch-ic-screw-torx:before{content:\"\\f026f\"}.bosch-ic-tape:before{content:\"\\f0270\"}.bosch-ic-tape-teflon:before{content:\"\\f0271\"}.bosch-ic-security-fire:before{content:\"\\f0272\"}.bosch-ic-led-fault:before{content:\"\\f0273\"}.bosch-ic-led-alarm:before{content:\"\\f0274\"}.bosch-ic-firepanel:before{content:\"\\f0275\"}.bosch-ic-fire-output-disabled:before{content:\"\\f0276\"}.bosch-ic-user-fire-alarm:before{content:\"\\f0277\"}.bosch-ic-car-desktop-graph:before{content:\"\\f0278\"}.bosch-ic-desktop-management-statistics:before{content:\"\\f0279\"}.bosch-ic-document-key:before{content:\"\\f027a\"}.bosch-ic-options-vertical:before{content:\"\\f027b\"}.bosch-ic-euro-arrows:before{content:\"\\f027c\"}.bosch-ic-monkey:before{content:\"\\f027d\"}.bosch-ic-appliance-resistance:before{content:\"\\f027e\"}.bosch-ic-pipe-cutter:before{content:\"\\f027f\"}.bosch-ic-flash-arrow:before{content:\"\\f0280\"}.bosch-ic-flash-arrow-disabled-light:before{content:\"\\f0281\"}.bosch-ic-heating-ceiling:before{content:\"\\f0282\"}.bosch-ic-heating-ceiling-off-light:before{content:\"\\f0283\"}.bosch-ic-heating-ceiling-off-bold:before{content:\"\\f0284\"}.bosch-ic-battery-analysis:before{content:\"\\f0285\"}.bosch-ic-battery-ice:before{content:\"\\f0286\"}.bosch-ic-battery-refresh:before{content:\"\\f0287\"}.bosch-ic-device-ids:before{content:\"\\f0288\"}.bosch-ic-device-thermostat-bcc50-72:before{content:\"\\f0289\"}.bosch-ic-device-thermostat-bcc100-72:before{content:\"\\f028a\"}.bosch-ic-silos-gas-natural:before{content:\"\\f028b\"}.bosch-ic-silos-gas-leaf-biogas:before{content:\"\\f028c\"}.bosch-ic-circle-decentralized:before{content:\"\\f028d\"}.bosch-ic-molecules-h2:before{content:\"\\f028e\"}.bosch-ic-megawatt-range-arrow-scalable:before{content:\"\\f028f\"}.bosch-ic-fan-arrows:before{content:\"\\f0290\"}.bosch-ic-recycle-arrows-pvc3:before{content:\"\\f0291\"}.bosch-ic-recycle-arrows-ps6:before{content:\"\\f0292\"}.bosch-ic-recycle-arrows-pp5:before{content:\"\\f0293\"}.bosch-ic-recycle-arrows-pet1:before{content:\"\\f0294\"}.bosch-ic-recycle-arrows-pap22:before{content:\"\\f0295\"}.bosch-ic-recycle-arrows-pap21:before{content:\"\\f0296\"}.bosch-ic-recycle-arrows-pap20:before{content:\"\\f0297\"}.bosch-ic-recycle-arrows-oth7:before{content:\"\\f0298\"}.bosch-ic-recycle-arrows-ldpe4:before{content:\"\\f0299\"}.bosch-ic-recycle-arrows-hdpe2:before{content:\"\\f029a\"}.bosch-ic-car-petrol-plug:before{content:\"\\f029b\"}.bosch-ic-user-diverse:before{content:\"\\f029c\"}.bosch-ic-fire-emergency-run-map:before{content:\"\\f029d\"}.bosch-ic-automatic-mode-fan:before{content:\"\\f029e\"}.bosch-ic-air-condition-swing-pac:before{content:\"\\f029f\"}.bosch-ic-user-fire-alarm-check:before{content:\"\\f02a0\"}.bosch-ic-handlewithcare-team:before{content:\"\\f02a1\"}.bosch-ic-settings-user-dollar:before{content:\"\\f02a2\"}.bosch-ic-skyscraper-connected:before{content:\"\\f02a3\"}.bosch-ic-security-ac:before{content:\"\\f02a4\"}.bosch-ic-windshield-glare-sun:before{content:\"\\f02a5\"}.bosch-ic-user-flag:before{content:\"\\f02a6\"}.bosch-ic-hammer-legal:before{content:\"\\f02a7\"}.bosch-ic-handover-star:before{content:\"\\f02a8\"}.bosch-ic-laptop-wrench:before{content:\"\\f02a9\"}/*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.Bosch-IC-find-part:before{content:\"\\e38f\"}.Bosch-IC-next-station:before{content:\"\\e8a9\"}.Bosch-Ic-HMI:before{content:\"\\e293\"}.Bosch-Ic-add-outline:before{content:\"\\e8a1\"}.Bosch-Ic-apas:before{content:\"\\e224\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-Ic-cash:before{content:\"\\e890\"}.Bosch-Ic-pinterest:before{content:\"\\ea8c\"}.Bosch-Ic-problem:before{content:\"\\e899\"}.Bosch-Ic-promotion:before{content:\"\\e89a\"}.Bosch-Ic-arrow-diagonal:before{content:\"\\e095\"}.Bosch-Ic-arrow-down-filled:before{content:\"\\e887\"}.Bosch-Ic-arrow-down-circle:before{content:\"\\e887\"}.Bosch-Ic-arrow-end-left:before{content:\"\\e0a1\"}.Bosch-Ic-arrow-end-right:before{content:\"\\e182\"}.Bosch-Ic-arrow-left-down-filled:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-filled:before{content:\"\\e889\"}.Bosch-Ic-arrow-left-up-filled:before{content:\"\\e88a\"}.Bosch-Ic-arrow-right-down-filled:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-filled:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-up-filled:before{content:\"\\e88d\"}.Bosch-Ic-arrow-up-filled:before{content:\"\\e88e\"}.Bosch-Ic-batch:before{content:\"\\e384\"}.Bosch-Ic-calculator:before{content:\"\\e0c7\"}.Bosch-Ic-calendar-delete:before{content:\"\\e385\"}.Bosch-Ic-call-bluetooth:before{content:\"\\e386\"}.Bosch-Ic-call-deny-outline:before{content:\"\\e8a2\"}.Bosch-Ic-call-deny:before{content:\"\\e387\"}.Bosch-Ic-call-pick-up-outline:before{content:\"\\e8a3\"}.Bosch-Ic-call-pick-up:before{content:\"\\e0cb\"}.Bosch-Ic-call-settings:before{content:\"\\e388\"}.Bosch-Ic-clear-all-annotations:before{content:\"\\e0ec\"}.Bosch-Ic-connection:before{content:\"\\e107\"}.Bosch-Ic-core-data:before{content:\"\\e38b\"}.Bosch-Ic-curve-scaling-view:before{content:\"\\e2bb\"}.Bosch-Ic-dashboard:before{content:\"\\e190\"}.Bosch-Ic-delete-material-manually:before{content:\"\\e394\"}.Bosch-Ic-dmc-code:before{content:\"\\e38e\"}.Bosch-Ic-document-to-svg:before{content:\"\\e34e\"}.Bosch-Ic-emergency-order:before{content:\"\\e12f\"}.Bosch-Ic-fast-backwards:before{content:\"\\e8a4\"}.Bosch-Ic-fast-forwards:before{content:\"\\e8a5\"}.Bosch-Ic-favorite:before{content:\"\\e2c5\"}.Bosch-Ic-go-to:before{content:\"\\e1ca\"}.Bosch-Ic-gps:before{content:\"\\e1c6\"}.Bosch-Ic-home-filled:before{content:\"\\e19d\"}.Bosch-Ic-input:before{content:\"\\e1ca\"}.Bosch-Ic-interface:before{content:\"\\e391\"}.Bosch-Ic-list:before{content:\"\\e1c1\"}.Bosch-Ic-malfunction-history:before{content:\"\\e389\"}.Bosch-Ic-malfunction-order:before{content:\"\\e38a\"}.Bosch-Ic-material-list:before{content:\"\\e393\"}.Bosch-Ic-material:before{content:\"\\e395\"}.Bosch-Ic-maximize-window:before{content:\"\\e269\"}.Bosch-Ic-menu-tree:before{content:\"\\e396\"}.Bosch-Ic-minimize-window:before{content:\"\\e1ba\"}.Bosch-Ic-notification-hint:before{content:\"\\e899\"}.Bosch-Ic-notification-question:before{content:\"\\e89b\"}.Bosch-Ic-notification-info:before{content:\"\\eb0f\"}.Bosch-Ic-notification-error:before{content:\"\\eb08\"}.Bosch-Ic-notification-error-outline:before{content:\"\\eb09\"}.Bosch-Ic-notification-outline:before{content:\"\\eb0d\"}.Bosch-Ic-notification-success:before{content:\"\\eb0a\"}.Bosch-Ic-notification-success-outline:before{content:\"\\eb0b\"}.Bosch-Ic-notification-warning:before{content:\"\\eb0c\"}.Bosch-Ic-notification:before{content:\"\\e1f5\"}.Bosch-Ic-output:before{content:\"\\e1cc\"}.Bosch-Ic-paste:before{content:\"\\e0ed\"}.Bosch-Ic-qr-code:before{content:\"\\e397\"}.Bosch-Ic-remote-cursor:before{content:\"\\e38c\"}.Bosch-Ic-scale:before{content:\"\\e167\"}.Bosch-Ic-search-chart:before{content:\"\\e383\"}.Bosch-Ic-search-web:before{content:\"\\e390\"}.Bosch-Ic-search-window:before{content:\"\\e39b\"}.Bosch-Ic-shape-ellipse:before{content:\"\\e3d1\"}.Bosch-Ic-shape-rectangle:before{content:\"\\e269\"}.Bosch-Ic-show-more-vertical:before{content:\"\\e1fb\"}.Bosch-Ic-signal:before{content:\"\\e249\"}.Bosch-Ic-skip-back:before{content:\"\\e8a6\"}.Bosch-Ic-skip-backward:before{content:\"\\e8a7\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-Ic-skip-for:before{content:\"\\e8a9\"}.Bosch-Ic-skip-forward:before{content:\"\\e8a8\"}.Bosch-Ic-sort-alphabetically:before{content:\"\\e398\"}.Bosch-Ic-switch-language:before{content:\"\\e18d\"}.Bosch-Ic-snyc-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-problem:before{content:\"\\e37f\"}.Bosch-Ic-sync:before{content:\"\\e21e\"}.Bosch-Ic-test:before{content:\"\\e105\"}.Bosch-Ic-to-order:before{content:\"\\e127\"}.Bosch-Ic-to-raw:before{content:\"\\e126\"}.Bosch-Ic-transport-orders:before{content:\"\\e0ee\"}.Bosch-Ic-unpin:before{content:\"\\e463\"}.Bosch-Ic-user-filled:before{content:\"\\e29b\"}.Bosch-Ic-video-play:before{content:\"\\e20b\"}.Bosch-Ic-weblink:before{content:\"\\e1be\"}.Bosch-Ic-warning-outline:before{content:\"\\eb0d\"}.Bosch-Ic-add-material-manually:before{content:\"\\e392\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-IC-3d-print:before{content:\"\\e085\"}.Bosch-Ic-tours:before{content:\"\\e086\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-IC-agility:before{content:\"\\e08a\"}.Bosch-Ic-clock-stop:before{content:\"\\e08b\"}.Bosch-IC-ambulance:before{content:\"\\e08c\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-Ic-arrow-down:before{content:\"\\e08f\"}.Bosch-Ic-arrow-left:before{content:\"\\e090\"}.Bosch-Ic-arrow-left-down:before{content:\"\\e091\"}.Bosch-Ic-arrow-left-up:before{content:\"\\e092\"}.Bosch-Ic-arrow-right:before{content:\"\\e093\"}.Bosch-Ic-arrow-right-down:before{content:\"\\e094\"}.Bosch-Ic-arrow-right-up:before{content:\"\\e095\"}.Bosch-IC-arrow-turn-left:before{content:\"\\e096\"}.Bosch-IC-arrow-turn-right:before{content:\"\\e097\"}.Bosch-Ic-arrow-up:before{content:\"\\e098\"}.Bosch-IC-arrows:before{content:\"\\e099\"}.Bosch-IC-arrows-x-y-z:before{content:\"\\e09a\"}.Bosch-IC-artificial-intelligence:before{content:\"\\e09b\"}.Bosch-IC-assembly-line:before{content:\"\\e09c\"}.Bosch-IC-atom:before{content:\"\\e09d\"}.Bosch-IC-augmented-reality:before{content:\"\\e09e\"}.Bosch-Ic-autobod:before{content:\"\\e09f\"}.Bosch-Ic-arrow-expand-left:before{content:\"\\e0a0\"}.Bosch-Ic-arrow-expand-double-left:before{content:\"\\e0a1\"}.Bosch-IC-bar-chart:before{content:\"\\e0a2\"}.Bosch-IC-bar-chart-double:before{content:\"\\e0a3\"}.Bosch-Ic-battery-empty:before{content:\"\\e0a4\"}.Bosch-Ic-battery-25:before{content:\"\\e0a5\"}.Bosch-Ic-battery-50:before{content:\"\\e0a6\"}.Bosch-Ic-battery-75:before{content:\"\\e0a7\"}.Bosch-Ic-battery-100:before{content:\"\\e0a8\"}.Bosch-Ic-battery-error:before{content:\"\\e0a9\"}.Bosch-IC-battery_Li-Ion:before{content:\"\\e0aa\"}.Bosch-Ic-battery-charging:before{content:\"\\e0ab\"}.Bosch-Ic-help:before{content:\"\\e89b\"}.Bosch-IC-bed:before{content:\"\\e0ac\"}.Bosch-IC-bicycle:before{content:\"\\e0ad\"}.Bosch-IC-bicycle-e:before{content:\"\\e0ae\"}.Bosch-IC-blog:before{content:\"\\e0af\"}.Bosch-IC-bluetooth:before{content:\"\\e0b0\"}.Bosch-IC-bluetooth-LE:before{content:\"\\e0b1\"}.Bosch-IC-board-speaker:before{content:\"\\e0b2\"}.Bosch-IC-boiler-connected:before{content:\"\\e0b3\"}.Bosch-IC-book:before{content:\"\\e0b4\"}.Bosch-Ic-bookmark:before{content:\"\\e0b5\"}.Bosch-Ic-bookmark-add:before{content:\"\\e0b6\"}.Bosch-Ic-bookmark-check:before{content:\"\\e0b7\"}.Bosch-Ic-bookmark-delete:before{content:\"\\e0b8\"}.Bosch-IC-books:before{content:\"\\e0b9\"}.Bosch-IC-bookshelf:before{content:\"\\e0ba\"}.Bosch-Ic-box:before{content:\"\\e0bb\"}.Bosch-IC-box-closed:before{content:\"\\e0bc\"}.Bosch-IC-box-delivery:before{content:\"\\e0bd\"}.Bosch-IC-box-delivery-connected:before{content:\"\\e0be\"}.Bosch-IC-brake-disk:before{content:\"\\e0bf\"}.Bosch-IC-briefcase:before{content:\"\\e0c0\"}.Bosch-IC-broom:before{content:\"\\e0c1\"}.Bosch-IC-broom-cloud-dirt:before{content:\"\\e0c2\"}.Bosch-Ic-brush:before{content:\"\\e0c3\"}.Bosch-Ic-bug:before{content:\"\\e0c4\"}.Bosch-IC-building:before{content:\"\\e0c5\"}.Bosch-IC-cactus:before{content:\"\\e0c6\"}.Bosch-IC-calculate:before{content:\"\\e0c7\"}.Bosch-Ic-calendar:before{content:\"\\e0c8\"}.Bosch-Ic-calendar-add:before{content:\"\\e0c9\"}.Bosch-IC-calendar-settings:before{content:\"\\e0ca\"}.Bosch-Ic-call:before{content:\"\\e0cb\"}.Bosch-IC-call-add:before{content:\"\\e0cc\"}.Bosch-IC-call-remove:before{content:\"\\e0cd\"}.Bosch-IC-call-sos:before{content:\"\\e0ce\"}.Bosch-IC-call-team:before{content:\"\\e0cf\"}.Bosch-Ic-call-wifi:before{content:\"\\e0d0\"}.Bosch-Ic-camera:before{content:\"\\e0d1\"}.Bosch-Ic-camera-switch:before{content:\"\\e0d2\"}.Bosch-IC-car:before{content:\"\\e0d3\"}.Bosch-IC-car-clock:before{content:\"\\e0d4\"}.Bosch-IC-car-connected:before{content:\"\\e0d5\"}.Bosch-IC-car-locator:before{content:\"\\e0d6\"}.Bosch-IC-car-mechanic:before{content:\"\\e0d7\"}.Bosch-IC-car-rental:before{content:\"\\e0d8\"}.Bosch-IC-car-seat-connected:before{content:\"\\e0d9\"}.Bosch-IC-car-side:before{content:\"\\e0da\"}.Bosch-IC-car-side-user:before{content:\"\\e0db\"}.Bosch-Ic-milkrun-cart:before{content:\"\\e0dc\"}.Bosch-Ic-certificate:before{content:\"\\e0dd\"}.Bosch-IC-certificate-pki:before{content:\"\\e0de\"}.Bosch-IC-change:before{content:\"\\e0df\"}.Bosch-Ic-chart:before{content:\"\\e0e0\"}.Bosch-IC-chart-check:before{content:\"\\e0e1\"}.Bosch-IC-chart-dummy:before{content:\"\\e0e2\"}.Bosch-Ic-diagram:before{content:\"\\e0e3\"}.Bosch-Ic-chat:before{content:\"\\e0e4\"}.Bosch-IC-chat-add:before{content:\"\\e0e5\"}.Bosch-IC-chat-question-answer:before{content:\"\\e0e6\"}.Bosch-IC-chat-remove:before{content:\"\\e0e7\"}.Bosch-Ic-checklist:before{content:\"\\e0e8\"}.Bosch-Ic-checkmark:before{content:\"\\e0e9\"}.Bosch-Ic-chip:before{content:\"\\e0ea\"}.Bosch-IC-circle-segment:before{content:\"\\e0eb\"}.Bosch-IC-clear-all:before{content:\"\\e0ec\"}.Bosch-Ic-clipboard:before{content:\"\\e0ed\"}.Bosch-Ic-clipboard-list:before{content:\"\\e0ee\"}.Bosch-Ic-clock:before{content:\"\\e0ef\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-IC-close-all:before{content:\"\\e0f1\"}.Bosch-Ic-cloud:before{content:\"\\e0f2\"}.Bosch-IC-cloud-co2:before{content:\"\\e0f3\"}.Bosch-Ic-cloud-download:before{content:\"\\e0f4\"}.Bosch-IC-cloud-nox:before{content:\"\\e0f5\"}.Bosch-Ic-cloud-upload:before{content:\"\\e0f6\"}.Bosch-IC-club-arm-chair:before{content:\"\\e0f7\"}.Bosch-IC-coffee-break:before{content:\"\\e0f8\"}.Bosch-IC-coin-currency:before{content:\"\\e0f9\"}.Bosch-Ic-currency:before{content:\"\\e0fa\"}.Bosch-Ic-color-picker:before{content:\"\\e0fb\"}.Bosch-IC-colors:before{content:\"\\e0fc\"}.Bosch-Ic-command:before{content:\"\\e0fd\"}.Bosch-IC-communicator:before{content:\"\\e0fe\"}.Bosch-Ic-compare:before{content:\"\\e0ff\"}.Bosch-Ic-components:before{content:\"\\e100\"}.Bosch-Ic-components-available:before{content:\"\\e101\"}.Bosch-Ic-components-reservation:before{content:\"\\e102\"}.Bosch-IC-concierge-bell:before{content:\"\\e103\"}.Bosch-IC-configuration:before{content:\"\\e104\"}.Bosch-Ic-confirmation:before{content:\"\\e105\"}.Bosch-Ic-unconnected:before{content:\"\\e106\"}.Bosch-Ic-connected:before{content:\"\\e107\"}.Bosch-IC-connectivity:before{content:\"\\e108\"}.Bosch-Ic-builder:before{content:\"\\e109\"}.Bosch-IC-consumer-goods:before{content:\"\\e10a\"}.Bosch-IC-contract:before{content:\"\\e10b\"}.Bosch-Ic-copy:before{content:\"\\e10c\"}.Bosch-IC-cube:before{content:\"\\e10d\"}.Bosch-IC-cube-arrows:before{content:\"\\e10e\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-IC-cube-shock:before{content:\"\\e110\"}.Bosch-IC-cube-stacked:before{content:\"\\e111\"}.Bosch-Ic-customerservice:before{content:\"\\e112\"}.Bosch-IC-cutlery:before{content:\"\\e113\"}.Bosch-IC-damper:before{content:\"\\e114\"}.Bosch-IC-danger-flash:before{content:\"\\e115\"}.Bosch-IC-data-loss:before{content:\"\\e116\"}.Bosch-Ic-dealer:before{content:\"\\e117\"}.Bosch-Ic-delete:before{content:\"\\e118\"}.Bosch-Ic-delivery:before{content:\"\\e119\"}.Bosch-IC-denied:before{content:\"\\e11a\"}.Bosch-Ic-desktop:before{content:\"\\e11b\"}.Bosch-IC-desktop-application-2d:before{content:\"\\e11c\"}.Bosch-IC-desktop-application-3d:before{content:\"\\e11d\"}.Bosch-IC-development:before{content:\"\\e11e\"}.Bosch-Ic-devices:before{content:\"\\e11f\"}.Bosch-IC-devices-mobile:before{content:\"\\e120\"}.Bosch-Ic-directory:before{content:\"\\e121\"}.Bosch-Ic-dna:before{content:\"\\e122\"}.Bosch-Ic-do-not-disturb:before{content:\"\\e123\"}.Bosch-Ic-document:before{content:\"\\e124\"}.Bosch-Ic-document-add:before{content:\"\\e125\"}.Bosch-IC-document-arrow-down:before{content:\"\\e126\"}.Bosch-IC-document-arrow-up:before{content:\"\\e127\"}.Bosch-Ic-document-audio:before{content:\"\\e128\"}.Bosch-IC-document-check:before{content:\"\\e129\"}.Bosch-Ic-document-to-cloud:before{content:\"\\e12a\"}.Bosch-Ic-document-copy:before{content:\"\\e12b\"}.Bosch-Ic-document-delete:before{content:\"\\e12c\"}.Bosch-Ic-document-to-doc:before{content:\"\\e12d\"}.Bosch-Ic-document-edit:before{content:\"\\e12e\"}.Bosch-Ic-document-error:before{content:\"\\e12f\"}.Bosch-Ic-document-locked:before{content:\"\\e130\"}.Bosch-Ic-log-file:before{content:\"\\e131\"}.Bosch-Ic-document-to-one:before{content:\"\\e132\"}.Bosch-Ic-document-to-pdf:before{content:\"\\e133\"}.Bosch-Ic-document-plain:before{content:\"\\e134\"}.Bosch-Ic-document-plain-add:before{content:\"\\e135\"}.Bosch-Ic-document-plain-delete:before{content:\"\\e136\"}.Bosch-Ic-document-to-ppt:before{content:\"\\e137\"}.Bosch-Ic-document-to-pub:before{content:\"\\e138\"}.Bosch-Ic-document-to-rtf:before{content:\"\\e139\"}.Bosch-Ic-document-save-to:before{content:\"\\e13a\"}.Bosch-Ic-search-document:before{content:\"\\e13b\"}.Bosch-Ic-document-settings:before{content:\"\\e13c\"}.Bosch-Ic-document-test:before{content:\"\\e13d\"}.Bosch-Ic-document-text:before{content:\"\\e13e\"}.Bosch-Ic-document-to-txt:before{content:\"\\e13f\"}.Bosch-Ic-document-to-vsd:before{content:\"\\e140\"}.Bosch-Ic-document-to-xls:before{content:\"\\e141\"}.Bosch-Ic-document-to-xml:before{content:\"\\e142\"}.Bosch-Ic-document-to-zip:before{content:\"\\e143\"}.Bosch-IC-dome:before{content:\"\\e144\"}.Bosch-IC-door:before{content:\"\\e145\"}.Bosch-IC-door-sensor:before{content:\"\\e146\"}.Bosch-Ic-arrow-expand-down:before{content:\"\\e147\"}.Bosch-Ic-arrow-expand-double-down:before{content:\"\\e148\"}.Bosch-Ic-arrow-left-circle:before{content:\"\\e889\"}.Bosch-Ic-arrow-right-circle:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-down-circle:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-up-circle:before{content:\"\\e88d\"}.Bosch-Ic-arrow-left-down-circle:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-up-circle:before{content:\"\\e88a\"}.Bosch-Ic-arrow-up-circle:before{content:\"\\e88e\"}.Bosch-Ic-download:before{content:\"\\e149\"}.Bosch-IC-drag-handle:before{content:\"\\e14a\"}.Bosch-IC-dragdrop:before{content:\"\\e14b\"}.Bosch-IC-drop:before{content:\"\\e14c\"}.Bosch-IC-e-Charging:before{content:\"\\e14d\"}.Bosch-IC-e-mobility:before{content:\"\\e14e\"}.Bosch-IC-ear-off:before{content:\"\\e14f\"}.Bosch-IC-ear-on:before{content:\"\\e150\"}.Bosch-IC-eco-system:before{content:\"\\e151\"}.Bosch-Ic-edit:before{content:\"\\e152\"}.Bosch-IC-education:before{content:\"\\e153\"}.Bosch-IC-efficiency:before{content:\"\\e154\"}.Bosch-IC-elevator:before{content:\"\\e155\"}.Bosch-IC-elevator-alarm:before{content:\"\\e156\"}.Bosch-IC-elevator-cloud:before{content:\"\\e157\"}.Bosch-IC-elevator-headset:before{content:\"\\e158\"}.Bosch-IC-elevator-service:before{content:\"\\e159\"}.Bosch-IC-emergency-exit:before{content:\"\\e15a\"}.Bosch-IC-emoji-happy:before{content:\"\\e15b\"}.Bosch-IC-emoji-neutral:before{content:\"\\e15c\"}.Bosch-IC-emoji-sad:before{content:\"\\e15d\"}.Bosch-IC-emoji-super-happy:before{content:\"\\e15e\"}.Bosch-IC-emoji-very-sad:before{content:\"\\e15f\"}.Bosch-IC-EU-energy-label:before{content:\"\\e160\"}.Bosch-IC-excavator:before{content:\"\\e161\"}.Bosch-Ic-exit:before{content:\"\\e163\"}.Bosch-IC-expansion-arrows:before{content:\"\\e164\"}.Bosch-IC-explosion:before{content:\"\\e165\"}.Bosch-Ic-export:before{content:\"\\e166\"}.Bosch-IC-externallink:before{content:\"\\e167\"}.Bosch-IC-fast-forward:before{content:\"\\e168\"}.Bosch-IC-faucet:before{content:\"\\e169\"}.Bosch-IC-favorites:before{content:\"\\e16a\"}.Bosch-IC-fax:before{content:\"\\e16b\"}.Bosch-IC-female:before{content:\"\\e16c\"}.Bosch-IC-film:before{content:\"\\e16d\"}.Bosch-Ic-filter:before{content:\"\\e16e\"}.Bosch-IC-filter-success:before{content:\"\\e16f\"}.Bosch-IC-fingerprint:before{content:\"\\e170\"}.Bosch-IC-fire:before{content:\"\\e171\"}.Bosch-IC-fire-emergency:before{content:\"\\e172\"}.Bosch-IC-fireworks:before{content:\"\\e173\"}.Bosch-Ic-first-aid:before{content:\"\\e174\"}.Bosch-IC-first-aid-cross:before{content:\"\\e175\"}.Bosch-IC-fit-to-sceen:before{content:\"\\e176\"}.Bosch-Ic-flag:before{content:\"\\e177\"}.Bosch-Ic-danger-flash-outline:before{content:\"\\e178\"}.Bosch-IC-flask:before{content:\"\\e179\"}.Bosch-Ic-flexpress:before{content:\"\\e17a\"}.Bosch-Ic-twitter:before{content:\"\\ea8f\"}.Bosch-Ic-folder:before{content:\"\\e17b\"}.Bosch-Ic-download-from-folder:before{content:\"\\e17c\"}.Bosch-Ic-download-frame:before{content:\"\\e893\"}.Bosch-IC-folder-open:before{content:\"\\e17d\"}.Bosch-Ic-upload-to-folder:before{content:\"\\e17e\"}.Bosch-Ic-fork-lift:before{content:\"\\e17f\"}.Bosch-IC-fork-lift-locator:before{content:\"\\e180\"}.Bosch-Ic-arrow-expand-right:before{content:\"\\e181\"}.Bosch-Ic-arrow-expand-double-right:before{content:\"\\e182\"}.Bosch-IC-full-empty:before{content:\"\\e183\"}.Bosch-Ic-fullscreen:before{content:\"\\e184\"}.Bosch-IC-fullscreen-arrows:before{content:\"\\e185\"}.Bosch-Ic-fullscreen-exit:before{content:\"\\e186\"}.Bosch-IC-g-arrow-down:before{content:\"\\e187\"}.Bosch-IC-g-arrow-up:before{content:\"\\e188\"}.Bosch-IC-generator:before{content:\"\\e189\"}.Bosch-IC-gift:before{content:\"\\e18a\"}.Bosch-IC-glance:before{content:\"\\e18b\"}.Bosch-IC-glas-plate:before{content:\"\\e18c\"}.Bosch-IC-globe:before{content:\"\\e18d\"}.Bosch-IC-globe-arrow:before{content:\"\\e18e\"}.Bosch-IC-glossary:before{content:\"\\e18f\"}.Bosch-Ic-grid:before{content:\"\\e190\"}.Bosch-IC-hammer:before{content:\"\\e191\"}.Bosch-Ic-hand:before{content:\"\\e192\"}.Bosch-IC-hand-motion:before{content:\"\\e193\"}.Bosch-IC-hand-motion-off:before{content:\"\\e194\"}.Bosch-IC-handlewithcare:before{content:\"\\e195\"}.Bosch-Ic-add-material-manually:before{content:\"\\e196\"}.Bosch-Ic-handover-check:before{content:\"\\e197\"}.Bosch-Ic-remove-from-handover:before{content:\"\\e198\"}.Bosch-IC-hanger:before{content:\"\\e199\"}.Bosch-Ic-health:before{content:\"\\e19a\"}.Bosch-IC-heart:before{content:\"\\e19b\"}.Bosch-Ic-history:before{content:\"\\e19c\"}.Bosch-Ic-home-outline:before{content:\"\\e19d\"}.Bosch-IC-home-locator:before{content:\"\\e19e\"}.Bosch-Ic-sandclock:before{content:\"\\e19f\"}.Bosch-IC-hydrant:before{content:\"\\e1a0\"}.Bosch-IC-ice:before{content:\"\\e1a1\"}.Bosch-Ic-image:before{content:\"\\e1a2\"}.Bosch-IC-imagery-add:before{content:\"\\e1a3\"}.Bosch-IC-imagery-remove:before{content:\"\\e1a4\"}.Bosch-Ic-import:before{content:\"\\e1a5\"}.Bosch-Ic-imprint:before{content:\"\\e1a6\"}.Bosch-IC-impulse:before{content:\"\\e1a7\"}.Bosch-Ic-fabric:before{content:\"\\e1a8\"}.Bosch-IC-industry-clock:before{content:\"\\e1a9\"}.Bosch-IC-industry-connected:before{content:\"\\e1aa\"}.Bosch-Ic-information:before{content:\"\\e1ac\"}.Bosch-Ic-information-tooltip:before{content:\"\\e1ac\"}.Bosch-IC-interval:before{content:\"\\e1ad\"}.Bosch-IC-it-device:before{content:\"\\e1ae\"}.Bosch-IC-jewel:before{content:\"\\e1af\"}.Bosch-IC-keyboard:before{content:\"\\e1b0\"}.Bosch-IC-label:before{content:\"\\e1b1\"}.Bosch-Ic-laptop-01:before{content:\"\\e1b2\"}.Bosch-IC-laser:before{content:\"\\e1b3\"}.Bosch-IC-layout:before{content:\"\\e1b4\"}.Bosch-IC-leaf:before{content:\"\\e1b5\"}.Bosch-IC-led:before{content:\"\\e1b6\"}.Bosch-Ic-world:before{content:\"\\e8a0\"}.Bosch-Ic-facebook:before{content:\"\\ea87\"}.Bosch-Ic-checkmark-outline:before{content:\"\\e891\"}.Bosch-Ic-youtube:before{content:\"\\ea96\"}.Bosch-Ic-share:before{content:\"\\e242\"}.Bosch-Ic-user:before{content:\"\\e29b\"}.Bosch-Ic-upload_outline:before{content:\"\\e89f\"}.Bosch-Ic-download_outline:before{content:\"\\e893\"}.Bosch-Ic-wishlist-add:before{content:\"\\e3d3\"}.Bosch-Ic-youtube-frame:before{content:\"\\ea96\"}.Bosch-Ic-less-minimize:before{content:\"\\e1ba\"}.Bosch-IC-lightbulb:before{content:\"\\e1bb\"}.bosch-Ic-innovation:before{content:\"\\e1bc\"}.Bosch-IC-line-chart:before{content:\"\\e1bd\"}.Bosch-Ic-link:before{content:\"\\e1be\"}.Bosch-IC-link-broken:before{content:\"\\e1bf\"}.Bosch-Ic-list-view:before{content:\"\\e1c0\"}.Bosch-Ic-menu:before{content:\"\\e1c1\"}.Bosch-IC-livechat:before{content:\"\\e1c2\"}.Bosch-Ic-location-pin:before{content:\"\\e1c3\"}.Bosch-IC-locator-ip-connected:before{content:\"\\e1c4\"}.Bosch-IC-locator-ip-disconnected:before{content:\"\\e1c5\"}.Bosch-IC-locator-spot:before{content:\"\\e1c6\"}.Bosch-IC-locator-spot-check:before{content:\"\\e1c7\"}.Bosch-Ic-lock-closed:before{content:\"\\e1c8\"}.Bosch-Ic-lock-open:before{content:\"\\e1c9\"}.Bosch-Ic-log-in:before{content:\"\\e1ca\"}.Bosch-Ic-logistics:before{content:\"\\e1cb\"}.Bosch-Ic-log-out:before{content:\"\\e1cc\"}.Bosch-IC-machine:before{content:\"\\e1cd\"}.Bosch-Ic-magnet:before{content:\"\\e1ce\"}.Bosch-Ic-mail:before{content:\"\\e1cf\"}.Bosch-Ic-mail-forward:before{content:\"\\e1d0\"}.Bosch-Ic-mail-open:before{content:\"\\e1d1\"}.Bosch-Ic-mail-out:before{content:\"\\e1d2\"}.Bosch-Ic-mail-answer:before{content:\"\\e1d3\"}.Bosch-IC-male:before{content:\"\\e1d4\"}.Bosch-Ic-map:before{content:\"\\e1d5\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-IC-material-check:before{content:\"\\e1d7\"}.Bosch-Ic-remove-material:before{content:\"\\e1d8\"}.Bosch-IC-maximize:before{content:\"\\e1d9\"}.Bosch-IC-mechanic:before{content:\"\\e1da\"}.Bosch-IC-megaphone:before{content:\"\\e1db\"}.Bosch-Ic-message:before{content:\"\\e1dc\"}.Bosch-IC-mic:before{content:\"\\e1dd\"}.Bosch-Ic-microphone:before{content:\"\\e1de\"}.Bosch-IC-microphone-classic:before{content:\"\\e1df\"}.Bosch-IC-minimize:before{content:\"\\e1e0\"}.Bosch-IC-money-currency:before{content:\"\\e1e1\"}.Bosch-IC-money-dollar:before{content:\"\\e1e2\"}.Bosch-IC-moneybag-currency:before{content:\"\\e1e3\"}.Bosch-IC-moneybag-dollar:before{content:\"\\e1e4\"}.Bosch-IC-monument:before{content:\"\\e1e5\"}.Bosch-IC-moon:before{content:\"\\e1e6\"}.Bosch-IC-mouse:before{content:\"\\e1ea\"}.Bosch-IC-mouse-left:before{content:\"\\e1eb\"}.Bosch-IC-mouse-right:before{content:\"\\e1ec\"}.Bosch-IC-mouth:before{content:\"\\e1ed\"}.Bosch-Ic-my-product:before{content:\"\\e1ee\"}.Bosch-IC-new-way-work:before{content:\"\\e1ef\"}.Bosch-Ic-mail-sent:before{content:\"\\e1f0\"}.Bosch-Ic-news:before{content:\"\\e1f1\"}.Bosch-IC-nose:before{content:\"\\e1f2\"}.Bosch-Ic-notes:before{content:\"\\e1f3\"}.Bosch-Ic-notes-edit:before{content:\"\\e1f4\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-IC-notification-active:before{content:\"\\e1f6\"}.Bosch-IC-notification-add:before{content:\"\\e1f7\"}.Bosch-IC-notification-remove:before{content:\"\\e1f8\"}.Bosch-Ic-oil-trailer:before{content:\"\\e1f9\"}.Bosch-IC-omnichannel:before{content:\"\\e1fa\"}.Bosch-Ic-show-more-horizontal:before{content:\"\\e1fb\"}.Bosch-IC-origami-boat:before{content:\"\\e1fc\"}.Bosch-Ic-palette-trailer:before{content:\"\\e1fd\"}.Bosch-IC-pallete-car-connected:before{content:\"\\e1fe\"}.Bosch-IC-panel:before{content:\"\\e1ff\"}.Bosch-Ic-paperclip:before{content:\"\\e200\"}.Bosch-IC-paperplane:before{content:\"\\e201\"}.Bosch-IC-parking:before{content:\"\\e202\"}.Bosch-Ic-pause:before{content:\"\\e203\"}.Bosch-Ic-creditcard:before{content:\"\\e204\"}.Bosch-IC-people:before{content:\"\\e205\"}.Bosch-IC-petrol-station:before{content:\"\\e206\"}.Bosch-IC-piggybank:before{content:\"\\e207\"}.Bosch-Ic-pin:before{content:\"\\e208\"}.Bosch-IC-pin-modern:before{content:\"\\e209\"}.Bosch-IC-plane-side:before{content:\"\\e20a\"}.Bosch-Ic-play:before{content:\"\\e20b\"}.Bosch-IC-plug-12V:before{content:\"\\e20c\"}.Bosch-IC-podium-speaker:before{content:\"\\e20d\"}.Bosch-IC-police:before{content:\"\\e20e\"}.Bosch-IC-poop:before{content:\"\\e20f\"}.Bosch-IC-post-it:before{content:\"\\e210\"}.Bosch-IC-postal-code:before{content:\"\\e211\"}.Bosch-IC-power:before{content:\"\\e212\"}.Bosch-Ic-print:before{content:\"\\e213\"}.Bosch-IC-prototyping:before{content:\"\\e214\"}.Bosch-IC-puzzle:before{content:\"\\e215\"}.Bosch-IC-quad:before{content:\"\\e216\"}.Bosch-IC-question:before{content:\"\\e217\"}.Bosch-IC-radar:before{content:\"\\e218\"}.Bosch-IC-radio:before{content:\"\\e219\"}.Bosch-Ic-radio:before{content:\"\\e21a\"}.Bosch-IC-redirect:before{content:\"\\e21b\"}.Bosch-Ic-redo:before{content:\"\\e21c\"}.Bosch-IC-reference:before{content:\"\\e21d\"}.Bosch-Ic-refresh:before{content:\"\\e21e\"}.Bosch-IC-refresh-cloud:before{content:\"\\e21f\"}.Bosch-Ic-registration:before{content:\"\\e220\"}.Bosch-IC-remote:before{content:\"\\e221\"}.Bosch-Ic-reset:before{content:\"\\e222\"}.Bosch-IC-resolution:before{content:\"\\e223\"}.Bosch-Ic-robot-automation:before{content:\"\\e224\"}.Bosch-IC-robot-connected:before{content:\"\\e225\"}.Bosch-IC-robothead:before{content:\"\\e226\"}.Bosch-IC-rocket:before{content:\"\\e227\"}.Bosch-Ic-rotation:before{content:\"\\e228\"}.Bosch-Ic-rotate-360-degrees:before{content:\"\\e229\"}.Bosch-Ic-rotate-clockwise:before{content:\"\\e22a\"}.Bosch-Ic-rotate-counterclockwise:before{content:\"\\e22b\"}.Bosch-IC-rotation-y-down:before{content:\"\\e22c\"}.Bosch-IC-rotation-y-up:before{content:\"\\e22d\"}.Bosch-Ic-navigation:before{content:\"\\e22e\"}.Bosch-Ic-measure:before{content:\"\\e22f\"}.Bosch-IC-ruler-pen:before{content:\"\\e230\"}.Bosch-Ic-satelite:before{content:\"\\e231\"}.Bosch-Ic-save:before{content:\"\\e232\"}.Bosch-IC-scale:before{content:\"\\e233\"}.Bosch-Ic-cut:before{content:\"\\e234\"}.Bosch-Ic-search:before{content:\"\\e235\"}.Bosch-IC-security:before{content:\"\\e236\"}.Bosch-IC-security-check:before{content:\"\\e237\"}.Bosch-IC-security-user:before{content:\"\\e238\"}.Bosch-Ic-sensor:before{content:\"\\e239\"}.Bosch-Ic-server:before{content:\"\\e23a\"}.Bosch-IC-server-expansion:before{content:\"\\e23b\"}.Bosch-IC-server-rate:before{content:\"\\e23c\"}.Bosch-IC-service-agent:before{content:\"\\e23d\"}.Bosch-IC-service-time:before{content:\"\\e23e\"}.Bosch-Ic-settings:before{content:\"\\e23f\"}.Bosch-IC-settings-arrows:before{content:\"\\e240\"}.Bosch-IC-settings-connected:before{content:\"\\e241\"}.Bosch-Ic-share-outline:before{content:\"\\e242\"}.Bosch-IC-shield-stop:before{content:\"\\e243\"}.Bosch-IC-ship:before{content:\"\\e244\"}.Bosch-IC-ship-side:before{content:\"\\e245\"}.Bosch-Ic-shoppingcart:before{content:\"\\e246\"}.Bosch-IC-shoppingcart-add:before{content:\"\\e247\"}.Bosch-IC-shoppingcart-switch:before{content:\"\\e248\"}.Bosch-IC-signal-full:before{content:\"\\e249\"}.Bosch-IC-signal-half:before{content:\"\\e24a\"}.Bosch-IC-signal-lost:before{content:\"\\e24b\"}.Bosch-IC-signpost:before{content:\"\\e24c\"}.Bosch-IC-simcard:before{content:\"\\e24d\"}.Bosch-IC-skyscraper:before{content:\"\\e24e\"}.Bosch-IC-smartcity:before{content:\"\\e24f\"}.Bosch-IC-smarthome:before{content:\"\\e250\"}.Bosch-Ic-phone:before{content:\"\\e251\"}.Bosch-IC-smartphone-acoustic:before{content:\"\\e252\"}.Bosch-IC-smartphone-arrow-right:before{content:\"\\e253\"}.Bosch-IC-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.Bosch-IC-smartphone-clock:before{content:\"\\e255\"}.Bosch-IC-smartphone-cloud:before{content:\"\\e256\"}.Bosch-IC-smartphone-loading-wheel:before{content:\"\\e257\"}.Bosch-IC-smartphone-smashed:before{content:\"\\e258\"}.Bosch-IC-smartphone-vibration:before{content:\"\\e259\"}.Bosch-IC-smartphone-z:before{content:\"\\e25a\"}.Bosch-Ic-spareparts:before{content:\"\\e25c\"}.Bosch-IC-spark-plug:before{content:\"\\e25d\"}.Bosch-IC-speaker-acoustic:before{content:\"\\e25e\"}.Bosch-IC-speedometer:before{content:\"\\e25f\"}.Bosch-IC-stadium:before{content:\"\\e260\"}.Bosch-IC-stairs:before{content:\"\\e261\"}.Bosch-IC-stamp:before{content:\"\\e262\"}.Bosch-IC-standby:before{content:\"\\e263\"}.Bosch-IC-steering-wheel:before{content:\"\\e264\"}.Bosch-IC-steering-wheel-connected:before{content:\"\\e265\"}.Bosch-IC-steering-wheel-hands:before{content:\"\\e266\"}.Bosch-IC-steps:before{content:\"\\e267\"}.Bosch-IC-stethoscope:before{content:\"\\e268\"}.Bosch-Ic-stop:before{content:\"\\e269\"}.Bosch-IC-stopwatch:before{content:\"\\e26a\"}.Bosch-IC-storage-local:before{content:\"\\e26b\"}.Bosch-IC-structure:before{content:\"\\e26c\"}.Bosch-IC-subtitles:before{content:\"\\e26d\"}.Bosch-Ic-summary:before{content:\"\\e26f\"}.Bosch-IC-sun:before{content:\"\\e270\"}.Bosch-IC-sun-half-filled:before{content:\"\\e271\"}.Bosch-IC-sunshade-table:before{content:\"\\e272\"}.Bosch-IC-surveillance:before{content:\"\\e273\"}.Bosch-IC-table-chairs:before{content:\"\\e274\"}.Bosch-Ic-tablet:before{content:\"\\e275\"}.Bosch-IC-team:before{content:\"\\e276\"}.Bosch-IC-team-lightbulb:before{content:\"\\e277\"}.Bosch-IC-technical-service:before{content:\"\\e278\"}.Bosch-Ic-temperature:before{content:\"\\e279\"}.Bosch-IC-temperature-high:before{content:\"\\e27a\"}.Bosch-IC-temperature-low:before{content:\"\\e27b\"}.Bosch-IC-theater:before{content:\"\\e27c\"}.Bosch-Ic-dislike:before{content:\"\\e27d\"}.Bosch-Ic-like:before{content:\"\\e27e\"}.Bosch-Ic-ticket:before{content:\"\\e27f\"}.Bosch-IC-touch:before{content:\"\\e280\"}.Bosch-IC-touch-sos:before{content:\"\\e281\"}.Bosch-IC-towing-truck:before{content:\"\\e282\"}.Bosch-IC-train:before{content:\"\\e283\"}.Bosch-IC-train-side:before{content:\"\\e284\"}.Bosch-Ic-transport-movements:before{content:\"\\e285\"}.Bosch-IC-transscript:before{content:\"\\e286\"}.Bosch-IC-transscript-off:before{content:\"\\e287\"}.Bosch-IC-transscript-on:before{content:\"\\e288\"}.Bosch-IC-tree:before{content:\"\\e289\"}.Bosch-IC-tricycle:before{content:\"\\e28a\"}.Bosch-Ic-default-trailer:before{content:\"\\e28b\"}.Bosch-Ic-taxi-trailer:before{content:\"\\e28c\"}.Bosch-Ic-shelve-trailer:before{content:\"\\e28d\"}.Bosch-IC-trophy:before{content:\"\\e28e\"}.Bosch-IC-truck:before{content:\"\\e28f\"}.Bosch-IC-tuktuk:before{content:\"\\e290\"}.Bosch-Ic-tune:before{content:\"\\e291\"}.Bosch-IC-typography:before{content:\"\\e292\"}.Bosch-IC-ui:before{content:\"\\e293\"}.Bosch-IC-umbrella:before{content:\"\\e294\"}.Bosch-Ic-undo:before{content:\"\\e295\"}.Bosch-Ic-arrow-expand-up:before{content:\"\\e296\"}.Bosch-Ic-arrow-expand-double-up:before{content:\"\\e297\"}.Bosch-Ic-sort:before{content:\"\\e298\"}.Bosch-Ic-upload:before{content:\"\\e299\"}.Bosch-IC-usb:before{content:\"\\e29a\"}.Bosch-Ic-user-outline:before{content:\"\\e29b\"}.Bosch-IC-user-advanced:before{content:\"\\e29c\"}.Bosch-IC-user-beginner:before{content:\"\\e29d\"}.Bosch-IC-user-desktop:before{content:\"\\e29e\"}.Bosch-IC-user-down:before{content:\"\\e29f\"}.Bosch-IC-user-falling:before{content:\"\\e2a0\"}.Bosch-IC-user-hand:before{content:\"\\e2a1\"}.Bosch-IC-user-hand-stop:before{content:\"\\e2a2\"}.Bosch-IC-user-professional:before{content:\"\\e2a3\"}.Bosch-IC-user-run:before{content:\"\\e2a4\"}.Bosch-IC-user-share:before{content:\"\\e2a5\"}.Bosch-IC-user-voice:before{content:\"\\e2a6\"}.Bosch-IC-user-walk:before{content:\"\\e2a7\"}.Bosch-Ic-videocam:before{content:\"\\e2a8\"}.Bosch-Ic-videocam-disabled:before{content:\"\\e2a9\"}.Bosch-IC-video-record:before{content:\"\\e2aa\"}.Bosch-IC-virtual-reality:before{content:\"\\e2ab\"}.Bosch-Ic-volume-disabled:before{content:\"\\e2ac\"}.Bosch-Ic-volume-high:before{content:\"\\e2ad\"}.Bosch-Ic-volume-low:before{content:\"\\e2ae\"}.Bosch-Ic-volume-off:before{content:\"\\e2af\"}.Bosch-Ic-magic-wand:before{content:\"\\e2b0\"}.Bosch-Ic-hidden-project:before{content:\"\\e2b2\"}.Bosch-Ic-show:before{content:\"\\e2b3\"}.Bosch-IC-water-shower:before{content:\"\\e2b4\"}.Bosch-IC-welcome:before{content:\"\\e2b5\"}.Bosch-IC-wellness:before{content:\"\\e2b6\"}.Bosch-IC-wheelchair:before{content:\"\\e2b7\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-IC-wide-angled-arrow:before{content:\"\\e2b9\"}.Bosch-Ic-wifi:before{content:\"\\e2ba\"}.Bosch-Ic-window-analysis:before{content:\"\\e2bb\"}.Bosch-Ic-window-browser:before{content:\"\\e2bc\"}.Bosch-Ic-window-console:before{content:\"\\e2bd\"}.Bosch-Ic-gaussian-view:before{content:\"\\e2be\"}.Bosch-Ic-window-info:before{content:\"\\e2bf\"}.Bosch-IC-window-new:before{content:\"\\e2c0\"}.Bosch-IC-window-resize:before{content:\"\\e2c1\"}.Bosch-Ic-general-scaling-view:before{content:\"\\e2c2\"}.Bosch-Ic-restore-window:before{content:\"\\e2c3\"}.Bosch-Ic-window-statistic:before{content:\"\\e2c4\"}.Bosch-Ic-wishlist:before{content:\"\\e2c5\"}.Bosch-Ic-work-orders:before{content:\"\\e2c6\"}.Bosch-IC-worldwideweb:before{content:\"\\e2c7\"}.Bosch-Ic-wrench:before{content:\"\\e2c8\"}.Bosch-Ic-zoom-in:before{content:\"\\e2c9\"}.Bosch-Ic-zoom-out:before{content:\"\\e2ca\"}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */input{color:#000000;background-color:#e0e2e5;box-shadow:none;width:100%;min-width:160px;box-sizing:border-box;caret-color:#000000;cursor:auto;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;border-width:0px 0px 1px 0px;border-color:#000000;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0;padding:12px 16px 11px 16px}input:hover:enabled,input:hover{background-color:#c1c7cc;box-shadow:none}input:hover:enabled~.label-top,input:hover~.label-top{background-color:#c1c7cc}input:hover:enabled::-moz-placeholder,input:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input:hover:enabled:-ms-input-placeholder,input:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:hover:enabled::-webkit-input-placeholder,input:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:hover:enabled::placeholder,input:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:hover:enabled label,input:hover label{background-color:#c1c7cc}input:active:enabled,input.active,input:active:enabled:hover,input.active:hover{background-color:#a4abb3;box-shadow:none}input:active:enabled~.label-top,input.active~.label-top,input:active:enabled:hover~.label-top,input.active:hover~.label-top{background-color:#a4abb3}input:active:enabled::-moz-placeholder,input.active::-moz-placeholder,input:active:enabled:hover::-moz-placeholder,input.active:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input:active:enabled:-ms-input-placeholder,input.active:-ms-input-placeholder,input:active:enabled:hover:-ms-input-placeholder,input.active:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:active:enabled::-webkit-input-placeholder,input.active::-webkit-input-placeholder,input:active:enabled:hover::-webkit-input-placeholder,input.active:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:active:enabled::placeholder,input.active::placeholder,input:active:enabled:hover::placeholder,input.active:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:active:enabled label,input.active label,input:active:enabled:hover label,input.active:hover label{background-color:#a4abb3}input:focus,input:focus:enabled,input:focus:hover,input:focus:enabled:hover{background-color:#d1e4ff;box-shadow:none}input:focus~.label-top,input:focus:enabled~.label-top,input:focus:hover~.label-top,input:focus:enabled:hover~.label-top{background-color:#d1e4ff}input:focus::-moz-placeholder,input:focus:enabled::-moz-placeholder,input:focus:hover::-moz-placeholder,input:focus:enabled:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input:focus:-ms-input-placeholder,input:focus:enabled:-ms-input-placeholder,input:focus:hover:-ms-input-placeholder,input:focus:enabled:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:focus::-webkit-input-placeholder,input:focus:enabled::-webkit-input-placeholder,input:focus:hover::-webkit-input-placeholder,input:focus:enabled:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:focus::placeholder,input:focus:enabled::placeholder,input:focus:hover::placeholder,input:focus:enabled:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:focus label,input:focus:enabled label,input:focus:hover label,input:focus:enabled:hover label{background-color:#d1e4ff}input:disabled,input:hover:disabled,input.disabled{color:#a4abb3;background-color:#e0e2e5;cursor:default;border-width:0px 0px 1px 0px;border-color:#a4abb3;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}input:disabled::-moz-placeholder,input:hover:disabled::-moz-placeholder,input.disabled::-moz-placeholder{color:#a4abb3;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input:disabled:-ms-input-placeholder,input:hover:disabled:-ms-input-placeholder,input.disabled:-ms-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:disabled::-webkit-input-placeholder,input:hover:disabled::-webkit-input-placeholder,input.disabled::-webkit-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:disabled::placeholder,input:hover:disabled::placeholder,input.disabled::placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input:disabled~.label-top,input:hover:disabled~.label-top,input.disabled~.label-top{color:#a4abb3;cursor:not-allowed}input:disabled label,input:hover:disabled label,input.disabled label{background-color:#e0e2e5}input[readonly]{color:#a4abb3;background-color:#e0e2e5;cursor:default;border-width:0px 0px 1px 0px;border-color:#000000;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}input[readonly]::-moz-placeholder{color:#a4abb3;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input[readonly]:-ms-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input[readonly]::-webkit-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input[readonly]::placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input[readonly]~.label-top{color:#a4abb3;cursor:not-allowed}input[readonly] label{background-color:#e0e2e5}input:focus{box-shadow:none;outline:0}input::-ms-expand{border:0;background-color:transparent}input::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}input:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}input[type=number]{-moz-appearance:textfield}input::-webkit-outer-spin-button,input::-webkit-inner-spin-button{-webkit-appearance:none;margin:0}.search-input input{padding-right:44px}.search-input::after{font-family:\"Bosch-Ic\";content:\"\\e235\";font-size:24px;color:#000000;display:inline-block;position:relative;right:12px;bottom:14px;margin-left:-24px;z-index:1;pointer-events:none}textarea{color:#000000;background-color:#e0e2e5;box-shadow:none;width:100%;min-width:160px;box-sizing:border-box;caret-color:#000000;cursor:auto;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;border-width:0px 0px 1px 0px;border-color:#000000;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0;padding:12px 16px 11px 16px;border-width:0px 0px 1px 0px;border-color:#000000;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0;min-height:96px;resize:vertical}textarea:hover:enabled,textarea:hover{background-color:#c1c7cc;box-shadow:none}textarea:hover:enabled~.label-top,textarea:hover~.label-top{background-color:#c1c7cc}textarea:hover:enabled::-moz-placeholder,textarea:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea:hover:enabled:-ms-input-placeholder,textarea:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:hover:enabled::-webkit-input-placeholder,textarea:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:hover:enabled::placeholder,textarea:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:hover:enabled label,textarea:hover label{background-color:#c1c7cc}textarea:active:enabled,textarea.active,textarea:active:enabled:hover,textarea.active:hover{background-color:#a4abb3;box-shadow:none}textarea:active:enabled~.label-top,textarea.active~.label-top,textarea:active:enabled:hover~.label-top,textarea.active:hover~.label-top{background-color:#a4abb3}textarea:active:enabled::-moz-placeholder,textarea.active::-moz-placeholder,textarea:active:enabled:hover::-moz-placeholder,textarea.active:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea:active:enabled:-ms-input-placeholder,textarea.active:-ms-input-placeholder,textarea:active:enabled:hover:-ms-input-placeholder,textarea.active:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:active:enabled::-webkit-input-placeholder,textarea.active::-webkit-input-placeholder,textarea:active:enabled:hover::-webkit-input-placeholder,textarea.active:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:active:enabled::placeholder,textarea.active::placeholder,textarea:active:enabled:hover::placeholder,textarea.active:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:active:enabled label,textarea.active label,textarea:active:enabled:hover label,textarea.active:hover label{background-color:#a4abb3}textarea:focus,textarea:focus:enabled,textarea:focus:hover,textarea:focus:enabled:hover{background-color:#d1e4ff;box-shadow:none}textarea:focus~.label-top,textarea:focus:enabled~.label-top,textarea:focus:hover~.label-top,textarea:focus:enabled:hover~.label-top{background-color:#d1e4ff}textarea:focus::-moz-placeholder,textarea:focus:enabled::-moz-placeholder,textarea:focus:hover::-moz-placeholder,textarea:focus:enabled:hover::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea:focus:-ms-input-placeholder,textarea:focus:enabled:-ms-input-placeholder,textarea:focus:hover:-ms-input-placeholder,textarea:focus:enabled:hover:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:focus::-webkit-input-placeholder,textarea:focus:enabled::-webkit-input-placeholder,textarea:focus:hover::-webkit-input-placeholder,textarea:focus:enabled:hover::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:focus::placeholder,textarea:focus:enabled::placeholder,textarea:focus:hover::placeholder,textarea:focus:enabled:hover::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:focus label,textarea:focus:enabled label,textarea:focus:hover label,textarea:focus:enabled:hover label{background-color:#d1e4ff}textarea:disabled,textarea:hover:disabled,textarea.disabled{color:#a4abb3;background-color:#e0e2e5;cursor:default;border-width:0px 0px 1px 0px;border-color:#a4abb3;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}textarea:disabled::-moz-placeholder,textarea:hover:disabled::-moz-placeholder,textarea.disabled::-moz-placeholder{color:#a4abb3;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea:disabled:-ms-input-placeholder,textarea:hover:disabled:-ms-input-placeholder,textarea.disabled:-ms-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:disabled::-webkit-input-placeholder,textarea:hover:disabled::-webkit-input-placeholder,textarea.disabled::-webkit-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:disabled::placeholder,textarea:hover:disabled::placeholder,textarea.disabled::placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea:disabled~.label-top,textarea:hover:disabled~.label-top,textarea.disabled~.label-top{color:#a4abb3;cursor:not-allowed}textarea:disabled label,textarea:hover:disabled label,textarea.disabled label{background-color:#e0e2e5}textarea[readonly]{color:#a4abb3;background-color:#e0e2e5;cursor:default;border-width:0px 0px 1px 0px;border-color:#000000;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}textarea[readonly]::-moz-placeholder{color:#a4abb3;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea[readonly]:-ms-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea[readonly]::-webkit-input-placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea[readonly]::placeholder{color:#a4abb3;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea[readonly]~.label-top{color:#a4abb3;cursor:not-allowed}textarea[readonly] label{background-color:#e0e2e5}textarea:focus{box-shadow:none;outline:0}textarea::-ms-expand{border:0;background-color:transparent}textarea::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}textarea:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}textarea[type=number]{-moz-appearance:textfield}textarea::-webkit-outer-spin-button,textarea::-webkit-inner-spin-button{-webkit-appearance:none;margin:0}input.valid,textarea.valid,select.valid{border-color:#00884a}input.warning,textarea.warning,select.warning{border-color:#ffcf00}input.invalid,input.error,textarea.invalid,textarea.error,select.invalid,select.error{border-color:#ed0007}.form-group{position:relative}.form-group input:first-child,.form-group textarea:first-child,.form-group select:first-child,.form-group .dropdown:first-child{padding:20px 16px 3px 16px}.form-group input:first-child:last-child,.form-group textarea:first-child:last-child,.form-group select:first-child:last-child,.form-group .dropdown:first-child:last-child{padding:12px 16px 11px 16px}.form-group.form-field-numeric .bosch-ic-clickable{position:absolute;top:12px;cursor:pointer}.form-group.form-field-numeric .bosch-ic-clickable:first-of-type+.bosch-ic-clickable{right:12px}.form-group.form-field-numeric .bosch-ic-clickable:first-of-type{right:60px}.form-group.form-field-numeric .label-top{width:calc(100% - 112px)}.form-group.form-field-numeric input{padding-right:106px}.form-group.form-field-numeric input:disabled~.bosch-ic-clickable{cursor:default}.form-group.form-field-numeric input:disabled~.bosch-ic-clickable,.form-group.form-field-numeric input:disabled~.bosch-ic-clickable:active,.form-group.form-field-numeric input:disabled~.bosch-ic-clickable:hover{color:#a4abb3}.form-group.form-field-numeric input:nth-last-child(3){padding:12px 16px 11px 16px;padding-right:106px}.form-group div.hint,.form-group div.valid,.form-group div.error,.form-group div.warning,.form-group span.hint,.form-group span.valid,.form-group span.error,.form-group span.warning,.form-group p.hint,.form-group p.valid,.form-group p.error,.form-group p.warning{font-size:12px;padding:8px 16px;font-weight:500}.form-group div.valid,.form-group span.valid,.form-group p.valid{color:#00884a}.form-group div.warning,.form-group span.warning,.form-group p.warning{color:#ffcf00}.form-group div.error,.form-group span.error,.form-group p.error{color:#ed0007}@-moz-document url-prefix(){.form-group{position:relative}.form-group input:first-child,.form-group textarea:first-child,.form-group select:first-child,.form-group .dropdown:first-child{padding:20px 16px 3px 16px}.form-group input:first-child:last-child,.form-group textarea:first-child:last-child,.form-group select:first-child:last-child,.form-group .dropdown:first-child:last-child{padding:12px 16px 11px 16px}.form-group select:first-child{padding-left:13px}.form-group.form-field-numeric .bosch-ic-clickable{position:absolute;top:12px;cursor:pointer}.form-group.form-field-numeric .bosch-ic-clickable:first-of-type+.bosch-ic-clickable{right:12px}.form-group.form-field-numeric .bosch-ic-clickable:first-of-type{right:60px}.form-group.form-field-numeric .label-top{width:calc(100% - 112px)}.form-group.form-field-numeric input{padding-right:106px}.form-group.form-field-numeric input:disabled~.bosch-ic-clickable{cursor:default}.form-group.form-field-numeric input:disabled~.bosch-ic-clickable,.form-group.form-field-numeric input:disabled~.bosch-ic-clickable:active,.form-group.form-field-numeric input:disabled~.bosch-ic-clickable:hover{color:#a4abb3}.form-group.form-field-numeric input:nth-last-child(3){padding:12px 16px 11px 16px;padding-right:106px}}.dark-background input{background-color:#ffffff}.dark-background input:active:enabled,.dark-background input.active,.dark-background input:focus:enabled{background-color:#eff1f2}.dark-background input:active:enabled~.label-top,.dark-background input.active~.label-top,.dark-background input:focus:enabled~.label-top{background-color:#eff1f2}.dark-background textarea{background-color:#ffffff}.dark-background textarea:active:enabled,.dark-background textarea.active,.dark-background textarea:focus:enabled{background-color:#eff1f2}.dark-background textarea:active:enabled~.label-top,.dark-background textarea.active~.label-top,.dark-background textarea:focus:enabled~.label-top{background-color:#eff1f2}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */label{font-family:var(--bci-global-font-family);font-weight:400;font-size:12px;line-height:1.5;color:#000000;display:inline;line-height:18px;padding:0 16px 0 0}label.label-top{font-family:var(--bci-global-font-family);font-weight:400;font-size:12px;line-height:1.5;color:#000000;background-color:#e0e2e5;display:block}label.label-top:last-of-type{position:absolute;left:1px;top:1px;z-index:1;padding:4px 0 0 15px;margin:0;width:calc(100% - 48px);max-width:calc(100% - 48px);white-space:nowrap;overflow:hidden;text-overflow:ellipsis}label.label-top-disabled{color:#a4abb3;cursor:not-allowed}label.label-top-focused{background-color:#d1e4ff}.dark-background label.label-top{background-color:#ffffff}.dark-background label.label-top-focused{background-color:#eff1f2}:host{display:block;position:relative}:host .bci-core-datetime-picker{display:inline-block;position:relative;width:100%}:host .bci-core-datetime-picker input{padding-right:50px;height:48px}:host .bci-core-datetime-picker input::-moz-placeholder{color:#000000;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;opacity:0.5;transform:translateX(-0.5px)}:host .bci-core-datetime-picker input:-ms-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}:host .bci-core-datetime-picker input::-webkit-input-placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}:host .bci-core-datetime-picker input::placeholder{color:#000000;opacity:0.5;font-family:var(--bci-global-font-family);font-weight:400;font-size:16px;line-height:1.5;transform:translateX(-0.5px)}:host .bci-core-datetime-picker input:placeholder-shown{text-overflow:ellipsis}:host .bci-core-datetime-picker label{background:transparent}:host .bci-core-datetime-picker input:not(.has-label){padding-top:12px;padding-bottom:12px}:host .bci-core-datetime-picker .input-field{cursor:text;border-bottom:1px solid black !important}:host .bci-core-datetime-picker .label-top{width:fit-content;max-width:calc(100% - 64px);pointer-events:none}:host .bci-core-datetime-picker .icon{position:absolute;right:0;top:0;width:48px;aspect-ratio:1;border-bottom:1px solid black;box-sizing:border-box;display:grid;place-items:center;z-index:2;background:#e0e2e5;cursor:pointer}:host .bci-core-datetime-picker .icon:hover{background:#c1c7cc}:host .bci-core-datetime-picker .icon:active{background:#a4abb3}:host .bci-core-datetime-picker input:disabled{cursor:default;pointer-events:none;border-color:#7d8389 !important;color:#7d8389;background:#eff1f2}:host .bci-core-datetime-picker input:disabled:hover+label{background:transparent}:host .bci-core-datetime-picker input:disabled::placeholder{color:#7d8389}:host .bci-core-datetime-picker input:disabled+label{cursor:default}:host .bci-core-datetime-picker input:disabled+label+.icon{pointer-events:none;cursor:default;color:#7d8389;border-color:#7d8389;background:#e0e2e5}:host .bci-core-datetime-picker input:disabled+label+.icon:hover,:host .bci-core-datetime-picker input:disabled+label+.icon:active{background:initial}:host .bci-core-datetime-picker input:disabled+label+.icon span{cursor:default}:host .hidden{display:none}";

const EMPTYDATEPICKERINPUTOBJ = {
  dateStart: null,
  dateEnd: null,
  timeStart: null,
  timeEnd: null,
  timestampStart: null,
  timestampEnd: null,
  valid: true,
};
const DatetimePicker = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.newDateSelected = index.createEvent(this, "newDateSelected", 7);
    /** Label of the datepicker input field. */
    this.label = '';
    /** Placeholder of the datepicker input field. */
    this.placeholder = '';
    /** Format of the date. */
    this.formatDate = 'dd.MM.yyyy';
    /** The max date that should be selectable. */
    this.maxDate = 20000000000000;
    /** The min date that should be selectable. */
    this.minDate = 0;
    /** please use timeFormat 'H:mm'(hours and minutes) or 'hh:mm a'(hours and minutes with AM/PM) or 'hh:mm:ss a(hours,minutes,seconds with AM/PM)',for the date range picker  */
    this.timeFormat = 'H:mm';
    this.dateTimePickerInputObj = EMPTYDATEPICKERINPUTOBJ;
    /** Reactive state for UI changes depening on overlay opening state */
    this.overlayVisible = false;
    this.overlayHandler = new overlayHandler.OverlayHandler();
    this.formatService = formattingServiceFactory();
    this.dateService = dateServiceFactory({
      formatService: this.formatService,
    });
    this.showTemplates = false;
  }
  async componentWillLoad() {
    await utils.setLocale();
    this.setInputPlaceholderText();
    utils.listenToLanguageChange(() => this.setInputPlaceholderText());
    this.timePickerConfig = {
      timeFormat: this.timeFormat,
      rangeDateTime: this.rangeDateTime,
    };
    this.datePickerConfig = {
      formatDate: this.formatDate === undefined ? utils.state.dateFormat : this.formatDate,
      maxDate: this.maxDate,
      minDate: this.minDate,
      rangeDateTime: this.rangeDateTime,
      sundayFirst: this.sundayFirst,
      timeZone: this.timeZone,
    };
    /** Remove with next Major Update */
    if (!this.hide && this.hideTime) {
      this.hide = 'time';
    }
  }
  componentDidLoad() {
    this.overlayHandler.dialog.onclick = (ev) => {
      ev.stopPropagation();
    };
    this.overlayHandler.host.onclick = (ev) => {
      ev.stopPropagation();
    };
    this.formatService.dateFormat = this.formatDate;
    this.formatService.timeFormat = this.timeFormat;
    this.formatService.timeZone = this.timeZone;
    if (utils.isHideSeconds(this.timeFormat)) {
      this.hideSeconds = false;
      this.dateService.hideSeconds = false;
    }
    else {
      this.hideSeconds = true;
      this.dateService.hideSeconds = true;
    }
    this.dateService.rangeDateTime = this.rangeDateTime;
    this.dateService.minDate = this.minDate;
    this.dateService.maxDate = this.maxDate;
    const dateTime = this.getStartupDateTime();
    if (dateTime === undefined) {
      return;
    }
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), dateTime);
    this.updateDatePickerInput();
  }
  onDateTimeChanged(dateTime) {
    var _a;
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), dateTime);
    this.selectedTemplateId = null;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.close();
    this.updateDatePickerInput();
  }
  onTimeStampChanged(timeStamp) {
    var _a;
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { dateStart: this.formatService.formatDate(timeStamp.timestampStart), timeStart: this.formatService.formatTime(timeStamp.timestampStart), dateEnd: this.formatService.formatDate(timeStamp.timestampEnd), timeEnd: this.formatService.formatTime(timeStamp.timestampEnd) });
    this.selectedTemplateId = null;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.close();
    this.updateDatePickerInput();
  }
  onPlaceholderChanged() {
    this.setInputPlaceholderText();
  }
  /**
   * - listens to the 'OverlayOpenedEvent' event which is triggered when another datetimepicker on the page is opened (see overlay component)
   * - if this is the case and the overlay of this datetimepicker is open (which closes through the event) the placeholder text needs to be updated
   * - therefore the placeholder is shown again after the user opens the datetimepicker and closes it through the opening of another datetimepicker on the page
   */
  updatePlaceholderOnOverlayOpenedEvent() {
    var _a;
    this.overlayVisible = false;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.closeYearMonthSelection();
    if (this.overlayHandler.isVisible()) {
      this.setInputPlaceholderText();
    }
  }
  handleMouseDown(event) {
    this.handleClickOutside(event);
  }
  handleTouchStart(event) {
    this.handleClickOutside(event);
  }
  handleClickOutside(event) {
    var _a, _b, _c;
    if (this.overlayHandler.isHidden()) {
      // Don't stopPropagation if dateTimePicket is closed
      return;
    }
    if (((_a = this.datepickerRef) === null || _a === void 0 ? void 0 : _a.contains(event.target)) || ((_b = this.overlayHandler.dialog) === null || _b === void 0 ? void 0 : _b.contains(event.target))) {
      return;
    }
    if (this.overlayHandler.isVisible()) {
      this.save();
      this.overlayVisible = false;
      (_c = this.overlayHandler.dialog) === null || _c === void 0 ? void 0 : _c.closeYearMonthSelection();
    }
  }
  handleEscape(ev) {
    // in case of 'Escape'
    if (ev.key !== 'Escape') {
      return;
    }
    if (this.overlayHandler.isVisible()) {
      this.save();
    }
    this.dateTimePickerInput.blur();
  }
  handleTab(ev) {
    var _a;
    if (ev.key !== 'Tab') {
      return;
    }
    const sourceTarget = ev.composedPath()[0];
    if (this.hide === 'time' ||
      (sourceTarget.dataset.seconds &&
        ((sourceTarget.parentElement && sourceTarget.parentElement.classList.contains('rangeTimePicker')) || !this.rangeDateTime) &&
        !ev.shiftKey) ||
      (this.hideSeconds && !this.rangeDateTime && sourceTarget.dataset.minutes && !ev.shiftKey) ||
      (this.hideSeconds &&
        sourceTarget.dataset.minutes &&
        sourceTarget.parentElement &&
        sourceTarget.parentElement.classList.contains('rangeTimePicker')) ||
      (sourceTarget && sourceTarget.classList.contains('input-field') && ev.shiftKey)) {
      this.save();
    }
    this.overlayHandler.hide();
    this.overlayVisible = false;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.closeYearMonthSelection();
  }
  handleEnter(ev) {
    // in case of 'Enter'
    if (ev.key !== 'Enter') {
      return;
    }
    if (this.overlayHandler.isVisible()) {
      this.save();
    }
    this.dateTimePickerInput.blur();
  }
  async toggle() {
    if (!this.overlayHandler.isReady()) {
      return;
    }
    if (this.overlayHandler.isVisible()) {
      this.save();
    }
    else {
      this.overlayHandler.show();
      this.overlayVisible = true;
    }
    this.setInputPlaceholderText();
  }
  showPickerContainer() {
    this.inputPlaceholder = '';
    this.overlayHandler.show();
    this.overlayVisible = true;
  }
  getStartupDateTime() {
    if (this.timeStamp) {
      return {
        dateStart: this.formatService.formatDate(this.timeStamp.timestampStart),
        timeStart: this.formatService.formatTime(this.timeStamp.timestampStart),
        dateEnd: this.formatService.formatDate(this.timeStamp.timestampEnd),
        timeEnd: this.formatService.formatTime(this.timeStamp.timestampEnd),
      };
    }
    if (this.dateTime) {
      const { dateStart, dateEnd, timeStart, timeEnd } = this.dateTime;
      return {
        dateStart: dateStart !== null && dateStart !== void 0 ? dateStart : null,
        dateEnd: dateEnd !== null && dateEnd !== void 0 ? dateEnd : null,
        timeStart: timeStart !== null && timeStart !== void 0 ? timeStart : null,
        timeEnd: timeEnd !== null && timeEnd !== void 0 ? timeEnd : null,
      };
    }
  }
  updateDatePickerInput() {
    if (this.dateTimePickerInput) {
      this.dateTimePickerInput.value = this.getInputValue();
    }
  }
  /** Function to reset the datepicker's selection */
  async resetSelection() {
    var _a;
    this.dateTimePickerInputObj = EMPTYDATEPICKERINPUTOBJ;
    this.selectedTemplateId = null;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.close();
    this.updateDatePickerInput();
    this.newDateSelected.emit(this.dateTimePickerInputObj);
  }
  getInputValue() {
    var _a;
    let returnValue = '';
    if (this.isNull()) {
      return returnValue;
    }
    if (this.showTemplates && this.selectedTemplateId) {
      /** Either a template or the template group with the selected template in it */
      const foundElement = this.templatesList.find((x) => 'templates' in x ? x.templates.find((y) => y.id === this.selectedTemplateId) : x.id === this.selectedTemplateId);
      returnValue =
        'templates' in foundElement
          ? (_a = foundElement.templates.find((template) => template.id === this.selectedTemplateId)) === null || _a === void 0 ? void 0 : _a.label
          : foundElement === null || foundElement === void 0 ? void 0 : foundElement.label;
    }
    if (returnValue === '') {
      const { dateStart, dateEnd, timeStart, timeEnd } = this.dateTimePickerInputObj;
      returnValue = this.formatService.formatRange(dateStart, dateEnd, timeStart, timeEnd);
    }
    return returnValue;
  }
  get openingIcon() {
    return this.hide === 'calendar' ? 'clock' : this.hide === 'time' ? 'calendar' : 'calendar-clock';
  }
  updateDatePickerComponent(ev) {
    const value = ev.target.value;
    const dateInput = this.dateService.getDataFromInput(value);
    const validityCheck = this.dateService.validateInput(dateInput);
    this.dateTimePickerInputObj = Object.assign(Object.assign(Object.assign({}, this.dateTimePickerInputObj), dateInput), validityCheck);
  }
  /** Corrects data based on configuration */
  correctData() {
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), this.dateService.correctInputData(this.dateTimePickerInputObj));
    if (this.hide === 'calendar') {
      this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { dateStart: null, dateEnd: null });
    }
    else if (this.hide === 'time') {
      this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { timeStart: null, timeEnd: null });
    }
  }
  setInputPlaceholderText() {
    if (this.placeholder) {
      this.inputPlaceholder = this.placeholder;
    }
    else {
      if (this.hide === 'time') {
        this.inputPlaceholder = utils.translate('setDate');
        if (this.rangeDateTime) {
          this.inputPlaceholder += ' ' + utils.translate('range');
        }
      }
      else if (this.hide === 'calendar') {
        this.inputPlaceholder = utils.translate('setTime');
        if (this.rangeDateTime) {
          this.inputPlaceholder += ' ' + utils.translate('range');
        }
      }
      else {
        this.inputPlaceholder = utils.translate('setDateTime');
        if (this.rangeDateTime) {
          this.inputPlaceholder = utils.translate('setRange');
        }
      }
    }
  }
  save() {
    var _a;
    if (!this.overlayHandler.isReady()) {
      return;
    }
    this.overlayHandler.hide();
    this.overlayVisible = false;
    (_a = this.overlayHandler.dialog) === null || _a === void 0 ? void 0 : _a.closeYearMonthSelection();
    if (this.dateTimePickerInput.value.trim().length === 0) {
      this.setInputPlaceholderText();
    }
    if (this.rangeDateTime && !this.dateTimePickerInputObj.dateEnd) {
      this.dateTimePickerInputObj.dateEnd = this.dateTimePickerInputObj.dateStart;
    }
    if (this.hide === 'calendar' && !this.dateTimePickerInputObj.timeStart) {
      this.dateTimePickerInputObj.timeStart = this.dateTimePickerInputObj.timeEnd;
    }
    this.correctData();
    const timestampStart = this.dateService.toSelectionTimestamp(this.dateTimePickerInputObj.dateStart, this.dateTimePickerInputObj.timeStart);
    const timestampEnd = this.dateService.toSelectionTimestamp(this.dateTimePickerInputObj.dateEnd, this.dateTimePickerInputObj.timeEnd);
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { timestampStart: timestampStart !== null && timestampStart !== void 0 ? timestampStart : null, timestampEnd: timestampEnd !== null && timestampEnd !== void 0 ? timestampEnd : null });
    this.updateDatePickerInput();
    if (!this.showTemplates) {
      this.selectedTemplateId = '';
    }
    this.newDateSelected.emit(this.dateTimePickerInputObj);
  }
  isNull() {
    return (this.dateTimePickerInputObj.dateStart === null &&
      this.dateTimePickerInputObj.dateEnd === null &&
      this.dateTimePickerInputObj.timeStart === null &&
      this.dateTimePickerInputObj.timeEnd === null);
  }
  handleSelectionChanged(event) {
    event.stopPropagation();
    switch (event.detail.reason) {
      case 'date':
        this.showTemplates = false;
        this.handleDateChange(event.detail);
        break;
      case 'time':
        this.showTemplates = false;
        this.handleTimeChange(event.detail);
        break;
      case 'frame':
        this.showTemplates = true;
        this.handleRangeChange(event.detail);
        break;
    }
    this.updateDatePickerInput();
  }
  handleDateChange({ startDate: dateStart, endDate: dateEnd, rangeCompleted }) {
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { dateStart, dateEnd: dateEnd !== null && dateEnd !== void 0 ? dateEnd : null });
    if (this.hide === 'time') {
      if (!this.rangeDateTime || (this.rangeDateTime && rangeCompleted)) {
        this.save();
      }
    }
  }
  handleTimeChange(newTimes) {
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), newTimes);
  }
  handleRangeChange(event) {
    var _a, _b;
    if (event.firstEmit) {
      this.selectedTemplateId = event.id;
    }
    const timestampStart = (_a = this.dateService.toSelectionTimestamp(this.formatService.formatDate(event.timestampStart), this.formatService.formatTime(event.timestampStart))) !== null && _a !== void 0 ? _a : null;
    const timestampEnd = (_b = this.dateService.toSelectionTimestamp(this.formatService.formatDate(event.timestampEnd), this.formatService.formatTime(event.timestampEnd))) !== null && _b !== void 0 ? _b : null;
    this.dateTimePickerInputObj = Object.assign(Object.assign({}, this.dateTimePickerInputObj), { dateStart: this.formatService.formatDate(event.timestampStart), dateEnd: this.formatService.formatDate(event.timestampEnd), timeStart: this.formatService.formatTime(event.timestampStart), timeEnd: this.formatService.formatTime(event.timestampEnd), timestampStart,
      timestampEnd });
    if (this.overlayHandler.isReady()) {
      this.newDateSelected.emit(this.dateTimePickerInputObj);
    }
    if (event.firstEmit) {
      this.hideOverlay();
    }
    this.updateDatePickerInput();
  }
  hideOverlay() {
    this.updateDatePickerInput();
    this.overlayHandler.hide();
    this.overlayVisible = false;
  }
  render() {
    return (index.h(index.Host, { ref: (el) => (this.datepickerRef = el) }, index.h("div", { class: { 'bci-core-datetime-picker': true, 'form-group': true, 'dark-background': !!this.darkBackground }, id: this.dateTimePickerId, ref: (el) => (this.overlayHandler.host = el) }, index.h("input", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.input-field", id: this.dateTimePickerId + '-input', class: { 'input-field': true, invalid: !this.dateTimePickerInputObj.valid, 'has-label': !!this.label }, type: "text", ref: (el) => (this.dateTimePickerInput = el), onFocus: () => this.showPickerContainer(), onInput: (event) => this.updateDatePickerComponent(event), placeholder: this.inputPlaceholder, disabled: this.disabled, value: "" }), index.h("label", { class: "label-top" }, this.label), !this.overlayVisible ? (index.h("div", { class: "icon", "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.toggle-picker", onClick: () => this.dateTimePickerInput.focus(), title: "Open" }, index.h("span", { class: `bosch-ic bci-core-calendar-icon bosch-ic-clickable bosch-ic-${this.openingIcon} has-label` }))) : this.dateTimePickerInput.value.trim() ? (index.h("div", { class: "icon", "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.reset-picker", onClick: async () => {
        await this.resetSelection();
        this.dateTimePickerInput.focus();
      }, title: "Clear" }, index.h("span", { class: "bosch-ic bci-core-calendar-icon bosch-ic-clickable bosch-ic-delete has-label" }))) : (index.h("div", { class: "icon", "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.toggle-picker", onClick: () => this.toggle(), title: "Close" }, index.h("span", { class: "bosch-ic bci-core-calendar-icon bosch-ic-clickable bosch-ic-close-small has-label" })))), index.h("bci-overlay", { ref: (el) => (this.overlayHandler.overlay = el) }, index.h("bci-datetime-dialog", { ref: (el) => (this.overlayHandler.dialog = el), dateTimePickerId: this.dateTimePickerId, onSelectionChanged: (timeStamp) => this.handleSelectionChanged(timeStamp), toggleDialogHandler: () => this.toggle(), hide: this.hide, templatesList: this.templatesList, datePickerConfig: this.datePickerConfig, timePickerConfig: this.timePickerConfig, datePickerInputObj: this.dateTimePickerInputObj, formatService: this.formatService, dateService: this.dateService, templatesDisabled: this.templatesDisabled }))));
  }
  static get watchers() { return {
    "dateTime": ["onDateTimeChanged"],
    "timeStamp": ["onTimeStampChanged"],
    "placeholder": ["onPlaceholderChanged"]
  }; }
};
DatetimePicker.style = datetimePickerCss;

const templateCss = "/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:\"Bosch-Sans\";font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */label{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;line-height:1.5;color:#000000;display:inline;line-height:18px;padding:0 16px 0 0}label.label-top{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;line-height:1.5;color:#000000;background-color:#e0e2e5;display:block}label.label-top:last-of-type{position:absolute;left:1px;top:1px;z-index:1;padding:4px 0 0 15px;margin:0;width:calc(100% - 48px);max-width:calc(100% - 48px);white-space:nowrap;overflow:hidden;text-overflow:ellipsis}label.label-top-disabled{color:#a4abb3;cursor:not-allowed}label.label-top-focused{background-color:#d1e4ff}.dark-background label.label-top{background-color:#ffffff}.dark-background label.label-top-focused{background-color:#eff1f2}/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */:host{display:block}:host .templates-container{display:block;background-color:#ffffff;min-width:292px;flex-shrink:0}:host .templates-container .mdc-list{margin:0;background:#ffffff;outline:none;list-style-type:none;padding:0 !important;overflow-y:auto;overflow-y:overlay;max-height:240px}:host .templates-container .mdc-list::-webkit-scrollbar{width:16px;height:16px;padding-right:2px}:host .templates-container .mdc-list::-webkit-scrollbar-track{background:transparent}:host .templates-container .mdc-list::-webkit-scrollbar-thumb{background:#7d8389;background-clip:padding-box;border:5px solid transparent;min-height:48px}:host .templates-container .mdc-list::-webkit-scrollbar-thumb:hover{background:#43464a;background-clip:padding-box;border:3px solid transparent}:host .templates-container .mdc-list-item{line-height:1.5;padding:12px 16px !important;cursor:pointer;display:flex;align-items:center;overflow:hidden;margin:0 !important;background-color:#ffffff;color:#000000;font-size:16px;text-overflow:ellipsis}:host .templates-container .mdc-list-item::before{content:none !important}:host .templates-container .mdc-list-item:hover{background-color:#e0e2e5}:host .templates-container .mdc-list-item:active{background-color:#c1c7cc}:host .templates-container .mdc-list-item>*{color:inherit;font-size:inherit;line-height:inherit;cursor:inherit;padding:0;margin:0}:host .templates-container .mdc-list-item.selected{line-height:1.5;padding:12px 16px !important;cursor:pointer;display:flex;align-items:center;overflow:hidden;margin:0 !important;background-color:#007bc0;color:#ffffff}:host .templates-container .mdc-list-item.selected::before{content:none !important}:host .templates-container .mdc-list-item.selected:hover{background-color:#00629a}:host .templates-container .mdc-list-item.selected:active{background-color:#004975}:host .templates-container .mdc-list-item.disabled{background:#ffffff;cursor:default;pointer-events:none;color:#a4abb3}:host .templates-container .mdc-list-item .mdc-list-item__text{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}:host .templates-container .optiongroup-label{font-weight:700;font-size:12px;line-height:1.5;padding:12px 16px 4px 16px;width:100%;box-sizing:border-box;user-select:none;overflow:hidden;display:-webkit-box;-webkit-line-clamp:1;-webkit-box-orient:vertical}:host .templates-container .optiongroup-label.disabled,:host .templates-container .optiongroup-label:disabled{color:#a4abb3}:host .templates-container :not(.mdc-list--non-interactive)>:not(.mdc-list-item--disabled).mdc-list-item::after{cursor:pointer}:host .templates-container .mdc-list-item__graphic{margin-right:8px}@media (min-width: 767px){.templates-container{max-width:584px}}@media (max-width: 767px){.templates-container{max-width:292px}}";

const TemplateList = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.showTimeRanges = index.createEvent(this, "showTimeRanges", 7);
    this.templateUpdate = index.createEvent(this, "templateUpdate", 7);
    /**
     * Whether the templatesList should be visible.
     * Using this prop instead of traditional conditional rendering to preserve the state within this component.
     */
    this.visible = false;
  }
  /**
   * Clears internal timer and resets selected template
   */
  async reset() {
    this.selectedTemplateId = '';
    this.clearTimer();
  }
  clearTimer() {
    if (this.timerId) {
      clearInterval(this.timerId);
      this.timerId = 0;
    }
  }
  updateTemplate(event, option) {
    event.stopPropagation();
    this.selectedTemplateId = option.id;
    this.clearTimer();
    this.applyTemplate(option, true);
    if (option.liveUpdates && option.refreshInterval && option.refreshInterval > 0) {
      this.timerId = window.setInterval(() => {
        this.applyTemplate(option, false);
      }, option.refreshInterval);
    }
  }
  applyTemplate(option, firstEmit) {
    this.templateUpdate.emit({ timeStamp: option.calculateTemplate(), firstEmit, id: this.selectedTemplateId });
  }
  render() {
    var _a;
    return this.visible ? (index.h(index.Host, { class: "templates-container" }, index.h("ul", { class: "mdc-list", role: "listbox" }, (_a = this.templatesList) === null || _a === void 0 ? void 0 : _a.map((element) => 'id' in element ? (index.h("li", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.selecttemplate", class: {
        'mdc-list-item': true,
        selected: this.selectedTemplateId === element.id,
        disabled: element.disabled,
      }, role: "option", "aria-label": `Select ${element.label}`, title: element.label, onClick: (item) => this.updateTemplate(item, element) }, index.h("label", { class: "mdc-list-item__text" }, element.label))) : (
    // element is TemplateGroup
    element.templates.length > 0 && (index.h("div", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.optiongroup" }, index.h("span", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.optiongrouplabel", class: { 'optiongroup-label': true, disabled: element.disabled } }, element.label), element.templates.map((template) => (index.h("li", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.selecttemplate", class: {
        'mdc-list-item': true,
        selected: this.selectedTemplateId === template.id,
        disabled: template.disabled || element.disabled,
      }, role: "option", "aria-label": `Select ${template.label}`, title: template.label, onClick: (item) => this.updateTemplate(item, template) }, index.h("label", { class: "mdc-list-item__text" }, template.label))))))))), index.h("div", { class: "bci-core-datepicker-selection" }, index.h("button", { "data-test": "webcore.webcomponents.datetime-picker.core-datetime-picker.timerange", class: "button-secondary no-arrow", onClick: () => this.showTimeRanges.emit() }, utils.translate('calendar'))))) : null;
  }
};
TemplateList.style = templateCss;

const yearMonthCss = "/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:\"Bosch-Sans\";font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Sans\";font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.clickable-element{transition:0.4s;cursor:pointer}.clickable-element:active,.clickable-element.active{color:#007bc0}.clickable-element:active:hover,.clickable-element:hover .clickable-element:focus{color:#9dc9ff}.clickable-element:disabled,.clickable-element.disabled{color:#a4abb3}.clickable-element:disabled:active,.clickable-element:disabled.active,.clickable-element:disabled:active:hover,.clickable-element:disabled:hover,.clickable-element.disabled:active,.clickable-element.disabled.active,.clickable-element.disabled:active:hover,.clickable-element.disabled:hover{color:#a4abb3}button.button-primary,button.button-default,input.button-primary,input.button-default{color:#ffffff;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;background-color:#007bc0;background-size:100% 200%;border-width:0;border-color:transparent;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-primary:hover,button.button-primary:focus-visible,button.button-default:hover,button.button-default:focus-visible,input.button-primary:hover,input.button-primary:focus-visible,input.button-default:hover,input.button-default:focus-visible{background-color:#00629a;color:#ffffff;outline:0}button.button-primary:active,button.button-default:active,input.button-primary:active,input.button-default:active{background-color:#004975;color:#ffffff;outline:0}button.button-primary:disabled,button.button-primary.disabled,button.button-default:disabled,button.button-default.disabled,input.button-primary:disabled,input.button-primary.disabled,input.button-default:disabled,input.button-default.disabled{background-color:#c1c7cc;color:#656a6f;background-image:none;cursor:default}button.button-primary.button-lg,button.button-default.button-lg,input.button-primary.button-lg,input.button-default.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-sm,button.button-default.button-sm,input.button-primary.button-sm,input.button-default.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary.button-xs,button.button-default.button-xs,input.button-primary.button-xs,input.button-default.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-primary:not(.mat-icon-button):has(mat-icon),button.button-default:not(.mat-icon-button):has(mat-icon),input.button-primary:not(.mat-icon-button):has(mat-icon),input.button-default:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-primary.mat-icon-button,button.button-default.mat-icon-button,input.button-primary.mat-icon-button,input.button-default.mat-icon-button{padding:12px}button.button-secondary,input.button-secondary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 15px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-width:1px;border-color:#007bc0;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:hover,button.button-secondary:focus-visible,input.button-secondary:hover,input.button-secondary:focus-visible{border-color:#004975;outline:0;color:#004975;background-color:#d1e4ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:active,input.button-secondary:active{border-color:#004975;color:#004975;outline:0;background-color:#9dc9ff;border-width:1px;border-color:#004975;border-style:solid;-webkit-border-radius:0;-moz-border-radius:0;-ms-border-radius:0;border-radius:0}button.button-secondary:disabled,button.button-secondary.disabled,input.button-secondary:disabled,input.button-secondary.disabled{color:#979ea4;border-color:#979ea4;background-color:transparent;cursor:default}button.button-secondary.button-lg,input.button-secondary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-sm,input.button-secondary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary.button-xs,input.button-secondary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-secondary:not(.mat-icon-button):has(mat-icon),input.button-secondary:not(.mat-icon-button):has(mat-icon){padding:0 15px 0 13px}button.button-secondary.mat-icon-button,input.button-secondary.mat-icon-button{padding:11px}button.button-tertiary,input.button-tertiary{color:#007bc0;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-tertiary:hover,button.button-tertiary:focus-visible,input.button-tertiary:hover,input.button-tertiary:focus-visible{outline:0;color:#00629a;background-color:#d1e4ff}button.button-tertiary:active,input.button-tertiary:active{color:#004975;background-color:#9dc9ff;outline:0}button.button-tertiary:disabled,button.button-tertiary.disabled,input.button-tertiary:disabled,input.button-tertiary.disabled{color:#979ea4;background-color:transparent;cursor:default}button.button-tertiary.button-lg,input.button-tertiary.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-sm,input.button-tertiary.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:12px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary.button-xs,input.button-tertiary.button-xs{font-family:\"Bosch-Sans\";font-weight:400;font-size:10px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-tertiary:not(.mat-icon-button):has(mat-icon),input.button-tertiary:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-tertiary.mat-icon-button,input.button-tertiary.mat-icon-button{padding:12px}button.button-integrated,input.button-integrated{color:#000000;background-color:transparent;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0px 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto;border-style:none}button.button-integrated:hover,button.button-integrated:focus-visible,input.button-integrated:hover,input.button-integrated:focus-visible{color:#007bc0;outline:0}button.button-integrated:active,input.button-integrated:active{color:#00629a;outline:0}button.button-integrated:disabled,button.button-integrated.disabled,input.button-integrated:disabled,input.button-integrated.disabled{color:#979ea4 !important;background-color:transparent;cursor:default}button.button-integrated:not(.mat-icon-button):has(mat-icon),input.button-integrated:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-integrated.mat-icon-button,input.button-integrated.mat-icon-button{padding:12px}button.button-link,input.button-link{color:#007bc0;background-color:transparent;border:none;-webkit-box-shadow:none;box-shadow:none;font-family:\"Bosch-Sans\";font-weight:400;font-size:16px;height:48px;padding:0 16px 0 16px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link::after,input.button-link::after{position:absolute;font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:18px;content:\"\\e181\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}button.button-link:hover,button.button-link:focus-visible,input.button-link:hover,input.button-link:focus-visible{color:#00629a;text-decoration:underline;outline:0}button.button-link:active,input.button-link:active{color:#004975;text-decoration:underline}button.button-link.disabled,button.button-link[disabled],button.button-link fieldset[disabled],input.button-link.disabled,input.button-link[disabled],input.button-link fieldset[disabled]{color:#c1c7cc;text-decoration:none;-webkit-transition:none;-o-transition:none;transition:none}button.button-link.disabled::after,button.button-link[disabled]::after,button.button-link fieldset[disabled]::after,input.button-link.disabled::after,input.button-link[disabled]::after,input.button-link fieldset[disabled]::after{-webkit-transform:translate(0, 0);-ms-transform:translate(0, 0);-o-transform:translate(0, 0);transform:translate(0, 0)}button.button-link.button-lg,input.button-link.button-lg{font-family:\"Bosch-Sans\";font-weight:400;font-size:20px;height:48px;padding:0 24px 0 24px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-lg::after,input.button-link.button-lg::after{font-size:22px}button.button-link.button-sm,input.button-link.button-sm{font-family:\"Bosch-Sans\";font-weight:400;font-size:14px;height:48px;padding:0 12px 0 12px;display:inline-flex;justify-content:center;align-items:center;gap:8px;cursor:pointer;width:auto}button.button-link.button-sm::after,input.button-link.button-sm::after{font-size:14px}button.button-link.no-arrow::after,input.button-link.no-arrow::after{content:\"\"}button.button-link:not(.mat-icon-button):has(mat-icon),input.button-link:not(.mat-icon-button):has(mat-icon){padding:0px 16px 0px 14px}button.button-link.mat-icon-button,input.button-link.mat-icon-button{padding:12px}button.button-block,input[type=submit].button-block,input[type=reset].button-block,input[type=button].button-block{display:block;width:100%}/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */bci-year-month{width:100%;height:100%}.year-month-grid{width:100%;height:100%;padding:16px;display:flex;box-sizing:border-box;flex-direction:column;justify-content:space-between}.year-month-grid .row{width:100%;display:flex;align-items:center;justify-content:space-between}.year-month-grid .row .selection-chip{width:60px;height:30px;font-size:16px;background:transparent;display:grid;place-items:center;line-height:1.5;border:none;border-radius:15px;cursor:pointer;outline:none;user-select:none}.year-month-grid .row .selection-chip:hover,.year-month-grid .row .selection-chip:focus{background:#e0e2e5}.year-month-grid .row .selection-chip:active{background:#c1c7cc}.year-month-grid .row .selection-chip.today{border:1px solid #000000}.year-month-grid .row .selection-chip.selected{background:#007bc0;color:white;border:none}.year-month-grid .row .selection-chip.selected:hover,.year-month-grid .row .selection-chip.selected:focus{background:#00629a}.year-month-grid .row .selection-chip.selected:active{background:#004975}.year-month-grid .row .selection-chip:disabled{pointer-events:none;color:#c1c7cc}.year-month-grid .row .selection-chip:disabled.today{border-color:#c1c7cc}.year-month-grid .row .selection-chip:disabled.selected{background:#c1c7cc;color:#8a9097}";

/** number of years displayed per page */
const YEAR_PAGE_SIZE = 20;
/** Where the currently selected year is displayed on the yearSelection Page. 0-based. */
const SELECTED_YEAR_SPOT = 9;
const YearMonthSelection = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.yearMonthSelected = index.createEvent(this, "yearMonthSelected", 7);
  }
  handleSelectionStepChange(newValue, oldValue) {
    // Reopen of YearMonthSelection
    if (!oldValue && !!newValue) {
      this.displayedYears = this.getDisplayedYearsFromYear(this.dateObj.calendarDateYear);
    }
  }
  handleSelectionPageChange(newValue) {
    this.displayedYears = this.getDisplayedYearsFromYear(this.dateObj.calendarDateYear + newValue * YEAR_PAGE_SIZE);
  }
  getDisplayedYearsFromYear(year) {
    const yearsBefore = [];
    for (let i = year - SELECTED_YEAR_SPOT; i < year; i++) {
      yearsBefore.push(i);
    }
    const yearsAfter = [];
    for (let i = year + 1; i < year + (YEAR_PAGE_SIZE - SELECTED_YEAR_SPOT); i++) {
      yearsAfter.push(i);
    }
    const years = [...yearsBefore, year, ...yearsAfter];
    return this.createGrid(years, YEAR_PAGE_SIZE, 4);
  }
  get months() {
    const locale = { locale: utils.LOCALE_MAP[utils.state.language || utils.DEFAULT_LOCALE.language] };
    const months = [];
    for (let i = 0; i < 12; i++) {
      months.push(utils.format(utils.setMonth(new Date(), i), 'LLL', locale));
    }
    return this.createGrid(months, 12, 3);
  }
  // use generics here because years are numbers and months are strings
  createGrid(array, gridItems, rowSize) {
    const grid = [];
    for (let i = 0; i < gridItems; i += rowSize) {
      const row = array.slice(i, i + rowSize);
      grid.push(row);
    }
    return grid;
  }
  handleMonthClick(monthIndex) {
    this.yearMonthSelected.emit({ selectedYearMonth: monthIndex });
  }
  handleYearClick(year) {
    this.yearMonthSelected.emit({ selectedYearMonth: year });
  }
  render() {
    const selectedYear = this.dateObj.calendarDateYear;
    const selectedMonth = this.dateObj.calendarDateMonth;
    const selectionAsDate = utils.createDate(new Date(), selectedYear, selectedMonth);
    return (index.h(index.Host, null, this.selectionStep === 'year' ? (index.h("div", { class: "year-month-grid" }, this.displayedYears.map((yearRow) => (index.h("div", { class: "row" }, yearRow.map((year) => {
      const selected = isSameYear(utils.createDate(new Date(), year, 0), selectionAsDate);
      const isTodaysDate = isSameYear(utils.createDate(new Date(), year, 0), new Date());
      const notInRange = isBefore(utils.createDate(new Date(), year, 0, 1), this.minDate) ||
        isAfter(utils.createDate(new Date(), year, 11, utils.getDaysInMonth(new Date(year, 11))), this.maxDate);
      const buttonClasses = {
        'selection-chip': true,
        selected,
        today: isTodaysDate,
      };
      return (index.h("button", { class: buttonClasses, "aria-label": `Select Year ${year}`, key: year, disabled: notInRange, onClick: () => this.handleYearClick(year) }, year));
    })))))) : this.selectionStep === 'month' ? (index.h("div", { class: "year-month-grid" }, this.months.map((monthRow, rowI) => (index.h("div", { class: "row" }, monthRow.map((month, monthI) => {
      const monthIndex = rowI * (this.months.length - 1) + monthI;
      const selected = isSameMonth(utils.createDate(new Date(), selectedYear, monthIndex), selectionAsDate);
      const isTodaysDate = isSameMonth(utils.createDate(new Date(), selectedYear, monthIndex), new Date());
      const notInRange = isBefore(utils.createDate(new Date(), selectedYear, monthIndex, 1), this.minDate) ||
        isAfter(utils.createDate(new Date(), selectedYear, monthIndex, utils.getDaysInMonth(new Date(selectedYear, monthIndex))), this.maxDate);
      const buttonClasses = {
        'selection-chip': true,
        selected,
        today: isTodaysDate,
      };
      return (index.h("button", { class: buttonClasses, "aria-label": `Select Month ${month}`, disabled: notInRange, onClick: () => this.handleMonthClick(monthIndex), key: month }, month));
    })))))) : undefined));
  }
  static get watchers() { return {
    "selectionStep": ["handleSelectionStepChange"],
    "yearSelectionPageOffset": ["handleSelectionPageChange"]
  }; }
};
YearMonthSelection.style = yearMonthCss;

exports.bci_datepicker = Datepicker;
exports.bci_datetime_dialog = DateTimeDialog;
exports.bci_datetime_picker = DatetimePicker;
exports.bci_month_header = MonthHeader;
exports.bci_template = TemplateList;
exports.bci_timepicker = Timepicker;
exports.bci_week_header = WeekHeader;
exports.bci_weekdays = Weekdays;
exports.bci_year_month = YearMonthSelection;

